c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1992  by  Jay William Ponder  ##
c     ###################################################
c
c     #################################################################
c     ##                                                             ##
c     ##  function analyze  --  callable version of analyze for X11  ##
c     ##                                                             ##
c     #################################################################
c
c
c     "analyze" computes and displays the total potential; options
c     are provided to partition the energy by atom or by potential
c     function type; parameters used in computing interactions can
c     also be displayed by atom; output of large energy interactions
c     and the total dipole moment and components are available
c
c
      subroutine analyze (molname)
      implicit none
      include 'sizes.i'
      include 'analyz.i'
      include 'angle.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bond.i'
      include 'charge.i'
      include 'couple.i'
      include 'dipole.i'
      include 'electr.i'
      include 'energi.i'
      include 'files.i'
      include 'imptor.i'
      include 'inform.i'
      include 'iounit.i'
      include 'kvdws.i'
      include 'moment.i'
      include 'potent.i'
      include 'strbnd.i'
      include 'strtor.i'
      include 'tors.i'
      include 'vdw.i'
      integer i,j,ia,ib,ic,id,ikey,next
      integer start,stop,freeunit
      real*8 e_total,moment
      character*20 keyword
      character*60 molname,keyfile
      character*80 record,string
      logical dosteric,doatom,dolarge,dodipole,doparam
      logical exist,active(maxatm),header
c
c
c     set up the structure and mechanics calculation
c
      call initial
      call getxyz (molname)
      call mechanic
c
c     initialize the computation type to perform
c
      dosteric = .false.
      doatom = .false.
      dolarge = .false.
      dodipole = .false.
      doparam = .false.
      do i = 1, n
         active(i) = .true.
      end do
c
c     check for the existence of a keyword file
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
c
c     search each line of the keyword file for options
c
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,10,end=40)  record
   10       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
c
c     get the values for output control keywords
c
            if (keyword(1:7) .eq. 'STERIC ') then
               dosteric = .true.
            else if (keyword(1:8) .eq. 'DOLARGE ') then
               dolarge = .true.
            else if (keyword(1:9) .eq. 'DODIPOLE ') then
               dodipole = .true.
            else if (keyword(1:7) .eq. 'DOATOM ') then
               doatom = .true.
               string = record(next:80)
               read (string,*,err=20)  start,stop
               do i = 1, start-1
                  active(i) = .false.
               end do
               do i = start, stop
                  active(i) = .true.
               end do
               do i = stop+1, n
                  active(i) = .false.
               end do
   20          continue
            else if (keyword(1:8) .eq. 'DOPARAM ') then
               doparam = .true.
               string = record(next:80)
               read (string,*,err=30)  start,stop
               do i = 1, start-1
                  active(i) = .false.
               end do
               do i = start, stop
                  active(i) = .true.
               end do
               do i = stop+1, n
                  active(i) = .false.
               end do
   30          continue
            end if
         end do
   40    continue
         close (unit=ikey)
      end if
c
c     write out large interactions if desired
c
      if (dolarge) then
         verbose = .true.
         write (iout,50)
   50    format ()
      end if
c
c     now make the call to get the partitioned energy
c
      if (dosteric .or. doatom .or. dolarge .or. dodipole)
     &   call analysis
c
c     sum up the components to find total steric energy
c
      if (dosteric .or. doatom .or. dolarge) then
         e_total = eb + ea + eba + eit + et + ebt + e14
     &                + ev + ec + ecd + ed + ex
         write (iout,80)  e_total
   80    format (/' Total Potential Energy :',f20.4,' Kcal/mole')
      end if
c
c     energy partitioning by types of steric energy
c
      if (dosteric) then
         write (iout,90)
   90    format (/' Energy Component Breakdown',9x,'Kcal/mole',
     &             4x,'Interactions'/)
         if (use_bond .and. nbond.ne.0) then
            write (iout,100)  eb,nbond
  100       format (' Bond Stretching    ',5x,f20.4,i12)
         end if
         if (use_angle .and. nangle.ne.0) then
            write (iout,110)  ea,nangle
  110       format (' Angle Bending      ',5x,f20.4,i12)
         end if
         if (use_angle .and. nstrbnd.ne.0) then
            write (iout,120)  eba,nstrbnd
  120       format (' Stretch-Bend       ',5x,f20.4,i12)
         end if
         if (use_imptor .and. nitors.ne.0) then
            write (iout,130)  eit,nitors
  130       format (' Improper Torsions  ',5x,f20.4,i12)
         end if
         if (use_tors .and. ntors.ne.0) then
            write (iout,140)  et,ntors
  140       format (' Torsional Angles   ',5x,f20.4,i12)
         end if
         if (use_tors .and. nstrtor.ne.0) then
            write (iout,150)  ebt,nstrtor
  150       format (' Stretch-Torsion    ',5x,f20.4,i12)
         end if
         if ((use_buck .or. use_lj) .and. nv14.ne.0) then
            write (iout,160)  e14,nv14
  160       format (' 1-4 van der Waals  ',5x,f20.4,i12)
         end if
         if ((use_buck .or. use_lj) .and. nvdw.ne.0) then
            write (iout,170)  ev,nvdw
  170       format (' Other van der Waals',5x,f20.4,i12)
         end if
         if (use_charge .and. nchg.ne.0) then
            write (iout,180)  ec,nchg
  180       format (' Charge-Charge      ',5x,f20.4,i12)
         end if
         if (use_chgdpl .and. nchgdpl.ne.0) then
            write (iout,190)  ecd,nchgdpl
  190       format (' Charge-Dipole      ',5x,f20.4,i12)
         end if
         if (use_dipole .and. ndpl.ne.0) then
            write (iout,200)  ed,ndpl
  200       format (' Dipole-Dipole      ',5x,f20.4,i12)
         end if
         if (use_extra .and. ex.ne.0.0d0) then
            write (iout,210)  ex
  210       format (' Extra & User Terms ',5x,f20.4)
         end if
      end if
c
c     compute the total molecular dipole moment
c
      if (dodipole) then
         moment = sqrt(xdipole**2 + ydipole**2 + zdipole**2)
         write (iout,220)  moment
  220    format (/' Total Dipole Moment :',11x,f12.4,' Debyes')
         write (iout,230)  xdipole,ydipole,zdipole
  230    format (/' X-Component',21x,f12.4,
     &           /' Y-Component',21x,f12.4,
     &           /' Z-Component',21x,f12.4)
      end if
c
c     energy partitioning over the individual atoms
c
      if (doatom) then
         write (iout,240)
  240    format (/' Potential Energy Breakdown over Atoms :')
         write (iout,250)
  250    format (/'  Atom',7x,'EB',6x,'EA',5x,'EBA',5x,'EIT',
     &            6x,'ET',5x,'EBT',5x,'E14',6x,'EV',6x,'EC',
     &            5x,'ECD',6x,'ED',6x,'EX'/)
         do i = 1, n
            if (active(i)) then
               write (iout,260)  i,aeb(i),aea(i),aeba(i),aeit(i),
     &                           aet(i),aebt(i),ae14(i),aev(i),
     &                           aec(i),aecd(i),aed(i),aex(i)
  260          format (1x,i5,1x,12f8.2)
            end if
         end do
      end if
c
c     list parameters used to compute each interaction
c
      if (doparam) then
         header = .true.
         do i = 1, n
            if (active(i)) then
               if (header) then
                  header = .false.
                  write (iout,270)
  270             format (/' Atom Type Definition Parameters :',
     &                    //3x,'Atom',5x,'Symbol',3x,'Type',
     &                    2x,'Atomic',6x,'Mass',3x,'Valence',
     &                    3x,'Description')
               end if
               write (iout,280)  i,name(i),itype(i),atomic(i),
     &                           mass(i),valence(i),story(i)
  280          format (i6,8x,a3,2i7,f12.4,i7,6x,a20)
            end if
         end do
         if (use_buck .or. use_lj) then
            header = .true.
            do i = 1, n
               if (active(i)) then
                  if (header) then
                     header = .false.
                     write (iout,290)
  290                format (/' Van der Waals Parameters :',
     &                       //10x,'Atom Number',13x,'Radius',
     &                       3x,'Epsilon',3x,'Reduction')
                  end if
                  j = itype(i)
                  if (reduct(j) .eq. 1.0d0) then
                     write (iout,300)  i,i,rad(j),eps(j)
  300                format (i6,4x,i5,15x,2f10.4)
                  else
                     write (iout,310)  i,i,rad(j),eps(j),reduct(j)
  310                format (i6,4x,i5,15x,3f10.4)
                  end if
               end if
            end do
         end if
         if (use_bond) then
            header = .true.
            do i = 1, nbond
               ia = ibnd(1,i)
               ib = ibnd(2,i)
               if (active(ia) .or. active(ib)) then
                  if (header) then
                     header = .false.
                     write (iout,320)
  320                format (/' Bond Stretching Parameters :',
     &                       //10x,'Atom Numbers',15x,'KS',
     &                          5x,'Length')
                  end if
                  write (iout,330)  i,ia,ib,bk(i),bl(i)
  330             format (i6,4x,2i5,10x,2f10.4)
               end if
            end do
         end if
         if (use_angle) then
            header = .true.
            do i = 1, nangle
               ia = iang(1,i)
               ib = iang(2,i)
               ic = iang(3,i)
               if (active(ia) .or. active(ib) .or. active(ic)) then
                  if (header) then
                     header = .false.
                     write (iout,340)
  340                format (/' Angle Bending Parameters :',
     &                       //12x,'Atom Numbers',13x,'KB',6x,
     &                          'Angle',4x,'StrBnd',3x,'O-P-Bend')
                  end if
                  if (kopb(i) .eq. 0.0d0) then
                     write (iout,350)  i,ia,ib,ic,acon(i),
     &                                 anat(i),ksb(i)
  350                format (i6,4x,3i5,5x,3f10.4)
                  else
                     write (iout,360)  i,ia,ib,ic,acon(i),
     &                                 anat(i),ksb(i),kopb(i)
  360                format (i6,4x,3i5,5x,4f10.4)
                  end if
               end if
            end do
         end if
         if (use_imptor) then
            header = .true.
            do i = 1, nitors
               ia = iitors(1,i)
               ib = iitors(2,i)
               ic = iitors(3,i)
               id = iitors(4,i)
               if (active(ia) .or. active(ib) .or.
     &             active(ic) .or. active(id)) then
                  if (header) then
                     header = .false.
                     write (iout,370)
  370                format (/' Improper Torsion Parameters :',
     &                       //15x,'Atom Numbers',12x,'KIT1',
     &                          7x,'KIT2',7x,'KIT3')
                  end if
                  write (iout,380)  i,ia,ib,ic,id,
     &                              itors1(1,i),nint(itors1(2,i)),
     &                              itors2(1,i),nint(itors2(2,i)),
     &                              itors3(1,i),nint(itors3(2,i))
  380             format (i6,4x,4i5,4x,3(f8.3,i3))
               end if
            end do
         end if
         if (use_tors) then
            header = .true.
            do i = 1, ntors
               ia = itors(1,i)
               ib = itors(2,i)
               ic = itors(3,i)
               id = itors(4,i)
               if (active(ia) .or. active(ib) .or.
     &             active(ic) .or. active(id)) then
                  if (header) then
                     header = .false.
                     write (iout,390)
  390                format (/' Torsional Angle Parameters :',
     &                       //15x,'Atom Numbers',13x,'KT1',
     &                          8x,'KT2',8x,'KT3',4x,'StrTor')
                  end if
                  if (kst(i) .eq. 0.0d0) then
                     write (iout,400)  i,ia,ib,ic,id,
     &                                 tors1(1,i),nint(tors1(2,i)),
     &                                 tors2(1,i),nint(tors2(2,i)),
     &                                 tors3(1,i),nint(tors3(2,i))
  400                format (i6,4x,4i5,4x,3(f8.3,i3))
                  else
                     write (iout,410)  i,ia,ib,ic,id,
     &                                 tors1(1,i),nint(tors1(2,i)),
     &                                 tors2(1,i),nint(tors2(2,i)),
     &                                 tors3(1,i),nint(tors3(2,i)),
     &                                 kst(i)
  410                format (i6,4x,4i5,4x,3(f8.3,i3),f8.3)
                  end if
               end if
            end do
         end if
         if (use_charge .or. use_chgdpl) then
            header = .true.
            do i = 1, nion
               ia = iion(i)
               if (active(ia)) then
                  if (header) then
                     header = .false.
                     write (iout,420)
  420                format (/' Atomic Partial Charge Parameters :',
     &                       //10x,'Atom Number',13x,'Charge')
                  end if
                  write (iout,430)  i,ia,pchg(i)
  430             format (i6,4x,i5,15x,f10.4)
               end if
            end do
         end if
         if (use_dipole .or. use_chgdpl) then
            header = .true.
            do i = 1, ndipole
               ia = idpl(1,i)
               ib = idpl(2,i)
               if (active(ia) .or. active(ib)) then
                  if (header) then
                     header = .false.
                     write (iout,440)
  440                format (/' Bond Dipole Moment Parameters :',
     &                       //10x,'Atom Numbers',12x,'Dipole',
     &                            3x,'Position')
                  end if
                  write (iout,450)  i,ia,ib,bdpl(i),sdpl(i)
  450             format (i6,4x,2i5,10x,2f10.4)
               end if
            end do
         end if
      end if
      return
      end
