/* dialog.c Collection of dialog boxes used for search files, warning
 *          messages, and options setting for each single programm
 */

#include <stdio.h>
#include <Xm/DialogS.h>
#include <Xm/RowColumn.h>
#include <Xm/PanedW.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>
#include <Xm/FileSB.h>
#include <Xm/MenuShell.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeBG.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Scale.h>
#include <Xm/Separator.h>
#include <Xm/ToggleBG.h>
#include <Xm/ToggleB.h>
#include <Xm/MessageB.h>
#include "vars.h"

Widget GetTopShell();
void dlg_callback();
void open_dialog_box();
void open_warning();
void open_sorry();
void open_help();
void reset_option();
void quit_option();
void toggled();
void do_analyze();
void change_format();

extern void analyze_();
extern void get_molnm();
extern void get_parnm();
extern void analyze_I();
extern void DrawText();
extern void TryDrawText();
extern void Makebtn();
extern void check_list();
extern void toggle_checked();
extern void toggle_tossed();



extern unsigned long toggles_set;
/*static Widget dialog;*/
extern char mol_file_name[];
extern Widget sw_text;
extern FILE *fp, *fp0;


void
toggle_tossed(w, item_no, cbs)
Widget w;
int item_no;
XmToggleButtonCallbackStruct *cbs;
{
    Widget dialog;
    Widget label, from_w, to_w;
    XmString msg;
    TxtWins *txt_wins;

    dialog = XtParent(XtParent(XtParent(XtParent(w))));
    if (cbs->set == False) {
	write_buf(w, item_no, DOWN);
	return;
    }

    write_buf(w, item_no, UP);
    XtVaGetValues(dialog, XmNuserData, &txt_wins, NULL);
    label = txt_wins->label;
    from_w = txt_wins->from;
    to_w = txt_wins->to;

    msg = XmStringCreateSimple(hint[item_no]);
    XtVaSetValues(label,
	XmNlabelString,	    msg,
	NULL);	  

    XtVaSetValues(from_w,
	XmNuserData,	    item_no, 
	NULL);	  

    XtVaSetValues(to_w,
	XmNuserData,	    item_no,
	NULL);

    XmProcessTraversal(w, XmTRAVERSE_NEXT_TAB_GROUP);
/*    XmProcessTraversal(from_w, XmTRAVERSE_CURRENT);*/
/* works but no blinking cursor */
}


static ButtonData analyze_I_optn_data[] = {
    {(Widget)0, "steric", &xmToggleButtonGadgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)6, (MenuItem *)NULL },
    {(Widget)0, "atom", &xmToggleButtonGadgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_tossed, (XtPointer)7, (MenuItem *)NULL },
    {(Widget)0, "large", &xmToggleButtonGadgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_tossed, (XtPointer)8, (MenuItem *)NULL },
    {(Widget)0, "dipole", &xmToggleButtonGadgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)9, (MenuItem *)NULL },
    {(Widget)0, "param", &xmToggleButtonGadgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_tossed, (XtPointer)10, (MenuItem *)NULL },
	NULL,
};

static ButtonData action_area_data[] = {
    {(Widget)0, "Run", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, do_analyze, NULL, (MenuItem *)NULL },
    {(Widget)0, "Default", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, reset_option, NULL, (MenuItem *)NULL },
    {(Widget)0, "Cancel", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, quit_option, NULL, (MenuItem *)NULL },
    {(Widget)0, "Help", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, open_help, NULL, (MenuItem *)NULL },
	NULL,
};

static MenuItem option_data[] = {
	{ "   PDB   ", &xmPushButtonWidgetClass, NULL, NULL, NULL,
	    change_format, NULL, (MenuItem *)NULL },
	{ "   XYZ   ", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    change_format, NULL, (MenuItem *)NULL },
	NULL,
};

static ButtonData option_btn_data[] = {
	{(Widget)0, "Format: ", &xmRowColumnWidgetClass, XmMENU_OPTION, 
	    NULL, NULL, NULL, NULL, NULL, (MenuItem *)option_data },
	NULL,
};

/* callback for some of the menu items declared in the MenuItem struct.
 * The client data is a function that creates some sort of Motif dialog.
 * Since the function returns this dialog, associate it with the menu
 * item via XmNuserData so we (a) don't have a to keep a global and
 * (b) don't have to repeatedly create one.
 *
 * revised from ch16/popups.c
 */
void
open_dialog_box(w, sfx, cbs)
Widget w;           /* the menu item that was invoked */
int sfx;    /* indicates the file suffix MOL or PARA */
XmAnyCallbackStruct *cbs; /* unused */
{
    Widget work_area;
    /* initialize in case get values doesn't return anything sensible. */
    Widget dialog = NULL;

    /* first see if this menu item's dialog has been created yet */
    XtVaGetValues(w, XmNuserData, &dialog, NULL);
    if (!dialog) {
        /* menu item hasn't been chosen yet -- create the dialog.
         * Use the toplevel as the parent because we don't want the
         * parent of a dialog to be a menu item!
         *
         * Give the dialog's title the menu item lable (arbitrary, but
         * convenient).
         *
         * You could modify this open_dialog to pass
         * all necessary data in the form of a new data structure as the
         * client data.  This structure should contain the dialog creation
         * function and other stuff as needed.
         */
        dialog = XmCreateFileSelectionDialog(toplevel, "dialog", NULL, 0);
        XtVaSetValues(XtParent(dialog), XmNtitle, XtName(w), NULL);

	/* create a option button*/
	work_area = XtVaCreateWidget("option_btn",
		xmRowColumnWidgetClass,	    dialog,
		XmNnumColumns,              1,
		XmNpacking,		    XmPACK_COLUMN,
		NULL);		     
	Makebtn(work_area, option_btn_data);
	XtManageChild(work_area);

	switch (sfx) {
	case MOL:
	    XtAddCallback(dialog, XmNokCallback, get_molnm, w);
	    break;
	case PARA:
	    XtAddCallback(dialog, XmNokCallback, get_parnm, w);
	    break;
	default:
	    break;
	}

        XtAddCallback(dialog, XmNcancelCallback, XtUnmanageChild, w);

        /* store the newly created dialog in the XmNuserData for the menu
         * item for easy retrieval next time. (see get-values above.)
         */
        XtVaSetValues(w, XmNuserData, dialog, NULL);
    }
    XtManageChild(dialog);

    /* call XtPopup() for good form... */
    XtPopup(XtParent(dialog), XtGrabNone);
    /* If the dialog was already open, XtPopup does nothing.  In
     * this case, at least make sure the window is raised to the top
     * of the window tree (or as high as it can get).
     */
    XRaiseWindow(XtDisplay(dialog), XtWindow(XtParent(dialog)));
}

void
open_warning(w)
Widget w;
{
    static Widget dialog;
    XmString t;
    extern void dlg_callback();

    /* See if we've already created this dialog -- if so,
     * we don't need to create it again.  Just re-pop it up.
     */
    if (!dialog) {
        Arg args[2];
        XmString ok = XmStringCreateSimple("OK");
        XtSetArg(args[0], XmNautoUnmanage, False);
        XtSetArg(args[1], XmNcancelLabelString, ok);
        dialog = XmCreateInformationDialog(w, "warning", args, 2);
        XtAddCallback(dialog, XmNcancelCallback, dlg_callback, NULL);
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    }
    t = XmStringCreateSimple("You must select a molecule file first!");
    XtVaSetValues(dialog,
        XmNmessageString,    t,
        XmNdialogStyle,      XmDIALOG_FULL_APPLICATION_MODAL,
        NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}

void
dlg_callback(dialog, client_data, cbs)
Widget dialog;
XtPointer client_data;
XmAnyCallbackStruct *cbs;
{
    XtPopdown(XtParent(dialog));
}

void
quit_option(w, client_data, cbs)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cbs;
{

    XtDestroyWidget(GetTopShell(w));
}

void
open_info(w)
Widget w;           /* the menu item that was invoked */
{
    static Widget dialog;
    XmString t;
    extern void dlg_callback();

    /* See if we've already created this dialog -- if so,
     * we don't need to create it again.  Just re-pop it up.
     */
    if (!dialog) {
        Arg args[2];
        XmString ok = XmStringCreateSimple("BYE!");
        XtSetArg(args[0], XmNautoUnmanage, False);
        XtSetArg(args[1], XmNcancelLabelString, ok);
        dialog = XmCreateInformationDialog(w, "info", args, 2);
        XtAddCallback(dialog, XmNcancelCallback, dlg_callback, NULL);
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    }
    t = XmStringCreateSimple("Let's wait until Master \
	Gu Chi gives me to life.");
    XtVaSetValues(dialog,
        XmNmessageString,    t,
        XmNdialogStyle,      XmDIALOG_SYSTEM_MODAL,
        NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}

/* popup a info box to tell the programe is not written yet. It will
 * be replaced by the call to programm itself soon.
 */
void
open_sorry(w)
Widget w;           /* the menu item that was invoked */
{
    static Widget dialog;
    XmString t;
    extern void dlg_callback();

    /* See if we've already created this dialog -- if so,
     * we don't need to create it again.  Just re-pop it up.
     */
    if (!dialog) {
        Arg args[2];
        XmString ok = XmStringCreateSimple("LET ME GO!");
        XtSetArg(args[0], XmNautoUnmanage, False);
        XtSetArg(args[1], XmNcancelLabelString, ok);
        dialog = XmCreateInformationDialog(w, "sorry", args, 2);
        XtAddCallback(dialog, XmNcancelCallback, dlg_callback, NULL);
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    }
    t = XmStringCreateLtoR("ARRGGGHHHH...\n\
	\n\
	:=}",
	XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(dialog,
        XmNmessageString,    t,
        XmNdialogStyle,      XmDIALOG_PRIMARY_APPLICATION_MODAL,
        NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}

void
open_help(w)
Widget w;           /* the menu item that was invoked */
{
    static Widget dialog;
    XmString t;
    extern void dlg_callback();

    /* See if we've already created this dialog -- if so,
     * we don't need to create it again.  Just re-pop it up.
     */
    if (!dialog) {
        Arg args[2];
        XmString ok = XmStringCreateSimple("Good Luck!");
        XtSetArg(args[0], XmNautoUnmanage, False);
        XtSetArg(args[1], XmNcancelLabelString, ok);
        dialog = XmCreateInformationDialog(w, "Help", args, 2);
        XtAddCallback(dialog, XmNcancelCallback, dlg_callback, NULL);
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_OK_BUTTON));
        XtUnmanageChild(
            XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));
    }
    t = XmStringCreateLtoR("Rule of Thumb: \n\
	Don't panic and \n\
	Ask Dr. Ponder to draw a \n\
	Molecule for you.",
	XmSTRING_DEFAULT_CHARSET);
    XtVaSetValues(dialog,
        XmNmessageString,    t,
        XmNdialogStyle,      XmDIALOG_PRIMARY_APPLICATION_MODAL,
        NULL);
    XmStringFree(t);
    XtManageChild(dialog);
    XtPopup(XtParent(dialog), XtGrabNone);
}

Widget
GetTopShell(w)
Widget w;
{
    while (w && !XtIsWMShell(w))
        w = XtParent(w);
    return w;
}


/* dialog box for option selections, then dispatch the application.
 * Haven't decided yet to use a DialogShell or TopLevelShell. Maybe
 * user will want to iconify the box for later use
 */
void
open_options(w)
Widget w;
{
    Widget option_dialog = NULL;
    Widget pane, form_ctrl, form_action, widget;
    Widget frame, check_box, txt_box, action_box;
    Widget rowcol, from_w, to_w;
    Widget label;
    XmString str = XmStringCreateSimple("Type in Number of Atoms Here");
    TxtWins *txt_wins;

    option_dialog = XtVaCreatePopupShell("Analyze",
	xmDialogShellWidgetClass,	GetTopShell(w),
	XmNdeleteResponse,		XmDESTROY,
	NULL);

    pane = XtVaCreateWidget("pane",
	xmPanedWindowWidgetClass,	option_dialog,		
	XmNsashWidth,			1,
	XmNsashHeight,			1,
	NULL);

    form_ctrl = XtVaCreateWidget("form_ctrl",
	xmFormWidgetClass,	pane,
	NULL);
    frame = XtVaCreateManagedWidget("frame",
        xmFrameWidgetClass,   form_ctrl,
        XmNtopAttachment,     XmATTACH_FORM,
        XmNleftAttachment,    XmATTACH_FORM,
        NULL);
    check_box = XtVaCreateWidget("check_box",
	xmRowColumnWidgetClass,	    frame,
	XmNnumColumns,              1,
	XmNpacking,		    XmPACK_COLUMN,
	NULL);
    Makebtn(check_box, analyze_I_optn_data);
    XtManageChild(check_box);

    /* create hint label... */
    rowcol = XtVaCreateWidget("label_box",
	xmRowColumnWidgetClass,	    form_ctrl,
        XmNleftAttachment,    XmATTACH_WIDGET,
 	XmNleftWidget,	      frame,
	XmNtopAttachment,     XmATTACH_OPPOSITE_WIDGET,
	XmNtopWidget,         frame,
	XmNleftOffset,	      10,    
	NULL);

    label = XtVaCreateManagedWidget("Hint",
        xmLabelGadgetClass, rowcol,
	XmNlabelString,	    str,
        NULL);

    txt_box = XtVaCreateWidget("txt_box",
	xmRowColumnWidgetClass,	    form_ctrl,
        XmNleftAttachment,    XmATTACH_WIDGET,
 	XmNleftWidget,	      frame,
	XmNtopAttachment,     XmATTACH_WIDGET,
	XmNtopWidget,         rowcol,
	XmNleftOffset,	      10,
	NULL);

    /* we need two text fields for the number of atoms */

    /* Create two Text widgets with Labels... */
    XtVaCreateManagedWidget("Atom:",
        xmLabelGadgetClass, txt_box, NULL);
    from_w = XtVaCreateManagedWidget("from_num",
        xmTextWidgetClass, txt_box,
	XmNcolumns,	   5,
	NULL);
    XtAddCallback(from_w, XmNactivateCallback,
            XmProcessTraversal, XmTRAVERSE_NEXT_TAB_GROUP);

    XtVaCreateManagedWidget("to:",
        xmLabelGadgetClass, txt_box, NULL);
    to_w = XtVaCreateManagedWidget("to_num",
        xmTextWidgetClass, txt_box,
	XmNcolumns,	   5,
	NULL);
    /* callback that filter the digit input and cast and store the
     * values 
     */
    XtAddCallback(from_w, XmNmodifyVerifyCallback, check_list, NULL);
    XtAddCallback(from_w, XmNvalueChangedCallback, check_list, NULL);
    XtAddCallback(from_w, XmNactivateCallback, check_list, FROM);
    XtAddCallback(to_w, XmNmodifyVerifyCallback, check_list, NULL);
    XtAddCallback(to_w, XmNvalueChangedCallback, check_list, NULL);
    XtAddCallback(to_w, XmNactivateCallback, check_list, TO);

    /* store a structure TxtWins as the userData of pane so that we can 
     * extract from_w and to_w later.
     */

    txt_wins = (TxtWins *)malloc(sizeof(TxtWins));
    txt_wins->label = label;
    txt_wins->from = from_w;
    txt_wins->to = to_w;
    XtVaSetValues(pane, XmNuserData, txt_wins, NULL);

    XtManageChild(rowcol);

    XtManageChild(txt_box);
    
    XtManageChild(form_ctrl);
			 
    form_action = XtVaCreateWidget("form_action",
	xmFormWidgetClass,	pane,
	XmNfractionBase,	5,		    
	NULL);
    action_box = XtVaCreateWidget("action_box",
	xmRowColumnWidgetClass,	    form_action,
	XmNnumColumns,              4,
	XmNpacking,		    XmPACK_COLUMN,
	NULL);
    Makebtn(action_box, action_area_data);
    XtManageChild(action_box);
    XtManageChild(form_action);
    {
	Dimension h;
	XtVaGetValues(action_box, XmNheight, &h, NULL);
	XtVaSetValues(form_action, XmNpaneMaximum, 3*h,
		      XmNpaneMinimum, 3*h, NULL);
    }


    XtManageChild(pane);
    XtManageChild(option_dialog);

    XtPopup(option_dialog, XtGrabNone);
}


/* callback for all ToggleButtons. It store the bit for toggle btns which
 * have been togggled.
 */
void
toggled(widget, bit, toggle_data)
Widget widget;
int bit;
XmToggleButtonCallbackStruct *toggle_data;
{
    if (toggle_data->set) /* if the toggle button is set, flip its bit */
        toggles_set |= (1 << bit);
    else /* if the toggle is "off", turn off the bit. */
        toggles_set &= ~(1 << bit);
}

void
do_analyze(w)
Widget w;
{
    int i;
    char *type;
    char tmp_name[60];	/* temp name for mol_file */
    char *tmp_filenm;	/* temp name for output, will be removed after run */
    int len;
    FILE *fld;
    int n;
    Widget shell = GetTopShell(w);


    printf("Before write_key mol_file_name is %s \n", mol_file_name);

    write_key();

    for (n = 0; n < 60; n++) {
	tmp_name[n] = ' ';
    }

    type[0] = '\0';


    /* There was a fuss over the passing of mol_file_name to analyze_().
     * char pointer seems dosen't work, because it always cause adding
     * a strange tail to the mol_file_name passed. We fixed it by declare
     * mol_file_name as a char "array" with length 60. 
     *
     * Also worth to notice that because the mol_file_name passesd usually
     * is less that 60, so pass a length parameter 60 to analyze_()
     * will cause a segmentation fault. My guess is that when the length
     * does not match you let the machine add the length automatically.
     */

    printf("length of mol_file_name is: %i \n", strlen(mol_file_name));
    fprintf(stdout, "on stdout now mol_file_name is: %s \n", mol_file_name);


    strcpy(tmp_name, mol_file_name); /* seems analyze_() eats up 
				      *	mol_file_name
				      * after execution, so we need a tmp_name
				      * for it. mol_file_name stays untouched.
				      */


    tmp_filenm = tempnam("/usr/tmp", "tnk"); /* generate a tmpfile name to 
					      * write output.
					      */

    fp = freopen(tmp_filenm, "w+", stdout);
    fprintf(stderr, "file name passed to analyze_() is: %s \n", tmp_name);
    analyze_(tmp_name);	/* new version of analyze_() need only one arg */

    fprintf(stdout, "analyze_() did nothing, though didn't crash either. \n");
    fflush(stdout);    /* without this fflush, text cannot be read  
			* into the textwidget. I tried that fflush(fp) 
			* and fflush(stdout) are the same.
			*/
    fclose(fp);


    fp0 = fopen(tmp_filenm, "r");
    TryDrawText(sw_text, fp0);
    unlink(tmp_filenm);	/* remove the tmp file */

    XtPopdown(shell);
}

void
reset_option()
{
    return;
}


void
change_format()
{
    return;
}
