/* interface.c  Collection of routines to build the interface for Tinker */

#include <Xm/RowColumn.h>
#include <Xm/ScrolledW.h>
#include <Xm/Text.h>
#include <Xm/FileSB.h>
#include <Xm/MenuShell.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/DrawingA.h>
#include <Xm/CascadeBG.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleBG.h>
#include <Xm/ToggleB.h>
#include "vars.h"

static Widget composite;
Widget sw_draw, sw_text;


static Widget CreateProgCtrl();
static Widget CreateDefaultCtrl();
static Widget CreateFileCtrl();
static void CreateDrawingArea();
static void CreateTextArea();

void MakeInterface();
void Makebtn();
void PostIt();

extern void do_analyze();
extern void analyze_I();
extern void analysis_II();
extern void analysis_III();
extern void analysis_IV();
extern void exit();
extern void open_dialog_box();
extern void open_info();
extern void open_options();
extern void open_sorry();
extern void open_help();
extern void toggle_checked();

static MenuItem analysis_data[] = {
	{ "Analyze", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_options, NULL, (MenuItem *)NULL },
	{ "Vibrate", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    analysis_II, NULL, (MenuItem *)NULL },
	{ "Vibrot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Superpose", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Spacefill", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Distgeom", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	NULL,
};

static MenuItem minimization_data[] = {
	{ "Minimize", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Minirot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Optimize", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Optirot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Newton", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Newtrot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	NULL,
};

static MenuItem dynamics_data[] = {
	{ "Dynamics", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Mutate", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	NULL,
};

static MenuItem conversion_data[] = {
	{ "Intedit", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Intxyz", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Pdbxyz", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Archive", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Xyzedit", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Xyzpdb", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Xyzsybyl", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Crystal", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	{ "Protein", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, NULL, (MenuItem *)NULL },
	NULL,
};

static MenuItem fitting_data[] = {
	{ "Xtalfit", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	NULL,
};

static MenuItem utility_data[] = {
	{ "Parameter", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Testhess", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Testgrad", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Testrot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Timer", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Timerot", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	NULL,
};

static MenuItem transition_data[] = {
	{ "Saddle", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	{ "Path", &xmPushButtonWidgetClass, NULL, NULL, NULL, 
	    open_info, (XtPointer)1, (MenuItem *)NULL },
	NULL,
};

static ButtonData file_ctrl_data[]  = {
    {(Widget)0, "Molecule", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, open_dialog_box, (XtPointer)MOL, (MenuItem *)NULL },
    {(Widget)0, "Parameter", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, open_dialog_box, (XtPointer)PARA, (MenuItem *)NULL },
    NULL,
};

static ButtonData prog_ctrl_data[]  = {
    {(Widget)0, "Analysis", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)analysis_data },
    {(Widget)0, "Conversion", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)conversion_data },
    {(Widget)0, "Dynamics", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)dynamics_data },
    {(Widget)0, "Fitting", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)fitting_data },
    {(Widget)0, "Minimize", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)minimization_data },
    {(Widget)0, "Transition", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)transition_data },
    {(Widget)0, "Utility", &xmPushButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, NULL, (XtPointer)NULL, (MenuItem *)utility_data },
    NULL,
};

static ButtonData info_optn_data[] = {
    {(Widget)0, "savecycle", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)11, (MenuItem *)NULL },
    {(Widget)0, "overwrite", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)12, (MenuItem *)NULL },
    {(Widget)0, "verbose", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL, 
	NULL, NULL, toggle_checked, (XtPointer)13, (MenuItem *)NULL },
    {(Widget)0, "debug", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)14, (MenuItem *)NULL },
    {(Widget)0, "octahedron", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)15, (MenuItem *)NULL },
    {(Widget)0, "water", &xmToggleButtonWidgetClass, XmMENU_POPUP, NULL,
	NULL, NULL, toggle_checked, (XtPointer)16, (MenuItem *)NULL },
    NULL,
};

String molecule[] = { "Molecule", "Format", "Default" };

String translations =
        "<Btn1Down>:   draw(down)\n\
         <Btn1Up>:     draw(up)  \n\
         <Btn1Motion>: draw(motion)";

GC gc;
Pixmap pixmap; 

Dimension width, height;


/* Build popup, option and pulldown menus, depending on the menu_type.
 * It may be XmMENU_PULLDOWN, XmMENU_OPTION or  XmMENU_POPUP.  Pulldowns
 * return the CascadeButton that pops up the menu.  Popups return the menu.
 * Option menus are created, but the RowColumn that acts as the option
 * "area" is returned unmanaged. (The user must manage it.)
 * Pulldown menus are built from cascade buttons, so this function
 * also builds pullright menus.  The function also adds the right
 * callback for PushButton or ToggleButton menu items.
 */
Widget
BuildMenu(parent, menu_type, menu_title, menu_mnemonic, items)
Widget parent;
int menu_type;
char *menu_title, menu_mnemonic;
MenuItem *items;
{
    Widget menu, cascade, widget;
    int i;
    XmString str;

    if (menu_type == XmMENU_PULLDOWN || menu_type == XmMENU_OPTION)
        menu = XmCreatePulldownMenu(parent, "_pulldown", NULL, 0);
    else if (menu_type == XmMENU_POPUP)
        menu = XmCreatePopupMenu(parent, "_popup", NULL, 0);
    else {
        XtWarning("Invalid menu type passed to BuildMenu()");
        return NULL;
    }

    /* Pulldown menus require a cascade button to be made */
    if (menu_type == XmMENU_PULLDOWN) {
        str = XmStringCreateSimple(menu_title);
	cascade = XtVaCreateManagedWidget(menu_title,
            xmCascadeButtonGadgetClass, parent,
            XmNsubMenuId,   menu,
            XmNlabelString, str,
            XmNmnemonic,    menu_mnemonic,
            NULL);
	
        XmStringFree(str);
    } else if (menu_type == XmMENU_OPTION) {
        /* Option menus are a special case, but not hard to handle */
        Arg args[2];
        str = XmStringCreateSimple(menu_title);
        XtSetArg(args[0], XmNsubMenuId, menu);
        XtSetArg(args[1], XmNlabelString, str);
        /* This really isn't a cascade, but this is the widget handle
         * we're going to return at the end of the function.
         */
        cascade = XmCreateOptionMenu(parent, menu_title, args, 2);
        XmStringFree(str);
    }

    /* Now add the menu items */
    for (i = 0; items[i].label != NULL; i++) {
        /* If subitems exist, create the pull-right menu by calling this
         * function recursively.  Since the function returns a cascade
         * button, the widget returned is used..
         */
        if (items[i].subitems)
            if (menu_type == XmMENU_OPTION) {
                XtWarning("You can't have submenus from option menu items.");
                continue;
            } else
                widget = BuildMenu(menu, XmMENU_PULLDOWN,
                    items[i].label, items[i].mnemonic, items[i].subitems);
        else
            widget = XtVaCreateManagedWidget(items[i].label,
                *items[i].class, menu,
                NULL);

        /* Whether the item is a real item or a cascade button with a
         * menu, it can still have a mnemonic.
         */
        if (items[i].mnemonic)
            XtVaSetValues(widget, XmNmnemonic, items[i].mnemonic, NULL);

        /* any item can have an accelerator, except cascade menus. But,
         * we don't worry about that; we know better in our declarations.
         */
        if (items[i].accelerator) {
            str = XmStringCreateSimple(items[i].accel_text);
            XtVaSetValues(widget,
                XmNaccelerator, items[i].accelerator,
                XmNacceleratorText, str,
                NULL);
            XmStringFree(str);
        }

        if (items[i].callback)
            XtAddCallback(widget,
                (items[i].class == &xmToggleButtonWidgetClass ||
                 items[i].class == &xmToggleButtonGadgetClass)?
                    XmNvalueChangedCallback : /* ToggleButton class */
                    XmNactivateCallback,      /* PushButton class */
                items[i].callback, items[i].callback_data);
    }

    /* for popup menus, just return the menu; pulldown menus, return
     * the cascade button; option menus, return the thing returned
     * from XmCreateOptionMenu().  This isn't a menu, or a cascade button!
     */
    return menu_type == XmMENU_POPUP? menu : cascade;
}


/* use Buildmenu to create buttons with menus attatched from a btn_set data
 * structure ( or an array of them.
 */
void
Makebtn(ctrl, btn_data)
Widget ctrl;
ButtonData *btn_data;
{
    Widget popup;
    ButtonData *ptr;

    ptr = btn_data;
    while (ptr->label) {
	if (ptr->type_of_menu == XmMENU_OPTION){
	    ptr->btn = BuildMenu(ctrl, 
		XmMENU_OPTION, ptr->label, NULL, ptr->subitems);
	    XtManageChild(ptr->btn);
	}
	else if (ptr->type_of_menu == XmMENU_POPUP) {
	    ptr->btn = XtVaCreateManagedWidget(ptr->label,
		*ptr->class,    ctrl,
		NULL);
	    if (ptr->callback) {
            XtAddCallback(ptr->btn,
                (ptr->class == &xmToggleButtonWidgetClass ||
                 ptr->class == &xmToggleButtonGadgetClass)?
                    XmNvalueChangedCallback : /* ToggleButton class */
                    XmNactivateCallback,      /* PushButton class */
                 ptr->callback, ptr->callback_data);
	    }
	    if (ptr->subitems) {
	    popup = BuildMenu(ctrl, 
			      XmMENU_POPUP, ptr->label, NULL, ptr->subitems);
	    XtVaSetValues(XtParent(popup), 
			  XmNmenuPost, "<Btn1Down>", NULL);
	    XtAddEventHandler(ptr->btn, ButtonPressMask, False, PostIt, popup);
	    }
	}
	ptr++;
    }
}


static void
PostIt(pb, popup, event)
Widget pb; /* the pushbutton -- unused here */
Widget popup;  /* the client_data passed to XtAddEventHandler() */
XButtonPressedEvent *event; /* We know it's a button press because of how
                               we registered the event handler. */
{
    if (event->button != 1)
        return;
    printf("mol_file_name is %s \n", mol_file_name);
    if (!(*mol_file_name)) {
	open_warning(pb);
	return;
    }
    /* position the menu at the location of the button press!  If we wanted
     * to position it elsewhere, we could change the x,y fields of the
     * event structure.
     */
    XmMenuPosition(popup, event);
    XtManageChild(popup);
}

/* Create the prog_control panel. */
static Widget
CreateProgCtrl(pw)
Widget pw;	/* The parent widget of prog_control. */
{
    Widget frame, prog_control, pb, popup;
    int i;

    /* Create a frame for the prog_contol panel */
    frame = XtVaCreateManagedWidget("frame",
        xmFrameWidgetClass,   pw,
        XmNtopAttachment,     XmATTACH_FORM,
        XmNleftAttachment,    XmATTACH_FORM,
        XmNtopOffset,         20,
        XmNleftOffset,        10,
        XmNmarginHeight,      6,
        NULL);

    /* Create a 2-column array of pushbuttons */
    prog_control = XtVaCreateWidget("prog_control",
        xmRowColumnWidgetClass, frame,
	XmNrowColumnType,       XmWORK_AREA,
        XmNnumColumns,          1,
        XmNpacking,             XmPACK_COLUMN,
	XmNspacing,		4,
        NULL);
    Makebtn(prog_control, prog_ctrl_data);

    XtManageChild(prog_control);
    return frame;
}


/* Create a check_box for default setting for all the applications? */
static Widget
CreateDefaultCtrl(ref, pw)
Widget ref, pw;    /* Parent widget and a attatchment widget. */
{
    int i;
    Widget frame, default_control, w;

    /* First create a frme for the check_box */
    frame = XtVaCreateManagedWidget("default_frmae",
	xmFrameWidgetClass,        pw,
        XmNleftAttachment,         XmATTACH_FORM,
        XmNleftOffset,             10,
        XmNtopAttachment,          XmATTACH_WIDGET,
        XmNtopWidget,              ref,
        XmNtopOffset,              20,
/*        XmNrightAttachment,        XmATTACH_OPPOSITE_WIDGET,
        XmNrightWidget,            ref,*/
        NULL);

    /* Then create the Check_box. */
    default_control = XtVaCreateWidget("default_control",
	xmRowColumnWidgetClass,    frame,
        XmNnumColumns,             1,
        XmNpacking,                XmPACK_COLUMN,
        NULL);

    /* Create toggle buttons */
/*    for (i = 0; i < XtNumber(de_fault); i++){
	w = XtVaCreateManagedWidget(de_fault[i],
	xmToggleButtonGadgetClass, default_control,
	NULL);/*

	/* Add callbacks for each toggle button */
	/* XtAddCallback(w, XmNValueChangedCallback, toggled, i);
	*/

/*    }*/
    /* use btn_data to generate toggles */
    Makebtn(default_control, info_optn_data);
    XtManageChild(default_control);
    return(frame);
}

/* Create file_control panel */
static Widget
CreateFileCtrl(ref, pw)
Widget ref, pw;
{
    Widget file_control;

        file_control = XtVaCreateManagedWidget("file_control",
	xmRowColumnWidgetClass,    pw,
	XmNnumColumns,             1,
        XmNpacking,                XmPACK_COLUMN,
        XmNleftAttachment,         XmATTACH_OPPOSITE_WIDGET,
        XmNleftWidget,             ref,
        XmNtopAttachment,       XmATTACH_WIDGET,
        XmNtopWidget,           ref,
        XmNtopOffset,           30,
        NULL);
    /*  Create three pushbuttons using the extern data structure */
    Makebtn(file_control, file_ctrl_data);
    XtManageChild(file_control);
    return(file_control);
}


/* Create composite control which do quit and help. */
static void
CreateCompositeCtrl(ref, pw)
Widget ref, pw;
{
    Widget pb1, pb2;

    pb1 = XtVaCreateManagedWidget("Quit",
        xmPushButtonGadgetClass, pw,
        XmNleftAttachment,    XmATTACH_OPPOSITE_WIDGET,
        XmNleftWidget,        ref,
        XmNleftOffset,        4,
        XmNbottomAttachment,  XmATTACH_FORM,
        XmNbottomOffset,      10,
        NULL);
    XtAddCallback(pb1, XmNactivateCallback, exit, NULL);

    /* Clear button -- wait till DrawingArea is created so we can use
     * it to pass as client data.
     */
    pb2 = XtVaCreateManagedWidget("Help",
        xmPushButtonWidgetClass, pw, /* if Gadget, cann't popup help box */
        XmNbottomAttachment,     XmATTACH_FORM,
        XmNbottomOffset,         10,
        XmNrightAttachment,      XmATTACH_OPPOSITE_WIDGET,
        XmNrightWidget,          ref,
        NULL);
    XtAddCallback(pb2, XmNactivateCallback, open_help, NULL);
}

/* Create the composite widget of a base window for the drawing area and
 * the drawing area itself.
 */
static void
CreateDrawingArea(ref, pw)
Widget ref;
Widget pw;
{
    XGCValues gcv;

        sw_draw = XtVaCreateManagedWidget("scrolled_win",
        xmScrolledWindowWidgetClass, pw,
        XmNwidth,                  700,
        XmNheight,                 450,
        XmNscrollingPolicy,        XmAUTOMATIC,
        XmNscrollBarDisplayPolicy, XmAS_NEEDED,
        XmNtopAttachment,          XmATTACH_FORM,
        XmNleftAttachment,         XmATTACH_WIDGET,
        XmNleftWidget,             ref,
        XmNleftOffset,             10,
        XmNrightAttachment,        XmATTACH_FORM,
        NULL);

    /* Create a DrawingArea widget.  Make it 5 inches wide by 6 inches tall.
     * Don't let it resize so the Clear Button doesn't force a resize.
     */
    drawing_a = XtVaCreateManagedWidget("drawing_a",
        xmDrawingAreaWidgetClass, sw_draw,
        XmNtranslations, XtParseTranslationTable(translations),
        XmNunitType,     Xm1000TH_INCHES,
        XmNwidth,        19000, /* 5 inches */
        XmNheight,       19000, /* 6 inches */
        XmNresizePolicy, XmNONE,  /* remain this a fixed size */
        NULL);
    
    /* Create a GC for drawing (callback).  Used a lot -- make global
     * Create  GC etc.
     */
    gcv.foreground = WhitePixelOfScreen(XtScreen(sw_draw));
    gc = XCreateGC(XtDisplay(sw_draw),
	        RootWindowOfScreen(XtScreen(sw_draw)), GCForeground, &gcv);

    /* convert drawing area back to pixels to get its width and height */
    XtVaSetValues(drawing_a, XmNunitType, XmPIXELS, NULL);
    XtVaGetValues(drawing_a, XmNwidth, &width, XmNheight, &height, NULL);
    /* create a pixmap the same size as the drawing area. */
    pixmap = XCreatePixmap(XtDisplay(drawing_a),
        RootWindowOfScreen(XtScreen(drawing_a)), width, height,
        DefaultDepthOfScreen(XtScreen(drawing_a)));
    /* clear pixmap with white */
    set_color(drawing_a, "White");
    XFillRectangle(XtDisplay(drawing_a), pixmap, gc, 0, 0, width, height);
}

/* Create the text output pane. We will put this routine in canvas.c later.*/
static void
CreateTextArea(ref, pw)
Widget ref, pw;
{
    Arg args[8];

    XtSetArg(args[0], XmNrows,	    120);
    XtSetArg(args[1], XmNcolumns,   85);
    XtSetArg(args[2], XmNeditable,  False);
    XtSetArg(args[3], XmNeditMode,  XmMULTI_LINE_EDIT);

    sw_text = XmCreateScrolledText(pw, "scrolled_text", args, 4);

    XtVaSetValues(XtParent(sw_text),
        XmNwidth,                    700,
        XmNheight,                   400,
        XmNtopAttachment,            XmATTACH_WIDGET,
        XmNtopWidget,                sw_draw,
        XmNbottomAttachment,         XmATTACH_FORM,
        XmNrightAttachment,          XmATTACH_FORM,
        XmNleftAttachment,           XmATTACH_WIDGET,
        XmNleftWidget,               ref,
        XmNleftOffset,               10,
        NULL);
    XtManageChild(sw_text);
}

/* Make the interface by create the baseframe, the prog_control panel,
 * default_control panel, file_control panel, base_control panel,
 * the drawing_a window, and the text_w window.
 */
void
MakeInterface(topwidget)
Widget topwidget;
{
     Widget Yref;


    /* Create a composite for everything. */
    composite = XtVaCreateManagedWidget("composite",
        xmFormWidgetClass, topwidget, NULL);

    /* Create the prog_control etc */
    Yref = CreateProgCtrl(composite);
    Yref = CreateDefaultCtrl(Yref, composite);
    CreateFileCtrl(Yref, composite);
    CreateCompositeCtrl(Yref, composite);
    CreateDrawingArea(Yref, composite);
    CreateTextArea(Yref, composite);
}
