#!/usr/local/bin/perl -w
# add applix index entries to tinker documentation
# pulls tags out by heuristics (see if statements below) and inserts
# index tags.
# usage: ./addindex.pl book/r*.aw book/[a-qs-z]*.aw
# routines must be done first.

$RCSId = '$Id: addindex.pl,v 0.6 1998/03/24 21:54:02 reece Exp $ ';
($jemappelle=$0) =~ s,^.*/,,;


print(STDERR '# ', $RCSId, "\n");


use Getopt::Mixed 1.006;
use vars qw( $opt_version $opt_verbose );
my(@OPTS)=
  (
   'verbose     v>verbose',
   'version     V>version',
  );
Getopt::Mixed::getOptions(@OPTS);
if ($opt_version) 
  {
  print($RCSId,"\n");
  exit(0);
  }


use FileHandle;
$ifh=new FileHandle || die("$jemappelle: couldn't allocate filehandle\n");
$ofh=new FileHandle || die("$jemappelle: couldn't allocate filehandle\n");


while($#ARGV+1)
  {
  my($fn)=shift;
  if (not $ifh->open("<$fn"))
	{
	warn("$jemappelle: $fn: $!\n");
	next;
	}
  if (not $ofh->open(">$fn.tmp"))
	{
	warn("$jemappelle: $fn: $!\n");
	next;
	}
  print(STDERR "\n* $fn ", '='x(75-3-length($fn)), "\n") if $opt_verbose;
  while(<$ifh>)
	{
	s%/user/ponder/tinker/applix/%%;

	if (m/<T \"SAMPLE\.(\w{1,8})\" bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='File Type'; $section="${type}s";
	  $ofh->print(idxentry("$match File Type"));
	  $ofh->print(idxentry("$section:$match"));
	  }

	elsif (m/<T \"(\w+)\s+(?:Program).*bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='Program'; $section="${type}s";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxentry("$section:$match"));
	  $type{$match}=$type;
	  }
   
	elsif (m/<T \"(\w+)\s+(?:Subroutine|Function).*bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='Routine'; $section="${type}s";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxentry("$section:$match"));
	  $type{$match}=$type;
	  }
   
	elsif (m/<T \"(.+)\s+Example.*bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='Example'; $section="${type}s";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxentry("$section:$match"));
	  }
   

	elsif ($fn=~/common/ && m/<T \"([A-Z]+)\^aj.*\" bold face:\"New/)
	  {
	  my($match)=$1;
	  $type='Common Block'; $section="${type}s";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxsee("$section:$match","$match $type"));
	  $cb=$match;
	  }

	elsif ($fn=~/common/ && m/<T \"([a-z]+)\^aj.+\" face:\"New/)
	  {
	  my($match)=$1;
	  $section="Common Block Variables";
	  $ofh->print(idxsee("$section:$match","$cb common block"));
	  }


	elsif ($fn=~/crossindex/ && m/<T \"([-A-Z]+)(?:^aj)?\" bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type=$type{$match};
	  if (defined $type)
		{
		$section="${type}s";
		$ofh->print(idxentry("$match $type"));
		$ofh->print(idxentry("$section:$match"));
		}
	  else
		{
		warn("$fn: $.: $match not seen before!\n");
		}
	  }

	elsif ($fn=~/keywords/ && m/<T \"([-\dA-Z]+).* bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='Keyword'; $section="${type}s";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxentry("$section:$match"));
	  }

	elsif ($fn=~/parameters/ && m/<T \"(?:^aj)?([-\w\/]{3,}).* bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type='Parameters'; $section="Parameter Sets";
	  $ofh->print(idxentry("$match $type"));
	  $ofh->print(idxsee("$section:$match","$match $type"));
	  $type{$match}=$type;
	  if (not defined $paramxrefs)
		{
		$ofh->print(idxsee("Force Fields","$section"));
		$ofh->print(idxsee("Potential Functions","$section"));
		$paramxrefs=1;
		}
	  }

	elsif (m/<T \"([A-Z\d\.\,][A-Z\d\/\.\,\s]+)\" bold face:\"New/)
	  {
	  my($match)=uc($1);
	  $type=$type{$match};
	  if (defined $type)
		{
		$section="${type}s";
		$ofh->print(idxentry("$match $type"));
		$ofh->print(idxentry("$section:$match"));
		}
	  else
		{
		$ofh->print(idxentry($match))
		}
	  }

	elsif (m/<T.+bold face:\"New/)
	  {
	  warn("$fn: $.: IGNORED: $_");
	  }

	$ofh->print($_);
	}

  $ifh->close();
  $ofh->close();
  rename($fn,"$fn.bak");
  rename("$fn.tmp",$fn);
  }
exit(0);


sub idxentry
  {
  print(STDERR $_[0],"\n") if $opt_verbose;
return
'<S_F fieldType:12
<start_eval_value>
<T "--field--">
<end_eval_value>
>
<T "index \"'.$_[0].'\" -span">
<FV>
<E_F>
'
  }

sub idxsee
  {
  print(STDERR $_[0],' -> ',$_[1],"\n") if $opt_verbose;
return
'<S_F fieldType:12
<start_eval_value>
<T "--field--">
<end_eval_value>
>
<T "index \"'.$_[0].'\" -span -see \"'.$_[1].'\" ">
<FV>
<E_F>
'
  }

