c
c
c
c     ##############################################
c     ##                                          ##
c     ##  subroutine ace  --  parameters for ACE  ##
c     ##                                          ##
c     ##############################################
c
c
c     "ace" sets the atom type specific parameters for the pairwise
c     analytital continuum electrostatics solvation model

c
c     literature references:
c
c     M. Schaefer and M. Karplus, "A Comprehensive Treatment of
c     Continuum Electrostatics", J. Phys. Chem., (1990) 100,
c     1578-1599. 
c
c
      subroutine ace
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'iounit.i'
      include 'kace.i'
      include 'math.i'
      include 'solute.i'
      integer i,k,ic,kc
      real*8 ratio,rirk,qik,fik,tik2,rk,rk2,ri2,vk,ri,pi2
      real*8 prod,prod2,prod4,s4ri,sigik2,sigik3,muik
      real*8 alphik,alphik2,alphik4,omgik
      real*8 tmp1,tmp2,c1,c2,c3,diff1,diff2,term

c 
c     set all zero effective radii to 1
      do i = 1,n
        ic = class(i)
        if(rvorn(ic) .eq. 0.0d0) rvorn(ic) = 1.0d0
        if(vvorn(ic) .eq. 0.0d0) vvorn(ic) = 4.0d0*pi/3.0d0
      end do
c     
      c1 = 0.42441318157839d0
      c2 = 0.66816794187147d0
      c3 = 11.13665599366342d0
      pi2 = 1.0d0/pi**2
c
c     calculate the pairwise parameters for the ace self energy
      do i = 1, n
        ic = class(i)
        ri = rsolv(i)
c       R_i**2
        ri2 = ri**2
        do k = i, n
          kc = class(k)
c         R_k**2
          rk = rvorn(kc)
          vk = vvorn(kc)
          rk2 = rk**2
c         alpha_ik
          rirk = ri/rk
          alphik = max(width,rirk)
          alphik2 = alphik**2
          prod = alphik*rk
          prod2 = prod**2
          prod4 = prod**4
          alphik4 = alphik2**2
          s4ri = alphik4*ri
c         theta_ik**2
          ratio = alphik2*rk2/ri2
          tik2 = 0.5d0*pi*ratio
c         f_ik
          tmp1 = 2.0d0/(tik2 + 1.0d0)
          tmp2 = 1.0d0/(2.0d0*tik2 + 1.0d0)
          fik = tmp1 - tmp2
c         Q_ik
          qik = tik2*sqrt(tmp2)
          diff1 = qik - atan(qik)
          diff2 = (3.0d0+fik)*qik - 4.0d0*atan(qik)
c         1/omega_ik
          if(k .ne. i) then
            omgik = vk*diff1*pi2/prod4
          else
            omgik = c1*diff1/s4ri
          end if
c         gamma_ik**2
          sigik2 = 3.0d0*diff1*prod2/diff2
          sigik3 = sigik2*sqrt(sigik2)
          term = 1.0d0 - (c3*sigik3*ri*omgik/vk)
c         mu_ik
          muik = c2*ri/term
c         copy all calculated parameters to global arrays
          alpha(ic,kc) = alphik
          alpha(kc,ic) = alphik
          sigma2(kc,ic) = sigik2
          sigma2(ic,kc) = sigik2
          omega(ic,kc) = omgik
          omega(kc,ic) = omgik
          mu(kc,ic) = muik
          mu(ic,kc) = muik
        end do
      end do
      return
      end
      
