c
c
c
c     ######################################################
c     ##                                                  ##
c     ##  subroutine born1  --  gradients for Born radii  ##
c     ##                                                  ##
c     ######################################################
c
c
c     "born1" computes the gradients for Born radii of each atom using
c     the analytical form in ACE
c
c
c     M. Schaefer and M. Karplus, "A Comprehensive Treatment of
c     Continuum Electrostatics", J. Phys. Chem., (1990) 100,
c     1578-1599  
c
c
      subroutine born1
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'dborn.i'
      include 'kace.i'
      include 'math.i'
      include 'solute.i'
      include 'units.i'
      integer i,k,ii,kk
      real*8 xi,yi,zi,xr,yr,zr,vk,vi
      real*8 rik,rik2,rik3,rik4,rik6,omgik,sigik2,ratio3
      real*8 muik4,omgs2,rmusum,rmusum2,rbi,rbi2,ratio
      real*8 c1,expterm,drb,drb1,drb21,drb22,drbdx,drbdy,drbdz
      
c     get gradients for the effective born radii using ACE
      do i = 1,n
        do k = 1,n
          drbx(k,i) = 0.0d0
          drby(k,i) = 0.0d0
          drbz(k,i) = 0.0d0
        end do
      end do
c     
      call born
      c1 = 1.0d0/pi
      do ii = 1,n
        xi = x(ii)
        yi = y(ii)
        zi = z(ii)
        i = class(ii)
        rbi = rborn(ii)
        rbi2 = rbi*rbi
        vi = vvorn(i)
        do kk = 1,n
          if(kk .ne. ii) then
            xr = xi - x(kk)
            yr = yi - y(kk)
            zr = zi - z(kk)
            k = class(kk)
            vk = vvorn(k)
            omgik = omega(i,k)
            sigik2 = 1.0d0/sigma2(i,k)
            omgs2 = omgik*sigik2
            muik4 = mu(i,k)**4
            rik = sqrt(xr**2 + yr**2 + zr**2)
            rik2 = rik*rik
            rik3 = rik2*rik
            rik4 = rik2*rik2
            rik6 = rik4*rik2
            rmusum = rik4 + muik4
            rmusum2 = rmusum**2
            ratio = rik3/rmusum
            ratio3 = ratio**3
            expterm = exp(-rik2*sigik2)
            
            drb1 = -4.0d0*rik*omgs2*expterm
            drb21 = 3.0d0*rik2/rmusum
            drb22 = -4.0d0*rik6/rmusum2
            drb = rbi2*(drb1 + c1*vk*ratio3*(drb21 + drb22))
            
            drb = drb / rik
            
            drbdx = drb * xr
            drbdy = drb * yr
            drbdz = drb * zr
            
c           drb(ii)/dxii
            drbx(ii,ii) = drbx(ii,ii) + drbdx
            drby(ii,ii) = drby(ii,ii) + drbdy
            drbz(ii,ii) = drbz(ii,ii) + drbdz
            
c           drb(ii)/dxkk
            drbx(kk,ii) = drbx(kk,ii) - drbdx
            drby(kk,ii) = drby(kk,ii) - drbdy
            drbz(kk,ii) = drbz(kk,ii) - drbdz
c           
          end if
        end do
      end do
      return
      end
      
