c
c
c
c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine eace1  --  generalized Born energy for ACE  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "eace1" calculates the generalized Born energy and first
c     derivatives with respect to Cartesian coordinates for the
c     ACE solvation model
c
c
      subroutine eace1 (epol,depol)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'atmtyp.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'charge.i'
      include 'dborn.i'
      include 'inter.i'
      include 'math.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'solute.i'
      include 'units.i'
      include 'usage.i'
      include 'virial.i'
      integer i,j,k,ii,kk,jj
      real*8 epol,e,f,fi,fik,fgb,fgb3,rb,rb2,rm2
      real*8 xi,yi,zi,xr,yr,zr,rbi,rbk,pi2,pj,pj2
      real*8 dedxi,dedyi,dedzi,dedxk,dedyk,dedzk
      real*8 de1,de2,dwater,sqrtik,sqrtki
      real*8 shift,taper,dtaper,trans,dtrans
      real*8 r,r2,r3,r4,r5,r6,r7,expterm
      real*8 depol(3,maxatm)
      logical iuse
c     
c     
c     zero out the generalized Born polarization energy and derivatives
c     
      epol = 0.0d0
      do i = 1, n
        depol(1,i) = 0.0d0
        depol(2,i) = 0.0d0
        depol(3,i) = 0.0d0
      end do
      if (nion .eq. 0)  return
c     
c     set the dielectric constant and energy conversion factor
c     
      dwater = 78.3d0
      f = -0.5d0*electric * (1.0d0 - 1.0d0/dwater)
c     
c     set cutoff distances and switching function coefficients
c     
      call switch ('CHARGE')
c     
c     
c     calculate gradients of individual Born radii 
      call born1
      
c     calculate GB electrostatic polarization energy term and gradients
      do ii = 1, nion
        i = iion(ii)
        iuse = use(i)
        xi = x(i)
        yi = y(i)
        zi = z(i)
        fi = f * pchg(ii)
        rbi = rborn(i)
        do kk = 1, nion
          k = iion(kk)
          if (iuse .or. use(k)) then
            rbk = rborn(k)
            xr = xi - x(k)
            yr = yi - y(k)
            zr = zi - z(k)
            if (use_image)  call image (xr,yr,zr,0)
            r2 = xr*xr + yr*yr + zr*zr
            r = sqrt(r2)
c
c           calculate the prefactors for the mixed terms
c
            if (r2 .le. off2) then
              fik = fi * pchg(kk)
              rb2 = rbi*rbk
              rb = sqrt(rb2)
              expterm = exp(-0.25d0*r2/rb2)
              fgb = sqrt(r2 + rb2*expterm)
              fgb3 = fgb**3
              e = fik / fgb
              de1 = -fik*(r - 0.25d0*r*expterm)/fgb3
              de2 = -fik*(rb + 0.25d0*r2/rb)*expterm/fgb3
c             
c             use energy switching if near the cutoff distance
c             
              rm2 = (0.5d0 * (off+cut))**2
              shift = fik / sqrt(rm2 + rb2*exp(-0.25d0*rm2/rb2))
              e = e - shift
              if (r2 .gt. cut2) then
                r3 = r2 * r
                r4 = r2 * r2
                r6 = r3 * r3
                r5 = r2 * r3
                r7 = r3 * r4
                taper = c5*r5 + c4*r4 + c3*r3
     &                        + c2*r2 + c1*r + c0
                dtaper = 5.0d0*c5*r4 + 4.0d0*c4*r3 + 
     &                   3.0d0*c3*r2 + 2.0d0*c2*r + c1
                trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4 + 
     &                         f3*r3 + f2*r2 + f1*r + f0)
                dtrans = fik * (7.0d0*f7*r6 + 6.0d0*f6*r5 + 
     &                          5.0d0*f5*r4 + 4.0d0*f4*r3 + 
     &                          3.0d0*f3*r2 + 2.0d0*f2*r + f1)
                de1 = e*dtaper + de1*taper + dtrans
                de2 = de2*taper
                e = e * taper + trans
              end if
c
c             increment the overall energy 
c             
              epol = epol + e
c             
c             for interaction energies compute pairwise gradients
c             as well as gradients on atoms that contribute to 
c             the Born radii
              if (i .ne. k) then
                sqrtki = 0.5d0*sqrt(rbk/rbi)
                sqrtik = 0.5d0*sqrt(rbi/rbk)
                de1 = de1 / r  
c
c               gradients for atom i in the pairwise interaction 
                dedxi = de1*xr + de2*sqrtki*drbx(i,i) + 
     &                           de2*sqrtik*drbx(i,k)
                dedyi = de1*yr + de2*sqrtki*drby(i,i) + 
     &                           de2*sqrtik*drby(i,k)
                dedzi = de1*zr + de2*sqrtki*drbz(i,i) + 
     &                           de2*sqrtik*drbz(i,k)

c               gradients for atom k in the pairwise interaction 
                dedxk = -de1*xr + de2*sqrtki*drbx(k,i) + 
     &                            de2*sqrtik*drbx(k,k)
                dedyk = -de1*yr + de2*sqrtki*drby(k,i) + 
     &                            de2*sqrtik*drby(k,k)
                dedzk = -de1*zr + de2*sqrtki*drbz(k,i) + 
     &                            de2*sqrtik*drbz(k,k)

c               increment the overall derivatives for the pair
                depol(1,i) = depol(1,i) + dedxi  
                depol(2,i) = depol(2,i) + dedyi
                depol(3,i) = depol(3,i) + dedzi
                
                depol(1,k) = depol(1,k) + dedxk 
                depol(2,k) = depol(2,k) + dedyk
                depol(3,k) = depol(3,k) + dedzk
c
c               increment derivatives for atoms not in the pair that 
c               contribute to the Born radii
                do j = 1,n
                  if((j .ne. i) .and. (j .ne. k)) then
                   depol(1,j) = depol(1,j) + de2*sqrtki*drbx(j,i)  
     &                                     + de2*sqrtik*drbx(j,k)
                   depol(2,j) = depol(2,j) + de2*sqrtki*drby(j,i)  
     &                                     + de2*sqrtik*drby(j,k)
                   depol(3,j) = depol(3,j) + de2*sqrtki*drbz(j,i)  
     &                                     + de2*sqrtik*drbz(j,k)
                 end if
               end do
             else
c              get the self energy derivatives in terms of all   
c              charged atoms that define the Born radius for atom i 
               dedxi = 0.0d0
               dedyi = 0.0d0
               dedzi = 0.0d0
               do jj = 1,nion
                 j = iion(jj)
                 pj = pchg(jj)
                 pj2 = pj*pj
                 rb2 = rborn(j)*rborn(j)
                 de2 = -f*pj2/rb2
                 dedxi = dedxi + de2*drbx(i,j)
                 dedyi = dedyi + de2*drby(i,j)
                 dedzi = dedzi + de2*drbz(i,j)
                end do
c   
c               increment self energy derivatives for atom i
c             
                depol(1,i) = depol(1,i) + dedxi  
                depol(2,i) = depol(2,i) + dedyi
                depol(3,i) = depol(3,i) + dedzi
c
c               increment the derivatives for all non-charged 
c               atoms that define the Born radii i 

                do jj = 1,zrion
                  j = kzion(jj)
                  pi2 = pchg(ii)*pchg(ii)
                  rb2 = rborn(i)*rborn(i)
                  de2 = -f*pi2/rb2
                  depol(1,j) = depol(1,j) + de2*drbx(j,i)
                  depol(2,j) = depol(2,j) + de2*drby(j,i)
                  depol(3,j) = depol(3,j) + de2*drbz(j,i)
                end do
              end if
c
c             increment the virial for use in pressure computation
c             
              if (isobaric) then
                virx = virx + xr*dedxi
                viry = viry + yr*dedyi
                virz = virz + zr*dedzi
              end if
c             
c             increment the total intermolecular energy
c             
              if (molcule(i) .ne. molcule(k)) then
                einter = einter + e
              end if
            end if
          end if
        end do
      end do

c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     this part needs to be inserted in accord with the gradients above
      return
      end
