c
c
c     ##########################################################
c     ##                                                      ##
c     ##  program testgborn  --  derivative test; Born radii  ##
c     ##                                                      ##
c     ##########################################################
c
c
c     "testgborn" computes and compares the analytical and numerical
c     gradient vectors of the Born radii
c
c
      program testgborn
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'iounit.i'
      include 'kace.i'
      include 'solute.i'
      include 'potent.i'
      integer i,j,k
      real*8 old,nrbx(maxatm,maxatm),nrby(maxatm,maxatm)
      real*8 nrbz(maxatm,maxatm),rb0(maxatm),rb1(maxatm)
      real*8 rold(maxatm),eps,rb20,rb21,nrb12,grb12
c
c     set up the structure and mechanics calculation
c
      call initial
      call getxyz
      call mechanic
c
      eps = 1.0d-4
      use_ace = .true.
      width = 1.90d0  
      call born1 
c

c     get the x-component two-sided numerical gradient
c
      do i = 1, n
        old = x(i)
        x(i) = x(i) - 0.5d0*eps
        call born
        do k = 1,n
           rb0(k) = rborn(k)
        end do
        x(i) = x(i) + eps
        call born
        do k = 1,n
           rb1(k) = rborn(k)
        end do
        do k = 1,n
           nrbx(i,k) = (rb1(k) - rb0(k))/eps
        end do
        x(i) = old
c
c     get the y-component two-sided numerical gradient
c
        old = y(i)
        y(i) = y(i) - 0.5d0*eps
        call born
        do k = 1,n
           rb0(k) = rborn(k)
        end do
        y(i) = y(i) + eps
        call born
        do k = 1,n
           rb1(k) = rborn(k)
        end do
        do k = 1,n
           nrby(i,k) = (rb1(k) - rb0(k))/eps
        end do
        y(i) = old
c     get the z-component two-sided numerical gradient
        old = z(i)
        z(i) = z(i) - 0.5d0*eps
        call born
        do k = 1,n
           rb0(k) = rborn(k)
        end do
        z(i) = z(i) + eps
        call born
        do k = 1,n
           rb1(k) = rborn(k)
        end do
        do k = 1,n
           nrbz(i,k) = (rb1(k) - rb0(k))/eps
        end do
        z(i) = old
      end do
c
      do i = 1,n
         do j = 1,n
             write(iout,10)i,j,drbx(i,j),drby(i,j),drbz(i,j),
     &                     nrbx(i,j),nrby(i,j),nrbz(i,j),
     &                     abs(drbx(i,j)-nrbx(i,j)),
     &                     abs(drby(i,j)-nrby(i,j)),
     &                     abs(drbz(i,j)-nrbz(i,j))
   10        format(2i4,4x,3f12.4,/12x,3f12.4,/7x,'Diff ',3f12.4,/)
         end do
      end do
	
c     
      old = x(1)
      x(1) = x(1) - 0.5d0*eps
      call born
      rb20 = sqrt(rborn(1)*rborn(2))
      x(1) = x(1) + eps
      call born
      x(1) = old
      rb21 = sqrt(rborn(1)*rborn(2))
      nrb12 = (rb21 - rb20)/eps
      call born
      grb12 = 0.5d0*sqrt(rborn(2)/rborn(1))*drbx(1,1) + 
     &        0.5d0*sqrt(rborn(1)/rborn(2))*drbx(1,2)
      write(iout,20)rborn(1),rborn(2),nrb12,grb12
   20 format(4f12.4)
 
	         	         
	call final
      end
