c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine gridfit  --  approximate fit via grid search  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "gridfit" performs a crude fit on two coordinate sets,
c     used to get an initial least squares fit for refinement;
c     this procedure may be omitted if the coordinate sets are
c     already aligned approximately correctly
c
c
      subroutine gridfit (x1,y1,z1,n2,x2,y2,z2)
      implicit none
      include 'sizes.i'
      include 'math.i'
      integer i,j,k,m,n2,nrot
      real*8 rmsfit,rmsvalue,rmsold
      real*8 angle,angfac,weight
      real*8 xnew,ynew,znew
      real*8 xrot,yrot,zrot
      real*8 xcos,ycos,zcos
      real*8 xsin,ysin,zsin
      real*8 x1(maxatm),y1(maxatm),z1(maxatm)
      real*8 x2(maxatm),y2(maxatm),z2(maxatm)
      real*8 xinit(maxatm),yinit(maxatm),zinit(maxatm)
      real*8 xnext(maxatm),ynext(maxatm),znext(maxatm)
      real*8 xlast(maxatm),ylast(maxatm),zlast(maxatm)
      real*8 xbest(maxatm),ybest(maxatm),zbest(maxatm)
c
c
c     set the number of rotations about each axis
c
      angle = 45.0d0
      nrot = nint(360.0d0/angle)
      angfac = angle / radian
      rmsold = 1000000.0d0
c
c     store initial ccordinates for molecule to be rotated
c
      do i = 1, n2
         xinit(i) = x2(i)
         yinit(i) = y2(i)
         zinit(i) = z2(i)
      end do
c
c     find the best fit among the possible rotations
c
      do i = 1, nrot
         xrot = angfac * i
         xcos = cos(xrot)
         xsin = sin(xrot)
         do j = 1, n2
            ynew = yinit(j) * xcos - zinit(j) * xsin
            znew = yinit(j) * xsin + zinit(j) * xcos
            xnext(j) = xinit(j)
            ynext(j) = ynew
            znext(j) = znew
         end do
         do j = 1, nrot
            yrot = angfac * j
            ycos = cos(yrot)
            ysin = sin(yrot)
            do k = 1, n2
               xnew = xnext(k) * ycos - znext(k) * ysin
               znew = xnext(k) * ysin + znext(k) * ycos
               xlast(k) = xnew
               ylast(k) = ynext(k)
               zlast(k) = znew
            end do
            do k = 1, nrot
               zrot = angfac * k
               zcos = cos(zrot)
               zsin = sin(zrot)
               do m = 1, n2
                  xnew = xlast(m) * zcos - ylast(m) * zsin
                  ynew = xlast(m) * zsin + ylast(m) * zcos
                  x2(m) = xnew
                  y2(m) = ynew
                  z2(m) = zlast(m)
               end do
               rmsvalue = rmsfit (x1,y1,z1,x2,y2,z2)
               if (rmsvalue .lt. rmsold) then
                  do m = 1, n2
                     xbest(m) = x2(m)
                     ybest(m) = y2(m)
                     zbest(m) = z2(m)
                  end do
                  rmsold = rmsvalue
               end if
            end do
         end do
      end do
c
c     transfer best fit coordinates for rotated molecule
c
      do i = 1, n2
         x2(i) = xbest(i)
         y2(i) = ybest(i)
         z2(i) = zbest(i)
      end do
      return
      end
