c
c
c     ############################################################
c     ##                                                        ##
c     ##   subroutine lupgrdchk  --  test analytical gradients  ##
c     ##                                                        ##
c     ############################################################
c
c
c      subroutine "lupgrdchk" can be used to check the
c      analytical gradients for LUP
c
c
c
      subroutine lupgrdchk(nn,x,lup1)
      implicit none
      include 'sizes.i'
      include 'pathsz.i'
      include 'lup.i'
      integer i,j,k,nn,ibase
      real*8 anal_g(3*maxatm),num_g(3*maxatm)
      real*8 num_proj(3*maxatm)
      real*8 fval,lup1
      real*8 fx1,fx2,numderiv
      real*8 x(nn),xtmp,xh,h,two
      real*8 dum(3*maxatm),xtrial(3*maxatm)
      real*8 ovlp
      external lup1
      real*8 db,dc,b2,b1,cterm1,cterm2,res
c
c     check constraints if desired
c
c     call numgc(nn,x,lup1)
c
      do i = 1, 3*maxatm
         anal_g(i) = 0.0d0
         num_g(i) = 0.0d0
      end do
c
c     get analytical energy gradient
c
      write(50,*) 'in lupgrdchk'
      fval=lup1(x,anal_g)
c
c     get numerical approximation to gradient
c
      h=0.00001d0
      two=2.0d0
c
c     get numerical derivatives
c     wrt atomic coordinates
c
      do j = 1, nn
      xtmp=x(j)
      xh=x(j)+h
      x(j)=xh
      fx1=lup1(x,dum)
      xh=xtmp-h
      x(j)=xh
           if (j.eq.1) then
           b1 = potnrg
           cterm1 = target-potnrg
           endif
      fx2=lup1(x,dum)
           if (j.eq.1) then
           b2 = potnrg
           cterm2 = target-potnrg
           db = (b1-b2)/two/h
           dc = (cterm1 -cterm2)/two/h
           res = db+dc
           write(50,'(a,3f8.4)') 'num_g,cterm,', db,dc,res
           endif
      num_g(j)=(fx1-fx2)/two/h
      x(j)=xtmp
      end do
c
c     restore original quantities in common block arrays
c
      dum(1)=lup1(x,dum)
c
c     from these numerically obtained derivatives
c     project the direction of the path
c
      ovlp = 0.0d0
      do j = 1, nn
         ovlp = ovlp + num_g(j)*pvect(j,izet)
      end do
      write(50,*) 'num_g ovlp qj ',num_g(1),ovlp,pvect(1,izet)
      do j = 1, nn
         num_proj(j) = num_g(j) -ovlp*pvect(j,1)
      end do
      do j=1,nn
      write(50,'(i,4(f12.8,2x))') j,num_g(j),num_proj(j),anal_g(j)
      end do
      write(50,*) 'end of first method'
c
      stop
      return
      end
c
c
      subroutine numgc(nn,x,lup1)
      implicit none
      include 'sizes.i'
      include 'pathsz.i'
      include 'lup.i'
      integer i,j,k,nn,ibase
      real*8 anal_g(3*maxatm,ncmax),num_g(3*maxatm,ncmax)
      real*8 num_proj(3*maxatm,ncmax)
      real*8 fval,lup1
      real*8 fx1,fx2
      real*8 x(nn),xtmp,xh,h,two
      real*8 dum(3*maxatm),xtrial(3*maxatm)
      real*8 ovlp
      external lup1
      real*8 c1(ncmax),c2(ncmax)
c
      do i = 1, ncmax
      do j = 1, 3*maxatm
         anal_g(j,i) = 0.0d0
         num_g(j,i) = 0.0d0
         num_proj(j,i) = 0.0d0
      end do
      end do
c
c     get analytical gradient
c
      do i = 1, ncmax
      do j = 1, 3*maxatm
         anal_g(j,i) = gc(j,i)
      end do
      end do
c
c     get numerical approximation to gradient
c

      h=0.00001d0
      two=2.0d0
c
c     get numerical derivatives
c     wrt atomic coordinates
c
c
      do j = 1, 2
      xtmp=x(j)
      xh=x(j)+h
      x(j)=xh
      call lupcc(x)
      do i = 1, nsix
         c1(i) = cnst(i)
      end do
      xh=xtmp-h
      x(j)=xh
      call lupcc(x)
      do i = 1, nsix
         c2(i) = cnst(i)
      end do
      do i = 1, nsix
      num_g(j,i)=(c1(i)-c2(i))/two/h
c     write(50,'(a,2i,3e23.15)') 'the cnsts;',j,i,c1(i),c2(i)
c    :           ,(c2(i)-c1(i))/h
      end do
      x(j)=xtmp
      end do
c
c     restore original quantities in common block arrays
c
      dum(1)=lup1(x,dum)
      write(50,*) ' constraint gradients '
      do i = 1, nsix
         do j=1,2
c           write(50,'(2i,4(f12.8,2x))') j,i,num_g(j,i)
c    :                                     ,anal_g(j,i)
         end do
      end do
c
      stop
      return
      end
