c
c
c     ########################################################
c     ##                                                    ##
c     ##  subroutine mincheck  --  minimize SPW results     ##
c     ##                                                    ##
c     ########################################################
c
c
c     subroutine "mincheck" minimizes the individual pathpoint
c     energies.
c
c
      subroutine mincheck(spwp)
      implicit none
      include 'sizes.i'
      include 'output.i'
      include 'files.i'
      include 'pathsz.i'
      include 'pathcoord.i'
      include 'pathdata.i'
      include 'pathtol.i'
      include 'spw.i'
      integer nummin,imin
      real*8 emin(maxpp+2),fctval
      real*8 mincheck1,dele
      real*8 spwp(nat3,0:lpath+1)
      logical newmin
      external mincheck1,mincheck2,mincheckout
c
c
      savecycle = .true.
      if (grdmin.gt.0.01d0) grdmin = 0.01d0
      nummin = 2
      emin(1) = ptnl(0)
      emin(2) = ptnl(lpath+1)
      do imin = 3,maxpp+2
         emin(imin) = 0.0d0
      end do
      do izet = 1, lpath
         call tncg('tncg','auto',nat3,spwp(1,izet),fctval,grdmin
     &                       ,mincheck1,mincheck2,mincheckout)
c        call mqn(nat3,spwp(1,izet),fctval,grdmin
c    &                       ,mincheck1,mincheckout)
         newmin = .true.
         do imin = 1, nummin
            dele = fctval - emin(imin)
            dele = abs(dele)
            if ( dele .lt. (10.0d0*grdmin) ) newmin = .false.
         end do
         if (newmin) then
            nummin = nummin + 1
            emin(nummin) = fctval
            call writeout(spwp(1,izet),izet)
            write(6,*) 'file written for ',izet,'nrg =',fctval
         endif
      end do
      return
      end
c
c
      function mincheck1(p,deriv)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'pathsz.i'
      include 'pathdata.i'
      integer i
      real*8 p(nat3),mincheck1
      real*8 energy,deriv(3,maxatm)
c
c     copy current coordinates to TINKER
c
      do i = 1, n
         x(i) = p(3*i-2)
         y(i) = p(3*i-1)
         z(i) = p(3*i)
      end do
c
c     compute TINKER energy and gradient
c
      call gradient (energy,deriv)
      mincheck1 = energy
      return
      end
c
c
      subroutine mincheckout(p,iter)
      implicit none
      include 'sizes.i'
      integer iter
      real*8 p(3*maxatm)
      return
      end

c
c
c     #########################################################
c     ##                                                       ##
c     ##  subroutine mincheck2  --  Hessian values for newton  ##
c     ##                                                       ##
c     #########################################################
c
c
      subroutine mincheck2 (mode,xx,h,h_init,h_stop,h_index,h_diag)
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      integer maxvar
      parameter (maxvar=3*maxatm)
      integer i,nvar
      integer h_init(maxvar),h_stop(maxvar)
      integer h_index(maxhess)
      real*8 xx(maxvar),h_diag(maxvar),h(maxhess)
      character*4 mode
c
c
c     translate optimization parameters to atomic coordinates
c
      if (mode .eq. 'none')  return
      nvar = 0
      do i = 1, n
         if (use(i)) then
            nvar = nvar + 1
            x(i) = xx(nvar)
            nvar = nvar + 1
            y(i) = xx(nvar)
            nvar = nvar + 1
            z(i) = xx(nvar)
         end if
      end do
c
c     compute and store the Hessian elements
c
      call hessian (h,h_init,h_stop,h_index,h_diag)
c
c     translate the coordinates of each active atom;
c     this may be needed when using periodic boundaries
c
      nvar = 0
      do i = 1, n
         if (use(i)) then
            nvar = nvar + 1
            xx(nvar) = x(i)
            nvar = nvar + 1
            xx(nvar) = y(i)
            nvar = nvar + 1
            xx(nvar) = z(i)
         end if
      end do
      return
      end
