c
c
c     #########################################################################
c     ##                                                                     ##
c     ##    subroutine monmon  -- calculate the monomer-monomer interaction  ##
c     ##                                                                     ##
c     #########################################################################
c
c
      subroutine monmon(p)
      implicit none
      include 'sizes.i'
      include 'pathsz.i'
      include 'pathdata.i'
      include 'spw.i'
      include 'spwprm.i'
      integer j
      real*8 a,b
      real*8 d2ave,d2ave2,fac,den
     :      ,c1,arg,prefac,pref(0:maxpp)
     :      ,result
      real*8 spwp(3*maxatm,0:maxpp+1)
      real*8 p(nat3,lpath)
      real*8 gd2ave(3*maxatm,0:maxpp+1)
      real*8 gdrms(3*maxatm,0:maxpp+1)
c
c     calculate the monomer-monomer interaction
c
      fac=2.0d0*flinv
c
c     the zeroth and (lpath+1) elements of SPWP correspond to the fixed points
c
      do j=1,nat3
      spwp(j,0)=p0(j)
      spwp(j,lpath+1)=p1(j)
      end do
      do izet=1,lpath
         do j=1,nat3
            spwp(j,izet)=p(j,izet)
         end do
      end do
c
c     construct vector differences of adjacent path points
c
      do izet=0,lpath
         do j=1,nat3
            pdiff(j,izet)=spwp(j,izet+1)-spwp(j,izet)
         end do
      end do
c
c    construct vector differences between even-numbered path points
c
      do izet=0,lpath-1
         do j=1,nat3
            pdtwo(j,izet)=spwp(j,izet+2)-spwp(j,izet)
         end do
      end do
c
c     construct magnitude-squared of pdiff columns (vectors)
c
      do izet=0,lpath
        dpath(izet)=0.0d0
        do j=1,nat3
          dpath(izet)=dpath(izet)+pdiff(j,izet)*pdiff(j,izet)
        end do
      end do
c
c     construct magnitude-squared of pdtwo columns (vectors)
c
      do izet=0,lpath
        dtwo(izet)=0.0d0
      end do
      do izet=0,lpath-1
        do j=1,nat3
          dtwo(izet)=dtwo(izet)+pdtwo(j,izet)*pdtwo(j,izet)
        end do
      end do
c
c     construct mean-squared coordinate vector differences (and rms)
c
      d2ave=0.0d0
      do izet=0,lpath
         d2ave=d2ave+dpath(izet)
      end do
      d2ave=d2ave*flinv
      d2ave2=d2ave*d2ave
      drms=sqrt(d2ave)
c
c     convert to norm of path-difference vectors
c
      do izet=0,lpath
         dpath(izet)=sqrt(dpath(izet))
      end do
c
c     set up some needed intermediate gradients
c
      do izet=1,lpath
         do j=1,nat3
            gd2ave(j,izet)=(pdiff(j,izet-1)-pdiff(j,izet))*fac
         end do
      end do
      do j=1,nat3
        gd2ave(j,0)=-pdiff(j,0)*fac
        gd2ave(j,lpath+1)=pdiff(j,lpath)*fac
      end do
      do izet=0,lpath+1
        do j=1,nat3
            gdrms(j,izet)=gd2ave(j,izet)/2.0d0/drms
        end do
      end do
c
c     construct REPULSION term for SPW [ N.B. this is not Elber term ]
c
      repuls=0.0d0
      den=rlam*d2ave
      do izet=0,lpath-1
         arg=dtwo(izet)/den
         repuls=repuls+dexp(-arg)
      end do
      repuls=repuls*rho
c
c     construct CHAIN term for SPW using drms
c
      chain=0.0d0
      do izet=0,lpath
         arg=dpath(izet)-drms
         arg=arg*arg
         chain=chain+arg
      end do
      chain=chain*gamma
c
c     construct gradient of CHAIN term of polymer energy using drms
c
      do izet=0,lpath+1
         do j=1,nat3
            grdchn(j,izet)=0.0d0
         end do
      end do
      do izet=0,lpath
         arg=1.0d0-drms/dpath(izet)
         prefac=-2.0d0*gamma*arg
         do j=1,nat3
            result=prefac*pdiff(j,izet)
            grdchn(j,izet)=grdchn(j,izet)+result
            grdchn(j,izet+1)=grdchn(j,izet+1)-result
         end do
      end do
c
c     calaculate gradient of drms term
c
      prefac=0.0d0
      do izet=0,lpath
         prefac=prefac+dpath(izet)
      end do
      prefac=prefac-(lpath+1)*drms
      prefac=-2.0d0*gamma*prefac
      do izet=0,lpath+1
         do j=1,nat3
            grdchn(j,izet)=grdchn(j,izet)+prefac*gdrms(j,izet)
         end do
      end do
c
c
c     construct CHAIN term for SPW using constant dzero
c
c     chain=0.0d0
c     do izet=0,lpath
c        arg=dpath(izet)-dzero
c        arg=arg*arg
c        chain=chain+arg
c     end do
c     chain=chain*gamma
c
c     construct gradient of CHAIN term of polymer energy using dzero
c
c     do izet=0,lpath+1
c        do j=1,nat3
c           grdchn(j,izet)=0.0d0
c        end do
c     end do
c     do izet=0,lpath
c        arg=1.0d0-dzero/dpath(izet)
c        prefac=-2.0d0*gamma*arg
c        do j=1,nat3
c           result=prefac*pdiff(j,izet)
c           grdchn(j,izet)=grdchn(j,izet)+result
c           grdchn(j,izet+1)=grdchn(j,izet+1)-result
c        end do
c     end do
c
c     construct gradient of repulsion term of polymer energy
c
c
c  N.B. it's ok to make a force on the fixed points as long as it's not used
c
c
c     zero array
c
      do izet=0,lpath+1
         do j=1,nat3
            grdrep(j,izet)=0.0d0
         end do
      end do
c
c      exponential prefactor
c
      do izet=0,lpath-1
         arg=dtwo(izet)/den
         pref(izet)=rho*dexp(-arg)
      end do
c
c     get gradient of dtwo
c
      c1=-2.0d0/den
      do izet=0,lpath-1
         do j=1,nat3
            result=pdtwo(j,izet)*c1
            grdrep(j,izet)=grdrep(j,izet)-1.0d0*result*pref(izet)
            grdrep(j,izet+2)=grdrep(j,izet+2)+result*pref(izet)
         end do
      end do
c
c     get position independent prefactor and gradient of d2ave
c
      prefac=0.0d0
      do izet=0,lpath-1
         arg=dtwo(izet)/den
         pref(izet)=pref(izet)*arg
         prefac=prefac+pref(izet)
      end do
      prefac=prefac/d2ave
      do izet=0,lpath+1
         do j=1,nat3
            grdrep(j,izet)=grdrep(j,izet)+prefac*gd2ave(j,izet)
         end do
      end do

      return
      end
