c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine startup  --  path calculation  initialization ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "startup" gets options for SPW/LUP path
c     routines and initializes variables
c
c     Input Output flags
c
c     spwoutput     write restart file upon termination of SPW
c     refinement    read restart file at initiation of LUP
c
c     Job Type flag
c
c     jobtype       restart flag
c
      subroutine startup
      implicit none
      include 'sizes.i'
      include 'align.i'
      include 'atoms.i'
      include 'atmtyp.i'
      include 'files.i'
      include 'iounit.i'
      include 'cntrl.i'
      include 'title.i'
      include 'pathsz.i'
      include 'lupgrdmin.i'
      include 'pathdata.i'
      include 'pathatm.i'
      include 'pathcoord.i'
      include 'pathfiles.i'
      include 'spwgrdmin.i'
      include 'spwprm.i'
      integer ikey,next,freeunit
      integer i,j,izet,end,newlpath
      character*10 jobtype
      character*20 keyword
      character*60 keyfile
      character*80 record,substring
      real*8 xtmp(maxatm),ytmp(maxatm),ztmp(maxatm)
      real*8 rmsvalue
      logical exist,luprestart
      external freeunit
c
c     set default option values
c
      findpath = .false.
      relaxpath = .false.
      spwoutput = .true.
      refinement = .false.
      spwgrdmin = 1.0d0
      lupgrdmin = 0.5d0
      lpath = 0
      gamma = 0.0d0
      rho = 0.0d0
      rlam = 0.0d0
c
c     set the number of constraints
c
      nc = nsix
c
c     initialize path
c
      do izet=0,maxpp+1
         do j=1,3*maxatm
            pathcrd(j,izet)=0.0d0
         end do
      end do
c
c     initialize some TINKER stuff
c
      call initial
c
c     read Reactant coordinates and write Title
c
      call getxyz
      write (6,22)  title(1:ltitle)
c
      do i = 1, n
         p0(3*i-2) = x(i)
         p0(3*i-1) = y(i)
         p0(3*i) = z(i)
      end do
c
c     set number of atoms and 3 x natoms
c
      natoms = n
      nat3 = 3 * natoms
c
c     read the key file to determine Options
c
      keyfile = filename(1:leng)//'.key'
      inquire ( file=keyfile,exist=exist )
      if ( exist ) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,20,end=40) record
  20        format(a80)
            next = 1
            call gettext ( record,keyword,next )
            call upcase ( keyword )
            if ( keyword(1:9) .eq. 'PATHNAME ' ) then
               read(ikey,15,err=40) pathname
  15           format (a60)
            else if ( keyword(1:9) .eq. 'FINDPATH ' ) then
               findpath = .true.
               restart = .false.
               call getword(record,jobtype,next)
               call upcase(jobtype)
               if (jobtype .eq. 'RESTART   ') restart = .true.
            else if ( keyword(1:10) .eq. 'RELAXPATH ' ) then
               relaxpath = .true.
               luprestart = .true.
               call getword(record,jobtype,next)
               call upcase(jobtype)
               if (jobtype .eq. 'INITIATE  ') luprestart = .false.
            else if ( keyword(1:10) .eq. 'SPWGRDMIN ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  spwgrdmin
            else if ( keyword(1:10) .eq. 'LUPGRDMIN ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  lupgrdmin
            else if ( keyword(1:6) .eq. 'LPATH ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  newlpath
            else if ( keyword(1:6) .eq. 'GAMMA ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  gamma
            else if ( keyword(1:6) .eq. 'RHO   ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  rho
            else if ( keyword(1:6) .eq. 'RLAM  ' ) then
               substring = record(next:80)
               read (substring,*,err=30)  rlam
            end if
  30        continue
         end do
  40     continue
         close ( unit=ikey )
c
c        determine the length of the filename
c
         pleng = 0
         do i = 1, 60
            if (pathname(i:i) .ne. ' ')  pleng = i
         end do
         end = pleng
c
c        strip any extension from filename
c
         do i = 1, pleng
            if (pathname(i:i) .eq. '/')  end = pleng
            if (pathname(i:i) .eq. ']')  end = pleng
            if (pathname(i:i) .eq. ':')  end = pleng
            if (pathname(i:i) .eq. '.')  end = i - 1
         end do
         pleng = min(pleng,end)
      end if
c
c     set LUP restart condition if there is to
c     be no FINDPATH operation
c
      if (.not.findpath) restart = luprestart
c
c     set LPATH to user specified value for new jobs
c
      if ( .not.restart ) then
        lpath = newlpath
      else
        write (iout,45)
   45   format (' STARTUP  --  Job Restart: Default LPATH value used')
      endif
c
c     Initiation of a new job
c
      if ( .not.restart ) then
c
c        for new jobs store Reactant structure for
c        superpositioning with Product structure
c
         do i = 1, n
            xtmp(i)=x(i)
            ytmp(i)=y(i)
            ztmp(i)=z(i)
         end do
c
c        get coordinates for Product structure and write Title
c
         call getxyz
         write(6,*)
         write (6,22)  title(1:ltitle)
   22    format (a)
c
c     get restart information
c
      else
c
c        read path and Product coordinates from restart file
c
         call getrst
         write (6,*)
         write (6,22)  title(1:ltitle)
c
c        enter Reactant and Product coordinates
c        into path coordinate vector
c
         do j = 1, nat3
            pathcrd(j,0) = p0(j)
            pathcrd(j,lpath+1) = p1(j)
         end do
      end if
c
c     setup the molecular mechanics energy stuff
c
      call mechanic
c
c     set the filename to the pathname
c
      filename = pathname
      leng = pleng
c
c     alter the atomic masses if desired
c
      do i = 1, n
      mass(i) = 1.0d0
      end do
c
c     set up mass array for path calculation
c
      do i = 1, n
         sysms(i) = mass(i)
      end do
c
c     for new jobs superposition the two structures
c
      if ( .not.restart ) then
         nfit = n
         do i = 1, nfit
            ifit(1,i) = i
            ifit(2,i) = i
            wfit (i) = mass(i)
         end do
         call impose (n,xtmp,ytmp,ztmp,n,x,y,z,rmsvalue)
         write(6,*)
         write(6,*) 'RMS value from Superpose  :',rmsvalue
         write(6,*)
c
c        store the structures for the path calculation
c
         do i = 1, n
            p1(3*i-2) = x(i)
            p1(3*i-1) = y(i)
            p1(3*i) = z(i)
         end do
      end if
c
c     write selected options to output file
c
      write(6,*)
      if ( (findpath) .and. (.not.relaxpath) )
     &   write(6,*) 'SPW Calculation Only'
      if ( (findpath) .and. (relaxpath) )
     &   write(6,*) 'SPW/LUP calculation  '
      if ( (relaxpath) .and. (.not.findpath) )
     &   write(6,*) 'LUP Calculation Only'
      write(6,*)
      end
