c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1991  by  Jay William Ponder  ##
c     ###################################################
c
c     #########################################################
c     ##                                                     ##
c     ##  subroutine eimptor4  --  improper torsional energy  ##
c     ##                                                     ##
c     #########################################################
c
c
c     "eimptor4" calculates the improper torsional potential energy
c      derivative with respect to lambda
c
c
      subroutine eimptor4
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'bond.i'
      include 'dlambda.i'
      include 'imptor.i'
      include 'mutant.i'
      include 'units.i'
      integer i,ia,ib,ic,id,ih
      real*8 e,cosine,cos1,cos2,cos3
      real*8 de,de1,de2,de3,dcos1,dcos2,dcos3
      real*8 a1,b1,c1,a2,b2,c2
      real*8 v1,v2,v3,s1,s2,s3
      real*8 xia,yia,zia,xib,yib,zib
      real*8 xic,yic,zic,xid,yid,zid
      real*8 xba,yba,zba,xcb,ycb,zcb
      real*8 xdc,ydc,zdc
c
c
c     zero out improper torsional derivative
c
      ddlitors = 0.0d0
c
c     for each improper torsional angle we first calculate
c     the cosine of the dihedral between ia-ib-ic-id
c
      do ih = 1, nhitors
         i = ihitors(ih)
         ia = iitors(1,i)
         ib = iitors(2,i)
         ic = iitors(3,i)
         id = iitors(4,i)
         if (use(ia) .or. use(ib) .or. use(ic) .or. use(id)) then
            xia = x(ia)
            yia = y(ia)
            zia = z(ia)
            xib = x(ib)
            yib = y(ib)
            zib = z(ib)
            xic = x(ic)
            yic = y(ic)
            zic = z(ic)
            xid = x(id)
            yid = y(id)
            zid = z(id)
            xba = xib - xia
            yba = yib - yia
            zba = zib - zia
            xcb = xic - xib
            ycb = yic - yib
            zcb = zic - zib
            xdc = xid - xic
            ydc = yid - yic
            zdc = zid - zic
            a1 = yba*zcb - ycb*zba
            b1 = xcb*zba - xba*zcb
            c1 = xba*ycb - xcb*yba
            a2 = ycb*zdc - ydc*zcb
            b2 = xdc*zcb - xcb*zdc
            c2 = xcb*ydc - xdc*ycb
            cosine = (a1*a2+b1*b2+c1*c2) / sqrt((a1*a1+b1*b1+c1*c1)
     &                                         *(a2*a2+b2*b2+c2*c2))
c
c     set the improper torsional parameters for this angle
c
            v1 = itors1(1,i)
            s1 = itors1(2,i)
            v2 = itors2(1,i)
            s2 = itors2(2,i)
            v3 = itors3(1,i)
            s3 = itors3(2,i)
c
c     series expansion of torsional angle; note trig identities
c     cos(2x)=2*cos(x)**2-1, and cos(3x)=4*cos(x)**3-3*cos(x)
c
            cos1 = 1.0d0 + s1*cosine
            cos2 = 1.0d0 + s2*(2.0d0*cosine**2 - 1.0d0)
            cos3 = 1.0d0 + s3*(4.0d0*cosine**3 - 3.0d0*cosine)
c
c     calculate the improper torsional energy
c
            dcos1 = -sqrt(1.0d0 - s1**2)
            dcos2 = -sqrt(1.0d0 - s2**2)
            dcos3 = -sqrt(1.0d0 - s3**2)
            de1 = hitors1_10(1,ih)*(1.0d0 + s1) +
     :                       v1*dcos1*hitors1_10(2,ih)
            de1 = de1 * cosine
            de2 = hitors2_10(1,ih)*s2 + v2*dcos2*hitors1_10(2,ih)
            de2 = de2 * (2.0d0*cosine**2 - 1.0d0)
            de2 = hitors2_10(1,ih) + de2
            de3 = hitors3_10(1,ih)*s3 + v3*dcos3*hitors3_10(2,ih)
            de3 = de3 * (4.0d0*cosine**3 - 3.0d0*cosine)
            de3 = hitors3_10(1,ih) + de3
            de = 0.5d0 * ( de1 + de2 + de3)
            ddlitors = ddlitors + de
         end if
      end do
      return
      end
