c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine elj4  --  Lennard-Jones van der Waals energy  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "elj4" calculates the van der Waals Lambda derivative
c     using the Lennard-Jones 6-12 formalism
c
c
      subroutine elj4
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'bound.i'
      include 'couple.i'
      include 'dlambda.i'
      include 'mutant.i'
      include 'latice.i'
      include 'switch.i'
      include 'vdw.i'
      include 'vdwpot.i'
      integer i,j,k,ii,it,kt,skip(maxatm)
      real*8 xi,yi,zi,xr,yr,zr
      real*8 de,p5,p6,p11,p12,rv,eps,rdn
      real*8 x_red(maxatm),y_red(maxatm),z_red(maxatm)
      real*8 rik,rik2,rik3,rik4,rik5,taper
c
c
c     zero out the van der Waals derivative
c
      ddlvdw = 0.0d0
      ddlvdw14 = 0.0d0
c
c     set the coefficients for the tapering function
c
      call smooth ('VDW')
c
c     calculate "reduced" atomic coordinates
c
      do i = 1, n
         skip(i) = 0
         rdn = reduce(i)
         if (rdn.eq.0.0d0 .or. n12(i).eq.0) then
            x_red(i) = x(i)
            y_red(i) = y(i)
            z_red(i) = z(i)
         else
            ii = i12(1,i)
            x_red(i) = rdn*(x(i)-x(ii)) + x(ii)
            y_red(i) = rdn*(y(i)-y(ii)) + y(ii)
            z_red(i) = rdn*(z(i)-z(ii)) + z(ii)
         end if
      end do
c
c     find the van der Waals energy via double loop search
c
      do i = 1, n-1
         do k = 1, n12(i)
            skip(i12(k,i)) = i
         end do
         do k = 1, n13(i)
            skip(i13(k,i)) = i
         end do
         do k = 1, n14(i)
            skip(i14(k,i)) = -i
         end do
         it = itype(i)
         xi = x_red(i)
         yi = y_red(i)
         zi = z_red(i)
         do k = i+1, n
            if (use(i) .or. use(k)) then
               if (skip(k) .ne. i) then
                  kt = itype(k)
                  xr = xi - x_red(k)
                  yr = yi - y_red(k)
                  zr = zi - z_red(k)
                  if (use_images)  call images (xr,yr,zr)
                  rik2 = xr*xr + yr*yr + zr*zr
                  if (rik2 .le. off2) then
                     rv = radii(kt,it)
                     eps = epsilon(kt,it)
c
c     compute the energy contribution for this interaction
c
                     p5 = rv**5 / rik2**3
                     p6 = rv * p5
                     p11 = p5 * p6
                     p12 = p6 * p6
                     de = (p12 - 2.0d0*p6)*hepsilon(kt,it)
                     de = de + 12.0d0*(p11 - p5)*eps*hradii(kt,it)
                     if (skip(k) .eq. -i)  de = de / v14scale
c
c     fifth order multiplicative tapering if near cutoff
c
                     if (rik2 .gt. cut2) then
                        rik = sqrt(rik2)
                        rik3 = rik2 * rik
                        rik4 = rik2 * rik2
                        rik5 = rik2 * rik3
                        taper = c5*rik5 + c4*rik4 + c3*rik3
     &                             + c2*rik2 + c1*rik + c0
                        de = de * taper
                     end if
c
c     increment the overall van der Waals energy components
c
                     if (skip(k) .eq. -i) then
                        ddlvdw14 = ddlvdw14 + de
                     else
                        ddlvdw = ddlvdw + de
                     end if
                  end if
               end if
            end if
         end do
      end do
c
      return
      end
