c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine encharge1  --  charge-charge energy & derivs  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "encharge1" calculates the charge-charge interaction energy
c     and first derivatives with respect to Cartesian coordinates
c     This routine loops over ATOMS (n) with use of NEIGH list
c
c
      subroutine encharge1
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'charge.i'
      include 'couple.i'
      include 'deriv1.i'
      include 'electr.i'
      include 'einter.i'
      include 'energi.i'
      include 'latice.i'
      include 'molcul.i'
      include 'neigh.i'
      include 'switch.i'
      include 'virial.i'
      integer i,ii,j,k,k0,k1,kk,ic,kc,skip(maxatm)
      real*8 e,de,dc,f,fi,fik,rik,rik2
      real*8 xi,yi,zi,xr,yr,zr
      real*8 xic,yic,zic,xc,yc,zc
      real*8 dedx,dedy,dedz,dedxc,dedyc,dedzc
      real*8 rc,rc2,rc3,rc4,rc5,taper,dtaper
c     real*8 elapsed
c     call setime
c
c
c     zero out the charge interaction energy and derivatives
c
      ec = 0.0d0
      do i = 1, n
         dec(1,i) = 0.0d0
         dec(2,i) = 0.0d0
         dec(3,i) = 0.0d0
         skip(i) = 0
      end do
      if (nion .eq. 0)  return
c
c     set conversion factor and tapering function coefficients
c
      f = 332.05382d0 / dielec
      call smooth ('CHARGE')
c
c     compute charge interaction energy and first derivatives
c
      do ii = 1, nion-1
         i = iion(ii)
         ic = swsite(i)
         do k = 1, n12(i)
            skip(i12(k,i)) = i
         end do
         do k = 1, n13(i)
            skip(i13(k,i)) = i
         end do
         do k = 1, n14(i)
            skip(i14(k,i)) = -i
         end do
         xi = x(i)
         yi = y(i)
         zi = z(i)
         xic = x(ic)
         yic = y(ic)
         zic = z(ic)
         fi = f * pchg(ii)
         k0 = npoint(1,i)
         k1 = npoint(2,i)
         do kk = k0,k1
            k = nlist(kk)
            kc = swsite(k)
            if (use(i) .or. use(k)) then
               if (skip(k) .ne. i) then
                  xr = xi - x(k)
                  yr = yi - y(k)
                  zr = zi - z(k)
                  xc = xic - x(kc)
                  yc = yic - y(kc)
                  zc = zic - z(kc)
                  if (use_images)  call images2 (xc,yc,zc,xr,yr,zr)
                  rc2 = xc*xc + yc*yc + zc*zc
                  if (rc2 .le. off2) then
                     rik2 = xr*xr + yr*yr + zr*zr
                     rik = sqrt(rik2)
                     fik = fi * pchg(nqloc(k))
                     if (skip(k) .eq. -i)  fik = fik / chgscale
                     e = fik / rik
                     de = -fik / rik2
                     dc = 0.0d0
c
c     fifth order multiplicative tapering if near cutoff
c
                     if (rc2 .gt. cut2) then
                        rc = sqrt(rc2)
                        rc3 = rc2 * rc
                        rc4 = rc2 * rc2
                        rc5 = rc2 * rc3
                        taper = c5*rc5 + c4*rc4 + c3*rc3
     &                             + c2*rc2 + c1*rc + c0
                        dtaper = 5.0d0*c5*rc4 + 4.0d0*c4*rc3
     &                              + 3.0d0*c3*rc2 + 2.0d0*c2*rc + c1
                        dc = e * dtaper / rc
                        de = de * taper
                        e = e * taper
                     end if
c
c     form the chain rule terms for derivative expressions
c
                     de = de / rik
                     dedx = de * xr
                     dedy = de * yr
                     dedz = de * zr
                     dedxc = dc * xc
                     dedyc = dc * yc
                     dedzc = dc * zc
c
c     increment the overall energy and derivative expressions
c
                     ec = ec + e
                     dec(1,i) = dec(1,i) + dedx
                     dec(2,i) = dec(2,i) + dedy
                     dec(3,i) = dec(3,i) + dedz
                     dec(1,ic) = dec(1,ic) + dedxc
                     dec(2,ic) = dec(2,ic) + dedyc
                     dec(3,ic) = dec(3,ic) + dedzc
                     dec(1,k) = dec(1,k) - dedx
                     dec(2,k) = dec(2,k) - dedy
                     dec(3,k) = dec(3,k) - dedz
                     dec(1,kc) = dec(1,kc) - dedxc
                     dec(2,kc) = dec(2,kc) - dedyc
                     dec(3,kc) = dec(3,kc) - dedzc
c
c     increment the total intermolecular energy
c
                     if (molcule(i) .ne. molcule(k)) then
                        einter = einter + e
                     end if
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*dedx + xc*dedxc
                        viry = viry + yr*dedy + yc*dedyc
                        virz = virz + zr*dedz + zc*dedzc
                     end if
                  end if
               end if
            end if
         end do
      end do
c     call getime(elapsed)
c     write(6,*) 'ENCHARGE1: Elapsed CPU time:',elapsed
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replicate)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, nion-1
         i = iion(ii)
         ic = swsite(i)
         xi = x(i)
         yi = y(i)
         zi = z(i)
         xic = x(ic)
         yic = y(ic)
         zic = z(ic)
         fi = f * pchg(ii)
         k0 = npoint(1,i)
         k1 = npoint(2,i)
         do kk = k0,k1
            k = nlist(kk)
            kc = swsite(k)
            if (use(i) .or. use(k)) then
               do j = 1, ncell
                  xc = xic - x(kc)
                  yc = yic - y(kc)
                  zc = zic - z(kc)
                  call cells (xc,yc,zc,j)
                  rc2 = xc*xc + yc*yc + zc*zc
                  if (rc2 .le. off2) then
                     xr = xi - x(k)
                     yr = yi - y(k)
                     zr = zi - z(k)
                     call cells (xr,yr,zr,j)
                     rik2 = xr*xr + yr*yr + zr*zr
                     rik = sqrt(rik2)
                     fik = fi * pchg(kk)
                     e = fik / rik
                     de = -fik / rik2
                     dc = 0.0d0
c
c     fifth order multiplicative tapering if near cutoff
c
                     if (rc2 .gt. cut2) then
                        rc = sqrt(rc2)
                        rc3 = rc2 * rc
                        rc4 = rc2 * rc2
                        rc5 = rc2 * rc3
                        taper = c5*rc5 + c4*rc4 + c3*rc3
     &                             + c2*rc2 + c1*rc + c0
                        dtaper = 5.0d0*c5*rc4 + 4.0d0*c4*rc3
     &                              + 3.0d0*c3*rc2 + 2.0d0*c2*rc + c1
                        dc = e * dtaper / rc
                        de = de * taper
                        e = e * taper
                     end if
c
c     form the chain rule terms for derivative expressions
c
                     de = de / rik
                     dedx = de * xr
                     dedy = de * yr
                     dedz = de * zr
                     dedxc = dc * xc
                     dedyc = dc * yc
                     dedzc = dc * zc
c
c     increment the energy and gradient values
c
                     if (i .eq. k)  e = 0.5d0 * e
                     ec = ec + e
                     dec(1,i) = dec(1,i) + dedx
                     dec(2,i) = dec(2,i) + dedy
                     dec(3,i) = dec(3,i) + dedz
                     dec(1,ic) = dec(1,ic) + dedxc
                     dec(2,ic) = dec(2,ic) + dedyc
                     dec(3,ic) = dec(3,ic) + dedzc
                     if (i .ne. k) then
                        dec(1,k) = dec(1,k) - dedx
                        dec(2,k) = dec(2,k) - dedy
                        dec(3,k) = dec(3,k) - dedz
                        dec(1,kc) = dec(1,kc) - dedxc
                        dec(2,kc) = dec(2,kc) - dedyc
                        dec(3,kc) = dec(3,kc) - dedzc
                     end if
c
c     increment the total intermolecular energy
c
                     einter = einter + e
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*dedx + xc*dedxc
                        viry = viry + yr*dedy + yc*dedyc
                        virz = virz + zr*dedz + zc*dedzc
                     end if
                  end if
               end do
            end if
         end do
      end do
      return
      end
