c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine enlj1  --  Lennard-Jones energy & derivatives  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "enlj1" calculates the van der Waals energy and its first
c     derivatives with respect to Cartesian coordinates using
c     the Lennard-Jones 6-12 formalism. Interactions are counted
c     using neighbor a list. PBC, and tapering function
c     are functions of the set of switching-sites
c
c     NOTES:
c    1)	the replicates code at the end is not outfitted with
c     residue based switching. To implement this need to do something
c     with CELLS code.
c 	    2)  the reduce stuff may or may not work correctly with
c     residue based switching.
c
      subroutine enlj1
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'couple.i'
      include 'deriv1.i'
      include 'einter.i'
      include 'energi.i'
      include 'latice.i'
      include 'molcul.i'
      include 'neigh.i'
      include 'switch.i'
      include 'vdw.i'
      include 'vdwpot.i'
      include 'virial.i'
      integer i,j,k,ii,kk,ij,it,kt,skip(maxatm)
      integer ic,kc,kindx,k0,k1
      real*8 xi,yi,zi,xr,yr,zr
      real*8 xic,yic,zic,xc,yc,zc
      real*8 e,p6,p12,rv,eps,rdn
      real*8 redi,redii,redk,redkk
      real*8 dedx,dedy,dedz,de,dc,dedxc,dedyc,dedzc
      real*8 x_red(maxatm),y_red(maxatm),z_red(maxatm)
      real*8 rc,rc2,rc3,rc4,rc5
      real*8 rik,rik2,rik3,rik4,rik5
      real*8 taper,dtaper
c
c
c     zero out the van der Waals energy and first derivatives
c
      ev = 0.0d0
      e14 = 0.0d0
      do i = 1, n
         dev(1,i) = 0.0d0
         dev(2,i) = 0.0d0
         dev(3,i) = 0.0d0
         de14(1,i) = 0.0d0
         de14(2,i) = 0.0d0
         de14(3,i) = 0.0d0
      end do
c
c     set the coefficients for the tapering function
c
      call smooth ('VDW')
c
c     calculate "reduced" atomic coordinates
c
      do i = 1, n
         skip(i) = 0
         rdn = reduce(i)
         if (rdn.eq.0.0d0 .or. n12(i).eq.0) then
            x_red(i) = x(i)
            y_red(i) = y(i)
            z_red(i) = z(i)
         else
            ii = i12(1,i)
            x_red(i) = rdn*(x(i)-x(ii)) + x(ii)
            y_red(i) = rdn*(y(i)-y(ii)) + y(ii)
            z_red(i) = rdn*(z(i)-z(ii)) + z(ii)
         end if
      end do
c
c     find van der Waals energy and derivatives via double loop
c
      do ij = 1, navdw-1
         i = ivdw(ij)
         ic = swsite(i)
         do k = 1, n12(i)
            skip(i12(k,i)) = i
         end do
         do k = 1, n13(i)
            skip(i13(k,i)) = i
         end do
         do k = 1, n14(i)
            skip(i14(k,i)) = -i
         end do
         redi = reduce(i)
         if (redi .eq. 0.0d0) then
            ii = i
         else
            ii = i12(1,i)
            redii = 1.0d0 - redi
         end if
         it = itype(i)
         xi = x_red(i)
         yi = y_red(i)
         zi = z_red(i)
         xic = x_red(ic)
         yic = y_red(ic)
         zic = z_red(ic)
c
c      Can still improve this - will have lots of zeros
c
         k0 = npoint(1,i)
         k1 = npoint(2,i)
         do kindx = k0,k1
            k = nlist(kindx)
            kc = swsite(k)
            redk = reduce(k)
            if (redk .eq. 0.0d0) then
               kk = k
            else
               kk = i12(1,k)
               redkk = 1.0d0 - redk
            end if
            if (use(i) .or. use(ii) .or. use(k) .or. use(kk)) then
               if (skip(k) .ne. i) then
                  kt = itype(k)
                  xr = xi - x_red(k)
                  yr = yi - y_red(k)
                  zr = zi - z_red(k)
                  xc = xic - x_red(kc)
                  yc = yic - y_red(kc)
                  zc = zic - z_red(kc)
                  if (use_images)  call images2 (xc,yc,zc,xr,yr,zr)
                  if (rc2 .le. off2) then
                     rik2 = xr*xr + yr*yr + zr*zr
                     rik = sqrt(rik2)
                     rv = radii(kt,it)
                     eps = epsilon(kt,it)
                     if (skip(k) .eq. -i)  eps = eps / v14scale
c
c     compute energy and derivatives for this interaction
c
                     p6 = rv**6 / rik2**3
                     p12 = p6 * p6
                     e = eps * (p12 - 2.0d0 * p6)
                     de = eps * (p12 - p6) * (-12.0d0/rik)
                     dc = 0.0d0
c
c     fifth order multiplicative tapering if near cutoff
c
                     if (rc2 .gt. cut2) then
                        rc = sqrt(rc2)
                        rc3 = rc2 * rc
                        rc4 = rc2 * rc2
                        rc5 = rc2 * rc3
                        taper = c5*rc5 + c4*rc4 + c3*rc3
     &                             + c2*rc2 + c1*rc + c0
                        dtaper = 5.0d0*c5*rc4 + 4.0d0*c4*rc3
     &                              + 3.0d0*c3*rc2 + 2.0d0*c2*rc + c1
                        dc = e * dtaper / rc
                        de = de * taper / rik
                        e = e * taper
                     end if
c
c     find the chain rule terms for derivative components
c
                     dedx = de * xr
                     dedy = de * yr
                     dedz = de * zr
                     dedxc = dc * xc
                     dedyc = dc * yc
                     dedzc = dc * zc
c
c     increment the total van der Waals energy and derivatives
c
                     if (skip(k) .eq. -i) then
                        e14 = e14 + e
                        if (redi .eq. 0.0d0) then
                           de14(1,i) = de14(1,i) + dedx
                           de14(2,i) = de14(2,i) + dedy
                           de14(3,i) = de14(3,i) + dedz
                           de14(1,ic) = de14(1,ic) + dedxc
                           de14(2,ic) = de14(2,ic) + dedyc
                           de14(3,ic) = de14(3,ic) + dedzc
                        else
                           de14(1,i) = de14(1,i) + dedx*redi
                           de14(2,i) = de14(2,i) + dedy*redi
                           de14(3,i) = de14(3,i) + dedz*redi
                           de14(1,ic) = de14(1,ic) + dedxc*redi
                           de14(2,ic) = de14(2,ic) + dedyc*redi
                           de14(3,ic) = de14(3,ic) + dedzc*redi
                           de14(1,ii) = de14(1,ii) + dedx*redii
                           de14(2,ii) = de14(2,ii) + dedy*redii
                           de14(3,ii) = de14(3,ii) + dedz*redii
                        end if
                        if (redk .eq. 0.0d0) then
                           de14(1,k) = de14(1,k) - dedx
                           de14(2,k) = de14(2,k) - dedy
                           de14(3,k) = de14(3,k) - dedz
                           de14(1,kc) = de14(1,kc) - dedxc
                           de14(2,kc) = de14(2,kc) - dedyc
                           de14(3,kc) = de14(3,kc) - dedzc
                        else
                           de14(1,k) = de14(1,k) - dedx*redk
                           de14(2,k) = de14(2,k) - dedy*redk
                           de14(3,k) = de14(3,k) - dedz*redk
                           de14(1,kc) = de14(1,kc) - dedxc*redk
                           de14(2,kc) = de14(2,kc) - dedyc*redk
                           de14(3,kc) = de14(3,kc) - dedzc*redk
                           de14(1,kk) = de14(1,kk) - dedx*redkk
                           de14(2,kk) = de14(2,kk) - dedy*redkk
                           de14(3,kk) = de14(3,kk) - dedz*redkk
                        end if
                     else
                        ev = ev + e
                        if (redi .eq. 0.0d0) then
                           dev(1,i) = dev(1,i) + dedx
                           dev(2,i) = dev(2,i) + dedy
                           dev(3,i) = dev(3,i) + dedz
                           dev(1,ic) = dev(1,ic) + dedxc
                           dev(2,ic) = dev(2,ic) + dedyc
                           dev(3,ic) = dev(3,ic) + dedzc
                        else
                           dev(1,i) = dev(1,i) + dedx*redi
                           dev(2,i) = dev(2,i) + dedy*redi
                           dev(3,i) = dev(3,i) + dedz*redi
                           dev(1,ic) = dev(1,ic) + dedxc*redi
                           dev(2,ic) = dev(2,ic) + dedyc*redi
                           dev(3,ic) = dev(3,ic) + dedzc*redi
                           dev(1,ii) = dev(1,ii) + dedx*redii
                           dev(2,ii) = dev(2,ii) + dedy*redii
                           dev(3,ii) = dev(3,ii) + dedz*redii
                        end if
                        if (redk .eq. 0.0d0) then
                           dev(1,k) = dev(1,k) - dedx
                           dev(2,k) = dev(2,k) - dedy
                           dev(3,k) = dev(3,k) - dedz
                           dev(1,kc) = dev(1,kc) - dedxc
                           dev(2,kc) = dev(2,kc) - dedyc
                           dev(3,kc) = dev(3,kc) - dedzc
                        else
                           dev(1,k) = dev(1,k) - dedx*redk
                           dev(2,k) = dev(2,k) - dedy*redk
                           dev(3,k) = dev(3,k) - dedz*redk
                           dev(1,kc) = dev(1,kc) - dedxc*redk
                           dev(2,kc) = dev(2,kc) - dedyc*redk
                           dev(3,kc) = dev(3,kc) - dedzc*redk
                           dev(1,kk) = dev(1,kk) - dedx*redkk
                           dev(2,kk) = dev(2,kk) - dedy*redkk
                           dev(3,kk) = dev(3,kk) - dedz*redkk
                        end if
                     end if
c
c     increment the total intermolecular energy
c
                     if (molcule(i) .ne. molcule(k)) then
                        einter = einter + e
                     end if
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*dedx
                        viry = viry + yr*dedy
                        virz = virz + zr*dedz
                     end if
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replicate)  return
c
c     calculate interaction energy with other unit cells
c
      do i = 1, n
         ic = swsite(i)
         redi = reduce(i)
         if (redi .eq. 0.0d0) then
            ii = i
         else
            ii = i12(1,i)
            redii = 1.0d0 - redi
         end if
         it = itype(i)
         xi = x_red(i)
         yi = y_red(i)
         zi = z_red(i)
         k0 = npoint(1,i)
         k1 = npoint(2,i)
         do kindx = k0,k1
            k = nlist(kindx)
            kc = swsite(k)
            redk = reduce(k)
            if (redk .eq. 0.0d0) then
               kk = k
            else
               kk = i12(1,k)
               redkk = 1.0d0 - redk
            end if
            if (use(i) .or. use(ii) .or. use(k) .or. use(kk)) then
               do j = 1, ncell
                  kt = itype(k)
                  xr = xi - x_red(k)
                  yr = yi - y_red(k)
                  zr = zi - z_red(k)
                  call cells (xr,yr,zr,j)
                  rik2 = xr*xr + yr*yr + zr*zr
                  if (rik2 .le. off2) then
                     rv = radii(kt,it)
                     eps = epsilon(kt,it)
                     rik = sqrt(rik2)
c
c     compute energy and derivatives for this interaction
c
                     p6 = rv**6 / rik2**3
                     p12 = p6 * p6
                     e = eps * (p12 - 2.0d0 * p6)
                     de = eps * (p12 - p6) * (-12.0d0/rik)
c
c     fifth order multiplicative tapering if near cutoff
c
                     if (rik2 .gt. cut2) then
                        rik3 = rik2 * rik
                        rik4 = rik2 * rik2
                        rik5 = rik2 * rik3
                        taper = c5*rik5 + c4*rik4 + c3*rik3
     &                             + c2*rik2 + c1*rik + c0
                        dtaper = 5.0d0*c5*rik4 + 4.0d0*c4*rik3
     &                              + 3.0d0*c3*rik2 + 2.0d0*c2*rik + c1
                        de = e*dtaper + de*taper
                        e = e * taper
                     end if
c
c     find the chain rule terms for derivative components
c
                     de = de / rik
                     dedx = de * xr
                     dedy = de * yr
                     dedz = de * zr
c
c     increment the total van der Waals energy and derivatives
c
                     if (i .eq. k)  e = 0.5d0 * e
                     ev = ev + e
                     if (redi .eq. 0.0d0) then
                        dev(1,i) = dev(1,i) + dedx
                        dev(2,i) = dev(2,i) + dedy
                        dev(3,i) = dev(3,i) + dedz
                     else
                        dev(1,i) = dev(1,i) + dedx*redi
                        dev(2,i) = dev(2,i) + dedy*redi
                        dev(3,i) = dev(3,i) + dedz*redi
                        dev(1,ii) = dev(1,ii) + dedx*redii
                        dev(2,ii) = dev(2,ii) + dedy*redii
                        dev(3,ii) = dev(3,ii) + dedz*redii
                     end if
                     if (i .ne. k) then
                        if (redk .eq. 0.0d0) then
                           dev(1,k) = dev(1,k) - dedx
                           dev(2,k) = dev(2,k) - dedy
                           dev(3,k) = dev(3,k) - dedz
                        else
                           dev(1,k) = dev(1,k) - dedx*redk
                           dev(2,k) = dev(2,k) - dedy*redk
                           dev(3,k) = dev(3,k) - dedz*redk
                           dev(1,kk) = dev(1,kk) - dedx*redkk
                           dev(2,kk) = dev(2,kk) - dedy*redkk
                           dev(3,kk) = dev(3,kk) - dedz*redkk
                        end if
                     end if
c
c     increment the total intermolecular energy
c
                     einter = einter + e
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*dedx
                        viry = viry + yr*dedy
                        virz = virz + zr*dedz
                     end if
                  end if
               end do
            end if
         end do
      end do
      return
      end
