c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ########################################################
c     ##                                                    ##
c     ##  subroutine etors4  --  torsional potential energy  ##
c     ##                                                    ##
c     ########################################################
c
c
c     "etors4" calculates the torsional and stretch-torsion
c     potential energies
c
c
      subroutine etors4
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'bond.i'
      include 'dlambda.i'
      include 'mutant.i'
      include 'strtor.i'
      include 'torpot.i'
      include 'tors.i'
      integer i,ih,ia,ib,ic,id
      real*8 de,cosine,rcb,dr,de1,de2,de3,dcos1,dcos2,dcos3
      real*8 a1,b1,c1,a2,b2,c2
      real*8 v1,v2,v3,s1,s2,s3
      real*8 cos1,cos2,cos3
      real*8 xia,yia,zia,xib,yib,zib
      real*8 xic,yic,zic,xid,yid,zid
      real*8 xba,yba,zba,xcb,ycb,zcb
      real*8 xdc,ydc,zdc
c
c
c     zero out torsional and stretch-torsion energies
c
      ddltors = 0.0d0
c
c     for each torsional angle we first calculate
c     the cosine of the dihedral between ia-ib-ic-id
c
      do ih = 1, nhtors
         i = ihtors(ih)
         ia = itors(1,i)
         ib = itors(2,i)
         ic = itors(3,i)
         id = itors(4,i)
         if (use(ia) .or. use(ib) .or. use(ic) .or. use(id)) then
            xia = x(ia)
            yia = y(ia)
            zia = z(ia)
            xib = x(ib)
            yib = y(ib)
            zib = z(ib)
            xic = x(ic)
            yic = y(ic)
            zic = z(ic)
            xid = x(id)
            yid = y(id)
            zid = z(id)
            xba = xib - xia
            yba = yib - yia
            zba = zib - zia
            xcb = xic - xib
            ycb = yic - yib
            zcb = zic - zib
            xdc = xid - xic
            ydc = yid - yic
            zdc = zid - zic
            a1 = yba*zcb - ycb*zba
            b1 = xcb*zba - xba*zcb
            c1 = xba*ycb - xcb*yba
            a2 = ycb*zdc - ydc*zcb
            b2 = xdc*zcb - xcb*zdc
            c2 = xcb*ydc - xdc*ycb
            cosine = (a1*a2+b1*b2+c1*c2) / sqrt((a1*a1+b1*b1+c1*c1)
     &                                         *(a2*a2+b2*b2+c2*c2))
c
c     set the torsional parameters for this angle
c
            v1 = tors1(1,i)
            s1 = tors1(2,i)
            v2 = tors2(1,i)
            s2 = tors2(2,i)
            v3 = tors3(1,i)
            s3 = tors3(2,i)
c
c     series expansion of torsional angle; note trig identities
c     cos(2x)=2*cos(x)**2-1, and cos(3x)=4*cos(x)**3-3*cos(x)
c
            cos1 = 1.0d0 + s1*cosine
            cos2 = 1.0d0 + s2*(2.0d0*cosine**2 - 1.0d0)
            cos3 = 1.0d0 + s3*(4.0d0*cosine**3 - 3.0d0*cosine)
c
c     calculate the torsional energy
c
            dcos1 = -sqrt(1.0d0 - s1**2)
            dcos2 = -sqrt(1.0d0 - s2**2)
            dcos3 = -sqrt(1.0d0 - s3**2)
            de1 = htors1_10(1,ih)*(1.0d0 + s1) +
     :                       v1*dcos1*htors1_10(2,ih)
            de1 = de1 * cosine
            de2 = htors2_10(1,ih)*s2 + v2*dcos2*htors1_10(2,ih)
            de2 = de2 * (2.0d0*cosine**2 - 1.0d0)
            de2 = htors2_10(1,ih) + de2
            de3 = htors3_10(1,ih)*s3 + v3*dcos3*htors3_10(2,ih)
            de3 = de3 * (4.0d0*cosine**3 - 3.0d0*cosine)
            de3 = htors3_10(1,ih) + de3
            de = 0.5d0 * ( de1 + de2 + de3)
            ddltors = ddltors + de
         end if
      end do
      return
      end
