c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine kcharge  --  assign partial charge parameters  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "kcharge" assigns partial charges to the atoms within
c     the structure and processes any new or changed values
c
c
      subroutine kcharge
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'charge.i'
      include 'couple.i'
      include 'files.i'
      include 'inform.i'
      include 'iounit.i'
      include 'kelect.i'
      integer i,j,k,ikey,next,freeunit
      real*8 pcg
      character*20 keyword
      character*60 keyfile
      character*80 record,string
      logical exist,header
c
c
c     open the keyword file for the current computation
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
      end if
c
c     read any user defined atomic partial charge parameters
c
      if (exist) then
         rewind (unit=ikey)
         header = .true.
         dowhile (.true.)
            read (ikey,10,end=60)  record
   10       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
            if (keyword(1:7) .eq. 'CHARGE ') then
               string = record(next:80)
               read (string,*,err=50,end=50)  k,pcg
               if (k .gt. 0) then
                  if (header) then
                     header = .false.
                     write (iout,20)
   20                format (/' User Defined Atomic Partial',
     &                         ' Charge Parameters :',
     &                       //8x,'Atom Type',7x,'Charge')
                  end if
                  if (k .le. maxtyp) then
                     chg(k) = pcg
                     write (iout,30)  k,pcg
   30                format (11x,i3,4x,f12.3)
                  else
                     write (iout,40)
   40                format(' KCHARGE  --  Too Many Partial',
     &                      ' Charge Parameters')
                     abort = .true.
                  end if
               end if
   50          continue
            end if
         end do
   60    continue
         if (.not. header) then
            write (iout,70)
   70       format ()
         end if
      end if
c
c     find and store all the atomic partial charges
c
      do i = 1, n
         pchg(i) = chg(itype(i))
         nqloc(i) = 0
      end do
c
c     read any user defined partial charges for specific atoms
c
      if (exist) then
         rewind (unit=ikey)
         header = .true.
         dowhile (.true.)
            read (ikey,80,end=120)  record
   80       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
            if (keyword(1:7) .eq. 'CHARGE ') then
               k = 0
               pcg = 0.0d0
               string = record(next:80)
               read (string,*,err=110,end=110)  k,pcg
               if (k.lt.0 .and. k.ge.-n) then
                  k = -k
                  if (header) then
                     header = .false.
                     write (iout,90)
   90                format (/' User Defined Partial Charges',
     &                          ' for Specific Atoms :',
     &                       //10x,'Atom',10x,'Charge')
                  end if
                  write (iout,100)  k,pcg
  100             format (8x,i6,4x,f12.3)
                  pchg(k) = pcg
               end if
  110          continue
            end if
         end do
  120    continue
         if (.not. header) then
            write (iout,130)
  130       format ()
         end if
      end if
c
c     remove zero partial charges from the list of charges
c     Store point charge for each of n sites
c
      nion = 0
      do i = 1, n
         if (pchg(i) .ne. 0.0d0) then
            nion = nion + 1
            iion(nion) = i
            kion(nion) = i
            pchg(nion) = pchg(i)
            nqloc(i) = nion
         end if
      end do
      if (exist)  close (unit=ikey)
c
c     if there are no partial charges pairs, turn off the potential
c
      if (nion .le. 1)  use_charge = .false.
      return
      end
