c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine mechanic  --  initialize molecular mechanics  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "mechanic" sets up needed parameters for the potential energy
c     calculation and reads in many of the user selectable options
c
c
      subroutine mechanic
      implicit none
      include 'sizes.i'
      include 'boxes.i'
      include 'files.i'
      include 'hescut.i'
      include 'inform.i'
      include 'iounit.i'
      include 'output.i'
      include 'potent.i'
      integer ikey,next,freeunit
      character*20 keyword
      character*60 keyfile
      character*80 record,string
      logical exist
c
c
c     set the default values for the output variables
c
      savecycle = .false.
      verbose = .false.
      debug = .false.
      abort = .false.
c
c     set the default value for minimum Hessian element
c
      hesscut = 0.0d0
c
c     check for the existence of a keyword file
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
c
c     search each line of the keyword file for options
c
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         rewind (unit=ikey)
         dowhile (.true.)
            read (ikey,10,end=30)  record
   10       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
c
c     get the values for output control keywords
c
            if (keyword(1:10) .eq. 'SAVECYCLE ') then
               savecycle = .true.
            else if (keyword(1:10) .eq. 'OVERWRITE ') then
               savecycle = .false.
            else if (keyword(1:8) .eq. 'VERBOSE ') then
               verbose = .true.
            else if (keyword(1:6) .eq. 'DEBUG ') then
               debug = .true.
               verbose = .true.
c
c     get the minimum allowed value for Hessian elements
c
            else if (keyword(1:12) .eq. 'HESS-CUTOFF ') then
               string = record(next:80)
               read (string,*,err=20)  hesscut
            end if
   20       continue
         end do
   30    continue
         close (unit=ikey)
      end if
c
c     set the connectivity, active atoms and restraints
c
      call attach
      call active
      call restrain
c
c     find and store each bond, angle and torsion
c
      call bonds
      call angles
      call torsions
c
c     find cutoff values, small rings and pisystem
c
      call cutoffs
      call rings
      call orbital
c
c     determine type of force field and assign atom types
c
      call field
      call katom
c
c     assign local geometry potential function parameters
c
      if (use_bond)  call kbond
      if (use_angle)  call kangle
      if (use_strbnd)  call kstrbnd
      if (use_urey)  call kurey
c     if (use_angang)  call kangang
c     if (use_improp)  call kimprop
      if (use_imptor)  call kimptor
      if (use_tors)  call ktors
      if (use_strtor)  call kstrtor
c
c     assign long range potential function parameters
c
      if (use_vdw)  call kvdw
      if (use_charge .or. use_chgdpl)  call kcharge
      if (use_dipole .or. use_chgdpl)  call kdipole
c
c     assign pisystem potential function parameters
c
      if (use_orbit)  call korbit
c
c     set hybrid parameters for free energy perturbation
c
      call hybrids
c
c     count the number of molcules
c
      call molecule
c
c     switching sites
c
      call switchsite
c
c     setup use of empirical solvation models
c
      call solvate
c
c     find any crystal lattice or periodic box parameters
c
      call unitcell
      call lattice
c
c     quit if essential parameter information is missing
c
      if (abort) then
         write (iout,40)
   40    format (/,' MECHANIC  --  Some Needed Potential Energy',
     &              ' Parameters are Missing')
         call fatal
      end if
      return
      end
