c
c
c     ###############################################################
c     ##  COPYRIGHT (C) 1991 by Shawn Huston & Jay William Ponder  ##
c     ###############################################################
c
c     ############################################################
c     ##                                                        ##
c     ##  program mutate  --  perform free energy perturbation  ##
c     ##                                                        ##
c     ############################################################
c
c
c     "mutate" computes the free energy difference corresponding
c     to a small perturbation by Boltzmann weighting the potential
c     energy difference over a number of sample states; current
c     version (incorrectly) considers the charge energy to be
c     intermolecular in finding the perturbation energies
c     This routine uses the analyis energy routines so that
c     binding energy per group or atom may be obtained.
c
c     nlamb   number of lambda values for energy computation
c     delta   step size for the perturbation in lambda
c     nrun    number of steps over which to calculate averages
c
c     ptnl,epot0   total system potential energy
c     dep,dem      energy change for postive (negative) delta lambda
c     sdep,sdem    accumulated energy changes
c     adep,adem    (running) average energy changes
c     adapb,adamb  average over block averages of free energy changes
c     bdep,bdem    (block) accumulated energy changes
c     badep,badem  (block) average energy changes
c     vdep,vdem    SD of energy changes from Variance of subaverages
c     fdep,fdem    fluctuations of perturbation energies
c     se_ap,se_am  standard error of free energy changes
c
c
      program mutate
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'analyz.i'
      include 'atoms.i'
      include 'charge.i'
      include 'energi.i'
      include 'files.i'
      include 'inform.i'
      include 'iounit.i'
      include 'katoms.i'
      include 'molcul.i'
      include 'mutant.i'
      include 'potent.i'
      include 'units.i'
      integer maxblock,maxstep,maxdih
      parameter (maxblock=100)
      parameter (maxdih=10)
      parameter (maxstep=10000)
      integer i,j,start,stop,next,last
      integer ixyz,ikey,ipot,iblk,isum,ife
      integer ihyb,it0,it1,idih(maxdih)
      integer jb,nblock,nrun,nprev,modblock
      integer istep,nstep,ilamb,nlamb
      integer ia,ib,ic,id,ndih,jdih(4,maxdih)
      integer ii,iu,solute1,solute2
      integer freeunit,trimtext,lext
      real*8 delta,lam0,lamp,lamm
      real*8 dih,dihedral,nrg(3,maxstep),ptnl(maxstep)
      real*8 cb(3,maxstep),ca(3,maxstep)
      real*8 epot0,e0,ep,em,dem,dep,e_pot,energy
      real*8 pos,neg
      real*8 temp,rt,beta,kb,deltat,tp,tm,lp,lm
      real*8 coul_uv,vdw_uv
      real*8 sdep,s2dep,sdep2,sdem,s2dem,sdem2
      real*8 spos,sneg,bpos,bneg,sdapb,sdamb
      real*8 bdep,adep,a2dep,adep2
      real*8 bdem,adem,a2dem,adem2
      real*8 da,dap,dam,adapb,adamb
      real*8 bda,badep(maxblock),badem(maxblock)
      real*8 bdap(maxblock),bdam(maxblock)
      real*8 v,vdap,vdam,vdep,vdem,fdep,fdem
      real*8 se_ep,se_em,se_ap,se_am
      real*8 sepot0,s2epot0,aepot0,a2epot0,aepot02,fepot0
      real*8 cit(3,maxstep),ct(3,maxstep)
      real*8 c14(3,maxstep),cv(3,maxstep),cc(3,maxstep)
      real*8 eb0,ebp,ebm,ea0,eap,eam
      real*8 eit0,eitp,eitm,et0,etp,etm
      real*8 e140,e14p,e14m,ev0,evp,evm
      real*8 ec0,ecp,ecm
      real*8 enb0,enbp,enbm,eg0,egp,egm
      real*8 debp,deap,deitp,detp,de14p,devp,decp
      real*8 debm,deam,deitm,detm,de14m,devm,decm
      real*8 denbp,denbm,degp,degm
      real*8 sbp,sap,sitp,stp,s14p,svp,scp
      real*8 sbm,sam,sitm,stm,s14m,svm,scm
      real*8 s2bp,s2ap,s2itp,s2tp,s214p,s2vp,s2cp
      real*8 s2bm,s2am,s2itm,s2tm,s214m,s2vm,s2cm
      real*8 abp,aap,aitp,atp,a14p,avp,acp
      real*8 abm,aam,aitm,atm,a14m,avm,acm
      real*8 a2bp,a2ap,a2itp,a2tp,a214p,a2vp,a2cp
      real*8 a2bm,a2am,a2itm,a2tm,a214m,a2vm,a2cm
      real*8 abp2,aap2,aitp2,atp2,a14p2,avp2,acp2
      real*8 abm2,aam2,aitm2,atm2,a14m2,avm2,acm2
      real*8 fbp,fap,fitp,ftp,f14p,fvp,fcp
      real*8 fbm,fam,fitm,ftm,f14m,fvm,fcm
      real*8 eptm,eptp,emtm,emtp,e0lm,e0lp
      real*8 septm,septp,semtm,semtp,se0lm,se0lp
      real*8 aeptm,aeptp,aemtm,aemtp
      real*8 entp,entm,enrgp,enrgm
      real*8 beptm,beptp,bemtm,bemtp,be0lm,be0lp
      real*8 bentp,bentm,baentp(maxblock),baentm(maxblock)
      real*8 baeptm,baeptp,baemtp,baemtm
      real*8 sentpb,sentmb,aentpb,aentmb
      real*8 ventp,ventm,se_entp,se_entm
      character*1 letter,answer
      character*7 ext
      character*10 name
      character*20 keyword
      character*60 keyfile,potfile,coordfile,blkfile
      character*60 sumfile,fefile
      character*60 dihfile(maxdih)
      character*80 record,string
      logical use_def(maxatm),def_a,def_b
      logical def_i,def_s,def_st,def_t
      logical done,exist,dogeom
      logical def_debug
      external freeunit,trimtext,dihedral,energy
c
c
c     get the name of the coordinates files
c
      call initial
      write (iout,10)
   10 format (' Enter Name of the Coordinate Files :  ',$)
      read (input,20)  filename
   20 format (a60)
      leng = trimtext (filename)
c
c     remove any extension from the filename
c
      last = leng
      do i = 1, leng
         letter = filename(i:i)
         if (letter .eq. '/')  last = leng
         if (letter .eq. ']')  last = leng
         if (letter .eq. ':')  last = leng
         if (letter .eq. '.')  last = i - 1
      end do
      leng = min(leng,last)
c
c     get the numbers of the files to be used
c
      write (iout,30)
   30 format (' Numbers of First and Last File to Analyze :  ',$)
      read (input,40)  start,stop
   40 format (2i10)
c
c     check for the existence of a keyword file
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
c
c     search each line of the keyword file for options
c
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,50,end=70)  record
   50       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
            if (keyword(1:7) .eq. 'LAMBDA ') then
               nhybrid = 0
               lambda = 0.0d0
               string = record(next:80)
               read (string,*,err=60)  lambda
            else if (keyword(1:7) .eq. 'MUTATE ') then
               string = record(next:80)
               read (string,*,err=60)  ihyb,it0,it1
               nhybrid = nhybrid + 1
               ihybrid(nhybrid) = ihyb
               itype0(nhybrid) = it0
               itype1(nhybrid) = it1
               alter(ihyb) = .true.
            end if
   60       continue
         end do
   70    continue
         close (unit=ikey)
      end if
c
c     check for the existence of an accumulator file
c
      sumfile = filename(1:leng)//'.sum'
      inquire (file=sumfile,exist=exist)
      nprev = 0
      if (exist) then
         isum = freeunit ()
         open (unit=isum,file=sumfile,status='old')
         read (isum,*) nprev,nblock
         read (isum,*) delta,temp,dogeom
         read (isum,*) (badep(jb),badem(jb),jb = 1,
     :                        nblock )
         read (isum,*) (bdap(jb), bdam(jb), jb = 1,
     :                        nblock )
         read (isum,*) (baentp(jb), baentm(jb), jb = 1,
     :                        nblock )
         read (isum,*) sdep,sdem,spos,sneg,sepot0,s2epot0
         read (isum,*) septm,septp,semtm,semtp,se0lm,se0lp
         read(isum,*)  sdapb,sdamb,sentpb,sentmb
         read (isum,*) s14p,s14m,svp,svm,scp,scm
         read (isum,*) sbp,sbm,sap,sam,sitp,sitm,stp,stm
         read (isum,*) s2dep,s2dem
         read (isum,*) s214p,s214m,s2vp,s2vm,s2cp,s2cm
         read (isum,*) s2bp,s2bm,s2ap,s2am,s2itp,s2itm,s2tp,s2tm
         close(isum)
         jb = nblock
      else
c
c     obtain the lambda values to be calculated
c
      delta = 0.0d0
      write (iout,80)
   80 format (//' Enter Lambda Increment for FEP :  ',$)
      read (input,90)  delta
   90 format (f7.4)
      if (delta .gt. 0.0d0) then
         write (iout,100) delta
  100    format (' Lambda Increment of ',f6.3)
      end if
c
c     obtain target temperature
c
      temp = 0.0d0
      if (delta .gt. 0.0d0)  write (iout,110)
  110 format (' Enter the System Temperature [300 K] :  ',$)
      read (input,120)  temp
  120 format (f10.5)
      if (temp .eq. 0.0d0)  temp = 300.0d0
c
c     set number of steps for running averages and block averages
c
      jb = 0
      write (iout,130)
  130 format (' Enter Number of Blocks for Sub-Averages :  ',$)
      read (input,140)  nblock
  140 format (i5)
c
c     obtain the quantities to be calculated
c
      dogeom = .true.
      write (iout,150)
  150 format (' Consider Intermolecular Perturbation',
     &           ' Energy Only? [N] :  ',$)
      read (input,160)  answer
  160 format (a1)
      call upcase (answer)
      if (answer .eq. 'Y')  dogeom = .false.
c
      endif
c
c
c
      nlamb = 3
      lam0 = lambda
      lamp = lambda + delta
      lamm = lambda - delta
c
c     open various Gnuplot data files
c
      ipot = freeunit ()
      potfile = filename(1:leng)//'.pot'
      call version (potfile,'new')
      open (unit=ipot,file=potfile,status='new')
      write (iout,170)
  170 format (//' Individual Dihedral Monitoring :')
      i = 0
      done = .false.
      dowhile (.not. done)
         i = i + 1
         do j = 1, 4
            jdih(j,i) = 0
         end do
         write (iout,180)  i
  180    format (' Enter Name and Four Atoms of Dihedral',i3,' :  ',$)
         read (input,190)  record
  190    format (a80)
         next = 1
         call getword (record,name,next)
         string = record(next:80)
         read (string,*,end=200,err=200)  (jdih(j,i),j=1,4)
  200    continue
         if (name .eq. '   ') then
            done = .true.
            ndih = i - 1
         else
            lext = trimtext (name)
            idih(i) = freeunit ()
            dihfile(i) = filename(1:leng)//"."//name(1:lext)
            call version (dihfile(i),'new')
            open (unit=idih(i),file=dihfile(i),status='new')
         end if
      end do
c
c     write out the type of computation to be performed
c
      if (dogeom) then
         write (iout,210)
  210    format (//' Calculation will Involve Full Perturbation',
     &              ' Energy '/)
      else
         write (iout,220)
  220    format (//' Calculation will Consider Only Intermolecular',
     &              ' Interactions '/)
      end if
c
c     cycle over the user specified coordinate files
c     once per lambda value
c
      do ilamb = 1, nlamb
         write (iout,230)  ilamb
  230    format (' ILAMB ',i8)
         i = start
         istep = 0
c
c     find the name of the next coordinates file
c
         dowhile (i.ge.start .and. i.le.stop)
            call numeral (i,ext,lext)
            coordfile = filename(1:leng)//'.'//ext(1:lext)
            call version (coordfile,'old')
c
c     read in the next molecular dynamics coordinate frame
c
            ixyz = freeunit ()
            open (unit=ixyz,file=coordfile,status='old',err=280)
            call readxyz (ixyz)
            close (unit=ixyz)
            istep = istep + 1
c
c     set up the appropriate force field first time through
c
            if (istep .eq. 1) then
               if (ilamb .eq. 1) then
                  call mechanic
                  verbose = .false.
                  if (.not. dogeom) then
                     def_b = use_bond
                     def_a = use_angle
                     def_s = use_strbnd
                     def_i = use_imptor
                     def_t = use_tors
                     def_st = use_strtor
                  end if
                  do j = 1, n
                     use_def(j) = use(j)
                  end do
                  solute1 = imol(1,1)
                  solute2 = imol(2,1)
                  def_debug = debug
                  if (debug) write (iout,260)
  260             format (' istep   eb   ea   eit   et   e14   ev   ec')
               else
                  if (ilamb .eq. 2)  lambda = lamp
                  if (ilamb .eq. 3)  lambda = lamm
                  call hatom
                  call hbond
                  call hangle
                  call hstrbnd
                  call himptor
                  call htors
                  call hstrtor
                  call hvdw
                  call hcharge
                  call hdipole
               end if
            end if
c
c     calculate the potential energy of the system
c
            if (ilamb .eq. 1) then
               do j = 1, n
                  use(j) = use_def(j)
               end do
               if (.not. dogeom) then
                  use_bond = def_b
                  use_angle = def_a
                  use_strbnd = def_s
                  use_imptor = def_i
                  use_tors = def_t
                  use_strtor =  def_st
               end if
               debug = .false.
               call analysis
               debug = def_debug
c
c     sum up the components to find total steric energy
c
               e_pot = eb + ea + eba + eit + et + ebt + e14
     &                    + ev + ec + ecd + ed + ex
               ptnl(istep) = e_pot
               write (ipot,240)  istep,ptnl(istep)
  240          format (i10,f15.4)
c
c     calculate solute-solvent coulombic interaction ( assumes no uu type)
c
               do ii = 1, nion
                  iu = iion(ii)
                  if (iu .gt. solute2) goto 278
                  coul_uv = coul_uv + aec(iu)
  278             continue
               end do
c
c     calculate solute-solvent vdw interaction ( assumes no uu type)
c
               do iu = solute1, solute2
                  vdw_uv = vdw_uv + aev(iu)
               end do
c
c     calculate the chosen structural quantities
c
               do j = 1, ndih
                  ia = jdih(1,j)
                  ib = jdih(2,j)
                  ic = jdih(3,j)
                  id = jdih(4,j)
                  dih = dihedral (ia,ib,ic,id)
                  write (idih(j),250)  istep,dih
  250             format (i10,f12.4)
               end do
            end if
c
c     select interactions for perturbation energy calculation
c
            do j = 1, n
               use(j) = .false.
            end do
            do j = 1, nhybrid
               use(ihybrid(j)) = .true.
            end do
            if (.not. dogeom) then
               use_bond = .false.
               use_angle = .false.
               use_strbnd = .false.
               use_imptor = .false.
               use_tors = .false.
               use_strtor = .false.
            end if
            eb = 0.0d0
            ea = 0.0d0
            eba = 0.0d0
            eit = 0.0d0
            et = 0.0d0
            ebt = 0.0d0
            ev = 0.0d0
            ed = 0.0d0
            ex = 0.0d0
c
c     store the energy components for the current lambda
c
            nrg(ilamb,istep) = energy ()
            if (debug) then
               if (dogeom) then
                  write (iout,270)  istep,eb,ea,eit,et,e14,ev,ec
               else
                  write (iout,270)  istep,eb,ea,eit,et,0.0,ev,ec
                endif
  270          format (i8,7f10.4)
            end if
            if (dogeom) then
               cb(ilamb,istep) = eb
               ca(ilamb,istep) = ea
               cit(ilamb,istep) = eit
               ct(ilamb,istep) = et
            end if
            c14(ilamb,istep) = e14
            cv(ilamb,istep) = ev
            cc(ilamb,istep) = ec
  280       continue
            i = i + 1
         end do
      end do
      nstep = istep
      close (unit=ipot)
      do i = 1, ndih
         close (unit=idih(i))
      end do
c
c     open the output file for block averages
c
      iblk = freeunit ()
      blkfile = filename(1:leng)//'.blk'
      call version (blkfile,'new')
      open (unit=iblk,file=blkfile,status='new')
      write (iblk,290)
  290 format (' Block    DA+     DA-         -TdS+    ',
     :        ' -TdS-        DE+     DE-')
c
c     open the output file for Free Energy trajectory
c
      ife = freeunit ()
      fefile = filename(1:leng)//'.fe'
      call version (fefile,'new')
      open (unit=ife,file=fefile,status='new')
c
c     set some constants
c
      nrun  = stop - start + 1 + nprev
      if (nprev .eq. 0) then
         nblock = nrun / nblock
      else
         nblock = nprev / nblock
      endif
      if (nblock .eq. 0)  nblock = 1
      kb = boltzmann / convert
      rt = kb * temp
      beta = 1.0d0 / rt
      deltat = 10.0d0
      tp = temp + deltat
      tm = temp - deltat
      lp = 1.0d0/kb * (1.0d0/tp - 1.0d0/temp)
      lm = 1.0d0/kb * (1.0d0/tm - 1.0d0/temp)
      tp = 1.0d0 / kb / tp
      tm = 1.0d0 / kb / tm
c
c     get free energy change via average over all frames
c
      do istep = 1, nstep
         epot0 = ptnl(istep)
         e0 = nrg(1,istep)
         ep = nrg(2,istep)
         em = nrg(3,istep)
         eptm = exp(e0*beta - ep*tm)
         eptp = exp(e0*beta - ep*tp)
         emtm = exp(e0*beta - em*tm)
         emtp = exp(e0*beta - em*tp)
         e0lm = exp(-e0*lm)
         e0lp = exp(-e0*lp)
c
c     potential energy components
c
         e140 = c14(1,istep)
         e14p = c14(2,istep)
         e14m = c14(3,istep)
         if (.not. dogeom) then
            e0 = e0 - e140
            ep = ep - e14p
            em = em - e14m
            e140 = 0.0d0
            e14p = 0.0d0
            e14m = 0.0d0
         end if
         ev0 = cv(1,istep)
         evp = cv(2,istep)
         evm = cv(3,istep)
         ec0 = cc(1,istep)
         ecp = cc(2,istep)
         ecm = cc(3,istep)
         if (dogeom) then
            eb0 = cb(1,istep)
            ebp = cb(2,istep)
            ebm = cb(3,istep)
            ea0 = ca(1,istep)
            eap = ca(2,istep)
            eam = ca(3,istep)
            eit0 = cit(1,istep)
            eitp = cit(2,istep)
            eitm = cit(3,istep)
            et0 = ct(1,istep)
            etp = ct(2,istep)
            etm = ct(3,istep)
         end if
         modblock = mod(nprev+istep-1,nblock)
c
c     zero out summation variables for new block average
c
         if (modblock .eq. 0) then
            bdep = 0.0d0
            bdem = 0.0d0
            bpos = 0.0d0
            bneg = 0.0d0
            beptm = 0.0d0
            beptp =  0.0d0
            bemtm =  0.0d0
            bemtp =  0.0d0
            be0lm =  0.0d0
            be0lp =  0.0d0
         end if
         modblock = mod(nprev+istep,nblock)
c
c     accumulate statistics
c
         dep = ep - e0
         dem = em - e0
         de14p = e14p - e140
         de14m = e14m - e140
         devp = evp - ev0
         devm = evm - ev0
         decp = ecp - ec0
         decm = ecm - ec0
         if (debug) then
            write (iout,300)  istep,e0,ep,em,dep,dem
  300       format (1x,'e0,ep,em,dep,dem',i4,5f8.3)
         end if
         pos = exp(-dep*beta)
         neg = exp(-dem*beta)
         bdep = bdep + dep
         bdem = bdem + dem
         bpos = bpos + pos
         bneg = bneg + neg
         sdep = sdep + dep
         sdem = sdem + dem
         s2dep = s2dep + dep*dep
         s2dem = s2dem + dem*dem
         spos = spos + pos
         sneg = sneg + neg
         sepot0 = sepot0 + epot0
         s2epot0 = s2epot0 + epot0*epot0
         beptm = beptm + eptm
         beptp = beptp + eptp
         bemtm = bemtm + emtm
         bemtp = bemtp + emtp
         be0lm = be0lm + e0lm
         be0lp = be0lp + e0lp
         septm = septm + eptm
         septp = septp + eptp
         semtm = semtm + emtm
         semtp = semtp + emtp
         se0lm = se0lm + e0lm
         se0lp = se0lp + e0lp
c
c     potential energy components
c
         s14p = s14p + de14p
         s14m = s14m + de14m
         svp = svp + devp
         svm = svm + devm
         scp = scp + decp
         scm = scm + decm
         s214p = s214p + de14p**2
         s214m = s214m + de14m**2
         s2vp = s2vp + devp **2
         s2vm = s2vm + devm **2
         s2cp = s2cp + decp**2
         s2cm = s2cm + decm **2
         if (dogeom) then
            debp = ebp - eb0
            debm = ebm - eb0
            deap = eap - ea0
            deam = eam - ea0
            deitp = eitp - eit0
            deitm = eitm - eit0
            detp = etp - et0
            detm = etm - et0
            sbp = sbp + debp
            sbm = sbm + debm
            sap = sap + deap
            sam = sam + deam
            sitp = sitp + deitp
            sitm = sitm + deitm
            stp = stp + detp
            stm = stm + detm
            s2bp = s2bp + debp**2
            s2bm = s2bm + debm**2
            s2ap = s2ap + deap**2
            s2am = s2am + deam**2
            s2itp = s2itp + deitp**2
            s2itm = s2itm + deitm**2
            s2tp = s2tp + detp**2
            s2tm = s2tm + detm**2
c
c     contains error in bonded charge contribution
c
            enb0 = ev0 + ec0
            enbp = evp + ecp
            enbm = evm + ecm
            eg0 = e0 - enb0
            egp = ep - enbp
            egm = em - enbm
            degp = egp - eg0
            degm = egm - eg0
            denbp = enbp - enb0
            denbm = enbm - enb0
         end if
c
c     write out Free Energy as a function of trajectory length
c
            da = spos / dble(istep+nprev)
            da = -rt * log (da)
            dap = da
            da = sneg / dble(istep+nprev)
            da = -rt * log (da)
            dam = da
            write(ife,*) istep+nprev,dap,dam
c
c     calculate block averages
c
         if (modblock .eq. 0) then
            jb = jb + 1
            badep(jb) = bdep / dble(nblock)
            badem(jb) = bdem / dble(nblock)
c
c     free energy change
c
            bda = bpos / dble(nblock)
            bda = -rt * log(bda)
            bdap(jb) = bdap(jb) +  bda
            bda = bneg / dble(nblock)
            bda = -rt * log(bda)
            bdam(jb) = bdam(jb) + bda
            sdapb = sdapb + bdap(jb)
            sdamb = sdamb + bdam(jb)
c
c     entropy change
c
            baeptm = beptm / be0lm
            baeptp = beptp / be0lp
            baemtm = bemtm / be0lm
            baemtp = bemtp / be0lp
            bentp = baeptm / baeptp
            bentm = baemtm / baemtp
            bentp = rt * temp * log(bentp)
            bentm = rt * temp * log(bentm)
            bentp = bentp / 2.0d0 / deltat
            bentm = bentm / 2.0d0 / deltat
            baentp(jb) = bentp + bdap(jb)
            baentm(jb) = bentm + bdam(jb)
            sentpb = sentpb + baentp(jb)
            sentmb = sentmb + baentm(jb)
            if (debug) then
               write (iout,310)  jb,istep,badep(jb),badem(jb)
  310          format (' E jb istep ',2i8,2f15.4)
               write (iout,320)  jb,istep,bdap(jb),bdam(jb)
  320          format (' A jb istep ',2i8,2f15.4)
               write (iout,325)  jb,istep,bentp,bentm
  325          format (' -TdS jb istep ',2i8,2f15.4)
            end if
         end if
      end do
c
c     calculate running averages
c
c
c     potential energy:
c
            aepot0 = sepot0 / dble(nrun)
            a2epot0 = s2epot0 / dble(nrun)
            aepot02 = aepot0 * aepot0
            fepot0 = sqrt(a2epot0 - aepot02)
            adep = sdep / dble(nrun)
            adem = sdem / dble(nrun)
            a2dep = s2dep / dble(nrun)
            a2dem = s2dem / dble(nrun)
            adep2 = adep * adep
            adem2 = adem * adem
            fdep =  sqrt(a2dep - adep2)
            fdem =  sqrt(a2dem - adem2)
            do jb = 1, nrun / nblock
               v = (badep(jb) - adep)**2
               vdep = vdep + v
               v = (badem(jb) - adem)**2
               vdem = vdem + v
            end do
            vdep = vdep / dble(nrun/nblock)
            se_ep = sqrt(vdep / dble(nrun/nblock))
            vdem = vdem / dble(nrun/nblock)
            se_em = sqrt(vdem / dble(nrun/nblock))
c
c      solute-solvent energy for this segment of run
c
            coul_uv = 2.0d0 * coul_uv / dble(nrun-nprev)
            vdw_uv =  2.0d0 * vdw_uv / dble(nrun-nprev)
c
c     free energy:
c
            da = spos / dble(nrun)
            da = -rt * log (da)
            dap = da
            da = sneg / dble(nrun)
            da = -rt * log (da)
            dam = da
            adapb = sdapb / dble(nrun/nblock)
            adamb = sdamb / dble(nrun/nblock)
            do jb = 1, nrun/nblock
               v = (bdap(jb) - adapb)**2
               vdap = vdap + v
               v = (bdam(jb) - adamb)**2
               vdam = vdam + v
            end do
            vdap = vdap / dble(nrun/nblock)
            se_ap = sqrt(vdap / dble(nrun/nblock))
            vdam = vdam / dble(nrun/nblock)
            se_am = sqrt(vdam / dble(nrun/nblock))
c
c     entropy change
c
            aeptm = septm / se0lm
            aeptp = septp / se0lp
            aemtm = semtm / se0lm
            aemtp = semtp / se0lp
            entp = aeptm / aeptp
            entm = aemtm / aemtp
            entp = rt * temp * log(entp)
            entm = rt * temp * log(entm)
            entp = entp / 2.0d0 / deltat
            entm = entm / 2.0d0 / deltat
            enrgp = -entp
            enrgm = -entm
            entp = entp + dap
            entm = entm + dam
            aentpb = sentpb / dble(nrun/nblock)
            aentmb = sentmb / dble(nrun/nblock)
            do jb = 1, nrun/nblock
               v = (baentp(jb) - aentpb)**2
               ventp = ventp + v
               v = (baentm(jb) - aentmb)**2
               ventm = ventm + v
            end do
            ventp = ventp / dble(nrun/nblock)
            se_entp = sqrt(ventp / dble(nrun/nblock))
            ventm = ventm / dble(nrun/nblock)
            se_entm = sqrt(ventm / dble(nrun/nblock))
c
c     energy components:
c
            a14p = s14p / dble(nrun)
            a14m = s14m / dble(nrun)
            avp = svp / dble(nrun)
            avm = svm / dble(nrun)
            acp = scp / dble(nrun)
            acm = scm / dble(nrun)
            a214p = s214p / dble(nrun)
            a214m = s214m / dble(nrun)
            a2vp = s2vp / dble(nrun)
            a2vm = s2vm / dble(nrun)
            a2cp = s2cp / dble(nrun)
            a2cm = s2cm / dble(nrun)
            a14p2 = a14p * a14p
            a14m2 = a14m * a14p
            avp2 = avp * avp
            avm2 = avm * avm
            acp2 = acp * acp
            acm2 = acm * acm
            f14p = sqrt(a214p - a14p2)
            f14m = sqrt(a214m - a14m2)
            fvp = sqrt(a2vp - avp2)
            fvm = sqrt(a2vm - avm2)
            fcp = sqrt(a2cp - acp2)
            fcm = sqrt(a2cm - acm2)
            if (dogeom) then
               abp = sbp / dble(nrun)
               abm = sbm / dble(nrun)
               aap = sap / dble(nrun)
               aam = sam / dble(nrun)
               aitp = sitp / dble(nrun)
               aitm = sitm / dble(nrun)
               atp = stp / dble(nrun)
               atm = stm / dble(nrun)
               a2bp = s2bp / dble(nrun)
               a2bm = s2bm / dble(nrun)
               a2ap = s2ap / dble(nrun)
               a2am = s2am / dble(nrun)
               a2itp = s2itp / dble(nrun)
               a2itm = s2itm / dble(nrun)
               a2tp = s2tp / dble(nrun)
               a2tm = s2tm / dble(nrun)
               abp2 = abp * abp
               abm2 = abm * abm
               aap2 = aap * aap
               aam2 = aam * aam
               aitp2 = aitp * aitp
               aitm2 = aitm * aitm
               atp2 = atp * atp
               atm2 = atm * atm
               fbp = sqrt(a2bp - abp2)
               fbm = sqrt(a2bm - abm2)
               fap = sqrt(a2ap - aap2)
               fam = sqrt(a2am - aam2)
               fitp = sqrt(a2itp - aitp2)
               fitm = sqrt(a2itm - aitm2)
               ftp = sqrt(a2tp - atp2)
               ftm = sqrt(a2tm - atp2)
            end if
c
c     write information about running and block averages
c
            write (iout,330)  nstep+nprev
  330       format (/' Running Averages over ',i7,' Steps : ')
            write (iout,'(8x,a,i3,a)')  ' (SD Computed Using ',
     &                                  nrun/nblock,' Subaverages) '
            write (iout,340)
  340       format (/' Free Energy :',/)
            write (iout,342)  dap,se_ap
  342       format (' dA(+) = ',f9.4,' with Std Error ',f9.4)
            write (iout,344)  dam,se_am
  344       format (' dA(-) = ',f9.4,' with Std Error ',f9.4)
            write (iout,346)
  346       format (/' Entropy :',/)
            write (iout,348)  entp,se_entp
  348       format (' -TdS(+) = ',f9.4,' with Std Error ',f9.4)
            write (iout,350)  entm,se_entm
  350       format (' -TdS(-) = ',f9.4,' with Std Error ',f9.4)
            write (iout,352)
  352       format (/' Thermodynamic Energy Calculation :',/)
            write (iout,354)  enrgp
  354       format (' dA+TdS(+) = ',f9.4)
            write (iout,356)  enrgm
  356       format (' dA+TdS(-) = ',f9.4)
            write (iout,358)
  358       format (/' Direct Mechanical Energy Calculation :',/)
            write (iout,360)  adep,fdep,se_ep
  360       format (' dE(+) = ',f9.4,' with Fluc ',f9.4,
     &                      ' and Std Error',f9.4)
            write (iout,362)  adem,fdem,se_em
  362       format (' dE(-) = ',f9.4,' with Fluc ',f9.4,
     &                      ' and Std Error',f9.4)
            write (iout,364)  aepot0,fepot0
  364       format (' E(0) = ',f10.3,' with Fluc ',f10.3)
            write (iout,366)
  366       format (/24x,' Component Energies :'/,
     :               2(12x,'Average',12x,'Std. Dev.'),/)
            if (dogeom) then
               write (iout,368)  'BOND  +/- :',abp,fbp,abm,fbm
               write (iout,368)  'ANGLE +/- :',aap,fap,aam,fam
               write (iout,368)  'IMPT  +/- :',aitp,fitp,aitm,fitm
               write (iout,368)  'TORS  +/- :',atp,ftp,atm,ftm
            end if
            write (iout,368)  '14VDW +/- :',a14p,f14p,a14m,f14m
            write (iout,368)  'VDW   +/- :',avp,fvp,avm,fvm
            write (iout,368)  'COUL  +/- :',acp,fcp,acm,fcm
  368       format(a,4f10.4)
            write(iout,*)
            write(iout,368) 'Solute-Solvent interaction energy',
     :                   coul_uv+vdw_uv
            write(iout,368) 'Coulombic portion: ',coul_uv
            write(iout,368) '   VDW    portion: ',vdw_uv
            do jb = 1, nrun / nblock
               write (iblk,370)  jb,bdap(jb),bdam(jb),
     &                          baentp(jb),baentm(jb),
     &                          badep(jb),badem(jb)
  370          format (i5,6f10.4)
            end do
      close (unit=iblk)
c
c     write out accumulator file
c
      isum = freeunit ()
      sumfile = filename(1:leng)//'.sum'
      call version (sumfile,'new')
      open (unit=isum,file=sumfile,status='new')
      write (isum,*) nrun,nrun/nblock
      write (isum,*) delta,temp,dogeom
      write (isum,*) (badep(jb),badem(jb),jb = 1,
     :                      (nrun/nblock) )
      write (isum,*) (bdap(jb), bdam(jb), jb = 1,
     :                      (nrun/nblock) )
      write (isum,*) (baentp(jb), baentm(jb), jb = 1,
     :                      (nrun/nblock) )
      write (isum,*) sdep,sdem,spos,sneg,sepot0,s2epot0
      write (isum,*) septm,septp,semtm,semtp,se0lm,se0lp
      write(isum,*)  sdapb,sdamb,sentpb,sentmb
      write (isum,*) s14p,s14m,svp,svm,scp,scm
      write (isum,*) sbp,sbm,sap,sam,sitp,sitm,stp,stm
      write (isum,*) s2dep,s2dem
      write (isum,*) s214p,s214m,s2vp,s2vm,s2cp,s2cm
      write (isum,*) s2bp,s2bm,s2ap,s2am,s2itp,s2itm,s2tp,s2tm
      close (unit=isum)
      end
