c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1993  by  Jay William Ponder  ##
c     ###################################################
c
c     ###########################################################
c     ##                                                       ##
c     ##  subroutine shakeup  --  setup holonomic constraints  ##
c     ##                                                       ##
c     ###########################################################
c
c
c     "shakeup" initializes any holonomic constraints for use
c     with the rattle algorithm during molecular dynamics
c
c
      subroutine shakeup
      implicit none
      include 'sizes.i'
      include 'angle.i'
      include 'atmbnd.i'
      include 'atmtyp.i'
      include 'bond.i'
      include 'couple.i'
      include 'files.i'
      include 'math.i'
      include 'shake.i'
      integer i,j,ikey,next,freeunit
      integer ia,ib,ic,ita,itb,itc
      real*8 rab,rbc,rac,cosine
      character*6 rattyp
      character*20 keyword
      character*60 keyfile
      character*80 record,string
      logical exist
c
c
c     first turn off use of rattle until needed
c
      use_rattle = .false.
      nrat = 0
c
c     check for the existence of a keyword file
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
c
c     search each line of the keyword file for options
c
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,10,end=20)  record
   10       format (a80)
            next = 1
            call upcase (record)
            call gettext (record,keyword,next)
c
c     get the distance constraint types for the rattle method;
c
            if (keyword(1:7) .eq. 'RATTLE ') then
               call getword (record,rattyp,next)
c
c     constrain all bond lengths to their ideal values
c
               if (rattyp(1:5) .eq. 'BONDS') then
                  do i = 1, nbond
                     ia = ibnd(1,i)
                     ib = ibnd(2,i)
                     nrat = nrat + 1
                     irat(1,nrat) = ia
                     irat(2,nrat) = ib
                     krat(nrat) = bl(i)
                  end do
c
c     constrain 1-3 distance across each angle at ideal value
c
               else if (rattyp(1:6) .eq. 'ANGLES') then
                  do i = 1, nangle
                     ia = iang(1,i)
                     ib = iang(2,i)
                     ic = iang(3,i)
                     do j = 1, n12(ib)
                        if (i12(j,ib) .eq. ia) then
                           rab = bl(iatmbnd(j,ib))
                        else if (i12(j,ib) .eq. ic) then
                           rbc = bl(iatmbnd(j,ib))
                        end if
                     end do
                     cosine = cos(anat(i)/radian)
                     rac = sqrt(rab**2+rbc**2-2.0d0*rab*rbc*cosine)
                     nrat = nrat + 1
                     irat(1,nrat) = ia
                     irat(2,nrat) = ic
                     krat(nrat) = rac
                  end do
c
c     fix bonds and angle of waters to give a rigid molecule
c
               else if (rattyp(1:5) .eq. 'WATER') then
                  do i = 1, nangle
                     ia = iang(1,i)
                     ib = iang(2,i)
                     ic = iang(3,i)
                     ita = atomic(ia)
                     itb = atomic(ib)
                     itc = atomic(ic)
                     if (ita.eq.1 .and. itb.eq.8 .and. itc.eq.1) then
                        rab = bl(iatmbnd(1,ia))
                        rbc = bl(iatmbnd(1,ic))
                        cosine = cos(anat(i)/radian)
                        rac = sqrt(rab**2+rbc**2-2.0d0*rab*rbc*cosine)
                        nrat = nrat + 1
                        irat(1,nrat) = ia
                        irat(2,nrat) = ib
                        krat(nrat) = rab
                        nrat = nrat + 1
                        irat(1,nrat) = ib
                        irat(2,nrat) = ic
                        krat(nrat) = rbc
                        nrat = nrat + 1
                        irat(1,nrat) = ia
                        irat(2,nrat) = ic
                        krat(nrat) = rac
                     end if
                  end do
c
c     fix all X-H bonds at ideal length, the current default
c
               else
                  do i = 1, nbond
                     ia = ibnd(1,i)
                     ib = ibnd(2,i)
                     ita = atomic(ia)
                     itb = atomic(ib)
                     if (ita.eq.1 .or. itb.eq.1) then
                        nrat = nrat + 1
                        irat(1,nrat) = ia
                        irat(2,nrat) = ib
                        krat(nrat) = bl(i)
                     end if
                  end do
               end if
c
c     get the specific distance constraints for the rattle method;
c
            else if (keyword(1:11) .eq. 'RATTLEBOND ') then
              nrat = nrat + 1
              krat(nrat) = 0.0d0
              string = record(next:80)
              read (string,*,err=20)  (irat(j,nrat),j=1,2),
     &                                 krat(nrat)
              if (nrat .ne. 0)  use_rattle = .true.
            end if
         end do
   20    continue
      end if
      if (nrat .ne. 0)  use_rattle = .true.
      return
      end
