c
c
c     ###############################################################
c     ##  COPYRIGHT (C) 1991 by Shawn Huston & Jay William Ponder  ##
c     ###############################################################
c
c     ############################################################
c     ##                                                        ##
c     ##  program ti  --  obtain TI integrand                   ##
c     ##                                                        ##
c     ############################################################
c
c
c     "ti" computes the required integrand for TI, for the case in
c     which the potential parameters depend linearly on Lambda.
c
c 	NOTES on current version :
c
c      		The set of solutes dmp-dmp-tmb has the property that
c		the u-v derivative  may be obtained by calling the
c		vdw and coul routines. There will be no u-u component.
c               ( i.e. there are no solute 1-5 vdw, and no solute 1-4
c               coul interactions)
c
c		Similarly, the u-u derivative may be obtained by calling
c  		the geom routines, and collecting the vdw1-4 contribution.
c               There will be no u-v component, as these ae intramolecular
c               quantities.
c
c	NOTES on a more general version
c
c		In general to obtain the u-u derivative where vdw and
c  		coul forces may play a role, the following may be done:
c			1) shut off all non-hybrid sites via 'use(i)
c			2) use the logical 'if ( use(i) .and. use(k) ) then'
c			   in derivative routines
c
c     nrun    number of steps over which to calculate averages
c
c     se_auv,se_auu  standard error of integrands
c
c
      program ti
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'analyz.i'
      include 'atoms.i'
      include 'charge.i'
      include 'dlambda.i'
      include 'files.i'
      include 'inform.i'
      include 'iounit.i'
      include 'katoms.i'
      include 'molcul.i'
      include 'mutant.i'
      include 'potent.i'
      include 'units.i'
      integer maxblock,maxstep,maxdih
      parameter (maxblock=100)
      parameter (maxdih=10)
      parameter (maxstep=10000)
      integer i,j,start,stop,next,last
      integer ixyz,ikey,ipot,iblk,iuv,iuu
      integer ihyb,it0,it1
      integer jb,nblock,nrun,modblock
      integer istep,nstep,ilamb,nlamb
      integer ia,ib,ic,id
      integer ii,iu,solute1,solute2
      integer freeunit,trimtext,lext
      real*8  ddltot(maxstep),ddluv(maxstep),ddluu(maxstep)
      real*8  duv,duu,dtot
      real*8 epot(maxstep),energy,dbydl
      real*8 da,kb,temp,deltat,tplus,tminus
      real*8 lp,lm
      real*8 bduv,bduu,bdtot
      real*8 bda,bdauv(maxblock),bdauu(maxblock),bdatot(maxblock)
      real*8 dauvb,dauub,datotb
      real*8 sdauv,sdauu,sdatot
      real*8 sduv,sduu,sdtot
      real*8 s2duv,s2duu,s2dtot
      real*8 dauv,dauu,datot
      real*8 vauu,vauv,vatot
      real*8 se_auv,se_auu,se_atot
      real*8 d2auv,d2auu,d2atot,dauv2,dauu2,datot2,fauv,fauu,fatot
      real*8 e,se,s2e,ae,ae2,a2e,fe
      real*8 pd,pnuv,pnuu,pntot
      real*8 md,mnuv,mnuu,mntot
      real*8 bpd,bpnuv,bpnuu,bpntot
      real*8 bmd,bmnuv,bmnuu,bmntot
      real*8 bapuv,bapuu,baptot
      real*8 bamuv,bamuu,bamtot
      real*8 bentuv(maxblock),bentuu(maxblock),benttot(maxblock)
      real*8 spd,spnuv,spnuu,spntot
      real*8 smd,smnuv,smnuu,smntot
      real*8 apuv,apuu,aptot
      real*8 amuv,amuu,amtot
      real*8 entuv,entuu,enttot
      real*8 entuvb,entuub,enttotb
      real*8 sentuv,sentuu,senttot
      real*8 v,ventuv,ventuu,venttot
      real*8 se_entuv,se_entuu,se_enttot
      character*1 letter,answer
      character*7 ext
      character*10 name
      character*20 keyword
      character*60 keyfile,potfile,coordfile,blkfile
      character*60 uvfile,uufile
      character*80 record,string
      logical use_def(maxatm),def_a,def_b
      logical def_i,def_s,def_st,def_t
      logical done,exist
      external freeunit,trimtext,dbydl,energy
c
c
c     get the name of the coordinates files
c
      call initial
      write (iout,10)
   10 format (' Enter Name of the Coordinate Files :  ',$)
      read (input,20)  filename
   20 format (a60)
      leng = trimtext (filename)
c
c     remove any extension from the filename
c
      last = leng
      do i = 1, leng
         letter = filename(i:i)
         if (letter .eq. '/')  last = leng
         if (letter .eq. ']')  last = leng
         if (letter .eq. ':')  last = leng
         if (letter .eq. '.')  last = i - 1
      end do
      leng = min(leng,last)
c
c     get the numbers of the files to be used
c
      write (iout,30)
   30 format (' Numbers of First and Last File to Analyze :  ',$)
      read (input,40)  start,stop
   40 format (2i10)
c
c     check for the existence of a keyword file
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
c
c     search each line of the keyword file for options
c
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,50,end=70)  record
   50       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
            if (keyword(1:7) .eq. 'LAMBDA ') then
               nhybrid = 0
               lambda = 0.0d0
               string = record(next:80)
               read (string,*,err=60)  lambda
            else if (keyword(1:7) .eq. 'MUTATE ') then
               string = record(next:80)
               read (string,*,err=60)  ihyb,it0,it1
               nhybrid = nhybrid + 1
               ihybrid(nhybrid) = ihyb
               itype0(nhybrid) = it0
               itype1(nhybrid) = it1
               alter(ihyb) = .true.
            end if
   60       continue
         end do
   70    continue
         close (unit=ikey)
      end if
c
c     obtain target temperature
c
      temp = 0.0d0
      write (iout,110)
  110 format (' Enter the System Temperature [300 K] :  ',$)
      read (input,120)  temp
  120 format (f10.5)
      if (temp .eq. 0.0d0)  temp = 300.0d0
      kb = boltzmann / convert
      deltat = 10.0
      tplus = temp + deltat
      tminus = temp - deltat
      lp = 1.0d0/kb * (1.0d0/tplus - 1.0d0/temp)
      lm = 1.0d0/kb * (1.0d0/tminus - 1.0d0/temp)
c
c     set number of steps for running averages and block averages
c
      jb = 0
  129 continue
      write (iout,130)
  130 format (' Enter Number of Blocks for Sub-Averages :  ',$)
      read (input,140)  nblock
      if (nblock .eq. 0) goto 129
  140 format (i5)
c
c     obtain the quantities to be calculated
c
c
c     cycle over the user specified coordinate files
c
         i = start
         istep = 0
c
c     find the name of the next coordinates file
c
         dowhile (i.ge.start .and. i.le.stop)
            call numeral (i,ext,lext)
            coordfile = filename(1:leng)//'.'//ext(1:lext)
            call version (coordfile,'old')
c
c     read in the next molecular dynamics coordinate frame
c
            ixyz = freeunit ()
            open (unit=ixyz,file=coordfile,status='old',err=280)
            call readxyz (ixyz)
            close (unit=ixyz)
            istep = istep + 1
c
c     set up the appropriate force field first time through
c
            if (istep .eq. 1) then
               call mechanic
               verbose = .false.
               do j = 1, n
                  use_def(j) = use(j)
               end do
               solute1 = imol(1,1)
               solute2 = imol(2,1)
            end if
c
c     select interactions for derivative calculation
c
            do j = 1, n
               use(j) = .false.
            end do
            do j = 1, nhybrid
               use(ihybrid(j)) = .true.
            end do
c
c     get the interaction derivatives
c
            use_bond = .true.
            use_angle = .true.
            use_imptor = .true.
            use_tors = .true.
            use_lj = .true.
            use_charge = .true.
            ddltot(istep) = dbydl ()
            ddluu(istep) = ddlbond + ddlangle + ddlitors +
     :                     ddltors + ddlvdw14
            ddluv(istep) = ddlvdw + ddlcharge
c
c     get the total interaction energy
c
            do j = 1, n
               use(j) = .true.
            end do
            use_bond = .true.
            use_angle = .true.
            use_imptor = .true.
            use_tors = .true.
            use_lj = .true.
            use_charge = .true.
            epot(istep) = energy ()
  280       continue
            i = i + 1
         end do
      nstep = istep
c
c     open the output file for block averages
c
      iblk = freeunit ()
      blkfile = filename(1:leng)//'.blk'
      call version (blkfile,'new')
      open (unit=iblk,file=blkfile,status='new')
      write (iblk,290)
  290 format (' Block  (A) Total  (A) u-v  (A) u-u  ',
     :              '(-TdS) Total  (-TdS) u-v  (-TdS) u-u)')
c
c     open the output file for potential energy
c
      ipot = freeunit ()
      potfile = filename(1:leng)//'.pot'
      call version (potfile,'new')
      open (unit=ipot,file=potfile,status='new')
c
c     open the output files for integrand trajectories
c
      iuv = freeunit ()
      uvfile = filename(1:leng)//'.uvtim'
      call version (uvfile,'new')
      open (unit=iuv,file=uvfile,status='new')
c
      iuu = freeunit ()
      uufile = filename(1:leng)//'.uutim'
      call version (uufile,'new')
      open (unit=iuu,file=uufile,status='new')
c
c     set some constants
c
      nrun  = stop - start + 1
      nblock = nrun / nblock
      if (nblock .eq. 0)  nblock = 1
c
c     get thermodynamic quantities via average over all frames
c
      do istep = 1, nstep
         duv = ddluv(istep)
         duu = ddluu(istep)
         dtot = ddltot(istep)
         e = epot(istep)
         pd = exp(-e*lp)
         pnuv = pd * duv
         pnuu = pd * duu
         pntot = pd * dtot
         md = exp(-e*lm)
         mnuv = md * duv
         mnuu = md * duu
         mntot = md * dtot
         modblock = mod(istep-1,nblock)
c
c     zero out summation variables for new block average
c
         if (modblock .eq. 0) then
            bduv = 0.0d0
            bduu = 0.0d0
            bdtot = 0.0d0
            bpd = 0.0d0
            bpnuv = 0.0d0
            bpnuu = 0.0d0
            bpntot = 0.0d0
            bmd = 0.0d0
            bmnuv = 0.0d0
            bmnuu = 0.0d0
            bmntot = 0.0d0
         end if
         modblock = mod(istep,nblock)
c
c     accumulate statistics
c
         bduv = bduv + duv
         bduu = bduu + duu
         bdtot = bdtot + dtot
         sduv = sduv + duv
         sduu = sduu + duu
         sdtot = sdtot + dtot
         bpd = bpd + pd
         bpnuv = bpnuv + pnuv
         bpnuu = bpnuu + pnuu
         bpntot = bpntot + pntot
         bmd = bmd + md
         bmnuv = bmnuv + mnuv
         bmnuu = bmnuu + mnuu
         bmntot = bmntot + mntot
         spd = spd + pd
         spnuv = spnuv + pnuv
         spnuu = spnuu + pnuu
         spntot = spntot + pntot
         smd = smd + md
         smnuv = smnuv + mnuv
         smnuu = smnuu + mnuu
         smntot = smntot + mntot
         se = se + e
         s2duv = s2duv + duv**2
         s2duu = s2duu + duu**2
         s2dtot = s2dtot + dtot**2
         s2e = s2e + e**2
c
c     write out integrand as a function of trajectory length
c
         if (mod(istep,10) .eq. 0) then
            apuv = spnuv / spd
            apuu = spnuu / spd
            amuv = smnuv / smd
            amuu = smnuu / smd
            entuv = apuv - amuv
            entuu = apuu - amuu
            entuv =  entuv / 2.0d0 / deltat * temp
            entuu =  entuu / 2.0d0 / deltat * temp
            da = sduv / dble(istep)
            write(iuv,*) istep,da,entuv
            da = sduu / dble(istep)
            write(iuu,*) istep,da,entuu
         end if
c
c     calculate block averages
c
         if (modblock .eq. 0) then
            jb = jb + 1
            bda = bduv / dble(nblock)
            bdauv(jb) = bda
            bda = bduu / dble(nblock)
            bdauu(jb) =  bda
            bda = bdtot / dble(nblock)
            bdatot(jb) = bda
            sdauv = sdauv + bdauv(jb)
            sdauu = sdauu + bdauu(jb)
            sdatot = sdatot + bdatot(jb)
            bapuv = bpnuv / bpd
            bapuu = bpnuu / bpd
            baptot = bpntot / bpd
            bamuv = bmnuv / bmd
            bamuu = bmnuu / bmd
            bamtot = bmntot / bmd
            bentuv(jb) = (bapuv - bamuv) / 2.0d0 / deltat * temp
            bentuu(jb) = (bapuu - bamuu) / 2.0d0 / deltat * temp
            benttot(jb) =(baptot - bamtot) / 2.0d0 / deltat * temp
            sentuv = sentuv + bentuv(jb)
            sentuu = sentuu + bentuu(jb)
            senttot = senttot + benttot(jb)
         end if
      end do
c
c     calculate running averages
c
c
c     free energy:
c
      dauv = sduv / dble(nrun)
      dauu = sduu/ dble(nrun)
      datot = sdtot / dble(nrun)
      d2auv = s2duv / dble(nrun)
      dauv2 = dauv * dauv
      fauv = sqrt(d2auv - dauv2)
      d2auu = s2duu / dble(nrun)
      dauu2 = dauu * dauu
      fauu = sqrt(d2auu - dauu2)
      d2atot = s2dtot / dble(nrun)
      datot2 = datot * datot
      fatot = sqrt(d2atot - datot2)
c
c     standard error calculation
c
      dauub = sdauu / dble(nrun/nblock)
      dauvb = sdauv / dble(nrun/nblock)
      datotb = sdatot / dble(nrun/nblock)
      do jb = 1, nrun / nblock
         v = (bdauu(jb) - dauub)**2
         vauu = vauu + v
         v = (bdauv(jb) - dauvb)**2
         vauv = vauv + v
         v = (bdatot(jb) - datotb)**2
         vatot = vatot + v
      end do
      vauu = vauu / dble(nrun/nblock)
      se_auu = sqrt(vauu / dble(nrun/nblock))
      vauv = vauv / dble(nrun/nblock)
      se_auv = sqrt(vauv / dble(nrun/nblock))
      vatot = vatot / dble(nrun/nblock)
      se_atot = sqrt(vatot / dble(nrun/nblock))
c
c     potential energies
c
      ae = se / dble(nrun)
      a2e = s2e / dble(nrun)
      ae2 = ae * ae
      fe = sqrt(a2e - ae2)
c
c     entropy
c
      apuv = spnuv / spd
      apuu = spnuu / spd
      aptot = spntot / spd
      amuv = smnuv / smd
      amuu = smnuu / smd
      amtot = smntot / smd
      entuv = apuv - amuv
      entuu = apuu - amuu
      enttot = aptot - amtot
      entuv =  entuv / 2.0d0 / deltat * temp
      entuu =  entuu / 2.0d0 / deltat * temp
      enttot = enttot / 2.0d0 / deltat * temp
      entuvb = sentuv / dble(nrun/nblock)
      entuub = sentuu / dble(nrun/nblock)
      enttotb = senttot / dble(nrun/nblock)
      do jb = 1, nrun / nblock
         v = (bentuv(jb) - entuvb)**2
         ventuv = ventuv + v
         v = (bentuu(jb) - entuub)**2
         ventuu = ventuu + v
         v = (benttot(jb) - enttotb)**2
         venttot = venttot + v
      end do
      ventuv = ventuv / dble(nrun/nblock)
      se_entuv = sqrt(ventuv / dble(nrun/nblock))
      ventuu = ventuu / dble(nrun/nblock)
      se_entuu = sqrt(ventuu / dble(nrun/nblock))
      venttot = venttot / dble(nrun/nblock)
      se_enttot = sqrt(venttot / dble(nrun/nblock))
c
c     write information about running and block averages
c
         write (iout,330)  nstep
  330    format (/' Running Averages over ',i7,' Steps : ')
         write (iout,'(8x,a,i3,a)')  ' (SD Computed Using ',
     &                               nrun/nblock,' Subaverages) '
         write (iout,340)
  340    format (/' Free Energy Integrands :',/)
         write (iout,342)  datot,fatot,se_atot
  342    format (' Total = ',f9.4,' with Fluctuation ',f9.4,
     :           ' and Std Error ',f9.4)
         write (iout,344)  dauv,fauv,se_auv
  344    format (' u-v = ',f9.4,' with Fluctuation ',f9.4,
     :           ' and Std Error ',f9.4)
         write (iout,346)  dauu,fauu,se_auu
  346    format (' u-u = ',f9.4,' with Fluctuation ',f9.4,
     :           ' and Std Error ',f9.4)
         write (iout,348)
  348    format (/' Entropy Integrands ( -TdS) :',/)
         write (iout,350)  enttot,se_enttot
  350    format (' Total = ',f9.4,' with Std Error ',f9.4)
         write (iout,352)  entuv,se_entuv
  352    format (' u-v = ',f9.4,' with Std Error ',f9.4)
         write (iout,354)  entuu,se_entuu
  354    format (' u-u = ',f9.4,' with Std Error ',f9.4)
         write (iout,356) ae,fe
  356    format (/' Potential Energy :',f13.5,
     :            ' with Fluctuation ',f13.5)
         do jb = 1, nrun / nblock
            write (iblk,370)  jb,bdatot(jb),bdauv(jb),bdauu(jb)
     :                          ,benttot(jb),bentuv(jb),bentuu(jb)
  370       format (i5,3f10.4,3x,3f10.4)
         end do
c
c     write out potential energies
c
      do istep = 1, nstep
         write(ipot,*) istep,epot(istep)
      end do
      close (unit=iblk)
      close (unit=ipot)
      end
