c     
c     
c     ###################################################
c     ##  COPYRIGHT (C)  1994  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c     
c     ############################################################
c     ##                                                        ##
c     ##  subroutine egauss  --  Gaussian van der Waals energy  ##
c     ##                                                        ##
c     ############################################################
c     
c     
c     "egauss" calculates the van der Waals interaction energy
c     using a Gaussian expansion approximation
c     

c     This file was generated automatically from
c     $Id: egauss.F,v 0.2 1998/11/15 20:46:06 reece Exp $
c     ANY CHANGES MADE TO THIS FILE WILL BE LOST.
c     Make changes to egauss.F instead.

      subroutine egauss 
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'energi.i'
      include 'group.i'
      include 'shunt.i'
      include 'usage.i'
      include 'vdw.i'
      include 'vdwpot.i'
      include 'warp.i'
      integer i,j,k,ii,kk,iv,kv
      integer it,kt,skip(maxatm)
      real*8 e,rik2,rdn,eps,rad2,fgrp
      real*8 xi,yi,zi,xr,yr,zr
      real*8 expcut,expterm
      real*8 width,wterm
      real*8 t1,t2,a(4),b(4)
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      logical proceed,iuse
      integer g1,g2,iis,iie,kks,kke
      integer gne14,gnev
      real*8 ge14,gev

c     zero out the van der Waals energy contributions
      ev = 0.0d0
      e14 = 0.0d0

c     zero out the list of atoms to be skipped
      do i = 1, n
        skip(i) = 0
      end do

c     set cutoff distances and switching function coefficients
      call switch ('VDW')
      expcut = -50.0d0

c     scale extent of smoothing by the diffusion coefficient
      if (use_deform)  width = 4.0d0 * diffv * deform
      if (use_gda)  wterm = (2.0d0/3.0d0) * diffv

c     calculate the "reduced" atomic coordinates
      do k = 1, nvdw
        i = ivdw(k)
        iv = ired(i)
        rdn = kred(i)
        xred(i) = rdn*(x(i)-x(iv)) + x(iv)
        yred(i) = rdn*(y(i)-y(iv)) + y(iv)
        zred(i) = rdn*(z(i)-z(iv)) + z(iv)
      end do

c     loop over group pairs, g1 and g2.  For each, decide whether the
c     g1-g2 interaction is relevant and compute it in its entirety, or
c     skip to the next group.  For each group pair, the trick is to
c     choose [iis,iie] and [kks,kke], the index bounds of vdw sites in
c     groups 1 and 2 respectively, and to account for the special case
c     of g1==g2.

      do g1 = 0, ngrp
        iis = vdwgs(g1)
        if (iis .eq. 0) goto 100            ! g1 contains no atoms

        do g2 = g1, ngrp
          fgrp = wgrp(g1,g2)
          if ( fgrp .eq. 0.0d0) then
            if ( g1 .ne. 0 ) goto 101       ! skip group if grp weight is 0
            fgrp = 1.0d0                    ! always compute intx with grp 0
          end if

c         zero accumulators for this group pair
          ge14 = 0.0d0
          gev = 0.0d0

c         determine ii end point
          iie = vdwge(g1)
          if (g1 .eq. g2) iie = iie - 1     ! intragroup

c         compute all g1-g2 interactions by double looping over 
c         g1 vdw sites (ii) and g2 vdw sites (kk)

          do ii = iis, iie
            i = ivdw(ii)
            iv = ired(i)
            iuse = (use(i) .or. use(iv))

            it = class(i)
            xi = xred(i)
            yi = yred(i)
            zi = zred(i)
            do j = 1, n12(i)
              skip(i12(j,i)) = i * vdw12use
            end do
            do j = 1, n13(i)
              skip(i13(j,i)) = i * vdw13use
            end do
            do j = 1, n14(i)
              skip(i14(j,i)) = i * vdw14use
            end do

c           determine kk bounds
            kks = vdwgs(g2)
            if (g1 .eq. g2) kks = ii + 1   ! intragroup
            kke = vdwge(g2)

c           loop over g2 atoms
            do kk = kks, kke
              k = ivdw(kk)
              kv = ired(k)

c             decide whether to compute the current interaction
              proceed = .true.
              if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
              if (proceed)  proceed = (skip(k) .ne. i)

c             compute the energy contribution for this interaction
              if (proceed) then
                kt = class(k)
                xr = xi - xred(k)
                yr = yi - yred(k)
                zr = zi - zred(k)
                rik2 = xr*xr + yr*yr + zr*zr
                if (rik2 .le. off2) then
                  eps = epsilon(kt,it)
                  rad2 = radmin(kt,it)**2
                  do j = 1, ngauss
                    a(j) = igauss(1,j) * eps
                    b(j) = igauss(2,j) / rad2
                    if (skip(k) .eq. -i)  a(j) = a(j) / vdwscale
                  end do
                  e = 0.0d0

c                 transform the potential via diffusional smoothing
                  if (use_gda)  width = wterm * (m2(i)+m2(k))
                  do j = 1, ngauss
                    t1 = 1.0d0 + b(j)*width
                    t2 = sqrt(t1**3)
                    expterm = -b(j) * rik2 / t1
                    if (expterm .gt. expcut) then
                      expterm = (a(j)/t2)*exp(expterm)
                      e = e + expterm

                    end if
                  end do

c                 scale the interaction based on its group membership
                  if (use_group) then
                    e = e * fgrp

                  end if

c                 increment the van der Waals energy components
                  if (skip(k) .eq. -i) then
                    ge14 = ge14 + e

                  else
                    gev = gev + e

                  end if

                end if
              end if

   99         continue                      ! skip interaction
            end do                          ! kk
          end do                            ! ii

c         accumulate g1-g2 interaction
          e14 = e14 + ge14
          ev = ev + gev

  101     continue                          ! skip this group pair
        end do                              ! g2

  100   continue                            ! skip g1 entirely
      end do                                ! g1

      return
      end
