c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1999  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine lbfgs  --  limited memory BFGS optimization  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "lbfgs" is a limited memory BFGS quasi-newton nonlinear
c     optimization routine
c
c     literature references:
c
c     J. Nocedal, "Updating Quasi-Newton Matrices with Limited
c     Storage", Mathematics of Computation, 35, 773-782 (1980)
c
c     D. C. Lui and J. Nocedal, "On the Limited Memory BFGS Method
c     for Large Scale Optimization", Mathematical Programming,
c     45, 503-528 (1989)
c
c
      subroutine lbfgs (nvar,xx,minimum,grdmin,fgvalue,writeout)
      implicit none
      include 'sizes.i'
      include 'iounit.i'
      integer msave,nxm,nwork
      parameter (msave=15)
      parameter (nxm=maxvar*msave)
      parameter (nwork=maxvar+2*msave)
      integer nvar,m
      integer iter,icall
      integer iflag,iprint(2)
      real*8 minimum,grdmin
      real*8 f,fgvalue
      real*8 xx(maxvar),g(maxvar),diag(maxvar)
      real*8 point,s(nxm),yy(nxm),w(nwork)
      character*80 string
      logical exist,diagco
      external fgvalue,writeout
c
c
c     initialize some minimization parameters
c
      iprint(1) = 100
      iprint(2) = 0
      diagco = .false.
      icall = 0
      iflag = 0
c
c     get number of correction vectors to be used
c
      m = -1
      call nextarg (string,exist)
      if (exist)  read (string,*,err=10,end=10)  m
   10 continue
      if (m .le. 0) then
         write (iout,20)
   20    format (/,' Enter Number of Correction Vectors',
     &              ' [7] :  ',$)
         read (input,30)  m
   30    format (i10)
      end if
      if (m .le. 0)  m = 7
c
c     L-BFGS iteration starts here
c
      dowhile (.true.)
         f = fgvalue (xx,g)
         call va15ad (nvar,m,xx,f,g,diagco,diag,iprint,
     &                grdmin,s,yy,point,w,iflag,iter)
         if (iflag .le. 0)  goto 40
         icall = icall + 1
         if (icall .gt. 100000)  goto 40
      end do
   40 continue
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine vd15ad  ##
c     ##                     ##
c     #########################
c
c
      SUBROUTINE VA15AD(N,M,X,F,G,DIAGCO,DIAG,IPRINT,EPS,S,Y,
     &          POINT,W,IFLAG,ITER)
C
C
      REAL*8 X(N),G(N),S(M*N),Y(M*N),DIAG(N),W(N+2*M)
      REAL*8 FTOL,GTOL,XTOL,STPMIN,STPMAX,STP,F,YS,SQ,
     &              YR,BETA,ONE,ZERO,EPS,XNORM,GNORM,YY,DDOT,STP1
C
      INTEGER BOUND,LP,ITER,NFUN,NFEV,IPRINT(2),POINT,CP,IFLAG
      LOGICAL FINISH,DIAGCO
      COMMON /VA15DD/MP,LP, GTOL
      SAVE
      DATA ONE,ZERO/1.0D+0,0.0D+0/
C
C     ------------------------------------------------------------
C     INITIALIZE
C     ------------------------------------------------------------
C
      IF(IFLAG.EQ.0) GO TO 10
      GO TO (180,70) IFLAG
   10 ITER= 0
      IF(N.LE.0.OR.M.LE.0) GO TO 220
      IF(GTOL.LE.1.D-04) THEN
         IF(LP.GT.0) WRITE(LP,340)
         GTOL=1.D-02
      ENDIF
      NFUN= 1
      POINT= 0
      FINISH= .FALSE.
      IF(DIAGCO) THEN
         DO 20 I=1,N
   20    IF (DIAG(I).LE.ZERO) GO TO 210
      ELSE
         DO 30 I=1,N
   30    DIAG(I)= 1.0D0
      ENDIF
      DO 40 I=1,N
   40 S(I)= -G(I)*DIAG(I)
      GNORM= DSQRT(DDOT(N,G,1,G,1))
      STP1= ONE/GNORM
C
C     PARAMETERS FOR LINE SEARCH ROUTINE
C     ----------------------------------
      FTOL= 1.0D-4
      XTOL= 1.0D-17
      STPMIN= 1.0D-20
      STPMAX= 1.0D+20
      MAXFEV= 20
C
      WRITE (*,230)
      WRITE (*,250) ITER,NFUN,F,GNORM*(12.0d0/sqrt(dble(n/3)))
C
C    ------------------------------------------------------------
C     MAIN ITERATION LOOP
C    --------------------------------------------------------
C
   50 ITER= ITER+1
      INFO=0
      BOUND=ITER-1
      IF (ITER .GT. M)BOUND=M
      IF(ITER.EQ.1) GO TO 160
C
C     ------------------------------------------------------------
C     COMPUTE -HG AND THE DIAGONAL SCALING MATRIX IN DIAG
C     ------------------------------------------------------------
C
      IF(.NOT.DIAGCO) THEN
CZOU         PRINT*,'form scaling ITER=', ITER
         DO 60 I=1,N
   60    DIAG(I)= YS/YY
      ELSE
         IFLAG=2
         RETURN
      ENDIF
   70 CONTINUE
      DO 80 I=1,N
   80 IF (DIAG(I).LE.ZERO) GO TO 210
C
      CP= POINT
      IF (POINT.EQ.0) CP=M
      W(N+CP)= ONE/YS
      DO 90 I=1,N
   90 W(I)= -G(I)
      CP= POINT
      DO 110 II= 1,BOUND
         CP=CP-1
         IF (CP.EQ. -1)CP=M-1
         SQ= DDOT(N,S(CP*N+1),1,W,1)
         W(N+M+CP+1)= W(N+CP+1)*SQ
         DO 100 K=1,N
  100    W(K)= W(K)-W(N+M+CP+1)*Y(CP*N+K)
  110 CONTINUE
C
      DO 120 I=1,N
  120 W(I)=DIAG(I)*W(I)
      DO 140 II=1,BOUND
         YR= DDOT(N,Y(CP*N+1),1,W,1)
         BETA= W(N+CP+1)*YR
         DO 130 K=1,N
  130    W(K)= W(K)+S(CP*N+K)*(W(N+M+CP+1)-BETA)
         CP=CP+1
         IF (CP.EQ.M)CP=0
  140 CONTINUE
C
C     ------------------------------------------------------------
C     STORE THE NEW DIRECTION IN S
C     ------------------------------------------------------------
C
      DO 150 J=1,N
  150 S(POINT*N+J)= W(J)
C
C     ------------------------------------------------------------
C     OBTAIN THE MINIMIZER OF THE FUNCTION ALONG THE
C     DIRECTION S BY USING THE LINE SEARCH ROUTINE OF VD05AD
C     ------------------------------------------------------------
  160 NFEV=0
      STP=ONE
      IF (ITER.EQ.1) STP=STP1
      DO 170 I=1,N
  170 W(I)=G(I)
  180 CONTINUE
C
      CALL VD05AD(N,X,F,G,S(POINT*N+1),STP,FTOL,GTOL,
     &            XTOL,STPMIN,STPMAX,MAXFEV,INFO,NFEV,DIAG)
C
      IF (INFO .EQ. -1) THEN
         IFLAG=1
         RETURN
      ENDIF
      IF (INFO .NE. 1) GO TO 200
      NFUN= NFUN + NFEV
C
C     ------------------------------------------------------------
C     COMPUTE THE NEW S AND Y
C     ------------------------------------------------------------
C
      NPT=POINT*N
      DO 190 I=1,N
         S(NPT+I)= STP*S(NPT+I)
  190 Y(NPT+I)= G(I)-W(I)
      YS= DDOT(N,Y(NPT+1),1,S(NPT+1),1)
      YY= DDOT(N,Y(NPT+1),1,Y(NPT+1),1)
      POINT=POINT+1
      IF (POINT.EQ.M)POINT=0
C
C     ------------------------------------------------------------
C     CONVERGENCE CHECK
C     ------------------------------------------------------------
C
      GNORM= DDOT(N,G,1,G,1)
      GNORM=DSQRT(GNORM)
      XNORM= DDOT(N,X,1,X,1)
      XNORM=DSQRT(XNORM)
      XNORM= DMAX1(1.0D0,XNORM)
C
C     IF (GNORM/XNORM .LE. EPS) FINISH=.TRUE.
      IF (GNORM*(12.0d0/sqrt(dble(n/3))) .LE. EPS) FINISH=.TRUE.
C
      IF (MOD(ITER,IPRINT(1)).EQ.0 .OR. FINISH) THEN
         WRITE (*,240) ITER,NFUN,F,
     &                 GNORM*(12.0d0/sqrt(dble(n/3))),STP
      ENDIF
      IF (FINISH) THEN
         IFLAG=0
         RETURN
      ENDIF
      GO TO 50
C
C     ------------------------------------------------------------
C     END OF MAIN ITERATION LOOP. ERROR EXITS.
C     ------------------------------------------------------------
C
  200 IF(LP.LE.0) RETURN
      IF (INFO.EQ.0) THEN
         IFLAG= -1
         WRITE(LP,260)IFLAG
      ELSE IF (INFO.EQ.2) THEN
         IFLAG= -2
         WRITE(LP,270)IFLAG
      ELSE IF (INFO.EQ.3) THEN
         IFLAG= -3
         WRITE(LP,280)IFLAG
      ELSE IF (INFO.EQ.4) THEN
         IFLAG= -4
         WRITE(LP,290)IFLAG
      ELSE IF (INFO.EQ.5) THEN
         IFLAG= -5
         WRITE(LP,300)IFLAG
      ELSE IF (INFO.EQ.6) THEN
         IFLAG= -6
         WRITE(LP,310)IFLAG
      ENDIF
      RETURN
C
  210 IFLAG= -7
      IF(LP.GT.0) WRITE(LP,320)IFLAG,I
      RETURN
  220 IFLAG= -8
      IF(LP.GT.0) WRITE(LP,330)IFLAG
C
C     ------------------------------------------------------------
C     FORMATS
C     ------------------------------------------------------------
C
  230 FORMAT(/5x,'I',4x,'NFN',10X,'FUNC',12X,'GNORM',10X,'STEPLENGTH'/)
  240 FORMAT(2(I6,1X),3X,3(E15.8,2X))
  250 FORMAT(2(I6,1X),3X,2(E15.8,2X))
  260 FORMAT(/' IFLAG= ',I2,/' IMPROPER INPUT PARAMETERS DURING',
     &       ' THE LINE SEARCH.')
  270 FORMAT(/' IFLAG= ',I2,/' RELATIVE WIDTH OF THE INTERVAL OF',
     &       ' UNCERTAINTY IN THE LINE SEARCH'/'IS OF THE ORDER OF
     &          MACHINE ROUNDOFF.')
  280 FORMAT(/' IFLAG= ',I2,/' NUMBER OF CALLS TO FUNCTION IN THE',
     &       ' LINE SEARCH HAS REACHED 20.')
  290 FORMAT(/' IFLAG= ',I2,/' THE STEP IN THE LINE SEARCH IS',
     &       ' TOO SMALL.')
  300 FORMAT(/' IFLAG= ',I2,/' THE STEP IN THE LINE SEARCH IS',
     &       ' TOO LARGE.')
  310 FORMAT(/' IFLAG= ',I2,/' ROUNDING ERRORS PREVENT FURTHER',
     &       ' PROGRESS IN THE LINE SEARCH.')
  320 FORMAT(/' IFLAG= ',I2,/' THE',I5,'-TH DIAGONAL ELEMENT OF THE',
     &       ' INVERSE HESSIAN APPROXIMATION IS NOT POSITIVE')
  330 FORMAT(/' IFLAG= ',I2,/' IMPROPER INPUT PARAMETERS (N OR M',
     &       ' ARE NOT POSITIVE)')
  340 FORMAT(/'  GTOL IS LESS THAN OR EQUAL TO 1.D-04',
     &       / 'IT HAS BEEN RESET TO 1.D-02')
      RETURN
      END
c
c
c     #########################
c     ##                     ##
c     ##  data block vd15cd  ##
c     ##                     ##
c     #########################
c
c
      BLOCK DATA VA15CD
      COMMON /VA15DD/MP,LP, GTOL
      INTEGER LP
      REAL*8 GTOL
      DATA MP,LP,GTOL/6,6,9.0D-01/
      END
c
c
c     #########################
c     ##                     ##
c     ##  subroutine vd05ad  ##
c     ##                     ##
c     #########################
c
c
      SUBROUTINE VD05AD(N,X,F,G,S,STP,FTOL,GTOL,XTOL,
     &                  STPMIN,STPMAX,MAXFEV,INFO,NFEV,WA)
      INTEGER N,MAXFEV,INFO,NFEV
      REAL*8 F,STP,FTOL,GTOL,XTOL,STPMIN,STPMAX
      REAL*8 X(N),G(N),S(N),WA(N)
      SAVE
      INTEGER INFOC,J
      LOGICAL BRACKT,STAGE1
      REAL*8 DG,DGM,DGINIT,DGTEST,DGX,DGXM,DGY,DGYM,
     &       FINIT,FTEST1,FM,FX,FXM,FY,FYM,P5,P66,STX,STY,
     &       STMIN,STMAX,WIDTH,WIDTH1,XTRAPF,ZERO
      DATA P5,P66,XTRAPF,ZERO /0.5D0,0.66D0,4.0D0,0.0D0/
      IF(INFO.EQ.-1) GO TO 50
      INFOC = 1
C
C     CHECK THE INPUT PARAMETERS FOR ERRORS.
C
      IF (N .LE. 0 .OR. STP .LE. ZERO .OR. FTOL .LT. ZERO .OR.
     &    GTOL .LT. ZERO .OR. XTOL .LT. ZERO .OR. STPMIN .LT. ZERO
     &    .OR. STPMAX .LT. STPMIN .OR. MAXFEV .LE. 0) RETURN
C
C     COMPUTE THE INITIAL GRADIENT IN THE SEARCH DIRECTION
C     AND CHECK THAT S IS A DESCENT DIRECTION.
C
      DGINIT = ZERO
      DO 10 J = 1, N
         DGINIT = DGINIT + G(J)*S(J)
   10 CONTINUE
      IF (DGINIT .GE. ZERO) RETURN
C
C     INITIALIZE LOCAL VARIABLES.
C
      BRACKT = .FALSE.
      STAGE1 = .TRUE.
      NFEV = 0
      FINIT = F
      DGTEST = FTOL*DGINIT
      WIDTH = STPMAX - STPMIN
      WIDTH1 = WIDTH/P5
      DO 20 J = 1, N
         WA(J) = X(J)
   20 CONTINUE
C
C     THE VARIABLES STX, FX, DGX CONTAIN THE VALUES OF THE STEP,
C     FUNCTION, AND DIRECTIONAL DERIVATIVE AT THE BEST STEP.
C     THE VARIABLES STY, FY, DGY CONTAIN THE VALUE OF THE STEP,
C     FUNCTION, AND DERIVATIVE AT THE OTHER ENDPOINT OF
C     THE INTERVAL OF UNCERTAINTY.
C     THE VARIABLES STP, F, DG CONTAIN THE VALUES OF THE STEP,
C     FUNCTION, AND DERIVATIVE AT THE CURRENT STEP.
C
      STX = ZERO
      FX = FINIT
      DGX = DGINIT
      STY = ZERO
      FY = FINIT
      DGY = DGINIT
C
C     START OF ITERATION.
C
   30 CONTINUE
C
C        SET THE MINIMUM AND MAXIMUM STEPS TO CORRESPOND
C        TO THE PRESENT INTERVAL OF UNCERTAINTY.
C
      IF (BRACKT) THEN
         STMIN = MIN(STX,STY)
         STMAX = MAX(STX,STY)
      ELSE
         STMIN = STX
         STMAX = STP + XTRAPF*(STP - STX)
      END IF
C
C        FORCE THE STEP TO BE WITHIN THE BOUNDS STPMAX AND STPMIN.
C
      STP = MAX(STP,STPMIN)
      STP = MIN(STP,STPMAX)
C
C        IF AN UNUSUAL TERMINATION IS TO OCCUR THEN LET
C        STP BE THE LOWEST POINT OBTAINED SO FAR.
C
      IF ((BRACKT .AND. (STP .LE. STMIN .OR. STP .GE. STMAX))
     &      .OR. NFEV .GE. MAXFEV-1 .OR. INFOC .EQ. 0
     &      .OR. (BRACKT .AND. STMAX-STMIN .LE. XTOL*STMAX)) STP = STX
C
C        EVALUATE THE FUNCTION AND GRADIENT AT STP
C        AND COMPUTE THE DIRECTIONAL DERIVATIVE.
C
      DO 40 J = 1, N
         X(J) = WA(J) + STP*S(J)
   40 CONTINUE
      INFO=-1
      RETURN
C
   50 INFO=0
      NFEV = NFEV + 1
      DG = ZERO
      DO 60 J = 1, N
         DG = DG + G(J)*S(J)
   60 CONTINUE
      FTEST1 = FINIT + STP*DGTEST
C
C        TEST FOR CONVERGENCE.
C
      IF ((BRACKT .AND. (STP .LE. STMIN .OR. STP .GE. STMAX))
     &      .OR. INFOC .EQ. 0) INFO = 6
      IF (STP .EQ. STPMAX .AND.
     &       F .LE. FTEST1 .AND. DG .LE. DGTEST) INFO = 5
      IF (STP .EQ. STPMIN .AND.
     &       (F .GT. FTEST1 .OR. DG .GE. DGTEST)) INFO = 4
      IF (NFEV .GE. MAXFEV) INFO = 3
      IF (BRACKT .AND. STMAX-STMIN .LE. XTOL*STMAX) INFO = 2
      IF (F .LE. FTEST1 .AND. ABS(DG) .LE. GTOL*(-DGINIT)) INFO = 1
C
C        CHECK FOR TERMINATION.
C
      IF (INFO .NE. 0) RETURN
C
C        IN THE FIRST STAGE WE SEEK A STEP FOR WHICH THE MODIFIED
C        FUNCTION HAS A NONPOSITIVE VALUE AND NONNEGATIVE DERIVATIVE.
C
      IF (STAGE1 .AND. F .LE. FTEST1 .AND.
     &       DG .GE. MIN(FTOL,GTOL)*DGINIT) STAGE1 = .FALSE.
C
C        A MODIFIED FUNCTION IS USED TO PREDICT THE STEP ONLY IF
C        WE HAVE NOT OBTAINED A STEP FOR WHICH THE MODIFIED
C        FUNCTION HAS A NONPOSITIVE FUNCTION VALUE AND NONNEGATIVE
C        DERIVATIVE, AND IF A LOWER FUNCTION VALUE HAS BEEN
C        OBTAINED BUT THE DECREASE IS NOT SUFFICIENT.
C
      IF (STAGE1 .AND. F .LE. FX .AND. F .GT. FTEST1) THEN
C
C           DEFINE THE MODIFIED FUNCTION AND DERIVATIVE VALUES.
C
         FM = F - STP*DGTEST
         FXM = FX - STX*DGTEST
         FYM = FY - STY*DGTEST
         DGM = DG - DGTEST
         DGXM = DGX - DGTEST
         DGYM = DGY - DGTEST
C
C           CALL CSTEP TO UPDATE THE INTERVAL OF UNCERTAINTY
C           AND TO COMPUTE THE NEW STEP.
C
         CALL VD05BD(STX,FXM,DGXM,STY,FYM,DGYM,STP,FM,DGM,
     &                 BRACKT,STMIN,STMAX,INFOC)
C
C           RESET THE FUNCTION AND GRADIENT VALUES FOR F.
C
         FX = FXM + STX*DGTEST
         FY = FYM + STY*DGTEST
         DGX = DGXM + DGTEST
         DGY = DGYM + DGTEST
      ELSE
C
C           CALL VD05BD TO UPDATE THE INTERVAL OF UNCERTAINTY
C           AND TO COMPUTE THE NEW STEP.
C
         CALL VD05BD(STX,FX,DGX,STY,FY,DGY,STP,F,DG,
     &                 BRACKT,STMIN,STMAX,INFOC)
      END IF
C
C        FORCE A SUFFICIENT DECREASE IN THE SIZE OF THE
C        INTERVAL OF UNCERTAINTY.
C
      IF (BRACKT) THEN
         IF (ABS(STY-STX) .GE. P66*WIDTH1)
     &         STP = STX + P5*(STY - STX)
         WIDTH1 = WIDTH
         WIDTH = ABS(STY-STX)
      END IF
C
C        END OF ITERATION.
C
      GO TO 30
      END
c
c
c     #########################
c     ##                     ##
c     ##  subroutine vd05bd  ##
c     ##                     ##
c     #########################
c
c
      SUBROUTINE VD05BD(STX,FX,DX,STY,FY,DY,STP,FP,DP,BRACKT,
     &                 STPMIN,STPMAX,INFO)
      INTEGER INFO
      REAL*8 STX,FX,DX,STY,FY,DY,STP,FP,DP,STPMIN,STPMAX
      LOGICAL BRACKT,BOUND
      REAL*8 GAMMA,P,Q,R,S,SGND,STPC,STPF,STPQ,THETA
      INFO = 0
C
C     CHECK THE INPUT PARAMETERS FOR ERRORS.
C
      IF ((BRACKT .AND. (STP .LE. MIN(STX,STY) .OR.
     &    STP .GE. MAX(STX,STY))) .OR.
     &    DX*(STP-STX) .GE. 0.0 .OR. STPMAX .LT. STPMIN) RETURN
C
C     DETERMINE IF THE DERIVATIVES HAVE OPPOSITE SIGN.
C
      SGND = DP*(DX/ABS(DX))
C
C     FIRST CASE. A HIGHER FUNCTION VALUE.
C     THE MINIMUM IS BRACKETED. IF THE CUBIC STEP IS CLOSER
C     TO STX THAN THE QUADRATIC STEP, THE CUBIC STEP IS TAKEN,
C     ELSE THE AVERAGE OF THE CUBIC AND QUADRATIC STEPS IS TAKEN.
C
      IF (FP .GT. FX) THEN
         INFO = 1
         BOUND = .TRUE.
         THETA = 3*(FX - FP)/(STP - STX) + DX + DP
         S = MAX(ABS(THETA),ABS(DX),ABS(DP))
         GAMMA = S*SQRT((THETA/S)**2 - (DX/S)*(DP/S))
         IF (STP .LT. STX) GAMMA = -GAMMA
         P = (GAMMA - DX) + THETA
         Q = ((GAMMA - DX) + GAMMA) + DP
         R = P/Q
         STPC = STX + R*(STP - STX)
         STPQ = STX + ((DX/((FX-FP)/(STP-STX)+DX))/2)*(STP - STX)
         IF (ABS(STPC-STX) .LT. ABS(STPQ-STX)) THEN
            STPF = STPC
         ELSE
            STPF = STPC + (STPQ - STPC)/2
         END IF
         BRACKT = .TRUE.
C
C     SECOND CASE. A LOWER FUNCTION VALUE AND DERIVATIVES OF
C     OPPOSITE SIGN. THE MINIMUM IS BRACKETED. IF THE CUBIC
C     STEP IS CLOSER TO STX THAN THE QUADRATIC (SECANT) STEP,
C     THE CUBIC STEP IS TAKEN, ELSE THE QUADRATIC STEP IS TAKEN.
C
      ELSE IF (SGND .LT. 0.0) THEN
         INFO = 2
         BOUND = .FALSE.
         THETA = 3*(FX - FP)/(STP - STX) + DX + DP
         S = MAX(ABS(THETA),ABS(DX),ABS(DP))
         GAMMA = S*SQRT((THETA/S)**2 - (DX/S)*(DP/S))
         IF (STP .GT. STX) GAMMA = -GAMMA
         P = (GAMMA - DP) + THETA
         Q = ((GAMMA - DP) + GAMMA) + DX
         R = P/Q
         STPC = STP + R*(STX - STP)
         STPQ = STP + (DP/(DP-DX))*(STX - STP)
         IF (ABS(STPC-STP) .GT. ABS(STPQ-STP)) THEN
            STPF = STPC
         ELSE
            STPF = STPQ
         END IF
         BRACKT = .TRUE.
C
C     THIRD CASE. A LOWER FUNCTION VALUE, DERIVATIVES OF THE
C     SAME SIGN, AND THE MAGNITUDE OF THE DERIVATIVE DECREASES.
C     THE CUBIC STEP IS ONLY USED IF THE CUBIC TENDS TO INFINITY
C     IN THE DIRECTION OF THE STEP OR IF THE MINIMUM OF THE CUBIC
C     IS BEYOND STP. OTHERWISE THE CUBIC STEP IS DEFINED TO BE
C     EITHER STPMIN OR STPMAX. THE QUADRATIC (SECANT) STEP IS ALSO
C     COMPUTED AND IF THE MINIMUM IS BRACKETED THEN THE THE STEP
C     CLOSEST TO STX IS TAKEN, ELSE THE STEP FARTHEST AWAY IS TAKEN.
C
      ELSE IF (ABS(DP) .LT. ABS(DX)) THEN
         INFO = 3
         BOUND = .TRUE.
         THETA = 3*(FX - FP)/(STP - STX) + DX + DP
         S = MAX(ABS(THETA),ABS(DX),ABS(DP))
C
C        THE CASE GAMMA = 0 ONLY ARISES IF THE CUBIC DOES NOT TEND
C        TO INFINITY IN THE DIRECTION OF THE STEP.
C
         GAMMA = S*SQRT(MAX(0.0D0,(THETA/S)**2 - (DX/S)*(DP/S)))
         IF (STP .GT. STX) GAMMA = -GAMMA
         P = (GAMMA - DP) + THETA
         Q = (GAMMA + (DX - DP)) + GAMMA
         R = P/Q
         IF (R .LT. 0.0 .AND. GAMMA .NE. 0.0) THEN
            STPC = STP + R*(STX - STP)
         ELSE IF (STP .GT. STX) THEN
            STPC = STPMAX
         ELSE
            STPC = STPMIN
         END IF
         STPQ = STP + (DP/(DP-DX))*(STX - STP)
         IF (BRACKT) THEN
            IF (ABS(STP-STPC) .LT. ABS(STP-STPQ)) THEN
               STPF = STPC
            ELSE
               STPF = STPQ
            END IF
         ELSE
            IF (ABS(STP-STPC) .GT. ABS(STP-STPQ)) THEN
               STPF = STPC
            ELSE
               STPF = STPQ
            END IF
         END IF
C
C     FOURTH CASE. A LOWER FUNCTION VALUE, DERIVATIVES OF THE
C     SAME SIGN, AND THE MAGNITUDE OF THE DERIVATIVE DOES
C     NOT DECREASE. IF THE MINIMUM IS NOT BRACKETED, THE STEP
C     IS EITHER STPMIN OR STPMAX, ELSE THE CUBIC STEP IS TAKEN.
C
      ELSE
         INFO = 4
         BOUND = .FALSE.
         IF (BRACKT) THEN
            THETA = 3*(FP - FY)/(STY - STP) + DY + DP
            S = MAX(ABS(THETA),ABS(DY),ABS(DP))
            GAMMA = S*SQRT((THETA/S)**2 - (DY/S)*(DP/S))
            IF (STP .GT. STY) GAMMA = -GAMMA
            P = (GAMMA - DP) + THETA
            Q = ((GAMMA - DP) + GAMMA) + DY
            R = P/Q
            STPC = STP + R*(STY - STP)
            STPF = STPC
         ELSE IF (STP .GT. STX) THEN
            STPF = STPMAX
         ELSE
            STPF = STPMIN
         END IF
      END IF
C
C     UPDATE THE INTERVAL OF UNCERTAINTY. THIS UPDATE DOES NOT
C     DEPEND ON THE NEW STEP OR THE CASE ANALYSIS ABOVE.
C
      IF (FP .GT. FX) THEN
         STY = STP
         FY = FP
         DY = DP
      ELSE
         IF (SGND .LT. 0.0) THEN
            STY = STX
            FY = FX
            DY = DX
         END IF
         STX = STP
         FX = FP
         DX = DP
      END IF
C
C     COMPUTE THE NEW STEP AND SAFEGUARD IT.
C
      STPF = MIN(STPMAX,STPF)
      STPF = MAX(STPMIN,STPF)
      STP = STPF
      IF (BRACKT .AND. BOUND) THEN
         IF (STY .GT. STX) THEN
            STP = MIN(STX+0.66*(STY-STX),STP)
         ELSE
            STP = MAX(STX+0.66*(STY-STX),STP)
         END IF
      END IF
      RETURN
      END
c
c
      function ddot (n,d,i1,s,i2)
      implicit none
      integer i,i1,i2,n
      real*8 ddot,sum
      real*8 d(n),s(n)
c
c
      sum = 0.0D0
      do i = 1, n
         sum = sum + d(i)*s(i)
      end do
      ddot = sum
      return
      end
