c
c
c     ###############################################################
c     ##                                                           ##
c     ##  mpole.i  --  multipole components for current structure  ##
c     ##                                                           ##
c     ###############################################################
c
c
      integer maxrank,max,mcd2
      parameter (maxrank=2)
      parameter (max=maxrank*maxrank)
      parameter (mcd2=(3**(maxrank+1)-1)/2)
      integer nmdq,mdqsiz,c,mindex
      integer zt,zb,z2t,z2b,xt,xb
      real*8 multip,rmultip
      logical noshort
      common /mpole/ multip(mcd2,maxatm),rmultip(mcd2,maxatm),
     &               nmdq,mdqsiz,c(1:2*max+1,0:max,0:max),
     &               mindex(4,mcd2,mcd2),zt(maxatm),zb(maxatm),
     &               z2t(maxatm),z2b(maxatm),xt(maxatm),xb(maxatm),
     &               noshort
c
c
c     *************************************************************
c     **                                                         **
c     **  subroutine kmpole  --  multipole parameter assignment  **
c     **                                                         **
c     *************************************************************
c
c
      subroutine kmpole
      implicit none
      include 'sizes.i'
      include 'iounit.i'
      include 'mpole.i'
      character*1 answer
c
c
c     get the order of the multipole expansion to be used
c
      write (iout,10)
   10 format (/,' Enter Highest Order Multipole (M, D or [Q]) :  ',$)
      read (input,20)  answer
   20 format (a1)
      call upcase (answer)
      if (answer .eq. 'M') then
         mdqsiz = 1
      else if (answer .eq. 'D') then
         mdqsiz = 4
      else
         mdqsiz = 13
      end if
c
c     decide whether to include 1-2 and 1-3 interactions
c
      write (iout,30)
   30 format (/,' Exclude 1-2 and 1-3 Interactions ([Y] or N) :  ',$)
      read (input,40)  answer
   40 format (a1)
      call upcase (answer)
      if (answer .eq. 'N') then
         noshort = .false.
      else
         noshort = .true.
      end if
c
c     set coefficients and indices for Dijkstra's multipole method
c
      call coeffi (9,4)
      call maxind (2)
c
c     get the local axis definitions and multipole moments
c
      call getpole
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine coeffi  ##
c     ##                     ##
c     #########################
c
c
      subroutine coeffi (nn,kk)
      implicit none
      include 'sizes.i'
      include 'mpole.i'
      integer i,j,nn,kk,nnn
c
c
      do nnn = 1, nn, 2
         c(nnn,0,0) = 1
         do j = 1, kk
            c(nnn,j,j) = c(nnn,j-1,j-1) * (-nnn-(j-1)*2)
         end do
      end do
      do nnn = 1, nn, 2
         do i = 2, kk, 2
            c(nnn,i,0) = c(nnn,i-1,1)
            do j = 1, kk-i
               c(nnn,j+i,j) = c(nnn,j+i-1,j-1)*(-nnn-i-(j-1)*2)
     &                            + c(nnn,j+i-1,j+1)*(j+1)
            end do
         end do
      end do
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine maxind  ##
c     ##                     ##
c     #########################
c
c
      subroutine maxind (order)
      implicit none
      include 'sizes.i'
      include 'mpole.i'
      integer i,j,order,mlen
      integer i1,j1,k1,i2,j2,k2
      integer index(13)
      data index / 000, 001, 010, 100, 002, 011, 101,
     &             011, 020, 110, 101, 110, 200 /
c
c
      mlen = (3**(order+1)-1) / 2
      do i = 1, mlen
         i1 = index(i) / 100
         j1 = (index(i) - 100*i1) / 10
         k1 = index(i) - 100*i1 - 10*j1
         do j = 1, mlen
            i2 = index(j) / 100
            j2 = (index(j) - 100*i2) / 10
            k2 = index(j) - 100*i2 - 10*j2
            mindex(1,i,j) = i1 + i2
            mindex(2,i,j) = j1 + j2
            mindex(3,i,j) = k1 + k2
            mindex(4,i,j) = i2 + j2 + k2
         end do
      end do
      return
      end
c
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine getpole  --  get atomic multipole parameters  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     getpole reads in the local coordinate axis definitions
c     and atomic multipole moments over spherical harmonics
c
c
      subroutine getpole
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'files.i'
      include 'mpole.i'
      include 'units.i'
      integer i,k,iaxes,idma,freeunit
      integer zzt,zzb,zz2t,zz2b,xxt,xxb
      integer z0,z1,x0,x1
      real*8 q20,q21c,q21s,q22c,q22s
      character*60 axesfile,dmafile
      character*80 record
c
c
c     open the file containing the local axes definitions
c
      axesfile = filename(1:leng)
      call suffix (axesfile,'axe')
      call version (axesfile,'old')
      iaxes = freeunit ()
      open (unit=iaxes,file=axesfile,status='old')
      rewind (unit=iaxes)
c
c     read in the local axes definitions for each atom
c
      do i = 1, n
         read (iaxes,10)  record
   10    format (a80)
         read (record,*)  k,zzt,zzb,zz2t,zz2b,xxt,xxb
         zt(k) = zzt
         zb(k) = zzb
         z2t(k) = zz2t
         z2b(k) = zz2b
         xt(k) = xxt
         xb(k) = xxb
      end do
      close (iaxes)
c
c     open the file containing the multipole moments
c
      dmafile = filename(1:leng)
      call suffix (dmafile,'dma')
      call version (dmafile,'old')
      idma = freeunit ()
      open (unit=idma,file=dmafile,status='old')
      rewind (unit=idma)
c
c     read in the distributed multipole moments
c
      do i = 1, 26
         read (idma,20)
   20    format ()
      end do
      do i = 1, n
         read (idma,20)
         read (idma,30)  record
   30    format (a80)
         read (record,*)  multip(1,i)
         read (idma,40)  record
   40    format (a80)
         read (record,*)  multip(4,i),multip(2,i),multip(3,i)
         read (idma,50)  record
   50    format (a80)
         read (record,*)  q20,q21c,q21s
         read (idma,60)  record
   60    format (a80)
         read (record,*)  q22c,q22s
         multip(13,i) = q20 / 3.0d0
         multip(7,i) = q21c * (0.866025404d0/3.0d0)
         multip(10,i) = q21s * (0.866025404d0/3.0d0)
         multip(5,i) = (q22c*0.866025404d0 - q20*0.5d0) / 3.0d0
         multip(6,i) = q22s * (0.866025404d0/3.0d0)
         multip(8,i) = multip(6,i)
         multip(9,i) = -(multip(5,i) + multip(13,i))
         multip(11,i) = multip(7,i)
         multip(12,i) = multip(10,i)
         do k = 1, 25
            read (idma,70)
   70       format ()
         end do
      end do
c
c     convert the dipole and quadrupole moments to Angstroms
c
      do i = 1, n
         do k = 2, 4
            multip(k,i) = multip(k,i) * bohr
         end do
         do k = 5, 13
            multip(k,i) = multip(k,i) * bohr**2
         end do
      end do
      close (idma)
      return
      end
c
c
c     #########################################################
c     ##                                                     ##
c     ##  subroutine empole  --  multipole potential energy  ##
c     ##                                                     ##
c     #########################################################
c
c
      subroutine empole
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'analyz.i'
      include 'atoms.i'
      include 'couple.i'
      include 'energi.i'
      include 'mpole.i'
      integer i,j,k,skip(maxatm)
      real*8 eij,emdqij,a(3,3)
      real*8 rm1(13),rm2(13)
      logical iuse
c
c
c     zero out the multipole interaction energy and partitioning
c
      emdq = 0.0d0
      do i = 1, n
         aemdq(i) = 0.0d0
      end do
      nmdq = 0
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     rotate the multipole components into the global frame
c
      do i = 1, n
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c     compute and partition multipole interaction energy
c
      do i = 1, n-1
         iuse = use(i)
         if (noshort) then
            do k = 1, n12(i)
               skip(i12(k,i)) = i
            end do
            do k = 1, n13(i)
               skip(i13(k,i)) = i
            end do
         end if
         do k = 1, mdqsiz
            rm1(k) = rmultip(k,i)
         end do
         do j = i+1, n
            if (iuse .or. use(j)) then
               if (skip(j) .ne. i) then
                  do k = 1, mdqsiz
                     rm2(k) = rmultip(k,j)
                  end do
                  nmdq = nmdq + 1
                  eij = emdqij(i,j,rm1,rm2)
                  emdq = emdq + eij
                  aemdq(i) = aemdq(i) + 0.5d0*eij
                  aemdq(j) = aemdq(j) + 0.5d0*eij
               end if
            end if
         end do
      end do
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine rotmat  ##
c     ##                     ##
c     #########################
c
c
      subroutine rotmat (i,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i
      real*8 dx,dy,dz
      real*8 r,dotxz,a(3,3)
      real*8 zi,zj,zk
      real*8 z2i,z2j,z2k
      real*8 xi,xj,xk
c
c
c     find rotation matrix that converts local to global coordinates
c
      dx = x(zt(i)) - x(zb(i))
      dy = y(zt(i)) - y(zb(i))
      dz = z(zt(i)) - z(zb(i))
      r = sqrt(dx*dx + dy*dy + dz*dz)
      zi = dx / r
      zj = dy / r
      zk = dz / r
c
c
      dx = x(z2t(i)) - x(z2b(i))
      dy = y(z2t(i)) - y(z2b(i))
      dz = z(z2t(i)) - z(z2b(i))
      r = sqrt(dx*dx + dy*dy + dz*dz)
      z2i = dx / r
      z2j = dy / r
      z2k = dz / r
c
c
      dx = zi + z2i
      dy = zj + z2j
      dz = zk + z2k
      r = sqrt(dx*dx + dy*dy + dz*dz)
      a(1,3) = dx / r
      a(2,3) = dy / r
      a(3,3) = dz / r
c
c
      dx = x(xt(i)) - x(xb(i))
      dy = y(xt(i)) - y(xb(i))
      dz = z(xt(i)) - z(xb(i))
      dotxz = dx*a(1,3) + dy*a(2,3) + dz*a(3,3)
      xi = dx - dotxz*a(1,3)
      xj = dy - dotxz*a(2,3)
      xk = dz - dotxz*a(3,3)
      r = sqrt(xi*xi + xj*xj + xk*xk)
      a(1,1) = xi / r
      a(2,1) = xj / r
      a(3,1) = xk / r
c
c
      a(1,2) = a(3,1)*a(2,3) - a(2,1)*a(3,3)
      a(2,2) = a(1,1)*a(3,3) - a(3,1)*a(1,3)
      a(3,2) = a(2,1)*a(1,3) - a(1,1)*a(2,3)
      return
      end
c
c
c     ##########################
c     ##                      ##
c     ##  subroutine rotpole  ##
c     ##                      ##
c     ##########################
c
c
      subroutine rotpole (iatom,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,m,iatom
      real*8 m2(3,3),r2(3,3),a(3,3)
c
c
c     monopoles have the same value in any coordinate frame
c
      rmultip(1,iatom) = multip(1,iatom)
c
c     rotate the dipoles to the global coordinate frame
c
      do i = 2, 4
         rmultip(i,iatom) = 0.0d0
         do j = 2, 4
            rmultip(i,iatom) = rmultip(i,iatom) +
     &                            multip(j,iatom)*a(i-1,j-1)
         end do
      end do
c
c     rotate the quadrupoles to the global coordinate frame
c
      k = 5
      do i = 1, 3
         do j = 1, 3
            m2(i,j) = multip(k,iatom)
            r2(i,j) = 0.0d0
            k = k + 1
         end do
      end do
      do i = 1, 3
         do j = 1, 3
            if (j .lt. i) then
               r2(i,j) = r2(j,i)
            else
               do k = 1, 3
                  do m = 1, 3
                     r2(i,j) = r2(i,j) + a(i,k)*a(j,m)*m2(k,m)
                  end do
               end do
            end if
         end do
      end do
      k = 5
      do i = 1, 3
         do j = 1, 3
            rmultip(k,iatom) = r2(i,j)
            k = k + 1
         end do
      end do
      return
      end
c
c
c     #######################
c     ##                   ##
c     ##  function emdqij  ##
c     ##                   ##
c     #######################
c
c
      function emdqij (i1,i2,rm1,rm2)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      include 'units.i'
      integer i,j,k,i1,i2
      real*8 emdqij,t2matrix
      real*8 dx,dy,dz
      real*8 rm1(13),rm2(13)
      real*8 inter(13),t2(13,13)
c
c
c     construct the Applequist-Dijkstra T2 matrix
c
      dx = x(i2) - x(i1)
      dy = y(i2) - y(i1)
      dz = z(i2) - z(i1)
c
c     explicit calculation of each T2 matrix element (..slow..)
c
c     do i = 1, mdqsiz
c        do j = 1, mdqsiz
c           t2(j,i) = t2matrix (mindex(1,j,i),mindex(2,j,i),
c    &                            mindex(3,j,i),dx,dy,dz)
c           if (mindex(4,j,i) .eq. 1)  t2(j,i) = -t2(j,i)
c        end do
c     end do
c
c     zeroth order T2 matrix elements
c
      if (mdqsiz .ge. 1) then
         t2(1,1) = t2matrix (0,0,0,dx,dy,dz)
      end if
c
c     first order T2 matrix elements
c
      if (mdqsiz .ge. 4) then
         do i = 2, 4
            t2(i,1) = t2matrix (mindex(1,1,i),mindex(2,1,i),
     &                            mindex(3,1,i),dx,dy,dz)
            t2(1,i) = -t2(i,1)
         end do
c
c     second order T2 matrix elements
c
         do i = 2, 3
            do j = i, 4
               t2(j,i) = -t2matrix (mindex(1,j,i),mindex(2,j,i),
     &                                mindex(3,j,i),dx,dy,dz)
            end do
         end do

         t2(4,4) = -t2(2,2) - t2(3,3)
         t2(2,3) = t2(3,2)
         t2(2,4) = t2(4,2)
         t2(3,4) = t2(4,3)

         k = 5
         do i = 2, 4
            do j = 2, 4
               t2(1,k) = -t2(i,j)
               t2(k,1) = t2(1,k)
               k = k + 1
            end do
         end do
      end if
c
c     third order T2 matrix elements
c
      if (mdqsiz .ge. 13) then
         do i = 5, 6
            do j = i-3, 4
               t2(j,i) = t2matrix (mindex(1,j,i),mindex(2,j,i),
     &                               mindex(3,j,i),dx,dy,dz)
            end do
         end do

         t2(4,7) = -t2(2,5) - t2(3,6)
         t2(2,6) = t2(3,5)
         t2(2,7) = t2(4,5)
         t2(3,7) = t2(4,6)

         t2(2,8) = t2(2,6)
         t2(2,9) = t2(3,6)
         t2(2,10) = t2(4,6)
         t2(3,9) = t2matrix (0,3,0,dx,dy,dz)
         t2(3,10) = t2matrix (1,2,0,dx,dy,dz)
         t2(4,10) = -t2(2,8) - t2(3,9)
         t2(3,8) = t2(2,9)
         t2(4,8) = t2(2,10)
         t2(4,9) = t2(3,10)

         t2(2,11) = t2(2,7)
         t2(2,12) = t2(3,7)
         t2(2,13) = t2(4,7)
         t2(3,12) = t2(3,10)
         t2(3,13) = t2(4,10)
         t2(4,13) = -t2(2,11) - t2(3,12)
         t2(3,11) = t2(2,12)
         t2(4,11) = t2(2,13)
         t2(4,12) = t2(3,13)

         do i = 5, 13
            do j = 2, 4
               t2(i,j) = -t2(j,i)
            end do
         end do
c
c     fourth order T2 matrix elements
c
         do i = 5, 6
            do j = i, 7
               t2(j,i) = t2matrix (mindex(1,j,i),mindex(2,j,i),
     &                               mindex(3,j,i),dx,dy,dz)
            end do
         end do

         t2(7,7) = -t2(5,5) - t2(6,6)

         t2(8,5) = t2(6,5)
         t2(9,5) = t2(6,6)
         t2(10,5) = t2(7,6)
         t2(9,6) = t2matrix (0,3,1,dx,dy,dz)
         t2(10,6) = t2matrix (1,2,1,dx,dy,dz)
         t2(10,7) = -t2(8,5) - t2(9,6)
         t2(8,6) = t2(9,5)
         t2(8,7) = t2(10,5)
         t2(9,7) = t2(10,6)

         t2(11,5) = t2(7,5)
         t2(12,5) = t2(7,6)
         t2(13,5) = t2(7,7)
         t2(12,6) = t2(10,6)
         t2(13,6) = t2(10,7)
         t2(13,7) = -t2(11,5) - t2(12,6)
         t2(11,6) = t2(12,5)
         t2(11,7) = t2(13,5)
         t2(12,7) = t2(13,6)

         t2(8,8) = t2(8,6)
         t2(9,8) = t2(9,6)
         t2(10,8) = t2(10,6)
         t2(9,9) = t2matrix (0,4,0,dx,dy,dz)
         t2(10,9) = t2matrix (1,3,0,dx,dy,dz)
         t2(10,10) = -t2(8,8) - t2(9,9)

         t2(11,8) = t2(11,6)
         t2(12,8) = t2(12,6)
         t2(13,8) = t2(13,6)
         t2(12,9) = t2(10,9)
         t2(13,9) = t2(10,10)
         t2(13,10) = -t2(11,8) - t2(12,9)
         t2(11,9) = t2(12,8)
         t2(11,10) = t2(13,8)
         t2(12,10) = t2(13,9)

         t2(11,11) = t2(11,7)
         t2(12,11) = t2(12,7)
         t2(13,11) = t2(13,7)
         t2(12,12) = t2(12,10)
         t2(13,12) = t2(13,10)
         t2(13,13) = -t2(11,11) - t2(12,12)

         do i = 5, 12
            do j = i+1, 13
               t2(i,j) = t2(j,i)
            end do
         end do
      end if
c
c     compute interaction between the two multipole sites
c
      do i = 1, mdqsiz
         inter(i) = 0.0d0
         do j = 1, mdqsiz
            inter(i) = inter(i) + rm2(j)*t2(j,i)
         end do
      end do
      emdqij = 0.0d0
      do i = 1, mdqsiz
         emdqij = emdqij + inter(i)*rm1(i)
      end do
      emdqij = 332.05382d0 * emdqij
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  function t2matrix  ##
c     ##                     ##
c     #########################
c
c
      function t2matrix (di,dj,dk,dx,dy,dz)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,di,dj,dk
      real*8 t2matrix,sumy,sumz
      real*8 dx,dy,dz
      real*8 r,xr,yr,zr
      real*8 xrk,yrj,zri
c
c
c     get a single element of the Applequist-Dijkstra T2 matrix
c
      t2matrix = 0.0d0
      r = sqrt(dx*dx + dy*dy + dz*dz)
      xr = dx / r
      yr = dy / r
      zr = dz / r
      xrk = 1.0d0
      do k = 0, dk
         sumy = 0.0d0
         yrj = 1.0d0
         do j = 0, dj
            sumz = 0.0d0
            zri = 1.0d0
            do i = 0, di
               sumz = sumz + c(k+dk+j+dj+1,di,i) * zri
               zri = zri * zr
            end do
            sumy = sumy + c(k+dk+1,dj,j) * yrj * sumz
            yrj = yrj * yr
         end do
         t2matrix = t2matrix + c(1,dk,k) * xrk * sumy
         xrk = xrk * xr
      end do
      t2matrix = t2matrix / (r**(di+dj+dk+1))
      return
      end
c
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine empole1  --  multipole energy & derivatives  ##
c     ##                                                          ##
c     ##############################################################
c
c
      subroutine empole1
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'deriv.i'
      integer i
      real*8 eps,e,e0,old
c
c
c     zero out the multipole interaction first derivatives
c
      do i = 1, n
         demdq(1,i) = 0.0d0
         demdq(2,i) = 0.0d0
         demdq(3,i) = 0.0d0
      end do
c
c     find the value of the full multipole interaction energy
c
      call empole
c
c     set the value of the numerical gradient stepsize
c
      eps = 0.0001d0
c
c     compute one-sided numerical gradient from energy values
c
      do i = 1, n
         call dempole (i,e)
         e0 = e
         old = x(i)
         x(i) = x(i) + eps
         call dempole (i,e)
         x(i) = old
         demdq(1,i) = (e - e0) / eps
         old = y(i)
         y(i) = y(i) + eps
         call dempole (i,e)
         y(i) = old
         demdq(2,i) = (e - e0) / eps
         old = z(i)
         z(i) = z(i) + eps
         call dempole (i,e)
         z(i) = old
         demdq(3,i) = (e - e0) / eps
      end do
      return
      end
c
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine dempole  --  multipole energy for derivatives  ##
c     ##                                                            ##
c     ################################################################
c
c
      subroutine dempole (iatom,e)
      implicit none
      include 'sizes.i'
      include 'activ.i'
      include 'atoms.i'
      include 'couple.i'
      include 'mpole.i'
      integer i,j,k,iatom,skip(maxatm)
      real*8 e,eij,emdqij,a(3,3)
      real*8 rm1(13),rm2(13)
      logical iuse,ipart
c
c
c     zero out the multipole interaction energy and partitioning
c
      e = 0.0d0
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     rotate the multipole components into the global frame
c
      do i = 1, n
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c     compute and partition multipole interaction energy
c
      do i = 1, n-1
         iuse = use(i)
         if (i.eq.iatom .or. zt(i).eq.iatom .or. zb(i).eq.iatom
     &        .or. z2t(i).eq.iatom .or. z2b(i).eq.iatom
     &        .or. xt(i).eq.iatom .or. xb(i).eq.iatom) then
            ipart = .true.
         else
            ipart = .false.
         end if
         if (noshort) then
            do k = 1, n12(i)
               skip(i12(k,i)) = i
            end do
            do k = 1, n13(i)
               skip(i13(k,i)) = i
            end do
         end if
         do k = 1, mdqsiz
            rm1(k) = rmultip(k,i)
         end do
         do j = i+1, n
            if (iuse .or. use(j)) then
               if (skip(j) .ne. i) then
                  if (ipart) then
                     do k = 1, mdqsiz
                        rm2(k) = rmultip(k,j)
                     end do
                     eij = emdqij(i,j,rm1,rm2)
                     e = e + eij
                  else
                     if (j.eq.iatom .or. zt(j).eq.iatom
     &                    .or. zb(j).eq.iatom .or. z2t(j).eq.iatom
     &                    .or. z2b(j).eq.iatom .or. xt(j).eq.iatom
     &                    .or. xb(j).eq.iatom) then
                        do k = 1, mdqsiz
                           rm2(k) = rmultip(k,j)
                        end do
                        eij = emdqij(i,j,rm1,rm2)
                        e = e + eij
                     end if
                  end if
               end if
            end if
         end do
      end do
      return
      end
