c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1992  by  Jay William Ponder  ##
c     ###################################################
c
c     ##################################################################
c     ##                                                              ##
c     ##  subroutine smooth  --  get switching function coefficients  ##
c     ##                                                              ##
c     ##################################################################
c
c
c     "smooth" sets the coeffcients used by the fifth and seventh
c     order polynomial switching functions for spherical cutoffs
c
c
      subroutine smooth (type)
      implicit none
      include 'cutoff.i'
      include 'shunt.i'
      real*8 denom,term
      real*8 off3,off4,off5,off6,off7
      real*8 cut3,cut4,cut5,cut6,cut7
      real*8 pos,pos2,pos3,pos4
      character*6 type
c
c
c     set switching coefficients to zero for truncation cutoffs
c
      c0 = 0.0d0
      c1 = 0.0d0
      c2 = 0.0d0
      c3 = 0.0d0
      c4 = 0.0d0
      c5 = 0.0d0
      f0 = 0.0d0
      f1 = 0.0d0
      f2 = 0.0d0
      f3 = 0.0d0
      f4 = 0.0d0
      f5 = 0.0d0
      f6 = 0.0d0
      f7 = 0.0d0
c
c     get the switching window for the current potential type
c
      if (type(1:3) .eq. 'VDW') then
         off = vdwcut
         cut = vdwtaper
      else if (type .eq. 'CHARGE') then
         off = chgcut
         cut = chgtaper
      else if (type .eq. 'CHGDPL') then
         off = sqrt(chgcut*dplcut)
         cut = sqrt(chgtaper*dpltaper)
      else if (type .eq. 'DIPOLE') then
         off = dplcut
         cut = dpltaper
      end if
c
c     store the powers of the switching window cutoffs
c
      off2 = off * off
      off3 = off2 * off
      off4 = off2 * off2
      off5 = off2 * off3
      off6 = off3 * off3
      off7 = off3 * off4
      cut2 = cut * cut
      cut3 = cut2 * cut
      cut4 = cut2 * cut2
      cut5 = cut2 * cut3
      cut6 = cut3 * cut3
      cut6 = cut3 * cut3
      cut7 = cut3 * cut4
c
c     get 3th degree multiplicative switching function coefficients
c
      if (cut .lt. off) then
         denom = (cut-off)**3
         c0 = off2 * (3.0d0*cut-off) / denom
         c1 = -6.0d0 * off*cut / denom
         c2 = 3.0d0 * (off+cut) / denom
         c3 = -2.0d0 / denom
c
c     get 4th degree additive switching function coefficients
c               (9.0 is optimal value in "term")
c
c        term = 9.0d0 * cut*off / (off-cut)
c        denom = off4 - 4.0d0*off3*cut + 6.0d0*off2*cut2
c    &              - 4.0d0*off*cut3 + cut4
c        denom = term * denom
c        f0 = 16.0d0 * off2*cut2 / denom
c        f1 = -32.0d0 * off*cut * (off+cut) / denom
c        f2 = 16.0d0 * (off2+4.0d0*off*cut+cut2) / denom
c        f3 = -32.0d0 * (off+cut) / denom
c        f4 = 16.0d0 / denom
c
c     get 5th degree additive switching function coefficients
c               (9.0 is optimal value in "term")
c
         term = 9.0d0 * cut*off / (off-cut)
         denom = off5 - 5.0d0*off4*cut + 10.0d0*off3*cut2
     &              - 10.0d0*off2*cut3 + 5.0d0*off*cut4 - cut5
         denom = term * denom
         f0 = off2*cut2 * (18.0d0*off-13.0d0*cut) / denom
         f1 = off*cut
     &           * (-36.0d0*off2-15.0d0*off*cut+26.0d0*cut2) / denom
         f2 = (18.0d0*off3+69.0d0*off2*cut
     &         -24.0d0*off*cut2-13.0d0*cut3) / denom
         f3 = (-41.0d0*off2-30.0d0*off*cut+21.0d0*cut2) / denom
         f4 = (28.0d0*off-3.0d0*cut) / denom
         f5 = -5.0d0 / denom
c
c     get 5th degree multiplicative switching function coefficients
c
c        denom = (off-cut)**5
c        c0 = off*off2 * (off2-5.0d0*off*cut+10.0d0*cut2) / denom
c        c1 = -30.0d0 * off2*cut2 / denom
c        c2 = 30.0d0 * (off2*cut+off*cut2) / denom
c        c3 = -10.0d0 * (off2+4.0*off*cut+cut2) / denom
c        c4 = 15.0d0 * (off+cut) / denom
c        c5 = -6.0d0 / denom
c
c     get 6th degree additive switching function coefficients
c               (11.5 is optimal value in "term")
c
c        term = 11.5d0 * cut*off / (off-cut)
c        denom = cut6 - 6.0d0*cut5*off + 15.0d0*cut4*off2
c    &              - 20.0d0*cut3*off3 + 15.0d0*cut2*off4
c    &              - 6.0d0*cut*off5 + off6
c        denom = term * denom
c        f0 = -64.0d0 * cut3*off3 / denom
c        f1 = 192.0d0 * cut2*off2 * (cut+off) / denom
c        f2 = -192.0d0 * cut*off * (cut2+3.0d0*cut*off+off2) / denom
c        f3 = 64.0d0 * (cut3+9.0d0*cut2*off+9.0d0*cut*off2+off3) / denom
c        f4 = -192.0d0 * (cut2+3.0d0*cut*off+off2) / denom
c        f5 = 192.0d0 * (cut+off) / denom
c        f6 = -64.0d0 / denom
c
c     get 7th degree additive switching function coefficients
c               (9.3 is optimal value in "term")
c
c        term = 9.3d0 * cut*off / (off-cut)
c        denom = cut7 - 7.0d0*cut6*off + 21.0d0*cut5*off2
c    &              - 35.0d0*cut4*off3 + 35.0d0*cut3*off4
c    &              - 21.0d0*cut2*off5 + 7.0d0*cut*off6 - off7
c        denom = term * denom
c        f0 = cut3*off3 * (-39.0d0*cut+64.0d0*off) / denom
c        f1 = cut2*off2
c    &           * (117.0d0*cut2-100.0d0*cut*off-192.0d0*off2) / denom
c        f2 = cut*off * (-117.0d0*cut3-84.0d0*cut2*off
c    &                   +534.0d0*cut*off2+192.0d0*off3) / denom
c        f3 = (39.0d0*cut4+212.0d0*cut3*off-450.0d0*cut2*off2
c    &            -612.0d0*cut*off3-64.0d0*off4) / denom
c        f4 = (-92.0d0*cut3+66.0d0*cut2*off
c    &            +684.0d0*cut*off2+217.0d0*off3) / denom
c        f5 = (42.0d0*cut2-300.0d0*cut*off-267.0d0*off2) / denom
c        f6 = (36.0d0*cut+139.0d0*off) / denom
c        f7 = -25.0d0 / denom
c
c     get 7th degree additive switching function coefficients
c                  (pos=0.48 is optimal value)
c
c        term = 11.667d0*cut*off / (off-cut)
c        pos = 0.48d0
c        pos2 = pos * pos
c        pos3 = pos2 * pos
c        pos4 = pos2 * pos2
c        denom = 7.0d0*off*cut6 + 21.0d0*cut2*off5
c    &           - 21.0d0*off2*cut5 - 7.0d0*cut*off6
c    &           + 84.0d0*pos3*off2*cut5 + 35.0d0*pos4*off3*cut4
c    &           - 21.0d0*pos4*off2*cut5 - 140.0d0*pos3*off3*cut4
c    &           + 84.0d0*pos*off2*cut5 - 4.0d0*pos3*off7
c    &           + off7*pos4 + 6.0d0*off7*pos2 - cut7*pos4
c    &           - 7.0d0*off6*pos4*cut + 28.0d0*pos3*off6*cut
c    &           - 42.0d0*pos2*off6*cut + 35.0d0*off3*cut4
c    &           - 140.0d0*pos*off3*cut4 - 84.0d0*pos3*off5*cut2
c    &           + 126.0d0*pos2*off5*cut2 + 210.0d0*pos2*off3*cut4
c    &           + 140.0d0*pos*off4*cut3 - 84.0d0*pos*off5*cut2
c    &           - 210.0d0*pos2*off4*cut3 + 140.0d0*pos3*off4*cut3
c    &           - 35.0d0*off4*cut3 + 21.0d0*off5*pos4*cut2
c    &           + 28.0d0*pos*off6*cut - 4.0d0*pos*off7 - cut7 + off7
c    &           - 28.0d0*pos*off*cut6 - 35.0d0*pos4*off4*cut3
c    &           - 28.0d0*pos3*off*cut6 + 7.0d0*pos4*off*cut6
c    &           + 42.0d0*pos2*off*cut6 - 126.0d0*pos2*off2*cut5
c    &           - 6.0d0*cut7*pos2 + 4.0d0*cut7*pos3 + 4.0d0*cut7*pos
c        denom = denom * term * pos4
c        f0 = off3*cut3 * (-4.0d0*pos*off+7.0d0*off*pos2-3.0d0*cut
c    &                       +10.0d0*cut*pos-7.0d0*pos2*cut) / denom
c        f1 = -3.0d0 * off2*cut2 * (8.0d0*off*cut*pos-4.0d0*off*cut
c    &                              -4.0d0*off2*pos-3.0d0*cut2
c    &                              +10.0d0*cut2*pos-7.0d0*cut2*pos2
c    &                              +7.0d0*off2*pos2) / denom
c        f2 = 3.0d0 * cut*off*(32.0d0*off*cut2*pos-14.0d0*cut2*off*pos2
c    &                  -6.0d0*cut*off2-12.0d0*off*cut2+7.0d0*off3*pos2
c    &                  -4.0d0*off3*pos+4.0d0*cut*off2*pos-3.0d0*cut3
c    &                     +10.0d0*pos*cut3-7.0d0*pos2*cut3
c    &                     +14.0d0*cut*off2*pos2) / denom
c        f3 = -(7.0d0*off4*pos2-4.0d0*off4*pos+56.0d0*off3*pos2*cut
c    &          -8.0d0*off3*cut*pos-12.0d0*off3*cut-54.0d0*off2*cut2
c    &          +108.0d0*off2*cut2*pos+104.0d0*off*pos*cut3
c    &          -36.0d0*off*cut3-56.0d0*off*pos2*cut3
c    &          -3.0d0*cut4+10.0d0*pos*cut4-7.0d0*cut4*pos2) / denom
c        f4 = 3.0d0 * (-14.0d0*cut2*off*pos2+14.0d0*cut*off2*pos2
c    &                 +44.0d0*off*cut2*pos+16.0d0*cut*off2*pos-off3
c    &                 -4.0d0*cut3+12.0d0*pos*cut3-7.0d0*pos2*cut3
c    &                 +7.0d0*off3*pos2-18.0d0*off*cut2
c    &                 -12.0d0*cut*off2-2.0d0*off3*pos) / denom
c        f5 = -3.0d0 * (2.0d0*off2*pos-3.0d0*off2+24.0d0*off*cut*pos
c    &                  -12.0d0*off*cut+16.0d0*cut2*pos-6.0d0*cut2
c    &                  +7.0d0*off2*pos2-7.0d0*cut2*pos2) / denom
c        f6 = (14.0d0*pos*off+7.0d0*off*pos2-9.0d0*off
c    &           +28.0d0*cut*pos-7.0d0*pos2*cut-12.0d0*cut) / denom
c        f7 = -3.0d0 * (2.0d0*pos - 1.0d0) / denom
      end if
      return
      end
