c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##################################################################
c     ##                                                              ##
c     ##  subroutine be  --  find energy & gradient components for    ##
c     ##  bonded energy terms and intramolecular VDW and dipole       ## 
c     ##                                                              ##
c     ##################################################################
c
c Wijnand Mooij, may 1997
c VDW terms are only calculated when special intramolecular parameters are
c being used, else they are just calculated in the nonbe routine.
c
      subroutine be (energy,derivs)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'deriv.i'
      include 'energi.i'
      include 'potent.i'
      include 'derivxtra.i'
      include 'energixtra.i'
      include 'potentxtra.i'
      include 'inform.i'
      include 'bound.i'
      integer i,j
      real*8 energy,derivs(3,maxatm)
c
c
c     zero out each of the potential energy components
c
      eb = 0.0d0
      ea = 0.0d0
      eba = 0.0d0
      eub = 0.0d0
      eaa = 0.0d0
      eopb = 0.0d0
      eid = 0.0d0
      eit = 0.0d0
      et = 0.0d0
      ebt = 0.0d0
      ed = 0.0d0
c
c     zero out each of the first derivative components
c
      do i = 1, n
         do j = 1, 3
            deb(j,i) = 0.0d0
            dea(j,i) = 0.0d0
            deba(j,i) = 0.0d0
            deub(j,i) = 0.0d0
            deaa(j,i) = 0.0d0
            deopb(j,i) = 0.0d0
            deid(j,i) = 0.0d0
            deit(j,i) = 0.0d0
            det(j,i) = 0.0d0
            debt(j,i) = 0.0d0
            ded(j,i) = 0.0d0
         end do
      end do
c
c     call the local geometry energy and gradient routines
c
      if (use_vdw_intra) then
         if (use_buck_intra) call ebuckintra1
         if (use_hal_intra) call ehalintra1
         if (use_lj_intra) call eljintra1
      endif
      if (use_bond)  call ebond1
      if (use_angle)  call eangle1
      if (use_strbnd)  call estrbnd1
      if (use_urey)  call eurey1
      if (use_angang)  call eangang1
      if (use_opbend)  call eopbend1
      if (use_improp)  call eimprop1
      if (use_imptor)  call eimptor1
      if (use_tors)  call etors1
      if (use_strtor)  call estrtor1
c      print *,'use_bounds',use_bounds
      if (use_dipole) call eintradipole1


      energy = eb + ea + eba + eub + eaa + eopb + eid
     &     + eit + et + ebt + ed + et2 + evintra + ehbintra

      if (verbose) then
         if (eb .ne. 0.0)   print 111,'E-bond          ',eb
         if (ea .ne. 0.0)   print 111,'E-angle         ',ea
         if (eba .ne. 0.0)  print 111,'E-strbnd        ',eba
         if (eub.ne.0.0)  print 111,'E-ureyb         ',eub
         if (eaa.ne.0.0) print 111,'E-angang        ',eaa
         if (eopb.ne.0.0)  print 111,'E-opbend        ',eopb
         if (eid.ne.0.0)  print 111,'E-impdih        ',eid
         if (eit.ne.0.0)  print 111,'E-impdih        ',eit
         if (et.ne.0.0)   print 111,'E-torsion       ',et
         if (ebt.ne.0.0)  print 111,'E-strtor        ',ebt
         if (ed.ne.0.0)   print 111,'E-dipole        ',ed
         if (evintra.ne.0.0) print 111,'E-vdwintra      ',evintra
         if (ehbintra.ne.0.0) print 111,'E-hbintra       ',ehbintra
      endif
 111  format (a17,f10.4)
      if (verbose) print 111,'E-bonded total  ',energy


      do i = 1, n
         do j = 1, 3
            derivs(j,i) = deb(j,i) + dea(j,i) + deba(j,i)
     &                       + deub(j,i) + deaa(j,i) + deopb(j,i)
     &                       + deid(j,i) + deit(j,i) + det(j,i)
     &           + debt(j,i) + dett(j,i) + ded(j,i) + det2(j,i) + 
     +           devintra(j,i) + dehbintra(j,i)
         end do
      end do
      return
      end


