c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine ebuckintra1  --  Buckingham energy & derivatives  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "ebuck1" calculates the van der Waals energy and its first
c     derivatives with respect to Cartesian coordinates using
c     the Buckingham exp-6 formalism.

c     Uses special intramolecular parameters
c     Wijnand Mooij, june 1997
c     Loops only over intramolecular interactions
c

      subroutine ebuckintra1
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'derivxtra.i'
      include 'energixtra.i'
      include 'inter.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'usage.i'
      include 'vdwintra.i'
      include 'vdwpotintra.i'
      include 'virial.i'
     
      include 'chgpot.i'
      include 'hbondintra.i'

      integer i,j,k,ii,kk,iv,kv
      integer it,kt,skip(maxatm)
      real*8 e,p,p2,p6,p12,rv,eps,rdn
      real*8 xi,yi,zi,xr,yr,zr
      real*8 redi,rediv,redk,redkv,rvterm
      real*8 dedx,dedy,dedz,de
      real*8 rik,rik2,rik3,rik4,rik5,taper,dtaper
      real*8 expcut,expcut2,expterm,expmerge
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      logical iuse
      integer kkk
      real*8 F,D
      integer ia,ib,ic
      real*8 angle,dot,cosine
      real*8 deddt,terma,termc,term
      real*8 xia,yia,zia
      real*8 xib,yib,zib
      real*8 xic,yic,zic
      real*8 xid,yid,zid
      real*8 xab,yab,zab,rab2,rab,rab0,rterm,ecp
      real*8 xcb,ycb,zcb,rcb2
      real*8 xdb,ydb,zdb
      real*8 xp,yp,zp,rp
      real*8 dedxia,dedyia,dedzia
      real*8 dedxib,dedyib,dedzib
      real*8 dedxic,dedyic,dedzic
      logical hbond

c
c
c     zero out the van der Waals energy and first derivatives
c
c      print *,'EBUCKINTRA'
      evintra = 0.0d0
      ehbintra = 0.0d0
c      e14intra = 0.0d0
      do i = 1, n
         devintra(1,i) = 0.0d0
         devintra(2,i) = 0.0d0
         devintra(3,i) = 0.0d0
         dehbintra(1,i) = 0.0d0
         dehbintra(2,i) = 0.0d0
         dehbintra(3,i) = 0.0d0
c         de14intra(1,i) = 0.0d0
c         de14intra(2,i) = 0.0d0
c         de14intra(3,i) = 0.0d0
      end do
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c
c     switch from exponential to R^12 at very short range
c
      expcut = 2.0d0
      expcut2 = expcut * expcut
      expmerge = (aterm*exp(-bterm/expcut) - cterm*(expcut**6))
     &                               / (expcut**12)
c
c     calculate the "reduced" atomic coordinates
c
      do k = 1, nvdw
         i = ivdw(k)
         iv = ired(i)
         rdn = kred(i)
         xred(i) = rdn*(x(i)-x(iv)) + x(iv)
         yred(i) = rdn*(y(i)-y(iv)) + y(iv)
         zred(i) = rdn*(z(i)-z(iv)) + z(iv)
      end do
c
c     find van der Waals energy and derivatives via double loop
c
      do ii = 1, nvdw-1
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
         do j = 1, n12(i)
            skip(i12(j,i)) = i * vdw12use
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i * vdw13use
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = i * vdw14use
         end do
c Loop over atoms in molcule(ii)
         do kkk = imol(1,molcule(ii)), imol(2,molcule(ii))
            kk = kmol(kkk)

c         do kk = ii , nvdw
c            if (molcule(ii) .eq. molcule(kk) ) then
            k = ivdw(kk)

            if (k.gt.i) then
               kv = ired(k)
               if (iuse .or. use(k) .or. use(kv)) then
                  if (skip(k) .ne. i) then
                     kt = class(k)
                     xr = xi - xred(k)
                     yr = yi - yred(k)
                     zr = zi - zred(k)
c                  if (use_image)  call image (xr,yr,zr,0)
                     rik2 = xr*xr + yr*yr + zr*zr
                     rv = radmin(kt,it)
                     eps = epsilon(kt,it)
                     if (skip(k) .eq. -i)  eps = eps / vdwscale
ccccccccccccccccc NEW cccccccccccccccccccccccccccccccc
                     F=1.0d0
                     D=1.0d0
                     hbond=.false.
                  
c Chech for hydrogen bonding:
c Normal parameters used for 1,4 interactions  

                     if (radminhb(it,kt).ne.0.0d0 .and. skip(k).ne.-i) 
     +                    hbond=.true.
               
                     if (hbond) then
                        D=dielec
                        rv = radminhb(it,kt)
                        eps = epsilonhb(it,kt)
                        if (atomic(k).eq.1) then
                           ia=k
                           ic=i
                        else 
                           ia=i
                           ic=k
                        endif
c Calculate angle and X-H bondlength
                        ib=i12(1,ia)
                        rab0 = bondlength(class(ia),class(ib))
                        xia = x(ia)
                        yia = y(ia)
                        zia = z(ia)
                        xib = x(ib)
                        yib = y(ib)
                        zib = z(ib)
                        xic = x(ic)
                        yic = y(ic)
                        zic = z(ic)
                        xab = xia - xib
                        yab = yia - yib
                        zab = zia - zib
                        rab2 = xab*xab + yab*yab + zab*zab
                        rab=sqrt(rab2)
                        xcb = xic - xib
                        ycb = yic - yib
                        zcb = zic - zib
                        rcb2 = xcb*xcb + ycb*ycb + zcb*zcb
                        xp = ycb*zab - zcb*yab
                        yp = zcb*xab - xcb*zab
                        zp = xcb*yab - ycb*xab
                        rp = sqrt(xp*xp + yp*yp + zp*zp)
                        dot = xab*xcb + yab*ycb + zab*zcb
                        cosine = dot / sqrt(rab2*rcb2)
                        cosine = min(1.0d0,max(-1.0d0,cosine))
                        angle = acos(cosine)
                        rterm=rab/rab0
                        F = cosine*rterm
c Angle derivative terms:
                        deddt = -sin(angle)*rterm
                        terma = -deddt / (rab2*rp)
                        termc = deddt / (rcb2*rp)
                        dedxia = terma * (yab*zp-zab*yp)
                        dedyia = terma * (zab*xp-xab*zp)
                        dedzia = terma * (xab*yp-yab*xp)
                        dedxic = termc * (ycb*zp-zcb*yp)
                        dedyic = termc * (zcb*xp-xcb*zp)
                        dedzic = termc * (xcb*yp-ycb*xp)
                        dedxib = -dedxia - dedxic
                        dedyib = -dedyia - dedyic
                        dedzib = -dedzia - dedzic
                     endif
c              Divide by dielectric if Hydrogen bonded;else D=1
                     eps=eps/D
cccccccc NEW ccccccccccccccc
                     p2 = (rv*rv) / rik2
                     p6 = p2 * p2 * p2
                     rik = sqrt(rik2)
                     if (p2 .le. expcut2) then
                        p = sqrt(p2)
                        rvterm = -bterm / rv
                        expterm = aterm * exp(-bterm/p)
                        e =  eps * (expterm - F * cterm*p6)
                        de = eps *(rvterm*expterm +
     +                       F * 6.0d0*cterm*p6/rik)
                     else
                        print *,'WARNING: EBUCKINTRA, p2 <= expcut2'
                        print *,i,k,rik
                        p12 = p6 * p6
                        e = expmerge * eps * p12
                        de  = -12.0d0 * expmerge * eps * p12/rik
                     end if
                           
c                        print *,ii,kk,'      ',rik,e
c
c     find the chain rule terms for derivative components
c
                     de = de / rik
                     dedx = de * xr
                     dedy = de * yr
                     dedz = de * zr
c
c     increment the total van der Waals energy and derivatives
c
                     if (.not. hbond) then
                        evintra = evintra + e
c                            if (abs(e).gt.0.1) 
c     +                          print'(2i5,4f10.4,"   1,4")',i,k,rik,
c     +                          rv,eps,e
                        if (i .eq. iv) then
                           devintra(1,i) = devintra(1,i) + dedx
                           devintra(2,i) = devintra(2,i) + dedy
                           devintra(3,i) = devintra(3,i) + dedz
                        else
                           devintra(1,i) = devintra(1,i) + dedx*redi
                           devintra(2,i) = devintra(2,i) + dedy*redi
                           devintra(3,i) = devintra(3,i) + dedz*redi
                           devintra(1,iv) = devintra(1,iv)+dedx*rediv
                           devintra(2,iv) = devintra(2,iv)+dedy*rediv
                           devintra(3,iv) = devintra(3,iv)+dedz*rediv
                        end if
                        if (k .eq. kv) then
                           devintra(1,k) = devintra(1,k) - dedx
                           devintra(2,k) = devintra(2,k) - dedy
                           devintra(3,k) = devintra(3,k) - dedz
                        else 
                           redk = kred(k)
                           redkv = 1.0d0 - redk
                           devintra(1,k) = devintra(1,k) - dedx*redk
                           devintra(2,k) = devintra(2,k) - dedy*redk
                           devintra(3,k) = devintra(3,k) - dedz*redk
                           devintra(1,kv) = devintra(1,kv)-dedx*redkv
                           devintra(2,kv) = devintra(2,kv)-dedy*redkv
                           devintra(3,kv) = devintra(3,kv)-dedz*redkv
                        end if
                     else
c Hydrogen bonded term:
c                           if (abs(e).gt.0.1) 
c     +                          print'(2i5,4f10.4,"   H")',i,k,rik,rv
c     +                          ,eps,e
                        ehbintra = ehbintra + e
                        if (i .eq. iv) then
                           dehbintra(1,i) = dehbintra(1,i) + dedx
                           dehbintra(2,i) = dehbintra(2,i) + dedy
                           dehbintra(3,i) = dehbintra(3,i) + dedz
                        else
                           dehbintra(1,i) = dehbintra(1,i)+ dedx*redi
                           dehbintra(2,i) = dehbintra(2,i)+ dedy*redi
                           dehbintra(3,i) = dehbintra(3,i)+ dedz*redi
                           dehbintra(1,iv) = dehbintra(1,iv) +
     +                          dedx*rediv
                           dehbintra(2,iv) = dehbintra(2,iv) +
     +                          dedy*rediv
                           dehbintra(3,iv) = dehbintra(3,iv) +
     +                          dedz*rediv
                        end if
                        if (k .eq. kv) then
                           dehbintra(1,k) = dehbintra(1,k) - dedx
                           dehbintra(2,k) = dehbintra(2,k) - dedy
                           dehbintra(3,k) = dehbintra(3,k) - dedz
                        else 
                           redk = kred(k)
                           redkv = 1.0d0 - redk
                           dehbintra(1,k) = dehbintra(1,k)- dedx*redk
                           dehbintra(2,k) = dehbintra(2,k)- dedy*redk
                           dehbintra(3,k) = dehbintra(3,k)- dedz*redk
                           dehbintra(1,kv) = dehbintra(1,kv) -
     +                          dedx*redkv
                           dehbintra(2,kv) = dehbintra(2,kv) -
     +                          dedy*redkv
                           dehbintra(3,kv) = dehbintra(3,kv) -
     +                          dedz*redkv
                        end if

c Add derivatives of angle dependency and rab dependency
                        ecp=eps*cterm*p6
                        rterm=ecp*cosine/(rab0*rab)
                        dehbintra(1,ia) = dehbintra(1,ia) - 
     +                       ecp*dedxia - rterm * xab
                        dehbintra(2,ia) = dehbintra(2,ia) - 
     +                       ecp*dedyia - rterm * yab
                        dehbintra(3,ia) = dehbintra(3,ia) - 
     +                       ecp*dedzia - rterm * zab
                        dehbintra(1,ib) = dehbintra(1,ib) - 
     +                       ecp*dedxib + rterm * xab
                        dehbintra(2,ib) = dehbintra(2,ib) - 
     +                       ecp*dedyib + rterm * yab
                        dehbintra(3,ib) = dehbintra(3,ib) - 
     +                       ecp*dedzib + rterm * zab
                        dehbintra(1,ic) = dehbintra(1,ic) - 
     +                       ecp*dedxic
                        dehbintra(2,ic) = dehbintra(2,ic) - 
     +                       ecp*dedyic
                        dehbintra(3,ic) = dehbintra(3,ic) - 
     +                       ecp*dedzic
                     endif

c
c     increment the total intermolecular energy
c
                     if (molcule(i) .ne. molcule(k)) then
                        einter = einter + e
                     end if
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*dedx
                        viry = viry + yr*dedy
                        virz = virz + zr*dedz
                     end if
                  end if
               end if
            end if
c           endif
         end do
      end do
      return
      end

