c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine edipole1  --  dipole-dipole energy & derivs  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "eintradipole1" calculates the dipole-dipole interaction energy
c     and first derivatives with respect to Cartesian coordinates only for
c intramolecular interactions
c 
c
c
      subroutine eintradipole1
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'deriv.i'
      include 'dipole.i'
      include 'chgpot.i'
      include 'energi.i'
      include 'inter.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'usage.i'
      include 'virial.i'
      include 'units.i'

      integer i,j,k,i1,i2,k1,k2
      real*8 xi,yi,zi,xk,yk,zk
      real*8 xq,yq,zq,xr,yr,zr
      real*8 f,fi,fik
      real*8 e,r2,ri2,rk2,rirkr3
      real*8 doti,dotk,dotp
      real*8 si1,si2,sk1,sk2
      real*8 term,dedr,dedrirk
      real*8 deddoti,deddotk,deddotp
      real*8 termx,termy,termz
      real*8 dedrirkri2,dedrirkrk2
      real*8 termxi,termyi,termzi
      real*8 termxk,termyk,termzk
      real*8 dedxi1,dedyi1,dedzi1
      real*8 dedxi2,dedyi2,dedzi2
      real*8 dedxk1,dedyk1,dedzk1
      real*8 dedxk2,dedyk2,dedzk2
      real*8 r,r3,r4,r5,taper,dtaper
      real*8 dtaperx,dtapery,dtaperz

      integer kk
c
c
c     zero out the overall dipole interaction energy and derivs,
c     then set up the constants for the calculation
c
      
      ed = 0.0d0
      do i = 1, n
         ded(1,i) = 0.0d0
         ded(2,i) = 0.0d0
         ded(3,i) = 0.0d0
      end do
      if (ndipole .eq. 0)  return
c
c     set conversion factor and switching function coefficients
c
c      f = 14.388d0 / dielec
c     from tinker 3.4:

      f = electric / (debye**2 * dielec)

c      call switch ('DIPOLE')
c
c     compute the dipole interaction energy and first derivatives
c
      do i = 1, ndipole-1
         i1 = idpl(1,i)
         i2 = idpl(2,i)
         si1 = 1.0d0 - sdpl(i)
         si2 = sdpl(i)
         xi = x(i2) - x(i1)
         yi = y(i2) - y(i1)
         zi = z(i2) - z(i1)
         ri2 = xi*xi + yi*yi + zi*zi
         xq = x(i1) + xi*si2
         yq = y(i1) + yi*si2
         zq = z(i1) + zi*si2
         fi = f * bdpl(i)
c         do kk = imol(1,molcule(i)), imol(2,molcule(i))
         do k = i+1, ndipole
c            k = kmol(kk)
c            print *,i,k,'      ',kk
c            if (k.gt.i) then
            k1 = idpl(1,k)
            k2 = idpl(2,k)
            if (molcule(i1) .eq. molcule(k1)) then
               if (use(i1) .or. use(i2) .or. use(k1) .or. use(k2)) then
                  if (k1.ne.i1 .and. k1.ne.i2 .and.
     +                 k2.ne.i1 .and. k2.ne.i2) then
                     sk1 = 1.0d0 - sdpl(k)
                     sk2 = sdpl(k)
                     xk = x(k2) - x(k1)
                     yk = y(k2) - y(k1)
                     zk = z(k2) - z(k1)
                     xr = xq - x(k1) - xk*sk2
                     yr = yq - y(k1) - yk*sk2
                     zr = zq - z(k1) - zk*sk2
c                  if (use_image)  call image (xr,yr,zr,0)
                     r2 = xr*xr + yr*yr + zr*zr
c                  if (r2 .le. off2) then
                     rk2 = xk*xk + yk*yk + zk*zk
                     rirkr3 = sqrt(ri2*rk2*r2) * r2
                     dotp = xi*xk + yi*yk + zi*zk
                     doti = xi*xr + yi*yr + zi*zr
                     dotk = xk*xr + yk*yr + zk*zr
                     fik = fi * bdpl(k)
c
c     now, form the energy and chain rule terms for derivatives
c
                     e = fik * (dotp-3.0d0*doti*dotk/r2) / rirkr3
c                     print *,i1,k1,'      ',r2,e
                     term = -fik / (rirkr3*r2)
                     deddotp = -term * r2
                     deddoti = term * 3.0d0*dotk
                     deddotk = term * 3.0d0*doti
                     dedr = term * (3.0d0*dotp-15.0d0*doti*dotk/r2)
                     dedrirk = -e
                     dedrirkri2 = dedrirk / ri2
                     dedrirkrk2 = dedrirk / rk2
c
c     more chain rule terms for derivative expressions
c
                     termx = dedr*xr + deddoti*xi + deddotk*xk
                     termy = dedr*yr + deddoti*yi + deddotk*yk
                     termz = dedr*zr + deddoti*zi + deddotk*zk
                     termxi = dedrirkri2*xi + deddotp*xk + deddoti*xr
                     termyi = dedrirkri2*yi + deddotp*yk + deddoti*yr
                     termzi = dedrirkri2*zi + deddotp*zk + deddoti*zr
                     termxk = dedrirkrk2*xk + deddotp*xi + deddotk*xr
                     termyk = dedrirkrk2*yk + deddotp*yi + deddotk*yr
                     termzk = dedrirkrk2*zk + deddotp*zi + deddotk*zr
c
c     finally, the individual first derivative components
c
                     dedxi1 = si1*termx - termxi
                     dedyi1 = si1*termy - termyi
                     dedzi1 = si1*termz - termzi
                     dedxi2 = si2*termx + termxi
                     dedyi2 = si2*termy + termyi
                     dedzi2 = si2*termz + termzi
                     dedxk1 = -sk1*termx - termxk
                     dedyk1 = -sk1*termy - termyk
                     dedzk1 = -sk1*termz - termzk
                     dedxk2 = -sk2*termx + termxk
                     dedyk2 = -sk2*termy + termyk
                     dedzk2 = -sk2*termz + termzk
c
c     increment the overall energy and derivative expressions
c
                     ed = ed + e
                     ded(1,i1) = ded(1,i1) + dedxi1
                     ded(2,i1) = ded(2,i1) + dedyi1
                     ded(3,i1) = ded(3,i1) + dedzi1
                     ded(1,i2) = ded(1,i2) + dedxi2
                     ded(2,i2) = ded(2,i2) + dedyi2
                     ded(3,i2) = ded(3,i2) + dedzi2
                     ded(1,k1) = ded(1,k1) + dedxk1
                     ded(2,k1) = ded(2,k1) + dedyk1
                     ded(3,k1) = ded(3,k1) + dedzk1
                     ded(1,k2) = ded(1,k2) + dedxk2
                     ded(2,k2) = ded(2,k2) + dedyk2
                     ded(3,k2) = ded(3,k2) + dedzk2
c
c     increment the total intermolecular energy
c
                     if (molcule(i1) .ne. molcule(k1)) then
                        einter = einter + e
                     end if
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        virx = virx + xr*(dedxi1+dedxi2)
                        viry = viry + yr*(dedyi1+dedyi2)
                        virz = virz + zr*(dedzi1+dedzi2)
                     end if
                  end if
               end if
            end if
         end do
      end do
      return
      end
