c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine elj1  --  Lennard-Jones energy & derivatives  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "elj1" calculates the van der Waals energy and its first
c     derivatives with respect to Cartesian coordinates using
c     the Lennard-Jones 6-12 formalism
c XTINKER version:
c Only intramolecular interactions
c
      subroutine eljintra1
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'derivxtra.i'
      include 'energixtra.i'
      include 'group.i'
      include 'inter.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'usage.i'
      include 'vdwintra.i'
      include 'vdwpotintra.i'
      include 'virial.i'
      integer i,j,k,ii,kk,iv,kv,kkk
      integer it,kt,skip(maxatm)
      real*8 e,p6,p12,rv,eps,rdn,fgrp
      real*8 xi,yi,zi,xr,yr,zr
      real*8 redi,rediv,redk,redkv
      real*8 dedx,dedy,dedz,de
      real*8 rik,rik2,rik3,rik4,rik5,taper,dtaper
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      logical proceed,iuse
c
c
c     zero out the van der Waals energy and first derivatives
c
      evintra = 0.0d0
      do i = 1, n
         devintra(1,i) = 0.0d0
         devintra(2,i) = 0.0d0
         devintra(3,i) = 0.0d0
      end do
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     calculate the "reduced" atomic coordinates
c
      do k = 1, nvdw
         i = ivdw(k)
         iv = ired(i)
         rdn = kred(i)
         xred(i) = rdn*(x(i)-x(iv)) + x(iv)
         yred(i) = rdn*(y(i)-y(iv)) + y(iv)
         zred(i) = rdn*(z(i)-z(iv)) + z(iv)
      end do
c
c     find van der Waals energy and derivatives via double loop
c
      do ii = 1, nvdw-1
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
         do j = 1, n12(i)
            skip(i12(j,i)) = i * vdw12use
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i * vdw13use
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = i * vdw14use
         end do
c        Loop over atoms in molcule(ii)  
         do kkk = imol(1,molcule(ii)),imol(2,molcule(ii))
            kk = kmol(kkk)
            k = ivdw(kk)
            if (k.gt.i) then
               kv = ired(k)
               if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
               if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
               if (proceed)  proceed = (skip(k) .ne. i)
c
c     compute the energy contribution for this interaction
c
               if (proceed) then
                  kt = class(k)
                  xr = xi - xred(k)
                  yr = yi - yred(k)
                  zr = zi - zred(k)
c               if (use_image)  call image (xr,yr,zr,0)
                  rik2 = xr*xr + yr*yr + zr*zr
c
c     compute energy and derivatives for this interaction
c
                  rv = radmin(kt,it)
                  eps = epsilon(kt,it)
                  if (skip(k) .eq. -i)  eps = eps / vdwscale
                  rik = sqrt(rik2)
                  p6 = rv**6 / rik2**3
                  p12 = p6 * p6
                  e = eps * (p12 - 2.0d0 * p6)
                  de = eps * (p12 - p6) * (-12.0d0/rik)
c
c     scale the interaction based on its group membership
c
                  if (use_group) then
                     e = e * fgrp
                     de = de * fgrp
                  end if
c
c     find the chain rule terms for derivative components
c
                  de = de / rik
                  dedx = de * xr
                  dedy = de * yr
                  dedz = de * zr
c
c     increment the total van der Waals energy and derivatives
c
                  evintra = evintra + e
                  if (i .eq. iv) then
                     devintra(1,i) = devintra(1,i) + dedx
                     devintra(2,i) = devintra(2,i) + dedy
                     devintra(3,i) = devintra(3,i) + dedz
                  else
                     devintra(1,i) = devintra(1,i) + dedx*redi
                     devintra(2,i) = devintra(2,i) + dedy*redi
                     devintra(3,i) = devintra(3,i) + dedz*redi
                     devintra(1,iv) = devintra(1,iv) + dedx*rediv
                     devintra(2,iv) = devintra(2,iv) + dedy*rediv
                     devintra(3,iv) = devintra(3,iv) + dedz*rediv
                  end if
                  if (k .eq. kv) then
                     devintra(1,k) = devintra(1,k) - dedx
                     devintra(2,k) = devintra(2,k) - dedy
                     devintra(3,k) = devintra(3,k) - dedz
                  else
                     redk = kred(k)
                     redkv = 1.0d0 - redk
                     devintra(1,k) = devintra(1,k) - dedx*redk
                     devintra(2,k) = devintra(2,k) - dedy*redk
                     devintra(3,k) = devintra(3,k) - dedz*redk
                     devintra(1,kv) = devintra(1,kv) - dedx*redkv
                     devintra(2,kv) = devintra(2,kv) - dedy*redkv
                     devintra(3,kv) = devintra(3,kv) - dedz*redkv
                  end if
               end if
c
c     increment the virial for use in pressure computation
c
               if (isobaric) then
                  virx = virx + xr*dedx
                  viry = viry + yr*dedy
                  virz = virz + zr*dedz
               end if
            end if
         end do
      end do
      return
      end

