c
c
c     ###################################################
c     ##  Wijnand Mooij  may 1997
c     ##             
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine vdwpair1  - Vandwerwaals pair energy & derivatives     ##
c     ##                                                            ##
c     ################################################################
c
c
c     "vdwpair1" calculates the van der Waals pair energy and its first
c     derivatives with respect to Cartesian coordinates using
c the 6-12 Lennard-Jones form 
c     or  Halgren's Buffered 14-7 formalism from the MMFF force field
c     or the 6-exp Buckingham potential of MM3
c     or the general 6-exp  Buckingham potential, using three parameters, 
c     and possibly an one-parameter anisotropic repulsion
c
c this version uses non-recuded coordinates for dispersion within the 
c Aexp(-br)-C/r6 potential
c
c The dispersion damping and the anisotropic repulsion:
c W.T.M. Mooij, B.P. van Eijck and F.B. van Duijneveldt, Ab initio
c intermolecular potentials. I. Derivation from methanol dimer and trimer
c calculations, manuscript in preparation
c
      subroutine evdwpair1(ii,kk,jj,xij,yij,zij,rij2,rij,rij2nr,rijnr,
     +     ddispdr,e,disp,rep,de,ani_i,ani_k,nt14)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'deriv.i'
      include 'energi.i'
      include 'molcul.i'
      include 'usage.i'
      include 'vdw.i'
      include 'vdw2.i'
      include 'vdwpot.i'
      include 'potent.i'
      include 'potentxtra.i'
      include 'aniso.i'
      include 'mpole.i'
      include 'chgpot.i'
      include 'hbond.i'
      include 'vdwpair.i'

      real*8 rdamp,term,f,dfdr
      integer i,j,k,ii,kk,iv,kv,jj,nt14
      integer it,kt,skip(maxatm)
      real*8 e,rv,rv7,rv14,rdn
      real*8 de(3,6),taper,dtaper
      real*8 rho,tau
      real*8 rij,rij2,rij3,rij4,rij5,rij6,rij7,
     +     rij6nr,rij7nr,rijnr,rij2nr,ddispdr

      real*8 p6,p12,p2,p,rvterm,expterm

      logical iuse
      real*8 xij,yij,zij,eik,a,b,c
      
      real*8 iz(3),ix(3),kx(3),kz(3),iy(3),ky(3),
     +     yi,yi2,yi3,yk,yk2,yk3,yir2,ykr2,yir,
     +     ykr,dedr,dedani,dedank,danidi(3),danidk(3),dankdi(3),
     +     dankdk(3),danidix(3),danidiz(3),dankdkx(3),dankdkz(3),r(3),
     +     rizx(3),riz(3),rix(3),rkzx(3),rkz(3),rkx(3),
     +     ryi,ryk,ryi3,ryk3,r3yi,r3yk

      

      real*8 temp,p01,p02,p11,afi,afk
      integer itest,iii,kkk
      real*8 scale,eps
      logical ani_i, ani_k
cccccc NEW cccccc      
      real*8 D
      integer ia,ib,ic
      real*8 angle,dot,cosine
      real*8 deddt,terma,termc
      real*8 xia,yia,zia
      real*8 xib,yib,zib
      real*8 xic,yic,zic
      real*8 xid,yid,zid
      real*8 xab,yab,zab,rab2,rab,rab0,rterm,ecp
      real*8 xcb,ycb,zcb,rcb2
      real*8 xdb,ydb,zdb
      real*8 xp,yp,zp,rp
      real*8 dedxia,dedyia,dedzia
      real*8 dedxib,dedyib,dedzib
      real*8 dedxic,dedyic,dedzic
      logical hbond
      real*8 rep,disp
cccccc NEW cccccc      

      kt=class(jj)
      it=class(ii)
      ani_i=.false.
      ani_k=.false.
      ddispdr=0.0d0
      rep=0.0d0
      disp=0.0d0
   
      r(1)=xij
      r(2)=yij
      r(3)=zij

      if (vdwtyp.eq.'WILLIAMS') then
         rij6nr = rij2nr*rij2nr*rij2nr
         rij7nr = rij6nr * rijnr
         A=Ause(kt,it)
         B=Buse(kt,it)
         C=Cuse(kt,it)

         if (nt14.eq.1) then
            A=A/vdwscale
            C=C/vdwscale
         endif

         f=1.0d0
         dfdr=0.0d0

c Damping function for dispersion:
         scale=vdwradius(it)+vdwradius(kt)
         if (scale.ne.0.0d0) then 
            term = - (cdamp*rijnr/scale)**3
            if (term .gt. -50.0d0) then
               term = exp(term)
               f = (1.0d0 - term)**2
               dfdr = 2.0d0*(1.0d0 - term) * 
     +              (3.0d0 * (cdamp/scale)**3 * rij2nr)
     +              * term
            endif
         endif
         
c Repulsion-cut off for efficiancy:
c
         if (rij.lt.repcut) then

            if (aniso(it).ne.0.0d0 .or. aniso(kt).ne.0.0d0) then
c
c Anisotropic repulsion part
c               
               rij3=rij2*rij
               yir2=0.0d0
               ykr2=0.0d0
               if (aniso(it).ne.0.0d0) then
                  ani_i=.true.
                  iii=nraniso(ii)               
                  call prod3(iii,r,zminx,rizx)
                  call prod3(iii,r,zaxes,riz)
                  call prod3(iii,r,xaxes,rix)
                  yi=leny(iii)
                  yi3=leny3(iii)
                  ryi=rij*yi
                  ryi3=rij*yi3
                  r3yi=rij3*yi
                  yir=(r(1)*yaxes(1,iii)+r(2)*yaxes(2,iii)+
     +                 r(3)*yaxes(3,iii))
                  yir2=(yir/ryi)**2
               endif

               if (aniso(kt).ne.0.0d0) then
                  ani_k=.true.
                  kkk=nraniso(kk)
                  call prod3(kkk,r,zminx,rkzx)
                  call prod3(kkk,r,zaxes,rkz)
                  call prod3(kkk,r,xaxes,rkx)
                  yk=leny(kkk)
                  yk3=leny3(kkk)
                  ryk=rij*yk
                  ryk3=rij*yk3
                  r3yk=rij3*yk
                  ykr=(r(1)*yaxes(1,kkk)+r(2)*yaxes(2,kkk)+
     +                 r(3)*yaxes(3,kkk))
                  ykr2=(ykr/ryk)**2
               endif
            
               e =A*exp(-B*(rij-yir2*aniso(it)-ykr2*aniso(kt)))
               dedr = -a * b * exp(-b*(rij-yir2*aniso(it)-
     +              ykr2*aniso(kt))) 
c           dph/di(x)
               if (aniso(it).ne.0.0d0) then
                  dedani = 2.0d0*A*aniso(it)*b*
     +                 exp(-b*(rij-yir2*aniso(it)-
     +                 ykr2*aniso(kt)))*yir/(rij*yi)
                  do i=1,3
                     danidi(i) = ycrosszx(i,iii)*yir/(ryi3) - 
     +                    (rizx(i)-yaxes(i,iii))/(ryi) - r(i)*yir/(r3yi)
                     danidk(i) =  -yaxes(i,iii)/(ryi) + r(i)*yir/(r3yi)
                     danidix(i) = -ycrossz(i,iii)*yir/(ryi3) +
     +                    riz(i)/(ryi)
                     danidiz(i) =  ycrossx(i,iii)*yir/(ryi3) - 
     +                    rix(i)/(ryi)
                  enddo
               else 
                  dedani = 0.0d0
               endif

               if (aniso(kt).ne.0.0d0) then
                  dedank = 2.0d0*A*aniso(kt)*b*
     +                 exp(-b*(rij-yir2*aniso(it)-
     +                 ykr2*aniso(kt)))*ykr/(rij*yk)               
                  do i=1,3
                     dankdk(i) = ycrosszx(i,kkk)*ykr/(ryk3) -
     +                    (rkzx(i)+yaxes(i,kkk))/(ryk) + r(i)*ykr/(r3yk)
                     dankdi(i) = yaxes(i,kkk)/(ryk) - r(i)*ykr/(r3yk)
                     dankdkx(i) = -ycrossz(i,kkk)*ykr/(ryk3) + 
     +                    rkz(i)/(ryk)
                     dankdkz(i) =  ycrossx(i,kkk)*ykr/(ryk3) - 
     +                    rkx(i)/(ryk)
                  enddo
               else 
                  dedank =0.0d0
               endif
c
c              Dispersion part, calculated with non-reduced coordinates
c
               rep = e
               disp = -f*C/rij6nr
               e = e + disp
               ddispdr = 6.0d0*f*C/rij7nr - dfdr*C/rij6nr
               dedr=dedr/rij
               do i=1,3
                  de(i,1)= dedr*r(i) + dedani*danidi(i)+dedank*dankdi(i)
                  de(i,6)= -dedr*r(i) +dedank*dankdk(i)+dedani*danidk(i)
                  de(i,2)= dedani*danidiz(i)
                  de(i,3)= dedani*danidix(i)
                  de(i,4)= dedank*dankdkz(i)
                  de(i,5)= dedank*dankdkx(i)
               enddo           
            else 
c
c Isotropic repulsion
c
               rep = A* exp(-b*rij)
               disp =  - f*C/rij6nr
               e = rep + disp
               dedr = -(a * b) * exp(-b*rij)
               ddispdr = 6.0d0*f*C/rij7nr - dfdr*C/rij6nr
               dedr =dedr/rij
               do i=1,3
                  de(i,1)=dedr*r(i)
                  de(i,6)=-de(i,1)
               enddo
            endif
         else
c
c Only dispersion (long-range)
c
            e = - f*C/rij6nr 
            disp = e
            dedr=0.0d0
            ddispdr = 6.0d0*f*C/rij7nr - dfdr*C/rij6nr
            do i=1,3
               de(i,1)=dedr*r(i)
               de(i,6)=-de(i,1)
            enddo
         endif
      else  if (vdwtyp .eq. 'LENNARD-JONES') then
         rij6 = rij2*rij2*rij2
         rij7 = rij6 * rij
         rv = radmin(kt,it)
         eps = epsilon(kt,it)
         if (nt14.eq.1) eps=eps/vdwscale
         p6 = rv**6/rij6
         p12 = p6*p6
         e = eps * (p12 - 2.0d0 *p6)
         dedr = eps * (p12 - p6) * (-12.0d0/rij)
         dedr = dedr / rij
         do i=1,3
            de(i,1)=dedr*r(i)
            de(i,6)=-de(i,1)
         enddo
      else  if (vdwtyp .eq. 'BUFFERED-14-7') then
         rij6 = rij2*rij2*rij2
         rij7 = rij6 * rij
         rv = radmin(kt,it)
         eps = epsilon(kt,it)
         if (nt14.eq.1) eps=eps/vdwscale
         rv7 = rv**7
         rv14 = rv7 * rv7
         rho = rij7 + (sigma-1.0d0)*rv7
         tau = rij + (kappa-1.0d0)*rv
         e = eps * kappa7 * (rv7/tau**7)
     &        * (sigma*rv7/rho-2.0d0)
         dedr = -7.0d0 * eps * kappa7 * (rv7/tau**8)
     &        * (sigma*rv7/rho-2.0d0)
     &        -7.0d0 * eps * kappa7 * sigma * rv14
     &        * rij6 / (rho**2 * tau**7)
         dedr = dedr/rij
        do i=1,3
            de(i,1)=dedr*r(i)
            de(i,6)=-de(i,1)
         enddo
      else  if (vdwtyp .eq. 'BUCKINGHAM') then
         rij6 = rij2*rij2*rij2
         rij7 = rij6 * rij
         rv = radmin(kt,it)
         eps = epsilon(kt,it)
         if (nt14.eq.1) eps=eps/vdwscale
ccccccccccccccccc NEW cccccccccccccccccccccccccccccccc
         F=1.0d0
         D=1.0d0
         hbond=.false.
                  
c Chech for hydrogen bonding:
c Normal parameters used for 1,4 interactions  

         if (radminhb(it,kt).ne.0.0d0 .and. nt14.ne.1) 
     +        hbond=.true.
         
         if (hbond) then
            D=dielec
            rv = radminhb(it,kt)
            eps = epsilonhb(it,kt)
            if (atomic(kk).eq.1) then
               ia=kk
               ic=ii
            else 
               ia=ii
               ic=kk
            endif
c Calculate angle and X-H bondlength
            ib=i12(1,ia)
            rab0 = bondlength(class(ia),class(ib))
            xia = x(ia)
            yia = y(ia)
            zia = z(ia)
            xib = x(ib)
            yib = y(ib)
            zib = z(ib)
            xic = x(ic)
            yic = y(ic)
            zic = z(ic)
            xab = xia - xib
            yab = yia - yib
            zab = zia - zib
            rab2 = xab*xab + yab*yab + zab*zab
            rab=sqrt(rab2)
            xcb = xic - xib
            ycb = yic - yib
            zcb = zic - zib
            rcb2 = xcb*xcb + ycb*ycb + zcb*zcb
            xp = ycb*zab - zcb*yab
            yp = zcb*xab - xcb*zab
            zp = xcb*yab - ycb*xab
            rp = sqrt(xp*xp + yp*yp + zp*zp)
            dot = xab*xcb + yab*ycb + zab*zcb
            cosine = dot / sqrt(rab2*rcb2)
            cosine = min(1.0d0,max(-1.0d0,cosine))
            angle = acos(cosine)
            rterm=rab/rab0
            F = cosine*rterm
c Angle derivative terms:
            deddt = -sin(angle)*rterm
            terma = -deddt / (rab2*rp)
            termc = deddt / (rcb2*rp)
            dedxia = terma * (yab*zp-zab*yp)
            dedyia = terma * (zab*xp-xab*zp)
            dedzia = terma * (xab*yp-yab*xp)
            dedxic = termc * (ycb*zp-zcb*yp)
            dedyic = termc * (zcb*xp-xcb*zp)
            dedzic = termc * (xcb*yp-ycb*xp)
            dedxib = -dedxia - dedxic
            dedyib = -dedyia - dedyic
            dedzib = -dedzia - dedzic
         endif
c               Divide by dielectric if Hydrogen bonded;else D=1
         eps=eps/D
cccccccc NEW ccccccccccccccc

         p2 = (rv*rv) / rij2
         p6 = p2 * p2 * p2
         if (p2 .le. expcut2) then
            p = sqrt(p2)
            rvterm = -bterm / rv
            expterm = aterm * exp(-bterm/p)
            e = eps * (expterm - cterm*p6)
            dedr = eps * (rvterm*expterm+6.0d0*cterm*p6/rij)
         else
            p12 = p6 * p6
            e = expmerge * eps * p12
            dedr = -12.0d0 * e / rij
         end if
         dedr = dedr/rij
         do i=1,3
            de(i,1)=dedr*r(i)
            de(i,6)=-de(i,1)
         enddo

      endif
      

      return
      end
      
      subroutine prod(v1,v2,v3)
      implicit none
      real*8 v1(3),v2(3),v3(3)

      v3(1)=v1(2)*v2(3)-v1(3)*v2(2)
      v3(2)=v1(3)*v2(1)-v1(1)*v2(3)
      v3(3)=v1(1)*v2(2)-v1(2)*v2(1)    
      end

      subroutine prod2(k,v1,v2,v3)
      implicit none
      real*8 v1(3,*),v2(3,*),v3(3,*)
      integer k
      v3(1,k)=v1(2,k)*v2(3,k)-v1(3,k)*v2(2,k)
      v3(2,k)=v1(3,k)*v2(1,k)-v1(1,k)*v2(3,k)
      v3(3,k)=v1(1,k)*v2(2,k)-v1(2,k)*v2(1,k)    
      end
      
      
      subroutine prod3(k,v1,v2,v3)
      implicit none
      real*8 v1(3),v2(3,*),v3(3)
      integer k
      v3(1)=v1(2)*v2(3,k)-v1(3)*v2(2,k)
      v3(2)=v1(3)*v2(1,k)-v1(1)*v2(3,k)
      v3(3)=v1(1)*v2(2,k)-v1(2)*v2(1,k)    
      end



