c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine ewilliams  --  Buckingham energy             ## 
c     ##                                                          ##
c     ##############################################################
c
c
c     "ewilliams" calculates the van der Waals energy using
c     the Buckingham exp-6 formalism, in the A exp(-b r)-c/r6 form
c     including anisotropic repulsion
c      this version does not use reduced coordinates for dispersion.
      
c The dispersion damping and the anisotropic repulsion:
c W.T.M. Mooij, B.P. van Eijck and F.B. van Duijneveldt, An ab initio
c intermolecular potential for alcohols, manuscript in preparation
c
c Energy shifting not implemented
            
      subroutine ewilliams
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'energi.i'
      include 'group.i'
      include 'inter.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'usage.i'
      include 'vdw.i'
      include 'vdw2.i'
      include 'vdwpot.i'
      include 'aniso.i'
      include 'mpole.i'


      integer i,j,k,ii,kk,iv,kv
      integer it,kt,skip(maxatm)
      real*8 e,rv,eps,rdn
      real*8 p,p2,p6,p12,fgrp
      real*8 redi,rediv,redk,redkv,rvterm
      real*8 dedx,dedy,dedz
      real*8 rik,rik2,rik3,rik4,rik5,taper,dtaper
      real*8 expcut,expcut2,expterm,expmerge
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      logical proceed,iuse
      real*8 a,b,c,rik6,rik7
      real*8 xiz,yiz,ziz,xix,yix,zix
      real*8 xkz,ykz,zkz,xkx,ykx,zkx
      real*8 xi,yi,zi,xr,yr,zr

      logical header,huge


      real*8 yir2,ykr2,yir,ykr,r(3),ryi,ryk

      

      real*8 f,term
      integer itest,iii,kkk
      real*8 scale,y2
      integer i1,i2
      real*8 xinr,yinr,zinr,rnr(3),riknr,rik2nr,rik6nr  

c
c     zero out the van der Waals energy contributions
c
      ev = 0.0d0
      e14 = 0.0d0
      ex = 0.0d0
      
c
c Calculate y-axis orientation for anisotropic repulsion term
c
      do k = 1, naniso
         i = ianiso(k)
         if (ipole(i).ne.i) then
            print*,'Multipoles on all atoms necessary',
     +           ' for anisotropic repulsion'
            call fatal
         endif
         zaxes(1,k) = x(zaxis(i)) - x(i)
         zaxes(2,k) = y(zaxis(i)) - y(i)
         zaxes(3,k) = z(zaxis(i)) - z(i)
         xaxes(1,k) = x(xaxis(i)) - x(i)
         xaxes(2,k) = y(xaxis(i)) - y(i)
         xaxes(3,k) = z(xaxis(i)) - z(i)
         call prod2(k,zaxes,xaxes,yaxes)
         y2=yaxes(1,k)*yaxes(1,k)+yaxes(2,k)*yaxes(2,k)+
     +        yaxes(3,k)*yaxes(3,k)
         leny(k)=sqrt(y2)
      enddo
      
c      
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set the coefficients for the switching function
c
      call switch ('VDW')
c
c     switch from exponential to R^12 at very short range
c
c      expcut = 2.0d0
c      expcut2 = expcut * expcut
c      expmerge = (aterm*exp(-bterm/expcut) - cterm*(expcut**6))
c     &                               / (expcut**12)
c
c     calculate the "reduced" atomic coordinates
c
      do k = 1, nvdw
         i = ivdw(k)
         iv = ired(i)
         rdn = kred(i)
         xred(i) = rdn*(x(i)-x(iv)) + x(iv)
         yred(i) = rdn*(y(i)-y(iv)) + y(iv)
         zred(i) = rdn*(z(i)-z(iv)) + z(iv)
      end do

c
c     find van der Waals energy via double loop
c
      do ii = 1, nvdw-1
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
c No-reduced coordinates for disp
         xinr = x(i)
         yinr = y(i)
         zinr = z(i)
c
         do j = 1, n12(i)
            skip(i12(j,i)) = i * vdw12use
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i * vdw13use
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = i * vdw14use
         end do
         do kk = ii+1, nvdw
            k = ivdw(kk)
            kv = ired(k)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
            if (proceed)  proceed = (skip(k) .ne. i)
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               kt = class(k)
               xr = xi - xred(k)
               yr = yi - yred(k)
               zr = zi - zred(k)
               if (use_image) call image (xr,yr,zr,0)
               rik2 = xr*xr + yr*yr + zr*zr
c
c     compute energy for this interaction
c
               if (rik2 .le. off2) then
c No reduded coord. for disp
                  rnr(1) = xinr - x(k)
                  rnr(2) = yinr - y(k)
                  rnr(3) = zinr - z(k)
                  call image (rnr(1),rnr(2),rnr(3),0)
                  rik2nr=rnr(1)*rnr(1) + rnr(2)*rnr(2) + rnr(3)*rnr(3)
                  rik6nr = rik2nr*rik2nr*rik2nr
                  riknr=sqrt(rik2nr)
c
                  A=Ause(kt,it)
                  B=Buse(kt,it)
                  C=Cuse(kt,it)
c                  rik6 = rik2 * rik2 * rik2
                  rik = sqrt(rik2)
c NEW
                  r(1)=xr
                  r(2)=yr
                  r(3)=zr
                  f=1.0d0
                  scale=vdwradius(it)+vdwradius(kt)
                  if (scale.ne.0.0d0) then 
                     term = - (cdamp*riknr/scale)**3
                     if (term .gt. -50.0d0) then
                        term = exp(term)
                        f = (1.0d0 - term)**2
                     endif
                  endif
                  
c
c Anisotropic repulsion part
c
                  if (aniso(it).ne.0.0d0 .or. aniso(kt).ne.0.0d0) then
c Ook al is aniso=0. 0x iets kan blijkbaar toch verkeerd uitpakken:
                     yir2=0.0d0
                     ykr2=0.0d0
                     if (aniso(it).ne.0.0d0) then
                        iii=nraniso(i)               
                        ryi=rik*leny(iii)
                        yir=(r(1)*yaxes(1,iii)+r(2)*yaxes(2,iii)+
     +                       r(3)*yaxes(3,iii))
                        yir2=(yir/ryi)**2
                     endif

                     if (aniso(kt).ne.0.0d0) then
                        kkk=nraniso(k)
                        ryk=rik*leny(kkk)
                        ykr=(r(1)*yaxes(1,kkk)+r(2)*yaxes(2,kkk)+
     +                       r(3)*yaxes(3,kkk))
                        ykr2=(ykr/ryk)**2
                     endif

                     e =A*exp(-B*(rik-yir2*aniso(it)-ykr2*aniso(kt)))
     +                    - f*C/rik6nr
                  else 
                     e = A* exp(-b*rik) - f*C/rik6nr
                  endif
                  if (skip(k) .eq. -i)  then
                     e = e / vdwscale
                  endif
c
c     use energy switching if near the cutoff distance
c
c
c !!!                This has to be adapted:  
c
                  
                  if (rik2 .gt. cut2) then
                     rik3 = rik2 * rik
                     rik4 = rik2 * rik2
                     rik5 = rik2 * rik3
                     taper = c5*rik5 + c4*rik4 + c3*rik3
     &                          + c2*rik2 + c1*rik + c0
                  end if
c ???
c
c     scale the interaction based on its group membership
c
                  if (use_group) then
                     e = e * fgrp
                  end if
c
c     increment the total van der Waals energy and derivatives
c
                  if (skip(k) .eq. -i) then
                     e14 = e14 + e
                  else
                     ev = ev + e
                  end if
c
c     increment the total intermolecular energy
c
                  if (molcule(i) .ne. molcule(k)) then
                     einter = einter + e
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, nvdw
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
c No-reduced coordinates for disp
         xinr = x(i)
         yinr = y(i)
         zinr = z(i)
         do kk = ii, nvdw
            k = ivdw(kk)
            kv = ired(k)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               do j = 1, ncell
                  kt = class(k)
                  xr = xi - xred(k)
                  yr = yi - yred(k)
                  zr = zi - zred(k)
                  call image (xr,yr,zr,j)
                  rik2 = xr*xr + yr*yr + zr*zr
c
c     compute energy and derivatives for this interaction
c
                  if (rik2 .le. off2) then

                     A=Ause(kt,it)
                     B=Buse(kt,it)
                     C=Cuse(kt,it)
                     rik6 = rik2 * rik2 * rik2
                     rik = sqrt(rik2)
c No reduded coord. for disp
                     rnr(1) = xinr - x(k)
                     rnr(2) = yinr - y(k)
                     rnr(3) = zinr - z(k)
                     call image (rnr(1),rnr(2),rnr(3),j)
                     rik2nr=rnr(1)*rnr(1)+rnr(2)*rnr(2) + rnr(3)*rnr(3)
                     rik6nr = rik2nr*rik2nr*rik2nr
                     riknr=sqrt(rik2nr)
c NEW
                     r(1)=xr
                     r(2)=yr
                     r(3)=zr
                     f=1.0d0
                     scale=vdwradius(it)+vdwradius(kt)
                     if (scale.ne.0.0d0) then 
                        term = - (cdamp*riknr/scale)**3
                        if  (term .gt. -50.0d0) then
                           term = exp(term)
                           f = (1.0d0 - term)**2
                        endif
                     endif
                  
c
c Anisotropic repulsion part
c
                     if (aniso(it).ne.0.0d0 .or. 
     +                    aniso(kt).ne.0.0d0) then
c Ook al is aniso=0. 0x iets kan blijkbaar toch verkeerd uitpakken:
                        yir2=0.0d0
                        ykr2=0.0d0
                        if (aniso(it).ne.0.0d0) then
                           iii=nraniso(i)               
                           ryi=rik*leny(iii)
                           yir=(r(1)*yaxes(1,iii)+r(2)*yaxes(2,iii)+
     +                          r(3)*yaxes(3,iii))
                           yir2=(yir/ryi)**2
                        endif 

                        if (aniso(kt).ne.0.0d0) then
                           kkk=nraniso(k)
                           ryk=rik*leny(kkk)
                           ykr=(r(1)*yaxes(1,kkk)+r(2)*yaxes(2,kkk)+
     +                          r(3)*yaxes(3,kkk))
                           ykr2=(ykr/ryk)**2
                        endif
                        
                        e =A*exp(-B*(rik-yir2*aniso(it)-ykr2*aniso(kt)))
     +                       - f*C/rik6nr
                     else 
                        e = A* exp(-b*rik) - f*C/rik6nr
                     endif
c                     
c     use energy switching if near the cutoff distance
c
c
c !!!                This has to be adapted:  
c
                  
                     if (rik2 .gt. cut2) then
                        rik3 = rik2 * rik
                        rik4 = rik2 * rik2
                        rik5 = rik2 * rik3
                        taper = c5*rik5 + c4*rik4 + c3*rik3
     &                       + c2*rik2 + c1*rik + c0
                     end if
c ???
c
c     scale the interaction based on its group membership
c
                     if (use_group) then
                        e = e * fgrp
                     end if 
c
c     increment the total van der Waals energy and derivatives
c
                     if (i .eq. k) e = 0.5d0 * e
                     ev  = ev + e
c
c     increment the total intermolecular energy
c
                     einter = einter + e
                  end if
               end do
            end if
         end do
      end do
      return
      end
