c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine ewilliams1  --  Buckingham energy & derivatives  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "ebuck1" calculates the van der Waals energy and its first
c     derivatives with respect to Cartesian coordinates using
c     the Buckingham exp-6 formalism, in the A exp(-b r)-c/r6 form
c      
c Including anisotropic repulsion
c      this version does not use reduced coordinates for dispersion.
c
c The dispersion damping and the anisotropic repulsion:
c W.T.M. Mooij, B.P. van Eijck and F.B. van Duijneveldt, An ab initio
c intermolecular potential for alcohols, manuscript in preparation
c
c Viral yet to be done !
c Energy shifting not implemented
c
      subroutine ewilliams1
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'deriv.i'
      include 'energi.i'
      include 'group.i'
      include 'inter.i'
      include 'molcul.i'
      include 'shunt.i'
      include 'usage.i'
      include 'vdw.i'
      include 'vdw2.i'
      include 'vdwpot.i'
      include 'virial.i'
      include 'aniso.i'
      include 'mpole.i'


      integer i,j,k,ii,kk,iv,kv
      integer it,kt,skip(maxatm)
      real*8 e,rv,eps,rdn
      real*8 p,p2,p6,p12,fgrp
      real*8 redi,rediv,redk,redkv,rvterm
      real*8 dedx,dedy,dedz
      real*8 rik,rik2,rik3,rik4,rik5,taper,dtaper
      real*8 expcut,expcut2,expterm,expmerge
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      logical proceed,iuse
      real*8 a,b,c,rik6,rik7
      real*8 xiz,yiz,ziz,xix,yix,zix
      real*8 xkz,ykz,zkz,xkx,ykx,zkx
      real*8 xi,yi,zi,xr,yr,zr


      real*8 yi2,yi3,yk,yk2,yk3,yir2,ykr2,yir,
     +     ykr,dedr,dedani,dedank,danidi(3),danidk(3),dankdi(3),
     +     dankdk(3),danidix(3),danidiz(3),dankdkx(3),dankdkz(3),r(3),
     +     rizx(3),riz(3),rix(3),rkzx(3),rkz(3),rkx(3),
     +     ryi,ryk,ryi3,ryk3,r3yi,r3yk,de(3,6)

      
      logical ani_i,ani_k
      real*8 f,dfdr,term
      integer itest,iii,kkk
      real*8 scale,y2,yi1,yk1
      integer i1,i2,ix,iz,kx,kz
      real*8 xinr,yinr,zinr,rnr(3),riknr,rik2nr,rik6nr,rik7nr,ddispdr

c
c
c     zero out the van der Waals energy and first derivatives
c
      ev = 0.0d0
      e14 = 0.0d0
      do i = 1, n
         dev(1,i) = 0.0d0
         dev(2,i) = 0.0d0
         dev(3,i) = 0.0d0
         de14(1,i) = 0.0d0
         de14(2,i) = 0.0d0
         de14(3,i) = 0.0d0
      end do
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
      
      
c
c Calculate y-axis orientation for anisotropic repulsion term
c
      do k = 1, naniso
         i = ianiso(k)
         if (ipole(i).ne.i) then
            print*,'Multipoles on all atoms necessary',
     +           ' for anisotropic repulsion'
            call fatal
         endif
         zaxes(1,k) = x(zaxis(i)) - x(i)
         zaxes(2,k) = y(zaxis(i)) - y(i)
         zaxes(3,k) = z(zaxis(i)) - z(i)
         xaxes(1,k) = x(xaxis(i)) - x(i)
         xaxes(2,k) = y(xaxis(i)) - y(i)
         xaxes(3,k) = z(xaxis(i)) - z(i)
         call prod2(k,zaxes,xaxes,yaxes)
         y2=yaxes(1,k)*yaxes(1,k)+yaxes(2,k)*yaxes(2,k)+
     +        yaxes(3,k)*yaxes(3,k)
         leny(k)=sqrt(y2)
         leny3(k)=leny(k)*y2
         call prod2(k,yaxes,zaxes,ycrossz)
         call prod2(k,yaxes,xaxes,ycrossx)
         zminx(1,k)=x(zaxis(i)) - x(xaxis(i))
         zminx(2,k)=y(zaxis(i)) - y(xaxis(i))
         zminx(3,k)=z(zaxis(i)) - z(xaxis(i))
         call prod2(k,yaxes,zminx,ycrosszx)
      enddo
      
c
c     set the coefficients for the switching function
c
      call switch ('VDW')
c
c     switch from exponential to R^12 at very short range
c
c      expcut = 2.0d0
c      expcut2 = expcut * expcut
c      expmerge = (aterm*exp(-bterm/expcut) - cterm*(expcut**6))
c     &                               / (expcut**12)
c
c     calculate the "reduced" atomic coordinates
c
      do k = 1, nvdw
         i = ivdw(k)
         iv = ired(i)
         rdn = kred(i)
         xred(i) = rdn*(x(i)-x(iv)) + x(iv)
         yred(i) = rdn*(y(i)-y(iv)) + y(iv)
         zred(i) = rdn*(z(i)-z(iv)) + z(iv)
      end do
c
c     find van der Waals energy and derivatives via double loop
c
      do ii = 1, nvdw-1
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
c No-reduced coordinates for disp
         xinr = x(i)
         yinr = y(i)
         zinr = z(i)
c
         do j = 1, n12(i)
            skip(i12(j,i)) = i * vdw12use
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i * vdw13use
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = i * vdw14use
         end do
         do kk = ii+1, nvdw
            k = ivdw(kk)
            kv = ired(k)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
            if (proceed)  proceed = (skip(k) .ne. i)
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               kt = class(k)
               xr = xi - xred(k)
               yr = yi - yred(k)
               zr = zi - zred(k)
               if (use_image) call image (xr,yr,zr,0)
               rik2 = xr*xr + yr*yr + zr*zr
c
c     compute energy and derivatives for this interaction
c
               if (rik2 .le. off2) then
c No reduced coord. for disp
                  rnr(1) = xinr - x(k)
                  rnr(2) = yinr - y(k)
                  rnr(3) = zinr - z(k) 
                  if (use_image) call image (rnr(1),rnr(2),rnr(3),0)
                  rik2nr=rnr(1)*rnr(1) + rnr(2)*rnr(2) + rnr(3)*rnr(3)
                  rik6nr = rik2nr*rik2nr*rik2nr
                  riknr=sqrt(rik2nr)
                  rik7nr=rik6nr*riknr
c
                  A=Ause(kt,it)
                  B=Buse(kt,it)
                  C=Cuse(kt,it)
c                  rik6 = rik2 * rik2 * rik2
                  rik = sqrt(rik2)
c                  rik7 = rik6 * rik
c NEW
                  r(1) = xr
                  r(2) = yr
                  r(3) = zr
                  iz = zaxis(i)
                  ix = xaxis(i)
                  kz = zaxis(k)
                  kx = xaxis(k)
                  ani_i=.false.
                  ani_k=.false.
                  f=1.0d0
                  dfdr=0.0d0
                  scale=vdwradius(it)+vdwradius(kt)
                  if (scale.ne.0.0d0) then 
                     term = - (cdamp*riknr/scale)**3
                     if (term .gt. -50.0d0) then
                        term = exp(term)
                        f = (1.0d0 - term)**2
                        dfdr = 2.0d0*(1.0d0 - term) * 
     +                       (3.0d0 * (cdamp/scale)**3 * rik2nr)
     +                       * term
                     endif
                  endif
                  
c
c Anisotropic repulsion part
c
                  if (aniso(it).ne.0.0d0 .or. aniso(kt).ne.0.0d0) then
                     rik3=rik2*rik
c Ook al is aniso=0. 0x iets kan blijkbaar toch verkeerd uitpakken:
                     yir2=0.0d0
                     ykr2=0.0d0
                     if (aniso(it).ne.0.0d0) then
                        ani_i=.true.
                        iii=nraniso(i)               
                        call prod3(iii,r,zminx,rizx)
                        call prod3(iii,r,zaxes,riz)
                        call prod3(iii,r,xaxes,rix)
                        yi1=leny(iii)
                        yi3=leny3(iii)
                        ryi=rik*yi1
                        ryi3=rik*yi3
                        r3yi=rik3*yi1
                        yir=(r(1)*yaxes(1,iii)+r(2)*yaxes(2,iii)+
     +                       r(3)*yaxes(3,iii))
                        yir2=(yir/ryi)**2
                     endif

                     if (aniso(kt).ne.0.0d0) then
                        ani_k=.true.
                        kkk=nraniso(k)
                        call prod3(kkk,r,zminx,rkzx)
                        call prod3(kkk,r,zaxes,rkz)
                        call prod3(kkk,r,xaxes,rkx)
                        yk1=leny(kkk)
                        yk3=leny3(kkk)
                        ryk=rik*yk1
                        ryk3=rik*yk3
                        r3yk=rik3*yk1
                        ykr=(r(1)*yaxes(1,kkk)+r(2)*yaxes(2,kkk)+
     +                       r(3)*yaxes(3,kkk))
                        ykr2=(ykr/ryk)**2
                     endif

                     e =A* exp(-B*(rik-yir2*aniso(it)-ykr2*aniso(kt)))
                     dedr = -a * b * exp(-b*(rik-yir2*aniso(it)-
     +                    ykr2*aniso(kt))) 
c           dph/di(x)
                     if (ani_i) then
                        dedani = 2.0d0*A*aniso(it)*b*
     +                       exp(-b*(rik-yir2*aniso(it)-
     +                       ykr2*aniso(kt)))*yir/(rik*yi1)
                        do i1=1,3
                           danidi(i1) = ycrosszx(i1,iii)*yir/(ryi3) - 
     +                          (rizx(i1)-yaxes(i1,iii))/(ryi) - 
     +                          r(i1)*yir/(r3yi)
                           danidk(i1) =  -yaxes(i1,iii)/(ryi) + 
     +                          r(i1)*yir/(r3yi)
                           danidix(i1) = -ycrossz(i1,iii)*yir/(ryi3) +
     +                          riz(i1)/(ryi)
                           danidiz(i1) =  ycrossx(i1,iii)*yir/(ryi3) - 
     +                          rix(i1)/(ryi)
                        enddo
                     else 
                        dedani = 0.0d0
                     endif

                     if (ani_k) then
                        dedank = 2.0d0*A*aniso(kt)*b*
     +                       exp(-b*(rik-yir2*aniso(it)-
     +                       ykr2*aniso(kt)))*ykr/(rik*yk1)               
                        do i1=1,3
                           dankdk(i1) = ycrosszx(i1,kkk)*ykr/(ryk3) -
     +                          (rkzx(i1)+yaxes(i1,kkk))/(ryk) +
     +                          r(i1)*ykr/(r3yk)
                           dankdi(i1) = yaxes(i1,kkk)/(ryk) - 
     +                          r(i1)*ykr/(r3yk)
                           dankdkx(i1) = -ycrossz(i1,kkk)*ykr/(ryk3) + 
     +                          rkz(i1)/(ryk)
                           dankdkz(i1) =  ycrossx(i1,kkk)*ykr/(ryk3) - 
     +                          rkx(i1)/(ryk)
                        enddo
                     else 
                        dedank =0.0d0
                     endif
                     dedr = dedr/rik
                     do i1=1,3
                        de(i1,1)= dedr*r(i1) + dedani*danidi(i1)+
     +                       dedank*dankdi(i1)
                        de(i1,6)= -dedr*r(i1) +dedank*dankdk(i1)+
     +                       dedani*danidk(i1)
                        de(i1,2)= dedani*danidiz(i1)
                        de(i1,3)= dedani*danidix(i1)
                        de(i1,4)= dedank*dankdkz(i1)
                        de(i1,5)= dedank*dankdkx(i1)
                     enddo 
c              Dispersion part:
                     e = e - f*C/rik6nr
                     ddispdr = 6.0d0*f*C/rik7nr - dfdr*C/rik6nr
                     ddispdr= ddispdr/riknr
                  else 
                     e = A* exp(-b*rik) - f*C/rik6nr
                     dedr = -(a * b) * exp(-b*rik) 
                     ddispdr = 6.0d0*f*C/rik7nr - dfdr*C/rik6nr
                     dedr =dedr/rik
                     ddispdr =ddispdr/riknr
                     do i1=1,3
                        de(i1,1)=dedr*r(i1)
                        de(i1,6)=-de(i1,1)
                     enddo
                  endif
                  
                  if (skip(k) .eq. -i)  then
                     e = e / vdwscale
c
                     dedr = dedr /vdwscale
                     ddispdr=ddispdr/vdwscale
                     dedank = dedank/vdwscale
                     dedani = dedani/vdwscale
c
                     do i1=1,6
                        do i2=1,3
                           de(i2,i1) = de(i2,i1)/vdwscale
                        enddo
                     enddo
                  endif
c
c     use energy switching if near the cutoff distance
c
c
c !!!                This has to be adapted:  
c
                  
                  if (rik2 .gt. cut2) then
                     rik3 = rik2 * rik
                     rik4 = rik2 * rik2
                     rik5 = rik2 * rik3
                     taper = c5*rik5 + c4*rik4 + c3*rik3
     &                          + c2*rik2 + c1*rik + c0
                     dtaper = 5.0d0*c5*rik4 + 4.0d0*c4*rik3
     &                           + 3.0d0*c3*rik2 + 2.0d0*c2*rik + c1
                     do i1=1,6
                        do i2=1,3
                           de(i2,i1) = e*dtaper + de(i2,i1)*taper
                        enddo
                     enddo
                  end if
c ???
c
c     scale the interaction based on its group membership
c
                  if (use_group) then
                     e = e * fgrp
                     dedr =dedr * fgrp
                     ddispdr = ddispdr * fgrp
                     do i1=1,6
                        do i2=1,3
                           de(i2,i1) = de(i2,i1)*fgrp
                        enddo
                     enddo
                  end if
c
c     increment the total van der Waals energy and derivatives
c
                  if (skip(k) .eq. -i) then
                     e14 = e14 + e
                     if (i .eq. iv) then
                        do i1=1,3
                           de14(i1,i) = de14(i1,i) + de(i1,1)
                        enddo
                        if (ani_i) then
                           do i1=1,3
                              de14(i1,iz) = de14(i1,iz) + de(i1,2)
                              de14(i1,ix) = de14(i1,ix) + de(i1,3)
                           enddo
                        endif
                     else
                        do i1=1,3
                           de14(i1,i) = de14(i1,i) + de(i1,1)*redi
                           de14(i1,iv) = de14(i1,iv) + de(i1,1)*rediv
                        enddo
                     end if
                     if (k .eq. kv) then
                        do i1=1,3
                           de14(i1,k) = de14(i1,k) + de(i1,6)
                        enddo
                        if (ani_k) then
                           do i1=1,3
                              de14(i1,kz) = de14(i1,kz) + de(i1,4)
                              de14(i1,kx) = de14(i1,kx) + de(i1,5)
                           enddo
                        endif
                     else
                        redk = kred(k)
                        redkv = 1.0d0 - redk
                        do i1=1,3
                           de14(i1,k) = de14(i1,k) + de(i1,6)*redk
                           de14(i1,kv) = de14(i1,kv) + de(i1,6)*redkv
                        enddo
                     end if
c Dispersion part: 
                     do i1=1,3
                        de14(i1,i) = de14(i1,i) + ddispdr*rnr(i1)
                        de14(i1,k) = de14(i1,k) - ddispdr*rnr(i1)
                     enddo

                  else
                     ev = ev + e
                     if (i .eq. iv) then
                        do i1=1,3
                           dev(i1,i) = dev(i1,i) + de(i1,1)
                        enddo
                        if (ani_i) then
                           do i1=1,3
                              dev(i1,iz) = dev(i1,iz) + de(i1,2)
                              dev(i1,ix) = dev(i1,ix) + de(i1,3)
                           enddo
                        endif
                     else
                        do i1=1,3
                           dev(i1,i) = dev(i1,i) + de(i1,1)*redi
                           dev(i1,iv) = dev(i1,iv) + de(i1,1)*rediv
                        enddo
                     end if
                     if (k .eq. kv) then
                        do i1=1,3
                           dev(i1,k) = dev(i1,k) + de(i1,6)
                        enddo
                        if (ani_k) then
                           do i1=1,3
                              dev(i1,kz) = dev(i1,kz) + de(i1,4)
                              dev(i1,kx) = dev(i1,kx) + de(i1,5)
                           enddo
                        endif
                     else
                        redk = kred(k)
                        redkv = 1.0d0 - redk
                        do i1=1,3
                           dev(i1,k) = dev(i1,k) + de(i1,6)*redk
                           dev(i1,kv) = dev(i1,kv) + de(i1,6)*redkv
                        enddo
                     end if
c Dispersion part: 
                     do i1=1,3
                        dev(i1,i) = dev(i1,i) + ddispdr*rnr(i1)
                        dev(i1,k) = dev(i1,k) - ddispdr*rnr(i1)
                     enddo
                     
                  end if
c
c     increment the total intermolecular energy
c
                  if (molcule(i) .ne. molcule(k)) then
                     einter = einter + e
                  end if
c
c     increment the virial for use in pressure computation
c
                  if (isobaric) then
                     if (ani_i) then
                        xiz = x(iz) - x(i)
                        yiz = y(iz) - y(i)
                        ziz = z(iz) - z(i)
                        xix = x(ix) - x(i)
                        yix = y(ix) - y(i)
                        zix = z(ix) - z(i)
                        virx = virx + xiz*de(1,2)+xix*de(1,3)+
     +                       xr*dedani*danidk(1)
                        viry = viry + yiz*de(2,2)+yix*de(2,3)+
     +                       yr*dedani*danidk(2)
                        virz = virz + ziz*de(3,2)+zix*de(3,3)+
     +                       zr*dedani*danidk(3)
                     endif
                     if (ani_k) then
                        xkz = x(kz) - x(k)
                        ykz = y(kz) - y(k)
                        zkz = z(kz) - z(k)
                        xkx = x(kx) - x(k)
                        ykx = y(kx) - y(k)
                        zkx = z(kx) - z(k)
                        virx = virx + xkz*de(1,4) + xkx*de(1,5)+
     +                       xr*dedank*dankdi(1)
                        viry = viry + ykz*de(2,4) + ykx*de(2,5)+
     +                       yr*dedank*dankdi(2)
                        virz = virz + zkz*de(3,4) + zkx*de(3,5)+
     +                       zr*dedank*dankdi(3)
                     endif
                     virx = virx + xr*dedr*xr + rnr(1)*ddispdr*rnr(1)
                     viry = viry + yr*dedr*yr + rnr(2)*ddispdr*rnr(2)
                     virz = virz + zr*dedr*zr + rnr(3)*ddispdr*rnr(3)
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, nvdw
         i = ivdw(ii)
         iv = ired(i)
         iuse = (use(i) .or. use(iv))
         redi = kred(i)
         rediv = 1.0d0 - redi
         it = class(i)
         xi = xred(i)
         yi = yred(i)
         zi = zred(i)
c No-reduced coordinates for disp
         xinr = x(i)
         yinr = y(i)
         zinr = z(i)
c     
         do kk = ii, nvdw
            k = ivdw(kk)
            kv = ired(k)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k) .or. use(kv))
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               do j = 1, ncell
                  kt = class(k)
                  xr = xi - xred(k)
                  yr = yi - yred(k)
                  zr = zi - zred(k)
                  call image (xr,yr,zr,j)
                  rik2 = xr*xr + yr*yr + zr*zr

c
c     compute energy and derivatives for this interaction
c
                  if (rik2 .le. off2) then
                     A=Ause(kt,it)
                     B=Buse(kt,it)
                     C=Cuse(kt,it)
c                     rik6 = rik2 * rik2 * rik2
                     rik = sqrt(rik2)
c                     rik7 = rik6 * rik
c No reduded coord. for disp
                     rnr(1) = xinr - x(k)
                     rnr(2) = yinr - y(k)
                     rnr(3) = zinr - z(k)
                     call image (rnr(1),rnr(2),rnr(3),j)
                     rik2nr=rnr(1)*rnr(1)+rnr(2)*rnr(2) + rnr(3)*rnr(3)
                     rik6nr = rik2nr*rik2nr*rik2nr
                     riknr=sqrt(rik2nr)
                     rik7nr=rik6nr*riknr
c NEW
                     r(1) = xr
                     r(2) = yr
                     r(3) = zr
                     iz = zaxis(i)
                     ix = xaxis(i)
                     kz = zaxis(k)
                     kx = xaxis(k)
                     ani_i=.false.
                     ani_k=.false.
                     f=1.0d0
                     dfdr=0.0d0
                     scale=vdwradius(it)+vdwradius(kt)
                     if (scale.ne.0.0d0) then 
                        term = - (cdamp*riknr/scale)**3
                        if  (term .gt. -50.0d0) then
                           term = exp(term)
                           f = (1.0d0 - term)**2
                           dfdr = 2.0d0*(1.0d0 - term) * 
     +                          (3.0d0 * (cdamp/scale)**3 * rik2nr)
     +                          * term
                        endif
                     endif
                  
c
c Anisotropic repulsion part
c
                     if (aniso(it).ne.0.0d0 .or. 
     +                    aniso(kt).ne.0.0d0) then
                        rik3=rik2*rik
c Ook al is aniso=0. 0x iets kan blijkbaar toch verkeerd uitpakken:
                        yir2=0.0d0
                        ykr2=0.0d0
                        if (aniso(it).ne.0.0d0) then
                           ani_i=.true.
                           iii=nraniso(i)               
                           call prod3(iii,r,zminx,rizx)
                           call prod3(iii,r,zaxes,riz)
                           call prod3(iii,r,xaxes,rix)
                           yi1=leny(iii)
                           yi3=leny3(iii)
                           ryi=rik*yi1
                           ryi3=rik*yi3
                           r3yi=rik3*yi1
                           yir=(r(1)*yaxes(1,iii)+r(2)*yaxes(2,iii)+
     +                          r(3)*yaxes(3,iii))
                           yir2=(yir/ryi)**2
                        endif 

                        if (aniso(kt).ne.0.0d0) then
                           ani_k=.true.
                           kkk=nraniso(k)
                           call prod3(kkk,r,zminx,rkzx)
                           call prod3(kkk,r,zaxes,rkz)
                           call prod3(kkk,r,xaxes,rkx)
                           yk1=leny(kkk)
                           yk3=leny3(kkk)
                           ryk=rik*yk1
                           ryk3=rik*yk3
                           r3yk=rik3*yk1
                           ykr=(r(1)*yaxes(1,kkk)+r(2)*yaxes(2,kkk)+
     +                          r(3)*yaxes(3,kkk))
                           ykr2=(ykr/ryk)**2
                        endif
                        
                        e =A*exp(-B*(rik-yir2*aniso(it)-ykr2*aniso(kt)))
                        dedr = -a * b * exp(-b*(rik-yir2*aniso(it)-
     +                       ykr2*aniso(kt))) 
c           dph/di(x)
                        if (ani_i) then
                           dedani = 2.0d0*A*aniso(it)*b*
     +                          exp(-b*(rik-yir2*aniso(it)-
     +                          ykr2*aniso(kt)))*yir/(rik*yi1)
                           do i1=1,3
                              danidi(i1) = ycrosszx(i1,iii)*yir/(ryi3) - 
     +                             (rizx(i1)-yaxes(i1,iii))/(ryi) - 
     +                             r(i1)*yir/(r3yi)
                              danidk(i1) =  -yaxes(i1,iii)/(ryi) + 
     +                             r(i1)*yir/(r3yi)
                              danidix(i1) = -ycrossz(i1,iii)*yir/(ryi3)+
     +                             riz(i1)/(ryi)
                              danidiz(i1) =  ycrossx(i1,iii)*yir/(ryi3)- 
     +                             rix(i1)/(ryi)
                           enddo
                        else 
                           dedani = 0.0d0
                        endif

                        if (ani_k) then
                           dedank = 2.0d0*A*aniso(kt)*b*
     +                          exp(-b*(rik-yir2*aniso(it)-
     +                          ykr2*aniso(kt)))*ykr/(rik*yk1)               
                           do i1=1,3
                              dankdk(i1) = ycrosszx(i1,kkk)*ykr/(ryk3) -
     +                             (rkzx(i1)+yaxes(i1,kkk))/(ryk) +
     +                             r(i1)*ykr/(r3yk)
                              dankdi(i1) = yaxes(i1,kkk)/(ryk) - 
     +                             r(i1)*ykr/(r3yk)
                              dankdkx(i1) = -ycrossz(i1,kkk)*ykr/(ryk3)+ 
     +                             rkz(i1)/(ryk)
                              dankdkz(i1) =  ycrossx(i1,kkk)*ykr/(ryk3)- 
     +                             rkx(i1)/(ryk)
                           enddo
                        else 
                           dedank =0.0d0
                        endif
                        dedr=dedr/rik
                        do i1=1,3
                           de(i1,1)= dedr*r(i1) + dedani*danidi(i1)+
     +                          dedank*dankdi(i1)
                           de(i1,6)= -dedr*r(i1) +dedank*dankdk(i1)+
     +                          dedani*danidk(i1)
                           de(i1,2)= dedani*danidiz(i1)
                           de(i1,3)= dedani*danidix(i1)
                           de(i1,4)= dedank*dankdkz(i1)
                           de(i1,5)= dedank*dankdkx(i1)
                        enddo 
c              Dispersion part:
                        e = e - f*C/rik6nr
                        ddispdr = 6.0d0*f*C/rik7nr - dfdr*C/rik6nr
                        ddispdr=ddispdr/riknr
                     else 
                        e = A* exp(-b*rik) - f*C/rik6nr
                        dedr = -(a * b) * exp(-b*rik) 
                        ddispdr = 6.0d0*f*C/rik7nr - dfdr*C/rik6nr
                        dedr =dedr/rik
                        ddispdr =ddispdr/riknr
                        do i1=1,3
                           de(i1,1)=dedr*r(i1)
                           de(i1,6)=-de(i1,1)
                        enddo
                     endif

c                     
c     use energy switching if near the cutoff distance
c
c
c !!!                This has to be adapted:  
c
                  
                     if (rik2 .gt. cut2) then
                        rik3 = rik2 * rik
                        rik4 = rik2 * rik2
                        rik5 = rik2 * rik3
                        taper = c5*rik5 + c4*rik4 + c3*rik3
     &                       + c2*rik2 + c1*rik + c0
                        dtaper = 5.0d0*c5*rik4 + 4.0d0*c4*rik3
     &                       + 3.0d0*c3*rik2 + 2.0d0*c2
     +                       *rik + c1
                        do i1=1,6
                           do i2=1,3
                              de(i2,i1) = e*dtaper + de(i2,i1)*taper
                           enddo
                        enddo
                     end if
c ???
c
c     scale the interaction based on its group membership
c
                     if (use_group) then
                        e = e * fgrp
                        dedr = dedr * fgrp
                        ddispdr = ddispdr * fgrp
                        do i1=1,6
                           do i2=1,3
                              de(i2,i1) = de(i2,i1)*fgrp
                           enddo 
                        enddo 
                     end if 
c
c     increment the total van der Waals energy and derivatives
c
                     if (i .eq. k)  e = 0.5d0 * e
                     ev  = ev + e
                     if (i .eq. iv) then
                        do i1=1,3
                           dev(i1,i) = dev(i1,i) + de(i1,1)
                        enddo
                        if (ani_i) then
                           do i1=1,3
                              dev(i1,iz) = dev(i1,iz) + de(i1,2)
                              dev(i1,ix) = dev(i1,ix) + de(i1,3)
                           enddo
                        endif
                     else
                        do i1=1,3
                           dev(i1,i) = dev(i1,i) + de(i1,1)*redi
                           dev(i1,iv) = dev(i1,iv) + de(i1,1)*rediv
                        enddo
                     end if
                     if (i.ne.k) then
                        if (k .eq. kv) then
                           do i1=1,3
                              dev(i1,k) = dev(i1,k) + de(i1,6)
                           enddo
                           if (ani_k) then
                              do i1=1,3
                                 dev(i1,kz) = dev(i1,kz) + de(i1,4)
                                 dev(i1,kx) = dev(i1,kx) + de(i1,5)
                              enddo
                           endif
                        else
                           redk = kred(k)
                           redkv = 1.0d0 - redk
                           do i1=1,3
                              dev(i1,k) = dev(i1,k) + de(i1,6)*redk
                              dev(i1,kv) = dev(i1,kv) + de(i1,6)*redkv
                           enddo
                        end if
                     end if
c Dispersion part: 
                     do i1=1,3
                        dev(i1,i) = dev(i1,i) + ddispdr*rnr(i1)
                        dev(i1,k) = dev(i1,k) - ddispdr*rnr(i1)
                     enddo
c
c     increment the total intermolecular energy
c
                     einter = einter + e
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        if (ani_i) then
                           xiz = x(iz) - x(i)
                           yiz = y(iz) - y(i)
                           ziz = z(iz) - z(i)
                           xix = x(ix) - x(i)
                           yix = y(ix) - y(i)
                           zix = z(ix) - z(i)
                           virx = virx + xiz*de(1,2)+xix*de(1,3)+
     +                          xr*dedani*danidk(1)
                           viry = viry + yiz*de(2,2)+yix*de(2,3)+
     +                          yr*dedani*danidk(2)
                           virz = virz + ziz*de(3,2)+zix*de(3,3)+
     +                          zr*dedani*danidk(3)
                        endif
                        if (ani_k) then
                           xkz = x(kz) - x(k)
                           ykz = y(kz) - y(k)
                           zkz = z(kz) - z(k)
                           xkx = x(kx) - x(k)
                           ykx = y(kx) - y(k)
                           zkx = z(kx) - z(k)
                           virx = virx + xkz*de(1,4) + xkx*de(1,5)+
     +                          xr*dedank*dankdi(1)
                           viry = viry + ykz*de(2,4) + ykx*de(2,5)+
     +                          yr*dedank*dankdi(2)
                           virz = virz + zkz*de(3,4) + zkx*de(3,5)+
     +                          zr*dedank*dankdi(3)
                        end if
                        virx = virx + xr*dedr*xr + rnr(1)*ddispdr*rnr(1)
                        viry = viry + yr*dedr*yr + rnr(2)*ddispdr*rnr(2)
                        virz = virz + zr*dedr*zr + rnr(3)*ddispdr*rnr(3)
                     end if
                  endif
               end do
            end if
         end do
      end do
      return
      end
      
      subroutine prod(v1,v2,v3)
      implicit none
      real*8 v1(3),v2(3),v3(3)

      v3(1)=v1(2)*v2(3)-v1(3)*v2(2)
      v3(2)=v1(3)*v2(1)-v1(1)*v2(3)
      v3(3)=v1(1)*v2(2)-v1(2)*v2(1)    
      end

      subroutine prod2(k,v1,v2,v3)
      implicit none
      real*8 v1(3,*),v2(3,*),v3(3,*)
      integer k
      v3(1,k)=v1(2,k)*v2(3,k)-v1(3,k)*v2(2,k)
      v3(2,k)=v1(3,k)*v2(1,k)-v1(1,k)*v2(3,k)
      v3(3,k)=v1(1,k)*v2(2,k)-v1(2,k)*v2(1,k)    
      end
      
      
      subroutine prod3(k,v1,v2,v3)
      implicit none
      real*8 v1(3),v2(3,*),v3(3)
      integer k
      v3(1)=v1(2)*v2(3,k)-v1(3)*v2(2,k)
      v3(2)=v1(3)*v2(1,k)-v1(1)*v2(3,k)
      v3(3)=v1(1)*v2(2,k)-v1(2)*v2(1,k)    
      end

