c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine kvdw2  --  A exp(-b r)-c/r6 van der Waals      ##
c     ##  parameter assignment                                     ##
c     ##                                                           ##
c     ###############################################################
c
c Wijand Mooij, july 1997

c     "kvdw2" assigns parameters to be used in computing the
c     van der Waals interactions and processes any new or
c changed values for these parameters. This versions reads the parameters for
c the 3-parameter type 6-exp potential: A exp(-b r) + C/r6  
C      
c VDW-type WIlliams
c VWD2  atomtype   A  B  C
c
c Also assigns parameters to be used in computing the
c     anisotropic repulsion, processes any new or
c changed values for these parameters. 
c VDW-type WIlliams
c ANISO  atomtype   A
c
c Also assings parameters to be used in dispersion/polarization damping
c VDWRADIUS atomtype A
c

      subroutine kvdw2
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'inform.i'
      include 'iounit.i'
      include 'keys.i'
      include 'kvdws2.i'
      include 'kvdwpr2.i'
      include 'math.i'
      include 'potent.i'
      include 'vdw.i'
      include 'vdw2.i'
      include 'vdwpot.i'
      include 'aniso.i'
      integer i,k,ia,ib,next,l,j
      integer size,number
      real*8 A,B,C,rdn,rd
      real*8 srad(maxtyp),seps(maxtyp)
      character*3 pa,pb
      character*6 pt
      character*20 keyword
      character*80 record,string
      logical header
c
c
c     process keywords containing van der Waals parameters
c

      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:5) .eq. 'VDW2 ') then
            k = 0
            A = 0.0d0
            B = 0.0d0
            C = 0.0d0
            rdn = 0.0d0
            string = record(next:80)
            read (string,*,err=10,end=10)  k,A,B,C,rdn
   10       continue
            if (k.ge.1 .and. k.le.maxtyp) then
               if (header) then
                  header = .false.
                  write (iout,20)
   20             format (/,' Additional van der Waals',
     &                          ' Parameters :',
     &                       //,6x,'Class      A     B',
     &                 '     C       rdn')
               end if
               if (rdn .eq. 0.0d0)  rdn = reduct2(k)
               Alist(k) = A
               Blist(k) = B
               Clist(k) = C
               reduct2(k) = rdn
               write (iout,30)  k,A,B,C,rdn
   30          format (4x,i6,1x,3f12.4,f11.3)
            else if (k .gt. maxclass) then
               write (iout,40)  maxclass
   40          format (/,' KVDW  --  Only Atom Classes through',i4,
     &                       ' are Allowed')
               abort = .true.
            end if
         else if  (keyword(1:6) .eq. 'EQUAL ') then
            k = 0
            l = 0
            string = record(next:80)
            read (string,*,err=11,end=11)  k,l
 11         continue
            print *,l,'set equal to', k
            do j=1,n
               if (type(i).eq.l) type(i)=k
            enddo
         end if
      end do
c
c     process keywords containing specific pair vdw parameters
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:7) .eq. 'VDWPR2 ') then
            ia = 0
            ib = 0
            A = 0.0d0
            B = 0.0d0
            C = 0.0d0
            string = record(next:80)
            read (string,*,err=80,end=80)  ia,ib,A,B,C
            if (header) then
               header = .false.
               write (iout,50)
   50          format (/,' User van der Waals Parameters',
     &                       ' for Specific Pairs :',
     &              //,6x,'Classes     A      B      C')
            end if
            write (iout,60)  ia,ib,A,B,C
   60       format (3x,2i4,2x,3f12.4)
            size = 3
            call numeral (ia,pa,size)
            call numeral (ib,pb,size)
            if (ia .le. ib) then
               pt = pa//pb
            else
               pt = pb//pa
            end if
            do k = 1, maxnvp2
               if (kvpr2(k).eq.'      ' .or. kvpr2(k).eq.pt) then
                  kvpr2(k) = pt
                  Alistpr(k) = A
                  Blistpr(k) = B
                  Clistpr(k) = C
                  goto 80
               end if
            end do
            write (iout,70)
   70       format (/,' KVDW  --  Too many Special VDW Pair',
     &                    ' Parameters')
            abort = .true.
   80       continue
         end if
      end do
     
c
c     use combination rules to set pairwise vdw radii sums
c
      do i = 1, maxclass
         do k = i, maxclass
            if (Blist(i).eq.0.0d0 .and. Blist(k).eq.0.0d0) then
               B = 0.0d0
            else if (brule(1:10) .eq. 'ARITHMETIC') then
               B = 0.5*(Blist(i) + Blist(k))
            else if (brule(1:9) .eq. 'GEOMETRIC') then
               B = sqrt(Blist(i) * Blist(k))
            else if (brule(1:10) .eq. 'CUBIC-MEAN') then
               B = 1.0d0 * (Blist(i)**3+Blist(k)**3)/
     +              (Blist(i)**2+Blist(k)**2)
            else if (brule(1:8) .eq. 'HARMONIC') then
               B = 2.0d0 * (Blist(i)*Blist(k)) / (Blist(i)+Blist(k))
            else if (brule(1:3) .eq. 'HHG') then
               B = 4.0d0 * (Blist(i)*Blist(k)) / (sqrt(Blist(i)) +
     +              sqrt(Blist(k)))**2
            else
               B = 0.5 * (Blist(i) + Blist(k))
            end if
            Buse(i,k) = B
            Buse(k,i) = B
         end do
      end do
c
c     use combination rules to set pairwise well depths
c
      do i = 1, maxclass
         do k = i, maxclass
            if (Alist(i).eq.0.0d0 .and. Alist(k).eq.0.0d0) then
               A = 0.0d0
            else if (arule(1:10) .eq. 'ARITHMETIC') then
               A = 0.5d0 * (Alist(i) + Alist(k))
            else if (arule(1:9) .eq. 'GEOMETRIC') then
               A = sqrt(Alist(i) * Alist(k))
            else if (arule(1:8) .eq. 'HARMONIC') then
               A = 2.0d0 * (Alist(i)*Alist(k)) / (Alist(i)+Alist(k))
            else if (arule(1:3) .eq. 'HHG') then
               A = 4.0d0 * (Alist(i)*Alist(k)) / (sqrt(Alist(i)) +
     +              sqrt(Alist(k)))**2
            else if (arule(1:10) .eq. 'CUBIC-MEAN') then
               A = 1.0d0 * (Alist(i)**3+Alist(k)**3)/
     +              (Alist(i)**2+Alist(k)**2)
            else
               A = sqrt(Alist(i) * Alist(k))
            end if
            Ause(i,k) = A
            Ause(k,i) = A
            if (Clist(i).eq.0.0d0 .and. Clist(k).eq.0.0d0) then
               C = 0.0d0
            else if (crule(1:10) .eq. 'ARITHMETIC') then
               C = 0.5d0 * (Clist(i) + Clist(k))
            else if (crule(1:9) .eq. 'GEOMETRIC') then
               C = sqrt(Clist(i) * Clist(k))
            else if (crule(1:8) .eq. 'HARMONIC') then
               C = 2.0d0 * (Clist(i)*Clist(k)) / (Clist(i)+Clist(k))
            else if (crule(1:3) .eq. 'HHG') then
               C = 4.0d0 * (Clist(i)*Clist(k)) / (sqrt(Clist(i)) +
     +              sqrt(Clist(k)))**2
            else if (crule(1:10) .eq. 'CUBIC-MEAN') then
               C = 1.0d0 * (Clist(i)**3+Clist(k)**3)/
     +              (Clist(i)**2+Clist(k)**2)
            else
               C = sqrt(Clist(i) * Clist(k))
            end if
            Cuse(i,k) = C
            Cuse(k,i) = C
         end do
      end do
c
c     vdw reduction factor information for each individual atom
c
      do i = 1, n
         kred(i) = reduct2(class(i))
         if (n12(i).ne.1 .or. kred(i).eq.0.0d0) then
            ired(i) = i
         else
            ired(i) = i12(1,i)
         end if
      end do
c
c     radii and well depths for special atom class pairs
c
      do i = 1, maxnvp2
         if (kvpr2(i) .eq. '      ')  goto 90
         ia = number (kvpr2(i)(1:3))
         ib = number (kvpr2(i)(4:6))
         Ause(ia,ib) = Alistpr(i)
         Ause(ib,ia) = Alistpr(i)
         Buse(ia,ib) = Blistpr(i)
         Buse(ib,ia) = Blistpr(i)
         Cuse(ia,ib) = Clistpr(i)
         Cuse(ib,ia) = Clistpr(i)
      end do
 90   continue
      
      if (debug) then
         do i=1,maxclass
            do k=i,maxclass
               if (ause(i,k).ne.0.0d0)
     +              print*, i,k,ause(i,k),buse(i,k),
     +              cuse(i,k)
            enddo
         enddo
      endif

c
c     do not remove zero-sized atoms from the list of vdw sites
c     for xtinker version

      nvdw = 0
      do i = 1, n
         if ((Alist(class(i)) .ne. 0.0d0) .and. 
     +        (Clist(class(i)).ne. 0.0d0)) then
            nvdw = nvdw + 1
         end if
         ivdw(i) = i
      end do
      if (nvdw.eq.0) then
         use_vdw=.false.
      else
         nvdw=n
      endif
      
c
c     process keywords containing vdwradii for damped dispersion
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:10) .eq. 'VDWRADIUS ') then
            ia = 0
            rd = 0.0d0
            string = record(next:80)
            read (string,*,err=91,end=91)  ia,rd
            if (header) then
               header = .false.
               write (iout,51)
 51            format (/,' Additional van der Waals radii',
     &              ' for damped dispersion/induction :',
     &              //,6x,'Class    Radius')
            end if
            write (iout,61)  ia,rd
 61         format (3x,i4,4x,f9.3)
 91         continue
            if (ia.gt.0) vdwradius(ia) = rd
         endif
      enddo
      
      do i = 1, maxclass
         aniso(i)=0.0d0
      end do
c
c     Process anisotropic repulsion parameters
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:6) .eq. 'ANISO ') then
            k = 0
            A = 0.0d0
            string = record(next:80)
            read (string,*,err=110,end=110)  k,A
 110        continue
            if (k.ge.1 .and. k.le.maxtyp) then
               if (header) then
                  header = .false.
                  write (iout,120)
 120               format (/,' Additional Anisotropic Repulsion',
     &                          ' Parameters :',
     &                       //,6x,'Class      A')
               end if
               write (iout,130)  k,A
 130            format (4x,i6,1x,3f12.4)
               aniso(k) = A
            else if (k .gt. maxclass) then
               write (iout,140)  maxclass
 140           format (/,' KVDW  --  Only Atom Classes through',i4,
     &                       ' are Allowed')
               abort = .true.
            end if
         end if
      end  do
      
c
c     remove zero-sized atoms from the list of vdw sites
c
      naniso = 0
      do i = 1, n
         nraniso(i) = 0
         if (aniso(class(i)) .ne. 0.0d0) then
            naniso = naniso + 1
            nraniso(i) = naniso
            ianiso(naniso) = i
         end if
      end do

      return
      end






