      SUBROUTINE list(enb,cellder,derivs)

C     Routine to calculates nonbonded energie for crystals.
c     Based on B.P. van Eijck's UPACK program  
c     This version uses TINKER multipole electrostatics, and many other TINKER
c     routines 
c     LIST uses the pairlist as created by NONBE.  
c
      
C     Iput through common blocks (in pairlist.i)
C     /NPL/ gives pair list information for LIST:
C        NPLA: number of pairs in the list
C           The various arrays give information for the calculation of
C           nonbonded pair interactions:
C        IPLA1, IPLA2: coordinates are located in X(IPLA),Y(IPLA),Z(IPLA)
C        SHIFT1..3: fractional shifts for required symmetry operation
C        NT14list: list for 1,4 scaling
C        factorlist: list double counting factor (0.5)

C     Output through parameters:
C     ENB: the nonbonded energy
C     CELLDER(1..6): the derivatives of ENB to BOX(1..6)
C     F(1..NR3): the derivatives of ENB to the coordinates
C     Array X,Y and Z: expanded with the symmetry-related molecules.

      implicit none

      include 'sizes.i'
      include 'atoms.i'      
      include 'mpole.i'
      include 'vdw.i'
      include 'boxes.i'
      include 'energi.i'
      include 'deriv.i'
      include 'iounit.i'
      include 'chgpot.i'
      include 'vdwpot.i'
      include 'polar.i'
      include 'math.i'
      include 'units.i'
      include 'potentxtra.i'
      include 'potent.i'
      include 'inform.i'
      include 'spgr.i'
      include 'pairlist.i'
      include 'energixtra.i'
      include 'aniso.i'
      include 'charge.i'
      include 'molcul.i'
      include 'polpot.i'


      real*8 cellder(6),derivs(3,maxatm)
      
      
      integer i,j,m,ix,iz,k,kx,kz,i1,i2
      integer ni,nj,ii,jj,iii,jjj,nsj,inter(maxatm)
      integer n1,n2,n3,njpt,jpt,inp(maxatm)
      real*8 enb,ax,bx,by,cx,cy,cz
      integer natm(maxatm),ibox
      real*8 xij,yij,zij,xij0,yij0,zij0,rij2,rij
      real*8 xi,yi,zi,xr,yr,zr,factor
      real*8 ei,eikv,eik,ek,de(3,6)
      real*8 a(3,3),d(3,3,3,3)
      real*8 rpi(13),rpk(13)
      real*8 indi(3),indk(3)
      real*8 dm(3),dp(3),utu
      real*8 dmi(3,3),dmk(3,3)
      real*8 dpi(3,3),dpk(3,3)

      real*8 xpole,ypole,zpole,finner,f(4)
      real*8 xred(maxatm),yred(maxatm),zred(maxatm)
      real*8 xijv,yijv,zijv,rijv,rij2v,rdn,xrv,yrv,zrv
      integer iv,kv
      real*8 redi,rediv,redk,redkv,dlcsave
      real*8 y2,ddispdr,ddispdx,ddispdy,ddispdz
      real*8 dedx,dedy,dedz,dedr,fc,qq,e
      integer nop,nop3,icj,jd,is1,is2,is3,nt14
      real*8 de1,de2,de3
      logical ani_i,ani_k
      real*8 v,uinnerpol,dumutual,field(3),indcell(3),indasym(3)
      real*8 vdwtail,etail,eikd,eikr

c Set dielectric constant to nonbonded-value
      dlcsave=dielec
      dielec=dlcnb

 10   format('Number of atoms',i5)
 11   format('Number of multipoles',i5)
 12   format('Number of VDW-atoms',i5)

      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)

c Fill up the unit-cell
      if (nspgr.gt.0) then
         do nop = 1,nspgr
            nop3=3*nop-3
            do  i= 1,n
               k=nop*n+i
c Expand coordinates:
               x(k) = msp(nop3+1)*x(i)+
     1              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               y(k) = msp(nop3+2)*y(i)+
     1              tsp(nop3+2)*by+tsp(nop3+3)*cy
               z(k) = msp(nop3+3)*z(i)+tsp(nop3+3)*cz
            enddo
         enddo
      endif
      
c
c     rotate multipole components and get rotation derivatives
c     also for multipoles of equivalent positions
c
      do i = 1, npole * (nspgr + 1)
         call rotmat (i,a)
         call rotpole (i,a)
         call drotmat (i,d)
         do j = 1, 3
            do k = 1, 3
               call drotpole (i,a,d,j,k)
            end do
         end do
      end do
      
c
c Calculate y-axis orientation for anisotropic repulsion term
c
      do k = 1, naniso * (nspgr + 1)
         i = ianiso(k)
         zaxes(1,k) = x(zaxis(i)) - x(i)
         zaxes(2,k) = y(zaxis(i)) - y(i)
         zaxes(3,k) = z(zaxis(i)) - z(i)
         xaxes(1,k) = x(xaxis(i)) - x(i)
         xaxes(2,k) = y(xaxis(i)) - y(i)
         xaxes(3,k) = z(xaxis(i)) - z(i)
         call prod2(k,zaxes,xaxes,yaxes)
         y2=yaxes(1,k)*yaxes(1,k)+yaxes(2,k)*yaxes(2,k)+
     +        yaxes(3,k)*yaxes(3,k)
         leny(k)=sqrt(y2)
         leny3(k)=leny(k)*y2
         call prod2(k,yaxes,zaxes,ycrossz)
         call prod2(k,yaxes,xaxes,ycrossx)
         zminx(1,k)=x(zaxis(i)) - x(xaxis(i))
         zminx(2,k)=y(zaxis(i)) - y(xaxis(i))
         zminx(3,k)=z(zaxis(i)) - z(xaxis(i))
         call prod2(k,yaxes,zminx,ycrosszx)
      enddo


c
c     zero out multipole , polarization and Vanderwaals energy and derivatives
c  
      em = 0.0d0
      ep = 0.0d0
      ev = 0.0d0
      uinner=0.d0
      uinnerpol=0.d0
      etail=0.0d0
      edisp=0.0d0
      erep=0.0d0

      do i = 1, n
         do  j = 1,3
            derivs(j,i) =  0.0d0
            dev(j,i) = 0.0d0
            dem(j,i) = 0.0d0
            dep(j,i) = 0.0d0
         enddo
      enddo
      do i=1,6
         cellder(i)=0.0d0
      enddo
      do i=1,3
         field(i)=0.0d0
      enddo

        
c Calculation of U_inner, for polar space groups only
      if (use_inner) then
         v = 4.0d0 * pi / (3.0d0 * ax * by * cz)
         finner = v * electric / (nspgr + 1)
C     This is twice the energy factor 2 pi / [3 V (nspgr+1)]
         xpole = 0.0d0
         ypole = 0.0d0
         zpole = 0.0d0
         if (use_mpole) then
            do ii = 1, n
               xpole = xpole + x(ii)*rpole(1,ii)*msom(1)
               ypole = ypole + y(ii)*rpole(1,ii)*msom(2)
               zpole = zpole + z(ii)*rpole(1,ii)*msom(3)
               xpole = xpole + rpole(2,ii)*msom(1)
               ypole = ypole + rpole(3,ii)*msom(2)
               zpole = zpole + rpole(4,ii)*msom(3)
            end do
         else if (use_charge) then
            do ii = 1, nion
               xpole = xpole + x(ii)*pchg(ii)*msom(1)
               ypole = ypole + y(ii)*pchg(ii)*msom(2)
               zpole = zpole + z(ii)*pchg(ii)*msom(3)
            enddo
         endif
            
         if (verbose) print 71,xpole*debye,ypole*debye,zpole*debye
 71      format(' Celldipole',2x,3f8.3)
         f(2)=finner*xpole
         f(3)=finner*ypole
         f(4)=finner*zpole
      
c        U_inner term itself:
         uinner = uinner - f(2) * xpole/2
         uinner = uinner - f(3) * ypole/2
         uinner = uinner - f(4) * zpole/2
c This term is the long-range correction to the electrostatic energy
         em = em + uinner

c Derivatives of Uinner to the coordinates:                  
c dU_inner/dx_i, dU_inner/dy_i, dU_inner/dz_i
         if (use_mpole) then
            do i = 1 , n 
               iz = zaxis(i)
               ix = xaxis(i)
               do  j = 1,3
                  derivs(j,i) =  derivs(j,i)-f(j+1)*rpole(1,i) * msom(j)
                  do k=1,3
                     derivs(j,i) =  derivs(j,i) - f(k+1) * 
     +                    dpole(k+1,1,j,i) * msom(k)
                     derivs(j,iz) = derivs(j,iz) - f(k+1) * 
     +                    dpole(k+1,2,j,i) * msom(k)
                     derivs(j,ix) = derivs(j,ix) - f(k+1) *
     +                    dpole(k+1,3,j,i) * msom(k)
                  enddo
               enddo 
            enddo
         else if (use_charge) then
            do i = 1 , n 
               do  j = 1,3
                  derivs(j,i) =  derivs(j,i)-f(j+1)*pchg(i) * msom(j)
               enddo
            enddo 
         endif
            
c Derivatives of Uinner to cell-volume                       
c dUinner/dax, dUinner/day, dUinner/daz, 
         do i = 1,3
            cellder(i) = -uinner/box(i)
         enddo
         field(1) = xpole * v
         field(2) = ypole * v
         field(3) = zpole * v
      endif
c End Uinner term
      
c      
c Calculation of long-range tail corrections to Vanderwaals interactions
c Initialize cell-derivatives.
c
      if (use_tail) then
         etail=vdwtail()
         do i=1,3
            cellder(i)=cellder(i)-etail/box(i)
         enddo
         ev=etail
         edisp=etail
      endif


c     
c     compute the induced dipoles at each atom
c
c Field contains the field due to the charge-distribtion on the cut
c -off sphere. This is the same kind of long-range correction that causes the 
c u-inner term for the electrostatic energy.
c
      call xinducelist(field)
c
c Now we calculate the u-inner correction for the polarization
c
      if (use_inner) then
         do i=1,3
            indasym(i)=0.0d0
         enddo
         do ii = 1, n
            do i=1,3
               indasym(i)= indasym(i)+uind(i,ii)
            enddo
         enddo
         do i=1,3
            indcell(i)=indasym(i)*msom(i)
         enddo
         if (verbose) print 72,indcell(1)*debye,indcell(2)*ypole*debye,
     +        indcell(3)*debye
 72      format(' Induced   ',2x,3f8.3)

         
         dumutual = 0.0d0
c
c The polarization energy calculated with empik1 does not contain the
c interaction of the induced dipoles with the u-inner field (celldipole).
c dumutual is a term for the induced dipole-induced dipole gradients
c
         do i=1,3
            uinnerpol= uinnerpol -0.5d0*electric*indasym(i)*
     +           field(i)
            dumutual = dumutual + electric*v*indcell(i)*indasym(i)
         enddo

         if (use_mpole) then
            do i = 1 , n 
               iz = zaxis(i)
               ix = xaxis(i)
               do  j = 1,3
                  dep(j,i) =  dep(j,i)- v*electric*
     +                 indasym(j)* rpole(1,i) * msom(j)
                  do k=1,3
                     dep(j,i) =  dep(j,i) - v*electric * 
     +                    indasym(k) * dpole(k+1,1,j,i) * msom(k)
                     dep(j,iz) = dep(j,iz) - v*electric * 
     +                    indasym(k) * dpole(k+1,2,j,i) * msom(k)
                     dep(j,ix) = dep(j,ix) - v*electric * 
     +                    indasym(k) * dpole(k+1,3,j,i) * msom(k)
                  enddo
               enddo 
            enddo
         endif
         do i = 1,3
            cellder(i) = cellder(i) - 2.0d0* uinnerpol/box(i)
            if (poltyp.eq.'MUTUAL') 
     +           cellder(i) = cellder(i) + 0.5d0*dumutual/box(i) 
         enddo
c This term is the long-range correction to the polarization energy
         ep = ep + uinnerpol
      endif

   
c
c Calculate reduced atomic coordinates for VDW interaction
c
      do i = 1, nvdw
         do j= 0,nspgr
c            i = ivdw(k)
            iv = ired(i) + j*n
            rdn = kred(i)
            ii=i + j*n
            xred(ii) = rdn*(x(ii)-x(iv)) + x(iv)
            yred(ii) = rdn*(y(ii)-y(iv)) + y(iv)
            zred(ii) = rdn*(z(ii)-z(iv)) + z(iv)
         enddo
      end do

      
      do ni=1,npla
         ii=ipla1(ni)
c k  is the atom number in cell-expansion
         k=ipla2(ni)
         icj=(k-1)/n
c jj is the atom numbern asymmetric unit
         jj=k-icj*n
         jd=3*icj

         if (jd.eq.0) then
            is1=1
            is2=1
            is3=1
            de1=0
            de2=0
            de3=0
         else
            is1=msp(jd-2)
            is2=msp(jd-1)
            is3=msp(jd)
            de1=-tsp(jd-2)
            de2=-tsp(jd-1)
            de3=-tsp(jd)
         endif 
         n1=shift1(ni)
         n2=shift2(ni)
         n3=shift3(ni)
         factor=factorlist(ni)
         nt14=nt14list(ni)

         xi = x(ii)
         yi = y(ii)
         zi = z(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)

C For "reduced" coordinates
         iv = ired(ii)
         redi = kred(ii)
         rediv = 1.0d0 - redi
         do j = 1, 13
            rpi(j) = rpole(j,ii)
         end do
         do j = 1, 3
            indi(j) = uind(j,ii)
         end do
         kz = zaxis(jj)
         kx = xaxis(jj)
         do j = 1, 13
            rpk(j) = rpole(j,k)
         end do

         do j = 1, 3
            indk(j) = uind(j,k)
         end do

         xr = x(ii) - x(k) 
         yr = y(ii) - y(k)
         zr = z(ii) - z(k)
         rij2=0.0
         xij=xr+n1*ax+n2*bx+n3*cx
         yij=yr+n2*by+n3*cy
         zij=zr+n3*cz
         rij2=xij**2+yij**2+zij**2
         rij=sqrt(rij2)

C For "reduced" coordinates
         kv = ired(jj)
         if ((ii.ne.iv) .or. (jj.ne.kv)) then 
            xrv = xred(ii) - xred(k)
            yrv = yred(ii) - yred(k)
            zrv = zred(ii) - zred(k)
            xijv=xrv+n1*ax+n2*bx+n3*cx
            yijv=yrv+n2*by+n3*cy
            zijv=zrv+n3*cz
            rij2v=xijv**2+yijv**2+zijv**2
            rijv=sqrt(rij2v)
         else 
            xijv=xij
            yijv=yij
            zijv=zij
            rij2v=rij2
            rijv=rij
         endif

         if (rij2.lt.0.003) then
            print*,'***** Short distance: ',sqrt(rij2)
         endif
         eik=0.0
         ei=0.0
         ek=0.0
c
c     increment the multipole energy and derivative expressions
c           
         if (use_mpole) then
            call empik1 (ii,k,-xij,-yij,-zij,rij,rij2,rpi,rpk,
     +           indi,indk,eik,ei,ek,dm,dmi,dmk,dp,dpi,dpk,utu)

            em = em + eik * factor
            if (factor.ne.1.d0) then
               do i1 = 1,3
                  dm(i1) = dm(i1) * factor
                  do i2 = 1,3
                     dmi(i1,i2)=dmi(i1,i2) * factor
                     dmk(i1,i2)=dmk(i1,i2) * factor
                  enddo
               enddo
            endif

            dem(1,ii) = dem(1,ii) - dm(1) + dmi(1,1)
            dem(2,ii) = dem(2,ii) - dm(2) + dmi(1,2)
            dem(3,ii) = dem(3,ii) - dm(3) + dmi(1,3)
            dem(1,iz) = dem(1,iz) + dmi(2,1)
            dem(2,iz) = dem(2,iz) + dmi(2,2)
            dem(3,iz) = dem(3,iz) + dmi(2,3)
            dem(1,ix) = dem(1,ix) + dmi(3,1)
            dem(2,ix) = dem(2,ix) + dmi(3,2)
            dem(3,ix) = dem(3,ix) + dmi(3,3)
c                       if (ii.ne. jj) then
            dem(1,jj) = dem(1,jj) + is1*(dm(1) + dmk(1,1))
            dem(2,jj) = dem(2,jj) + is2*(dm(2) + dmk(1,2))
            dem(3,jj) = dem(3,jj) + is3*(dm(3) + dmk(1,3))
            dem(1,kz) = dem(1,kz) + is1*dmk(2,1)
            dem(2,kz) = dem(2,kz) + is2*dmk(2,2)
            dem(3,kz) = dem(3,kz) + is3*dmk(2,3)
            dem(1,kx) = dem(1,kx) + is1*dmk(3,1)
            dem(2,kx) = dem(2,kx) + is2*dmk(3,2)
            dem(3,kx) = dem(3,kx) + is3*dmk(3,3)
c                     endif
                     
            dedx=-dm(1)+dmi(1,1) + dmi(2,1) + dmi(3,1)
            dedy=-dm(2)+dmi(1,2) + dmi(2,2) + dmi(3,2)
            dedz=-dm(3)+dmi(1,3) + dmi(2,3) + dmi(3,3)
            cellder(1)=cellder(1) + (n1+de1) * dedx
            cellder(2)=cellder(2) + (n2+de2) * dedy
            cellder(3)=cellder(3) + (n3+de3) * dedz
            cellder(4)=cellder(4) + (n2+de2) * dedx
            cellder(5)=cellder(5) + (n3+de3) * dedx
            cellder(6)=cellder(6) + (n3+de3) * dedy

c
c     increment the polarization energy and derivative expressions
c
                     
 13         format(3f10.4)
            if (use_polar) then
               ep = ep + (ei + ek) * factor
               if (factor.ne.1.d0) then
                  do i1 = 1,3
                     dp(i1) = dp(i1) * factor
                     do i2 = 1,3
                        dpi(i1,i2)=dpi(i1,i2) * factor
                        dpk(i1,i2)=dpk(i1,i2) * factor
                     enddo
                  enddo
               endif

               dep(1,ii) = dep(1,ii) - dp(1) + dpi(1,1)
               dep(2,ii) = dep(2,ii) - dp(2) + dpi(1,2)
               dep(3,ii) = dep(3,ii) - dp(3) + dpi(1,3)
               dep(1,iz) = dep(1,iz) + dpi(2,1)
               dep(2,iz) = dep(2,iz) + dpi(2,2)
               dep(3,iz) = dep(3,iz) + dpi(2,3)
               dep(1,ix) = dep(1,ix) + dpi(3,1)
               dep(2,ix) = dep(2,ix) + dpi(3,2)
               dep(3,ix) = dep(3,ix) + dpi(3,3)
               dep(1,jj) = dep(1,jj) + is1*(dp(1) + dpk(1,1))
               dep(2,jj) = dep(2,jj) + is2*(dp(2) + dpk(1,2))
               dep(3,jj) = dep(3,jj) + is3*(dp(3) + dpk(1,3))
               dep(1,kz) = dep(1,kz) + is1*dpk(2,1)
               dep(2,kz) = dep(2,kz) + is2*dpk(2,2)
               dep(3,kz) = dep(3,kz) + is3*dpk(2,3)
               dep(1,kx) = dep(1,kx) + is1*dpk(3,1)
               dep(2,kx) = dep(2,kx) + is2*dpk(3,2)
               dep(3,kx) = dep(3,kx) + is3*dpk(3,3)
                     
               dedx= -dp(1) + dpi(1,1) + dpi(2,1) + dpi(3,1)
               dedy= -dp(2) + dpi(1,2) + dpi(2,2) + dpi(3,2)
               dedz= -dp(3) + dpi(1,3) + dpi(2,3) + dpi(3,3)
               cellder(1)=cellder(1) + (n1+de1) * dedx
               cellder(2)=cellder(2) + (n2+de2) * dedy
               cellder(3)=cellder(3) + (n3+de3) * dedz
               cellder(4)=cellder(4) + (n2+de2) * dedx
               cellder(5)=cellder(5) + (n3+de3) * dedx
               cellder(6)=cellder(6) + (n3+de3) * dedy

            endif
         endif
         
c Charge-Charge energy and derivatives                  
         if (use_charge) then
            if (nt14.eq.1) then
               fc=factor/chgscale
            else
               fc=factor
            endif
            
            qq=fc*pchg(ii)*pchg(jj)*electric / dielec
            e=qq/rij
            dedr=-qq/rij2
c
c     form the chain rule terms for derivative expressions
c
            dedr = dedr / rij
            dedx = dedr * xij
            dedy = dedr * yij
            dedz = dedr * zij
c 
c      increment the 'multipole' energy and derivative expressions
c
            em = em + e
            dem(1,ii) = dem(1,ii) + dedx
            dem(2,ii) = dem(2,ii) + dedy
            dem(3,ii) = dem(3,ii) + dedz
            dem(1,jj) = dem(1,jj) - is1 * dedx
            dem(2,jj) = dem(2,jj) - is2 * dedy
            dem(3,jj) = dem(3,jj) - is3 * dedz
            cellder(1)=cellder(1) + (n1+de1) * dedx
            cellder(2)=cellder(2) + (n2+de2) * dedy
            cellder(3)=cellder(3) + (n3+de3) * dedz
            cellder(4)=cellder(4) + (n2+de2) * dedx
            cellder(5)=cellder(5) + (n3+de3) * dedx
            cellder(6)=cellder(6) + (n3+de3) * dedy
         endif

c        Vanderwaals energy and derivatives  

         
         eikv=0.0

         call evdwpair1(ii,k,jj,xijv,yijv,zijv,rij2v,rijv,
     +        rij2,rij,ddispdr,eikv,eikd,eikr,de,ani_i,ani_k,nt14)
         
         ev = ev + eikv  * factor
         edisp = edisp + eikd  * factor
         erep = erep + eikr  * factor

         if (factor.ne.1.0d0) then
            ddispdr=ddispdr*factor
            do i1=1,6
               do i2=1,3
                  de(i2,i1) = de(i2,i1)*factor
               enddo
            enddo
         endif
         
         if (ii.eq.iv) then
            dev(1,ii) = dev(1,ii) + de(1,1)
            dev(2,ii) = dev(2,ii) + de(2,1)
            dev(3,ii) = dev(3,ii) + de(3,1)
            dedx = de(1,1)
            dedy = de(2,1)
            dedz = de(3,1)
            if (ani_i) then
               dev(1,iz) = dev(1,iz) + de(1,2)
               dev(2,iz) = dev(2,iz) + de(2,2)
               dev(3,iz) = dev(3,iz) + de(3,2)
               dev(1,ix) = dev(1,ix) + de(1,3)
               dev(2,ix) = dev(2,ix) + de(2,3)
               dev(3,ix) = dev(3,ix) + de(3,3)
               dedx=dedx + de(1,2) + de (1,3)
               dedy=dedy + de(2,2) + de (2,3)
               dedz=dedz + de(3,2) + de (3,3)
            endif
         else
            dedx = de(1,1)
            dedy = de(2,1)
            dedz = de(3,1)
            dev(1,ii) = dev(1,ii) + de(1,1) * redi
            dev(2,ii) = dev(2,ii) + de(2,1) * redi
            dev(3,ii) = dev(3,ii) + de(3,1) * redi
            dev(1,iv) = dev(1,iv) + de(1,1) * rediv
            dev(2,iv) = dev(2,iv) + de(2,1) * rediv
            dev(3,iv) = dev(3,iv) + de(3,1) * rediv
         end if
         if (jj .eq. kv) then
            dev(1,jj) = dev(1,jj) + is1 * de(1,6)
            dev(2,jj) = dev(2,jj) + is2 * de(2,6)
            dev(3,jj) = dev(3,jj) + is3 * de(3,6)
            if (ani_k) then
               dev(1,kz) = dev(1,kz) + is1*de(1,4)
               dev(2,kz) = dev(2,kz) + is2*de(2,4)
               dev(3,kz) = dev(3,kz) + is3*de(3,4)
               dev(1,kx) = dev(1,kx) + is1*de(1,5)
               dev(2,kx) = dev(2,kx) + is2*de(2,5)
               dev(3,kx) = dev(3,kx) + is3*de(3,5)
            endif
         else
c                         print *,'k',i,iv,k,kv                        
            redk = kred(jj)
            redkv = 1.0d0 - redk
            dev(1,jj) = dev(1,jj) + is1 * de(1,6)*redk
            dev(2,jj) = dev(2,jj) + is2 * de(2,6)*redk
            dev(3,jj) = dev(3,jj) + is3 * de(3,6)*redk
            dev(1,kv) = dev(1,kv) + is1 * de(1,6) * redkv
            dev(2,kv) = dev(2,kv) + is2 * de(2,6) * redkv
            dev(3,kv) = dev(3,kv) + is3 * de(3,6) * redkv
         end if
c Derivatives for dispersion calculated without reduced coordinates  
         if (ddispdr.ne.0.0d0) then
            ddispdr=ddispdr/rij
            ddispdx=ddispdr*xij
            ddispdy=ddispdr*yij
            ddispdz=ddispdr*zij
            dev(1,ii) = dev(1,ii) + ddispdx
            dev(2,ii) = dev(2,ii) + ddispdy
            dev(3,ii) = dev(3,ii) + ddispdz
            dev(1,jj) = dev(1,jj) - is1 * ddispdx
            dev(2,jj) = dev(2,jj) - is2 * ddispdy
            dev(3,jj) = dev(3,jj) - is3 * ddispdz
            dedx = dedx + ddispdx
            dedy = dedy + ddispdy
            dedz = dedz + ddispdz
         endif
      


         cellder(1)=cellder(1) + (n1+de1) * dedx
         cellder(2)=cellder(2) + (n2+de2) * dedy
         cellder(3)=cellder(3) + (n3+de3) * dedz
         cellder(4)=cellder(4) + (n2+de2) * dedx
         cellder(5)=cellder(5) + (n3+de3) * dedx
         cellder(6)=cellder(6) + (n3+de3) * dedy

 889     format (12x,3i3,2x,4f8.3,3x,
     +        f8.3,2e13.4,12x,8f8.3)
 300  enddo


      enb= em + ep + ev

      if (verbose) then
         if (em .ne. 0.0) print 111,'E-multipole     ',em
         if (uinner .ne. 0.0) print 111,'incl. U2-inner  ',uinner
         if (ep .ne. 0.0) print 111,'E-polarisation  ',ep
         if (uinnerpol .ne. 0.0) print 111,'incl. U2-pol    ',uinnerpol
         if (ev .ne. 0.0) print 111,'E-Vanderwaals   ',ev
         if (etail .ne. 0.0) print 111,'incl. E-tail    ',etail
      endif
      if (verbose) print 111,'E-list         ',enb

 111  format(a17,f10.4)
      do i = 1, n
         do j = 1, 3
            derivs(j,i) = derivs(j,i) + dem(j,i) + dev(j,i) + dep(j,i)
         end do
      end do
      
c Reset dielectric constant to old value for intramolecular interaction:
      dielec=dlcsave


 297  format(a20,3f12.6)
 298  format(' Der: ',6f12.6)
 299  format(6f12.6)
      return

      end





