c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine mechanicxtra - initialize molecular mechanics  ##
c     ##                                                           ##
c     ###############################################################
c
c Wijnand Mooij, 1997:
c
c     "mechanicxtra" sets up needed parameters specific for the XTINKER
c crystal energy minimizer.
c e.g. sets up special intramolecular force field, if required
      
      subroutine mechanicxtra
      implicit none
      
      include 'sizes.i'      
      include 'inform.i'
      include 'iounit.i'
      include 'keys.i'
      include 'potentxtra.i'
      include 'math.i'
      include 'torpot.i'
      include 'vdwpotintra.i'
      
      include 'spgr.i'
      include 'molcul.i'
      include 'atoms.i'
      include 'cutoff.i'
      
      
      integer i,j,k
      character*80 record
c
c
c     set the default values for the active potentials
c
      use_vdw_intra = .false.
      use_lj_intra = .false.
      use_buck_intra = .false.
      use_hal_intra = .false.
c
c     set default values for force field control parameters
c
      vdwtyp = 'LENNARD-JONES'
      aterm = 0.0d0
      bterm = 0.0d0
      cterm = 0.0d0
      ngauss = 0
      vdw12use = 1
      vdw13use = 1
      vdw14use = -1
      vdwscale = 1.0d0
      radrule = 'ARITHMETIC'
      radtyp = 'R-MIN'
      radsiz = 'RADIUS'
      epsrule = 'GEOMETRIC'
      dlcnb = 1.0d0
      use_tail = .true.
      testcg = .false.
c
c Default cut-off for Exponential repulsion in 6-Exp vanderwaals
c
      repcut =vdwcut
c

c     read the potential energy force field parameter file
c
      call getprmxtra
c
c     check keywords for potential function control parameters
c
      do i = 1, nkey
         record = keyline(i)
         call prmkeyxtra (record)
      end do
c
c     if vdw is not used, turn off each vdw function type
c
      if (.not. use_vdw_intra) then
         use_buck_intra = .false.
         use_lj_intra = .false.
         use_hal_intra = .false.
      end if
      
      if (use_vdw_intra)  call kvdwintra
      if (use_buck_intra) call khbondintra
      call kaniso
      
      return
      end





