c
c
c     ###################################################
c     ## Wijnand Mooij, november 1997                     ##
c     ##                                               ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ## subroutine prepare  --  some preparation for xxtalmins   ##
c     ##                                                          ##
c     ##############################################################
c
      subroutine prepare
      
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      include 'polar.i'
      include 'spgr.i'
      include 'iounit.i'
      include 'potentxtra.i'
      include 'potent.i'
      include 'atmtyp.i'
      include 'aniso.i'
      include 'charge.i'
      include 'vdwpot.i'
      include 'vdwpair.i'
      include 'cutoff.i'
      
      integer ii,j,i,k,nop,nop3,sum_msp
      real*8 total,dihedral
      logical expand
c
c     set default values for the potential energy cutoffs
c
      if (chgcut.eq.0.0d0 .or. chgcut.ge.1000000.0d0)  chgcut = 15.0d0
      if (vdwcut.eq.0.0d0 .or. vdwcut.ge.1000000.0d0)  vdwcut = 15.0d0

c     test for total charge
      total=0.d0
      if (use_mpole) then
         do i=1,npole
            total=total+pole(1,i)
         enddo
      else if (use_charge) then
         do i=1,nion
            total=total+pchg(i)
         enddo
      endif
      
      if (abs(total).gt.1.0D-8) then
         write(iout,11) total
         if (abs(total).gt.1.0D-4) call fatal
      endif
 11   format('WARNING: System is not neutral. Total charge: ',f12.8)
      
c Scale the multipoles:
      
      if (msc(1).ne.1.0d0) then
         if (msc(2).eq.0.0d0 .and. msc(3).eq.0.0d0) then
            msc(2)=msc(1)
            msc(3)=msc(1)
         endif
 123     format(' Scaling multipoles(chg,dip,quadr) by',3f7.2)
         write(iout,123) msc(1),msc(2),msc(3)
         do i=1,n
            pole(1,i) = msc(1) * pole(1,i)
            do j=2,4
               pole(j,i)= msc(2) * pole(j,i)
            enddo
            do j=5,13
               pole(j,i)= msc(3) * pole(j,i)
            enddo
         enddo 
      endif
c      
c    Set VDW parameters for evdwpair
c
      sigma = 1.12d0
      kappa = 1.07d0
      kappa7 = kappa**7
      expcut = 2.0d0
      expcut2 = expcut * expcut
      expmerge = (aterm*exp(-bterm/expcut) - cterm*(expcut**6))
     &                               / (expcut**12)

      
 10   continue
      
c First expand the multipoles to unit-cell.
c When P1 expansion is required, sym. operations x multipoles
c should give desired number of multipoles. 
c The way pdamp is set in kpolar makes that it is not set for cell-expansion
c (as they do not have an multipole yet. Fix it here. Polarize is only needed
c for asymmetric unit.
c Symmetry constrained nonbe and list also need cell expansions of multipoles
c and pdamp.
      
      if (use_mpole) then
         expand=.false.
         if (npole.ne.n .and. mod(n,npole).eq.0) then
            ii = n/npole - 1
            if (ii.ne.nspgr) then
               print *,'Symmetry operations x npole does not give',
     +              ' enough multipoles'
               call fatal
            else
               write(iout,*) 
     +              'Expanding multipoles to unit cell'
               expand=.true.
            endif
         endif
         
      
         do nop = 1,nspgr
            nop3=3*nop-3
            do  i= 1,npole
               k=nop*npole+i
c Expand multipoles in global axes system:
               ipole(k) = ipole(i) + npole*nop
               zaxis(k) = zaxis(i) + npole*nop
               xaxis(k) = xaxis(i) + npole*nop
               polsiz(k) = polsiz(i)
               polaxe(k) = polaxe (i)
               pdamp(k) = pdamp(i)
               polarize(k) = polarize (i)
               do j= 1 , 13
                  pole (j,k) = pole(j,i)
               enddo
c If this symmetry relation is a mirror or an inversion: then invert all the y
c -components of the multipoles.
               sum_msp=msp(nop3+1) + msp(nop3+2) + msp(nop3+3)
               if (sum_msp.eq.-3 .or. sum_msp.eq.1) then
                  pole(3,k)=-pole(3,i)
                  pole(6,k)=-pole(6,i)
                  pole(8,k)=-pole(8,i)
                  pole(10,k)=-pole(10,i)
                  pole(12,k)=-pole(12,i)
               endif
c Expand atom classes, needed for anisotropic repulsion.
c Not needed in case of p1 expansion
               if (ispgr.ne.1) then
                  class(k)=class(i)
                  if (nraniso(i).ne.0) then
                     nraniso(k) = nraniso(i) + naniso*nop
                  else
                     nraniso(k) = 0
                  endif
               endif
            enddo
         enddo
      endif

c      
c Expand atomnumber list of anisotropic repulsion sites
c not needed in case of p1 expansion
      
      if (ispgr.ne.1) then
         do nop = 1,nspgr
            do i = 1,naniso
               k=nop*naniso + i
               ianiso(k)=ianiso(i) + npole*nop
            enddo
         enddo
      endif

      
c If  we have expanded the cell to P1, now we can set the number of symmetry
c operations to 0.
      if (ispgr.eq.1) nspgr=0

c Set number of multipoles to number of atoms
      if (expand)  npole=n

c
c  Get any requested torsion
c
      do i=1,nprt
         tau_orig(i)=dihedral(prt_tors(1,i),prt_tors(2,i),prt_tors(3,i),
     +        prt_tors(4,i))
      enddo

      
      end

