c     #############################################################
c     ##                                                         ##
c     ##  subroutine prmkeyxtra  --  interpret force field keywords  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "field" parses a text string to extract keywords related to
c     force field potential energy functional forms and constants
c
c Wijnand Mooij, 1997
c Check for XTINKER additional keywords
c     
      subroutine prmkeyxtra (text)
      implicit none
      include 'potentxtra.i'
      include 'potent.i'
      include 'vdwpotintra.i'
      include 'spgr.i'
      include 'iounit.i'
      include 'sizes.i'
      include 'atoms.i'



      integer next,ia,ib,ic,id,i
      character*4 value
      character*20 keyword
      character*80 text,record,string
c
c
c     parse the line to extract any possible keyword
c
      record = text
      next = 1
      call upcase (record)
      call gettext (record,keyword,next)
c
c     select the individual force field potential terms
c
c For getting rid of intramolecular nonbonded energies.
      if (keyword(1:15) .eq. 'NO-INTRA-NONBE ')  then
         nointranonbe= .true.
c Intramolecular forcefield terms:         
      else if (keyword(1:13) .eq. 'VDWINTRATERM ') then
         call getword (record,value,next)
         if (value .eq. 'ONLY') call potoff
         use_vdw_intra = .true.
         if (value .eq. 'NONE')  use_vdw_intra = .false.
      else if (keyword(1:16) .eq. 'CHARGEINTRATERM ') then
         call getword (record,value,next)
         if (value .eq. 'ONLY') call potoff
         use_charge_intra = .true.
         if (value .eq. 'NONE')  use_charge_intra = .false.        
      else if (keyword(1:14) .eq. 'VDWTYPE-INTRA ') then
         call getword (record,vdwtyp,next)
         use_lj_intra = .false.
         use_buck_intra = .false.
         use_hal_intra = .false.
c             use_gauss = .false.
         if (vdwtyp(1:13) .eq. 'LENNARD-JONES')  
     +        use_lj_intra = .true.
         if (vdwtyp(1:11) .eq. 'BUCKINGHAM')  
     +           use_buck_intra = .true.
         if (vdwtyp(1:13) .eq. 'BUFFERED-14-7')
     +           use_hal_intra = .true.
c         if (vdwtyp(1:8) .eq. 'GAUSSIAN')  use_gauss_inter = .true.
      else if (keyword(1:16) .eq. 'A-EXPTERM-INTRA ') then
         string = record(next:80)
         read (string,*,err=10)  aterm
      else if (keyword(1:16) .eq. 'B-EXPTERM-INTRA ') then
         string = record(next:80)
         read (string,*,err=10)  bterm
      else if (keyword(1:16) .eq. 'C-EXPTERM-INTRA ') then
         string = record(next:80)
         read (string,*,err=10)  cterm
c      else if (keyword(1:14) .eq. 'VDW-GAUSSIANS ') then
c         string = record(next:80)
c         read (string,*,err=10)  ngauss
      else if (keyword(1:13) .eq. 'VDW-14-SCALE ') then
         string = record(next:80)
         read (string,*,err=10)  vdwscale
         if (vdwscale .eq. 0.0d0)  vdw14use = 1
      else if (keyword(1:11) .eq. 'VDW-12-USE ') then
         vdw12use = 0
      else if (keyword(1:11) .eq. 'VDW-13-USE ') then
         vdw13use = 0
      else if (keyword(1:11) .eq. 'VDW-14-USE ') then
         vdw14use = -1
      else if (keyword(1:17) .eq. 'RADIUSTYPE-INTRA ') then
         call getword (record,radtyp,next)
      else if (keyword(1:17) .eq. 'RADIUSSIZE-INTRA ') then
         call getword (record,radsiz,next)
      else if (keyword(1:17) .eq. 'RADIUSRULE-INTRA ') then
         call getword (record,radrule,next)
      else if (keyword(1:18) .eq. 'EPSILONRULE-INTRA ') then
         call getword (record,epsrule,next)

      else if (keyword(1:12) .eq. 'MPOLE-SCALE ') then
         string=record(next:80)
         read(string,*,err=10) (msc(i),i=1,3)
      else if (keyword(1:14) .eq. 'GAMESS-OUTPUT ') then
         gamess_out=.true.
      else if (keyword(1:11) .eq. 'NO-U-INNER ') then
         use_inner=.false.
      else if (keyword(1:17) .eq. 'DIELECTRIC-NONBE ') then
         string=record(next:80)
         read(string,*,err=10) dlcnb               
      else if (keyword(1:11) .eq. 'REP-CUTOFF ') then
         string=record(next:80)
         read(string,*,err=10) repcut
c      else if (keyword(1:12) .eq. 'QUAD-CUTOFF ') then
c         string=record(next:80)
c         read(string,*,err=10) quadcut
      else if (keyword(1:8) .eq. 'NO-TAIL ') then
         use_tail=.false.
      else if (keyword(1:5) .eq. 'TEST ') then
         testcg=.true.
      else if (keyword(1:9).eq.'PRT-TORS ') then
         string = record(next:80)
         ia = 0
         ib = 0
         ic = 0
         id = 0
         read (string,*,err=250,end=250)  ia,ib,ic,id
 250     continue
         if (max(ia,ib,ic,id).le.n) then
            nprt = nprt + 1
            if (nprt.gt.maxprttors) then
               write(iout,*) 'Too much torsion-output requested'
               call fatal
            endif
            prt_tors(1,nprt)=ia
            prt_tors(2,nprt)=ib
            prt_tors(3,nprt)=ic
            prt_tors(4,nprt)=id
         else
            write(iout,*) 'Torsion requested of atomnr. > nr. of atoms '
         endif
      else if (keyword(1:8).eq.'REF-ATMS ') then
         string = record(next:80)
         read (string,*,err=260,end=260)  (refatms(i),i=1,3)
 260     continue
      endif

 10   continue
      return
      
      end













