c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine prtcssr -- output of fractional coordinates  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "prtcssr" writes out a set of fractional coordinates
c     to an external disk file in the CSSR format
c
c
      subroutine prtcssr (ixyz,string)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'files.i'
      include 'titles.i'
      include 'spgr.i'
      include 'math.i'
      include 'charge.i'
      include 'boxes.i'

      integer i,k,ixyz,j
      character*60 coordfile,string
      logical opened
      
      real*8  bb,cc,ax,bx,by,cx,cy,cz,cosa,cosb,cosc,sinc,vabc,cccs
      real*8 xfrac(maxatm),yfrac(maxatm),zfrac(maxatm),rtype
c
c
c     open output unit if not already done
c
      inquire (unit=ixyz,opened=opened)
      if (.not. opened) then
         coordfile = filename(1:leng)//'.cssr'
         call version (coordfile,'new')
         open (unit=ixyz,file=coordfile,status='new')
      end if
      
c     Fractional coordinates:  
      if (csf.eq.0) then
c     Calculate cell in a,b,c,alpha,beta,gamma:
         ax=box(1)
         by=box(2)
         cz=box(3)
         bx=box(4)
         cx=box(5)
         cy=box(6)
         bb=sqrt(bx**2+by**2)
         cc=sqrt(cx**2+cy**2+cz**2)
         cosa=(bx*cx+by*cy)/(bb*cc)
         cosb=cx/cc
         cosc=bx/bb
         alpha=radian*acos(cosa)
         beta=radian*acos(cosb)
         gamma=radian*acos(cosc)

c   Convert coordinates from cartesian to triclinic
         sinc=sqrt(1-cosc**2)
         vabc=sqrt(1-cosa**2-cosb**2-cosc**2+2*cosa*cosb*cosc)
         cccs=(cosa-cosb*cosc)/sinc

         do i = 1, n
            xfrac(i)=x(i)-(cosc/sinc)*y(i)+
     +           (cosc*cccs-sinc*cosb)*z(i)/vabc
            yfrac(i)=y(i)/sinc-(cccs/vabc)*z(i)
            zfrac(i)=(sinc/vabc)*z(i)
         enddo
      
c Convert coordinates to fractional:
      
         do i = 1, n
            xfrac(i) = xfrac(i)/ax
            yfrac(i) = yfrac(i)/bb
            zfrac(i) = zfrac(i)/cc
         enddo
c
c     write out the number of atoms and the title
c
         write (ixyz,9) ax,bb,cc
 9       format (38x,3f8.3) 
         write (ixyz,11)  alpha,beta,gamma,ispgr,spgr
 11      format(21x,3f8.3,'    SPGR=',i3,1x,a11)
         write (ixyz,12) n,csf,title
 12      format (2i4,1x,a60)
         write (ixyz,14) string
 14      format(a60)
         do i=1,n
            write (ixyz,13) tag(i),name(i),
     +           xfrac(i),yfrac(i),zfrac(i),(i12(j,i),j=1,4),0,0,0,0,
     +           pchg(i),type(i)
         enddo
      
 13      format(i4,1x,a4,2x,f9.5,1x,f9.5,1x,f9.5,1x,8i4,1x,f7.3,1x,i3)

      else if (csf.eq.1) then
         write (ixyz,*)
         write (ixyz,*)
         write (ixyz,12) n,csf,title
         write (ixyz,14) string
         do i=1,n
            write (ixyz,13) tag(i),name(i),
     +           x(i),y(i),z(i),(i12(j,i),j=1,4),0,0,0,0,
     +           pchg(i),type(i)
         enddo
      endif

      if  (.not. opened)  close (unit=ixyz)

      return
      end












