c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine prtspf  -- output of fractional coordinates  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "prtspf" writes out a set of fractional coordinates
c     to an external disk file in the platon SPF format
c
c
      subroutine prtspf (ixyz,string)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'files.i'
      include 'titles.i'
      include 'spgr.i'
      include 'math.i'
      include 'boxes.i'

      integer i,k,ixyz,j
      character*60 coordfile,string
      logical opened
      
      real*8  bb,cc,ax,bx,by,cx,cy,cz,cosa,cosb,cosc,sinc,vabc,cccs
      real*8 xfrac(maxatm),yfrac(maxatm),zfrac(maxatm),rtype
c
c
c     open output unit if not already done
c
      inquire (unit=ixyz,opened=opened)
      if (.not. opened) then
         coordfile = filename(1:leng)//'.spf'
         call version (coordfile,'new')
         open (unit=ixyz,file=coordfile,status='new')
      end if
      
c     Calculate cell in a,b,c,alpha,beta,gamma:
      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      alpha=radian*acos(cosa)
      beta=radian*acos(cosb)
      gamma=radian*acos(cosc)

c   Convert coordinates from cartesian to triclinic
      sinc=sqrt(1-cosc**2)
      vabc=sqrt(1-cosa**2-cosb**2-cosc**2+2*cosa*cosb*cosc)
      cccs=(cosa-cosb*cosc)/sinc

      do i = 1, n
         xfrac(i)=x(i)-(cosc/sinc)*y(i)+
     1        (cosc*cccs-sinc*cosb)*z(i)/vabc
         yfrac(i)=y(i)/sinc-(cccs/vabc)*z(i)
         zfrac(i)=(sinc/vabc)*z(i)
      enddo
      
c Convert coordinates to fractional:
      
      do i = 1, n
         xfrac(i) = xfrac(i)/ax
         yfrac(i) = yfrac(i)/bb
         zfrac(i) = zfrac(i)/cc
      enddo
c
c     write out the number of atoms and the title
c
      write(ixyz,51) title
      write(ixyz,50) ax,bb,cc,alpha,beta,gamma
      write(ixyz,53) spgr
      do i = 1,n
         write(ixyz,52) name(i),xfrac(i),yfrac(i),zfrac(i)
      enddo
      write(ixyz,54)
      
 50      format('CELL ',3f10.5,3f10.3)
 51      format('TITL',2x,a60)
 53      format('SPGR',2x,a8)
 52      format('ATOM ',a8,3x,3f10.6)
 54      format('END')
     
      if (.not. opened)  close (unit=ixyz)

      return
      end
