c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine readprm  --  input of force field parameters  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "readprm" processes the potential energy parameter file
c     in order to define the default force field parameters
c
c Reads in extra parameters for xtinker version, to be able to use different
c Intra and intermolecular forcefields
c
      subroutine readprmxtra (iprm)
      implicit none
      include 'iounit.i'
      include 'sizes.i'
      include 'potentxtra.i'
      include 'kvdwsintra.i'
      include 'kvdwprintra.i'
      include 'khbintra.i'
      include 'vdw2.i'
c      include 'kvdwpr2.i'
      include 'vdwpotintra.i'
      include 'math.i'
      include 'minima.i'

      integer i,j,iprm,size,next
      integer length,trimtext
      integer ia,ib,ic,id
      integer nt,nvp,nvp2,nhb
      real*8 rd,ep,rdn,fold
      real*8 vt(6),st(6),pl(13)
      character*3 pa,pb,pc,pd
      character*20 keyword
      character*80 record,string
      logical header
      real*8 A, B, C


c
c
c     initialize the counters for some parameter types
c
      nt = 0
      nvp = 0
      nvp2 = 0
      nhb = 0

c
c     initialize some of the force field parameters
c
      do i = 1, maxclass
         rad(i) = 0.0d0
         eps(i) = 0.0d0
         reduct(i) = 0.0d0
      end do
      
c Default scaling of the multipole moments:
      msc(1)=1.0d0
      msc(2)=0.0d0
      msc(3)=0.0d0
c Default no gamess output
      gamess_out = .false.

c
c     number of characters in an atom number text string
c
      size = 3
c
c     set blank line header before echoed comment lines
c
      header = .true.
c
c     process each line of the parameter file, first
c     extract the keyword at the start of each line
c
      dowhile (.true.)
         read (iprm,10,end=430)  record
   10    format (a80)
         next = 1
         call gettext (record,keyword,next)
         call upcase (keyword)

         call prmkeyxtra(record)
         
c
c     van der Waals parameters for individual atom types
c
         if (keyword(1:10) .eq. 'VDW-INTRA ') then
            ia = 0
            rd = 0.0d0
            ep = 0.0d0
            rdn = 0.0d0
            string = record(next:80)
            read (string,*,err=80,end=80)  ia,rd,ep,rdn
   80       continue
            rad(ia) = rd
            eps(ia) = ep
            reduct(ia) = rdn
c
c     van der Waals parameters for specific atom pairs
c
         else if (keyword(1:12) .eq. 'VDWPR-INTRA ') then
            ia = 0
            ib = 0
            rd = 0.0d0
            ep = 0.0d0
            string = record(next:80)
            read (string,*,err=90,end=90)  ia,ib,rd,ep
   90       continue
            call numeral (ia,pa,size)
            call numeral (ib,pb,size)
            nvp = nvp + 1
            if (ia .le. ib) then
               kvpr(nvp) = pa//pb
            else
               kvpr(nvp) = pb//pa
            end if
            radpr(nvp) = rd
            epspr(nvp) = ep
c
c     Hydrogen bond parameters
c
         else if (keyword(1:6) .eq. 'HBOND ') then
            ia = 0
            ib = 0
            rd = 0.0d0
            ep = 0.0d0
            string = record(next:80)
            read (string,*,err=90,end=95)  ia,ib,rd,ep
 95         continue
            call numeral (ia,pa,size)
            call numeral (ib,pb,size)
            nhb = nhb + 1
            if (ia .le. ib) then
               khb(nhb) = pa//pb
            else
               khb(nhb) = pb//pa
            end if
            radhb(nhb) = rd
            epshb(nhb) = ep
ccccccNEWccccNEWccccccccccccccc

         else if (keyword(1:10) .eq. 'VDWRADIUS ') then
            ia = 0
            rd = 0.0d0
            string = record(next:80)
            read (string,*,err=90,end=91)  ia,rd
 91         continue
            if (ia.gt.0) vdwradius(ia) = rd


         endif
      end do
 430  continue
      
c
c     zero out unused values for parameter atom types
c
      
c
      do i = nvp+1, maxnvp
         kvpr(i) = '      '
      end  do
      do i = nhb+1, maxnhb
         khb(i) = '      '
      end do
      
      return
      end

