c
c
c     ###################################################
c     ##  Wijnand Mooij     march 1997                 ##
c     ##                                               ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine readspf  --  input of fractional coordinates ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "readspf gets a set of fractional coordinates
c     from an external disk file
c
c
      subroutine readspf (ixyz,icon)

      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'files.i'
      include 'iounit.i'
      include 'titles.i'
      include 'boxes.i'
      include 'spgr.i'
      include 'math.i'

      integer i,j,k,m,ixyz,icon
      integer next,first,last
      integer nexttext,trimtext
      integer list(maxatm)
      character*60 spffile,confile
      character*120 record,string
      character*1 answer
      character*4 code
      character*7 flag
      logical opened,quit,reorder,exist

      real*8 cosa,cosb,cosc,bb,cc,bx,by,cx,cy,cz,sinc,vabc,cccs
      integer nword,freeunit,nop,nop3

      


c
c     read the title line 
c
      quit = .true.
      n = 0
 430  read (ixyz,10,err=50,end=40)  record
   10 format (a120)
      next = 1
      call gettext(record,code,next)
      if (code(1:3).eq.'END') goto 40
      if (code.eq.'TITL') then
c     extract the title and determine its length
         string = record(next:120)         
         first = nexttext (string)
         last = trimtext (string)
         if (last .eq. 0) then
            title = ' '
            ltitle = 0
         else
            title = string(first:last)
            ltitle = trimtext (title)
         end if
      
      else if (code.eq.'SPGR') then
         call gettext (record,spgr,next)
         print *,spgr

      else if (code.eq.'CELL') then
         string = record(next:120)         
         read (string,*,err=50,end=40) xbox,ybox,zbox,alpha,beta,gamma

      else if (code.eq.'ATOM') then
         n=n+1
         tag(n)=n
         call gettext (record,name(n),next)
         string = record(next:120)
         read (string,*,err=50,end=40) x(n),y(n),z(n)
      endif
      
      goto 430
      

 40   continue
      
      quit=.false.
 50   continue



c
c     check for too many total atoms in the file
c
      if (n .gt. maxatm) then
         write (iout,20)  maxatm
   20    format (/,' READXYZ  --  The Maximum of',i6,' Atoms',
     &        ' has been Exceeded')
         call fatal
      end if
      

c
c     read the atom-types and connectivities for each atom
c
      do i = 1, n
         next = 1
         do j = 1, maxval
            i12(j,i) = 0
         end do
         read (icon,30,err=50)  record
   30    format (a120)
         read (record,*,err=50,end=40)  type(i),(i12(j,i),j=1,maxval)
      end do

      
c
c     an error occurred in reading the connectivity file
c
      if (quit) then
         write (iout,61)  code
 61      format (/,' READXYZ  -- Error in Connectivity File at Atom',a6)
         call fatal
      end if
      
c
c     for each atom, count and sort its attached atoms
c
      do i = 1, n
         n12(i) = 0
         do j = maxval, 1, -1
            if (i12(j,i) .ne. 0) then
               n12(i) = j
               goto 70
            end if
         end do
   70    continue
         call sort (n12(i),i12(1,i))
      end do

      

c Expand to P1 unit cell
      call nextarg (answer,exist)
      if (.not. exist) then
         answer='N'
         write (iout,11)  answer
 11      format (/,' Fill up the unit cell [',a1,'] :  ',$)
         read (input,21)  answer
 21      format (a1)
      endif
      flag='NAME   '
      call xleesr(flag)
      call upcase (answer)
      if (answer .eq. 'Y')  then
         do nop=1,nspgr
            nop3 = 3*nop-3
            do  i = 1, n
               j=3*i-3
               k=i+nop*n
               x(k)=msp(nop3+1)*x(i) + tsp(nop3+1)
               y(k)=msp(nop3+2)*y(i) + tsp(nop3+2)
               z(k)=msp(nop3+3)*z(i) + tsp(nop3+3)
            enddo
         enddo
         
c        Expand connectivities to P1  
         do i=1,n
            do nop=1,nspgr
               k=i + nop*n
               tag(k)=  tag(i) + nop * n
               call cellatom(k,i)
            enddo
         enddo
c       Set number of atoms to atoms in unit cell and symmetry to P1
         n=n * (nspgr + 1)
         ispgr=1
         nspgr=0
         spgr='P1'
         
C*****Set  P1 ymmetry for dipole moment in Uinner calculation
            k=0
            do m = 1,3
               msom(m)=1
            enddo
            use_inner=.true.
            write(iout,41)
 41         format(' Polar space group, use calculation of U2-inner')

c           Set optimization of all cell-parameters on and disable
c optimization of first coordinate (origin is free to choose in P1)
c 
            do i=1,6
               iref(i)=1
            enddo
            do i=7,9
               iref(i)=0
            enddo
      endif


      
      bb=ybox
      cc=zbox
      cosa=cos(alpha/radian)
      cosb=cos(beta/radian)
      cosc=cos(gamma/radian)
      cx=cc*cosb
      bx=bb*cosc
      by=sqrt(bb*bb-bx*bx)
      cy=(bb*cc*cosa-bx*cx)/by
      cz=sqrt(cc*cc-cx*cx-cy*cy)
      box(1)=xbox
      box(2)=by
      box(3)=cz
      box(4)=bx
      box(5)=cx
      box(6)=cy


c Convert coordinates to cartesian
         do i=1,n
            x(i)=x(i)*xbox
            y(i)=y(i)*ybox
            z(i)=z(i)*zbox
         enddo
         
         sinc=sqrt(1-cosc**2)
         vabc=sqrt(1-cosa**2-cosb**2-cosc**2+2*cosa*cosb*cosc)
         cccs=(cosa-cosb*cosc)/sinc
      
         do i = 1, n
            x(i)=x(i)+cosc*y(i)+cosb*z(i)
            y(i)=sinc*y(i)+cccs*z(i)
            z(i)=(vabc/sinc)*z(i)
         enddo


c
c     check for scrambled atom order and attempt to renumber
c
      reorder = .false.
      do i = 1, n  
         list(tag(i)) = i
         if (tag(i) .ne. i)  reorder = .true.
      end do
      if (reorder) then
         write (iout,80)
   80    format (/,' READXYZ  --  Atom Labels not Sequential,',
     &              ' Attempting to Renumber')
         do i = 1, n
            tag(i) = i
            do j = 1, n12(i)
               i12(j,i) = list(i12(j,i))
            end do
            call sort (n12(i),i12(1,i))
         end do
      end if
c
c     make sure that all connectivities are bidirectional
c
      do i = 1, n
         do j = 1, n12(i)
            k = i12(j,i)
            do m = 1, n12(k)
               if (i12(m,k) .eq. i)  goto 100
            end do
            write (iout,90)  k,i
   90       format (/,' READXYZ  --  Check Connection of Atom',
     &           i6,' to Atom',i6)
            call fatal
  100       continue
         end do
      end do
      return
      end









