      SUBROUTINE xinducelist(field)

c     Routine to calculate the induced dipole moments in a crystal, using the
c LIST algorithm. This uses a pairlist previously created by nonbe, c.q.
c xinducenonbe

      include 'sizes.i'
      include 'atoms.i'      
      include 'mpole.i'
      include 'vdw.i'
      include 'boxes.i'
      include 'energi.i'
      include 'deriv.i'
      include 'iounit.i'
      include 'chgpot.i'
      include 'vdwpot.i'
      include 'polar.i'
      include 'math.i'
      include 'units.i'
      include 'potentxtra.i'
      include 'inform.i'
      include 'spgr.i'
      include 'pairlist.i'
      include 'polpot.i'
      include 'potent.i'


      real*8 cellder(6),derivs(3,maxatm)
      
      
      integer i,j,m,ix,iz,k,kx,kz,i1,i2
      integer ni,nj,ii,jj,iii,jjj,nsj,inter(maxatm)
      integer n1,n2,n3,njpt,jpt,inp(maxatm)
      real*8 ax,bx,by,cx,cy,cz
      real*8 rpi(13),rpk(13)
      integer natm(maxatm),ibox
      real*8 xij,yij,zij,xij0,yij0,zij0,rij2,rij
      real*8 xi,yi,zi,xr,yr,zr,factor


      integer iter,maxiter
      real*8 error,norm
      real*8 udir(3,maxatm)
      real*8 uold(3,maxatm)
      real*8 fieldi(3),fieldk(3),field(3),indcell(3),v

      if (.not. use_polar) then
c
c     zero out the induced dipole moment at each atom
c         
         do i = 1, n
            do j = 1, 3
               uind(j,i) = 0.0d0
            end do
         end do
         return
      else
c
c Set the field equal to the field of the long-range correction 
c (non-zero for polar space groups only)
c 
         do i = 1, n
            do j = 1, 3
               uind(j,i) = field(j)
            end do
         end do
      endif

      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)
      v = 4.0d0 * pi / (3.0d0 * ax * by * cz)

c
c     zero out the list of atoms to be skipped
c
c      if (chg12use .lt. 0)  chg12use = 0
c      if (chg13use .lt. 0)  chg13use = 0
c      if (chg14use .lt. 0)  chg14use = 0
      

      do ni=1,npla
         ii=ipla1(ni)
c k  is the atom number in cell-expansion
         k=ipla2(ni)
         icj=(k-1)/n
c jj is the atom numbern asymmetric unit
         jj=k-icj*n
         jd=3*icj

         if (jd.eq.0) then
            is1=1
            is2=1
            is3=1
         else
            is1=msp(jd-2)
            is2=msp(jd-1)
            is3=msp(jd)
         endif 
         n1=shift1(ni)
         n2=shift2(ni)
         n3=shift3(ni)
         factor=factorlist(ni)
         nt14=nt14list(ni)

c         xij=x(i+1)-is1*x(j+1)+s1*ax+s2*bx+s3*cx
c         yij=x(i+2)-is2*x(j+2)+s2*by+s3*cy
c         zij=x(i+3)-is3*x(j+3)+s3*cz
c         rij2=xij**2+yij**2+zij**2
         
c        Zinloos, i is gelijk aan ipole(i), anders werkt de hele routine niet:  
c         i = ipole(ii)
         i = ii

         xi = x(ii)
         yi = y(ii)
         zi = z(ii)
         iz = zaxis(i)
         ix = xaxis(i)

         do j = 1, 13
            rpi(j) = rpole(j,ii)
         end do
         
c         k = ipole(k)
         kz = zaxis(jj)
         kx = xaxis(jj)
         do j = 1, 13
            rpk(j) = rpole(j,k)
         end do



         xr = x(i) - x(k) 
         yr = y(i) - y(k)
         zr = z(i) - z(k)
         rij2=0.0
         xij=xr+n1*ax+n2*bx+n3*cx
         yij=yr+n2*by+n3*cy
         zij=zr+n3*cz
         rij2=xij**2+yij**2+zij**2
         rij=sqrt(rij2)


         if (rij2.lt.0.003) then
            print*,'***** Short distance: ',sqrt(rij2)
c          return
         endif
c         print *,'rij',rij
c         print *,'rijv',rijv
         
         call udirect (ii,k,-xij,-yij,-zij,rij,rij2,rpi,
     &        rpk,fieldi,fieldk)
         do j = 1, 3
            uind(j,i) = uind(j,i) + factor*fieldi(j)
         end do                 
         uind(1,jj) = uind(1,jj) + factor * is1 * fieldk(1)
         uind(2,jj) = uind(2,jj) + factor * is2 * fieldk(2)
         uind(3,jj) = uind(3,jj) + factor * is3 * fieldk(3)


 300  enddo


c
c     convert total field at each atom to induced dipole moment
c
      do ii = 1, npole
         i = ipole(ii)
         do j = 1, 3
            uind(j,i) = polarize(ii) * uind(j,i)
         end do
c Expand induced dipoles to unit-cell
         do nop = 1,nspgr
            nop3=3*nop-3
            k = nop*npole + ii
            do j= 1,3
               uind(j,k) = msp(nop3+j) * uind(j,ii)
            enddo
         enddo
      end do
      
c
      
      
      if (poltyp .eq. 'MUTUAL') then
         error = 1.0d0
         iter = 0
         maxiter = 100
         do ii = 1, npole * (nspgr+1)
            i = ipole(ii)
            do j = 1, 3
               udir(j,i) = uind(j,i)
            end do
         end do
c
c     compute mutual induced dipole moments by an iterative method
c
         dowhile (iter.lt.maxiter .and. error.gt.poleps)
            if (use_inner) then
               do j=1,3
                  indcell(j) = 0.0d0
               enddo
            endif
            do ii = 1, npole * (nspgr+1)
               i = ipole(ii)
               do j = 1, 3
                  uold(j,i) = uind(j,i)
                  if (use_inner) indcell(j) = indcell(j)+uind(j,i)
                  uind(j,i) = 0.0d0
               end do
            end do
            if (use_inner) then
               do ii = 1, npole
                  i = ipole(ii)
                  do j = 1, 3
                     uind(j,i)=indcell(j) * v
                  enddo
               enddo
            endif
            
            
            do ni=1,npla
               ii=ipla1(ni)
c k  is the atom number in cell-expansion
               k=ipla2(ni)
               icj=(k-1)/n
c jj is the atom numbern asymmetric unit
               jj=k-icj*n
               jd=3*icj
               
               if (jd.eq.0) then
                  is1=1
                  is2=1
                  is3=1
               else
                  is1=msp(jd-2)
                  is2=msp(jd-1)
                  is3=msp(jd)
               endif 
               n1=shift1(ni)
               n2=shift2(ni)
               n3=shift3(ni)
               factor=factorlist(ni)
               nt14=nt14list(ni)

c         xij=x(i+1)-is1*x(j+1)+s1*ax+s2*bx+s3*cx
c         yij=x(i+2)-is2*x(j+2)+s2*by+s3*cy
c         zij=x(i+3)-is3*x(j+3)+s3*cz
c         rij2=xij**2+yij**2+zij**2
         
c        Zinloos, i is gelijk aan ipole(i), anders werkt de hele routine niet:  
c         i = ipole(ii)
               i = ii
               
               xi = x(ii)
               yi = y(ii)
               zi = z(ii)
               iz = zaxis(i)
               ix = xaxis(i)

               do j = 1, 13
                  rpi(j) = rpole(j,ii)
               end do
         
c         k = ipole(k)
               kz = zaxis(jj)
               kx = xaxis(jj)
               do j = 1, 13
                  rpk(j) = rpole(j,k)
               end do

               xr = x(i) - x(k) 
               yr = y(i) - y(k)
               zr = z(i) - z(k)
               rij2=0.0
               xij=xr+n1*ax+n2*bx+n3*cx
               yij=yr+n2*by+n3*cy
               zij=zr+n3*cz
               rij2=xij**2+yij**2+zij**2
               rij=sqrt(rij2)

               if (rij2.lt.0.003) then
                  print*,'***** Short distance: ',sqrt(rij2)
c          return
               endif
c         print *,'rij',rij
c         print *,'rijv',rijv
               call umutual (ii,k,-xij,-yij,-zij,rij,rij2,uold(1,ii),
     +              uold(1,k),fieldi,fieldk)
               do j = 1, 3
                  uind(j,i) = uind(j,i) + factor*fieldi(j)
               end do                 
               uind(1,jj) = uind(1,jj) + factor * is1 * fieldk(1)
               uind(2,jj) = uind(2,jj) + factor * is2 * fieldk(2)
               uind(3,jj) = uind(3,jj) + factor * is3 * fieldk(3)
            enddo


c
c     check to see if the mutual induced dipoles have converged
c
            iter = iter + 1
            error = 0.0d0
            do ii = 1, npole
               i = ipole(ii)
               do j = 1, 3
                  uind(j,i) = udir(j,i) + polarize(ii) * uind(j,i)
                  error = error + (uind(j,i)-uold(j,i))**2
               end do
c Expand induced dipoles to unit-cell
               do nop = 1,nspgr
                  nop3=3*nop-3
                  k = nop*npole + ii
                  do j= 1,3
                     uind(j,k) = msp(nop3+j) * uind(j,ii)
                  enddo
               enddo
            end do
            error = debye * sqrt(error/dble(npole))
            if (debug) then
               if (iter .eq. 1) then
                  write (iout,10)
 10               format (/,' Determination of Induced Dipole',
     &                       ' Moments :',
     &                    //,4x,'Iter',8x,'RMS Change (Debyes)',/)
               end if
               write (iout,20)  iter,error
   20          format (i8,7x,f16.10)
            end if
         end do
c
c     terminate the calculation if dipoles failed to converge
c
         if (error .gt. poleps) then
            write (iout,30)
   30       format (/,' INDUCE  --  Warning, Induced Dipoles',
     &                 ' not Converged')
            call prterr
            call fatal
         end if
      end if
c
c     print out a list of the final induced dipole moments
c
      if (debug) then
         write (iout,40)
   40    format (/,' Induced Dipole Moments (Debyes) :',
     &           //,4x,'Atom',14x,'X',11x,'Y',11x,'Z',9x,'Total'/)
         do i = 1, npole
            k = ipole(i)
            norm = sqrt(uind(1,k)**2+uind(2,k)**2+uind(3,k)**2)
            write (iout,50)  k,(debye*uind(j,k),j=1,3),debye*norm
   50       format (i8,5x,4f12.4)
         end do
      end if
      return
      end
