      SUBROUTINE xinducenonbe(field)

c     Calculates the induced dipole moments in a crystal by using the NONBE
c algorithm. This creates a new pairlist, to be used by following induce, or
c energy calculations.
      
      implicit none
      
      include 'sizes.i'

      include 'atoms.i'
      include 'molcul.i'
      include 'mpole.i'
      include 'vdw.i'
      include 'boxes.i'
      include 'energi.i'
      include 'deriv.i'
      include 'iounit.i'
      include 'cutoff.i'
      include 'chgpot.i'
      include 'vdwpot.i'
      include 'couple.i'
      include 'polar.i'
      include 'math.i'
      include 'units.i'
      include 'potentxtra.i'
      include 'inform.i'
      include 'spgr.i'
      include 'pairlist.i'
      include 'polpot.i'
      include 'potent.i'




C     mxlist= maximum numbers of internal array in NONBE
      integer mxlist
      parameter(mxlist=500)
      
      real*8 rcut,rcut2,centrex(maxatm),
     +     centrey(maxatm),centrez(maxatm)
      real*8 cellder(6),derivs(3,maxatm)
      
      integer i,j,m,skip(maxatm),ix,iz,k,kx,kz,i1,i2
      integer ni,nj,ii,jj,iii,jjj,nsj,inter(maxatm)
      integer ing(3*mxlist*maxatm/3)
      integer n1,n2,n3,njpt,jpt,inp(maxatm)
      real*8 ax,bx,by,cx,cy,cz
      real*8 rpi(13),rpk(13)
      integer natm(maxatm/3),intra,nt13,nt14,ibox
      real*8 xij,yij,zij,xij0,yij0,zij0,rij2,rij
      real*8 xi,yi,zi,xr,yr,zr,factor


      integer nop,nop3,sum_msp,icj,jd,is1,is2,is3,nnj

      integer k1,k2

      integer iter,maxiter

      real*8 error,norm
      real*8 udir(3,maxatm)
      real*8 uold(3,maxatm)
      real*8 fieldi(3),fieldk(3),field(3)
      real*8 indcell(3),v

      if (.not. use_polar) then
c
c     zero out the induced dipole moment at each atom
c         
         do i = 1, n
            do j = 1, 3
               uind(j,i) = 0.0d0
            end do
         end do
         return
      else
c
c Set the field equal to the field of the long-range correction 
c (non-zero for polar space groups only)
c 
         do i = 1, n
            do j = 1, 3
               uind(j,i) = field(j)
            end do
         end do
      endif


      
c We can only use one cut-off: Choose the bigger of the two
      rcut = max(chgcut,vdwcut)
      rcut2 = rcut * rcut

      ax=box(1)
      by=box(2)
      cz=box(3)
      bx=box(4)
      cx=box(5)
      cy=box(6)

cuinner      
      v = 4.0d0 * pi / (3.0d0 * ax * by * cz)



c
c     zero out the list of atoms to be skipped
c
      do i = 1, n * (nspgr + 1)
         skip(i) = 0
      end do
c      if (chg12use .lt. 0)  chg12use = 0
c      if (chg13use .lt. 0)  chg13use = 0
c      if (chg14use .lt. 0)  chg14use = 0

      
c
c     compute the direct induced dipole moment at each atom
c
      

C*****CALCULATE AND STORE TEMPORARILY THE CENTERS OF GEOMETRY
C*****OF THE CHARGE GROUPS
c***** Char-groups are complete molecules 
      
      do i = 1, nmol
         natm(i) =  0
         centrex(i)=0.0d0
         centrey(i)=0.0d0
         centrez(i)=0.0d0
      enddo

      do j = 1, n
         k = molcule(j)
         centrex(k) = centrex(k) + x(j)
         centrey(k) = centrey(k) + y(j)
         centrez(k) = centrez(k) + z(j)
         natm(k) = natm(k) + 1
      enddo
      
      do i= 1 , nmol
         centrex(i) = centrex(i)/natm(i)
         centrey(i) = centrey(i)/natm(i)
         centrez(i) = centrez(i)/natm(i)
      enddo
      

c     centres of geometry of equivalent positions:  
      if (nspgr.gt.0) then
         do nop = 1,nspgr
            nop3=3*nop-3
            do  i= 1,nmol
               j=nop*nmol+i
               centrex(j) = msp(nop3+1)*centrex(i)+
     1              tsp(nop3+1)*ax+tsp(nop3+2)*bx+tsp(nop3+3)*cx
               centrey(j) = msp(nop3+2)*centrey(i)+
     1              tsp(nop3+2)*by+tsp(nop3+3)*cy
               centrez(j) = msp(nop3+3)*centrez(i)+tsp(nop3+3)*cz
            enddo
         enddo
      endif
      
c      print *,'Zwaartepunten'
c      do i=1,nmol * (1+nspgr)
          
c         print 332,centrex(i),centrey(i),centrez(i)
c      end do
c 332  format(3f12.4)

      

C     Doorloop alle moleculen NI van de asymetrische eenheid
      do 400 ni = 1,nmol
         if (debug) print 80,ni
 80      format(/,' Start calculation for charge group',i4)



C        Doorloop de moleculen NJ (ook equivalente posities) en maak
C         een lijst van translaties nodig om alle charge-group combinaties
C        binnen RCUT te vinden.
         do 200 nj = ni , nmol * (nspgr+1)
            njpt = (nj-1)*mxlist*3
C           Eis: intrabindingen eerst in de lijst;
C              aanname dat binnen het molecuul geen translatie nodig is.
            xij0 = centrex(ni) - centrex(nj)
            yij0 = centrey(ni) - centrey(nj)
            zij0 = centrez(ni) - centrez(nj)
c            print*,xij0,yij0,zij0
            if (xij0**2+yij0**2+zij0**2.lt.rcut2) then
               nsj = 1
c               print 155,nsj,0,0,0,xij0,yij0,zij0,
c     1           sqrt(xij0**2+yij0**2+zij0**2)
               inter(nj) = 0
               do  m = 1,3
                  ing(njpt+m) = 0
               enddo
            else
               nsj=0
               inter(nj)=1
            endif
            n3 = int((rcut - zij0)/cz)
            zij = zij0 + n3*cz
            n2 = int((rcut - yij0 - n3*cy)/by)
            yij = yij0 + n2*by + n3*cy
            n1 = int((rcut - xij0 - n2*bx - n3*cx)/ax)
            xij=xij0 + n1*ax + n2*bx + n3*cx

C           Zoek naar charge-groep afstanden binnen RCUT
 150        continue
            if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0) goto 160
            rij2 = xij**2 + yij**2 + zij**2
            if (rij2.lt.rcut2) then
               nsj = nsj + 1
c               print 155,nsj,n1,n2,n3,xij,yij,zij,sqrt(rij2)
 155           format(i5,3x,3i5,10f8.3)
               if (nsj.gt.mxlist) then
                  print*,' ***** mxlist te klein'
                  write(16,*) ' ***** mxlist te klein'
                  stop
               endif
               jpt = njpt + 3*(nsj-1)
               ing(jpt+1) = n1
               ing(jpt+2) = n2
               ing(jpt+3) = n3
            endif
 160        continue
            n1 = n1-1
            xij = xij-ax
 170        if (xij.lt.-rcut) then
               n2 = n2-1
               yij = yij - by
 180           if (yij.lt.-rcut) then
                  n3 = n3-1
                  zij = zij-cz
                  if (zij.lt.-rcut) goto 190
                  n2 = int((rcut - yij0 - n3*cy)/by)
                  yij = yij0 + n2*by + n3*cy
                  goto 180
               endif
               n1 = int((rcut - xij0 - n2*bx - n3*cx)/ax)
               xij = xij0 + n1*ax + n2*bx + n3*cx
               goto 170
            endif
            goto 150
 190        inp(nj) = nsj
c            print*,ni,nj,nsj
 200     continue
C        lijst gemaakt
         
         
C        Doorloop de atomen II (1..N) voorzover binnen de charge-groep NI.
c  First and last atom of molecule ni:
         do 390 iii = imol(1,ni), imol(2,ni)
            ii = kmol(iii)     
c i should equal ipole(i)
c            i = ipole(ii)
            i=ii
            xi = x(ii)
            yi = y(ii)
            zi = z(ii)
            iz = zaxis(i)
            ix = xaxis(i)

C For excluded neighbours the vdw settings are used: 
c 1-4 interactions are not scaled for multipole interactions, only for vdw
c interactions
            do j = 1, n12(i)
               skip(i12(j,i)) = i * vdw12use
            end do
            do j = 1, n13(i)
               skip(i13(j,i)) = i * vdw13use
            end do
            do j = 1, n14(i)
               skip(i14(j,i)) = i * vdw14use
            end do
            
            do j = 1, 13
               rpi(j) = rpole(j,ii)
            end do


C           Doorloop de charge-groepen NJ (ook equivalente posities).
            DO 300 NJ = ni , nmol * (nspgr + 1)
               IF (INP(NJ).EQ.0) GOTO 300
               njpt = (nj-1)*mxlist*3

c ICJ geeft aan of het equivalente positie's betreft
c Het nummer van icj geeft aan welke equivalente positie

               ICJ=(NJ-1)/nmol

c               NNJ=NJ-nmol*ICJ
c               JJ0=1
c               IF (NNJ.GT.1) JJ0=INC(NNJ-1)+1
c               JJMAX=INC(NNJ)

C              Doorloop de atomen JJ (1..NRP) voorzover gelegen
C                binnen de charge-group NJ.
               nnj=nj - nmol*icj
               do 280 jjj = imol(1,nnj) , imol(2,nnj)

c Loop is only over J>I atoms
                  if (jjj.lt.iii) goto 280
                  
c JJ is the number in the asymmetric unit numbering, k is the numbering in the
c cell-expansion                  
                  jj = kmol(jjj)
c                  k = ipole(jj) + icj * n
                  k = jj + icj * n

                  
c Axes-defining atoms in the asymmetric unit numbering:
                  kz = zaxis(jj)
                  kx = xaxis(jj)

 92               format(7x,'charge groups NI, NJ:',2i4,
     +                 8x,'atoms II, JJ,  JJ-exp',3i4)
                  if (debug) print 92,ni,nj,ii,jj,k

c                  print *,ii,jj

                  jd=3*icj
                  if (jd.eq.0) then
                     is1=1
                     is2=1
                     is3=1
                  else
                     is1=msp(jd-2)
                     is2=msp(jd-1)
                     is3=msp(jd)
                  endif 
                  
c                 Search for excluded neighbours  
                  intra=0
                  if (icj.eq.0 .and. inter(nj).eq.0) intra=1

                  nt13=0
                  nt14=0

c Alleen excluded neighbours skippen als intra flag 1 is:
c 
                  if (intra.eq.1) then
                     if (skip(k) .eq. ii ) nt13=1
                     if (skip(k) .eq.-ii) nt14=1
                  endif
                  
                  do j = 1, 13
                     rpk(j) = rpole(j,k)
                  end do
                  
                  factor=1.0d0 
                  if (jj.eq.ii) factor = 0.5d0
                     
                  xr = x(i) - x(k) 
                  yr = y(i) - y(k)
                  zr = z(i) - z(k)

C                 Doorloop alle translatie-geexpandeerde eenheidscellen.  
c                  print*,ii,k,jj,intra,nt13t14
                  do 260 ibox=1,inp(nj)
c                     print*,'260',(ing(njpt+(ibox-1)*3+m),m=1,3)
                     
                     n1=ing(njpt+(ibox-1)*3+1)
                     n2=ing(njpt+(ibox-1)*3+2)
                     n3=ing(njpt+(ibox-1)*3+3)

c If no single intra-molecular term is to be used:
c skip
c intra.eq.1 means we are not in an equivalent/translated position
c molcule(ii).eq.molecule(jj) for multiple molecules in the A.U.
                     if (intra.eq.1 .and. molcule(ii).eq.molcule(jj)
     +                    .and. nointranonbe) goto 250

c If atoms are the same in cell-expanded numbering and translation is zero:
c skip
                     if (n1.eq.0 .and. n2.eq.0 .and. n3.eq.0 
     +                    .and. ii.eq.k)  goto 250

c                    If it are excluded neighbours
c skip
                     if (nt13.eq.1) goto 250


                     rij2=0.0
c                     n1=ing(njpt+(ibox-1)*3+1)
c                     n2=ing(njpt+(ibox-1)*3+2)
c                     n3=ing(njpt+(ibox-1)*3+3)
                     xij=xr+n1*ax+n2*bx+n3*cx
                     yij=yr+n2*by+n3*cy
                     zij=zr+n3*cz
                     rij2=xij**2+yij**2+zij**2
                     rij=sqrt(rij2)

                     if (rij2.lt.1.0e-6) goto 250
c                if (rij2.lt.0.003) then
c                   print*,'***** Short distance: ',ii,jj,sqrt(rij2)
c                   enb=9999
c                   return
c                endif

C                    Maak lijst van alle atomen voor LIST
c                        if (nl.eq.1) then
                     npla=npla+1
                     if (npla.gt.maxlist) then
                        print*,'Vergroot maxlist in NONBE en LIST'
                        write(16,*)
     +                       'Vergroot maxlist in NONBE en LIST'
                        stop
                     endif
                     ipla1(npla)=ii
                     ipla2(npla)=k
c                     ipla3(npla)=jj
                     shift1(npla)=n1
                     shift2(npla)=n2
                     shift3(npla)=n3
c                     isig1(npla)=is1
c                     isig2(npla)=is2
c                     isig3(npla)=is3
                     factorlist(npla)=factor
                     nt14list(npla)=nt14
c                           plj6(npla)=ch6
c                           plj12(npla)=ch12
c                           if (coul) pcg(npla)=cgij
c                        endif
                     
                     call udirect (ii,k,-xij,-yij,-zij,rij,rij2,rpi,
     +                    rpk,fieldi,fieldk)
                     do j = 1, 3
                        uind(j,i) = uind(j,i) + factor*fieldi(j)
                     end do                 
                     uind(1,jj) = uind(1,jj) + factor * is1 * fieldk(1)
                     uind(2,jj) = uind(2,jj) + factor * is2 * fieldk(2)
                     uind(3,jj) = uind(3,jj) + factor * is3 * fieldk(3)

                     
 250                 continue
                     if (ibox.eq.1) then
                        intra=0
                        nt13=0
                        nt14=0
                     endif
C                    eind van deze translatie-actie
 260              continue
 280           continue
 300        continue
 390     continue
 400  continue
      
c
c     convert total field at each atom to induced dipole moment
c
      do ii = 1, npole
         i = ipole(ii)
         do j = 1, 3
            uind(j,i) = polarize(ii) * uind(j,i)
         end do
c Expand induced dipoles to unit-cell
         do nop = 1,nspgr
            nop3=3*nop-3
            k = nop*npole + ii
            do j= 1,3
               uind(j,k) = msp(nop3+j) * uind(j,ii)
            enddo
         enddo
      end do
      

      if (poltyp .eq. 'MUTUAL') then
         error = 1.0d0
         iter = 0
         maxiter = 100
         do ii = 1, npole * (nspgr+1)
            i = ipole(ii)
            do j = 1, 3
               udir(j,i) = uind(j,i)
            end do
         end do
c
c     compute mutual induced dipole moments by an iterative method
c
         dowhile (iter.lt.maxiter .and. error.gt.poleps)
            do j=1,3
               indcell(j) = 0.0d0
            enddo

            do ii = 1, npole * (nspgr+1)
               i = ipole(ii)
               do j = 1, 3
                  uold(j,i) = uind(j,i)
c uinner
                  if (use_inner) indcell(j) = indcell(j)+uind(j,i)
c
                  uind(j,i) = 0.0d0
               end do
            end do
            
c     uinner. 
            if (use_inner) then
               do ii = 1, npole
                  i = ipole(ii)
                  do j = 1, 3
                     uind(j,i)=indcell(j) * v
                  enddo
               enddo
            endif


            do ni=1,npla
               ii=ipla1(ni)
c k  is the atom number in cell-expansion
               k=ipla2(ni)
               icj=(k-1)/n
c jj is the atom numbern asymmetric unit
               jj=k-icj*n
               jd=3*icj
            
               if (jd.eq.0) then
                  is1=1
                  is2=1
                  is3=1
               else
                  is1=msp(jd-2)
                  is2=msp(jd-1)
                  is3=msp(jd)
               endif 
               n1=shift1(ni)
               n2=shift2(ni)
               n3=shift3(ni)
               factor=factorlist(ni)
               nt14=nt14list(ni)

c         xij=x(i+1)-is1*x(j+1)+s1*ax+s2*bx+s3*cx
c         yij=x(i+2)-is2*x(j+2)+s2*by+s3*cy
c         zij=x(i+3)-is3*x(j+3)+s3*cz
c         rij2=xij**2+yij**2+zij**2
         
c        Zinloos, i is gelijk aan ipole(i), anders werkt de hele routine niet:  
c         i = ipole(ii)
               i = ii
               
               xi = x(ii)
               yi = y(ii)
               zi = z(ii)
               iz = zaxis(i)
               ix = xaxis(i)

               do j = 1, 13
                  rpi(j) = rpole(j,ii)
               end do
         
c         k = ipole(k)
               kz = zaxis(jj)
               kx = xaxis(jj)
               do j = 1, 13
                  rpk(j) = rpole(j,k)
               end do

               xr = x(i) - x(k) 
               yr = y(i) - y(k)
               zr = z(i) - z(k)
               rij2=0.0
               xij=xr+n1*ax+n2*bx+n3*cx
               yij=yr+n2*by+n3*cy
               zij=zr+n3*cz
               rij2=xij**2+yij**2+zij**2
               rij=sqrt(rij2)

               if (rij2.lt.0.003) then
                  print*,'***** Short distance: ',sqrt(rij2)
c          return
               endif
c         print *,'rij',rij
c         print *,'rijv',rijv
               call umutual (ii,k,-xij,-yij,-zij,rij,rij2,uold(1,ii),
     +              uold(1,k),fieldi,fieldk)
               do j = 1, 3
                  uind(j,i) = uind(j,i) + factor*fieldi(j)
               end do                 
               uind(1,jj) = uind(1,jj) + factor * is1 * fieldk(1)
               uind(2,jj) = uind(2,jj) + factor * is2 * fieldk(2)
               uind(3,jj) = uind(3,jj) + factor * is3 * fieldk(3)
            enddo


c
c     check to see if the mutual induced dipoles have converged
c
            iter = iter + 1
            error = 0.0d0
            do ii = 1, npole
               i = ipole(ii)
               do  j = 1, 3
                  uind(j,i) = udir(j,i) + polarize(ii) * uind(j,i)
                  error = error + (uind(j,i)-uold(j,i))**2
               end do
c Expand induced dipoles to unit-cell
               do nop = 1,nspgr
                  nop3=3*nop-3
                  k = nop*npole + ii
                  do j= 1,3
                     uind(j,k) = msp(nop3+j) * uind(j,ii)
                  enddo
               enddo
            end do
            error = debye * sqrt(error/dble(npole))
            if (debug) then
               if (iter .eq. 1) then
                  write (iout,10)
 10               format (/,' Determination of Induced Dipole',
     &                 ' Moments :',
     &              //,4x,'Iter',8x,'RMS Change (Debyes)',/)
               end if
               write (iout,20)  iter,error
 20            format (i8,7x,f16.10)
            end if
         end do
c
c     terminate the calculation if dipoles failed to converge
c
         if (error .gt. poleps) then
            write (iout,30)
 30         format (/,' INDUCE  --  Warning, Induced Dipoles',
     &        ' not Converged')
            call prterr
            call fatal
         end if
      end if
c
c     print out a list of the final induced dipole moments
c
      if (debug) then
         write (iout,40)
   40    format (/,' Induced Dipole Moments (Debyes) :',
     &           //,4x,'Atom',14x,'X',11x,'Y',11x,'Z',9x,'Total'/)
         do i = 1, npole * (nspgr+1)
            k = ipole(i)
            norm = sqrt(uind(1,k)**2+uind(2,k)**2+uind(3,k)**2)
            write (iout,50)  k,(debye*uind(j,k),j=1,3),debye*norm
   50       format (i8,5x,4f15.9)
         end do
      end if
      return
      end





