            subroutine xleesr(flag)


c    B.P. van Eijck, 1997/98
C     Reads space group data for triclinic, monoclinic, and
C        orthorhombic space groups and selects data for space group
C        name SPGR (International Tables).
C---------------------------------------------------------------------------
C     Input from file IN.SPG (unit 11) for every implemented space group;
C     here a - signifies a new line in the input file.
C     -  NAAM: space group name
C     -  IS: space group number
C        NSPGR: number of space group operations (E excluded)
C     -  MSP(x), TSP(x),    MSP(y), TSP(y),    MSP(z), TSP(z)
C           where r'=MSP.r + TSP, so each MSP=+1 or -1 and each TSP=0 or 0.5.
C     -  IREF(1..12) =  0/1: disables/enables optimization of parameters:
C           cell axes:                1: AX    2: BY      3: CZ
C           cell angles:              4: BX    5: CX      6: CY
C           molecular centers:        7: X     8: Y       9: Z
C           molecular orientations:  10: PHI  11:THETA   12:PSI
C     -  ITEST(1..3): conditions for cell edges (not used in xxtalmins,
c     for UPACK prediction code only  )
C           ITEST(1) = 1: enables skipping of cells with AX > BY
C           ITEST(2) = 1: enables skipping of cells with BY > CZ
C           ITEST(3) = 1: enables skipping of cells with AX > CZ
C---------------------------------------------------------------------------
      
      include 'spgr.i'
      include 'files.i'
      include 'iounit.i'


      character naam*8,temp*8,flag*7

      logical exist,exit
      character*80 spgrfile
      character*80 record
      integer file_spgr,itest(3),freeunit,is,next,k,m

c      Try to get spacegroup file
      spgrfile = '/bot/mooy/xtinker35/source/all.spg'
c      spgrfile = '/cmsc/home/ruu/mooy/progs/xtinker/source/all.spg'
      inquire (file=spgrfile,exist=exist)

c     Try to get a spacegroup file from basename  
      if (.not. exist) then
         spgrfile = filename(1:leng)//'.spg'
         inquire (file=spgrfile,exist=exist)
      endif


      dowhile (.not. exist)
         write (iout,10)
 10      format (/,' Enter space group information File Name :  ',$ 
     +        )
         read (input,20)  spgrfile
         call suffix (spgrfile,'spg')
         inquire (file=spgrfile,exist=exist)
      end do
      
      file_spgr=freeunit()
      open(unit=file_spgr,file=spgrfile,status='OLD')
      
      
C*****Read space group file
      exit=.false.
      next=1
      if (flag.eq.'NAME   ') then
         call upcase(spgr)
         call gettext(spgr,spgr,next)
      endif
      
 100  next=1
      read(file_spgr,20,end=900,err=900) record
      call gettext(record,naam,next)
      call upcase(naam)
      read(file_spgr,*,end=910,err=910) is,nspgr
      do 110 i=1,nspgr
        j=3*i-3
        read(file_spgr,*) (msp(j+m),tsp(j+m),m=1,3)
  110 continue
      read(file_spgr,*) (iref(m),m=1,12)
      read(file_spgr,*) (itest(m),m=1,3)
      if (flag.eq.'NAME   ') then
         if (naam .eq. spgr) then
            exit=.true.
            ispgr=is
         endif
      else if (flag.eq.'NUMBER ') then
         if (is.eq.ispgr) then
            exit=.true.
            spgr=naam
         endif
      endif
      if (exit) then
         close(file_spgr)
         if (nspgr.gt.mxspgr) then
            write(iout,*) ' mxspgr te klein'
            stop
         endif
         write(iout,21) ispgr,spgr

C*****Investigate symmetry for dipole moments
         use_inner=.false.
         k=0
         do m = 1,3
            msom(m)=1
            do nop = 1,nspgr
               nop3=3*nop-3
               msom(m)=msom(m)+msp(nop3+m)
            enddo
            if (msom(m).ne.0 .and. iref(m+6).ne.0) then
               write(iout,30) m,msom(m),iref(m+6)
               stop
            endif
            k=k+msom(m)
         enddo
         if (k.ne.0) use_inner=.true.
         write(iout,35) (msom(m),m=1,3)

 30   format(' ***** Error in symmetry properties',3i6)
 35   format(' Dipole coefficients',3i5)
 40   format(' Polar space group, use calculation of U2-inner')


      return
      endif
      goto 100

 900  write(iout,*) ' ***** Space group ',spgr,'  number',
     +     ispgr,' not found'
      stop
 910  write(iout,*) ' ***** Error reading space group file'
      stop
 920  write(iout,*) '***** No space group file present'
      stop

   20 format(a80)
   21 format(' Space group',i4,2x,a10)
      end



