      subroutine xtalinfo(label)

c Wijnand Mooij, 1999
c  Routine to report change of crystal structure after minimization
c  Changes in cell parameters are reported, as well as overall rotation and
c translation of the molecules. Net rotation is determined in terms of Euler
c angles, net translation by change in fractional coordinates of the centre of
c mass coordinates. Use of fractional coordinates is used to exclude the
c influence of cell-expansion/contraction on this shift of the centre of mass.
c      
      implicit none
      include 'spgr.i'
      include 'math.i'
      include 'sizes.i'
      include 'atoms.i'
      include 'molcul.i'
      include 'atmtyp.i'
      include 'iounit.i'
      include 'units.i'


      real*8 ax,by,cz,bx,cx,cy,bb,cc,cosa,cosb,cosc,alpha,beta,gamma
      real*8 ax0,bb0,cc0,alpha0,beta0,gamma0
      real*8 xcm(3),phi,theta,psi
      real*8 xcm0(3,maxmol),phi0(maxmol),theta0(maxmol),psi0(maxmol),
     +     xcm1(3,maxmol),phi1(maxmol),theta1(maxmol),psi1(maxmol)
      real*8 massmol(maxatm)
      real*8 xx(maxvar)
      real*8 dihedral,rotation,rot,trns,tau(maxprttors),vol0,vol
      character*5 label
      save xcm0,phi0,theta0,psi0,ax0,bb0,cc0,alpha0,beta0,gamma0,vol0
      integer i,ii,iii,ni,ir
      real*8 x12,y12,z12,sinc,vabc,cccs,roff


      ax=box(1)
      by=box(2)
      cz=box(3)
      vol=ax*by*cz
      bx=box(4)
      cx=box(5)
      cy=box(6)
      bb=sqrt(bx**2+by**2)
      cc=sqrt(cx**2+cy**2+cz**2)
      cosa=(bx*cx+by*cy)/(bb*cc)
      cosb=cx/cc
      cosc=bx/bb
      alpha=radian*acos(cosa)
      beta=radian*acos(cosb)
      gamma=radian*acos(cosc)
      sinc=sqrt(1-cosc**2)
      vabc=sqrt(1-cosa**2-cosb**2-cosc**2+2*cosa*cosb*cosc)
      cccs=(cosa-cosb*cosc)/sinc

     
c Determine starting orientation of molecule in Euler angles
      do iii=1,nmol
         ir=1
         ni=0
         do ii = imol(1,iii),imol(2,iii)
            ni=ni+1
            i = kmol(ii)
            xx(3*(ni-1)+1)=x(i)
            xx(3*(ni-1)+2)=y(i)
            xx(3*(ni-1)+3)=z(i)
            massmol(ni)=mass(i)
c
c If the coordinate system reference atoms are not given, than set them
c to first three non-hydrogen/deuterium atoms in the molecule.
c 
            if (ir.le.3) then
               if ((refatms(ir).eq.0) .and. (mass(i).ge.3.5)) then
                  refatms(ir)=ni
                  ir=ir+1
               endif
            endif
         enddo
         call trans(ni,massmol,xx,refatms,xcm,phi,theta,psi)
c Convert centre of mass coordinated to fractional
         xcm(1)=(xcm(1)-(cosc/sinc)*xcm(2)+
     +        (cosc*cccs-sinc*cosb)*xcm(3)/vabc)/ax
         xcm(2)=(xcm(2)/sinc-(cccs/vabc)*xcm(3))/bb
         xcm(3)=((sinc/vabc)*xcm(3))/cc
         if (label.eq.'start') then
            xcm0(1,iii)=xcm(1)
            xcm0(2,iii)=xcm(2)
            xcm0(3,iii)=xcm(3)
            phi0(iii)=phi
            theta0(iii)=theta
            psi0(iii)=psi
            vol0=vol
         else
            xcm1(1,iii)=xcm(1)
            xcm1(2,iii)=xcm(2)
            xcm1(3,iii)=xcm(3)
            phi1(iii)=phi
            theta1(iii)=theta
            psi1(iii)=psi
         endif
            
      enddo
      
      if (label.eq.'start') then 
         ax0=ax
         bb0=bb
         cc0=cc
         alpha0=alpha
         beta0=beta
         gamma0=gamma
      endif

      
      if (label.eq.' end ') then
c
c  Get any requested torsions
c
         if (nprt.ne.0) then
            write(iout,10)
 10         format(/,12x,'ia   ib   ic   id   Final     Start')
         endif
         
         do i=1,nprt
            tau(i)=dihedral(prt_tors(1,i),prt_tors(2,i),prt_tors(3,i),
     +           prt_tors(4,i))
            print 20,prt_tors(1,i),prt_tors(2,i),prt_tors(3,i),
     +           prt_tors(4,i),tau(i),tau_orig(i)
         enddo
 20      format (' Torsion ',4i5,f8.1,2x,'(',f7.1,')')

         write(iout,30) ax,ax0,bb,bb0,cc,cc0,alpha,alpha0,beta,beta0,
     +        gamma,gamma0

 30      format (/,' Unit Cell Dimensions :      a    ='
     +     ,f10.4,2x,'(',f8.4,')'
     &     /,'                             b    =',f10.4,2x,'(',f8.4,')'
     &     /,'                             c    =',f10.4,2x,'(',f8.4,')'
     &     /,'                            Alpha =',f10.4,2x,'(',f8.4,')'
     &     /,'                            Beta  =',f10.4,2x,'(',f8.4,')'
     &     /,'                            Gamma =',f10.4,2x,'(',f8.4,')'
     +        )
         print 35,vol,vol0
         print 55,(nspgr+1)*(1.0d24/avogadro)*totmass/vol,
     +        (nspgr+1)*(1.0d24/avogadro)*totmass/vol0
 35      format(' Cell Volume (A^3):  ',f9.2,2x,'(',f9.3,')')
 55      format(' Density  (g/cc)  :  ',f9.4,2x,'(',f9.4,')'/)

         do i=1,nmol
            if (nmol.gt.1) write(iout,40) i
            write(iout,50) (roff(xcm1(ii,i),3),
     +           roff(xcm0(ii,i),3),ii=1,3)
            write(iout,60) roff(phi1(i),2),roff(phi0(i),2),
     +           roff(theta1(i),2),roff(theta0(i),2),
     +           roff(psi1(i),2),roff(psi0(i),2)

c Determine net rotation of molecule(s)
            rot=rotation(phi0(i),theta0(i),psi0(i),phi1(i),theta1(i),
     +           psi1(i))
c 
c
c Determine translation in Angstrom, from difference in fractional coordinates
            x12=xcm1(1,i)-xcm0(1,i)
            y12=xcm1(2,i)-xcm0(2,i)
            z12=xcm1(3,i)-xcm0(3,i)
c If origin is free in this space group, shift in that direction is not
c meaningfull for first molecule. For other molecules only with respect to first
c molecule
            if (iref(7).eq.0) x12=x12-(xcm1(1,1)-xcm0(1,1))
            if (iref(8).eq.0) y12=y12-(xcm1(2,1)-xcm0(2,1))
            if (iref(9).eq.0) z12=z12-(xcm1(3,1)-xcm0(3,1))

            trns=sqrt(x12*x12*ax*ax+y12*y12*bb*bb+z12*z12*cc*cc+
     +           2.0d0*x12*y12*ax*bb*cosc+
     +           2.0d0*x12*z12*ax*cc*cosb+
     +           2.0d0*y12*z12*bb*cc*cosa)
            write(iout,70) roff(trns,3),roff(rot,3)
         enddo
         
 40      format(' Molecule ',i4)
 50      format( ' Center of mass (frac.): ',f7.3,1x,'(',f7.3,')',
     +        f7.3,1x,'(',f7.3,')',f7.3,1x,'(',f7.3,')')
 60      format( ' Euler angles :   ',f7.2,1x,'(',f7.2,')',
     +        f7.2,1x,'(',f7.2,')',f7.2,1x,'(',f7.2,')')
 70      format(' Net translation (A), rotation (deg.) : ',f7.3,f7.3)
      endif
      

      return
      end
      

      function roff(number,power)

      real*8 roff,number,factor
      integer power
      
      factor=10**power
      roff=anint(factor*number)/factor
      return
      end

      

