c
c
c     ###################################################
c     ## Wijnand Mooij, 1997/1998                      ##
c     ##                                               ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  program xxtalmin  --  full crystal minimization         ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "xxtalmins" performs a full crystal energy minimization,
c     constraining symmetry, by variable metric optimization over the
c     atomic coordinates and the lattice parameters.
c     Crystal energy minimization code is based on UPACK program package by
c     B.P. van Eijck
      
c [1] B.P. van Eijck, W.T.M Mooij and J. Kroon, Acta Cryst B51, 99 (1995) 
c [2] W.T.M. Mooij, B.P. van Eijck, S.L. Price, P. Verwer and J. Kroon,
c     J. Comp. Chemistry 19, 459-474 (1998)
c [3] B.P. van Eijck, A.L. Spek, W.T.M. Mooij and J. Kroon, ActaCryst B54, 
c     291-299 (1998)
c [4] B.P van Eijck and J. Kroon, The upack program package for
c     crystal structure prediction. Force fields and crystal structure
c     generation for small carbohydrates. J. Comp. Chem., in press
c [5] W.T.M. Mooij, B.P. van Eijck and F.B. van Duijneveldt , Ab initio
c     intermolecular potentials. I. Derivation from methanol dimer and trimer
c     calculations, manuscript in preparation
c [6] W.T.M. Mooij, B.P. van Eijck and J. Kroon, Ab initio intermolecular
c     potentials. II. Validation and application to crystal structure
c     prediction, manuscript in preparation

     
c 
c Energy and derivatives routines (and many other subroutines) from the TINKER
c package by Jay. W. Ponder.
c
      program xxtalmin
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'boxes.i'
      include 'files.i'
      include 'iounit.i'
      include 'minima.i'
      include 'output.i'
      include 'math.i'
      include 'spgr.i'
      include 'potent.i'
      include 'potentxtra.i'
      include 'scales.i'
      include 'bound.i'
      include 'mpole.i'
      include 'polar.i'
      include 'energi.i'
      include 'energixtra.i'
      include 'inform.i'
      include 'fields.i'
      include 'atmtyp.i'
      include 'molcul.i'
      
      integer i,j,imin,nvar,ncycle,freeunit
      real*8 minimum,grdmin
      real*8 gnorm,grms,anorm,arms
      real*8 xx(maxvar),derivs1(3,maxatm),derivs2(3,maxatm)
      character*6 mode,method
      character*60 minfile,string1
      character*80 string
      character*1 answer
      logical exist,done
      external writeout2,xxtal1
      real*8  e,enb,ebond,cellderivs(6)
      real*8  grdmin0
      integer ii,iii,ni

c
c
c     set up the structure and mechanics calculation
c
      call initial
      
      call nextarg (answer,exist)
      if (.not. exist) then
         answer = 'C'
         write (iout,11)  answer
 11      format (/,' Choose CSSR or SPF file format [',a1,
     +        '] :  ',$)
         read (input,21)  answer
 21      format (a1)
      end if
      call upcase (answer)
      if (answer .eq. 'C')  coordtype='frac-cssr'
      if (answer .eq. 'S')  coordtype='frac-spf '
      
      if (coordtype.eq.'frac-cssr') call getcssr
      if (coordtype.eq.'frac-spf ') call getspf
c
c Write-out the name of the input file
c
      if (coordtype.eq.'frac-cssr') minfile = filename(1:leng)//'.cssr'
      if (coordtype.eq.'frac-spf ') minfile = filename(1:leng)//'.spf'
      call version (minfile,'old')
      write(iout,22) minfile
 22   format(' Coordinates read from:',4x,a60)
c
c     Set number of torsion to be output to zero, reference atoms for
c     trans routine to 0. If they are not set by refatms keywrd, than they are
c taken to be the first non-hydrogens in a molecule.
c
      nprt=0
      refatms(1)=0
      refatms(1)=0
      refatms(1)=0
c
      call mechanic
c      
c     Check for extra X-TINKER keywords and set up some things for crystal
c     calculations 
c
      call mechanicxtra
      call prepare
c
c     get termination criterion as RMS gradient per atom
c
      grdmin = -1.0d0
      call nextarg (string,exist)
      if (exist)  read (string,*,err=10,end=10)  grdmin
   10 continue
      if (grdmin .le. 0.0d0) then
         write (iout,20)
   20    format (/,' Enter RMS Gradient per Atom Criterion',
     &              ' [0.1] :  ',$)
         read (input,30)  grdmin
   30    format (f20.0)
      end if
      if (grdmin .le. 0.0d0)  grdmin = 0.1d0
c
c     write out a copy of coordinates for later update
c
      imin = freeunit ()
      if (coordtype.eq.'frac-cssr') minfile = filename(1:leng)//'.cssr'
      if (coordtype.eq.'frac-spf ') minfile = filename(1:leng)//'.spf'
      call version (minfile,'new')
      open (unit=imin,file=minfile,status='new')
      write(string1,12) 
 12   format('XXTALMIN:  Initial structure')
      if (coordtype.eq.'frac-cssr') call prtcssr(imin,string1)
      if (coordtype.eq.'frac-spf ') call prtspf(imin,string1)
      close (unit=imin)
      outfile = minfile

c
c     write out starting values of the crystal structure
c
      call xtalinfo('start')
c      
c Scale is out of use:
c
      scale = 1.0d0
c
c     set the parameters to be optimized: 
c     Cell-axes, depending on space-group
      nvar = 0
      do i = 1, 6
         if (iref(i).gt.0) then
            nvar = nvar +1
            xx(nvar) = box(i)
         endif
      enddo

c  Do not optimize the coordinate of the first atom if arbitrary
c  in this space-group
      
      if (iref(7).gt.0) then
         nvar = nvar + 1 
         xx(nvar) = x(1)
      endif
      if (iref(8).gt.0) then
         nvar = nvar + 1 
         xx(nvar) = y(1)
      endif
      if (iref(9).gt.0) then
         nvar = nvar + 1 
         xx(nvar) = z(1)
      endif
c      
c  Reset of the atomic coordinates
c
      do i = 2, n
         nvar = nvar + 1
         xx(nvar) = x(i)
         nvar = nvar + 1
         xx(nvar) = y(i)
         nvar = nvar + 1
         xx(nvar) = z(i)
      end do

      grdmin0=grdmin

c
c Derivatives test routine:
c
      if (testcg) call testcongra(xx,xxtal1,nvar)
c      
     
c
c Use upack variable metric/conjugate gradient minimizer:
c      
      call congra(nvar,xx,xxtal1,grdmin,minimum,writeout2)
c      
C Perform last energy calculation: Set 
c
      nvar = 0
      do i = 1, 6
         if (iref(i).gt.0) then
            nvar = nvar + 1
            box(i) = xx(nvar)
         endif
      enddo

      if (iref(7).gt.0) then
         nvar = nvar + 1 
         x(1) = xx(nvar)
      endif
      if (iref(8).gt.0) then
         nvar = nvar + 1
         y(1) = xx(nvar)
      endif
      if (iref(9).gt.0) then
         nvar = nvar + 1
         z(1) = xx(nvar)
      endif

      do i = 2, n
         nvar = nvar + 1
         x(i) = xx(nvar)
         nvar = nvar + 1
         y(i) = xx(nvar)
         nvar = nvar + 1
         z(i) = xx(nvar)
      end do
      
C Compute bonded terms
      call be(ebond,derivs1)

c compute nonbonded (vdw, multipole) terms
      call nonbe (enb,cellderivs,derivs2)

c Writeout information on energetics:
      if (eb .ne. 0.0)     print 111,'E-bond          ',eb
      if (ea .ne. 0.0)     print 111,'E-angle         ',ea
      if (eba .ne. 0.0)    print 111,'E-strbnd        ',eba
      if (eub.ne.0.0)      print 111,'E-ureyb         ',eub
      if (eaa.ne.0.0)      print 111,'E-angang        ',eaa
      if (eopb.ne.0.0)     print 111,'E-opbend        ',eopb
      if (eid.ne.0.0)      print 111,'E-impdih        ',eid
      if (eit.ne.0.0)      print 111,'E-impdih        ',eit
      if (et.ne.0.0)       print 111,'E-torsion       ',et
      if (et2.ne.0.0)      print 111,'E-torsion2      ',et2
      if (ebt.ne.0.0)      print 111,'E-strtor        ',ebt
      if (ed.ne.0.0)       print 111,'E-dipole        ',ed
      if (evintra.ne.0.0)  print 111,'E-vdwintra      ',evintra
      if (ehbintra.ne.0.0) print 111,'E-hbintra       ',ehbintra
      print 111,'E-bonded total  ',ebond
      if (em .ne. 0.0)     print 111,'E-multipole     ',em
      if (ev .ne. 0.0)     print 111,'E-Vanderwaals   ',ev
      if (edisp .ne. 0.0)  print 111,'E-dispersion    ',edisp
      if (erep .ne. 0.0)   print 111,'E-repulsion     ',erep
      if (ep .ne. 0.0)     print 111,'E-polarisation  ',ep
      print 111,'E-nonbe         ',enb
 111  format (a17,f10.4)

c
c     write out the current values of the lattice parameters
c
      call xtalinfo(' end ')
c
c     write out the final potential energy value and gradients
c
      write (iout,60)  minimum,grdmin
 60   format (/,' Final Potential Function Value :',f15.4,
     &        /,' Final RMS Gradient < ',d15.4)

c
c     write the final coordinates into a file
c
      imin = freeunit ()
      open (unit=imin,file=minfile,status='old')
      rewind (unit=imin)

      write(string1,13) minimum,enb,ebond
 13   format('XXTALMIN  Energy:',f7.2,2x,'Enb:',f7.2,2x,'Eb:',f7.2)

      if (coordtype.eq.'frac-cssr') call prtcssr(imin,string1)
      if (coordtype.eq.'frac-spf ') call prtspf(imin,string1)
      if (gamess_out) call prtgamess(.true.)
      close (unit=imin)
      end
c
c
c     ############################################################
c     ##                                                        ##
c     ##  function xtal1  --  energy and gradient for atoms ##
c     ##  and lattice parameters.                               ##
c     ##                                                        ##
c     ############################################################
c
c
c     energy and gradient routine for use with upack congra minimizer:
c     Deliveres negative gradients
      
      subroutine xxtal1 (nvarin,xx,g,en,isubr)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'usage.i'
      include 'spgr.i'
      include 'potent.i'
      include 'scales.i'
      include 'iounit.i'
      include 'math.i'

      integer i,j,nvar,nvarin,isubr
      real*8 ebond,enb,derivs1(3,maxatm),derivs2(3,maxatm),cellderivs(6)
      real*8 en,xx(maxvar),g(maxvar)
      real*8 ax,by,cz,bx,cx,cy,aa,bb,cc,grms,gnorm,cosa,cosb,cosc
      real*8 alpha,beta,gamma

cc
c     translate optimization parameters to cell parameters 
c  
      nvar = 0
      do i = 1, 6
         if (iref(i).gt.0) then
            nvar = nvar + 1
            box(i) = xx(nvar)
         endif
      enddo
      
c     
c Translate optimization parameters to first-atom coordinates, insofar not
c free in this space-group
c
      if (iref(7).gt.0) then
         nvar = nvar + 1 
         x(1) = xx(nvar)
      endif
      if (iref(8).gt.0) then
         nvar = nvar + 1 
         y(1) = xx(nvar)
      endif
      if (iref(9).gt.0) then
         nvar = nvar + 1 
         z(1) = xx(nvar)
      endif
      
c Translate optimization parameters to atom coordinates
      do i = 2, n
         nvar = nvar + 1
         x(i) = xx(nvar)
         nvar = nvar + 1
         y(i) = xx(nvar)
         nvar = nvar + 1
         z(i) = xx(nvar)
      end do
      

      if (nvarin.ne.nvar) then
         print *,'Error in xtalcongra, nvarin .ne. nvar',nvarin,nvar
      endif

c
c     compute and store the energy and gradient
c
C Bonded terms, and any special intra-molecular nonbonded terms
c
      call be(ebond,derivs1)
c
c Nonbonded terms
c Use either 'nonbe' and create a atom pairlist, or use a previously created
c pairlist by calling 'list'.
      if (isubr.eq.2) then 
         call list(enb,cellderivs,derivs2)
      else
         call nonbe(enb,cellderivs,derivs2)
      endif
      
      en = ebond + enb

c
c     store atom and cell gradients as optimization gradient
c      
      nvar=0
      do i=1,6
         if (iref(i).gt.0) then
            nvar=nvar+1
            g(nvar) = -cellderivs(i)
         endif
      enddo
      
      if (iref(7).gt.0) then
         nvar = nvar + 1 
         g(nvar) = -(derivs1(1,1) + derivs2(1,1))
      endif
      if (iref(8).gt.0) then
         nvar = nvar + 1 
         g(nvar) = -(derivs1(2,1) + derivs2(2,1))
      endif
      if (iref(9).gt.0) then
         nvar = nvar + 1 
         g(nvar) = -(derivs1(3,1) + derivs2(3,1))
      endif

      do i = 2, n
         nvar = nvar+1
         g(nvar) = -(derivs1(1,i) + derivs2(1,i))
         nvar=nvar+1
         g(nvar) = -(derivs1(2,i) + derivs2(2,i))
         nvar=nvar+1
         g(nvar) = -(derivs1(3,i) + derivs2(3,i))
      end do
      
      return
      end
