c
c     radial distribution functions for mixed solvent systems
c
c     written for Meoh/H20
c    
c     conventions:
c     solvent 1 (s1) water
c     solvent 2 (s2) other
c
c     gr(1)  O(w)-O(w)
c     gr(2)  O(w)-H(w)
c     gr(3)  H(w)-H(w)
c
c     gr(4)  O(m)-O(w)
c     gr(5)  C(m)-O(w)
c     gr(6)  H(m)-O(w)
c     gr(7)  O(m)-H(w)
c     gr(8)  C(m)-H(w)
c     gr(9)  H(m)-H(w)

c
c     variables
c     naps1    number of atoms per molecule of type s1
c
c     notes
c     loops for constructing g(r) assume 3 sites per molecule
c
      program radial
      implicit none
      include '../source/sizes.i'
      include '../source/activ.i'
      include '../source/atoms.i'
      include '../source/boxes.i'
      include '../source/charge.i'
      include '../source/cutoff.i'
      include '../source/files.i'
      include '../source/inform.i'
      include '../source/iounit.i'
      include '../source/math.i'
      include '../source/molcul.i'
      include '../source/potent.i'
      include '../source/units.i'
      integer mvvgr,mbingr,msa
      parameter (mvvgr = 9)
      parameter (mbingr = 240)
      parameter (msa = 10)
      integer i,j,k,start,stop,next,last,ifile
      integer ixyz,ikey,irduv,irdvv(mvvgr),ilog
      integer istep,nstep,indx,nsa,oct
      integer freeunit,trimtext,lext
      integer imin,imax,iimin,iimax
      integer iminvv(mvvgr),imaxvv(mvvgr)
      integer iminuv(2*msa),imaxuv(2*msa)
      integer nv1,nv2,iv1(maxatm),iv2(maxatm)
      integer w0,s1type,s2type,nfiles
      integer naps1,naps2
      integer j0,j1,j2,k0,k1,k2,nvvmx,mxbin
      integer noxox(0:mbingr),nhyhy(0:mbingr),noxhy(0:mbingr)
      integer nomow(0:mbingr),ncmow(0:mbingr),nhmow(0:mbingr)
      integer nomhw(0:mbingr),ncmhw(0:mbingr),nhmhw(0:mbingr)
      integer nuvo(0:mbingr,msa),nuvh(0:mbingr,msa)
      integer nvv(0:mbingr,mvvgr),nuv(0:mbingr,2*msa)
      real*8 gr(0:mbingr,mvvgr),runcor(0:mbingr,mvvgr)
      real*8 gruv(0:mbingr,2*msa),runcoruv(0:mbingr,2*msa)
      real*8 bin,fbin,gnorm,volume,gnrm,fnrm
      real*8 rsum,rdf,qint,upper,lower
      real*8 pr(mvvgr),fpr(mvvgr)
      real*8 pruv,fpruv
      real*8 xo1,yo1,zo1,xha1,yha1,zha1,xhb1,yhb1,zhb1
      real*8 xo2,yo2,zo2,xha2,yha2,zha2,xhb2,yhb2,zhb2
      real*8 r,xc,yc,zc,xoo,yoo,zoo
      real*8 xo1ha2,yo1ha2,zo1ha2,xo1hb2,yo1hb2,zo1hb2
      real*8 xha1o2,yha1o2,zha1o2,xhb1o2,yhb1o2,zhb1o2
      real*8 xha1ha2,yha1ha2,zha1ha2,xha1hb2,yha1hb2,zha1hb2
      real*8 xhb1ha2,yhb1ha2,zhb1ha2,xhb1hb2,yhb1hb2,zhb1hb2
      real*8 xu,yu,zu,xou,you,zou,xhau,yhau,zhau,xhbu,yhbu,zhbu
      character*1 letter,answer
      character*7 ext
      character*10 name
      character*20 keyword
      character*60 keyfile,coordfile,logfile
      character*60 rduvfile,rdvvfile(mvvgr)
      character*80 record,string
      character*11 text(mvvgr)
      logical done,exist
      external freeunit,trimtext
      equivalence (noxox(0),nvv(0,1))
      equivalence (noxhy(0),nvv(0,2))
      equivalence (nhyhy(0),nvv(0,3))
      equivalence (nomow(0),nvv(0,4))
      equivalence (ncmow(0),nvv(0,5))
      equivalence (nhmow(0),nvv(0,6))
      equivalence (nomhw(0),nvv(0,7))
      equivalence (ncmhw(0),nvv(0,8))
      equivalence (nhmhw(0),nvv(0,9))
      equivalence (nuvo(0,1),nuv(0,1))
      equivalence (nuvh(0,1),nuv(0,msa+1))
      data text/' O(w)-O(w) ',' O(w)-H(w) ',' H(w)-H(w) '   
     :         ,' O(m)-O(w) ',' C(m)-O(w) ',' H(m)-O(w) '
     :         ,' O(m)-H(w) ',' C(m)-H(w) ',' H(m)-H(w) '/
c
c     define some constants
c   
      nsa = 0
      bin = 0.1d0
      fbin = 1.0d0 / bin
c     wrong but, hey:
c     oct = sqrt(3.0d0)/2.0d0
      oct = 1.0d0
c
c     get the name of the coordinates files
c
      call initial
      write (iout,10)
   10 format (' Enter Name of the Coordinate Files :  ',$)
      read (input,20)  filename
   20 format (a60)
      leng = trimtext (filename)
c
c     remove any extension from the filename
c
      last = leng
      do i = 1, leng
         letter = filename(i:i)
         if (letter .eq. '/')  last = leng
         if (letter .eq. ']')  last = leng
         if (letter .eq. ':')  last = leng
         if (letter .eq. '.')  last = i - 1
      end do
      leng = min(leng,last)
c
c     get the numbers of the files to be used
c
      write (iout,30)
   30 format (' Numbers of First and Last File to Analyze :  ',$)
      read (input,40)  start,stop
   40 format (2i10)
c
c     open various Gnuplot data files
c
      ilog = freeunit ()
      logfile = filename(1:leng)//'.radlog'
      call version (logfile,'new')
      open (unit=ilog,file=logfile,status='new')
      irdvv(1) = freeunit ()
      rdvvfile(1) = filename(1:leng)//'.rdfoo'
      call version (rdvvfile(1),'new')
      open (unit=irdvv(1),file=rdvvfile(1),status='new')
      irdvv(2) = freeunit ()
      rdvvfile(2) = filename(1:leng)//'.rdfoh'
      call version (rdvvfile(2),'new')
      open (unit=irdvv(2),file=rdvvfile(2),status='new')
      irdvv(3) = freeunit ()
      rdvvfile(3) = filename(1:leng)//'.rdfhh'
      call version (rdvvfile(3),'new')
      open (unit=irdvv(3),file=rdvvfile(3),status='new')
      irdvv(4) = freeunit ()
      rdvvfile(4) = filename(1:leng)//'.rdfomow'
      call version (rdvvfile(4),'new')
      open (unit=irdvv(4),file=rdvvfile(4),status='new')
      irdvv(5) = freeunit ()
      rdvvfile(5) = filename(1:leng)//'.rdfcmow'
      call version (rdvvfile(5),'new')
      open (unit=irdvv(5),file=rdvvfile(5),status='new')
      irdvv(6) = freeunit ()
      rdvvfile(6) = filename(1:leng)//'.rdfhmow'
      call version (rdvvfile(6),'new')
      open (unit=irdvv(6),file=rdvvfile(6),status='new')
      irdvv(7) = freeunit ()
      rdvvfile(7) = filename(1:leng)//'.rdfomhw'
      call version (rdvvfile(7),'new')
      open (unit=irdvv(7),file=rdvvfile(7),status='new')
      irdvv(8) = freeunit ()
      rdvvfile(8) = filename(1:leng)//'.rdfcmhw'
      call version (rdvvfile(8),'new')
      open (unit=irdvv(8),file=rdvvfile(8),status='new')
      irdvv(9) = freeunit ()
      rdvvfile(9) = filename(1:leng)//'.rdfhmhw'
      call version (rdvvfile(9),'new')
      open (unit=irdvv(9),file=rdvvfile(9),status='new')
c
c     find the name of the next coordinates file
c
      ifile = start
      istep = 0
      dowhile (ifile.ge.start .and. ifile.le.stop)
         call numeral (ifile,ext,lext)
         coordfile = filename(1:leng)//'.'//ext(1:lext)
         call version (coordfile,'old')
c
c     read in the next molecular dynamics coordinate frame
c
         ixyz = freeunit ()
  280    continue
         open (unit=ixyz,file=coordfile,status='old',err=280)
         call readxyz (ixyz)
         close (unit=ixyz)
         istep = istep + 1
c
c     set up the first time through
c
         if (istep .eq. 1) then
            call mechanic
c
c     get system volume
c
         if (orthogonal) volume = xbox * ybox * zbox
         if (octahedron) volume = 0.5d0 * xbox * ybox * zbox
         gnorm = 3.0d0 * volume / (bin)**3 / 4.0d0 / pi
         nfiles = stop - start + 1
c
c      make compact arrays of locations of marker atoms for
c      each molecule by molecule type
c
         s1type = 70
         s2type = 73
         naps1 = 3
         naps2 = 3
         nv1 = 0
         nv2 = 0
         i = 1
   5     continue
         if (itype(i) .eq. s1type) then
            nv1 = nv1 + 1
            iv1(nv1) = i
            i = i + naps1
         else if (itype(i) .eq. s2type) then
            nv2 = nv2 + 1
            iv2(nv2) = i
            i = i + naps2
         end if
         if (i .le. n) goto 5
c
c     get normalization constants
c
         pr(1) = nfiles * nv1 * (nv1-1) / 2.0d0
         pr(2) = 4.0d0 * pr(1)
         pr(3) = pr(2)
         pr(4) = nfiles * nv1 * nv2
         pr(5) = pr(4)
         pr(6) = pr(4)
         pr(7) = 2.0d0 * pr(4)
         pr(8) = pr(7)
         pr(9) = pr(8)
         fpr(1) = 2.0d0 / dble(nfiles*nv1)
         fpr(2) = 0.5d0 * fpr(1)
         fpr(3) = fpr(2)
         fpr(4) = 1.0d0/ nfiles / nv2
         fpr(5) = fpr(4)
         fpr(6) = fpr(4)
         fpr(7) = fpr(4)
         fpr(8) = fpr(4)
         fpr(9) = fpr(4)
c        pruv = nfiles * nv1
c        fpruv = 1.0d0 / dble(nfiles)
         if ( nsa .gt. 0 ) then
            irduv = freeunit ()
            rduvfile = filename(1:leng)//'.rdfuv'
            call version (rduvfile,'new')
            open (unit=irduv,file=rduvfile,status='new')
         end if
c
c     Longest correlation length ( assumes oct or cube)
c
         nvvmx = nint(oct*fbin*chgcut)
         mxbin = nvvmx
         end if
c
c     accumulate statistics for Water-Water
c
            do j = 2, nv1
               j0 = iv1(j)
               j1 = j0 + 1
               j2 = j0 + 2
               xo1 = x(j0)
               yo1 = y(j0)
               zo1 = z(j0)
               xha1 = x(j1)
               yha1 = y(j1)
               zha1 = z(j1)
               xhb1 = x(j2)
               yhb1 = y(j2)
               zhb1 = z(j2)
c
c     Solvent - Solvent correlation
c
               do k = 1, j-1
                  k0 = iv1(k)
                  k1 = k0 + 1
                  k2 = k0 + 2
                  xo2 = x(k0)
                  yo2 = y(k0)
                  zo2 = z(k0)
                  xha2 = x(k1)
                  yha2 = y(k1)
                  zha2 = z(k1)
                  xhb2 = x(k2)
                  yhb2 = y(k2)
                  zhb2 = z(k2)
c
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  xoo = xc
                  yoo = yc
                  zoo = zc
                  xo1ha2 = xo1 - xha2
                  yo1ha2 = yo1 - yha2
                  zo1ha2 = zo1 - zha2
                  xo1hb2 = xo1 - xhb2
                  yo1hb2 = yo1 - yhb2
                  zo1hb2 = zo1 - zhb2
                  xha1o2 = xha1 - xo2
                  yha1o2 = yha1 - yo2
                  zha1o2 = zha1 - zo2
                  xhb1o2 = xhb1 - xo2
                  yhb1o2 = yhb1 - yo2
                  zhb1o2 = zhb1 - zo2
                  xha1ha2 = xha1 - xha2
                  yha1ha2 = yha1 - yha2
                  zha1ha2 = zha1 - zha2
                  xha1hb2 = xha1 - xhb2
                  yha1hb2 = yha1 - yhb2
                  zha1hb2 = zha1 - zhb2
                  xhb1ha2 = xhb1 - xha2
                  yhb1ha2 = yhb1 - yha2
                  zhb1ha2 = zhb1 - zha2
                  xhb1hb2 = xhb1 - xhb2
                  yhb1hb2 = yhb1 - yhb2
                  zhb1hb2 = zhb1 - zhb2
c
c     Periodic Boundary Conditions
c
                  call image2(xc,yc,zc,xoo,yoo,zoo)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xo1ha2,yo1ha2,zo1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xo1hb2,yo1hb2,zo1hb2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1o2,yha1o2,zha1o2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1o2,yhb1o2,zhb1o2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1ha2,yha1ha2,zha1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1hb2,yha1hb2,zha1hb2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1ha2,yhb1ha2,zhb1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1hb2,yhb1hb2,zhb1hb2)
c
c     Ox - Ox correlation
c
                  r = sqrt(xoo**2 + yoo**2 + zoo**2)
                  indx = nint(r*fbin)
                  noxox(indx) = noxox(indx) + 1
c
c     Ox - Hy correlation
c
                  r = sqrt(xo1ha2**2 + yo1ha2**2 + zo1ha2**2)
                  indx = nint(r*fbin)
                  noxhy(indx) = noxhy(indx) + 1
c
                  r = sqrt(xo1hb2**2 + yo1hb2**2 + zo1hb2**2)
                  indx = nint(r*fbin)
                  noxhy(indx) = noxhy(indx) + 1
c
                  r = sqrt(xha1o2**2 + yha1o2**2 + zha1o2**2)
                  indx = nint(r*fbin)
                  noxhy(indx) = noxhy(indx) + 1
c
                  r = sqrt(xhb1o2**2 + yhb1o2**2 + zhb1o2**2)
                  indx = nint(r*fbin)
                  noxhy(indx) = noxhy(indx) + 1
c
c     Hy - Hy correlation
c
                  r = sqrt(xha1ha2**2 + yha1ha2**2 + zha1ha2**2)
                  indx = nint(r*fbin)
                  nhyhy(indx) = nhyhy(indx) + 1
c
                  r = sqrt(xha1hb2**2 + yha1hb2**2 + zha1hb2**2)
                  indx = nint(r*fbin)
                  nhyhy(indx) = nhyhy(indx) + 1
c
                  r = sqrt(xhb1ha2**2 + yhb1ha2**2 + zhb1ha2**2)
                  indx = nint(r*fbin)
                  nhyhy(indx) = nhyhy(indx) + 1
c
                  r = sqrt(xhb1hb2**2 + yhb1hb2**2 + zhb1hb2**2)
                  indx = nint(r*fbin)
                  nhyhy(indx) = nhyhy(indx) + 1
               end do
           end do
c
c     accumulate statistics for MeOH-water
c
            do j = 1, nv1
               j0 = iv1(j)
               j1 = j0 + 1
               j2 = j0 + 2
               xo1 = x(j0)
               yo1 = y(j0)
               zo1 = z(j0)
               xha1 = x(j1)
               yha1 = y(j1)
               zha1 = z(j1)
               xhb1 = x(j2)
               yhb1 = y(j2)
               zhb1 = z(j2)
c
c     MeOH molecule ( here variables *ha2 refer to CH3)    
c
               do k = 1, nv2
                  k0 = iv2(k)
                  k1 = k0 + 1
                  k2 = k0 + 2
                  xo2 = x(k0)
                  yo2 = y(k0)
                  zo2 = z(k0)
                  xha2 = x(k1)
                  yha2 = y(k1)
                  zha2 = z(k1)
                  xhb2 = x(k2)
                  yhb2 = y(k2)
                  zhb2 = z(k2)
c
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  xoo = xc
                  yoo = yc
                  zoo = zc
                  xo1ha2 = xo1 - xha2
                  yo1ha2 = yo1 - yha2
                  zo1ha2 = zo1 - zha2
                  xo1hb2 = xo1 - xhb2
                  yo1hb2 = yo1 - yhb2
                  zo1hb2 = zo1 - zhb2
                  xha1o2 = xha1 - xo2
                  yha1o2 = yha1 - yo2
                  zha1o2 = zha1 - zo2
                  xhb1o2 = xhb1 - xo2
                  yhb1o2 = yhb1 - yo2
                  zhb1o2 = zhb1 - zo2
                  xha1ha2 = xha1 - xha2
                  yha1ha2 = yha1 - yha2
                  zha1ha2 = zha1 - zha2
                  xha1hb2 = xha1 - xhb2
                  yha1hb2 = yha1 - yhb2
                  zha1hb2 = zha1 - zhb2
                  xhb1ha2 = xhb1 - xha2
                  yhb1ha2 = yhb1 - yha2
                  zhb1ha2 = zhb1 - zha2
                  xhb1hb2 = xhb1 - xhb2
                  yhb1hb2 = yhb1 - yhb2
                  zhb1hb2 = zhb1 - zhb2
c
c     Periodic Boundary Conditions
c
                  call image2(xc,yc,zc,xoo,yoo,zoo)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xo1ha2,yo1ha2,zo1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xo1hb2,yo1hb2,zo1hb2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1o2,yha1o2,zha1o2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1o2,yhb1o2,zhb1o2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1ha2,yha1ha2,zha1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xha1hb2,yha1hb2,zha1hb2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1ha2,yhb1ha2,zhb1ha2)
                  xc = xo1 - xo2
                  yc = yo1 - yo2
                  zc = zo1 - zo2
                  call image2(xc,yc,zc,xhb1hb2,yhb1hb2,zhb1hb2)
c
c     Ox(m) - Ox(w) correlation
c
                  r = sqrt(xoo**2 + yoo**2 + zoo**2)
                  indx = nint(r*fbin)
                  nomow(indx) = nomow(indx) + 1
c
c     C(m) - Ox(w) correlation
c
                  r = sqrt(xo1ha2**2 + yo1ha2**2 + zo1ha2**2)
                  indx = nint(r*fbin)
                  ncmow(indx) = ncmow(indx) + 1
c
c     H(m) - Ox(w) correlation
c
                  r = sqrt(xo1hb2**2 + yo1hb2**2 + zo1hb2**2)
                  indx = nint(r*fbin)
                  nhmow(indx) = nhmow(indx) + 1
c
c     Ox(m) - Hy(w) correlation
c
                  r = sqrt(xha1o2**2 + yha1o2**2 + zha1o2**2)
                  indx = nint(r*fbin)
                  nomhw(indx) = nomhw(indx) + 1
c
                  r = sqrt(xhb1o2**2 + yhb1o2**2 + zhb1o2**2)
                  indx = nint(r*fbin)
                  nomhw(indx) = nomhw(indx) + 1
c
c     C(m) - Hy(w) correlation
c
                  r = sqrt(xha1ha2**2 + yha1ha2**2 + zha1ha2**2)
                  indx = nint(r*fbin)
                  ncmhw(indx) = ncmhw(indx) + 1
c
                  r = sqrt(xhb1ha2**2 + yhb1ha2**2 + zhb1ha2**2)
                  indx = nint(r*fbin)
                  ncmhw(indx) = ncmhw(indx) + 1
c
c     H(m) - Hy(w) correlation
c
                  r = sqrt(xha1hb2**2 + yha1hb2**2 + zha1hb2**2)
                  indx = nint(r*fbin)
                  nhmhw(indx) = nhmhw(indx) + 1
c
                  r = sqrt(xhb1hb2**2 + yhb1hb2**2 + zhb1hb2**2)
                  indx = nint(r*fbin)
                  nhmhw(indx) = nhmhw(indx) + 1
               end do
           end do
c  skip for now
           goto 567
c
c     Solute - Solvent correlation
c
           do j = 1, nv1
               do k = 1, nsa
                  xu = x(k)
                  yu = y(k)
                  zu = z(k)
                  xc = xo1 - xu
                  yc = yo1 - yu
                  zc = zo1 - zu
                  xou = xc
                  you = yc
                  zou = zc
                  xhau = xha1 - xu
                  yhau = yha1 - yu
                  zhau = zha1 - zu
                  xhbu = xhb1 - xu
                  yhbu = yhb1 - yu
                  zhbu = zhb1 - zu
c
c     Periodic Boundary Conditions
c
                  call image2(xc,yc,zc,xou,you,zou)
                  xc = xo1 - xu
                  yc = yo1 - yu
                  zc = zo1 - zu
                  call image2(xc,yc,zc,xhau,yhau,zhau)
                  xc = xo1 - xu
                  yc = yo1 - yu
                  zc = zo1 - zu
                  call image2(xc,yc,zc,xhbu,yhbu,zhbu)
c
c     Solute - Ox correlation
c
                  r = sqrt(xou**2 + you**2 + zou**2)
                  indx = nint(r*fbin)
                  nuvo(indx,k) = nuvo(indx,k) + 1
c
c     Solute - Hy correlation
c
                  r = sqrt(xhau**2 + yhau**2 + zhau**2)
                  indx = nint(r*fbin)
                  nuvh(indx,k) = nuvh(indx,k) + 1
                  r = sqrt(xhbu**2 + yhbu**2 + zhbu**2)
                  indx = nint(r*fbin)
                  nuvh(indx,k) = nuvh(indx,k) + 1
               end do
            end do
  567    continue
         ifile = ifile + 1
      end do
c
c     bracket each distribution function 
c

c     reprogram from here

      do j = 1, mvvgr
         imax = mbingr + 1
 25      imax = imax - 1
         if ( nvv(imax,j) .eq. 0) goto 25
         imaxvv(j) = imax
         imin = -1
 35      imin = imin + 1
         if ( nvv(imin,j) .eq. 0) goto 35
         iminvv(j) = imin
      end do
      do j = 1, nsa
         imax = mbingr + 1
 46      imax = imax - 1
         if ( nuv(imax,j) .eq. 0) goto 46
         imaxuv(j) = imax
         imin = -1
 56      imin = imin + 1
         if ( nuv(imin,j) .eq. 0) goto 56
         iminuv(j) = imin
      end do
      do j = msa+1, msa+nsa
         imax = mbingr + 1
 45      imax = imax - 1
         if ( nuv(imax,j) .eq. 0) goto 45
         imaxuv(j) = imax
         imin = -1
 55      imin = imin + 1
         if ( nuv(imin,j) .eq. 0) goto 55
         iminuv(j) = imin
      end do
      do j = 1, mvvgr
         imax = imaxvv(j)
         imin = iminvv(j)
         gnrm = gnorm / pr(j)
         fnrm = fpr(j)
         rsum = 0.0d0
         if (imin .eq. 0) then
            rsum = 0.5d0 * dble(nvv(0,j))
            runcor(0,j) = qint
            rdf = gnrm*nvv(0,j)
            rdf = rdf / (0.5d0)**3
            gr(0,j) = rdf
         end if
         do i = imin +1, imax
c
c     Running Coordination Number
c
            rsum = rsum + 0.5d0 * dble(nvv(i-1,j) + nvv(i,j))
            qint = rsum * fnrm
            runcor(i,j) = qint
c
c     Radial Distribution Function
c 
            rdf = gnrm * nvv(i,j)
            upper = dble(i) + 0.5d0
            lower = upper - 1.0d0
            rdf = rdf / (upper**3 - lower**3)
            gr(i,j) = rdf
         end do
c
c     Largest Maximum of correlation function
c
         iimax = imin
         do i = imin + 1, imax
            if ( gr(i,j) .gt. gr(iimax,j)) iimax = i
         end do
         r = dble(iimax) * bin
         write(ilog,*) text(j)
         write(ilog,101) r,gr(iimax,j),runcor(iimax,j)
  101 format (/,' Largest Maximum at R = ', f5.3,' ang.',
     :             ' amplitude',
     :             f7.3,' Coordination number: ',f7.3)
c
c     Following minimum of correlation function
c
         iimin = iimax
         do i = iimax+1, nvvmx
            if (gr(i,j) .lt. gr(iimin,j)) iimin = i
         end do
         r = dble(iimin) * bin
         write(ilog,103) r,gr(iimin,j),runcor(iimin,j)
  103    format (/,' Following Minimum at R = ', f8.3,
     :             ' ang.',' amplitude',
     :             f7.3,' Coordination number: ',f7.3)
      end do
      gnrm = gnorm / pruv
      fnrm = fpruv
      do j = 1, nsa
         imax = imaxuv(j)
         imin = iminuv(j)
         rsum = 0.0d0
         if (imin .eq. 0) then
            rsum = 0.5d0 * dble(nuv(0,j))
            runcoruv(0,j) = qint
            rdf = gnrm*nuv(0,j)
            rdf = rdf / (0.5d0)**3
            gruv(0,j) = rdf
         end if
         do i = imin +1, imax
c
c     Running Coordination Number
c
            rsum = rsum + 0.5d0 * dble(nuv(i-1,j) + nuv(i,j))
            qint = rsum * fnrm
            runcoruv(i,j) = qint
c
c     Radial Distribution Function
c 
            rdf = gnrm * nuv(i,j)
            upper = dble(i) + 0.5d0
            lower = upper - 1.0d0
            rdf = rdf / (upper**3 - lower**3)
            gruv(i,j) = rdf
         end do
c
c     Largest Maximum of correlation function
c
         iimax = imin
         do i = imin + 1, imax
            if ( gruv(i,j) .gt. gruv(iimax,j)) iimax = i
         end do
         r = dble(iimax) * bin
         write(ilog,*) 'Solute:solvent'
         write(ilog,101) r,gruv(iimax,j),runcoruv(iimax,j)
c
c     Following minimum of correlation function
c
         iimin = iimax
         do i = iimax+1, nvvmx
            if (gruv(i,j) .lt. gruv(iimin,j)) iimin = i
         end do
         r = dble(iimin) * bin
         write(ilog,103) r,gruv(iimin,j),runcoruv(iimin,j)
      end do
      pruv = 2.0d0 * pruv
      gnrm = gnorm / pruv
      fnrm = fpruv
      do j = msa+1, msa+nsa
         imax = imaxuv(j)
         imin = iminuv(j)
         rsum = 0.0d0
         if (imin .eq. 0) then
            rsum = 0.5d0 * dble(nuv(0,j))
            runcoruv(0,j) = qint
            rdf = gnrm*nuv(0,j)
            rdf = rdf / (0.5d0)**3
            gruv(0,j) = rdf
         end if
         do i = imin +1, imax
c
c     Running Coordination Number
c
            rsum = rsum + 0.5d0 * dble(nuv(i-1,j) + nuv(i,j))
            qint = rsum * fnrm
            runcoruv(i,j) = qint
c
c     Radial Distribution Function
c 
            rdf = gnrm * nuv(i,j)
            upper = dble(i) + 0.5d0
            lower = upper - 1.0d0
            rdf = rdf / (upper**3 - lower**3)
            gruv(i,j) = rdf
         end do
c
c     Largest Maximum of correlation function
c
         iimax = imin
         do i = imin + 1, imax
            if ( gruv(i,j) .gt. gruv(iimax,j)) iimax = i
         end do
         r = dble(iimax) * bin
         write(ilog,101) r,gruv(iimax,j),runcoruv(iimax,j)
c
c     Following minimum of correlation function
c
         iimin = iimax
         do i = iimax+1, nvvmx
            if (gruv(i,j) .lt. gruv(iimin,j)) iimin = i
         end do
         r = dble(iimin) * bin
         write(ilog,103) r,gruv(iimin,j),runcoruv(iimin,j)
      end do
      do j = 1, mvvgr
         do i = iminvv(j), imaxvv(j)
            if ( i .le. mxbin)  then
               r = i*bin
               write(irdvv(j),*) r,gr(i,j),runcor(i,j)
            end if
         end do
      end do
      do j = 1, nsa
         do i = iminuv(j), imaxuv(j)
            if ( i .le. mxbin)  then
               r = i*bin
               write(irduv,*) r,gruv(i,j),runcoruv(i,j)
            end if
         end do
      end do
      if (nsa.gt.0) write(irduv,*) 'Hyd Correlations'
      do j = msa+1, msa+nsa
         do i = iminuv(j), imaxuv(j)
            if ( i .le. mxbin)  then
               r = i*bin
               write(irduv,*) r,gruv(i,j),runcoruv(i,j)
            end if
         end do
      end do
      end 
