c
c
c
c     ##############################################################
c     ##                                                          ##
c     ##  program agdarigd-- adiabatic Gaussian Density Annealing ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "agdargd" implements the Adiabatic Gaussian  Density Annealing 
c     Method for global optimization in rigid body space. 
c     
c
c     literature reference :
c
c     J. Ma and J. E. Straub, "Simulated Annealing using the
c     Classical Density Distribution", Journal of Chemical Physics,
c     101, 533-541 (1994)
c
c
c     C.Tsoo and C.L. Brooks III "Cluster structure determintation
c     using Gaussian density distribution global minimization 
c     methods", Journal of Chemical Physics, 101, 6405-6411. (1994)
c
      program agdargd
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'files.i'
      include 'iounit.i'
      include 'minima.i'
      include 'potent.i'
      include 'warp.i'
      include 'group.i'
      include 'rigid.i'
      integer maxgda,maxrb
      parameter (maxgda=maxatm)
      parameter (maxrb=6*maxgrp)
      integer i,j,igda,lext,freeunit
      integer nvar,nok,nbad
      integer nstep,itrial
      real*8 bstart,bstop
      real*8 tol,h1,hmin
      real*8 minimum,grdmin
      real*8 agda1,agda2
      real*8 m2init(maxatm)
      real*8 xx(maxgda),rgd(maxgrp)
      real*8 last
      character*7 ext,status
      character*60 gdafile
      character*80 record,string
      logical exist
      external agda1,agda2,writeout
c
c
c     initialize and get the structure to be optimized
c
      call initial
      call getxyz
c
c     set flag for GDA method, then set up molecular mechanics
c
      use_gda = .true.
      call mechanic
      call orient
c
c     store the initial values of the squared mean Gaussian width
c
      do i = 1, n
         m2init(i) = m2(1)
      end do
c
c     get initial and final values of inverse temperature
c
      bstart = -1.0d0
      bstop = -1.0d0
      call nextarg (string,exist)
      if (exist)  read (string,*,err=60,end=60)  bstart
      call nextarg (string,exist)
      if (exist)  read (string,*,err=60,end=60)  bstop
   60 continue
      if (bstart.le.0.0d0 .or. bstop.le.0.0d0) then
         write (iout,70)
   70    format (/,' Enter Initial and Final Beta [0.01, 10**10] :  ',$)
         read (input,80)  record
   80    format (a80)
         read (record,*,err=90,end=90)  bstart,bstop
   90    continue
      end if
      if (bstart .le. 0.0d0)  bstart = 0.01d0
      if (bstop .le. 0.0d0)  bstop = 1.0d10
c
c     write out a copy of coordinates for later update
c
      itrial = 1
      lext = 3
      call numeral (itrial,ext,lext)
      gdafile = filename(1:leng)//'.'//ext(1:lext)
      call version (gdafile,'new')
      igda = freeunit ()
      open (unit=igda,file=gdafile,status='new')
      call prtxyz (igda)
      close (unit=igda)
      outfile = gdafile
c     
c     translate M2's to parameters for numerical integration
c     
      nvar = 0
      do i = 1, n
        nvar = nvar + 1
        xx(nvar) = m2init(i)
      end do
c
c     make changes to the potential to use potential smoothing
c     
      use_deform = .true.
      use_geom = .true.
      use_lj = .false.
      use_gauss = .true.
c     
c     make the call to the Bulirsch-Stoer integration routine
c
      nstep = 0
      status = '       '
      tol = 1.0d-8
      h1 = 0.1d0
      hmin = 0.0d0
      write (iout,100)
  100 format (//,' Adiabatic Gaussian Density Annealing   :',
     &        //,' BS Step',5x,'Kelvin   ',6x,'Energy',
     &        9x,'Rg',8x,'Log(M2)',7x,'Status',/)
      last = 0.0d0
      call gdastat (nstep,bstart,xx,status,last)
      call diffeq (nvar,xx,bstart,bstop,tol,h1,hmin,nok,nbad,
     &             agda1,last)
      nstep = nok + nbad
c
c     make changes to the potential for standard optimization
c     
      use_deform = .false.
      use_geom = .false.
      use_lj = .true.
      use_gauss = .false.
c     
c     make the call to the energy minimization routine
c     
      nvar = 0
      do i = 1,ngrp
        do j = 1,6
          nvar = nvar + 1
          rgd(nvar) = rbc(j,i)
        end do
      end do
      grdmin = 0.0001d0
      nextiter = nstep + 1
      call vmetric (nvar,rgd,minimum,grdmin,agda2,writeout)
      write (iout,110)  minimum
  110 format (/,' Global Energy Minimum :',f15.4)
c
c     translate optimization parameters into rigid body coordinates
c     
      nvar = 0
      do i = 1, ngrp
        do j = 1,6
          nvar = nvar + 1
          rbc(j,i) = rgd(nvar)
        end do
      end do
c     
c     write the final coordinates into a file
c     
      igda = freeunit ()
      open (unit=igda,file=gdafile,status='old')
      rewind (unit=igda)
      call prtxyz (igda)
      close (unit=igda)
      end
c
c
c     ##################################################################
c     ##                                                              ##
c     ##  function agda1  --  gradient for gaussian density annealing ##
c     ##                                                              ##
c     ##################################################################
c
c
      function agda1 (beta,xx,g)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'iounit.i'
      include 'warp.i'
      integer maxgda
      parameter (maxgda=maxatm)
      integer i,nvar
      integer hinit(maxvar),hstop(maxvar)
      integer hindex(maxhess)
      real*8 energy
      real*8 agda1,beta,sum
      real*8 hdiag(maxvar),h(maxhess)
      real*8 xx(maxgda),g(maxgda)
c
c
c     translate optimization parameters to M2's
c
      nvar = 0
      do i = 1, n
        nvar = nvar + 1
        m2(i) = xx(nvar)
        if (m2(i) .lt. 0.0d0) then
          m2(i) = -m2(i)
        end if
      end do
c     
c     compute and store the energy and gradient
c
      agda1 = energy ()
c
c
c     compute and store the Hessian elements
c
      call hessian (h,hinit,hstop,hindex,hdiag)
c
c     translate the Hessian diagonal into a dM2/dbeta vector
c
      nvar = 0
      do i = 1, n
        nvar = nvar + 1
        sum = hdiag(3*i-2) + hdiag(3*i-1) + hdiag(3*i)
        g(nvar) = -(m2(i)/3.0d0)**2 * sum 
      end do
      return
      end
c
c
c     ##############################################################
c     ##                                                          ##
c     ##  function agda2  --  energy/gradient for rigid body AGDA ##
c     ##                                                          ##
c     ##############################################################
c
c
      function agda2 (xx,g)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'group.i'
      include 'rigid.i'
      integer i,j,nvar
      real*8 agda2,e,derivs(6,maxgrp)
      real*8 xx(maxvar),g(maxvar)
c
c
c     translate optimization parameters to atomic coordinates
c
      nvar = 0
      do i = 1, ngrp
        do j = 1,6
          nvar = nvar + 1
          rbc(j,i) = xx(nvar)
        end do
      end do
c
c     compute and store the energy and gradient
c
      call rigidxyz
      call gradrgd (e,derivs)
      agda2 = e
c
c     store rigid body gradients as optimization gradient
c
      nvar = 0
      do i = 1, ngrp
        do j = 1,6
          nvar = nvar + 1
          xx(nvar) = rbc(j,i)
          g(nvar) = derivs(j,i)
        end do
      end do
      return
      end
c
c
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine gdastat  --  compute GDA trajectory averages  ##
c     ##                                                           ##
c     ###############################################################
c
c
      subroutine gdastat (nstep,beta,xx,status,last)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'iounit.i'
      include 'math.i'
      include 'warp.i'
      include 'minima.i'
      include 'inform.i'
      include 'units.i'
      include 'group.i'
      include 'rigid.i'
      include 'output.i'
      include 'kvdws.i'
      integer maxgda
      parameter (maxgda=maxatm)
      integer i,j,nstep,oldprt
      integer nvar,nmin,nxyz
      integer neigen
      real*8 beta,xx(maxgda)
      real*8 xc(maxvar),rgd(maxvar)
      real*8 e,rg,m2ave,temp
      real*8 grdmin,minimum
      real*8 mean,std
      real*8 last,curnt
      real*8 tol,energy
      logical oldverb,use_local
      character*7 status
      external agda2,writeout
c
c
c     translate optimization parameters to M2's
c
      nvar = 0
      do i = 1, n
        nvar = nvar + 1
        m2(i) = abs(xx(nvar))
      end do
c
c     translate rigid body coordinates to optimization parameters
      nmin = 0
      do i = 1, ngrp
        do j = 1,6
          nmin = nmin + 1
          rgd(nmin) = rbc(j,i) 
        end do
      end do
c
c
c     make the call to the energy minimization routine
c
      grdmin = 0.0001d0
      oldverb = verbose
      oldprt = iprint
      verbose = .false.
      iprint = 0
      coordtype= 'rigidbody' 
      call vmetric (nmin,rgd,minimum,grdmin,agda2,writeout)

c     translate optimization parameters to rigid body coordinates
      nmin = 0
      do i = 1, ngrp
        do j = 1,6
          nmin = nmin + 1
          rbc(j,i) = rgd(nmin)
        end do
      end do
      call rigidxyz

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      
c      temp = 1.0d0/(gasconst*beta)
c      use_local = .false.
c      nvar = 0 
c      do i =1,n
c         nvar = nvar + 1
c         xx(nvar) = m2(i)/rad(type(i))
c      end do
c      call stat(n,xx,mean,std)
c      curnt = beta*std
c      write(iout,*)beta,mean,std,beta*std
c      tol = 1.0d-4
c      if(curnt .ge. last+tol) then
c        use_local = .true.
c      end if
c
c      last = curnt
c      neigen = 5
c      if(use_local) call loclsrch (neigen,minimum)      
c      call rigidxyz
c      e = energy ()
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc      

      call gyrate (rg)
      m2ave = 0.0d0
      do i = 1, n
        m2ave = m2ave + m2(i)
      end do
      m2ave = m2ave / dble(n)
c     
c
      temp = 1.0d0/(gasconst*beta)
      write (iout,10)  nstep,temp,minimum,rg,
     &                 log(m2ave)/logten,status
   10 format (i6,2x,4f13.4,6x,a7)

c     save the current coordinates to a disk file
      nvar = 0
      do i = 1,n
         nvar = nvar + 1
         xx(nvar) = m2(i)
      end do
c      
      nxyz = 0
      do i = 1, n
        nxyz=nxyz + 1
        xc(nxyz)=x(i) 
        nxyz = nxyz + 1
        xc(nxyz)=y(i) 
        nxyz = nxyz + 1
        xc(nxyz)=z(i)  
      end do
      call writeout (xc,nstep)
      return
      end
c
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine loclsrch  --  local search for rigid body DEM  ##
c     ##                                                            ##
c     ################################################################
c
c
      subroutine loclsrch (neigen,minimum)
      implicit none
      include 'sizes.i'
      include 'group.i'
      include 'iounit.i'
      include 'rigid.i'
      integer maxrgd
      parameter (maxrgd=6*maxgrp)
      integer i,j,k,neigen,ndoi,nvar,nsearch
      real*8 minimum,minref,minbest
      real*8 tol,step(maxrgd)
      real*8 rbref(6,maxgrp),rbbest(6,maxgrp)
      real*8 eigen(maxrgd),vects(maxrgd,maxrgd)
      logical done
c
c
c     set parameters related to the local search procedure
c
      done = .false.
      tol = 0.0001d0
      minref = minimum
      minbest = minimum
      ndoi = 0
      nvar = 6 * ngrp
      do i = 1, ngrp
        do j = 1, 6
          rbref(j,i) = rbc(j,i)
        end do
      end do
c
c     find local minimum along each of the steepest directions
c
      dowhile (.not. done)
         ndoi = ndoi + 1
         write (iout,10)  ndoi,minref
   10    format (/,' Normal Mode Search :',8x,'Iteration',i4,
     &              5x,'Energy',f12.4,/)
         call rigidxyz
         call eigenrgd (eigen,vects)
c
c     search both directions along each eigenvector in turn
c
         nsearch = 0
         do i = 1, neigen
            do k = 1, nvar
               step(k) = vects(k,nvar-i+1)
            end do
            do k = 1, ngrp
               do j = 1, 6
                  rbc(j,k) = rbref(j,k)
               end do
            end do
            nsearch = nsearch + 1
            call climbrgd (nsearch,minimum,step)
            if (minimum .lt. minbest) then
               minbest = minimum
               do k = 1, ngrp
                  do j = 1, 6
                     rbbest(j,k) = rbc(j,k)
                  end do
               end do
            end if
            do k = 1, nvar
               step(k) = -vects(k,nvar-i+1)
            end do
            do k = 1, ngrp
               do j = 1, 6
                  rbc(j,k) = rbref(j,k)
               end do
            end do
            nsearch = nsearch + 1
            call climbrgd (nsearch,minimum,step)
            if (minimum .lt. minbest) then
               minbest = minimum
               do k = 1, ngrp
                  do j = 1, 6
                     rbbest(j,k) = rbc(j,k)
                  end do
               end do
            end if
         end do
c
c     check for convergence of the local search procedure
c
         if (minbest .lt. minref-tol) then
            done = .false.
            minref = minbest
            do k = 1, ngrp
               do j = 1, 6
                  rbref(j,k) = rbbest(j,k)
               end do
            end do
         else
            done = .true.
            minimum = minref
            do k = 1, ngrp
               do j = 1, 6
                  rbc(j,k) = rbref(j,k)
               end do
            end do
         end if
      end do
      return
      end
c
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine eigenrgd  --  rigid body Hessian eigenvectors  ##
c     ##                                                            ##
c     ################################################################
c
c
      subroutine eigenrgd (eigen,vects)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'group.i'
      integer maxrgd
      parameter (maxrgd=6*maxgrp)
      integer i,j,ihess,nvar
      real*8 hrigid(maxrgd,maxrgd),matrix((maxrgd+1)*maxrgd/2)
      real*8 a(maxrgd+1),b(maxrgd+1),p(maxrgd+1),ta(maxrgd+1)
      real*8 w(maxrgd+1),tb(maxrgd+1),ty(maxrgd+1)
      real*8 vnorm,eigen(maxrgd),vects(maxrgd,maxrgd)
c
c
c     compute the Hessian for rigid body motion
c
      call hessrgd ('full',hrigid)
c
c     place Hessian elements into triangular form
c
      nvar = 6 * ngrp
      ihess = 0
      do i = 1, nvar
         do j = i, nvar
            ihess = ihess + 1
            matrix(ihess) = hrigid(i,j)
         end do
      end do
c
c     diagonalize the Hessian to obtain eigenvalues
c
      call diagq (nvar,maxrgd,nvar,matrix,eigen,vects,
     &                     a,b,p,w,ta,tb,ty)
c
c     normalize the rigid body Hessian eigenvectors
c
      do i = 1, nvar
         vnorm = 0.0d0
         do j = 1, nvar
            vnorm = vnorm + vects(j,i)**2
         end do
         vnorm = sqrt(vnorm)
         do j = 1, nvar
            vects(j,i) = vects(j,i) / vnorm
         end do
      end do
      return
      end
c
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine climbrgd  --  minimum from a DEM local search  ##
c     ##                                                            ##
c     ################################################################
c
c
      subroutine climbrgd (nsearch,minimum,step)
      implicit none
      include 'sizes.i'
      include 'group.i'
      include 'iounit.i'
      include 'math.i'
      include 'rigid.i'
      integer maxrgd,maxstep
      parameter (maxrgd=6*maxgrp)
      parameter (maxstep=500)
      integer i,j,nvar,kstep,nstep,nsearch
      real*8 minimum,big,energy,size
      real*8 step(maxrgd),estep(0:maxstep)
      logical done
c
c
c     set the maximum number of steps and the step size
c
      done = .false.
      big = 100000.0d0
      minimum = big
      kstep = 0
      nstep = 65
c     size = 0.1d0
      size = 1.0d0
      nvar = 6 * ngrp
      do i = 1, nvar
         step(i) = size * step(i)
      end do
c
c     scan the search direction for a minimization candidate
c
      dowhile (.not. done)
         if (kstep .ne. 0) then
            nvar = 0
            do i = 1, ngrp
               do j = 1, 6
                  nvar = nvar + 1
                  rbc(j,i) = rbc(j,i) + step(nvar)
               end do
            end do
         end if
         call rigidxyz
         estep(kstep) = energy ()
         if (kstep.ge.2 .and. estep(kstep).le.10000.0d0) then
            if (estep(kstep) .lt. estep(kstep-2) .and.
     &          estep(kstep-1) .lt. estep(kstep-2)) then
               done = .true.
               nvar = 0
               do i = 1, ngrp
                  do j = 1, 6
                     nvar = nvar + 1
                     rbc(j,i) = rbc(j,i) - step(nvar)
                  end do
               end do
               call rigidxyz
               call localrgd (minimum)
               if (minimum .ge. -big) then
                  write (iout,10)  nsearch,kstep-1,minimum
   10             format (4x,'Search Direction',i4,10x,'Step',
     &                       i4,11x,f12.4)
               else
                  minimum = big
                  write (iout,20)  nsearch
   20             format (4x,'Search Direction',i4,35x,'------')
               end if
            end if
         end if
         if (kstep.ge.nstep .and. .not.done) then
            done = .true.
            write (iout,30)  nsearch
   30       format (4x,'Search Direction',i4,35x,'------')
         end if
         kstep = kstep + 1
      end do
      return
      end
c
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine localrgd  --  DEM local search optimization  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "localrgd" is during the the DEM local search procedure
c     to perform a rigid body energy minimization
c
c
      subroutine localrgd (minimum)
      implicit none
      include 'sizes.i'
      include 'inform.i'
      include 'group.i'
      include 'minima.i'
      include 'rigid.i'
      integer i,j,nvar,oldprt
      real*8 minimum,grdmin
      real*8 agda2,xx(maxvar)
      logical oldverb
      external agda2,writeout
c
c
c     transfer rigid body coordinates to optimization parameters
c
      nvar = 0
      do i = 1, ngrp
         do j = 1, 6
            nvar = nvar + 1
            xx(nvar) = rbc(j,i)
         end do
      end do
c
c     make the call to the optimization routine
c
      grdmin = 0.0001d0
      oldverb = verbose
      oldprt = iprint
      verbose = .false.
      iprint = 0
      call vmetric (nvar,xx,minimum,grdmin,agda2,writeout)
      verbose = oldverb
      iprint = oldprt
c
c     transfer optimization parameters to rigid body coordinates
c
      nvar = 0
      do i = 1, ngrp
         do j = 1, 6
            nvar = nvar + 1
            rbc(j,i) = xx(nvar)
         end do
      end do
      return
      end







