c
c
c     ############################################################
c     ##  COPYRIGHT (C) 1996 by Yong Kong & Jay William Ponder  ##
c     ##                  All Rights Reserved                   ##
c     ############################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine eimgchg  --  Variations of Friedman's image charge  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "eimgchg" calculates the macroscopic reaction field energy
c
c
      subroutine eimgchg
      implicit none

ccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     image method1
ccccccccccccccccccccccccccccccccccccccccccccccccccccc

      t1 = secnds(0.0)
      do iii=1,1


      factor_im = -(epi-1)*a/(epi+1)
      asq = a*a

      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         risq = xi*xi + yi*yi + zi*zi
         ri = dsqrt(risq)         
         r(i) = ri

         qim(i) = factor_im*q(i)/ri
         imsq = asq/risq
         xim(i) = x(i)*imsq
         yim(i) = y(i)*imsq
         zim(i) = z(i)*imsq
      end do

      en = 0.0d0
      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         do j=1,n
            dxim = xi - xim(j)
            dyim = yi - yim(j)
            dzim = zi - zim(j)
            rij = dsqrt(dxim*dxim + dyim*dyim + dzim*dzim)      
            en = en + q(i)*qim(j)/rij
         end do
      end do  

      en = 0.5d0*en

      total_q = 0.0d0
      do i=1,n
         total_q = total_q + q(i)
      end do
      e_corr = 0.5d0*total_q*total_q*(epi-1)/(a*epi*(epi+1))

      end do
      delta_t = secnds(t1)

      write(*,*) 
      write(*,*) "Image Method1"
      write(*,1000) en,delta_t
      write(*,1000) en-e_corr,delta_t


ccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     image method2
ccccccccccccccccccccccccccccccccccccccccccccccccccccc

      t1 = secnds(0.0)
      do iii=1,1


      factor_im = -(epi-1)*a/epi
      asq = a*a

      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         risq = xi*xi + yi*yi + zi*zi
         ri = dsqrt(risq)         
         r(i) = ri

         qim(i) = factor_im*q(i)/ri
         imsq = asq/risq
         xim(i) = x(i)*imsq
         yim(i) = y(i)*imsq
         zim(i) = z(i)*imsq
      end do

      en = 0.0d0
      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         do j=1,n
            dxim = xi - xim(j)
            dyim = yi - yim(j)
            dzim = zi - zim(j)
            rij = dsqrt(dxim*dxim + dyim*dyim + dzim*dzim)      
            en = en + q(i)*qim(j)/rij
         end do
      end do  

      en = 0.5d0*en
      end do
      delta_t = secnds(t1)

      write(*,*)
      write(*,*) "Image Method2"
      write(*,1000) en,delta_t

ccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     image method3
ccccccccccccccccccccccccccccccccccccccccccccccccccccc

      t1 = secnds(0.0)
      do iii=1,1


      factor_im = (2-epi)*a/epi
      asq = a*a

      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         risq = xi*xi + yi*yi + zi*zi
         ri = dsqrt(risq)         
         r(i) = ri

         qim(i) = factor_im*q(i)/ri
         imsq = asq/risq
         xim(i) = x(i)*imsq
         yim(i) = y(i)*imsq
         zim(i) = z(i)*imsq
      end do

      en = 0.0d0
      do i=1,n
         xi = x(i)
         yi = y(i)
         zi = z(i)
         do j=1,n
            dxim = xi - xim(j)
            dyim = yi - yim(j)
            dzim = zi - zim(j)
            rij = dsqrt(dxim*dxim + dyim*dyim + dzim*dzim)      
            en = en + q(i)*qim(j)/rij
         end do
      end do  

      en = 0.5d0*en

      total_q = 0.0d0
      do i=1,n
         total_q = total_q + q(i)
      end do
      e_corr = 0.5d0*total_q*total_q/(a*epi)

      end do
      delta_t = secnds(t1)

      write(*,*) 
      write(*,*) "Image Method3"
      write(*,1000) en,delta_t
      write(*,1000) en-e_corr,delta_t
      write(*,*) 

      return
      end
