c
c
c     ############################################################
c     ##  COPYRIGHT (C) 1996 by Yong Kong & Jay William Ponder  ##
c     ##                  All Rights Reserved                   ##
c     ############################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine epairmpole  --  Mike's pairwise method, multipole  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "epairmpole" calculates the macroscopic reaction field energy
c
c
      subroutine epairmpole
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'electr.i'
      include 'energi.i'
      include 'mpole.i'
      include 'shunt.i'
      include 'usage.i'
      integer i,j,k
      integer ii,iz,ix,kk,kz,kx
      real*8 eik,a(3,3)
      real*8 xr,yr,zr,r2
      real*8 rpi(13),rpk(13)
      logical iuse,kuse
c
c
c     zero out the macroscopic reaction field energy
c
      er = 0.0d0
c
c     set the switching function coefficients
c
      call switch ('CHARGE')
c
c     rotate the multipole components into the global frame
c
      do i = 1, npole
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c
      call ijk_pt
c
c     calculate the multipole interaction energy term
c
      do ii = 1, npole
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         do kk = ii, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               xr = x(k) - x(i)
               yr = y(k) - y(i)
               zr = z(k) - z(i)
               r2 = xr*xr + yr*yr + zr*zr
               if (r2 .le. off2) then
                  do j = 1, mdqsiz
                     rpk(j) = rpole(j,kk)
                  end do
                  call erfik (i,k,rpi,rpk,eik)
                  er = er + eik
               end if
            end if
         end do
      end do
      return
      end
c
c
c     #################################################################
c     ##                                                             ##
c     ##  subroutine erfik  --  reaction field energy of site pair   ##
c     ##                                                             ##
c     #################################################################
c
c
      subroutine erfik (i,k,rpi,rpk,eik)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'electr.i'
      include 'mpole.i'
      include 'rxnfld.i'
      include 'rxnpot.i'
      include 'units.i'
      integer i,k,m,n1,n2,nn,fii,fi,fj
      integer p_s1,p_s2,p_e1,p_e2
      integer ind1_x(13),ind1_y(13),ind1_z(13)
      integer ind2_x(13),ind2_y(13),ind2_z(13)
      real*8 eik,term,ratio,factor
      real*8 xi,yi,zi,xk,yk,zk
      real*8 size,size2,d,d2,ri2,rk2
      real*8 xi2,yi2,zi2,xk2,yk2,zk2
      real*8 rpi(13),rpk(13)
      real*8 rklij(13,13),d1d2
      real*8 m2t2(13)
c
c
c     integer ind_x,ind_y,ind_z
c     real*8 d1d2_r1,d1d2_r2s,d1d2_r2t,d1d2_r2u,d1d2_r2i
c     real*8 d1d2_r2is,d1d2_r2it,d1d2_r2iu,d1d2_r2j
c     real*8 d1d2_r2js,d1d2_r2jt,d1d2_r2ju,d1d2_r2k,d1d2_r
c
c
c     get numbers of the atoms
c
      xi = x(i)
      yi = y(i)
      zi = z(i)
      xk = x(k)
      yk = y(k)
      zk = z(k)
      d = xi*xk + yi*yk + zi*zk
      ri2 = xi*xi + yi*yi + zi*zi
      rk2 = xk*xk + yk*yk + zk*zk
c
c     set highest order of multipoles at each site (M=0, D=1, Q=2)
c
      eik = 0.0d0
      n1 = 2
      n2 = 2
      nn = rfterms
      ratio = rfbulkd / dielec
      factor = electric * (1.0d0-ratio)
      if (i .eq. k)  factor = 0.5d0 * factor
      size = 1.0d0 / rfsize
      size2 = size * size
c
c     get the values of the indices
c
      m = (3**(n1+1)-1)/2
      call rindex (n1,m,ind1_x,ind1_y,ind1_z,p_s1,p_e1)
      m = (3**(n2+1)-1)/2
      call rindex (n2,m,ind2_x,ind2_y,ind2_z,p_s2,p_e2)
c
c     initialize the stored matrix element areas
c
      do fi = 1, p_e1
         do fj = 1, p_e2
            b1(fi,fj) = 0.0d0
            b2(fi,fj) = 0.0d0
         end do
      end do
c
c     explicit formula for the 0th summation term
c
      if (nn .ge. 0) then
         eik = size * rpi(1) * rpk(1) / ratio
         size = size * size2
      end if
c     
c     explicit formula for the 1st summation term
c
      if (nn .ge. 1) then
         b2(1,1) = d
         b2(1,2) = xi
         b2(1,3) = yi
         b2(1,4) = zi
         b2(2,1) = xk
         b2(3,1) = yk
         b2(4,1) = zk
         b2(2,2) = 1.0d0
         b2(3,3) = 1.0d0
         b2(4,4) = 1.0d0
         do fi = 1, 4
            m2t2(fi) = 0.0d0
            do fj = 1, 4
               m2t2(fi) = m2t2(fi) + b2(fi,fj)*rpk(fj)
            end do
         end do
         term = 0.0d0
         do fi = 1, 4
            term = term + rpi(fi)*m2t2(fi)
         end do
         term = 2.0d0 * size * term / (2.0d0*ratio+1.0d0)
         eik = eik + term
         size = size * size2
      end if
c     
c     explicit formula for the 2nd summation term
c
      if (nn .ge. 2) then
         b2(1,1) = (3.0d0*d*d-ri2*rk2) * 0.5d0
         b2(1,2) = 3.0d0*xi*d - xk*ri2
         b2(1,3) = 3.0d0*yi*d - yk*ri2
         b2(1,4) = 3.0d0*zi*d - zk*ri2
         b2(1,5) = 3.0d0*xi*xi - ri2
         b2(1,6) = 3.0d0*xi*yi
         b2(1,7) = 3.0d0*xi*zi
         b2(1,8) = b2(1,6)
         b2(1,9) = 3.0d0*yi*yi - ri2
         b2(1,10) = 3.0d0*yi*zi
         b2(1,11) = b2(1,7)
         b2(1,12) = b2(1,10)
         b2(1,13) = 3.0d0*zi*zi - ri2
         b2(2,1) = 3.0d0*xk*d - xi*rk2
         b2(2,2) = 3.0d0*d + xi*xk
         b2(2,3) = 3.0d0*xk*yi - 2.0d0*xi*yk
         b2(2,4) = 3.0d0*zi*xk - 2.0d0*xi*zk     
         b2(2,5) = 4.0d0*xi      
         b2(2,6) = 3.0d0*yi      
         b2(2,7) = 3.0d0*zi      
         b2(2,8) = b2(2,6)    
         b2(2,9) = -2.0d0*xi
         b2(2,11) = b2(2,7)    
         b2(2,13) = b2(2,9)    
         b2(3,1) = 3.0d0*yk*d - yi*rk2
         b2(3,2) = 3.0d0*yk*xi - 2.0d0*yi*xk
         b2(3,3) = 3.0d0*d + yi*yk
         b2(3,4) = 3.0d0*yk*zi - 2.0d0*yi*zk
         b2(3,5) = -2.0d0*yi    
         b2(3,6) = 3.0d0*xi      
         b2(3,8) = b2(3,6)    
         b2(3,9) = 4.0d0*yi 
         b2(3,10) = 3.0d0*zi  
         b2(3,12) = b2(3,10)    
         b2(3,13) = b2(3,5)   
         b2(4,1) = 3.0d0*zk*d - zi*rk2
         b2(4,2) = 3.0d0*zk*xi - 2.0d0*zi*xk
         b2(4,3) = 3.0d0*zk*yi - 2.0d0*zi*yk
         b2(4,4) = 3.0d0*d + zi*zk
         b2(4,5) = -2.0d0*zi
         b2(4,7) = 3.0d0*xi
         b2(4,9) = b2(4,5)
         b2(4,10) = 3.0d0*yi
         b2(4,11) = b2(4,7)
         b2(4,12) = b2(4,10)
         b2(4,13) = 4.0d0*zi
         b2(5,1) = 3.0d0*xk*xk - rk2
         b2(5,2) = 4.0d0*xk
         b2(5,3) = -2.0d0*yk
         b2(5,4) = -2.0d0*zk
         b2(5,5) = 4.0d0
         b2(5,9) = -2.0d0
         b2(5,13) = -2.0d0
         b2(6,1) = 3.0d0*xk*yk
         b2(6,2) = 3.0d0*yk
         b2(6,3) = 3.0d0*xk
         b2(6,6) = 3.0d0
         b2(6,8) = 3.0d0
         b2(7,1) = 3.0d0*xk*zk
         b2(7,2) = 3.0d0*zk 
         b2(7,4) = 3.0d0*xk 
         b2(7,7) = 3.0d0 
         b2(7,11) = 3.0d0 
         b2(8,1) = b2(6,1)
         b2(8,2) = b2(6,2)
         b2(8,3) = b2(6,3)
         b2(8,6) = 3.0d0
         b2(8,8) = 3.0d0
         b2(9,1) = 3.0d0*yk*yk - rk2
         b2(9,2) = -2.0d0*xk 
         b2(9,3) = 4.0d0*yk 
         b2(9,4) = -2.0d0*zk
         b2(9,5) = -2.0d0 
         b2(9,9) = 4.0d0 
         b2(9,13) = -2.0d0
         b2(10,1) = 3.0d0*yk*zk
         b2(10,3) = 3.0d0*zk
         b2(10,4) = 3.0d0*yk
         b2(10,10) = 3.0d0
         b2(10,12) = 3.0d0
         b2(11,1) = b2(7,1)
         b2(11,2) = b2(7,2)
         b2(11,4) = b2(7,4)
         b2(11,7) = 3.0d0
         b2(11,11) = 3.0d0
         b2(12,1) = b2(10,1)
         b2(12,3) = b2(10,3)
         b2(12,4) = b2(10,4)
         b2(12,10) = 3.0d0
         b2(12,12) = 3.0d0
         b2(13,1) = 3.0d0*zk*zk - rk2
         b2(13,2) = -2.0d0*xk
         b2(13,3) = -2.0d0*yk
         b2(13,4) = 4.0d0*zk
         b2(13,5) = -2.0d0
         b2(13,9) = -2.0d0
         b2(13,13) = 4.0d0
         do fi = 1, 13
            m2t2(fi) = 0.0d0
            do fj = 1, 13
               m2t2(fi) = m2t2(fi) + b2(fi,fj)*rpk(fj)
            end do
         end do
         term = 0.0d0
         do fi = 1, 13
            term = term + rpi(fi)*m2t2(fi)
         end do
         term = 3.0d0 * size * term / (3.0d0*ratio+2.0d0)
         eik = eik + term
         size = size * size2
      end if
c
c     explicit formula for the 3rd summation term
c
      if (nn .ge. 3) then
         d2 = d*d
         xi2 = xi*xi
         yi2 = yi*yi
         zi2 = zi*zi
         xk2 = xk*xk
         yk2 = yk*yk
         zk2 = zk*zk
         b1(1,1) = d*(2.5d0*d2-1.5d0*ri2*rk2)
         b1(1,2) = 7.5d0*d2*xi-3.0d0*xk*ri2*d-1.5d0*xi*ri2*rk2
         b1(1,3) = 7.5d0*d2*yi-3.0d0*yk*ri2*d-1.5d0*yi*ri2*rk2
         b1(1,4) = 7.5d0*d2*zi-3.0d0*zk*ri2*d-1.5d0*zi*ri2*rk2
         b1(1,5) = 15.0d0*d*xi2-3.0d0*ri2*(d+2.0d0*xi*xk)
         b1(1,6) = 15.d0*xi*yi*d - 3.0d0*ri2*(xi*yk+xk*yi)
         b1(1,7) = 15.d0*xi*zi*d - 3.0d0*ri2*(xi*zk+xk*zi)
         b1(1,8) = b1(1,6)
         b1(1,9) = 15.0d0*d*yi2-3.0d0*ri2*(d+2.0d0*yi*yk)
         b1(1,10) = 15.d0*yi*zi*d - 3.0d0*ri2*(yi*zk+yk*zi)
         b1(1,11) = b1(1,7)
         b1(1,12) = b1(1,10)
         b1(1,13) = 15.0d0*d*zi2-3.0d0*ri2*(d+2.0d0*zi*zk)
         b1(2,1) = 7.5d0*d2*xk-3.0d0*xi*rk2*d-1.5d0*xk*ri2*rk2
         b1(2,2) = 7.5d0*d2+9.0d0*xi*xk*d-3.0*xi2*rk2-3.0*xk2*ri2
     &                -1.5*ri2*rk2
         b1(2,3) = 3.0d0*((5.0d0*xk*yi-2.0d0*xi*yk)*d
     &                -xi*yi*rk2-xk*yk*ri2)
         b1(2,4) = 3.0d0*((5.0d0*xk*zi-2.0d0*xi*zk)*d
     &                -xi*zi*rk2-xk*zk*ri2)
         b1(2,5) = 24.0d0*xi*yi*yk + 24.0d0*xi*zi*zk + 18.0d0*xi2*xk
     &                - 9.0d0*xk*yi2  - 9.0d0*xk*zi2      
         b1(2,6) = (8.0d0*yi*xk*xi - 3.0d0*xi2*yk + 4.0d0*yi2*yk
     &                - yk*zi2  + 5.0d0*yi*zi*zk)*3.0d0      
         b1(2,7) = 15.0d0*zi*yi*yk + 12.0d0*zi2*zk - 9.0d0*xi2*zk
     &                - 3.0d0*zk*yi2  + 24.0d0*zi*xk*xi     
         b1(2,8) = b1(2,6)    
         b1(2,9) = - 9.0d0*xi2*xk + 12.0d0*xk*yi2  - 3.0d0*xk*zi2
     &                - 18.0d0*xi*yi*yk - 6.0d0*xi*zi*zk
         b1(2,10) = 15.0d0*zi*xk*yi - 6.0d0*zi*xi*yk - 6.0d0*yi*xi*zk  
         b1(2,11) = b1(2,7)    
         b1(2,12) = b1(2,10)    
         b1(2,13) = - 6.0d0*xi*yi*yk - 9.0d0*xi2*xk - 3.0d0*xk*yi2
     &                 + 12.0d0*xk*zi2  - 18.0d0*xi*zi*zk    
         b1(3,1) = 7.5d0*d2*yk-3.0d0*yi*rk2*d-1.5d0*yk*ri2*rk2
         b1(3,2) = 3.0d0*((5.0d0*xi*yk-2.0d0*xk*yi)*d
     &                -xi*yi*rk2-xk*yk*ri2)
         b1(3,3) = 7.5d0*d2+9.0d0*yi*yk*d-3.0*yi2*rk2-3.0*yk2*ri2
     &                -1.5*ri2*rk2
         b1(3,4) = 3.0d0*((5.0d0*yk*zi-2.0d0*yi*zk)*d
     &                -yi*zi*rk2-yk*zk*ri2)
         b1(3,5) = - 9.0d0*yi2*yk - 6.0d0*yi*zi*zk - 18.0d0*yi*xk*xi
     &                + 12.0d0*xi2*yk - 3.0d0*yk*zi2
         b1(3,6) = 12.0d0*xi2*xk + 15.0d0*xi*zi*zk - 9.0d0*xk*yi2
     &                - 3.0d0*xk*zi2  + 24.0d0*xi*yi*yk
         b1(3,7) = 15.0d0*zi*xi*yk - 6.0d0*yi*xi*zk - 6.0d0*zi*xk*yi     
         b1(3,8) = b1(3,6)    
         b1(3,9) = - 9.0d0*xi2*yk + 18.0d0*yi2*yk - 9.0d0*yk*zi2
     &                + 24.0d0*yi*xk*xi + 24.0d0*yi*zi*zk
         b1(3,10) = 24.0d0*zi*yi*yk - 3.0d0*xi2*zk - 9.0d0*zk*yi2
     &                + 12.0d0*zi2*zk + 15.0d0*zi*xk*xi
         b1(3,11) = b1(3,7)  
         b1(3,12) = b1(3,10)    
         b1(3,13) = - 3.0d0*xi2*yk - 9.0d0*yi2*yk + 12.0d0*yk*zi2
     &                 - 18.0d0*yi*zi*zk - 6.0d0*yi*xk*xi   
         b1(4,1) = 7.5*d2*zk-3.0d0*zi*rk2*d-1.5d0*zk*ri2*rk2
         b1(4,2) = 3.0d0*((5.0d0*xi*zk-2.0d0*xk*zi)*d
     &                -xi*zi*rk2-xk*zk*ri2)
         b1(4,3) = 3.0d0*((5.0d0*yi*zk-2.0d0*yk*zi)*d
     &                -yi*zi*rk2-yk*zk*ri2)
         b1(4,4) = 7.5d0*d2+9.0d0*zi*zk*d-3.0*zi2*rk2-3.0*zk2*ri2
     &                -1.5*ri2*rk2
         b1(4,5) = 12.0d0*xi2*zk - 3.0d0*zk*yi2 - 9.0d0*zi2*zk
     &                - 18.0d0*zi*xk*xi - 6.0d0*zi*yi*yk
         b1(4,6) = 15.0d0*yi*xi*zk - 6.0d0*zi*xi*yk - 6.0d0*zi*xk*yi
         b1(4,7) = 24.0d0*xi*zi*zk + 12.0d0*xi2*xk - 3.0d0*xk*yi2
     &                - 9.0d0*xk*zi2  + 15.0d0*xi*yi*yk
         b1(4,8) = b1(4,6)
         b1(4,9) = - 6.0d0*zi*xk*xi - 9.0d0*zi2*zk - 3.0d0*xi2*zk
     &                + 12.0d0*zk*yi2  - 18.0d0*zi*yi*yk
         b1(4,10) = 15.0d0*yi*xk*xi + 12.0d0*yi2*yk - 9.0d0*yk*zi2
     &                + 24.0d0*yi*zi*zk - 3.0d0*xi2*yk
         b1(4,11) = b1(4,7)
         b1(4,12) = b1(4,10)
         b1(4,13) = 24.0d0*zi*xk*xi + 18.0d0*zi2*zk - 9.0d0*xi2*zk
     &                - 9.0d0*zk*yi2  + 24.0d0*zi*yi*yk
         b1(5,1) = 15.0d0*d*xk2-3.0d0*rk2*(d+2.0d0*xi*xk)
         b1(5,2) = 18.0d0*xi*xk2  + 24.0d0*xk*yi*yk + 24.0d0*xk*zi*zk
     &                - 9.0d0*xi*yk2  - 9.0d0*xi*zk2 
         b1(5,3) = 12.0d0*yi*xk2 - 9.0d0*yk2*yi - 3.0d0*yi*zk2
     &                - 18.0d0*xk*xi*yk - 6.0d0*yk*zi*zk
         b1(5,4) = - 9.0d0*zk2*zi - 6.0d0*zk*yi*yk - 18.0d0*xk*xi*zk
     &                + 12.0d0*zi*xk2  - 3.0d0*zi*yk2
         b1(5,5) = 24.0d0*zi*zk + 24.0d0*yi*yk + 36.0d0*xi*xk 
         b1(5,6) = -18.0d0*xi*yk + 24.0d0*yi*xk
         b1(5,7) = -18.0d0*xi*zk + 24.0d0*zi*xk
         b1(5,8) = b1(5,6)
         b1(5,9) = -6.0d0*zi*zk - 18.0d0*yi*yk - 18.0d0*xi*xk
         b1(5,10) = -6.0d0*(yi*zk + zi*yk)
         b1(5,11) = b1(5,7)
         b1(5,12) = b1(5,10)
         b1(5,13) = -6.0d0*yi*yk - 18.0d0*xi*xk - 18.0d0*zi*zk
         b1(6,1) = 15.d0*xk*yk*d - 3.0d0*rk2*(xi*yk+xk*yi)
         b1(6,2) = -9.0d0*yi*xk2 + 12.0d0*yk2*yi - 3.0d0*yi*zk2
     &                + 24.0d0*xk*xi*yk + 15.0d0*yk*zi*zk
         b1(6,3) = 12.0d0*xi*xk2 + 15.0d0*xk*zi*zk - 9.0d0*xi*yk2
     &                - 3.0d0*xi*zk2  + 24.0d0*xk*yi*yk
         b1(6,4) = -6.0d0*xk*yi*zk - 6.0d0*yk*xi*zk + 15.0d0*zi*xk*yk
         b1(6,5) = -18.0d0*yi*xk + 24.0d0*xi*yk
         b1(6,6) = 24.0d0*yi*yk + 24.0d0*xi*xk + 15.0d0*zi*zk
         b1(6,7) = -6.0d0*yi*zk + 15.0d0*zi*yk
         b1(6,8) = b1(6,6)
         b1(6,9) = -18.0d0*xi*yk + 24.0d0*yi*xk
         b1(6,10) = -6.0d0*xi*zk + 15.0d0*zi*xk
         b1(6,11) = b1(6,7)
         b1(6,12) = b1(6,10)
         b1(6,13) = -6.0d0*yi*xk - 6.0d0*xi*yk
         b1(7,1) = 15.d0*xk*zk*d - 3.0d0*rk2*(xi*zk+xk*zi)
         b1(7,2) = 15.0d0*zk*yi*yk + 12.0d0*zk2*zi - 9.0d0*zi*xk2
     &                - 3.0d0*zi*yk2  + 24.0d0*xk*xi*zk 
         b1(7,3) = - 6.0d0*zi*xk*yk - 6.0d0*yk*xi*zk + 15.0d0*xk*yi*zk
         b1(7,4) = 12.0d0*xi*xk2  - 3.0d0*xi*yk2  - 9.0d0*xi*zk2
     &                + 15.0d0*xk*yi*yk + 24.0d0*xk*zi*zk
         b1(7,5) = -18.0d0*zi*xk + 24.0d0*xi*zk
         b1(7,6) = -6.0d0*zi*yk + 15.0d0*yi*zk
         b1(7,7) = 24.0d0*xi*xk + 24.0d0*zi*zk + 15.0d0*yi*yk 
         b1(7,8) = b1(7,6)
         b1(7,9) = -6.0d0*zi*xk - 6.0d0*xi*zk
         b1(7,10) = -6.0d0*xi*yk + 15.0d0*yi*xk 
         b1(7,11) = b1(7,7)
         b1(7,12) = b1(7,10) 
         b1(7,13) = -18.0d0*xi*zk + 24.0d0*zi*xk
         b1(9,1) = 15.0d0*d*yk2-3.0d0*rk2*(d+2.0d0*yi*yk)
         b1(9,2) = -9.0d0*xi*xk2 + 12.0d0*xi*yk2 - 3.0d0*xi*zk2
     &                - 18.0d0*xk*yi*yk - 6.0d0*xk*zi*zk
         b1(9,3) = -9.0d0*yi*xk2  + 18.0d0*yk2*yi - 9.0d0*yi*zk2
     &                + 24.0d0*yk*zi*zk + 24.0d0*xk*xi*yk
         b1(9,4) = 12.0d0*zi*yk2 - 18.0d0*zk*yi*yk - 3.0d0*zi*xk2
     &                - 9.0d0*zk2*zi - 6.0d0*xk*xi*zk
         b1(9,5) = -18.0d0*xi*xk - 6.0d0*zi*zk - 18.0d0*yi*yk
         b1(9,6) = -18.0d0*yi*xk + 24.0d0*xi*yk
         b1(9,7) = -6.0d0*zi*xk - 6.0d0*xi*zk
         b1(9,8) = b1(9,6)
         b1(9,9) = 24.0d0*xi*xk + 24.0d0*zi*zk + 36.0d0*yi*yk 
         b1(9,10) = -18.0d0*yi*zk + 24.0d0*zi*yk
         b1(9,11) = b1(9,7)
         b1(9,12) = b1(9,10)
         b1(9,13) = -18.0d0*yi*yk - 6.0d0*xi*xk - 18.0d0*zi*zk
         b1(10,1) = 15.d0*yk*zk*d - 3.0d0*rk2*(yi*zk+yk*zi)
         b1(10,2) = -6.0d0*zi*xk*yk -6.0d0*xk*yi*zk + 15.0d0*yk*xi*zk
         b1(10,3) = 12.0d0* zk2*zi + 15.0d0*xk*xi*zk - 3.0d0*zi*xk2
     &                 - 9.0d0*zi*yk2  + 24.0d0*zk*yi*yk
         b1(10,4) = 15.0d0*xk*xi*yk + 12.0d0*yk2*yi - 3.0d0*yi*xk2
     &                 - 9.0d0*yi*zk2  + 24.0d0*yk*zi*zk
         b1(10,5) = -6.0d0*yi*zk - 6.0d0*zi*yk
         b1(10,6) = -6.0d0*zi*xk + 15.0d0*xi*zk
         b1(10,7) = -6.0d0*yi*xk + 15.0d0*xi*yk
         b1(10,8) = b1(10,6)
         b1(10,9) = 24.0d0*yi*zk - 18.0d0*zi*yk
         b1(10,10) = 15.0d0*xi*xk + 24.0d0*zi*zk + 24.0d0*yi*yk
         b1(10,11) = b1(10,7)
         b1(10,12) = b1(10,10)
         b1(10,13) = -18.0d0*yi*zk + 24.0d0*zi*yk
         b1(13,1) = 15.0d0*d*zk2-3.0d0*rk2*(d+2.0d0*zi*zk)
         b1(13,2) = 12.0d0*xi*zk2 - 18.0d0*xk*zi*zk - 9.0d0*xi*xk2
     &                 - 3.0d0*xi*yk2 - 6.0d0*xk*yi*yk
         b1(13,3) = 12.0d0*yi*zk2 - 3.0d0*yi*xk2 - 9.0d0*yk2*yi
     &                 - 18.0d0*yk*zi*zk - 6.0d0*xk*xi*yk
         b1(13,4) = -9.0d0*zi*xk2 - 9.0d0*zi*yk2 + 18.0d0*zk2*zi
     &                 + 24.0d0*xk*xi*zk + 24.0d0*zk*yi*yk
         b1(13,5) = -6.0d0*yi*yk - 18.0d0*zi*zk - 18.0d0*xi*xk
         b1(13,6) = -6.0d0*yi*xk - 6.0d0*xi*yk
         b1(13,7) = 24.0d0*xi*zk - 18.0d0*zi*xk
         b1(13,8) = b1(13,6)
         b1(13,9) = -18.0d0*yi*yk - 6.0d0*xi*xk - 18.0d0*zi*zk
         b1(13,10) = 24.0d0*yi*zk - 18.0d0*zi*yk
         b1(13,11) = b1(13,7)
         b1(13,12) = b1(13,10)
         b1(13,13) = 36.0d0*zi*zk + 24.0d0*xi*xk + 24.0d0*yi*yk
         do fi = 1, 13
            b1(8,fi) = b1(6,fi)
            b1(11,fi) = b1(7,fi)
            b1(12,fi) = b1(10,fi)
         end do
c        b1(14,1) = 3.0d0*xk*(5.0d0*xk2-3.0d0*rk2)
c        b1(14,2) = 9.0d0*(3.0d0*xk2 -rk2)
c        b1(14,3) = -18.0d0*xk*yk
c        b1(14,4) = -18.0d0*zk*xk
c        b1(14,5) = 36.0d0*xk
c        b1(14,6) = -18.0d0*yk
c        b1(14,7) = -18.0d0*zk
c        b1(14,8) = b1(14,6)
c        b1(14,9) = -18.0d0*xk
c        b1(14,10) = 0.0d0
c        b1(14,11) = b1(14,7)
c        b1(14,12) = 0.0d0
c        b1(14,13) = -18.0d0*xk
c        b1(15,1) = 3.0d0*yk*(4.0d0*xk2 - yk2 - zk2)
c        b1(15,2) = 24.0d0*xk*yk
c        b1(15,3) = -9.0d0*yk2 + 12.0d0*xk2 - 3.0d0*zk2
c        b1(15,4) = - 6.0d0*yk*zk
c        b1(15,5) = 24.0d0*yk
c        b1(15,6) = 24.0d0*xk
c        b1(15,7) = 0.0d0
c        b1(15,8) = b1(15,6)
c        b1(15,9) = -18.0d0*yk
c        b1(15,10) = -6.0d0*zk
c        b1(15,11) = 0.0d0
c        b1(15,12) = b1(15,10)
c        b1(15,13) = -6.0d0*yk
c        b1(16,1) = 3.0d0*zk*(4.0d0*xk2 - yk2 - zk2)
c        b1(16,2) = 24.0d0*zk*xk
c        b1(16,3) = -6.0d0*yk*zk
c        b1(16,4) = 12.0d0*xk2 - 3.0d0*yk2 - 9.0d0*zk2 
c        b1(16,5) = 24.0d0*zk
c        b1(16,6) = 0.0d0
c        b1(16,7) = 24.0d0*xk
c        b1(16,8) = 0.0d0
c        b1(16,9) = -6.0d0*zk
c        b1(16,10) = -6.0d0*yk
c        b1(16,11) = b1(16,7)
c        b1(16,12) = b1(16,10)
c        b1(16,13) = -18.0d0*zk
c        b1(18,1) = -3.0d0*xk*(xk2 - 4.0d0*yk2 + zk2)
c        b1(18,2) = -9.0d0*xk2 + 12.0d0*yk2 - 3.0d0*zk2
c        b1(18,3) = 24.0d0*xk*yk
c        b1(18,4) = -6.0d0*zk*xk
c        b1(18,5) = -18.0d0*xk
c        b1(18,6) = 24.0d0*yk
c        b1(18,7) = -6.0d0*zk
c        b1(18,8) = b1(18,6)
c        b1(18,9) = 24.0d0*xk
c        b1(18,10) = 0.0d0
c        b1(18,11) = b1(18,7)
c        b1(18,12) = 0.0d0
c        b1(18,13) = - 6.0d0*xk
c        b1(19,1) = 15.0d0*zk*xk*yk
c        b1(19,2) = 15.0d0*yk*zk
c        b1(19,3) = 15.0d0*zk*xk
c        b1(19,4) = 15.0d0*xk*yk
c        b1(19,5) = 0.0d0
c        b1(19,6) = 15.0d0*zk
c        b1(19,7) = 15.0d0*yk
c        b1(19,8) = b1(19,6)
c        b1(19,9) = 0.0d0
c        b1(19,10) = 15.0d0*xk
c        b1(19,11) = b1(19,7)
c        b1(19,12) = b1(19,10)
c        b1(19,13) = 0.0d0
c        b1(22,1) = -3.0d0*xk*(xk2 + yk2 - 4.0d0*zk2)
c        b1(22,2) = -9.0d0*xk2 - 3.0d0*yk2 + 12.0d0*zk2
c        b1(22,3) = - 6.0d0*xk*yk
c        b1(22,4) = 24.0d0*zk*xk
c        b1(22,5) = -18.0d0*xk
c        b1(22,6) = -6.0d0*yk
c        b1(22,7) = 24.0d0*zk
c        b1(22,8) = b1(22,6)
c        b1(22,9) = -6.0d0*xk
c        b1(22,10) = 0
c        b1(22,11) = b1(22,7)
c        b1(22,12) = 0
c        b1(22,13) = 24.0d0*xk
c        b1(27,1) = -3.0d0*yk*(3.0d0*xk2 - 2.0d0*yk2 + 3.0d0*zk2)
c        b1(27,2) = -18.0d0*xk*yk
c        b1(27,3) = 9.0d0*(-xk2 + 2.0d0*yk2 - zk2)
c        b1(27,4) = -18.0d0*yk*zk
c        b1(27,5) = -18.0d0*yk
c        b1(27,6) = - 18.0d0*xk
c        b1(27,7) = 0.0d0
c        b1(27,8) = b1(27,6)
c        b1(27,9) = 36.0d0*yk
c        b1(27,10) = -18.0d0*zk
c        b1(27,11) = 0.0d0
c        b1(27,12) = b1(27,10)
c        b1(27,13) = -18.0d0*yk
c        b1(28,1) = -3.0d0*zk*(xk2 - 4.0d0*yk2 + zk2)
c        b1(28,2) = -6.0d0*zk*xk
c        b1(28,3) = 24.0d0*yk*zk
c        b1(28,4) = 12.0d0*yk2 - 9.0d0*zk2 - 3.0d0*xk2 
c        b1(28,5) = -6.0d0*zk
c        b1(28,6) = 0.0d0
c        b1(28,7) = -6.0d0*xk
c        b1(28,8) = 0.0d0
c        b1(28,9) = 24.0d0*zk
c        b1(28,10) = 24.0d0*yk
c        b1(28,11) = b1(28,7)
c        b1(28,12) = b1(28,10)
c        b1(28,13) = -18.0d0*zk
c        b1(31,1) = -3.0d0*yk*(xk2 + yk2 - 4.0d0*zk2)
c        b1(31,2) = - 6.0d0*xk*yk
c        b1(31,3) = -9.0d0*yk2 + 12.0d0*zk2 - 3.0d0*xk2
c        b1(31,4) = 24.0d0*yk*zk
c        b1(31,5) = -6.0d0*yk
c        b1(31,6) = -6.0d0*xk
c        b1(31,7) = 0.0d0
c        b1(31,8) = b1(31,6)
c        b1(31,9) = -18.0d0*yk
c        b1(31,10) = 24.0d0*zk
c        b1(31,11) = 0.0d0
c        b1(31,12) = b1(31,10)
c        b1(31,13) = 24.0d0*yk
c        b1(40,1) = -3.0d0*zk*(3.0d0*xk2 + 3.0d0*yk2 - 2.0d0*zk2)
c        b1(40,2) = -18.0d0*zk*xk
c        b1(40,3) = -18.0d0*yk*zk
c        b1(40,4) = 9.0d0*(- xk2 - yk2 + 2.0d0*zk2)
c        b1(40,5) = -18.0d0*zk
c        b1(40,6) = 0.0d0
c        b1(40,7) = -18.0d0*xk 
c        b1(40,8) = 0.0d0
c        b1(40,9) = -18.0d0*zk
c        b1(40,10) = -18.0d0*yk
c        b1(40,11) = b1(40,7)
c        b1(40,12) = b1(40,10)
c        b1(40,13) = 36.0d0*zk
c        do fi = 1, 13
c           b1(17,fi) = b1(15,fi)
c           b1(23,fi) = b1(15,fi)
c           b1(20,fi) = b1(16,fi)
c           b1(32,fi) = b1(16,fi)
c           b1(24,fi) = b1(18,fi)
c           b1(26,fi) = b1(18,fi)
c           b1(21,fi) = b1(19,fi)
c           b1(25,fi) = b1(19,fi)
c           b1(29,fi) = b1(19,fi)
c           b1(33,fi) = b1(19,fi)
c           b1(35,fi) = b1(19,fi)
c           b1(34,fi) = b1(22,fi)
c           b1(38,fi) = b1(22,fi)
c           b1(30,fi) = b1(28,fi)
c           b1(36,fi) = b1(28,fi)
c           b1(37,fi) = b1(31,fi)
c           b1(39,fi) = b1(31,fi)
c        end do
         do fi = 1, 13
            m2t2(fi) = 0.0d0
            do fj = 1, 13
               m2t2(fi) = m2t2(fi) + b1(fi,fj)*rpk(fj)
            end do
         end do
         term = 0.0d0
         do fi = 1, 13
            term = term + rpi(fi)*m2t2(fi)
         end do
         term = 4.0d0 * size * term / (4.0d0*ratio+3.0d0)
         eik = eik + term
         size = size * size2
      end if
c
c     recursive formulation of 4th through nth summation terms
c
c     do fii = 4, nn
c        do fi = 1, p_e1
c           if (fi .eq. 8) then
c              do fj = 1, p_e2
c                 rklij(fi,fj) = rklij(6,fj)
c              end do
c           else if (fi .eq. 11) then
c              do fj = 1, p_e2
c                 rklij(fi,fj) = rklij(7,fj)
c              end do
c           else if (fi .eq. 12) then
c              do fj = 1, p_e2
c                 rklij(fi,fj) = rklij(10,fj)
c              end do
c           else if (fi .eq. 1) then
c              rklij(fi,1) = ((2.0d0*fii-1)*d*b1(1,1)
c    &                          -(fii-1)*ri2*rk2*b2(1,1))/fii
c              rklij(fi,2) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  1,0,0)
c              rklij(fi,3) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  0,1,0)
c              rklij(fi,4) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  0,0,1)
c              rklij(fi,5) = d1d2_r2s (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  2,0,0)
c              rklij(fi,6) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  1,1,0)
c              rklij(fi,7) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  1,0,1)
c              rklij(fi,8) = rklij(fi,6)
c              rklij(fi,9) = d1d2_r2t (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  0,2,0)
c              rklij(fi,10) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  0,1,1)
c              rklij(fi,11) = rklij(fi,7)
c              rklij(fi,12) = rklij(fi,10)
c              rklij(fi,13) = d1d2_r2u (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,0,  0,0,2)
c           else if (fi .eq. 5) then
c              rklij(fi,1) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,0,0)
c              rklij(fi,2) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  1,0,0)
c              rklij(fi,3) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,1,0)
c              rklij(fi,4) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,0,1)
c              rklij(fi,5) = d1d2_r2is (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  2,0,0)
c              rklij(fi,6) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  1,1,0)
c              rklij(fi,7) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  1,0,1)
c              rklij(fi,8) = rklij(fi,6)
c              rklij(fi,9) = d1d2_r2it (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,2,0)
c              rklij(fi,10) = d1d2_r2i (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,1,1)
c              rklij(fi,11) = rklij(fi,7)
c              rklij(fi,12) = rklij(fi,10)
c              rklij(fi,13) = d1d2_r2iu (fii,xi,yi,zi,xk,yk,zk,
c    &                              2,0,0,  0,0,2)
c           else if (fi .eq. 9) then
c              rklij(fi,1) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,0,0)
c              rklij(fi,2) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  1,0,0)
c              rklij(fi,3) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,1,0)
c              rklij(fi,4) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,0,1)
c              rklij(fi,5) = d1d2_r2js (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  2,0,0)
c              rklij(fi,6) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  1,1,0)
c              rklij(fi,7) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  1,0,1)
c              rklij(fi,8) = rklij(fi,6)
c              rklij(fi,9) = d1d2_r2jt (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,2,0)
c              rklij(fi,10) = d1d2_r2j (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,1,1)
c              rklij(fi,11) = rklij(fi,7)
c              rklij(fi,12) = rklij(fi,10)
c              rklij(fi,13) = d1d2_r2ju (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,2,0,  0,0,2)
c           else if (fi .eq. 13) then
c              rklij(fi,1) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,0,0)
c              rklij(fi,2) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  1,0,0)
c              rklij(fi,3) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,1,0)
c              rklij(fi,4) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,0,1)
c              rklij(fi,5) = d1d2_r (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  2,0,0)
c              rklij(fi,6) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  1,1,0)
c              rklij(fi,7) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  1,0,1)
c              rklij(fi,8) = rklij(fi,6)
c              rklij(fi,9) = d1d2_r (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,2,0)
c              rklij(fi,10) = d1d2_r2k (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,1,1)
c              rklij(fi,11) = rklij(fi,7)
c              rklij(fi,12) = rklij(fi,10)
c              rklij(fi,13) = d1d2_r (fii,xi,yi,zi,xk,yk,zk,
c    &                              0,0,2,  0,0,2)
c           else
c              ind_x = ind1_x(fi)
c              ind_y = ind1_y(fi)
c              ind_z = ind1_z(fi)
c              rklij(fi,1) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,  0,0,0)
c              rklij(fi,2) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,        1,0,0)
c              rklij(fi,3) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,         0,1,0)
c              rklij(fi,4) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,        0,0,1)
c              rklij(fi,5) = d1d2_r2s (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,        2,0,0)
c              rklij(fi,6) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,        1,1,0)
c              rklij(fi,7) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,        1,0,1)
c              rklij(fi,8) = rklij(fi,6)
c              rklij(fi,9) = d1d2_r2t (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,       0,2,0)
c              rklij(fi,10) = d1d2_r1 (fii,xi,yi,zi,xk,yk,zk,
c    &                        ind_x,ind_y,ind_z,       0,1,1)
c              rklij(fi,11) = rklij(fi,7)
c              rklij(fi,12) = rklij(fi,10)
c              rklij(fi,13) = d1d2_r2u (fii,xi,yi,zi,xk,yk,zk,
c    &                         ind_x,ind_y,ind_z,       0,0,2)               
c           end if
c        end do         
c
c     recursive formulation of 4th through nth summation terms
c
      do fii = 4, nn
         do fi = 1, p_e1
            if (fi .eq. 8) then
               do fj = 1, p_e2
                  rklij(fi,fj) = rklij(6,fj)
               end do
            else if (fi .eq. 11) then
               do fj = 1, p_e2
                  rklij(fi,fj) = rklij(7,fj)
               end do
            else if (fi .eq. 12) then
               do fj = 1, p_e2
                  rklij(fi,fj) = rklij(10,fj)
               end do
            else
               do fj = 1, p_e2
                  if (fj .eq. 8) then
                     rklij(fi,fj) = rklij(fi,6)
                  else if (fj .eq. 11) then
                     rklij(fi,fj) = rklij(fi,7)
                  else if (fj .eq. 12) then
                     rklij(fi,fj) = rklij(fi,10)
                  else
                     rklij(fi,fj) = d1d2 (fii,xi,yi,zi,xk,yk,zk,
     &                                    d,ri2,rk2,ind1_x(fi),
     &                                    ind1_y(fi),ind1_z(fi),
     &                              ind2_x(fj),ind2_y(fj),ind2_z(fj))
                  end if
               end do
            end if
         end do
c
c     update storage of the last two sets of matrix elements
c
         do fi = 1, p_e1
           do fj = 1, p_e2
              b2(fj,fi) = b1(fj,fi)
              b1(fj,fi) = rklij(fj,fi)
           end do
         end do
c
c     compute interaction energy between the two multipole sites
c
         do fi = 1, mdqsiz
            m2t2(fi) = 0.0d0
            do fj = 1, mdqsiz
               m2t2(fi) = m2t2(fi) + rklij(fi,fj)*rpk(fj)
            end do
         end do
         term = 0.0d0
         do fi = 1, mdqsiz
            term = term + rpi(fi)*m2t2(fi)
         end do
         term = term * size * dble(fii+1)
     &             / (dble(fii+1)*ratio+dble(fii))
         eik = eik + term
         size = size * size2
      end do
      eik = factor * eik
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine rindex  ##
c     ##                     ##
c     #########################
c
c
      subroutine rindex (n,m,ind_x,ind_y,ind_z,p_s,p_e)
      implicit none
      integer i,j,k,n,m,p_s,p_e
      integer ind_x(m),ind_y(m),ind_z(m)
c
c
      p_s = 1
      p_e = 1
c
c
      do i = 1, m
         ind_x(i) = 0
         ind_y(i) = 0
         ind_z(i) = 0
      end do
c
c
      k = 1
      do i = 1, n
         do j = p_s, p_e
            k = k + 1  
            ind_x(k) = ind_x(j) + 1
            ind_y(k) = ind_y(j) 
            ind_z(k) = ind_z(j)
         end do
         do j = p_s, p_e
            k = k + 1  
            ind_x(k) = ind_x(j) 
            ind_y(k) = ind_y(j) + 1
            ind_z(k) = ind_z(j) 
         end do
         do j = p_s, p_e
            k = k + 1  
            ind_x(k) = ind_x(j) 
            ind_y(k) = ind_y(j) 
            ind_z(k) = ind_z(j) + 1
         end do
         p_s = p_e + 1
         p_e = k
      end do
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine ijk_pt  ##
c     ##                     ##
c     #########################
c
c
      subroutine ijk_pt
      implicit none
      include 'rxnfld.i'
      integer i,j,k
c
c
      do i = 0, 5
         do j = 0, 5
            do k = 0, 5
               ijk(i,j,k) = (3**(i+j+k) + 3**(j+k) + 3**k - 1) / 2
            end do
         end do
      end do
      return
      end
c
c
c     #####################
c     ##                 ##
c     ##  function d1d2  ##
c     ##                 ##
c     #####################
c
c
      function d1d2 (n,x1,y1,z1,x2,y2,z2,d,r1sq,r2sq,i,j,k,s,t,u)
      implicit none
      include 'rxnfld.i'
      integer n,i,j,k,s,t,u
      integer is,it,iu,js,jt,ju,ks,kt,ku
      real*8 x1,y1,z1,x2,y2,z2
      real*8 d1d2,d,r1sq,r2sq,f,g
c
c
      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
         d1d2 = 0.0d0
         return
      end if
c
c
      is = i*s
      it = i*t
      iu = i*u
      js = j*s
      jt = j*t
      ju = j*u
      ks = k*s
      kt = k*t
      ku = k*u
c
c
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c
c
      if (i .ne. 0) then
         f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
         g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
         if (i .ne. 1)  g = g + i*(i-1)*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))
      end if
      if (j .ne. 0) then
         f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
         g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
         if (j .ne. 1)  g = g + j*(j-1)*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))
      end if
      if (k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
         if (k .ne. 1)  g = g + k*(k-1)*r2sq*b2(ijk(i,j,k-2),ijk(s,t,u))
      end if
      if (s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
         if (s .ne. 1)  g = g + s*(s-1)*r1sq*b2(ijk(i,j,k),ijk(s-2,t,u))
      end if
      if (t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
         if (t .ne. 1)  g = g + t*(t-1)*r1sq*b2(ijk(i,j,k),ijk(s,t-2,u))
      end if
      if (u .ne. 0) then
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
         if (u .ne. 1)  g = g + u*(u-1)*r1sq*b2(ijk(i,j,k),ijk(s,t,u-2))
      end if
c
c
      if (is .ne. 0) then
         f = f + is*b1(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 4.0d0*is*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
         if (i .ne. 1) then
            g = g + 2.0d0*(i-1)*is*x2*b2(ijk(i-2,j,k),ijk(s-1,t,u))
            if (s .ne. 1)
     &         g = g + (i-1)*(s-1)*is*b2(ijk(i-2,j,k),ijk(s-2,t,u))
         end if
         if (s .ne. 1)
     &      g = g + 2.0d0*(s-1)*is*x1*b2(ijk(i-1,j,k),ijk(s-2,t,u))
      end if
      if (jt .ne. 0) then
         f = f + jt*b1(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 4.0d0*jt*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
         if (j .ne. 1) then
            g = g + 2.0d0*(j-1)*jt*y2*b2(ijk(i,j-2,k),ijk(s,t-1,u))
            if (t .ne. 1)
     &         g = g + (j-1)*(t-1)*jt*b2(ijk(i,j-2,k),ijk(s,t-2,u))
         end if
         if (t .ne. 1)
     &      g = g + 2.0d0*(t-1)*jt*y1*b2(ijk(i,j-1,k),ijk(s,t-2,u))
      end if
      if (ku .ne. 0) then
         f = f + ku*b1(ijk(i,j,k-1),ijk(s,t,u-1))
         g = g + 4.0d0*ku*z1*z2*b2(ijk(i,j,k-1),ijk(s,t,u-1))
         if (k .ne. 1) then
            g = g + 2.0d0*(k-1)*ku*z2*b2(ijk(i,j,k-2),ijk(s,t,u-1))
            if (u .ne. 1)
     &         g = g + (k-1)*(u-1)*ku*b2(ijk(i,j,k-2),ijk(s,t,u-2))
         end if
         if (u .ne. 1)
     &      g = g + 2.0d0*(u-1)*ku*z1*b2(ijk(i,j,k-1),ijk(s,t,u-2))
      end if
      if (it .ne. 0) then
         g = g + 4.0d0*it*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))
         if (i .ne. 1) then
            g = g + 2.0d0*(i-1)*it*y2*b2(ijk(i-2,j,k),ijk(s,t-1,u))
            if (t .ne. 1)
     &         g = g + (i-1)*(t-1)*it*b2(ijk(i-2,j,k),ijk(s,t-2,u))
         end if
         if (t .ne. 1)
     &      g = g + 2.0d0*(t-1)*it*x1*b2(ijk(i-1,j,k),ijk(s,t-2,u))
      end if
      if (iu .ne. 0) then
         g = g + 4.0d0*iu*x1*z2*b2(ijk(i-1,j,k),ijk(s,t,u-1))
         if (i .ne. 1) then
            g = g + 2.0d0*(i-1)*iu*z2*b2(ijk(i-2,j,k),ijk(s,t,u-1))
            if (u .ne. 1)
     &         g = g + (i-1)*(u-1)*iu*b2(ijk(i-2,j,k),ijk(s,t,u-2))
         end if
         if (u .ne. 1)
     &      g = g + 2.0d0*(u-1)*iu*x1*b2(ijk(i-1,j,k),ijk(s,t,u-2))
      end if
      if (js .ne. 0) then
         g = g + 4.0d0*js*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
         if (j .ne. 1) then
            g = g + 2.0d0*(j-1)*js*x2*b2(ijk(i,j-2,k),ijk(s-1,t,u))
            if (s .ne. 1)
     &         g = g + (j-1)*(s-1)*js*b2(ijk(i,j-2,k),ijk(s-2,t,u))
         end if
         if (s .ne. 1)
     &      g = g + 2.0d0*(s-1)*js*y1*b2(ijk(i,j-1,k),ijk(s-2,t,u))
      end if
      if (ju .ne. 0) then
         g = g + 4.0d0*ju*y1*z2*b2(ijk(i,j-1,k),ijk(s,t,u-1))
         if (j .ne. 1) then
            g = g + 2.0d0*(j-1)*ju*z2*b2(ijk(i,j-2,k),ijk(s,t,u-1))
            if (u .ne. 1)
     &         g = g + (j-1)*(u-1)*ju*b2(ijk(i,j-2,k),ijk(s,t,u-2))
         end if
         if (u .ne. 1)
     &      g = g + 2.0d0*(u-1)*ju*y1*b2(ijk(i,j-1,k),ijk(s,t,u-2))
      end if
      if (ks .ne. 0) then
         g = g + 4.0d0*ks*z1*x2*b2(ijk(i,j,k-1),ijk(s-1,t,u))
         if (k .ne. 1) then
            g = g + 2.0d0*(k-1)*ks*x2*b2(ijk(i,j,k-2),ijk(s-1,t,u))
            if (s .ne. 1)
     &         g = g + (k-1)*(s-1)*ks*b2(ijk(i,j,k-2),ijk(s-2,t,u))
         end if
         if (s .ne. 1)
     &      g = g + 2.0d0*(s-1)*ks*z1*b2(ijk(i,j,k-1),ijk(s-2,t,u))
      end if
      if (kt .ne. 0) then
         g = g + 4.0d0*kt*z1*y2*b2(ijk(i,j,k-1),ijk(s,t-1,u))
         if (k .ne. 1) then
            g = g + 2.0d0*(k-1)*kt*y2*b2(ijk(i,j,k-2),ijk(s,t-1,u))
            if (t .ne. 1)
     &         g = g + (k-1)*(t-1)*kt*b2(ijk(i,j,k-2),ijk(s,t-2,u))
         end if
         if (t .ne. 1)
     &      g = g + 2.0d0*(t-1)*kt*z1*b2(ijk(i,j,k-1),ijk(s,t-2,u))
      end if
c
c
      f = dble(2*n-1) * f
      g = dble(n-1) * g
      d1d2 = (f-g) / dble(n)
      return
      end
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Below here are Mike's new versions of the routines
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c
c     #######################
c     ##                   ##
c     ##  function d1d2_r  ##
c     ##                   ##
c     #######################
c
c
      function d1d2_r (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer is,it,iu,js,jt,ju,ks,kt,ku 

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
         d1d2_r = 0.0d0
         return
      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      is = i*s
      it = i*t
      iu = i*u
      js = j*s
      jt = j*t
      ju = j*u
      ks = k*s
      kt = k*t
      ku = k*u
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      if(i .ne. 0) then
         f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
         g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
         if(i .ne. 1) g = g + i*(i-1)*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))
      end if
      if(j .ne. 0) then
         f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
         g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
         if(j .ne. 1) g = g + j*(j-1)*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))
      end if
      if(k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
         if(k .ne. 1) g = g + k*(k-1)*r2sq*b2(ijk(i,j,k-2),ijk(s,t,u))
      end if
      if(s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
         if(s .ne. 1) g = g + s*(s-1)*r1sq*b2(ijk(i,j,k),ijk(s-2,t,u))
      end if
      if(t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
         if(t .ne. 1) g = g + t*(t-1)*r1sq*b2(ijk(i,j,k),ijk(s,t-2,u))
      end if
      if(u .ne. 0) then 
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
         if(u .ne. 1) g = g + u*(u-1)*r1sq*b2(ijk(i,j,k),ijk(s,t,u-2))
      end if
      if(is .ne. 0)  then
         f = f + is*b1(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 4.0d0*is*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
         if(i .ne.1) then
            g = g + 2.0d0*(i-1)*is*x2*b2(ijk(i-2,j,k),ijk(s-1,t,u))
            if(s .ne.1) 
     &       g = g + (i-1)*(s-1)*is*b2(ijk(i-2,j,k),ijk(s-2,t,u))  
         end if
         if(s .ne.1) then
            g = g + 2.0d0*(s-1)*is*x1*b2(ijk(i-1,j,k),ijk(s-2,t,u))
         end if
      end if
      if(jt.ne. 0)  then
         f = f + jt*b1(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 4.0d0*jt*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
         if(j .ne.1) then
            g = g + 2.0d0*(j-1)*jt*y2*b2(ijk(i,j-2,k),ijk(s,t-1,u))
            if(t .ne.1) 
     &       g = g + (j-1)*(t-1)*jt*b2(ijk(i,j-2,k),ijk(s,t-2,u))  
         end if
         if(t .ne.1) then
            g = g + 2.0d0*(t-1)*jt*y1*b2(ijk(i,j-1,k),ijk(s,t-2,u))
         end if
      end if
      if(ku .ne. 0) then
         f=f + ku*b1(ijk(i,j,k-1),ijk(s,t,u-1))
         g = g + 4.0d0*ku*z1*z2*b2(ijk(i,j,k-1),ijk(s,t,u-1))
         if(k .ne.1) then
            g = g + 2.0d0*(k-1)*ku*z2*b2(ijk(i,j,k-2),ijk(s,t,u-1))
            if(u .ne.1) 
     &       g = g + (k-1)*(u-1)*ku*b2(ijk(i,j,k-2),ijk(s,t,u-2))  
         end if
         if(u .ne.1) then
            g = g + 2.0d0*(u-1)*ku*z1*b2(ijk(i,j,k-1),ijk(s,t,u-2))
         end if
      end if


      if(it .ne. 0) then
         g = g + 4.0d0*it*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))
         if(i .ne.1) then
            g = g + 2.0d0*(i-1)*it*y2*b2(ijk(i-2,j,k),ijk(s,t-1,u))
            if(t .ne.1) 
     &       g = g + (i-1)*(t-1)*it*b2(ijk(i-2,j,k),ijk(s,t-2,u))  
         end if
         if(t .ne.1) then
            g = g + 2.0d0*(t-1)*it*x1*b2(ijk(i-1,j,k),ijk(s,t-2,u))
         end if
      end if         
      if(iu .ne. 0) then
         g = g + 4.0d0*iu*x1*z2*b2(ijk(i-1,j,k),ijk(s,t,u-1))
         if(i .ne.1) then
            g = g + 2.0d0*(i-1)*iu*z2*b2(ijk(i-2,j,k),ijk(s,t,u-1))
            if(u .ne.1) 
     &       g = g + (i-1)*(u-1)*iu*b2(ijk(i-2,j,k),ijk(s,t,u-2))  
         end if
         if(u .ne.1) then
            g = g + 2.0d0*(u-1)*iu*x1*b2(ijk(i-1,j,k),ijk(s,t,u-2))
         end if
      end if
      if(js .ne. 0) then
         g = g + 4.0d0*js*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
         if(j .ne.1) then
            g = g + 2.0d0*(j-1)*js*x2*b2(ijk(i,j-2,k),ijk(s-1,t,u))
            if(s .ne.1) 
     &       g = g + (j-1)*(s-1)*js*b2(ijk(i,j-2,k),ijk(s-2,t,u))  
         end if
         if(s .ne.1) then
            g = g + 2.0d0*(s-1)*js*y1*b2(ijk(i,j-1,k),ijk(s-2,t,u))
         end if
      end if
      if(ju .ne. 0) then
         g = g + 4.0d0*ju*y1*z2*b2(ijk(i,j-1,k),ijk(s,t,u-1))
         if(j .ne.1) then
            g = g + 2.0d0*(j-1)*ju*z2*b2(ijk(i,j-2,k),ijk(s,t,u-1))
            if(u .ne.1) 
     &       g = g + (j-1)*(u-1)*ju*b2(ijk(i,j-2,k),ijk(s,t,u-2))  
         end if
         if(u .ne.1) then
            g = g + 2.0d0*(u-1)*ju*y1*b2(ijk(i,j-1,k),ijk(s,t,u-2))
         end if
      end if
      if(ks .ne. 0) then
         g = g + 4.0d0*ks*z1*x2*b2(ijk(i,j,k-1),ijk(s-1,t,u))
         if(k .ne.1) then
            g = g + 2.0d0*(k-1)*ks*x2*b2(ijk(i,j,k-2),ijk(s-1,t,u))
            if(s .ne.1) 
     &       g = g + (k-1)*(s-1)*ks*b2(ijk(i,j,k-2),ijk(s-2,t,u))  
         end if
         if(s .ne.1) then
            g = g + 2.0d0*(s-1)*ks*z1*b2(ijk(i,j,k-1),ijk(s-2,t,u))
         end if
      end if
      if(kt .ne. 0) then
         g = g + 4.0d0*kt*z1*y2*b2(ijk(i,j,k-1),ijk(s,t-1,u))
         if(k .ne.1) then
            g = g + 2.0d0*(k-1)*kt*y2*b2(ijk(i,j,k-2),ijk(s,t-1,u))
            if(t .ne.1) 
     &       g = g + (k-1)*(t-1)*kt*b2(ijk(i,j,k-2),ijk(s,t-2,u))  
         end if
         if(t .ne.1) then
            g = g + 2.0d0*(t-1)*kt*z1*b2(ijk(i,j,k-1),ijk(s,t-2,u))
         end if
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r = (f-g)/n

      return
      end

c
c     ########################
c     ##                    ##
c     ##  function d1d2_r1  ##
c     ##                    ##
c     ########################
c
c
      function d1d2_r1 (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r1
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer is,it,iu,js,jt,ju,ks,kt,ku 

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
         d1d2_r1 = 0.0d0
         return
      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      is = i*s
      it = i*t
      iu = i*u
      js = j*s
      jt = j*t
      ju = j*u
      ks = k*s
      kt = k*t
      ku = k*u
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      if(i .ne. 0) then
         f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
         g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      end if
      if(j .ne. 0) then
         f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
         g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      end if
      if(k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
      end if
      if(s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      end if
      if(t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      end if
      if(u .ne. 0) then 
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      end if
      if(is .ne. 0)  then
         f = f + is*b1(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 4.0d0*is*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
      end if
      if(jt.ne. 0)  then
         f = f + jt*b1(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 4.0d0*jt*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
      end if
      if(ku .ne. 0) then
         f=f + ku*b1(ijk(i,j,k-1),ijk(s,t,u-1))
         g = g + 4.0d0*ku*z1*z2*b2(ijk(i,j,k-1),ijk(s,t,u-1))
      end if
      if(it .ne. 0) then
         g = g + 4.0d0*it*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))
      end if         
      if(iu .ne. 0) then
         g = g + 4.0d0*iu*x1*z2*b2(ijk(i-1,j,k),ijk(s,t,u-1))
      end if
      if(js .ne. 0) then
         g = g + 4.0d0*js*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
      end if
      if(ju .ne. 0) then
         g = g + 4.0d0*ju*y1*z2*b2(ijk(i,j-1,k),ijk(s,t,u-1))
      end if
      if(ks .ne. 0) then
         g = g + 4.0d0*ks*z1*x2*b2(ijk(i,j,k-1),ijk(s-1,t,u))
      end if
      if(kt .ne. 0) then
         g = g + 4.0d0*kt*z1*y2*b2(ijk(i,j,k-1),ijk(s,t-1,u))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r1 = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2i  ##
c     ##                     ##
c     #########################
c
c
cccccccccccccccccccccccccccccccccccccc
c     (2,0,0),(s,t,u,)
cccccccccccccccccccccccccccccccccccccc
      function d1d2_r2i (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2i
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer is,it,iu

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r1 = 0.0d0
c        return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      is = i*s
      it = i*t
      iu = i*u

      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
      g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      g = g + i*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))

      if(s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      end if
      if(t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      end if
      if(u .ne. 0) then 
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      end if
      if(is .ne. 0)  then
         f = f + is*b1(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 4.0d0*is*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*is*x2*b2(ijk(i-2,j,k),ijk(s-1,t,u))
      end if
      if(it .ne. 0) then
         g = g + 4.0d0*it*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*it*y2*b2(ijk(i-2,j,k),ijk(s,t-1,u))
      end if         
      if(iu .ne. 0) then
         g = g + 4.0d0*iu*x1*z2*b2(ijk(i-1,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*iu*z2*b2(ijk(i-2,j,k),ijk(s,t,u-1))
      end if

      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2i = (f-g)/n

      return
      end

c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2j  ##
c     ##                     ##
c     #########################
c
c
cccccccccccccccccccccccccccccccccccccc
c     (0,2,0),(s,t,u,)
cccccccccccccccccccccccccccccccccccccc
      function d1d2_r2j (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2j
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer js,jt,ju

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r1 = 0.0d0
c         return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      js = j*s
      jt = j*t
      ju = j*u
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g


      f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
      g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      g = g + j*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))

      if(s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      end if
      if(t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      end if
      if(u .ne. 0) then 
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      end if
      if(jt.ne. 0)  then
         f = f + jt*b1(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 4.0d0*jt*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 2.0d0*jt*y2*b2(ijk(i,j-2,k),ijk(s,t-1,u))
      end if
      if(js .ne. 0) then
         g = g + 4.0d0*js*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
         g = g + 2.0d0*js*x2*b2(ijk(i,j-2,k),ijk(s-1,t,u))
      end if
      if(ju .ne. 0) then
         g = g + 4.0d0*ju*y1*z2*b2(ijk(i,j-1,k),ijk(s,t,u-1))
         g = g + 2.0d0*ju*z2*b2(ijk(i,j-2,k),ijk(s,t,u-1))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2j = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2k  ##
c     ##                     ##
c     #########################
c
c
ccccccccccccccccccccccccccccccccccccccccc
c    (0,0,2) (s,t,u)
ccccccccccccccccccccccccccccccccccccccccc 
      function d1d2_r2k (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2k
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer ks,kt,ku 

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r1 = 0.0d0
c         return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      ks = k*s
      kt = k*t
      ku = k*u
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g


      f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
      g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
      g = g + k*r2sq*b2(ijk(i,j,k-2),ijk(s,t,u))
      if(s .ne. 0) then
         f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      end if
      if(t .ne. 0) then
         f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      end if
      if(u .ne. 0) then 
         f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
         g = g + 2.0d0*u*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      end if
      if(ku .ne. 0) then
         f=f + ku*b1(ijk(i,j,k-1),ijk(s,t,u-1))
         g = g + 4.0d0*ku*z1*z2*b2(ijk(i,j,k-1),ijk(s,t,u-1))
         g = g + 2.0d0*ku*z2*b2(ijk(i,j,k-2),ijk(s,t,u-1))
      end if
      if(ks .ne. 0) then
         g = g + 4.0d0*ks*z1*x2*b2(ijk(i,j,k-1),ijk(s-1,t,u))
         g = g + 2.0d0*ks*x2*b2(ijk(i,j,k-2),ijk(s-1,t,u))
      end if
      if(kt .ne. 0) then
         g = g + 4.0d0*kt*z1*y2*b2(ijk(i,j,k-1),ijk(s,t-1,u))
         g = g + 2.0d0*kt*y2*b2(ijk(i,j,k-2),ijk(s,t-1,u))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2k = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2s  ##
c     ##                     ##
c     #########################
c
c
cccccccccccccccccccccccccccccccccccccccc
c     (i,j,k) (2,0,0)
cccccccccccccccccccccccccccccccccccccccc
      function d1d2_r2s (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2s
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer is,js,ks

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r2s = 0.0d0
c         return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      is = i*s
      js = j*s
      ks = k*s
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      if(i .ne. 0) then
         f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
         g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      end if
      if(j .ne. 0) then
         f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
         g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      end if
      if(k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
      end if
      f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
      g = g + 2.0d0*s*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      g = g + s*r1sq*b2(ijk(i,j,k),ijk(s-2,t,u))
      if(is .ne. 0)  then
         f = f + is*b1(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 4.0d0*is*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
         g = g + 2.0d0*is*x1*b2(ijk(i-1,j,k),ijk(s-2,t,u))
      end if
      if(js .ne. 0) then
         g = g + 4.0d0*js*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
         g = g + 2.0d0*js*y1*b2(ijk(i,j-1,k),ijk(s-2,t,u))
      end if
      if(ks .ne. 0) then
         g = g + 4.0d0*ks*z1*x2*b2(ijk(i,j,k-1),ijk(s-1,t,u))
         g = g + 2.0d0*ks*z1*b2(ijk(i,j,k-1),ijk(s-2,t,u))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2s = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2t  ##
c     ##                     ##
c     #########################
c
c
      function d1d2_r2t (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2t
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer it,jt,kt

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r1 = 0.0d0
c         return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      it = i*t
      jt = j*t
      kt = k*t
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      if(i .ne. 0) then
         f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
         g = g + 2.0d0*i*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      end if
      if(j .ne. 0) then
         f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
         g = g + 2.0d0*j*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      end if
      if(k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),ijk(s,t,u))
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),ijk(s,t,u))
      end if
      f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
      g = g + 2.0d0*t*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      g = g + t*r1sq*b2(ijk(i,j,k),ijk(s,t-2,u))
      if(jt.ne. 0)  then
         f = f + jt*b1(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 4.0d0*jt*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
         g = g + 2.0d0*jt*y1*b2(ijk(i,j-1,k),ijk(s,t-2,u))
      end if
      if(it .ne. 0) then
         g = g + 4.0d0*it*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))
         g = g + 2.0d0*it*x1*b2(ijk(i-1,j,k),ijk(s,t-2,u))
      end if         
      if(kt .ne. 0) then
         g = g + 4.0d0*kt*z1*y2*b2(ijk(i,j,k-1),ijk(s,t-1,u))
         g = g + 2.0d0*kt*z1*b2(ijk(i,j,k-1),ijk(s,t-2,u))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2t = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_r2u  ##
c     ##                     ##
c     #########################
c
c
      function d1d2_r2u (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2u
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      integer iu,ju,ku
      integer i_jk,stu,stu1,i1jk,ij1k

c      write(*,99) n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u
c   99 format(i8,6f10.5,6i8)


c      if (n.lt.i+j+k  .or. n.lt.s+t+u) then
c         d1d2_r1 = 0.0d0
c         return
c      end if
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      iu = i*u
      ju = j*u
      ku = k*u
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
c      write(*,*) "F0,G0",f,g

      stu = ijk(s,t,u)
      stu1 = ijk(s,t,u-1)
      i_jk = ijk(i,j,k)
      i1jk = ijk(i-1,j,k)
      ij1k = ijk(i,j-1,k)

      if(i .ne. 0) then
         f = f + i*x2*b1(i1jk,stu)
         g = g + 2.0d0*i*x1*r2sq*b2(i1jk,stu)
      end if
      if(j .ne. 0) then
         f = f + j*y2*b1(ij1k,stu)
         g = g + 2.0d0*j*y1*r2sq*b2(ij1k,stu)
      end if
      if(k .ne. 0) then
         f = f + k*z2*b1(ijk(i,j,k-1),stu)
         g = g + 2.0d0*k*z1*r2sq*b2(ijk(i,j,k-1),stu)
      end if
      f = f + u*z1*b1(i_jk,stu1)
      g = g + 2.0d0*u*z2*r1sq*b2(i_jk,stu1)
      g = g + u*r1sq*b2(i_jk,ijk(s,t,u-2))
      if(ku .ne. 0) then
         f=f + ku*b1(ijk(i,j,k-1),stu1)
         g = g + 4.0d0*ku*z1*z2*b2(ijk(i,j,k-1),stu1)
         g = g + 2.0d0*ku*z1*b2(ijk(i,j,k-1),ijk(s,t,u-2))
      end if
      if(iu .ne. 0) then
         g = g + 4.0d0*iu*x1*z2*b2(i1jk,stu1)
         g = g + 2.0d0*iu*x1*b2(i1jk,ijk(s,t,u-2))
      end if
      if(ju .ne. 0) then
         g = g + 4.0d0*ju*y1*z2*b2(ij1k,stu1)
         g = g + 2.0d0*ju*y1*b2(ij1k,ijk(s,t,u-2))
      end if


c      write(*,*) "F,G",f,g
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2u = (f-g)/n

      return
      end
c
c     ##########################
c     ##                      ##
c     ##  function d1d2_r2is  ##
c     ##                      ##
c     ##########################
c
c
cccccccccccccccccccccccccccccccccccccc
c     (2,0,0),(2,0,0,)
cccccccccccccccccccccccccccccccccccccc

      function d1d2_r2is (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2is
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2

      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))

      f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
      g = g + 4.0d0*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      g = g + i*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))
      f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
      g = g + 4.0d0*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      g = g + s*r1sq*b2(ijk(i,j,k),ijk(s-2,t,u))
      f = f + 4.0d0*b1(ijk(i-1,j,k),ijk(s-1,t,u))
      g = g + 16.0d0*x1*x2*b2(ijk(i-1,j,k),ijk(s-1,t,u))
      g = g + 8.0d0*x2*b2(ijk(i-2,j,k),ijk(s-1,t,u))
      g = g + 4.0d0*b2(ijk(i-2,j,k),ijk(s-2,t,u))  
      g = g + 8.0d0**x1*b2(ijk(i-1,j,k),ijk(s-2,t,u))
     
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2is = (f-g)/n

      return
      end
c
c     ##########################
c     ##                      ##
c     ##  function d1d2_r2it  ##
c     ##                      ##
c     ##########################
c
c
      function d1d2_r2it (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2it
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g

      
      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))

      
      f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
      g = g + 4.0d0*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      g = g + i*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))
      f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
      g = g + 4.0d0*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      g = g + t*r1sq*b2(ijk(i,j,k),ijk(s,t-2,u))
      g = g + 16.0d0*x1*y2*b2(ijk(i-1,j,k),ijk(s,t-1,u))         
      g = g + 8.0d0*y2*b2(ijk(i-2,j,k),ijk(s,t-1,u))
      g = g + 4.0d0*b2(ijk(i-2,j,k),ijk(s,t-2,u))  
      g = g + 8.0d0*x1*b2(ijk(i-1,j,k),ijk(s,t-2,u))

      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2it = (f-g)/n

      return
      end
c
c     ##########################
c     ##                      ##
c     ##  function d1d2_r2iu  ##
c     ##                      ##
c     ##########################
c
c
      function d1d2_r2iu (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2iu
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))


      f = f + i*x2*b1(ijk(i-1,j,k),ijk(s,t,u))
      g = g + 4.0d0*x1*r2sq*b2(ijk(i-1,j,k),ijk(s,t,u))
      g = g + i*r2sq*b2(ijk(i-2,j,k),ijk(s,t,u))
      f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
      g = g + 4.0d0*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      g = g + u*r1sq*b2(ijk(i,j,k),ijk(s,t,u-2))
      g = g + 16.0d0*x1*z2*b2(ijk(i-1,j,k),ijk(s,t,u-1))
      g = g + 8.0d0*z2*b2(ijk(i-2,j,k),ijk(s,t,u-1))
      g = g + 4.0d0*b2(ijk(i-2,j,k),ijk(s,t,u-2))
      g = g + 8.0d0*x1*b2(ijk(i-1,j,k),ijk(s,t,u-2))
         
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2iu = (f-g)/n

      return
      end

c
c     ##########################
c     ##                      ##
c     ##  function d1d2_r2js  ##
c     ##                      ##
c     ##########################
c
c
      function d1d2_r2js (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2js
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g
      
      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
      f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
      g = g + 4.0d0*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      g = g + j*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))
      
      f = f + s*x1*b1(ijk(i,j,k),ijk(s-1,t,u))
      g = g + 4.0d0*x2*r1sq*b2(ijk(i,j,k),ijk(s-1,t,u))
      g = g + s*r1sq*b2(ijk(i,j,k),ijk(s-2,t,u))
      g = g + 16.0d0*y1*x2*b2(ijk(i,j-1,k),ijk(s-1,t,u))
         
      g = g + 8.0d0*x2*b2(ijk(i,j-2,k),ijk(s-1,t,u))
            
      g = g + 4.0d0*b2(ijk(i,j-2,k),ijk(s-2,t,u))  
         

      g = g + 8.0d0*4.0d0*y1*b2(ijk(i,j-1,k),ijk(s-2,t,u))
      
      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2js = (f-g)/n

      return
      end
c
c     #########################
c     ##                     ##
c     ##  function d1d2_rjt  ##
c     ##                     ##
c     #########################
c
c
      function d1d2_r2jt (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2jt
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))
      
      f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
      g = g + 4.0d0*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      g = g + j*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))
      

      
      f = f + t*y1*b1(ijk(i,j,k),ijk(s,t-1,u))
      g = g + 4.0d0*y2*r1sq*b2(ijk(i,j,k),ijk(s,t-1,u))
      g = g + t*r1sq*b2(ijk(i,j,k),ijk(s,t-2,u))
      

      
      f = f + 4.0d0*b1(ijk(i,j-1,k),ijk(s,t-1,u))
      g = g + 16.0d0*y1*y2*b2(ijk(i,j-1,k),ijk(s,t-1,u))
         
      g = g + 8.0d0*y2*b2(ijk(i,j-2,k),ijk(s,t-1,u))
            
      g = g + 4.0d0*b2(ijk(i,j-2,k),ijk(s,t-2,u))  
         
      g = g + 8.0d0*y1*b2(ijk(i,j-1,k),ijk(s,t-2,u))
       

      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2jt = (f-g)/n

      return
      end
c
c     ##########################
c     ##                      ##
c     ##  function d1d2_r2ju  ##
c     ##                      ##
c     ##########################
c
c
      function d1d2_r2ju (n,x1,y1,z1,x2,y2,z2,i,j,k,s,t,u)
      implicit none

      include 'rxnfld.i'

      real*8 d1d2_r2ju
      real*8 x1,y1,z1,x2,y2,z2
      integer n,i,j,k,s,t,u

      real*8 d,r1sq,r2sq
      real*8 f,g

      d = x1*x2 + y1*y2 + z1*z2
      r1sq = x1*x1 + y1*y1 + z1*z1
      r2sq = x2*x2 + y2*y2 + z2*z2
      
      f = d*b1(ijk(i,j,k),ijk(s,t,u))
      g = r1sq*r2sq*b2(ijk(i,j,k),ijk(s,t,u))


      
      f = f + j*y2*b1(ijk(i,j-1,k),ijk(s,t,u))
      g = g + 4.0d0*y1*r2sq*b2(ijk(i,j-1,k),ijk(s,t,u))
      g = g + j*r2sq*b2(ijk(i,j-2,k),ijk(s,t,u))
     

      
      f = f + u*z1*b1(ijk(i,j,k),ijk(s,t,u-1))
      g = g + 4.0d0*z2*r1sq*b2(ijk(i,j,k),ijk(s,t,u-1))
      g = g + u*r1sq*b2(ijk(i,j,k),ijk(s,t,u-2))
      


      
      g = g + 16.0d0*y1*z2*b2(ijk(i,j-1,k),ijk(s,t,u-1))
         
      g = g + 8.0d0*z2*b2(ijk(i,j-2,k),ijk(s,t,u-1))
            
      g = g + 4.0d0*b2(ijk(i,j-2,k),ijk(s,t,u-2))  
         
      g = g + 8.0d0*y1*b2(ijk(i,j-1,k),ijk(s,t,u-2))
         

      f = (2*n-1)*f
      g = (n-1)*g
      d1d2_r2ju = (f-g)/n

      return
      end
