c
c     ####################################################################
c     ##                                                                ##
c     ##  program sdpath  --  Steepest-Descent path is followed from TS ##
c     ##                                                                ##
c     ####################################################################
c
c
c   what we want to do with this is to take the initial step along the
c   transition vector [ or in reverse direction ]  and then do SDP till
c   a minimum is found. use vibrate results as input.
c
c
c   need to make sequential 000n files from R to P
c   need to be able to pass an itial value from prog to prog
c
c
c
c
      program sdpath
      implicit none
      include 'sizes.i'
c     TINKER VARIABLES
      include 'align.i'
      include 'atoms.i'
      include 'atmtyp.i'
      include 'iounit.i'
      include 'output.i'
      include 'files.i'
c     PATH CALCULATION VARIABLES
      include 'pathsz.i'
      include 'pathdata.i'
      include 'pathatm.i'
      include 'pathtol.i'
      include 'sdpcycl.i'
      include 'sdpfiles.i'
      integer j,j1,j2,j3,freeunit
      real*8 fctval,sdp1,dume
      real*8 p(3*maxatm),tsvec(3*maxatm)
      real*8 deriv(3*maxatm)
      character*1 digit
      character*60 coordfile,hbdfile,nrgfile,radfile
      data digit / '1' /
      external sdpout,sdp1,freeunit
c
c     initialize some TINKER stuff
c
      call initial
c
c     get and store TS coordinates
c
      call getxyz
      do j = 1, n
         j1 = 3*(j-1)+1
         j2 = j1+1
         j3 = j2+1
         p0(j1) = x(j)
         p0(j2) = y(j)
         p0(j3) = z(j)
      end do
c
c     set number of atoms, and 3 x natoms
c
      natoms = n
      nat3 = 3 * natoms
c
c     setup the molecular mechanics energy stuff
c
      call mechanic
      savecycle=.true.
      grdmin = 0.001d0
c
c     Get the Transition Vector
c
      call vib(tsvec)
c
c     make file name for the current path
c
      sdpdir = -1
      coordfile = filename(1:leng)//digit
      leng = leng + 1
      filename = coordfile
c
c     open output files
c
      irad = freeunit ()
      radfile = filename(1:leng)//'.rdat'
      call version (radfile,'new')
      open (unit=irad,file=radfile,status='new')
c
      inrg = freeunit ()
      nrgfile = filename(1:leng)//'.edat'
      call version (nrgfile,'new')
      open (unit=inrg,file=nrgfile,status='new')
c
      ihbd = freeunit ()
      hbdfile = filename(1:leng)//'.hbout'
      call version (hbdfile,'new')
      open (unit=ihbd,file=hbdfile,status='new')
c
      do j = 1, nat3
         p(j) = p0(j) + tsvec(j)
      end do
      write(6,*)
      write(iout,*)
     &   '   IS STEEPEST_DESCENT SPECIFIED IN THE KEY FILE?'
      write(6,*)
c
c     follow the Steepest-Descent path in one direction
c
      call locmqn(nat3,p,fctval,grdmin,sdp1,sdpout)
c
c     write an appropriately numbered file for TS
c
      sdpdir = 0
      dume = sdp1(p0,deriv)
      call sdpout(p0,sdpiter)
c
c     ...  and in the other direction
c
      do j = 1, nat3
         p(j) = p0(j) - tsvec(j)
      end do
      sdpdir = 1
      call locmqn(nat3,p,fctval,grdmin,sdp1,sdpout)
c
      call writeinfo

      end
c
c
c     ########################################################
c     ##                                                    ##
c     ##  function sdp1  --  get the energy and derivative  ##
c     ##                                                    ##
c     ########################################################
c
c
      function sdp1(p,deriv)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'pathdata.i'
      integer j,i
      real*8 sdp1,energy
      real*8 deriv(3*maxatm)
      real*8 p(3*maxatm),nrgval
      common /cheat/ nrgval
c
c     get potential energy and gradient vector for Interminateediate
c
c
c     copy current coordinates to TINKER
c
      do i = 1, n
         x(i) = p(3*i-2)
         y(i) = p(3*i-1)
         z(i) = p(3*i)
      end do
c
c     compute TINKER energy and gradient
c
      call gradient (energy,deriv)
      sdp1 = energy
c
      nrgval = sdp1

      return
      end
c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ##################################################################
c     ##                                                              ##
c     ##  subroutine vib  --  vibrational analysis and normal modes   ##
c     ##                                                              ##
c     ##################################################################
c
c
c     "vib" performs a vibrational normal mode analysis; the
c     Hessian matrix of second derivatives is determinateined and then
c     diagonalized both directly and after mass weighting; output
c     consists of the eigenvalues of the force constant matrix as
c     well as the normal modes and their frequencies
c
c
      subroutine vib(tsvec)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'files.i'
      include 'hescut.i'
      include 'iounit.i'
      include 'math.i'
      integer i,j,k,m,icon,ihess,freeunit
      integer jstart,nfreq,ndummy,nvib,ivib,nview
      integer thousand,hundred,tens,ones
      integer hinit(3,maxatm),hstop(3,maxatm),hindex(maxhess)
      real*8 hess(maxhess),hdiag(3,maxatm)
      real*8 xold(maxatm),yold(maxatm),zold(maxatm)
      real*8 mass2(maxatm),factor,vnorm,ratio
      real*8 matrix((maxvib+1)*maxvib/2)
      real*8 eigen(maxvib),vects(maxvib**2),tsvec(3*maxatm)
      real*8 a(maxvib+1),b(maxvib+1),p(maxvib+1),w(maxvib+1)
      real*8 ta(maxvib+1),tb(maxvib+1),ty(maxvib+1)
      character*1 eigunits(maxvib),digit(0:9)
      character*5 seq
      character*60 coordfile
      data digit / '0','1','2','3','4','5','6','7','8','9' /
c
c
c     initialize various things needed for vibrations
c
      do j = 1, 3*maxatm
         tsvec(j) = 0.0d0
      end do
      nfreq = 3 * n
      ndummy = 0
      if (nfreq.gt.maxvib .or. nfreq**2.gt.maxhess) then
         write (iout,10)
   10    format (' VIBRATE  --  Too many Atoms in the Molecule')
         stop
      end if
      do i = 1, n
         if (atomic(i) .eq. 0) then
            ndummy = ndummy + 1
            mass(i) = 0.001d0
         end if
         mass2(i) = sqrt(mass(i))
      end do
      nvib = nfreq - 3*ndummy
c
c     calculate the Hessian matrix of second derivatives
c
      hesscut = -1.0d0
      call hessian (hess,hinit,hstop,hindex,hdiag)
c
c     store upper triangle of the Hessian in "matrix"
c
      ihess = 0
      do i = 1, n
         do j = 1, 3
            ihess = ihess + 1
            matrix(ihess) = hdiag(j,i)
            do k = hinit(j,i), hstop(j,i)
               ihess = ihess + 1
               matrix(ihess) = hess(k)
            end do
         end do
      end do
c
c     perform diagonalization to get Hessian eigenvalues
c
      call diagq (nfreq,nfreq,matrix,eigen,vects,a,b,p,w,ta,tb,ty)
c     write (iout,20)
   20 format (/' Eigenvalues of the Hessian Matrix :'/)
c     write (iout,30)  (i,eigen(i),i=1,nvib)
   30 format (5(i5,f9.2,1x))
c     write (iout,40)
   40 format ()
c
c     store upper triangle of mass-weighted Hessian in "matrix"
c
      ihess = 0
      do i = 1, n
         do j = 1, 3
            ihess = ihess + 1
            matrix(ihess) = hdiag(j,i) / mass(i)
            do k = hinit(j,i), hstop(j,i)
               m = (hindex(k)+2) / 3
               ihess = ihess + 1
               matrix(ihess) = hess(k) / (mass2(i)*mass2(m))
            end do
         end do
      end do
c
c     now, diagonalize and scale to get vibrational frequencies
c
      call diagq (nfreq,nfreq,matrix,eigen,vects,a,b,p,w,ta,tb,ty)
      factor = 4.6587d5 / (4.0d0 * pi**2)
      do i = 1, nvib
         if (eigen(i) .lt. 0) then
            eigunits(i) = 'I'
         else
            eigunits(i) = ' '
         end if
         eigen(i) = sqrt(abs(factor*eigen(i)))
      end do
c     write (iout,50)
   50 format (/' Vibrational Frequencies (cm-1) :'/)
c     write (iout,60)  (i,eigen(i),eigunits(i),i=1,nvib)
   60 format (5(i5,f9.2,a1))
c
c     form the mass weighted eigenvectors; ie, the normal modes
c
      do i = 1, nvib
         jstart = (i-1) * nfreq
         vnorm = 0.0d0
         do j = 1, nfreq
c           vects(jstart+j) = vects(jstart+j) * mass2((j+2)/3)
            vects(jstart+j) = vects(jstart+j)
            vnorm = vnorm + vects(jstart+j)**2
         end do
         vnorm = sqrt(vnorm)
         do j = 1, nfreq
            vects(jstart+j) = vects(jstart+j) / vnorm
         end do
      end do
      ivib = 1
      jstart = (ivib-1) * nfreq
      j = jstart
      do i = 1, n
         tsvec(j+1) = vects(j+1)
         tsvec(j+2) = vects(j+2)
         tsvec(j+3) = vects(j+3)
         j = j + 3
      end do
      return
      end
c
c     #######################################################
c     ##                                                   ##
c     ##  subroutine sdpout  --  write SDP coordinate file ##
c     ##                                                   ##
c     #######################################################
c
c
c     "sdpout" saves the current atomic
c      coordinates to a disk file
c
c
      subroutine sdpout (xx,ncycle)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'minima.i'
      include 'sdpcycl.i'
      include 'sdpdata.i'
      include 'pathdata.i'
      integer maxvar,ncycle,j
      parameter (maxvar=3*maxatm)
      real*8 xx(maxvar),xold(maxvar)
      real*8 xrms,xdel
      logical initiate,terminate,mqnstat
      save xold
      common /fudge/ mqnstat
c
c     write coordinates if they are significantly
c     different from those previously written
c     or if coordinates are a stationary point
c
      initiate = .false.
      terminate = .false.
      if ( mod(ncycle,iwrite) .ne. 0 ) terminate = .true.
      if (mqnstat) terminate = .true.
      if (ncycle .eq. iwrite) then
         initiate = .true.
         do j = 1, nat3
            xold(j) = p0(j)
         end do
         sdpiter = 5000.00
      end if
      if (initiate) then
      if ( name(38) .eq. 'O  ' ) npep = 12
      if ( name(32) .eq. 'O  ' ) npep = 10
      if ( name(29) .eq. 'O  ' ) npep = 9
      if ( name(26) .eq. 'O  ' ) npep = 8
      if ( name(23) .eq. 'O  ' ) npep = 7
      if ( name(20) .eq. 'O  ' ) npep = 6
      if ( name(17) .eq. 'O  ' ) npep = 5
      if ( name(14) .eq. 'O  ' ) npep = 4
      if ( npep .eq. 5 )   then
         endc = 13
         jomin = 17
         jomax = 25
         jhmin = 18
         jhmax = 26
      endif
      if ( npep .eq. 6 )   then
         endc = 16
         jomin = 20
         jomax = 30
         jhmin = 21
         jhmax = 31
      endif
      if ( npep .eq. 7 )   then
         endc = 19
         jomin = 23
         jomax = 35
         jhmin = 24
         jhmax = 36
      endif
      if ( npep .eq. 8 )   then
         endc = 22
         jomin = 26
         jomax = 40
         jhmin = 27
         jhmax = 41
      endif
      if ( npep .eq. 9 )   then
         endc = 25
         jomin = 29
         jomax = 45
         jhmin = 30
         jhmax = 46
      endif
      if ( npep .eq. 10 ) then
         endc = 28
         jomin = 32
         jomax = 50
         jhmin = 33
         jhmax = 51
      endif
      if ( npep .eq. 12 ) then
         endc = 34
         jomin = 38
         jomax = 60
         jhmin = 39
         jhmax = 61
      endif
      endif
      xrms = 0.0d0
      do j = 1, nat3
         xdel = xx(j) - xold(j)
         xrms = xrms +xdel*xdel
      end do
      xrms = sqrt(xrms/dfloat(natoms))
      if ( (xrms .gt. 0.2d0) .or. (sdpdir .eq. 0)
     :         .or. (initiate) .or. (terminate) ) then
         do j = 1, nat3
            xold(j) = xx(j)
         end do
c
c        label the SDP coordinate files consecutively
c
         if ( sdpdir .lt. 0 ) then
            sdpiter = sdpiter - 1
         else if ( sdpdir .eq. 0 ) then
            sdpiter = 5000.00
         else
            sdpiter = sdpiter + 1
         endif
         call writeout(xx,sdpiter)
         call storeinfo(xx)
         call hbnd(xx,sdpiter)
      endif
      write(6,*) ncycle,sdpiter,xrms,mqnstat
      return
      end
c
c     #######################################################
c     ##                                                   ##
c     ##  subroutine storeinfo  --  analyze SDP coordinates  ##
c     ##                                                   ##
c     #######################################################
c
      subroutine storeinfo(xx)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'pathsz.i'
      include 'pathdata.i'
      include 'sdpcycl.i'
      include 'sdpdata.i'
      integer maxvar
      parameter (maxvar=3*maxatm)
      real*8 xx(maxvar),xxrms,xdel
      real*8 nrgval,xcc,ycc,zcc,rcc
      integer j
      common /cheat/ nrgval
c
      if ( sdpiter .eq. 4999) istruct = 0
      istruct = istruct + 1
      xxrms = 0.0d0
      do j = 1, nat3
         xdel = xx(j) - p0(j)
         xxrms = xxrms +xdel*xdel
      end do
      rrms(istruct) = sqrt(xxrms/dfloat(natoms))
      xcc = x(4) - x(endc)
      ycc = y(4) - y(endc)
      zcc = z(4) - z(endc)
      rcc = sqrt(xcc*xcc + ycc*ycc + zcc*zcc)
      ree(istruct) = rcc
      nrg(istruct) = nrgval
      ndx(istruct) = sdpiter
      return
      end
c
c     ##########################################################
c     ##                                                      ##
c     ##  subroutine writeinfo  --   write reasonable output  ##
c     ##                                                      ##
c     ##########################################################
c
      subroutine writeinfo
      implicit none
      include 'sizes.i'
      include 'pathsz.i'
      include 'pathdata.i'
      include 'sdpcycl.i'
      include 'sdpdata.i'
      include 'sdpfiles.i'
      integer maxvar
      parameter (maxvar=3*maxatm)
      integer i,j,tsmark
      real*8 emax
c
c     index variable 'ndx' corresponds to sequence file indices: change
c     this such that -n --> -1 are structures bewtween R & TS; the TS
c     is labeled by '0'; and 1 --> m correspond to strucures between TS
c     and P.
c
      do i = 1, istruct
         if (ndx(i) .eq. 5000) then
            tsmark = i
            ndx(i) = 0
            do j = i + 1, istruct
               ndx(j) = j - i
            end do
            do j = 1, i - 1
               ndx(j) = -j
            end do
         end if
      end do
c
c     keep only relative energy information
c
      emax = -10.0e5
      do i = 1, istruct
         if (nrg(i) .gt. emax) emax = nrg(i)
      end do
      do i = 1, istruct
         nrg(i) = nrg(i) - emax
      end do
c
      do i = tsmark - 1, 1, -1
         write (irad,20) ndx(i),ree(i)
         write (inrg,20) ndx(i),nrg(i)
      end do
      do i = tsmark, istruct
         write (irad,20) ndx(i),ree(i)
         write (inrg,20) ndx(i),nrg(i)
      end do
  20  format (i5,3f10.3)
      return
      end
c
c     #######################################################
c     ##                                                   ##
c     ##  subroutine hbnd  --  get hbond info per residue ##
c     ##                                                   ##
c     #######################################################
c
c
c     "hbond" analyzes the hbond type
c      for each peptide residue
c
c      note:    "1" designates 3[10] type Hbond
c               "-1"  an alpha helix type Hbond
c               "0"   designates lack of either type Hbond
c               "2"   indicates a bifurcated Hbond of 3[10]/alpha type
c
      subroutine hbnd (xx,sdpiter)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'sdpdata.i'
      include 'sdpfiles.i'
      integer maxvar,mbig,sdpiter
      parameter (maxvar=3*maxatm)
      parameter(mbig = 100)
      integer hb_typ(mbig),hb_310(mbig),hb_alf(mbig)
      integer res_index,i,jc,jo,jh,jn
      integer jo1,jo2,jo3,jh1,jh2,jh3,jn1,jn2,jn3
      real*8 x2,y2,z2,roh,xnh,ynh,znh,rnh
      real*8 xoh,yoh,zoh
      real*8 xoc,yoc,zoc,roc
      real*8 angh,angc
      real*8 xx(maxvar)
      real*8 dihed,opa,roh_cut,opa_cut
      external dihed

      do i = 1, mbig
         hb_typ(i) = 0
         hb_310(i) = 0
         hb_alf(i) = 0
      end do
      roh_cut = 2.4
      opa_cut = 25.0d0
      jc = 2
      do jo = jomin, jomax, 2
        i = 0
        do jh = jhmin, jhmax, 2
         i = i+1
         res_index = i
         jn = 3*i
         jo1 = 3*(jo-1)+1
         jo2 = jo1+1
         jo3 = jo2+1
         jh1 = 3*(jh-1)+1
         jh2 = jh1+1
         jh3 = jh2+1
         jn1 = 3*(jn-1)+1
         jn2 = jn1+1
         jn3 = jn2+1
         xoh = xx(jo1) - xx(jh1)
         yoh = xx(jo2) - xx(jh2)
         zoh = xx(jo3) - xx(jh3)
         x2 = xoh*xoh
         y2 = yoh*yoh
         z2 = zoh*zoh
         roh = sqrt(x2+y2+z2)
c
         xnh = x(jn) - x(jh)
         ynh = y(jn) - y(jh)
         znh = z(jn) - z(jh)
         rnh = sqrt(xnh*xnh+ynh*ynh+znh*znh)
         angh = xnh*xoh + ynh*yoh + znh*zoh
         angh = angh/roh/rnh
         angh = acos(angh)*57.3248
c
         xoc = x(jo) - x(jc)
         yoc = y(jo) - y(jc)
         zoc = z(jo) - z(jc)
         roc = sqrt(xoc*xoc+yoc*yoc+zoc*zoc)
         angc = xoh*xoc + yoh*yoc + zoh*zoc
         angc = angc/roh/roc
         angc = acos(angc)*57.3248
         opa = dihed(jn,jn+1,jn-1,jo)
c
         if ( (roh .lt. roh_cut) .and. (abs(opa) .lt. opa_cut) ) then
           if  ( (jh-jo) .eq. 5 ) then
c
c     3[10] hbond type
c
               hb_310(res_index) = 1
           else if ( (jh-jo) .eq. 7 ) then
c
c     alpha hbond type
c
               hb_alf(res_index) = -1
           end if
         end if
        end do
        jc = jc + 3
      end do
      do i = 2,9
         if ( (hb_310(i) .ne. 0) .and. (hb_alf(i) .ne. 0) ) then
            hb_typ(i) = 2
         else if (hb_310(i) .ne. 0) then
            hb_typ(i) = hb_310(i)
         else if (hb_alf(i) .ne. 0) then
            hb_typ(i) = hb_alf(i)
         end if
      end do
      write(ihbd,10) sdpiter,ree(istruct),(hb_typ(i),i = 2,9 )
  10  format (i5,tr5,8(i3,tr2))
      write(ihbd,*)
      write(ihbd,103) 'r(O-H) cutoff used is ',roh_cut,
     &                ' opa cutoff used is ',opa_cut
  103 format (a,f5.2,5x,a,f5.2)
      return
      end
c
c
c     #############################################################
c     ##                                                         ##
c     ##  function dihed  --  dihedral angle between four atoms  ##
c     ##                                                         ##
c     #############################################################
c
c
      function dihed (ia,ib,ic,id)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'math.i'
      integer ia,ib,ic,id
      real*8 xia,yia,zia,xib,yib,zib
      real*8 xic,yic,zic,xid,yid,zid
      real*8 xba,yba,zba,xcb,ycb,zcb
      real*8 xdc,ydc,zdc
      real*8 a1,b1,c1,a2,b2,c2
      real*8 dihed,cosine,sign
c
c
c     compute the value of the dihedral angle
c
      xia = x(ia)
      yia = y(ia)
      zia = z(ia)
      xib = x(ib)
      yib = y(ib)
      zib = z(ib)
      xic = x(ic)
      yic = y(ic)
      zic = z(ic)
      xid = x(id)
      yid = y(id)
      zid = z(id)
      xba = xib - xia
      yba = yib - yia
      zba = zib - zia
      xcb = xic - xib
      ycb = yic - yib
      zcb = zic - zib
      xdc = xid - xic
      ydc = yid - yic
      zdc = zid - zic
      a1 = yba*zcb - ycb*zba
      b1 = xcb*zba - xba*zcb
      c1 = xba*ycb - xcb*yba
      a2 = ycb*zdc - ydc*zcb
      b2 = xdc*zcb - xcb*zdc
      c2 = xcb*ydc - xdc*ycb
      cosine = (a1*a2+b1*b2+c1*c2) / sqrt((a1*a1+b1*b1+c1*c1)
     &     *(a2*a2+b2*b2+c2*c2))
      cosine = min(1.0d0,max(-1.0d0,cosine))
      dihed = radian * acos(cosine)
      sign = xba*a2 + yba*b2 + zba*c2
      if (sign .lt. 0.0d0)  dihed = -dihed
      return
      end
c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ###################################################
c
c     ###########################################################
c     ##                                                       ##
c     ##  subroutine mqn  --  conjugate gradient optimization  ##
c     ##                                                       ##
c     ###########################################################
c
c     CAN REMOVE THIS LOCAL VERSION IF THERE IS SOME WAYO
C     SIGNAL END_OF_RUN BY PASSING TERM FLAG OR STATUS FLAG.
c
c     "mqn" is a gradient optimization routine which implements
c     the steepest descent, Fletcher-Reeves CG, Polak-Ribiere CG
c     and memoryless BFGS quasi-Newton methods
c
c     literature reference :
c
c     "Linear and Nonlinear Programming", 2nd Edition, 1984,
c     by David Luenberger; see in particular section 9-7
c
c     parameters used in the main iteration :
c
c     nvar     number of parameters to be minimized
c     method   steepest descent, FRCG, PRCG or MQN iteration
c     fctmin   normal exit if function gets less than
c     grdmin   normal exit if gradient norm gets less than
c     maxiter  error return if number of iterations exceeds
c     period   restart at least every period iterations
c     iprint   print iteration results every iprint iterations
c     iwrite   call user-supplied output every iwrite iterations
c     fast     steepest descent while function decrease exceeds
c     slow     restart if relative function decrease drops below
c     epsln    error if total move is less than
c     scale    factor by which actual function has been multiplied
c     rms      factor to convert grad norm and movement to rms
c     fctval   final value of the objective function
c     ncalls   number of function/gradient (fgvalue) calls
c     niter    number of conjugate gradient iterations performed
c     status   string containing informative termination message
c
c     parameters used in the line search :
c
c     cappa    reduction in projected gradient for termination
c     stpmin   minimum allowed line search step size
c     stpmax   maximum allowed line search step size
c     angmax   maximum angle between search and -grad directions
c     intmax   maximum number of interpolations in line search
c
c     vectors stored by the routine :
c
c     x        current parameter values
c     x_old    previous parameter values
c     g        current gradient vector
c     g_old    previous gradient vector
c     s        current minus previous parameter values
c     d        current minus previous gradient values
c     p        conjugate direction search vector
c
c     requried external routines :
c
c     fgvalue    function to evaluate function and gradient values
c     writeout   subroutine to write out info about current status
c
c
      subroutine locmqn (nvar,x,fctval,grdmin,fgvalue,writeout)
      implicit none
      include 'sizes.i'
      include 'files.i'
      include 'iounit.i'
      include 'linmin.i'
      include 'math.i'
      include 'minima.i'
      include 'output.i'
      include 'scale.i'
      integer maxvar
      parameter (maxvar=3*maxatm)
      integer i,ikey,nvar,ncalls,nerror,freeunit
      integer niter,initial,period,nstart,next
      real*8 grdmin,fast,slow,epsln
      real*8 f,f_old,f_new,f_move
      real*8 rms,beta,x_move,g_norm,g_rms
      real*8 fctval,fgvalue,gg,gg_old
      real*8 sg,dg,sd,dd,angle,ratio
      real*8 x(maxvar),g(maxvar)
      real*8 x_old(maxvar),g_old(maxvar)
      real*8 p(maxvar),s(maxvar),d(maxvar)
      character*3 method
      character*9 blank,status
      character*20 keyword
      character*60 keyfile
      character*80 record
      logical exist,restart,terminate
      external fgvalue,writeout
      common /fudge/ terminate
c
c
c     initialize some values to be used below
c
      if (nvar .gt. maxvar) then
         write (iout,10)
   10    format (' MQN  --  Too many Parameters,',
     &           ' Increase value of MAXVAR')
         return
      end if
      ncalls = 0
      rms = sqrt(dble(nvar))
      if (coordtype .eq. 'cartesian') then
         rms = rms / sqrt(3.0d0)
      end if
      blank = '         '
      status = blank
      method = 'mqn'
      restart = .true.
      terminate = .false.
c
c     set default parameters for the optimization
c
      if (fctmin .eq. 0.0d0)  fctmin = -1000000.0d0
      if (maxiter .eq. 0)  maxiter = 1000000
      if (iprint .eq. 0)  iprint = 1
      if (iwrite .eq. 0)  iwrite = 1
      if (scale .eq. 0.0d0)  scale = 1.0d0
      fast = 0.5d0
      slow = 0.0d0
      initial = 1
      period = max(200,nvar)
c
c     set default parameters for the line search
c
      if (cappa .eq. 0.0d0)  cappa = 0.1d0
      if (stpmin .eq. 0.0d0)  stpmin = 1.0d-20
      if (stpmax .eq. 0.0d0)  stpmax = 5.0d0
      if (angmax .eq. 0.0d0)  angmax = 88.0d0
      if (intmax .eq. 0)  intmax = 5
c
c     search each line of the keyword file for options
c
      keyfile = filename(1:leng)//'.key'
      inquire (file=keyfile,exist=exist)
      if (exist) then
         ikey = freeunit ()
         open (unit=ikey,file=keyfile,status='old')
         dowhile (.true.)
            read (ikey,20,end=40)  record
   20       format (a80)
            next = 1
            call gettext (record,keyword,next)
            call upcase (keyword)
            if (keyword(1:17) .eq. 'STEEPEST-DESCENT ') then
               method = 'sd'
            else if (keyword(1:16) .eq. 'FLETCHER-REEVES ') then
               method = 'fr'
            else if (keyword(1:14) .eq. 'POLAK-RIBIERE ') then
               method = 'pr'
            else if (keyword(1:7) .eq. 'FCTMIN ') then
               read (record(next:80),*,err=30)  fctmin
            else if (keyword(1:7) .eq. 'IWRITE ') then
               read (record(next:80),*,err=30)  iwrite
            else if (keyword(1:7) .eq. 'IPRINT ') then
               read (record(next:80),*,err=30)  iprint
            else if (keyword(1:8) .eq. 'MAXITER ') then
               read (record(next:80),*,err=30)  maxiter
            else if (keyword(1:8) .eq. 'INITIAL ') then
               read (record(next:80),*,err=30)  initial
            else if (keyword(1:7) .eq. 'STPMAX ') then
               read (record(next:80),*,err=30)  stpmax
            else if (keyword(1:7) .eq. 'STPMIN ') then
               read (record(next:80),*,err=30)  stpmin
            else if (keyword(1:6) .eq. 'CAPPA ') then
               read (record(next:80),*,err=30)  cappa
            else if (keyword(1:7) .eq. 'ANGMAX ') then
               read (record(next:80),*,err=30)  angmax
            else if (keyword(1:6) .eq. 'EPSLN ') then
               read (record(next:80),*,err=30)  epsln
            else if (keyword(1:5) .eq. 'FAST ') then
               read (record(next:80),*,err=30)  fast
            else if (keyword(1:5) .eq. 'SLOW ') then
               read (record(next:80),*,err=30)  slow
            else if (keyword(1:7) .eq. 'PERIOD ') then
               read (record(next:80),*,err=30)  period
            else if (keyword(1:7) .eq. 'INTMAX ') then
               read (record(next:80),*,err=30)  intmax
            end if
   30       continue
         end do
   40    continue
         close (unit=ikey)
      end if
c
c     get initial function and gradient values
c
      niter = initial - 1
      maxiter = niter + maxiter
      ncalls = ncalls + 1
      f = fgvalue (x,g)
      g_norm = 0.0d0
      do i = 1, nvar
         x_old(i) = x(i)
         g_old(i) = g(i)
         g_norm = g_norm + g(i)**2
      end do
      g_norm = sqrt(g_norm)
      f_move = 0.5d0 * stpmax * g_norm
      g_rms = g_norm * scale/rms
c
c     print header information prior to iterations
c
      if (iprint .gt. 0) then
         if (method .eq. 'sd') then
            write (iout,50)
   50       format (/' Steepest Descent Gradient Optimization :')
         else if (method .eq. 'fr') then
            write (iout,60)
   60       format (/' Fletcher-Reeves Conjugate Gradient',
     &                  ' Optimization :')
         else if (method .eq. 'pr') then
            write (iout,70)
   70       format (/' Polak-Ribiere Conjugate Gradient',
     &                  ' Optimization :')
         else if (method .eq. 'mqn') then
            write (iout,80)
   80       format (/' Memoryless BFGS Quasi-Newton Optimization :')
         end if
         write (iout,90)
   90    format (/' CG Iter    F Value      RMS G     F Move',
     &            '    X Move    Angle  FG Call  Comment',/)
         if (abs(f).lt.10000000.0d0 .and. g_rms.lt.100000.0d0) then
            write (iout,100)  niter,f,g_rms,ncalls
  100       format (i6,f13.4,f11.4,30x,i7)
         else
            write (iout,110)  niter,f,g_rms,ncalls
  110       format (i6,d13.4,d11.4,30x,i7)
         end if
      end if
c
c     tests of the various termination criteria
c
      if (niter .ge. maxiter) then
         status = 'IterLimit'
         terminate = .true.
      end if
      if (f .le. fctmin) then
         status = 'SmallFct '
         terminate = .true.
      end if
      if (g_rms .le. grdmin) then
         status = 'SmallGrad'
         terminate = .true.
      end if
c
c     start of a new conjugate gradient iteration
c
      dowhile (.not. terminate)
         niter = niter + 1
         if (status .eq. blank)  nerror = 0
c
c     compute the next search direction using Steepest Descent,
c     Fletcher-Reeves CG, Polak-Ribiere CG or Memoryless BFGS
c
  120    continue
         status = blank
         if (method.eq.'sd' .or. restart) then
            do i = 1, nvar
               p(i) = -g(i)
            end do
            nstart = niter
            restart = .false.
         else if (method .eq. 'fr') then
            gg = 0.0d0
            gg_old = 0.0d0
            do i = 1, nvar
               gg = gg + g(i)*g(i)
               gg_old = gg_old + g_old(i)*g_old(i)
            end do
            beta = gg / gg_old
            do i = 1, nvar
               p(i) = -g(i) + beta*p(i)
            end do
         else if (method .eq. 'pr') then
            dg = 0.0d0
            gg_old = 0.0d0
            do i = 1, nvar
               dg = dg + d(i)*g(i)
               gg_old = gg_old + g_old(i)*g_old(i)
            end do
            beta = dg / gg_old
            do i = 1, nvar
               p(i) = -g(i) + beta*p(i)
            end do
         else if (method .eq. 'mqn') then
            sg = 0.0d0
            dg = 0.0d0
            dd = 0.0d0
            sd = 0.0d0
            do i = 1, nvar
               sg = sg + s(i)*g(i)
               dg = dg + d(i)*g(i)
               dd = dd + d(i)*d(i)
               sd = sd + s(i)*d(i)
            end do
            do i = 1, nvar
               p(i) = -g(i) + (d(i)*sg+s(i)*dg)/sd
     &                   - (1.0d0+dd/sd)*(s(i)*sg/sd)
            end do
         end if
c
c     perform line search along the new conjugate direction
c
         f_old = f
         call search (nvar,f,g,x,p,f_move,angle,ncalls,fgvalue,status)
         f_new = f
c
c     if angle between the search direction and the negative
c     gradient vector was too large, use steepest descent
c
         if (status .eq. 'WideAngle') then
            restart = .true.
            goto 120
         end if
c
c     update variables based on results of this iteration
c
         f_move = f_old - f_new
         x_move = 0.0d0
         g_norm = 0.0d0
         do i = 1, nvar
            s(i) = x(i) - x_old(i)
            d(i) = g(i) - g_old(i)
            x_move = x_move + s(i)**2
            g_norm = g_norm + g(i)**2
            x_old(i) = x(i)
            g_old(i) = g(i)
         end do
         x_move = sqrt(x_move) / (scale * rms)
         if (coordtype .eq. 'internal') then
            x_move = radian * x_move
         end if
         g_norm = sqrt(g_norm)
         g_rms = g_norm * scale/rms
c
c     test for error/restart due to line search problems
c
         if (status .eq. 'BadIntpln') then
            nerror = nerror + 1
            if (nerror .eq. 3) then
               terminate = .true.
            else
               restart = .true.
            end if
         end if
         if (status .eq. 'IntplnErr') then
            nerror = nerror + 1
            if (nerror .eq. 3) then
               terminate = .true.
            else
               restart = .true.
            end if
            do i = 1, nvar
               x(i) = x_old(i)
               g(i) = g_old(i)
            end do
         end if
c
c     test for error/restart due to lack of movement
c
         if (x_move .lt. epsln) then
            status = 'SmallMove'
            nerror = nerror + 1
            if (nerror .eq. 3) then
               terminate = .true.
            else
               restart = .true.
            end if
         end if
c
c     test for error/restart due to function increase
c
         if (f_move .lt. 0.0d0) then
            status = 'Increase '
            nerror = nerror + 1
            if (nerror .eq. 3) then
               terminate = .true.
            else
               restart = .true.
            end if
            do i = 1, nvar
               x(i) = x_old(i)
               g(i) = g_old(i)
            end do
         end if
c
c     test for error/restart due to slow progress
c
         ratio = f_move / abs(f_new-fctmin)
         if (abs(ratio) .lt. slow) then
            status = 'SlowDecr '
            nerror = nerror + 1
            if (nerror .eq. 3) then
               terminate = .true.
            else
               restart = .true.
            end if
         end if
c
c     test for restart due to fast progress
c
         if (ratio .gt. fast) then
            status = 'FastDecr '
            nerror = 0
            restart = .true.
         end if
c
c     test for restart based on iterations since last restart
c
         if (niter-nstart .ge. period) then
            status = 'Periodic '
            restart = .true.
         end if
c
c     test for too many total iterations
c
         if (niter .ge. maxiter) then
            status = 'IterLimit'
            terminate = .true.
         end if
c
c     test the normal termination criteria
c
         if (f .le. fctmin) then
            status = 'SmallFct '
            terminate = .true.
         end if
         if (g_rms .le. grdmin) then
            status = 'SmallGrad'
            terminate = .true.
         end if
c
c     print intermediate results every few iterations
c
         if (iprint .gt. 0) then
            if (terminate .or. mod(niter,iprint).eq.0) then
               if (abs(f).lt.10000000.0d0 .and. g_rms.lt.100000.0d0
     &                  .and. f_move.lt.100000.0d0) then
                  write (iout,130)  niter,f,g_rms,f_move,
     &                              x_move,angle,ncalls,status
  130             format (i6,f13.4,f11.4,f11.4,f10.4,f9.2,i7,3x,a9)
               else
                  write (iout,140)  niter,f,g_rms,f_move,
     &                              x_move,angle,ncalls,status
  140             format (i6,d13.4,d11.4,d11.4,f10.4,f9.2,i7,3x,a9)
               end if
            end if
         end if
c
c     write intermediate results every few iterations
c
         if (iwrite .gt. 0) then
            if (terminate .or. mod(niter,iwrite).eq.0) then
               call writeout (x,niter)
            end if
         end if
      end do
c
c     set final value of the objective function
c
      if (iprint .gt. 0) then
         if (status.eq.'SmallGrad' .or. status.eq.'SmallFct ') then
            write (iout,150)  status
  150       format (/' MQN  --  Normal Termination due to ',a9)
         else
            write (iout,160)  status
  160       format (/' MQN  --  Error Return due to ',a9)
         end if
      end if
      fctval = f
      return
      end
