c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1999  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine sgstep  --  self-guided molecular dynamics  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "sgstep" performs a single molecular dynamics time step
c     for the self-guided trajectory method of Wu and Wang
c
c
      subroutine sgstep (istep,dt,dt_2,dt2_2,ndump)
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'moldyn.i'
      include 'shake.i'
      include 'units.i'
      include 'usage.i'
      integer i,j,istep,ndump
      real*8 dt,dt_2,dt2_2,e_tot,e_kin,e_pot
      real*8 temp,pres,derivs(3,maxatm)
      real*8 x_old(maxatm),y_old(maxatm),z_old(maxatm)
c
      include 'deriv.i'
      real*8 tau0,tau1,lambda
      real*8 force(3,maxatm)
      real*8 drag(3,maxatm)
      real*8 free(3,maxatm)
c
c
c     store the current atom positions, then find new atom
c     positions and half-step velocities via Verlet recursion
c
      tau0 = 0.999d0
      tau1 = 1.0d0 - tau0
      lambda = 0.1d0
c
      do i = 1, n
         if (use(i)) then
            x_old(i) = x(i)
            y_old(i) = y(i)
            z_old(i) = z(i)
            x(i) = x(i) + v(1,i)*dt + a(1,i)*dt2_2
            y(i) = y(i) + v(2,i)*dt + a(2,i)*dt2_2
            z(i) = z(i) + v(3,i)*dt + a(3,i)*dt2_2
            do j = 1, 3
               v(j,i) = v(j,i) + a(j,i)*dt_2
            end do
         end if
      end do
c
c     apply "rattle" to correct atom positions and velocities
c
      if (use_rattle)  call rattle (dt,x_old,y_old,z_old)
c
c     get the potential energy and atomic forces
c
      call gradient (e_pot,derivs)
c
c     use Newton's second law to get the next accelerations;
c     find the full-step velocities using the Verlet recursion
c
      do i = 1, n
         if (use(i)) then
            do j = 1, 3
               force(j,i) = -convert * derivs(j,i)
               drag(j,i) = -convert * (dev(j,i)+de14(j,i)+dec(j,i))
               free(j,i) = tau0*free(j,i)
     &                         + tau1*(force(j,i)+lambda*drag(j,i))
               a(j,i) = (force(j,i) + lambda*free(j,i)) / mass(i)
               v(j,i) = v(j,i) + a(j,i)*dt_2
            end do
         end if
      end do
c
c     use "rattle" method to get corrected full-step velocities
c
      if (use_rattle)  call rattle2 (dt)
c
c     sum the total kinetic energy over all atoms
c
      e_kin = 0.0d0
      do i = 1, n
         if (use(i)) then
            do j = 1, 3
               e_kin = e_kin + mass(i)*v(j,i)**2
            end do
         end if
      end do
      e_kin = 0.5d0 * e_kin / convert
c
c     determine system temperature and total energy
c
      temp = 2.0d0 * e_kin / (dble(3*nuse-nrat-6) * gasconst)
      e_tot = e_kin + e_pot
c
c     control temperature and pressure via external bath coupling
c
      if (isothermal)  call temper (dt,temp)
      if (isobaric)  call pressure (dt,pres,e_kin)
c
c     compute any averages or statistics for this step
c
      call mdstat (istep,dt,e_tot,e_pot,e_kin,temp,pres,ndump)
      return
      end
