c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine esolv6  --  atom-by-atom solvation Hessian  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "esolv6" calculates second derivatives of the smooth 
c     solvation energy using the Macromodel GB/SA solvation 
c     model
c
c
      subroutine esolv6 (i)
      implicit none
      integer i
c
c     get the second derivatives of the cavity term
      call ddarea(i)
c     get the generalized Born term for GB/SA solvation
c     
      call egborn6 (i)
      return
      end
c
c
c     #################################################################
c     ##                                                             ##
c     ##  subroutine egborn6  --  atom-wise generalized Born Hessian ##
c     ##                                                             ##
c     #################################################################
c
c
c     "egborn6" calculates second derivatives of the smooth generalized
c     Born energy term for the Macromodel GB/SA solvation model
c
c
      subroutine egborn6 (i)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'charge.i'
      include 'hessn.i'
      include 'math.i'
      include 'shunt.i'
      include 'solute.i'
      include 'units.i'
      include 'warp.i'
      integer i,j,k,kk
      real*8 fi,fik,de,d2e,rb,rb2,pidef
      real*8 xi,yi,zi,xr,yr,zr,term(3,3)
      real*8 expterm,expterm2
      real*8 s1,s2,s3,s4,s5,s6,asqpt,erfterm
      real*8 def1,def3,def4,def5,def7,erf
      real*8 dgb1,dgb2,dgb11,dgb12
      real*8 fgb1,fgb2,irb,irb2,irb4
      real*8 arg1,arg2,arg3,arg4
      real*8 d2gb11,d2gb12,d2gb21,d2gb22,d2gb3
      real*8 drdx,drdy,drdz
      real*8 d2rdx2,d2rdy2,d2rdz2
      real*8 d2rdxy,d2rdxz,d2rdyz
      real*8 dwater,r,r2,r3,rinv
c
c
	
c     first see if the atom of interest carries a charge
c
      do k = 1, nion
        if (iion(k) .eq. i) then
          dwater = 78.3d0
          fi = -electric * (1.0d0 - 1.0d0/dwater) * pchg(k)
          goto 10
        end if
      end do
      return
   10 continue
c     
c     store the coordinates of the atom of interest
c     
      xi = x(i)
      yi = y(i)
      zi = z(i)
c     
c     set cutoff distances and switching function coefficients
c     
      call switch ('CHARGE')
c     
c     calculate the smooth GB/SA polarization energy Hessian 
c     elements
c     
      do kk = 1, nion
        k = iion(kk)
        if (i .ne. k) then
          xr = xi - x(k)
          yr = yi - y(k)
          zr = zi - z(k)
          r2 = xr*xr + yr*yr + zr*zr
          r = sqrt(r2)
          rinv = 1.0d0/r
          r3 = rinv**3
          fik = fi * pchg(kk)
          rb2 = rborn(i) * rborn(k)
          rb = sqrt(rb2)
c         
c         compute chain rule terms for Hessian matrix elements
c         
          if (deform .ne. 0.0d0) then
            s1 = sqrt(r2 + 1.25d0*rb2)
            s2 = s1*s1
            s3 = s2*s1
            s4 = s2**2
            s5 = s1**5
            s6 = sqra*rb
            pidef = sqrt(pi*deform)
            arg1 = s1 + s6
            arg2 = s1 - s6
            arg3 = (arg1*arg1)
            arg4 = (arg2*arg2) 
            asqpt = rb2 + deform
            def1 = 0.5d0/sqrt(deform)
            def3 = (1.0d0/sqrt(asqpt))**3
            def4 = def1*def1
            def5 = (1.0d0/sqrt(asqpt))**5
            def7 = (1.0d0/sqrt(asqpt))**7
            erfterm = erf(arg1*def1) + erf(arg2*def1)
		  expterm = exp(-0.25d0*r2/asqpt)
		  expterm2 = exp(-arg3*def4) + exp(-arg4*def4)
            dgb11 = -0.5d0*r*erfterm/s3
            dgb12 = 0.5d0*r*expterm2 /(s2*pidef) 
            dgb1 = dgb11 + dgb12
            dgb2 = -0.5d0*screen*r*rb2*expterm*def5
            de = fik * (dgb1 + dgb2)
c           
            d2gb11 = (1.5d0*r2/s5 - 0.5d0/s3)*erfterm
            d2gb12 = -r2*expterm2/(s4*pidef)
            d2gb21 = 0.5d0*(1.0d0/s2 - r2/s4)*expterm2/pidef
            d2gb22 = -(0.25d0*r2/(pidef*deform*s3))*
     &                (arg1*exp(-arg3*def4)+arg2*exp(-arg4*def4))
            d2gb3 = screen*rb2*(0.25d0*r2*def7-0.5d0*def5)*expterm
            d2e = fik* (d2gb11 + d2gb12 + d2gb21 + d2gb22 + d2gb3)
          else
            irb = screen/rb
            irb2 = 0.5d0/rb2
            irb4 = irb2*irb2
            fgb1 = 1.0d0/sqrt(r2 + 1.25d0*rb2)
	      expterm = exp(-0.25d0*r2/rb2)
	      fgb2 = irb*expterm
            dgb1 = -fgb1**3
            dgb2 = -irb2 * fgb2
            de = fik * r*(dgb1 + dgb2)
            d2gb11 = 3.0d0*r2*fgb1**5 + dgb1
            d2gb21 = irb*(r2*irb4 - irb2)*expterm
            d2e = fik*(d2gb11 + d2gb21)
          end if
c         
c         now, form the individual Hessian element components
c         
          drdx = xr*rinv
          drdy = yr*rinv
          drdz = zr*rinv
          d2rdx2 = rinv - xr*xr*r3
          d2rdy2 = rinv - yr*yr*r3
          d2rdz2 = rinv - zr*zr*r3
          d2rdxy = -xr*yr*r3
          d2rdxz = -xr*zr*r3
          d2rdyz = -yr*zr*r3
          
          term(1,1) = d2e*drdx*drdx + de*d2rdx2
          term(1,2) = d2e*drdx*drdy + de*d2rdxy
          term(1,3) = d2e*drdx*drdz + de*d2rdxz
          term(2,1) = term(1,2)
          term(2,2) = d2e*drdy*drdy + de*d2rdy2
          term(2,3) = d2e*drdy*drdz + de*d2rdyz
          term(3,1) = term(1,3)
          term(3,2) = term(2,3)
          term(3,3) = d2e*drdz*drdz + de*d2rdz2
c         
c         increment diagonal and non-diagonal Hessian elements
c         
          do j = 1, 3
            hessx(j,i) = hessx(j,i) + term(1,j)
            hessy(j,i) = hessy(j,i) + term(2,j)
            hessz(j,i) = hessz(j,i) + term(3,j)
            hessx(j,k) = hessx(j,k) - term(1,j)
            hessy(j,k) = hessy(j,k) - term(2,j)
            hessz(j,k) = hessz(j,k) - term(3,j)
          end do
        end if
      end do
c     
      return
      end

c     ################################################################
c     ##                                                            ##
c     ##  subroutine ddarea  --  second derivatives of surface area ##
c     ##                                                            ##
c     ################################################################
c
c
c     "ddarea" calculates numerical second derivatives of the surface
c     area term in order to construct the hessian for the cavity term  
c     for the Macromodel GB/SA solvation model
c
      subroutine ddarea (i)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'hessn.i'
      include 'potent.i'
      include 'solute.i'
      include 'usage.i'
      integer i,j,k
      real*8 probe,earea,aarea(maxatm)
      real*8 gar(3,maxatm),gar0(3,maxatm)
      real*8 old,eps
      
c     get the two-sided numerical Hessian matrix elements
c     
      probe = 1.4d0
      eps = 1.0d-6
      if (use(i)) then
        old = x(i)
        x(i) = x(i) - 0.5d0*eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        do k = 1, n
          do j = 1, 3
            gar0(j,k) = gar(j,k)
          end do
        end do
        x(i) = x(i) + eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        x(i) = old
        do k = 1, n
          do j = 1, 3
            hessx(j,i) = hessx(j,i) - ((gar(j,k)-gar0(j,k))/eps)
            hessx(j,k) = hessx(j,k) + ((gar(j,k)-gar0(j,k))/eps)
          end do
        end do
        old = y(i)
        y(i) = y(i) - 0.5d0*eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        do k = 1, n
          do j = 1, 3
            gar0(j,k) = gar(j,k)
          end do
        end do
        y(i) = y(i) + eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        y(i) = old
        do k = 1, n
          do j = 1, 3
            hessy(j,i) = hessy(j,i) - ((gar(j,k)-gar0(j,k))/eps)
	    hessy(j,k) = hessy(j,k) + ((gar(j,k)-gar0(j,k))/eps)
          end do
        end do
        old = z(i)
        z(i) = z(i) - 0.5d0*eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        do k = 1, n
          do j = 1, 3
            gar0(j,k) = gar(j,k)
          end do
        end do
        z(i) = z(i) + eps
        call surface (earea,aarea,gar,rsolv,vsolv,probe)
        z(i) = old
        do k = 1, n
          do j = 1, 3
            hessz(j,i) = hessz(j,i) - ((gar(j,k)-gar0(j,k))/eps)
            hessz(j,k) = hessz(j,k) + ((gar(j,k)-gar0(j,k))/eps)
          end do
        end do
      end if
      
      return 
      end
