c
c
c     #################################################################
c     ##  COPYRIGHT (C)  1995  by  Yong Kong and Jay William Ponder  ##
c     ##                     All Rights Reserved                     ##
c     #################################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  program diffuse  --  find water self-diffusion constant  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "diffuse" reads a water box MD trajectory, unfolds the boundary
c     condition to get the "real" coordinates of the center of mass
c     of each molecule and calculates the self-diffusion constant
c
c
      program diffuse
      implicit none
      integer maxout,maxsiz,maxnum
      parameter (maxout=2000)
      parameter (maxsiz=216)
      parameter (maxnum=maxout*maxsiz)
      real*8 box,box2,box4
      real*8 mass_h,mass_o mass_water
      real*8 xo,yo,zo
      real*8 xh1,yh1,zh1
      real*8 xh2,yh2,zh2
      real*8 dx,dy,dz
      real*8 xcen(maxnum),ycen(maxnum),zcen(maxnum)
      real*8 msdx(maxout),msdy(maxout),msdz(maxout),count(maxout)
      real*8 x(maxsiz),y(maxsiz),z(maxsiz)
      real*8 x1(maxsiz),y1(maxsiz),z1(maxsiz)
      real*8 x_pre_inbox(maxsiz),y_pre_inbox(maxsiz),z_pre_inbox(maxsiz)
      real*8 x_now_inbox,y_now_inbox,z_now_inbox
      real*8 normx,normy,normz,normr,counter,rnum,rnum_tau
      real*8 savefile
      integer n,m,run,iarc,iout,next,start
      integer i,ii,j,jj,tau
      integer count_ii,count_jj
      integer num,sites
      integer freeunit,trimtext,leng
      character*120 record,string
      character*80 filename,filenamebase
      character*3 atom
c
c
c     cycle over the user specified coordinate files
c
      write (*,*)  "Arc file:"
      read (*,10)  record
      next = 1
      call gettext (record,filenamebase,next)
      leng = trimtext (filenamebase)
      filename = filenamebase(1:leng)//'.arc'
      write (*,*)  "Number of Sites:"
      read (*,10)  record
      read (record,*)  sites
      write (*,*)  "INPUT BOX:"
      read (*,10)  record
      read (record,*)  box
      write (*,*)  "Start Point:"
      read (*,10)  record
      read (record,*)  start
      write (*,*)  "Time Interval for Each File (in picoseconds):"
      read (*,10)  record
      read (record,*)  savefile
c
      run = 1
      box2 = box*0.5d0
c     box4 = box*0.25d0
      box4 = box*0.05d0
      mass_h = 1.008d0
      mass_o = 15.995d0
      mass_water = 2.0d0*mass_h + mass_o
      mass_h = mass_h/mass_water
      mass_o = mass_o/mass_water
c
      iarc = freeunit ()
      open (unit=iarc,file=filename,status='unknown')
      read (iarc,10)  record
  10  format (a120)
      read (record,*)  n
c
      ii = 1
      m = n/sites  
      write (*,*)  n,m
c
      do i = 1, m
         next = 1
         read (iarc,10) string
         read (string,*)  num
         call getword (string,atom,next)
         record = string(next:120)
         read (record,*) xo, yo, zo
         next = 1
         read (iarc,10) string
         read (string,*)  num
         call getword (string,atom,next)
         record = string(next:120)
         read (record,*) xh1, yh1, zh1
         next = 1
         read (iarc,10) string
         read (string,*)  num
         call getword (string,atom,next)
         record = string(next:120)
         read (record,*) xh2, yh2, zh2
         xcen(i) = xo*mass_o + (xh1+xh2)*mass_h      
         ycen(i) = yo*mass_o + (yh1+yh2)*mass_h      
         zcen(i) = zo*mass_o + (zh1+zh2)*mass_h
         x_pre_inbox(i) = xcen(i)
         y_pre_inbox(i) = ycen(i)
         z_pre_inbox(i) = zcen(i)
         ii = ii + 1      
         if (sites .eq. 4)  read (iarc,10)  string
      end do
c
      dowhile (.true.)
         read (iarc,10,err=50,end=50)  string
         do i = 1, m
            next = 1
            read (iarc,10)  string
            read (string,*)  num
            call getword (string,atom,next)
            record = string(next:120)
            read (record,*)  xo,yo,zo
c
            next = 1
            read (iarc,10)  string
            read (string,*)  num
            call getword (string,atom,next)
            record = string(next:120)
            read (record,*)  xh1,yh1,zh1
c
            next = 1
            read (iarc,10)  string
            read (string,*)  num
            call getword (string,atom,next)
            record = string(next:120)
            read (record,*)  xh2,yh2,zh2
c
            x_now_inbox = xo*mass_o + (xh1+xh2)*mass_h      
            y_now_inbox = yo*mass_o + (yh1+yh2)*mass_h      
            z_now_inbox = zo*mass_o + (zh1+zh2)*mass_h
            dx = x_now_inbox - x_pre_inbox(i)
            dy = y_now_inbox - y_pre_inbox(i)
            dz = z_now_inbox - z_pre_inbox(i)
c
c           if ((abs(dx) .gt. box4 .and. abs(dx) .lt. box2) .or.
c     &         (abs(dy) .gt. box4 .and. abs(dy) .lt. box2) .or.
c     &         (abs(dz) .gt. box4 .and. abs(dz) .lt. box2)) then
c              write (*,20)  run,ii,dx,dy,dz,xcen(ii),ycen(ii),
c     &                      zcen(ii),xcen(ii-m),ycen(ii-m),zcen(ii-m)
c   20         format ('B',2i7,3f10.6,5x,3f10.6,5x,3f10.6)
c           end if
c
            if(dx .gt. box2) then
               dx = dx - box
            else if(dx .lt. -box2) then
               dx = dx + box
            end if
            if(dy .gt. box2) then
               dy = dy - box
            else if(dy .lt. -box2) then
               dy = dy + box
            end if 
            if(dz .gt. box2) then
               dz = dz - box
            else if(dz .lt. -box2) then
               dz = dz + box
            end if
c
c           if ((abs(dx) .gt. box4 .and. abs(dx) .lt. box2) .or.
c     &         (abs(dy) .gt. box4 .and. abs(dy) .lt. box2) .or.
c     &         (abs(dz) .gt. box4 .and. abs(dz) .lt. box2)) then
c              write (*,30)  run,ii,dx,dy,dz,xcen(ii),ycen(ii),
c     &                      zcen(ii),xcen(ii-m),ycen(ii-m),zcen(ii-m)
c   30         format ('A',2i7,3f10.6,5x,3f10.6,5x,3f10.6)
c           end if
c
            xcen(ii) = xcen(ii-m) + dx  
            ycen(ii) = ycen(ii-m) + dy  
            zcen(ii) = zcen(ii-m) + dz  
c
c           write (*,40)  run,i, x_now_inbox,y_now_inbox,z_now_inbox,
c     &                   xcen(ii),ycen(ii),zcen(ii)
c   40      format(2i7,2x,3f10.5,2x,3f10.5)
c
            x_pre_inbox(i) = x_now_inbox
            y_pre_inbox(i) = y_now_inbox
            z_pre_inbox(i) = z_now_inbox
c
            ii = ii + 1      
            if (sites .eq. 4)  read (iarc,10)  string
         end do
         run = run + 1
      end do
   50 continue
      close (unit=iarc)
c
c     do i = 1, ii-1
c        write (*,60)  i,xcen(i),ycen(i),zcen(i)
c  60    format (i5,3f10.5)
c     end do
c
      write (*,*)  run,ii
c
      run = run - 1
      count_ii = (start-1)*m + 1
      do i = 1, run
         msdx(i) = 0.0d0
         msdy(i) = 0.0d0
         msdz(i) = 0.0d0
         count(i) = 0.0d0
      end do
c
      do ii = start, run-1
         do i = 1, m
            x1(i) = xcen(count_ii)
            y1(i) = ycen(count_ii)
            z1(i) = zcen(count_ii)
            count_ii = count_ii + 1
         end do
         count_jj = count_ii
         do jj = ii+1, run
            do i = 1, m
               x(i) = xcen(count_jj)
               y(i) = ycen(count_jj)
               z(i) = zcen(count_jj)
               count_jj = count_jj + 1
            end do
            tau = jj - ii
            do j = 1, m
               dx = x(j) - x1(j)
               dy = y(j) - y1(j)
               dz = z(j) - z1(j)
c              if (j .eq. 17)  write (*,*)  ii,jj,x1(j),x(j)
               msdx(tau) = msdx(tau) + dx*dx
               msdy(tau) = msdy(tau) + dy*dy
               msdz(tau) = msdz(tau) + dz*dz
            end do
            count(tau) = count(tau) + 1.0d0
         end do
      end do
c
      iout = freeunit ()
      filename = filenamebase(1:leng)//'.out'
      open (unit=iout,file=filename,status='unknown')
      rnum = 5.0d0 / m / savefile
      do i = 1, run-start-1
         rnum_tau = rnum/dble(i)
         counter = 1.0d0/count(i)
         normx = msdx(i)*counter
         normy = msdy(i)*counter
         normz = msdz(i)*counter
         normr = (normx + normy + normz) / 3.0d0
         write (iout,70)  i,count(i),normx,normy,normz,normr,
     &                    normx*rnum_tau,normy*rnum_tau,
     &                    normz*rnum_tau,normr*rnum_tau
   70    format (i4,9f14.6)
      end do
      close (iout)
c
c     perform any final tasks before program exit
c
      call final
      end
