c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine echarge2  --  atom-wise charge-charge Hessian  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "echarge2" calculates second derivatives of the
c     charge-charge interaction energy for a single atom
c
c
      subroutine echarge2 (i)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'charge.i'
      include 'couple.i'
      include 'electr.i'
      include 'hessn.i'
      include 'latice.i'
      include 'shunt.i'
      integer i,j,k,kk,ic,kc,jcell,skip(maxatm)
      real*8 fi,fik,rik,rik2,term(3,3)
      real*8 e,de,d2e,d2edx,d2edy,d2edz
      real*8 dc,d2c
      real*8 xi,yi,zi,xr,yr,zr
      real*8 xic,yic,zic,xc,yc,zc
      real*8 rc,rc2,rc3,rc4,rc5
      real*8 taper,dtaper,d2taper
c
c
c     first see if the atom of interest carries a charge
c
      do k = 1, nion
         if (iion(k) .eq. i) then
            ic = kion(k)
            fi = 332.05382d0 * pchg(k) / dielec
            goto 10
         end if
      end do
      return
   10 continue
c
c     set the coefficients of the tapering function
c
      call smooth ('CHARGE')
c
c     store the coordinates of atom and tapering center
c
      xi = x(i)
      yi = y(i)
      zi = z(i)
      xic = x(ic)
      yic = y(ic)
      zic = z(ic)
c
c     set the list of atoms to be skipped
c
      do k = 1, n
         skip(k) = 0
      end do
      skip(i) = i
      do k = 1, n12(i)
         skip(i12(k,i)) = i
      end do
      do k = 1, n13(i)
         skip(i13(k,i)) = i
      end do
      do k = 1, n14(i)
         skip(i14(k,i)) = -i
      end do
c
c     calculate charge interaction energy Hessian; excluding
c     atoms bound to each other or to a common atom
c
      do kk = 1, nion
         k = iion(kk)
         if (skip(k) .ne. i) then
            xr = xi - x(k)
            yr = yi - y(k)
            zr = zi - z(k)
            xc = xic - x(kc)
            yc = yic - y(kc)
            zc = zic - z(kc)
            if (use_images)  call images2 (xc,yc,zc,xr,yr,zr)
            rc2 = xc*xc + yc*yc + zc*zc
            if (rc2 .le. off2) then
               rik2 = xr*xr + yr*yr + zr*zr
               rik = sqrt(rik2)
               fik = fi * pchg(kk)
               if (skip(k) .eq. -i)  fik = fik / chgscale
c
c     compute chain rule terms for Hessian matrix elements
c
               de = -fik / rik2
               d2e = -2.0d0 * de/rik
c
c     fifth order multiplicative tapering if near cutoff
c
               if (rc2 .gt. cut2) then
                  rc = sqrt(rc2)
                  rc3 = rc2 * rc
                  rc4 = rc2 * rc2
                  rc5 = rc2 * rc3
                  taper = c5*rc5 + c4*rc4 + c3*rc3
     &                       + c2*rc2 + c1*rc + c0
                  dtaper = 5.0d0*c5*rc4 + 4.0d0*c4*rc3
     &                        + 3.0d0*c3*rc2 + 2.0d0*c2*rc + c1
                  d2taper = 20.0d0*c5*rc3 + 12.0d0*c4*rc2
     &                         + 6.0d0*c3*rc + 2.0d0*c2
                  d2c = ............
                  dc = e * dtaper / rc
                  e = fik / rik
                  d2e = e*d2taper + 2.0d0*de*dtaper + d2e*taper
                  de = de * taper
               end if
c
c     now, form the individual Hessian element components
c
               de = de / rik
               d2e = (d2e-de) / rik2
               d2edx = d2e * xr
               d2edy = d2e * yr
               d2edz = d2e * zr
               term(1,1) = d2edx*xr + de
               term(1,2) = d2edx*yr
               term(1,3) = d2edx*zr
               term(2,1) = term(1,2)
               term(2,2) = d2edy*yr + de
               term(2,3) = d2edy*zr
               term(3,1) = term(1,3)
               term(3,2) = term(2,3)
               term(3,3) = d2edz*zr + de
c
c     increment diagonal and non-diagonal Hessian elements
c
               do j = 1, 3
                  hessx(j,i) = hessx(j,i) + term(1,j)
                  hessy(j,i) = hessy(j,i) + term(2,j)
                  hessz(j,i) = hessz(j,i) + term(3,j)
                  hessx(j,ic) = hessx(j,ic) + term(1,j)
                  hessy(j,ic) = hessy(j,ic) + term(2,j)
                  hessz(j,ic) = hessz(j,ic) + term(3,j)
                  hessx(j,k) = hessx(j,k) - term(1,j)
                  hessy(j,k) = hessy(j,k) - term(2,j)
                  hessz(j,k) = hessz(j,k) - term(3,j)
                  hessx(j,kc) = hessx(j,kc) - term(1,j)
                  hessy(j,kc) = hessy(j,kc) - term(2,j)
                  hessz(j,kc) = hessz(j,kc) - term(3,j)
               end do
            end if
         end if
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replicate)  return
c
c     calculate interaction energy with other unit cells
c
      do kk = 1, nion
         k = iion(kk)
         kc = kion(kk)
         do jcell = 1, ncell
            xc = xic - x(kc)
            yc = yic - y(kc)
            zc = zic - z(kc)
            call cells (xc,yc,zc,jcell)
            rc2 = xc*xc + yc*yc + zc*zc
            if (rc2 .le. off2) then
               xr = xi - x(k)
               yr = yi - y(k)
               zr = zi - z(k)
               call crystal (xr,yr,zr,jcell)
               rik2 = xr*xr + yr*yr + zr*zr
               rik = sqrt(rik2)
               fik = fi * pchg(kk)
c
c     compute chain rule terms for Hessian matrix elements
c
               de = -fik / rik2
               d2e = -2.0d0 * de/rik
c
c     fifth order multiplicative tapering if near cutoff
c
               if (rc2 .gt. cut2) then
                  rc = sqrt(rc2)
                  rc3 = rc2 * rc
                  rc4 = rc2 * rc2
                  rc5 = rc2 * rc3
                  taper = c5*rc5 + c4*rc4 + c3*rc3
     &                       + c2*rc2 + c1*rc + c0
                  dtaper = 5.0d0*c5*rc4 + 4.0d0*c4*rc3
     &                        + 3.0d0*c3*rc2 + 2.0d0*c2*rc + c1
                  d2taper = 20.0d0*c5*rc3 + 12.0d0*c4*rc2
     &                         + 6.0d0*c3*rc + 2.0d0*c2
                  e = fik / rik
                  d2e = e*d2taper + 2.0d0*de*dtaper + d2e*taper
                  de = e*dtaper + de*taper
               end if
c
c     now, form the individual Hessian element components
c
               de = de / rik
               d2e = (d2e-de) / rik2
               d2edx = d2e * xr
               d2edy = d2e * yr
               d2edz = d2e * zr
               term(1,1) = d2edx*xr + de
               term(1,2) = d2edx*yr
               term(1,3) = d2edx*zr
               term(2,1) = term(1,2)
               term(2,2) = d2edy*yr + de
               term(2,3) = d2edy*zr
               term(3,1) = term(1,3)
               term(3,2) = term(2,3)
               term(3,3) = d2edz*zr + de
c
c     increment diagonal and non-diagonal Hessian elements
c
               do j = 1, 3
                  hessx(j,i) = hessx(j,i) + term(1,j)
                  hessy(j,i) = hessy(j,i) + term(2,j)
                  hessz(j,i) = hessz(j,i) + term(3,j)
                  hessx(j,ic) = hessx(j,ic) + term(1,j)
                  hessy(j,ic) = hessy(j,ic) + term(2,j)
                  hessz(j,ic) = hessz(j,ic) + term(3,j)
                  hessx(j,k) = hessx(j,k) - term(1,j)
                  hessy(j,k) = hessy(j,k) - term(2,j)
                  hessz(j,k) = hessz(j,k) - term(3,j)
                  hessx(j,kc) = hessx(j,kc) - term(1,j)
                  hessy(j,kc) = hessy(j,kc) - term(2,j)
                  hessz(j,kc) = hessz(j,kc) - term(3,j)
               end do
            end if
         end do
      end do
      return
      end
