c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1994  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #########################################################
c     ##                                                     ##
c     ##  subroutine empole  --  multipole potential energy  ##
c     ##                                                     ##
c     #########################################################
c
c
c     "empole" calculates the multipole electrostatic energy
c
c
      subroutine empole
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'electr.i'
      include 'energi.i'
      include 'mpole.i'
      include 'shunt.i'
      include 'units.i'
      include 'usage.i'
      integer i,j,k,m,skip(maxatm)
      integer ii,iz,ix,kk,kz,kx
      real*8 e,emdqik,a(3,3)
      real*8 shift,taper,trans
      real*8 f,fi,fik,xr,yr,zr
      real*8 r,r2,r3,r4,r5,r6,r7
      real*8 rpi(13),rpk(13)
      logical iuse,kuse
c
c
c     zero out the atomic multipole interaction energy
c
      em = 0.0d0
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set the switching function coefficients
c
      f = electric / dielec
      call switch ('CHARGE')
c
c     rotate the multipole components into the global frame
c
      do i = 1, npole
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c     calculate the multipole interaction energy term
c
      do ii = 1, npole-1
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, n12(i)
            skip(i12(j,i)) = i
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = -i
         end do
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii+1, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               if (skip(k) .ne. i) then
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  if (use_image)  call image (xr,yr,zr,0)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     r = sqrt(r2)
                     fik = fi * rpk(1)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)
                     if (skip(k) .eq. -i) then
                        fik = fik / chgscale
                        e = e / chgscale
                     end if
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if
                     em = em + e
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, npole
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               do m = 1, ncell
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  call image (xr,yr,zr,m)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if
                     if (i .eq. k)  e = 0.5d0 * e
                     em = em + e
                  end if
               end do
            end if
         end do
      end do
      return
      end
c
c
c     #############################################################
c     ##                                                         ##
c     ##  subroutine rotmat  --  rotation matrix for multipoles  ##
c     ##                                                         ##
c     #############################################################
c
c
c     "rotmat" find the rotation matrix that converts from the local
c     coordinate system at each multipole site to the global system
c     
c
      subroutine rotmat (i,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i
      real*8 r,dotxz
      real*8 dx,dy,dz
      real*8 dx1,dy1,dz1
      real*8 dx2,dy2,dz2
      real*8 a(3,3)
c
c
c     rotation matrix elements for z- and x-axes, first z then x
c
      if (polaxe(i) .eq. 'Z-then-X') then
         dx = x(zaxis(i)) - x(ipole(i))
         dy = y(zaxis(i)) - y(ipole(i))
         dz = z(zaxis(i)) - z(ipole(i))
         r = sqrt(dx*dx + dy*dy + dz*dz)
         a(1,3) = dx / r
         a(2,3) = dy / r
         a(3,3) = dz / r
         dx = x(xaxis(i)) - x(ipole(i))
         dy = y(xaxis(i)) - y(ipole(i))
         dz = z(xaxis(i)) - z(ipole(i))
         dotxz = dx*a(1,3) + dy*a(2,3) + dz*a(3,3)
         dx = dx - dotxz*a(1,3)
         dy = dy - dotxz*a(2,3)
         dz = dz - dotxz*a(3,3)
         r = sqrt(dx*dx + dy*dy + dz*dz)
         a(1,1) = dx / r
         a(2,1) = dy / r
         a(3,1) = dz / r
c
c     rotation matrix elements for z- and x-axes, bisector method
c
      else if (polaxe(i) .eq. 'Bisector') then
         dx = x(zaxis(i)) - x(ipole(i))
         dy = y(zaxis(i)) - y(ipole(i))
         dz = z(zaxis(i)) - z(ipole(i))
         r = sqrt(dx*dx + dy*dy + dz*dz)
         dx1 = dx / r
         dy1 = dy / r
         dz1 = dz / r
         dx = x(xaxis(i)) - x(ipole(i))
         dy = y(xaxis(i)) - y(ipole(i))
         dz = z(xaxis(i)) - z(ipole(i))
         r = sqrt(dx*dx + dy*dy + dz*dz)
         dx2 = dx / r
         dy2 = dy / r
         dz2 = dz / r
         dx = dx1 + dx2
         dy = dy1 + dy2
         dz = dz1 + dz2
         r = sqrt(dx*dx + dy*dy + dz*dz)
         a(1,3) = dx / r
         a(2,3) = dy / r
         a(3,3) = dz / r
         dotxz = dx2*a(1,3) + dy2*a(2,3) + dz2*a(3,3)
         dx = dx2 - dotxz*a(1,3)
         dy = dy2 - dotxz*a(2,3)
         dz = dz2 - dotxz*a(3,3)
         r = sqrt(dx*dx + dy*dy + dz*dz)
         a(1,1) = dx / r
         a(2,1) = dy / r
         a(3,1) = dz / r
      end if
c
c     finally, find rotation matrix elements for the y-axis
c
      a(1,2) = a(3,1)*a(2,3) - a(2,1)*a(3,3)
      a(2,2) = a(1,1)*a(3,3) - a(3,1)*a(1,3)
      a(3,2) = a(2,1)*a(1,3) - a(1,1)*a(2,3)
      return
      end
c
c
c     ##########################
c     ##                      ##
c     ##  subroutine rotpole  ##
c     ##                      ##
c     ##########################
c
c
      subroutine rotpole (imdq,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,m,imdq
      real*8 m2(3,3),r2(3,3),a(3,3)
c
c
c     monopoles have the same value in any coordinate frame
c
      rpole(1,imdq) = pole(1,imdq)
c
c     rotate the dipoles to the global coordinate frame
c
      do i = 2, 4
         rpole(i,imdq) = 0.0d0
         do j = 2, 4
            rpole(i,imdq) = rpole(i,imdq) + pole(j,imdq)*a(i-1,j-1)
         end do
      end do
c
c     rotate the quadrupoles to the global coordinate frame
c
      k = 5
      do i = 1, 3
         do j = 1, 3
            m2(i,j) = pole(k,imdq)
            r2(i,j) = 0.0d0
            k = k + 1
         end do
      end do
      do i = 1, 3
         do j = 1, 3
            if (j .lt. i) then
               r2(i,j) = r2(j,i)
            else
               do k = 1, 3
                  do m = 1, 3
                     r2(i,j) = r2(i,j) + a(i,k)*a(j,m)*m2(k,m)
                  end do
               end do
            end if
         end do
      end do
      k = 5
      do i = 1, 3
         do j = 1, 3
            rpole(k,imdq) = r2(i,j)
            k = k + 1
         end do
      end do
      return
      end
c
c
c     #######################
c     ##                   ##
c     ##  function emdqik  ##
c     ##                   ##
c     #######################
c
c
      function emdqik (r,xr,yr,zr,rpi,rpk)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'electr.i'
      include 'mpole.i'
      include 'units.i'
      integer i,j,k
      real*8 emdqik,t2matrix
      real*8 r,r2,r3,r4,r5
      real*8 xr,yr,zr,zrr
      real*8 xrr,xrr2,xrr3,xrr4
      real*8 yrr,yrr2,yrr3,yrr4
      real*8 rpi(13),rpk(13)
      real*8 m2t2(13),t2(13,13)
      real*8 bx(0:5,0:5),by(11,0:5,0:1),bz(11,0:1)
c
c
c     zeroth order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 1) then
         bz(1,0) = c(1,0,0)
         by(1,0,0) = c(1,0,0) * bz(1,0)
         bx(0,0) = c(1,0,0)
      end if
c
c     first order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 4) then
         r2 = r * r
         zrr = zr / r
         bz(3,0) = c(3,0,0)
         bz(1,1) = c(1,1,1) * zrr
         yrr = yr / r
         by(3,0,0) = c(3,0,0) * bz(3,0)
         by(1,0,1) = c(1,0,0) * bz(1,1)
         by(1,1,0) = c(1,1,1) * bz(3,0) * yrr
         xrr = xr / r
         bx(1,1) = c(1,1,1) * xrr
c
c     second order coefficients to speed the T2 calculation
c
         r3 = r2 * r
         bz(5,0) = c(5,0,0)
         bz(3,1) = c(3,1,1) * zrr
         yrr2 = yrr * yrr
         by(5,0,0) = c(5,0,0) * bz(5,0)
         by(3,0,1) = c(3,0,0) * bz(3,1)
         by(3,1,0) = c(3,1,1) * bz(5,0) * yrr
         by(1,1,1) = c(1,1,1) * bz(3,1) * yrr
         by(1,2,0) = c(1,2,0)*bz(3,0) + c(1,2,2)*bz(5,0)*yrr2
         xrr2 = xrr * xrr
         bx(2,0) = c(1,2,0)
         bx(2,2) = c(1,2,2) * xrr2
      end if
c
c     third order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 13) then
         r4 = r2 * r2
         bz(7,0) = c(7,0,0)
         bz(5,1) = c(5,1,1) * zrr
         yrr3 = yrr2 * yrr
         by(7,0,0) = c(7,0,0)*bz(7,0)
         by(5,0,1) = c(5,0,0)*bz(5,1)
         by(5,1,0) = c(5,1,1)*bz(7,0)*yrr
         by(3,1,1) = c(3,1,1)*bz(5,1)*yrr
         by(3,2,0) = c(3,2,0)*bz(5,0) + c(3,2,2)*bz(7,0)*yrr2
         by(1,2,1) = c(1,2,0)*bz(3,1) + c(1,2,2)*bz(5,1)*yrr2
         by(1,3,0) = c(1,3,1)*bz(5,0)*yrr + c(1,3,3)*bz(7,0)*yrr3
         xrr3 = xrr2 * xrr
         bx(3,1) = c(1,3,1) * xrr
         bx(3,3) = c(1,3,3) * xrr3
c
c     fourth order coefficients to speed the T2 calculation
c
         r5 = r2 * r3
         bz(9,0) = c(9,0,0)
         bz(7,1) = c(7,1,1) * zrr
         yrr4 = yrr2 * yrr2
         by(9,0,0) = c(9,0,0)*bz(9,0)
         by(7,0,1) = c(7,0,0)*bz(7,1)
         by(7,1,0) = c(7,1,1)*bz(9,0)*yrr
         by(5,1,1) = c(5,1,1)*bz(7,1)*yrr
         by(5,2,0) = c(5,2,0)*bz(7,0) + c(5,2,2)*bz(9,0)*yrr2
         by(3,2,1) = c(3,2,0)*bz(5,1) + c(3,2,2)*bz(7,1)*yrr2
         by(3,3,0) = c(3,3,1)*bz(7,0)*yrr + c(3,3,3)*bz(9,0)*yrr3
         by(1,3,1) = c(1,3,1)*bz(5,1)*yrr + c(1,3,3)*bz(7,1)*yrr3
         by(1,4,0) = c(1,4,0)*bz(5,0) + c(1,4,2)*bz(7,0)*yrr2
     &                  + c(1,4,4)*bz(9,0)*yrr4
         xrr4 = xrr2 * xrr2
         bx(4,0) = c(1,4,0)
         bx(4,2) = c(1,4,2) * xrr2
         bx(4,4) = c(1,4,4) * xrr4
      end if
c
c     zeroth order T2 matrix elements
c
      if (mdqsiz .ge. 1) then
         t2(1,1) = t2matrix (0,0,0,r,bx,by)
      end if
c
c     first order T2 matrix elements
c
      if (mdqsiz .ge. 4) then
         t2(2,1) = t2matrix (0,0,1,r2,bx,by)
         t2(3,1) = t2matrix (0,1,0,r2,bx,by)
         t2(4,1) = t2matrix (1,0,0,r2,bx,by)
         t2(1,2) = -t2(2,1)
         t2(1,3) = -t2(3,1)
         t2(1,4) = -t2(4,1)
c
c     second order T2 matrix elements
c
         t2(2,2) = -t2matrix (0,0,2,r3,bx,by)
         t2(3,2) = -t2matrix (0,1,1,r3,bx,by)
         t2(4,2) = -t2matrix (1,0,1,r3,bx,by)
         t2(3,3) = -t2matrix (0,2,0,r3,bx,by)
         t2(4,3) = -t2matrix (1,1,0,r3,bx,by)
         t2(4,4) = -t2(2,2) - t2(3,3)
         t2(2,3) = t2(3,2)
         t2(2,4) = t2(4,2)
         t2(3,4) = t2(4,3)
         k = 5
         do i = 2, 4
            do j = 2, 4
               t2(1,k) = -t2(i,j)
               t2(k,1) = t2(1,k)
               k = k + 1
            end do
         end do
      end if
c
c     third order T2 matrix elements
c
      if (mdqsiz .ge. 13) then
         t2(2,5) = t2matrix (0,0,3,r4,bx,by)
         t2(3,5) = t2matrix (0,1,2,r4,bx,by)
         t2(4,5) = t2matrix (1,0,2,r4,bx,by)
         t2(3,6) = t2matrix (0,2,1,r4,bx,by)
         t2(4,6) = t2matrix (1,1,1,r4,bx,by)
         t2(4,7) = -t2(2,5) - t2(3,6)
         t2(2,6) = t2(3,5)
         t2(2,7) = t2(4,5)
         t2(3,7) = t2(4,6)
         t2(2,8) = t2(2,6)
         t2(2,9) = t2(3,6)
         t2(2,10) = t2(4,6)
         t2(3,9) = t2matrix (0,3,0,r4,bx,by)
         t2(3,10) = t2matrix (1,2,0,r4,bx,by)
         t2(4,10) = -t2(2,8) - t2(3,9)
         t2(3,8) = t2(2,9)
         t2(4,8) = t2(2,10)
         t2(4,9) = t2(3,10)
         t2(2,11) = t2(2,7)
         t2(2,12) = t2(3,7)
         t2(2,13) = t2(4,7)
         t2(3,12) = t2(3,10)
         t2(3,13) = t2(4,10)
         t2(4,13) = -t2(2,11) - t2(3,12)
         t2(3,11) = t2(2,12)
         t2(4,11) = t2(2,13)
         t2(4,12) = t2(3,13)
         do i = 5, 13
            do j = 2, 4
               t2(i,j) = -t2(j,i)
            end do
         end do
c
c     fourth order T2 matrix elements
c
         t2(5,5) = t2matrix (0,0,4,r5,bx,by)
         t2(6,5) = t2matrix (0,1,3,r5,bx,by)
         t2(7,5) = t2matrix (1,0,3,r5,bx,by)
         t2(6,6) = t2matrix (0,2,2,r5,bx,by)
         t2(7,6) = t2matrix (1,1,2,r5,bx,by)
         t2(7,7) = -t2(5,5) - t2(6,6)
         t2(8,5) = t2(6,5)
         t2(9,5) = t2(6,6)
         t2(10,5) = t2(7,6)
         t2(9,6) = t2matrix (0,3,1,r5,bx,by)
         t2(10,6) = t2matrix (1,2,1,r5,bx,by)
         t2(10,7) = -t2(8,5) - t2(9,6)
         t2(8,6) = t2(9,5)
         t2(8,7) = t2(10,5)
         t2(9,7) = t2(10,6)
         t2(11,5) = t2(7,5)
         t2(12,5) = t2(7,6)
         t2(13,5) = t2(7,7)
         t2(12,6) = t2(10,6)
         t2(13,6) = t2(10,7)
         t2(13,7) = -t2(11,5) - t2(12,6)
         t2(11,6) = t2(12,5)
         t2(11,7) = t2(13,5)
         t2(12,7) = t2(13,6)
         t2(8,8) = t2(8,6)
         t2(9,8) = t2(9,6)
         t2(10,8) = t2(10,6)
         t2(9,9) = t2matrix (0,4,0,r5,bx,by)
         t2(10,9) = t2matrix (1,3,0,r5,bx,by)
         t2(10,10) = -t2(8,8) - t2(9,9)
         t2(11,8) = t2(11,6)
         t2(12,8) = t2(12,6)
         t2(13,8) = t2(13,6)
         t2(12,9) = t2(10,9)
         t2(13,9) = t2(10,10)
         t2(13,10) = -t2(11,8) - t2(12,9)
         t2(11,9) = t2(12,8)
         t2(11,10) = t2(13,8)
         t2(12,10) = t2(13,9)
         t2(11,11) = t2(11,7)
         t2(12,11) = t2(12,7)
         t2(13,11) = t2(13,7)
         t2(12,12) = t2(12,10)
         t2(13,12) = t2(13,10)
         t2(13,13) = -t2(11,11) - t2(12,12)
         do i = 5, 12
            do j = i+1, 13
               t2(i,j) = t2(j,i)
            end do
         end do
      end if
c
c     compute interaction energy between the two multipole sites
c
      do i = 1, mdqsiz
         m2t2(i) = 0.0d0
         do j = 1, mdqsiz
            m2t2(i) = m2t2(i) + rpk(j)*t2(j,i)
         end do
      end do
      emdqik = 0.0d0
      do i = 1, mdqsiz
         emdqik = emdqik + m2t2(i)*rpi(i)
      end do
      emdqik = (electric / dielec) * emdqik
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  function t2matrix  ##
c     ##                     ##
c     #########################
c
c
      function t2matrix (di,dj,dk,rr,bx,by)
      implicit none
      include 'sizes.i'
      include 'mpole.i'
      integer k,di,dj,dk
      real*8 t2matrix,rr
      real*8 bx(0:5,0:5),by(11,0:5,0:1)
c
c
c     get a single element of the Applequist-Dykstra T2 matrix
c
      t2matrix = 0.0d0
      do k = dk, 0, -2
         t2matrix = t2matrix + bx(dk,k)*by(k+dk+1,dj,di)
      end do
      t2matrix = t2matrix / rr
      return
      end
c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1994  by  Jay William Ponder  ##
c     ###################################################
c
c     ##############################################################
c     ##                                                          ##
c     ##  subroutine empole1  --  multipole energy & derivatives  ##
c     ##                                                          ##
c     ##############################################################
c
c
c     "empole1" calculates the multipole electrostatic energy and
c     first derivatives with respect to Cartesian coordinates
c
c
      subroutine empole1
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'deriv.i'
      include 'electr.i'
      include 'energi.i'
      include 'inter.i'
      include 'molcul.i'
      include 'mpole.i'
      include 'shunt.i'
      include 'units.i'
      include 'usage.i'
      include 'virial.i'
      integer i,j,k,m,skip(maxatm)
      integer ii,iz,ix,kk,kz,kx
      real*8 e,de,emdqik1,f,fi,fik
      real*8 shift,taper,dtaper
      real*8 trans,dtrans
      real*8 xi,yi,zi,xr,yr,zr
      real*8 xiz,yiz,ziz,xix,yix,zix
      real*8 xkz,ykz,zkz,xkx,ykx,zkx
      real*8 r,r2,r3,r4,r5,r6,r7
      real*8 a(3,3),d(3,3,3,3)
      real*8 rpi(13),rpk(13)
      real*8 d0(3),di(3,3),dk(3,3)
      logical iuse,kuse
c
c
c     zero out the multipole interaction energy and derivatives
c
      em = 0.0d0
      do i = 1, n
         do j = 1, 3
            dem(j,i) = 0.0d0
         end do
      end do
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set the switching function coefficients
c
      f = electric / dielec
      call switch ('CHARGE')
c
c     rotate multipole components and get rotation derivatives
c
      do i = 1, npole
         call rotmat (i,a)
         call rotpole (i,a)
         call drotmat (i,d)
         do j = 1, 3
            do k = 1, 3
               call drotpole (i,a,d,j,k)
            end do
         end do
      end do
c
c     compute the multipole interaction energy and derivatives
c
      do ii = 1, npole-1
         i = ipole(ii)
         xi = x(i)
         yi = y(i)
         zi = z(i)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, n12(i)
            skip(i12(j,i)) = i
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = -i
         end do
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii+1, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               if (skip(k) .ne. i) then
                  xr = x(k) - xi
                  yr = y(k) - yi
                  zr = z(k) - zi
                  if (use_image)  call image (xr,yr,zr,0)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik1 (ii,kk,r,xr,yr,zr,rpi,rpk,d0,di,dk)
                     if (skip(k) .eq. -i) then
                        fik = fik / chgscale
                        e = e / chgscale
                        do j = 1, 3
                           d0(j) = d0(j) / chgscale
                           do m = 1, 3
                              di(m,j) = di(m,j) / chgscale
                              dk(m,j) = dk(m,j) / chgscale
                           end do
                        end do
                     end if
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        dtaper = 5.0d0*c5*r4 + 4.0d0*c4*r3
     &                              + 3.0d0*c3*r2 + 2.0d0*c2*r + c1
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        dtrans = fik * (7.0d0*f7*r6 + 6.0d0*f6*r5
     &                                  + 5.0d0*f5*r4 + 4.0d0*f4*r3
     &                                + 3.0d0*f3*r2 + 2.0d0*f2*r + f1)
                        de = e * dtaper + dtrans
                        e = e * taper + trans
                        d0(1) = d0(1)*taper + de*(xr/r)
                        d0(2) = d0(2)*taper + de*(yr/r)
                        d0(3) = d0(3)*taper + de*(zr/r)
                        di(1,1) = di(1,1) * taper
                        di(1,2) = di(1,2) * taper
                        di(1,3) = di(1,3) * taper
                        di(2,1) = di(2,1) * taper
                        di(2,2) = di(2,2) * taper
                        di(2,3) = di(2,3) * taper
                        di(3,1) = di(3,1) * taper
                        di(3,2) = di(3,2) * taper
                        di(3,3) = di(3,3) * taper
                        dk(1,1) = dk(1,1) * taper
                        dk(1,2) = dk(1,2) * taper
                        dk(1,3) = dk(1,3) * taper
                        dk(2,1) = dk(2,1) * taper
                        dk(2,2) = dk(2,2) * taper
                        dk(2,3) = dk(2,3) * taper
                        dk(3,1) = dk(3,1) * taper
                        dk(3,2) = dk(3,2) * taper
                        dk(3,3) = dk(3,3) * taper
                     end if
c
c     increment the overall energy and derivative expressions
c
                     em = em + e
                     dem(1,i) = dem(1,i) - d0(1) + di(1,1)
                     dem(2,i) = dem(2,i) - d0(2) + di(1,2)
                     dem(3,i) = dem(3,i) - d0(3) + di(1,3)
                     dem(1,iz) = dem(1,iz) + di(2,1)
                     dem(2,iz) = dem(2,iz) + di(2,2)
                     dem(3,iz) = dem(3,iz) + di(2,3)
                     dem(1,ix) = dem(1,ix) + di(3,1)
                     dem(2,ix) = dem(2,ix) + di(3,2)
                     dem(3,ix) = dem(3,ix) + di(3,3)
                     dem(1,k) = dem(1,k) + d0(1) + dk(1,1)
                     dem(2,k) = dem(2,k) + d0(2) + dk(1,2)
                     dem(3,k) = dem(3,k) + d0(3) + dk(1,3)
                     dem(1,kz) = dem(1,kz) + dk(2,1)
                     dem(2,kz) = dem(2,kz) + dk(2,2)
                     dem(3,kz) = dem(3,kz) + dk(2,3)
                     dem(1,kx) = dem(1,kx) + dk(3,1)
                     dem(2,kx) = dem(2,kx) + dk(3,2)
                     dem(3,kx) = dem(3,kx) + dk(3,3)
c
c     increment the total intermolecular energy
c
                     if (molcule(i) .ne. molcule(k)) then
                        einter = einter + e
                     end if
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        xiz = x(iz) - x(i)
                        yiz = y(iz) - y(i)
                        ziz = z(iz) - z(i)
                        xix = x(ix) - x(i)
                        yix = y(ix) - y(i)
                        zix = z(ix) - z(i)
                        xkz = x(kz) - x(k)
                        ykz = y(kz) - y(k)
                        zkz = z(kz) - z(k)
                        xkx = x(kx) - x(k)
                        ykx = y(kx) - y(k)
                        zkx = z(kx) - z(k)
                        virx = virx + xr*d0(1) + xiz*di(2,1)
     &                            + xix*di(3,1) + xkz*dk(2,1)
     &                            + xkx*dk(3,1)
                        viry = viry + yr*d0(2) + yiz*di(2,2)
     &                            + yix*di(3,2) + ykz*dk(2,2)
     &                            + ykx*dk(3,2)
                        virz = virz + zr*d0(3) + ziz*di(2,3)
     &                            + zix*di(3,3) + zkz*dk(2,3)
     &                            + zkx*dk(3,3)
                     end if
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, npole
         i = ipole(ii)
         xi = x(i)
         yi = y(i)
         zi = z(i)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               do m = 1, ncell
                  xr = x(k) - xi
                  yr = y(k) - yi
                  zr = z(k) - zi
                  call image (xr,yr,zr,m)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik1 (ii,kk,r,xr,yr,zr,rpi,rpk,d0,di,dk)
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        dtaper = 5.0d0*c5*r4 + 4.0d0*c4*r3
     &                              + 3.0d0*c3*r2 + 2.0d0*c2*r + c1
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        dtrans = fik * (7.0d0*f7*r6 + 6.0d0*f6*r5
     &                                  + 5.0d0*f5*r4 + 4.0d0*f4*r3
     &                                + 3.0d0*f3*r2 + 2.0d0*f2*r + f1)
                        de = e * dtaper + dtrans
                        e = e * taper + trans
                        d0(1) = d0(1)*taper + de*(xr/r)
                        d0(2) = d0(2)*taper + de*(yr/r)
                        d0(3) = d0(3)*taper + de*(zr/r)
                        di(1,1) = di(1,1) * taper
                        di(1,2) = di(1,2) * taper
                        di(1,3) = di(1,3) * taper
                        di(2,1) = di(2,1) * taper
                        di(2,2) = di(2,2) * taper
                        di(2,3) = di(2,3) * taper
                        di(3,1) = di(3,1) * taper
                        di(3,2) = di(3,2) * taper
                        di(3,3) = di(3,3) * taper
                        dk(1,1) = dk(1,1) * taper
                        dk(1,2) = dk(1,2) * taper
                        dk(1,3) = dk(1,3) * taper
                        dk(2,1) = dk(2,1) * taper
                        dk(2,2) = dk(2,2) * taper
                        dk(2,3) = dk(2,3) * taper
                        dk(3,1) = dk(3,1) * taper
                        dk(3,2) = dk(3,2) * taper
                        dk(3,3) = dk(3,3) * taper
                     end if
c
c     increment the energy and gradient values
c
                     if (i .eq. k)  e = 0.5d0 * e
                     em = em + e
                     dem(1,i) = dem(1,i) - d0(1) + di(1,1)
                     dem(2,i) = dem(2,i) - d0(2) + di(1,2)
                     dem(3,i) = dem(3,i) - d0(3) + di(1,3)
                     dem(1,iz) = dem(1,iz) + di(2,1)
                     dem(2,iz) = dem(2,iz) + di(2,2)
                     dem(3,iz) = dem(3,iz) + di(2,3)
                     dem(1,ix) = dem(1,ix) + di(3,1)
                     dem(2,ix) = dem(2,ix) + di(3,2)
                     dem(3,ix) = dem(3,ix) + di(3,3)
                     if (i .ne. k) then
                        dem(1,k) = dem(1,k) + d0(1) + dk(1,1)
                        dem(2,k) = dem(2,k) + d0(2) + dk(1,2)
                        dem(3,k) = dem(3,k) + d0(3) + dk(1,3)
                        dem(1,kz) = dem(1,kz) + dk(2,1)
                        dem(2,kz) = dem(2,kz) + dk(2,2)
                        dem(3,kz) = dem(3,kz) + dk(2,3)
                        dem(1,kx) = dem(1,kx) + dk(3,1)
                        dem(2,kx) = dem(2,kx) + dk(3,2)
                        dem(3,kx) = dem(3,kx) + dk(3,3)
                     end if
c
c     increment the total intermolecular energy
c
                     einter = einter + e
c
c     increment the virial for use in pressure computation
c
                     if (isobaric) then
                        xiz = x(iz) - x(i)
                        yiz = y(iz) - y(i)
                        ziz = z(iz) - z(i)
                        xix = x(ix) - x(i)
                        yix = y(ix) - y(i)
                        zix = z(ix) - z(i)
                        xkz = x(kz) - x(k)
                        ykz = y(kz) - y(k)
                        zkz = z(kz) - z(k)
                        xkx = x(kx) - x(k)
                        ykx = y(kx) - y(k)
                        zkx = z(kx) - z(k)
                        virx = virx + xr*d0(1) + xiz*di(2,1)
     &                            + xix*di(3,1) + xkz*dk(2,1)
     &                            + xkx*dk(3,1)
                        viry = viry + yr*d0(2) + yiz*di(2,2)
     &                            + yix*di(3,2) + ykz*dk(2,2)
     &                            + ykx*dk(3,2)
                        virz = virz + zr*d0(3) + ziz*di(2,3)
     &                            + zix*di(3,3) + zkz*dk(2,3)
     &                            + zkx*dk(3,3)
                     end if
                  end if
               end do
            end if
         end do
      end do
      return
      end
c
c
c     ##########################
c     ##                      ##
c     ##  subroutine drotmat  ##
c     ##                      ##
c     ##########################
c
c
      subroutine drotmat (i,d)
      implicit none
      include 'sizes.i'
      include 'mpole.i'
      integer i
      real*8 d(3,3,3,3)
c
c
      if (polaxe(i) .eq. 'Z-then-X') then
         call drotmat1 (i,d)
      else if (polaxe(i) .eq. 'Bisector') then
         call drotmat2 (i,d)
      end if
      return
      end
c
c
c     ###########################
c     ##                       ##
c     ##  subroutine drotmat1  ##
c     ##                       ##
c     ###########################
c
c
      subroutine drotmat1 (i,d)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k
      real*8 d(3,3,3,3)
      real*8 xi,yi,zi,xz,yz,zz,xx,yx,zx
      real*8 t1,t2,t3,t4,t5,t6,t7,t8,t9
      real*8 t10,t12,t13,t14,t15,t16,t17,t18,t19
      real*8 t20,t21,t22,t23,t24,t25,t26,t27,t28,t29
      real*8 t30,t31,t32,t33,t34,t35,t36,t37,t38,t39
      real*8 t40,t41,t42,t43,t44,t46,t47,t49
      real*8 t50,t52,t58,t59
      real*8 t60,t62,t64,t65,t66,t68,t69
      real*8 t70,t72,t73,t74,t75,t76,t77,t78
      real*8 t80,t81,t86,t91,t93,t95,t97,t98
      real*8 t101,t105,t106,t111,t116,t118
      real*8 t120,t123,t124,t126,t130,t131,t136
      real*8 t141,t143,t145,t146,t148
      real*8 t150,t152,t153,t155,t157,t159
      real*8 t163,t167,t168,t173,t178,t180,t182,t186
      real*8 t190,t191,t196,t201,t203,t205,t209,t213,t214
      real*8 t219,t224,t226,t228,t230,t232,t234,t236,t238
      real*8 t240,t242,t243,t246,t249,t251,t253,t254,t255
      real*8 t256,t259,t263,t265,t266,t271,t273,t276,t278
      real*8 t282,t285,t287,t290,t292,t295,t297,t300,t304
      real*8 t308,t310,t314,t317,t324,t329,t332,t354,t358
      real*8 t362,t366,t370,t374,t381,t388,t395,t416,t418
      real*8 t422,t425,t428,t430,t432,t436,t438,t441,t446
c
c
c     coordinates of main atom and those defining local axes
c
      xi = x(ipole(i))
      yi = y(ipole(i))
      zi = z(ipole(i))
      xz = x(zaxis(i))
      yz = y(zaxis(i))
      zz = z(zaxis(i))
      xx = x(xaxis(i))
      yx = y(xaxis(i))
      zx = z(xaxis(i))
c
c     temporary variables from Maple symbolic algebra derivation
c
      t1 = xz - xi
      t2 = t1 * t1
      t3 = yz - yi
      t4 = t3 * t3
      t5 = zz - zi
      t6 = t5 * t5
      t7 = t2 + t4 + t6
      t8 = sqrt(t7)
      t9 = 1.0d0 / t8
      t10 = t7 * t7
      t12 = t8 / t10
      t13 = t1 * t12
      t14 = 2.0d0 * (xi-xz)
      t15 = t13 * t14
      t16 = 2.0d0 * (yi-yz)
      t17 = t13 * t16
      t18 = 2.0d0 * (zi-zz)
      t19 = t13 * t18
      t20 = t3 * t12
      t21 = t20 * t14
      t22 = t20 * t16
      t23 = t20 * t18
      t24 = t5 * t12
      t25 = t24 * t14
      t26 = t24 * t16
      t27 = t24 * t18
      t28 = 2.0d0 * (xz-xi)
      t29 = t13 * t28
      t30 = 2.0d0 * (yz-yi)
      t31 = t13 * t30
      t32 = 2.0d0 * (zz-zi)
      t33 = t13 * t32
      t34 = t20 * t28
      t35 = t20 * t30
      t36 = t20 * t32
      t37 = t24 * t28
      t38 = t24 * t30
      t39 = t24 * t32
      t40 = t1 * t9
      t41 = xx - xi
      t42 = t41 * t9
      t43 = t41 * t1
      t44 = t12 * t14
      t46 = yx - yi
      t47 = t46 * t3
      t49 = zx - zi
      t50 = t49 * t5
      t52 = -t40 - t42 - 0.5d0*t44*(t43+t47+t50)
      t58 = t9 * (t43+t47+t50)
      t59 = t58 * t9
      t60 = t58 * t1
      t62 = -1.0d0 - t52*t1*t9 + t59 + 0.5d0*t60*t44
      t64 = xx - xi - t60*t9
      t65 = t64 * t64
      t66 = t58 * t3
      t68 = yx - yi - t66*t9
      t69 = t68 * t68
      t70 = t58 * t5
      t72 = zx - zi - t70*t9
      t73 = t72 * t72
      t74 = t65 + t69 + t73
      t75 = sqrt(t74)
      t76 = 1.0d0 / t75
      t77 = t62 * t76
      t78 = t74 * t74
      t80 = t75 / t78
      t81 = t64 * t80
      t86 = -t52*t3*t9 + 0.5d0*t66*t44
      t91 = -t52*t5*t9 + 0.5d0*t70*t44
      t93 = 2.0d0 * (t64*t62 + t68*t86 + t72*t91)
      t95 = t12 * t16
      t97 = t3 * t9
      t98 = t46 * t9
      t101 =  -t97 - t98 - 0.5d0*t95*(t43+t47+t50)
      t105 = -t101*t1*t9 + 0.5d0*t60*t95
      t106 = t105 * t76
      t111 = -1.0d0 - t101*t3*t9 + t59 + 0.5d0*t66*t95
      t116 = -t101*t5*t9 + 0.5d0*t70*t95
      t118 = 2.0d0 * (t64*t105 + t68*t111 + t72*t116)
      t120 = t12 * t18
      t123 = t5 * t9
      t124 = t49 * t9
      t126 = - t123 - t124 - 0.5d0*t120*(t43+t47+t50)
      t130 = -t126*t1*t9 + 0.5d0*t60*t120
      t131 = t130 * t76
      t136 = -t126*t3*t9 + 0.5d0*t66*t120
      t141 = -1.0d0 - t126*t5*t9 + t59 + 0.5d0*t70*t120
      t143 = 2.0d0 * (t64*t130 + t68*t136 + t72*t141)
      t145 = t86 * t76
      t146 = t68 * t80
      t148 = t111 * t76
      t150 = t136 * t76
      t152 = t91 * t76
      t153 = t72 * t80
      t155 = t116 * t76
      t157 = t141 * t76
      t159 = t12 * t28
      t163 = t42 - 0.5d0*t159*(t43+t47+t50)
      t167 = -t163*t1*t9 - t59 + 0.5d0*t60*t159
      t168 = t167 * t76
      t173 = -t163*t3*t9 + 0.5d0*t66*t159
      t178 = -t163*t5*t9 + 0.5d0*t70*t159
      t180 = 2.0d0 * (t64*t167 + t68*t173 + t72*t178)
      t182 = t12 * t30
      t186 = t98 - 0.5d0*t182*(t43+t47+t50)
      t190 = -t186*t1*t9 + 0.5d0*t60*t182
      t191 = t190 * t76
      t196 = -t186*t3*t9 - t59 + 0.5d0*t66*t182
      t201 = -t186*t5*t9 + 0.5d0*t70*t182
      t203 = 2.0d0 * (t64*t190 + t68*t196 + t72*t201)
      t205 = t12 * t32
      t209 = t124 - 0.5d0*t205*(t43+t47+t50)
      t213 = -t209*t1*t9 + 0.5d0*t60*t205
      t214 = t213 * t76
      t219 = -t209*t3*t9 + 0.5d0*t66*t205
      t224 = -t209*t5*t9 - t59 + 0.5d0*t70*t205
      t226 = 2.0d0 * (t64*t213 + t68*t219 + t72*t224)
      t228 = t173 * t76
      t230 = t196 * t76
      t232 = t219 * t76
      t234 = t178 * t76
      t236 = t201 * t76
      t238 = t224 * t76
      t240 = 1.0d0 / t7
      t242 = 1.0d0 - t2*t240
      t243 = t242 * t76
      t246 = t240 * t3
      t249 = t240 * t5
      t251 = 2.0d0 * (t64*t242 - t68*t1*t246 - t72*t1*t249)
      t253 = t1 * t240
      t254 = t3 * t76
      t255 = t253 * t254
      t256 = t64 * t1
      t259 = 1.0d0 - t4*t240
      t263 =  2.0d0 * (t68*t259 - t256*t246 - t72*t3*t249)
      t265 = t5 * t76
      t266 = t253 * t265
      t271 = 1.0d0 - t6*t240
      t273 =  2.0d0 * (t72*t271 - t256*t249 - t68*t3*t249)
      t276 = t259 * t76
      t278 = t246 * t265
      t282 = t271 * t76
      t285 = t97 * t93
      t287 = t72 * t76
      t290 = t123 * t93
      t292 = t68 * t76
      t295 = t97 * t118
      t297 = t287 * t9
      t300 = t123 * t118
      t304 = t97 * t143
      t308 = t123 * t143
      t310 = t292 * t9
      t314 = t64 * t76
      t317 = t40 * t93
      t324 = t40 * t118
      t329 = t314 * t9
      t332 = t40 * t143
      t354 = t97 * t180
      t358 = t123 * t180
      t362 = t97 * t203
      t366 = t123 * t203
      t370 = t97 * t226
      t374 = t123 * t226
      t381 = t40 * t180
      t388 = t40 * t203
      t395 = t40 * t226
      t416 = t97 * t251
      t418 = t123 * t251
      t422 = t97 * t263
      t425 = t123 * t263
      t428 = t97 * t273
      t430 = t6 * t76
      t432 = t123 * t273
      t436 = t2 * t12
      t438 = t40 * t251
      t441 = t40 * t263
      t446 = t40 * t273
c
c     set values for the derivatives of the rotation matrix
c
      d(1,3,1,1) = -t9 - 0.5d0*t15
      d(1,3,1,2) = -0.5d0 * t17
      d(1,3,1,3) = -0.5d0 * t19
      d(2,3,1,1) = -0.5d0 * t21
      d(2,3,1,2) = -t9 - 0.5d0*t22
      d(2,3,1,3) = -0.5d0 * t23
      d(3,3,1,1) = -0.5d0 * t25
      d(3,3,1,2) = -0.5d0 * t26
      d(3,3,1,3) = -t9 - 0.5d0*t27
      d(1,3,2,1) = t9 - 0.5d0*t29
      d(1,3,2,2) = -0.5d0 * t31
      d(1,3,2,3) = -0.5d0 * t33
      d(2,3,2,1) = -0.5d0 * t34
      d(2,3,2,2) = t9 - 0.5d0*t35
      d(2,3,2,3) = -0.5d0 * t36
      d(3,3,2,1) = -0.5d0 * t37
      d(3,3,2,2) = -0.5d0 * t38
      d(3,3,2,3) = t9 - 0.5d0*t39
      d(1,1,1,1) = t77 - 0.5d0*t81*t93
      d(1,1,1,2) = t106 - 0.5d0*t81*t118
      d(1,1,1,3) = t131 - 0.5d0*t81*t143
      d(2,1,1,1) = t145 - 0.5d0*t146*t93
      d(2,1,1,2) = t148 - 0.5d0*t146*t118
      d(2,1,1,3) = t150 - 0.5d0*t146*t143
      d(3,1,1,1) = t152 - 0.5d0*t153*t93
      d(3,1,1,2) = t155 - 0.5d0*t153*t118
      d(3,1,1,3) = t157 - 0.5d0*t153*t143
      d(1,1,2,1) = t168 - 0.5d0*t81*t180
      d(1,1,2,2) = t191 - 0.5d0*t81*t203
      d(1,1,2,3) = t214 - 0.5d0*t81*t226
      d(2,1,2,1) = t228 - 0.5d0*t146*t180
      d(2,1,2,2) = t230 - 0.5d0*t146*t203
      d(2,1,2,3) = t232 - 0.5d0*t146*t226
      d(3,1,2,1) = t234 - 0.5d0*t153*t180
      d(3,1,2,2) = t236 - 0.5d0*t153*t203
      d(3,1,2,3) = t238 - 0.5d0*t153*t226
      d(1,1,3,1) = t243 - 0.5d0*t81*t251
      d(1,1,3,2) = -t255 - 0.5d0*t81*t263
      d(1,1,3,3) = -t266 - 0.5d0*t81*t273
      d(2,1,3,1) = -t255 - 0.5d0*t146*t251
      d(2,1,3,2) = t276 - 0.5d0*t146*t263
      d(2,1,3,3) = -t278 - 0.5d0*t146*t273
      d(3,1,3,1) = -t266 - 0.5d0*t153*t251
      d(3,1,3,2) = -t278 - 0.5d0*t153*t263
      d(3,1,3,3) = t282 - 0.5d0*t153*t273
      d(1,2,1,1) = t152*t97 - 0.5d0*(t153*t285 + t287*t21)
     &                - t145*t123 + 0.5d0*(t146*t290 + t292*t25)
      d(1,2,1,2) = t155*t97 - t297 - 0.5d0*(t153*t295 + t287*t22)
     &                - t148*t123 + 0.5d0*(t146*t300 + t292*t26)
      d(1,2,1,3) = t157*t97 + t310 - 0.5d0*(t153*t304 + t287*t23)
     &                - t150*t123 + 0.5d0*(t146*t308 + t292*t27)
      d(2,2,1,1) = t77*t123 + t297 - 0.5d0*(t81*t290 + t314*t25)
     &                - t152*t40 + 0.5d0*(t153*t317 + t287*t15)
      d(2,2,1,2) = t106*t123 - 0.5d0*(t81*t300 + t314*t26)
     &                - t155*t40 + 0.5d0*(t153*t324 + t287*t17)
      d(2,2,1,3) = t131*t123 - t329 - 0.5d0*(t81*t308 + t314*t27)
     &                - t157*t40 + 0.5d0*(t153*t332 + t287*t19)
      d(3,2,1,1) = t145*t40 - t310 - 0.5d0*(t146*t317 + t292*t15)
     &                - t77*t97 + 0.5d0*(t81*t285 + t314*t21)
      d(3,2,1,2) = t148*t40 + t329 - 0.5d0*(t146*t324 + t292*t17)
     &                - t106*t97 + 0.5d0*(t81*t295 + t314*t22)
      d(3,2,1,3) = t150*t40 - 0.5d0*(t146*t332 + t292*t19)
     &                - t131*t97 + 0.5d0*(t81*t304 + t314*t23)
      d(2,2,1,3) = t131*t123 - t329 - 0.5d0*(t81*t308 + t314*t27)
     &                - t157*t40 + 0.5d0*(t153*t332 + t287*t19)
      d(3,2,1,1) = t145*t40 - t310 - 0.5d0*(t146*t317 + t292*t15)
     &                - t77*t97 + 0.5d0*(t81*t285 + t314*t21)
      d(3,2,1,2) = t148*t40 + t329 - 0.5d0*(t146*t324 + t292*t17)
     &                - t106*t97 + 0.5d0*(t81*t295 + t314*t22)
      d(3,2,1,3) = t150*t40 - 0.5d0*(t146*t332 + t292*t19)
     &                - t131*t97 + 0.5d0*(t81*t304 + t314*t23)
      d(1,2,2,1) = t234*t97 - 0.5d0*(t153*t354 + t287*t34)
     &                - t228*t123 + 0.5d0*(t146*t358 + t292*t37)
      d(1,2,2,2) = t236*t97 + t297 - 0.5d0*(t153*t362 + t287*t35)
     &                - t230*t123 + 0.5d0*(t146*t366 + t292*t38)
      d(1,2,2,3) = t238*t97 - t310 - 0.5d0*(t153*t370 + t287*t36)
     &                - t232*t123 + 0.5d0*(t146*t374 + t292*t39)
      d(2,2,2,1) = t168*t123 - t297 - 0.5d0*(t81*t358 + t314*t37)
     &                - t234*t40 + 0.5d0*(t153*t381 + t287*t29)
      d(2,2,2,2) = t191*t123 - 0.5d0*(t81*t366 + t314*t38)
     &                - t236*t40 + 0.5d0*(t153*t388 + t287*t31)
      d(2,2,2,3) = t214*t123 + t329 - 0.5d0*(t81*t374 + t314*t39)
     &                - t238*t40 + 0.5d0*(t153*t395 + t287*t33)
      d(3,2,2,1) = t228*t40 + t310 - 0.5d0*(t146*t381 + t292*t29)
     &                - t168*t97 + 0.5d0*(t81*t354 + t314*t34)
      d(3,2,2,2) = t230*t40 - t329 - 0.5d0*(t146*t388 + t292*t31)
     &                - t191*t97 + 0.5d0*(t81*t362 + t314*t35)
      d(3,2,2,3) = t232*t40 - 0.5d0*(t146*t395 + t292*t33)
     &                - t214*t97 + 0.5d0*(t81*t370 + t314*t36)
      d(1,2,3,1) = 0.5d0 * (t146*t418 - t153*t416)
      d(1,2,3,2) = -t4*t12*t265 - t276*t123
     &                + 0.5d0*(t146*t425 - t153*t422)
      d(1,2,3,3) = t282*t97 + t20*t430 + 0.5d0*(t146*t432-t153*t428)
      d(2,2,3,1) = t243*t123 + t436*t265 + 0.5d0*(t153*t438-t81*t418)
      d(2,2,3,2) = 0.5d0 * (t153*t441 - t81*t425)
      d(2,2,3,3) = -t13*t430 - t282*t40 + 0.5d0*(t153*t446-t81*t432)
      d(3,2,3,1) = -t436*t254 - t243*t97 + 0.5d0*(t81*t416-t146*t438)
      d(3,2,3,2) = t276*t40 + t13*t4*t76 + 0.5d0*(t81*t422-t146*t441)
      d(3,2,3,3) = 0.5d0 * (t81*t428 - t146*t446)
c
c     some of the derivative matrix values are always zero
c
      do j = 1, 3
         do k = 1, 3
            d(k,3,3,j) = 0.0d0
         end do
      end do
      return
      end
c
c
c     ###########################
c     ##                       ##
c     ##  subroutine drotmat2  ##
c     ##                       ##
c     ###########################
c
c
      subroutine drotmat2 (i,d)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,m
      real*8 d(3,3,3,3)
      real*8 xi,yi,zi,xz,yz,zz,xx,yx,zx
      real*8 t1,t2,t3,t4,t5,t6,t7,t8,t9,t10
      real*8 t12,t13,t14,t16,t18,t19,t20
      real*8 t21,t22,t23,t24,t25,t26,t27,t28,t29
      real*8 t31,t32,t33,t35,t36,t37,t38,t39,t40
      real*8 t41,t42,t44,t45,t47,t48,t50
      real*8 t52,t53,t54,t55,t57,t58,t60
      real*8 t62,t65,t66,t67,t68,t70
      real*8 t73,t75,t77,t78,t79
      real*8 t81,t82,t83,t84,t86,t88,t89
      real*8 t91,t92,t93,t94,t95,t96,t97,t98,t99,t100
      real*8 t101,t103,t104,t106,t108,t109,t110
      real*8 t111,t113,t114,t116,t117,t118
      real*8 t121,t122,t123,t124,t126,t129,t130
      real*8 t131,t133,t134,t135,t137,t138,t139
      real*8 t141,t143,t145,t146,t147,t150
      real*8 t151,t154,t157,t160
      real*8 t162,t164,t166,t169,t170
      real*8 t172,t174,t175,t176,t178,t179,t180
      real*8 t182,t183,t184,t185,t186,t187,t188,t190
      real*8 t191,t195,t198,t202,t205,t207,t209,t210
      real*8 t217,t220,t222,t224,t225,t232,t238,t240
      real*8 t242,t249,t253,t255,t256,t262,t269
      real*8 t271,t273,t274,t276,t278,t280
      real*8 t281,t283,t285,t295,t300,t303,t308,t310
      real*8 t311,t316,t317,t320,t325,t328,t330
      real*8 t335,t348,t351,t352,t354,t356,t357
      real*8 t364,t371,t373,t390,t393,t395,t397,t398
      real*8 t405,t412,t414,t416,t418,t420
      real*8 t422,t424,t426,t429
      real*8 t431,t432,t436,t438,t439
      real*8 t443,t448,t453,t458,t464,t465,t469
      real*8 t478,t480,t488,t498,t503,t519
      real*8 t521,t525,t527,t531,t536,t541,t546
      real*8 t552,t556,t565,t567,t575,t585,t590
c
c
c     coordinates of main atom and those defining local axes
c
      xi = x(ipole(i))
      yi = y(ipole(i))
      zi = z(ipole(i))
      xz = x(zaxis(i))
      yz = y(zaxis(i))
      zz = z(zaxis(i))
      xx = x(xaxis(i))
      yx = y(xaxis(i))
      zx = z(xaxis(i))
c
c     temporary variables from Maple symbolic algebra derivation
c
      t1 = xz - xi
      t2 = t1**2
      t3 = yz - yi
      t4 = t3**2
      t5 = zz - zi
      t6 = t5**2
      t7 = t2 + t4 + t6
      t8 = dsqrt(t7)
      t9 = 1.0d0 / t8
      t10 = t7**2
      t12 = t8 / t10
      t13 = t1 * t12
      t14 = 2.0d0 * (xz-xi)
      t16 = t9 - 0.5d0*t13*t14
      t18 = xx - xi
      t19 = t18**2
      t20 = yx - yi
      t21 = t20**2
      t22 = zx - zi
      t23 = t22**2
      t24 = t19 + t21 + t23
      t25 = dsqrt(t24)
      t26 = 1.0d0 / t25
      t27 = t18 * t26
      t28 = t1*t9 + t27
      t29 = t28**2
      t31 = t20 * t26
      t32 = t3*t9 + t31
      t33 = t32**2
      t35 = t22 * t26
      t36 = t5*t9 + t35
      t37 = t36**2
      t38 = t29 + t33 + t37
      t39 = dsqrt(t38)
      t40 = 1.0d0 / t39
      t41 = t16 * t40
      t42 = t38**2
      t44 = t39 / t42
      t45 = t28 * t44
      t47 = t32 * t3
      t48 = t12 * t14
      t50 = t36 * t5
      t52 = 2.0d0*t28*t16 - t47*t48 - t50*t48
      t53 = t45 * t52
      t54 = 2.0d0 * (yz-yi)
      t55 = t54 * t40
      t57 = t28 * t1
      t58 = t12 * t54
      t60 = t3 * t12
      t62 = t9 - 0.5d0*t60*t54
      t65 = -t57*t58 + 2.0d0*t32*t62 - t50*t58
      t66 = t45 * t65
      t67 = 2.0d0 * (zz-zi)
      t68 = t67 * t40
      t70 = t12 * t67
      t73 = t5 * t12
      t75 = t9 - 0.5d0*t73*t67
      t77 = -t57*t70 - t47*t70 + 2.0d0*t36*t75
      t78 = t45 * t77
      t79 = t14 * t40
      t81 = t32 * t44
      t82 = t81 * t52
      t83 = t62 * t40
      t84 = t81 * t65
      t86 = t81 * t77
      t88 = t36 * t44
      t89 = t88 * t52
      t91 = t88 * t65
      t92 = t75 * t40
      t93 = t88 * t77
      t94 = t24**2
      t95 = 1.0d0 / t94
      t96 = t25 * t95
      t97 = t18 * t96
      t98 = 2.0d0 * (xx-xi)
      t99 = t97 * t98
      t100 = t26 - 0.5d0*t99
      t101 = t100 * t40
      t103 = t32 * t20
      t104 = t96 * t98
      t106 = t36 * t22
      t108 = 2.0d0*t28*t100 - t103*t104 - t106*t104
      t109 = t45 * t108
      t110 = 2.0d0 * (yx-yi)
      t111 = t110 * t40
      t113 = t28 * t18
      t114 = t96 * t110
      t116 = t20 * t96
      t117 = t116 * t110
      t118 = t26 - 0.5d0*t117
      t121 = -t113*t114 + 2.0d0*t32*t118 - t106*t114
      t122 = t45 * t121
      t123 = 2.0d0 * (zx-zi)
      t124 = t123 * t40
      t126 = t96 * t123
      t129 = t22 * t96
      t130 = t129 * t123
      t131 = t26 - 0.5d0*t130
      t133 = -t113*t126 - t103*t126 + 2.0d0*t36*t131
      t134 = t45 * t133
      t135 = t98 * t40
      t137 = t81 * t108
      t138 = t118 * t40
      t139 = t81 * t121
      t141 = t81 * t133
      t143 = t88 * t108
      t145 = t88 * t121
      t146 = t131 * t40
      t147 = t88 * t133
      t150 = t31 * t3
      t151 = t48 * t40
      t154 = t35 * t5
      t157 = t27*t41 - 0.5d0*(t27*t53 + t150*t151 + t31*t82
     &                            + t154*t151 + t35*t89)
      t160 = t28 * t40
      t162 = t32 * t40
      t164 = t36 * t40
      t166 = t27*t160 + t31*t162 + t35*t164
      t169 = t166 * t28
      t170 = t44 * t52
      t172 = -t157*t28*t40 - t166*t16*t40 + 0.5d0*t169*t170
      t174 = t27 - t169*t40
      t175 = t174**2
      t176 = t166 * t32
      t178 = t31 - t176*t40
      t179 = t178**2
      t180 = t166 * t36
      t182 = t35 - t180*t40
      t183 = t182**2
      t184 = t175 + t179 + t183
      t185 = dsqrt(t184)
      t186 = 1.0d0 / t185
      t187 = t172 * t186
      t188 = t184**2
      t190 = t185 / t188
      t191 = t174 * t190
      t195 = t166 * t3
      t198 = -t157*t32*t40 + 0.5d0*(t195*t151 + t176*t170)
      t202 = t166 * t5
      t205 = -t157*t36*t40 + 0.5d0*(t202*t151 + t180*t170)
      t207 = 2.0d0 * (t174*t172 + t178*t198 + t182*t205)
      t209 = t27 * t1
      t210 = t58 * t40
      t217 = t31*t83 - 0.5d0*(t209*t210 + t27*t66 + t31*t84
     &                            + t154*t210 + t35*t91)
      t220 = t166 * t1
      t222 = t44 * t65
      t224 = -t217*t28*t40 + 0.5d0*(t220*t210 + t169*t222)
      t225 = t224 * t186
      t232 = -t217*t32*t40 - t166*t62*t40 + 0.5d0*t176*t222
      t238 = -t217*t36*t40 + 0.5d0*(t202*t210 + t180*t222)
      t240 = 2.0d0 * (t174*t224 + t178*t232 + t182*t238)
      t242 = t70 * t40
      t249 = t35*t92 - 0.5d0*(t209*t242 + t27*t78 + t150*t242
     &                              + t31*t86 + t35*t93)
      t253 = t44 * t77
      t255 = -t249*t28*t40 + 0.5d0*(t220*t242 + t169*t253)
      t256 = t255 * t186
      t262 = -t249*t32*t40 + 0.5d0*(t195*t242 + t176*t253)
      t269 = -t249*t36*t40 - t166*t75*t40 + 0.5d0*t180*t253
      t271 = 2.0d0 * (t174*t255 + t178*t262 + t182*t269)
      t273 = t198 * t186
      t274 = t178 * t190
      t276 = t232 * t186
      t278 = t262 * t186
      t280 = t205 * t186
      t281 = t182 * t190
      t283 = t238 * t186
      t285 = t269 * t186
      t295 = t21 * t95
      t300 = t23 * t95
      t303 = t26*t28*t40 + t27*t101
     &          - 0.5d0 * (t97*t160*t98 + t27*t109 + t116*t162*t98
     &                        + t295*t135 + t31*t137 + t300*t135
     &                        + t35*t143 + t129*t164*t98)
      t308 = t44 * t108
      t310 = t26 - t303*t28*t40 - t166*t100*t40
     &          + 0.5d0*(t169*t308 - t99)
      t311 = t310 * t186
      t316 = t166 * t20
      t317 = t104 * t40
      t320 = -t303*t32*t40 + 0.5d0*(t316*t317 + t176*t308 - t116*t98)
      t325 = t166 * t22
      t328 = -t303*t36*t40 + 0.5d0*(t325*t317 + t180*t308 - t129*t98)
      t330 = 2.0d0 * (t174*t310 + t178*t320 + t182*t328)
      t335 = t19 * t95
      t348 = t26*t32*t40 + t31*t138
     &          - 0.5d0 * (t97*t160*t110 + t335*t111 + t27*t122
     &                        + t116*t162*t110 + t129*t164*t110
     &                        + t300*t111 + t35*t145 + t31*t139)
      t351 = t166 * t18
      t352 = t114 * t40
      t354 = t44 * t121
      t356 = -t348*t28*t40 + 0.5d0*(t351*t352 + t169*t354 - t97*t110)
      t357 = t356 * t186
      t364 = t26 - t348*t32*t40 - t166*t118*t40
     &          + 0.5d0*(t176*t354 - t117)
      t371 = -t348*t36*t40 + 0.5d0*(t325*t352 + t180*t354 - t129*t110)
      t373 = 2.0d0 * (t174*t356 + t178*t364 + t182*t371)
      t390 = t26*t36*t40 + t35*t146
     &          - 0.5d0 * (t97*t160*t123 + t335*t124 + t27*t134
     &                        + t116*t162*t123 + t129*t164*t123
     &                        + t295*t124 + t31*t141 + t35*t147)
      t393 = t126 * t40
      t395 = t44 * t133
      t397 = -t390*t28*t40 + 0.5d0*(t351*t393 + t169*t395 - t97*t123)
      t398 = t397 * t186
      t405 = -t390*t32*t40 + 0.5d0*(t316*t393 + t176*t395 - t116*t123)
      t412 = t26 - t390*t36*t40 - t166*t131*t40
     &          + 0.5d0*(t180*t395 - t130)
      t414 = 2.0d0 * (t174*t397 + t178*t405 + t182*t412)
      t416 = t320 * t186
      t418 = t364 * t186
      t420 = t405 * t186
      t422 = t328 * t186
      t424 = t371 * t186
      t426 = t412 * t186
      t429 = t162 * t207
      t431 = t182 * t186
      t432 = t431 * t3
      t436 = t164 * t207
      t438 = t178 * t186
      t439 = t438 * t5
      t443 = t162 * t240
      t448 = t164 * t240
      t453 = t162 * t271
      t458 = t164 * t271
      t464 = t174 * t186
      t465 = t464 * t5
      t469 = t160 * t207
      t478 = t160 * t240
      t480 = t431 * t1
      t488 = t160 * t271
      t498 = t464 * t3
      t503 = t438 * t1
      t519 = t162 * t330
      t521 = t431 * t20
      t525 = t164 * t330
      t527 = t438 * t22
      t531 = t162 * t373
      t536 = t164 * t373
      t541 = t162 * t414
      t546 = t164 * t414
      t552 = t464 * t22
      t556 = t160 * t330
      t565 = t160 * t373
      t567 = t431 * t18
      t575 = t160 * t414
      t585 = t464 * t20
      t590 = t438 * t18
c
c     set values for the derivatives of the rotation matrix
c
      d(1,3,2,1) = t41 - 0.5d0*t53
      d(1,3,2,2) = -0.5d0 * (t13*t55 + t66)
      d(1,3,2,3) = -0.5d0 * (t13*t68 + t78)
      d(2,3,2,1) = -0.5d0 * (t60*t79 + t82)
      d(2,3,2,2) = t83 - 0.5d0*t84
      d(2,3,2,3) = -0.5d0 * (t60*t68 + t86)
      d(3,3,2,1) = -0.5d0 * (t73*t79 + t89)
      d(3,3,2,2) = -0.5d0 * (t73*t55 + t91)
      d(3,3,2,3) = t92 - 0.5d0*t93
      d(1,3,3,1) = t101 - 0.5d0*t109
      d(1,3,3,2) = -0.5d0 * (t97*t111 + t122)
      d(1,3,3,3) = -0.5d0 * (t97*t124 + t134)
      d(2,3,3,1) = -0.5d0 * (t116*t135 + t137)
      d(2,3,3,2) = t138 - 0.5d0*t139
      d(2,3,3,3) = -0.5d0 * (t116*t124 + t141)
      d(3,3,3,1) = -0.5d0 * (t129*t135 + t143)
      d(3,3,3,2) = -0.5d0 * (t129*t111 + t145)
      d(3,3,3,3) = t146 - 0.5d0*t147
      d(1,1,2,1) = t187 - 0.5d0*t191*t207
      d(1,1,2,2) = t225 - 0.5d0*t191*t240
      d(1,1,2,3) = t256 - 0.5d0*t191*t271
      d(2,1,2,1) = t273 - 0.5d0*t274*t207
      d(2,1,2,2) = t276 - 0.5d0*t274*t240
      d(2,1,2,3) = t278 - 0.5d0*t274*t271
      d(3,1,2,1) = t280 - 0.5d0*t281*t207
      d(3,1,2,2) = t283 - 0.5d0*t281*t240
      d(3,1,2,3) = t285 - 0.5d0*t281*t271
      d(1,1,3,1) = t311 - 0.5d0*t191*t330
      d(1,1,3,2) = t357 - 0.5d0*t191*t373
      d(1,1,3,3) = t398 - 0.5d0*t191*t414
      d(2,1,3,1) = t416 - 0.5d0*t274*t330
      d(2,1,3,2) = t418 - 0.5d0*t274*t373
      d(2,1,3,3) = t420 - 0.5d0*t274*t414
      d(3,1,3,1) = t422 - 0.5d0*t281*t330
      d(3,1,3,2) = t424 - 0.5d0*t281*t373
      d(3,1,3,3) = t426 - 0.5d0*t281*t414
      d(1,2,2,1) = t280*t162 - t273*t164
     &                - 0.5d0*(t281*t429 + t432*t151 + t431*t82)
     &                + 0.5d0*(t274*t436 + t439*t151 + t438*t89)
      d(1,2,2,2) = t283*t162 + t431*t83 - t276*t164
     &                - 0.5d0*(t281*t443 + t431*t84)
     &                + 0.5d0*(t274*t448 + t439*t210 + t438*t91)
      d(1,2,2,3) = t285*t162 - t278*t164 - t438*t92
     &                - 0.5d0*(t281*t453 + t432*t242 + t431*t86)
     &                + 0.5d0*(t274*t458 + t438*t93)
      d(2,2,2,1) = t187*t164 - t280*t160 - t431*t41
     &                - 0.5d0*(t191*t436 + t465*t151 + t464*t89)
     &                + 0.5d0*(t281*t469 + t431*t53)
      d(2,2,2,2) = t225*t164 - t283*t160
     &                - 0.5d0*(t191*t448 + t465*t210 + t464*t91)
     &                + 0.5d0*(t281*t478 + t480*t210 + t431*t66)
      d(2,2,2,3) = t256*t164 + t464*t92 - t285*t160
     &                - 0.5d0*(t191*t458 + t464*t93)
     &                + 0.5d0*(t281*t488 + t480*t242 + t431*t78)
      d(3,2,2,1) = t273*t160 + t438*t41 - t187*t162
     &                - 0.5d0*(t274*t469 + t438*t53)
     &                + 0.5d0*(t191*t429 + t498*t151 + t464*t82)
      d(3,2,2,2) = t276*t160 - t225*t162 - t464*t83
     &                - 0.5d0*(t274*t478 + t503*t210 + t438*t66)
     &                + 0.5d0*(t191*t443 + t464*t84)
      d(3,2,2,3) = t278*t160 - t256*t162
     &                - 0.5d0*(t274*t488 + t503*t242 + t438*t78)
     &                + 0.5d0*(t191*t453 + t498*t242 + t464*t86)
      d(1,2,3,1) = t422*t162 - t416*t164
     &                - 0.5d0*(t281*t519 + t521*t317 + t431*t137)
     &                + 0.5d0*(t274*t525 + t527*t317 + t438*t143)
      d(1,2,3,2) = t424*t162 + t431*t138 - t418*t164
     &                - 0.5d0*(t281*t531 + t431*t139)
     &                + 0.5d0*(t274*t536 + t527*t352 + t438*t145)
      d(1,2,3,3) = t426*t162 - t420*t164 - t438*t146
     &                - 0.5d0*(t281*t541 + t521*t393 + t431*t141)
     &                + 0.5d0*(t274*t546 + t438*t147)
      d(2,2,3,1) = t311*t164 - t422*t160 - t431*t101
     &                - 0.5d0*(t191*t525 + t552*t317 + t464*t143)
     &                + 0.5d0*(t281*t556 + t431*t109)
      d(2,2,3,2) = t357*t164 - t424*t160
     &                - 0.5d0*(t191*t536 + t552*t352 + t464*t145)
     &                + 0.5d0*(t281*t565 + t567*t352 + t431*t122)
      d(2,2,3,3) = t398*t164 + t464*t146 - t426*t160
     &                - 0.5d0*(t191*t546 + t464*t147)
     &                + 0.5d0*(t281*t575 + t567*t393 + t431*t134)
      d(3,2,3,1) = t416*t160 + t438*t101 - t311*t162
     &                - 0.5d0*(t274*t556 + t438*t109)
     &                + 0.5d0*(t191*t519 + t585*t317 + t464*t137)
      d(3,2,3,2) = t418*t160 - t357*t162 - t464*t138
     &                - 0.5d0*(t274*t565 + t590*t352 + t438*t122)
     &                + 0.5d0*(t191*t531 + t464*t139)
      d(3,2,3,3) = t420*t160 - t398*t162
     &                - 0.5d0*(t274*t575 + t590*t393 + t438*t134)
     &                + 0.5d0*(t191*t541 + t585*t393 + t464*t141)
c
c     some derivative matrix values are combinations of others
c
      do j = 1, 3
         do k = 1, 3
            do m = 1, 3
               d(m,k,1,j) = -d(m,k,2,j) - d(m,k,3,j)
            end do
         end do
      end do
      return
      end
c
c
c     ###########################
c     ##                       ##
c     ##  subroutine drotpole  ##
c     ##                       ##
c     ###########################
c
c
      subroutine drotpole (ia,a,d,p,q)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,m,nn
      integer ia,p,q
      real*8 a(3,3),d(3,3,3,3)
      real*8 m2(3,3),dm2(3,3)
c
c
      dpole(1,p,q,ia) = 0.0d0
      do i = 2, 4
         dpole(i,p,q,ia) = 0.0d0
         do j = 2, 4
            dpole(i,p,q,ia) = dpole(i,p,q,ia)
     &                              + pole(j,ia)*d(i-1,j-1,p,q)
         end do
      end do
      nn = 5
      do i = 1, 3
         do j = 1, 3
            m2(i,j) = pole(nn,ia)
            dm2(i,j) = 0.0d0
            nn = nn + 1
         end do
      end do
      do i = 1, 3
         do j = 1, 3
            do k = 1, 3
               do m = 1, 3
                  dm2(i,j) = dm2(i,j) + m2(k,m) *
     &                         (d(i,k,p,q)*a(j,m)+a(i,k)*d(j,m,p,q))
               end do
            end do
          end do
      end do
      nn = 5
      do i = 1, 3
         do j = 1, 3
            dpole(nn,p,q,ia) = dm2(i,j)
            nn = nn + 1
         end do
      end do
      return
      end
c
c
c     ########################
c     ##                    ##
c     ##  function emdqik1  ##
c     ##                    ##
c     ########################
c
c
      function emdqik1 (ii,kk,r,xr,yr,zr,rpi,rpk,gt,gmi,gmj)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'electr.i'
      include 'mpole.i'
      include 'units.i'
      integer i,j,k,m,ii,kk
      real*8 emdqik1,t2matrix
      real*8 r,r2,r3,r4,r5,r6
      real*8 xr,yr,zr,zrr,factor
      real*8 xrr,xrr2,xrr3,xrr4,xrr5
      real*8 yrr,yrr2,yrr3,yrr4,yrr5
      real*8 rpi(13),rpk(13)
      real*8 gt(3),gmi(3,3),gmj(3,3)
      real*8 t2(40,13),t2m1(13),m2t2(13)
      real*8 m2t3x(13),m2t3y(13),m2t3z(13)
      real*8 bx(0:5,0:5),by(11,0:5,0:1),bz(11,0:1)
c
c
c     zeroth order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 1) then
         bz(1,0) = c(1,0,0)
         by(1,0,0) = c(1,0,0) * bz(1,0)
         bx(0,0) = c(1,0,0)
c
c     first order coefficients to speed the T2 calculation
c
         r2 = r * r
         zrr = zr / r
         bz(3,0) = c(3,0,0)
         bz(1,1) = c(1,1,1) * zrr
         yrr = yr / r
         by(3,0,0) = c(3,0,0) * bz(3,0)
         by(1,0,1) = c(1,0,0) * bz(1,1)
         by(1,1,0) = c(1,1,1) * bz(3,0) * yrr
         xrr = xr / r
         bx(1,1) = c(1,1,1) * xrr
      end if
c
c     second order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 4) then
         r3 = r2 * r
         bz(5,0) = c(5,0,0)
         bz(3,1) = c(3,1,1) * zrr
         yrr2 = yrr * yrr
         by(5,0,0) = c(5,0,0) * bz(5,0)
         by(3,0,1) = c(3,0,0) * bz(3,1)
         by(3,1,0) = c(3,1,1) * bz(5,0) * yrr
         by(1,1,1) = c(1,1,1) * bz(3,1) * yrr
         by(1,2,0) = c(1,2,0)*bz(3,0) + c(1,2,2)*bz(5,0)*yrr2
         xrr2 = xrr * xrr
         bx(2,0) = c(1,2,0)
         bx(2,2) = c(1,2,2) * xrr2
c
c     third order coefficients to speed the T2 calculation
c
         r4 = r2 * r2
         bz(7,0) = c(7,0,0)
         bz(5,1) = c(5,1,1) * zrr
         yrr3 = yrr2 * yrr
         by(7,0,0) = c(7,0,0)*bz(7,0)
         by(5,0,1) = c(5,0,0)*bz(5,1)
         by(5,1,0) = c(5,1,1)*bz(7,0)*yrr
         by(3,1,1) = c(3,1,1)*bz(5,1)*yrr
         by(3,2,0) = c(3,2,0)*bz(5,0) + c(3,2,2)*bz(7,0)*yrr2
         by(1,2,1) = c(1,2,0)*bz(3,1) + c(1,2,2)*bz(5,1)*yrr2
         by(1,3,0) = c(1,3,1)*bz(5,0)*yrr + c(1,3,3)*bz(7,0)*yrr3
         xrr3 = xrr2 * xrr
         bx(3,1) = c(1,3,1) * xrr
         bx(3,3) = c(1,3,3) * xrr3
      end if
c
c     fourth order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 13) then
         r5 = r2 * r3
         bz(9,0) = c(9,0,0)
         bz(7,1) = c(7,1,1) * zrr
         yrr4 = yrr2 * yrr2
         by(9,0,0) = c(9,0,0)*bz(9,0)
         by(7,0,1) = c(7,0,0)*bz(7,1)
         by(7,1,0) = c(7,1,1)*bz(9,0)*yrr
         by(5,1,1) = c(5,1,1)*bz(7,1)*yrr
         by(5,2,0) = c(5,2,0)*bz(7,0) + c(5,2,2)*bz(9,0)*yrr2
         by(3,2,1) = c(3,2,0)*bz(5,1) + c(3,2,2)*bz(7,1)*yrr2
         by(3,3,0) = c(3,3,1)*bz(7,0)*yrr + c(3,3,3)*bz(9,0)*yrr3
         by(1,3,1) = c(1,3,1)*bz(5,1)*yrr + c(1,3,3)*bz(7,1)*yrr3
         by(1,4,0) = c(1,4,0)*bz(5,0) + c(1,4,2)*bz(7,0)*yrr2
     &                  + c(1,4,4)*bz(9,0)*yrr4
         xrr4 = xrr2 * xrr2
         bx(4,0) = c(1,4,0)
         bx(4,2) = c(1,4,2) * xrr2
         bx(4,4) = c(1,4,4) * xrr4
c
c     fifth order coefficients to speed the T2 calculation
c
         r6 = r3 * r3
         bz(11,0) = c(11,0,0)
         bz(9,1) = c(9,1,1) * zrr
         yrr5 = yrr3 * yrr2
         by(11,0,0) = c(11,0,0)*bz(11,0)
         by(9,0,1) = c(9,0,0)*bz(9,1)
         by(9,1,0) = c(9,1,1)*bz(11,0)*yrr
         by(7,1,1) = c(7,1,1)*bz(9,1)*yrr
         by(7,2,0) = c(7,2,0)*bz(9,0) + c(7,2,2)*bz(11,0)*yrr2
         by(5,2,1) = c(5,2,0)*bz(7,1) + c(5,2,2)*bz(9,1)*yrr2
         by(5,3,0) = c(5,3,1)*bz(9,0)*yrr + c(5,3,3)*bz(11,0)*yrr3
         by(3,3,1) = c(3,3,1)*bz(7,1)*yrr + c(3,3,3)*bz(9,1)*yrr3
         by(3,4,0) = c(3,4,0)*bz(7,0) + c(3,4,2)*bz(9,0)*yrr2
     &                  + c(3,4,4)*bz(11,0)*yrr4
         by(1,4,1) = c(1,4,0)*bz(5,1) + c(1,4,2)*bz(7,1)*yrr2
     &                  + c(1,4,4)*bz(9,1)*yrr4
         by(1,5,0) = c(1,5,1)*bz(7,0)*yrr + c(1,5,3)*bz(9,0)*yrr3
     &                  + c(1,5,5)*bz(11,0)*yrr5
         xrr5 = xrr3 * xrr2
         bx(5,1) = c(1,5,1) * xrr
         bx(5,3) = c(1,5,3) * xrr3
         bx(5,5) = c(1,5,5) * xrr5
      end if
c
c     zeroth order T2 matrix elements
c
      if (mdqsiz .ge. 1) then
         t2(1,1) = t2matrix (0,0,0,r,bx,by)
c
c     first order T2 matrix elements
c
         t2(2,1) = t2matrix (0,0,1,r2,bx,by)
         t2(3,1) = t2matrix (0,1,0,r2,bx,by)
         t2(4,1) = t2matrix (1,0,0,r2,bx,by)
         t2(1,2) = -t2(2,1)
         t2(1,3) = -t2(3,1)
         t2(1,4) = -t2(4,1)
      end if
c
c     second order T2 matrix elements
c
      if (mdqsiz .ge. 4) then
         t2(2,2) = -t2matrix (0,0,2,r3,bx,by)
         t2(3,2) = -t2matrix (0,1,1,r3,bx,by)
         t2(4,2) = -t2matrix (1,0,1,r3,bx,by)
         t2(3,3) = -t2matrix (0,2,0,r3,bx,by)
         t2(4,3) = -t2matrix (1,1,0,r3,bx,by)
         t2(4,4) = -t2(2,2) - t2(3,3)
         t2(2,3) = t2(3,2)
         t2(2,4) = t2(4,2)
         t2(3,4) = t2(4,3)
         k = 5
         do i = 2, 4
            do j = 2, 4
               t2(1,k) = -t2(i,j)
               t2(k,1) = t2(1,k)
               k = k + 1
            end do
         end do
c
c     third order T2 matrix elements
c
         t2(2,5) = t2matrix (0,0,3,r4,bx,by)
         t2(3,5) = t2matrix (0,1,2,r4,bx,by)
         t2(4,5) = t2matrix (1,0,2,r4,bx,by)
         t2(3,6) = t2matrix (0,2,1,r4,bx,by)
         t2(4,6) = t2matrix (1,1,1,r4,bx,by)
         t2(4,7) = -t2(2,5) - t2(3,6)
         t2(2,6) = t2(3,5)
         t2(2,7) = t2(4,5)
         t2(3,7) = t2(4,6)
         t2(2,8) = t2(2,6)
         t2(2,9) = t2(3,6)
         t2(2,10) = t2(4,6)
         t2(3,9) = t2matrix (0,3,0,r4,bx,by)
         t2(3,10) = t2matrix (1,2,0,r4,bx,by)
         t2(4,10) = -t2(2,8) - t2(3,9)
         t2(3,8) = t2(2,9)
         t2(4,8) = t2(2,10)
         t2(4,9) = t2(3,10)
         t2(2,11) = t2(2,7)
         t2(2,12) = t2(3,7)
         t2(2,13) = t2(4,7)
         t2(3,12) = t2(3,10)
         t2(3,13) = t2(4,10)
         t2(4,13) = -t2(2,11) - t2(3,12)
         t2(3,11) = t2(2,12)
         t2(4,11) = t2(2,13)
         t2(4,12) = t2(3,13)
         do i = 5, 13
            do j = 2, 4
               t2(i,j) = -t2(j,i)
            end do
         end do
      end if
c
c     fourth order T2 matrix elements
c
      if (mdqsiz .ge. 13) then
         t2(5,5) = t2matrix (0,0,4,r5,bx,by)
         t2(6,5) = t2matrix (0,1,3,r5,bx,by)
         t2(7,5) = t2matrix (1,0,3,r5,bx,by)
         t2(6,6) = t2matrix (0,2,2,r5,bx,by)
         t2(7,6) = t2matrix (1,1,2,r5,bx,by)
         t2(7,7) = -t2(5,5) - t2(6,6)
         t2(8,5) = t2(6,5)
         t2(9,5) = t2(6,6)
         t2(10,5) = t2(7,6)
         t2(9,6) = t2matrix (0,3,1,r5,bx,by)
         t2(10,6) = t2matrix (1,2,1,r5,bx,by)
         t2(10,7) = -t2(8,5) - t2(9,6)
         t2(8,6) = t2(9,5)
         t2(8,7) = t2(10,5)
         t2(9,7) = t2(10,6)
         t2(11,5) = t2(7,5)
         t2(12,5) = t2(7,6)
         t2(13,5) = t2(7,7)
         t2(12,6) = t2(10,6)
         t2(13,6) = t2(10,7)
         t2(13,7) = -t2(11,5) - t2(12,6)
         t2(11,6) = t2(12,5)
         t2(11,7) = t2(13,5)
         t2(12,7) = t2(13,6)
         t2(8,8) = t2(8,6)
         t2(9,8) = t2(9,6)
         t2(10,8) = t2(10,6)
         t2(9,9) = t2matrix (0,4,0,r5,bx,by)
         t2(10,9) = t2matrix (1,3,0,r5,bx,by)
         t2(10,10) = -t2(8,8) - t2(9,9)
         t2(11,8) = t2(11,6)
         t2(12,8) = t2(12,6)
         t2(13,8) = t2(13,6)
         t2(12,9) = t2(10,9)
         t2(13,9) = t2(10,10)
         t2(13,10) = -t2(11,8) - t2(12,9)
         t2(11,9) = t2(12,8)
         t2(11,10) = t2(13,8)
         t2(12,10) = t2(13,9)
         t2(11,11) = t2(11,7)
         t2(12,11) = t2(12,7)
         t2(13,11) = t2(13,7)
         t2(12,12) = t2(12,10)
         t2(13,12) = t2(13,10)
         t2(13,13) = -t2(11,11) - t2(12,12)
         do i = 5, 12
            do j = i+1, 13
               t2(i,j) = t2(j,i)
            end do
         end do
c
c     first block of fifth order T2 matrix elements
c
         k = 14
         do i = 2, 4
            do j = 5, 13
               t2(k,1) = t2(i,j)
               k = k + 1
            end do
         end do
         k = 14
         do i = 5, 13 ,3
            do j = 5, 13
               t2(k,2) = -t2(i,j)
               t2(k,3) = -t2(i+1,j)
               t2(k,4) = -t2(i+2,j)
               k = k + 1
            end do
         end do
         t2(14,5) = t2matrix (0,0,5,r6,bx,by)
         t2(15,5) = t2matrix (0,1,4,r6,bx,by)
         t2(16,5) = t2matrix (1,0,4,r6,bx,by)
         t2(15,6) = t2matrix (0,2,3,r6,bx,by)
         t2(16,6) = t2matrix (1,1,3,r6,bx,by)
         t2(16,7) = -t2(14,5) - t2(15,6)
         t2(17,5) = t2(15,5)
         t2(18,5) = t2(15,6)
         t2(19,5) = t2(16,6)
         t2(18,6) = t2matrix (0,3,2,r6,bx,by)
         t2(19,6) = t2matrix (1,2,2,r6,bx,by)
         t2(19,7) = -t2(17,5) - t2(18,6)
         t2(17,6) = t2(18,5)
         t2(17,7) = t2(19,5)
         t2(18,7) = t2(19,6)
         t2(17,8) = t2(17,6)
         t2(18,8) = t2(18,6)
         t2(19,8) = t2(19,6)
         t2(18,9) = t2matrix (0,4,1,r6,bx,by)
         t2(19,9) = t2matrix (1,3,1,r6,bx,by)
         t2(19,10) = -t2(17,8) - t2(18,9)
         t2(20,5) = t2(16,5)
         t2(21,5) = t2(16,6)
         t2(22,5) = t2(16,7)
         t2(21,6) = t2(19,6)
         t2(22,6) = t2(19,7)
         t2(22,7) = -t2(20,5) - t2(21,6)
         t2(20,6) = t2(21,5)
         t2(20,7) = t2(22,5)
         t2(21,7) = t2(22,6)
         t2(20,8) = t2(17,7)
         t2(21,8) = t2(18,7)
         t2(22,8) = t2(19,7)
         t2(21,9) = t2(19,9)
         t2(22,9) = t2(19,10)
         t2(22,10) = -t2(20,8) - t2(21,9)
         t2(20,9) = t2(21,8)
         t2(20,10) = t2(22,8)
         t2(21,10) = t2(22,9)
         t2(20,11) = t2(20,7)
         t2(21,11) = t2(21,7)
         t2(22,11) = t2(22,7)
         t2(21,12) = t2(19,10)
         t2(22,12) = t2(22,10)
         t2(22,13) = -t2(20,11) - t2(21,12)
         do i = 14, 21
            do j = i-8, 13
               k = j + 9
               m = i - 9
               t2(i,j) = t2(k,m)
            end do
         end do
c
c     second block of fifth order T2 matrix elements
c
         t2(23,5) = t2(17,5)
         t2(24,5) = t2(18,5)
         t2(25,5) = t2(19,5)
         t2(24,6) = t2(18,6)
         t2(25,6) = t2(19,6)
         t2(25,7) = t2(19,7)

         do i = 8, 13
            do j= 17, 19
               k = j + 12
               m = i - 18
               t2(k,m) = t2(i,j)
            end do
         end do
         t2(26,5) = t2(24,5)
         t2(27,5) = t2(24,6)
         t2(28,5) = t2(25,6)
         t2(27,6) = t2(18,9)
         t2(28,6) = t2(19,9)
         t2(28,7) = -t2(26,5) - t2(27,6)
         t2(26,6) = t2(27,5)
         t2(26,7) = t2(28,5)
         t2(27,7) = t2(28,6)
         t2(26,8) = t2(26,6)
         t2(27,8) = t2(27,6)
         t2(28,8) = t2(28,6)
         t2(27,9) = t2matrix (0,5,0,r6,bx,by)
         t2(28,9) = t2matrix (1,4,0,r6,bx,by)
         t2(28,10) = -t2(26,8) - t2(27,9)
         t2(29,5) = t2(25,5)
         t2(30,5) = t2(25,6)
         t2(31,5) = t2(25,7)
         t2(30,6) = t2(28,6)
         t2(31,6) = t2(28,7)
         t2(31,7) = -t2(29,5) - t2(30,6)
         t2(29,6) = t2(30,5)
         t2(29,7) = t2(31,5)
         t2(30,7) = t2(31,6)
         t2(29,8) = t2(26,7)
         t2(30,8) = t2(27,7)
         t2(31,8) = t2(28,7)
         t2(30,9) = t2(28,9)
         t2(31,9) = t2(28,10)
         t2(31,10) = -t2(29,8) - t2(30,9)
         t2(29,9) = t2(30,8)
         t2(29,10) = t2(31,8)
         t2(30,10) = t2(31,9)
         t2(29,11) = t2(29,7)
         t2(30,11) = t2(30,7)
         t2(31,11) = t2(31,7)
         t2(30,12) = t2(30,10)
         t2(31,12) = t2(31,10)
         t2(31,13) = -t2(29,11) - t2(30,12)
         do i = 23, 30
            do j = i-17, 13
               k = j + 18
               m = i - 18
               t2(i,j) = t2(k,m)
            end do
         end do
c
c     third block of fifth order T2 matrix elements
c
         do i = 32, 40
            do j = 5, 7
               k = i - 18
               m = j + 6
               t2(i,j) = t2(k,m)
            end do
         end do
         do i = 32, 40
            do j = 8, 10
               k = i - 9
               m = j + 3
               t2(i,j) = t2(k,m)
            end do
         end do
         do i = 32, 34
            do j = 11, 13
               k = i - 12
               t2(i,j) = t2(k,j)
            end do
         end do
         do i = 35, 37
            do j = 11, 13
               k = i - 6
               t2(i,j) = t2(k,j)
            end do
         end do
         t2(38,11) = t2(20,13)
         t2(39,11) = t2(21,13)
         t2(40,11) = t2(22,13)
         t2(39,12) = t2(39,10)
         t2(40,12) = t2(40,10)
         t2(40,13) = -t2(38,11) - t2(39,12)
         t2(38,12) = t2(39,11)
         t2(38,13) = t2(40,11)
         t2(39,13) = t2(40,12)
      end if
c
c     compute interaction energy between the two multipole sites
c
      factor = electric / dielec
      do i = 1, mdqsiz
         m2t2(i) = 0.0d0
         do j = 1, mdqsiz
            m2t2(i) = m2t2(i) + rpk(j)*t2(j,i)
         end do
      end do
      emdqik1 = 0.0d0
      do i = 1, mdqsiz
         emdqik1 = emdqik1 + m2t2(i)*rpi(i)
      end do
      emdqik1 = factor * emdqik1
c
c     get the (dM2/dx)*T*M1 derivative terms
c
      do i = 1, mdqsiz
         t2m1(i) = 0.0d0
         do j = 1, mdqsiz
            t2m1(i) = t2m1(i) + t2(i,j)*rpi(j)
         end do
      end do
      do i = 1, 3
         do j = 1, 3
            gmi(j,i) = 0.0d0
            gmj(j,i) = 0.0d0
            do k = 1, mdqsiz
               gmi(j,i) = gmi(j,i) + m2t2(k)*dpole(k,j,i,ii)
               gmj(j,i) = gmj(j,i) + dpole(k,j,i,kk)*t2m1(k)
            end do
            gmi(j,i) = factor * gmi(j,i)
            gmj(j,i) = factor * gmj(j,i)
         end do
      end do
c
c     get the M2*(dT/dx)*M1 derivative terms
c
      do i = 1, mdqsiz
         m2t3x(i) = 0.0d0
         m2t3y(i) = 0.0d0
         m2t3z(i) = 0.0d0
         do j = 1, mdqsiz
            m2t3x(i) = m2t3x(i) + rpk(j)*t2(3*j-1,i)
            m2t3y(i) = m2t3y(i) + rpk(j)*t2(3*j,i)
            m2t3z(i) = m2t3z(i) + rpk(j)*t2(3*j+1,i)
         end do
      end do
      do i = 1, 3
         gt(i) = 0.0d0
      end do
      do i = 1, mdqsiz
         gt(1) = gt(1) + m2t3x(i)*rpi(i)
         gt(2) = gt(2) + m2t3y(i)*rpi(i)
         gt(3) = gt(3) + m2t3z(i)*rpi(i)
      end do
      do i = 1, 3
         gt(i) = factor * gt(i)
      end do
      return
      end
c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1994  by  Jay William Ponder  ##
c     ###################################################
c
c     ###########################################################
c     ##                                                       ##
c     ##  subroutine empole3  --  multipole energy & analysis  ##
c     ##                                                       ##
c     ###########################################################
c
c
      subroutine empole3
      implicit none
      include 'sizes.i'
      include 'analyz.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'electr.i'
      include 'energi.i'
      include 'inform.i'
      include 'iounit.i'
      include 'moment.i'
      include 'mpole.i'
      include 'shunt.i'
      include 'units.i'
      include 'usage.i'
      integer i,j,k,m,skip(maxatm)
      integer ii,iz,ix,kk,kz,kx
      real*8 e,emdqik,a(3,3)
      real*8 shift,taper,trans
      real*8 f,fi,fik,xr,yr,zr
      real*8 r,r2,r3,r4,r5,r6,r7
      real*8 rpi(13),rpk(13)
      real*8 weight,xcenter,ycenter,zcenter
      real*8 totchg,xsum,ysum,zsum
      logical iuse,kuse,huge,header
c
c
c     zero out the multipole interaction energy and partitioning
c
      em = 0.0d0
      do i = 1, n
         aem(i) = 0.0d0
      end do
      nmdq = 0
      header = .true.
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set the switching function coefficients
c
      f = electric / dielec
      call switch ('CHARGE')
c
c     rotate the multipole components into the global frame
c
      do i = 1, npole
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c     find the center of mass of the total system
c
      weight = 0.0d0
      xcenter = 0.0d0
      ycenter = 0.0d0
      zcenter = 0.0d0
      do i = 1, n
         weight = weight + mass(i)
         xcenter = xcenter + x(i)*mass(i)
         ycenter = ycenter + y(i)*mass(i)
         zcenter = zcenter + z(i)*mass(i)
      end do
      xcenter = xcenter / weight
      ycenter = ycenter / weight
      zcenter = zcenter / weight
c
c     get net charge and dipole components relative to center of mass
c
      totchg = 0.0d0
      xsum = 0.0d0
      ysum = 0.0d0
      zsum = 0.0d0
      do ii = 1, npole
         i = ipole(ii)
         totchg = totchg + rpole(1,ii)
         xsum = xsum + (x(i)-xcenter)*rpole(1,ii)
         ysum = ysum + (y(i)-ycenter)*rpole(1,ii)
         zsum = zsum + (z(i)-zcenter)*rpole(1,ii)
         xsum = xsum + rpole(2,ii)
         ysum = ysum + rpole(3,ii)
         zsum = zsum + rpole(4,ii)
      end do
      netchg = netchg + totchg
      xdipole = xdipole + debye*xsum
      ydipole = ydipole + debye*ysum
      zdipole = zdipole + debye*zsum
c
c     compute and partition the multipole interaction energy
c
      do ii = 1, npole-1
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, n12(i)
            skip(i12(j,i)) = i
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = -i
         end do
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii+1, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               if (skip(k) .ne. i) then
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  if (use_image)  call image (xr,yr,zr,0)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)
                     if (skip(k) .eq. -i) then
                        fik = fik / chgscale
                        e = e / chgscale
                     end if
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if
                     nmdq = nmdq + 1
                     em = em + e
                     aem(i) = aem(i) + 0.5d0*e
                     aem(k) = aem(k) + 0.5d0*e
c
c     print a warning if the energy of this interaction is large
c
                     huge = (abs(e) .gt. 100.0d0)
                     if (debug .or. (verbose.and.huge)) then
                        if (header) then
                           header = .false.
                           write (iout,10)
   10                      format (/,' Individual Multipole-Multipole',
     &                                ' Interactions :',
     &                             //,' Type',11x,'Atom Names',
     &                                15x,'Monopoles',4x,'Distance',
     &                                5x,'Energy',/)
                        end if
                        write (iout,20)  i,name(i),k,name(k),
     &                                   pole(1,ii),pole(1,kk),r,e
   20                   format (' M-Pole   ',i5,'-',a3,1x,i5,'-',
     &                             a3,8x,2f7.2,f10.4,f12.4)
                     end if
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, npole
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               do m = 1, ncell
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  call image (xr,yr,zr,m)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if
                     nmdq = nmdq + 1
                     if (i .eq. k) then
                        em = em + 0.5d0*e
                        aem(i) = aem(i) + 0.5d0*e
                     else
                        em = em + e
                        aem(i) = aem(i) + 0.5d0*e
                        aem(k) = aem(k) + 0.5d0*e
                     end if
c
c     print a warning if the energy of this interaction is large
c
                     huge = (abs(e) .gt. 100.0d0)
                     if (debug .or. (verbose.and.huge)) then
                        if (header) then
                           header = .false.
                           write (iout,30)
   30                      format (/,' Individual Multipole-Multipole',
     &                                ' Interactions :',
     &                             //,' Type',11x,'Atom Names',
     &                                15x,'Monopoles',4x,'Distance',
     &                                5x,'Energy',/)
                        end if
                        write (iout,40)  i,name(i),k,name(k),
     &                                   pole(1,ii),pole(1,kk),r,e
   40                   format (' M-Pole   ',i5,'-',a3,1x,i5,'-',
     &                             a3,' (XTAL) ',2f7.2,f10.4,f12.4)
                     end if
                  end if
               end do
            end if
         end do
      end do
      return
      end
