c
c
c     #################################################################
c     ##  COPYRIGHT (C)  1995  by  Yong Kong and Jay William Ponder  ##
c     ##                     All Rights Reserved                     ##
c     #################################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  program radial  --  water radial distributions functions  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "radial" finds the O-O, O-H and H-H radial distribution functions
c     for bulk water from a set of stored molecular dynamics frames
c
c
      program radial
      implicit none
      include 'sizes.i'
      include 'argue.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bound.i'
      include 'boxes.i'
      include 'cell.i'
      include 'files.i'
      include 'iounit.i'
      include 'math.i'
      include 'molcul.i'
      integer maxbin
      parameter (maxbin=10000)
      integer i,j,k,iarc,ixyz,irdf,nframe
      integer molj,molk,num_j,num_k
      integer lext,next,last,start,stop,step
      integer nbin,bin,hist(maxbin)
      integer freeunit,trimtext
      real*8 xj,yj,zj,dx,dy,dz
      real*8 rjk,rjksq,delr,rlower,rupper
      real*8 factor,pairs,volume,expect
      real*8 gr(maxbin),gs(maxbin)
      character*1 letter
      character*1 namej,namek
      character*7 ext
      character*60 coordfile
      character*80 record
      logical exist
c
c
c     get the structure and setup any unitcell parameters
c
      call initial
      call getxyz
      call cutoffs
      call molecule
      call unitcell
      call lattice
c
c     get the individual coordinate frames and rdf parameters
c
      if (narg .ge. 1) then
         filename = arg(1)
      else
         write (iout,10)
   10    format (/,' Enter Name of the Coordinate Archive File :  ',$)
         read (input,20)  filename
   20    format (a60)
      end if
      leng = trimtext (filename)
      iarc = freeunit ()
      last = leng
      do i = 1, leng
         letter = filename(i:i)
         if (letter .eq. '/')  last = leng
         if (letter .eq. ']')  last = leng
         if (letter .eq. ':')  last = leng
         if (letter .eq. '.')  last = i - 1
      end do
      leng = min(leng,last)
      start = 0
      stop = 0
      step = 0
      write (iout,30)
   30 format (/,' Numbers of First & Last File and Step',
     &              ' Increment :  ',$)
      read (input,40)  record
   40 format (a80)
      read (record,*,err=50,end=50)  start,stop,step
   50 continue
      if (stop .eq. 0)  stop = start
      if (step .eq. 0)  step = 1
      write (iout,60)
   60 format (/,' Names of First & Second Atoms :  ',$)
      read (input,70)  record
   70 format (a80)
      next = 1
      call getword (record,namej,next)
      call getword (record,namek,next)
      call upcase (namej)
      call upcase (namek)
      delr = 0.0d0
      write (iout,80)
   80 format (/,' Width of Bin [0.05] :  ',$)
      read (input,90)  delr
   90 format (f10.0)
      if (delr .eq. 0.0d0)  delr = 0.05d0
c
c     zero out the distance bins and distribution functions
c
      do i = 1, maxbin
         hist(i) = 0
         gr(i) = 0.0d0
         gs(i) = 0.0d0
      end do
c
c     cycle over coordinate files to generate distance bins
c
      i = start
      nframe = 0
      nbin = 0
      dowhile (i.ge.start .and. i.le.stop)
         ixyz = freeunit ()
         lext = 3
         call numeral (i,ext,lext)
         coordfile = filename(1:leng)//'.'//ext(1:lext)
         call version (coordfile,'old')
         inquire (file=coordfile,exist=exist)
         if (.not.exist .and. i.lt.100) then
            lext = 2
            call numeral (i,ext,lext)
            coordfile = filename(1:leng)//'.'//ext(1:lext)
            call version (coordfile,'old')
            inquire (file=coordfile,exist=exist)
         end if
         if (.not.exist .and. i.lt.10) then
            lext = 1
            call numeral (i,ext,lext)
            coordfile = filename(1:leng)//'.'//ext(1:lext)
            call version (coordfile,'old')
            inquire (file=coordfile,exist=exist)
         end if
         if (exist) then
            open (unit=ixyz,file=coordfile,status='old')
            rewind (unit=ixyz)
            call readxyz (ixyz)
            close (unit=ixyz)
            do j = 1, n
               if (name(j)(1:1) .eq. namej) then
                  xj = x(j)
                  yj = y(j)
                  zj = z(j)
                  molj = molcule(j)
                  do k = 1, n
                     if (name(k)(1:1) .eq. namek) then
                        molk = molcule(k)
                        if (molj .ne. molk) then
                           dx = x(k) - xj
                           dy = y(k) - yj
                           dz = z(k) - zj
                           call image (dx,dy,dz,0)
                           rjk = sqrt(dx*dx + dy*dy + dz*dz)
                           bin = int(rjk/delr) + 1
                           if (bin.le.maxbin .and. rjk.le.xcell2) then
                              hist(bin) = hist(bin) + 1
                              if (bin .gt. nbin)  nbin = bin
                           end if
                        end if
                     end if
                  end do
               end if
            end do
         end if
         i = i + step
         nframe = nframe + 1
      end do
c
c     normalize the distance bins to give radial distribution
c
      write (iout,100)  nframe
  100 format (/,' Number of Coordinate Frames Used :',i8)
      if (namej(1:1) .eq. 'O')  num_j = nmol
      if (namej(1:1) .eq. 'H')  num_j = 2 * nmol
      if (namek(1:1) .eq. 'O')  num_k = nmol
      if (namek(1:1) .eq. 'H')  num_k = 2 * nmol
      pairs = dble(num_j) * dble(num_k) * dble(nframe)
      volume = xbox * ybox * zbox
      factor = (4.0d0/3.0d0) * pi * pairs / volume
      do i = 1, nbin
         rupper = dble(i) * delr
         rlower = rupper - delr
         expect = factor * (rupper**3 - rlower**3)
         gr(i) = dble(hist(i)) / expect
      end do
c
c     find the 5th degree polynomial smoothed distribution function
c
      if (nbin .ge. 5) then
         gs(1) = (69.0d0*gr(1) + 4.0d0*gr(2) - 6.0d0*gr(3)
     &             + 4.0d0*gr(4) - gr(5)) / 70.0d0
         gs(2) = (2.0d0*gr(1) + 27.0d0*gr(2) + 12.0d0*gr(3)
     &             - 8.0d0*gr(4) + 2.0d0*gr(5)) / 35.0d0
         do i = 3, nbin-2
            gs(i) = (-3.0d0*gr(i-2) + 12.0d0*gr(i-1) + 17.0d0*gr(i)
     &                + 12.0d0*gr(i+1) - 3.0d0*gr(i+2)) / 35.0d0
         end do
         gs(nbin-1) = (2.0d0*gr(nbin-4) - 8.0d0*gr(nbin-3)
     &                  + 12.0d0*gr(nbin-2) + 27.0d0*gr(nbin-1)
     &                       + 2.0d0*gr(nbin)) / 35.0d0
         gs(nbin) = (-gr(nbin-4) + 4.0d0*gr(nbin-3) - 6.0d0*gr(nbin-2)
     &                + 4.0d0*gr(nbin-1) + 69.0d0*gr(nbin)) / 70.0d0
         do i = 1, nbin
            gs(i) = max(0.0d0,gs(i))
         end do
      end if
c
c     write out the radial distribution function to an output file
c
      outfile = 'gr.'//namej(1:1)//namek(1:1)
      irdf = freeunit ()
      open (unit=irdf,file=outfile,status='unknown')
      do i = 1, nbin
         write (irdf,110)  i,hist(i),(dble(i)-0.5d0)*delr,gr(i),gs(i)
  110    format (i8,i12,f12.4,2f16.8)
      end do
      close (unit=irdf)
c
c     perform any final tasks before program exit
c
      call final
      end
