c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1993  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  program xtalopt  --  optimize crystal lattice parameters  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "xtalopt" performs an optimization of the six crystal lattice
c     parameters using an optimally conditioned variable metric method
c
c
      program xtalopt
      implicit none
      include 'boxes.i'
      include 'iounit.i'
      include 'output.i'
      integer i,j,nvar,freeunit
      real*8 xtalopt1,minimum
      real*8 grms,grdmin,gnorm
      real*8 xx(6),derivs(6)
      external writeout,xtalopt1
c
c
c     set up the structure and mechanics calculation
c
      call initial
      call getxyz
      call mechanic
c
c     get termination criterion as RMS gradient per atom
c
      write (iout,10)
   10 format (' Enter the RMS Gradient Criterion [0.01] :  ',$)
      read (input,20)  grdmin
   20 format (f20.0)
      if (grdmin .eq. 0.0d0)  grdmin = 0.01d0
c
c     set the initial values of the lattice parameters
c
      nvar = 6
      xx(1) = xbox
      xx(2) = ybox
      xx(3) = zbox
      xx(4) = alpha
      xx(5) = beta
      xx(6) = gamma
c
c     make the call to the optimization routine
c
      coordtype = 'none'
      call vmetric (nvar,xx,minimum,grdmin,xtalopt1,writeout)
c
c     set lattice parameters to their optimized values
c
      xbox = xx(1)
      ybox = xx(2)
      zbox = xx(3)
      alpha = xx(4)
      beta = xx(5)
      gamma = xx(6)
c
c     write out final function value and gradient
c
      minimum = xtalopt1 (xx,derivs)
      gnorm = 0.0d0
      do i = 1, nvar
         gnorm = gnorm + derivs(i)**2
      end do
      gnorm = sqrt(gnorm)
      grms = gnorm / sqrt(dble(nvar))
      if (grms .gt. 0.0001d0) then
         write (iout,30)  minimum,grms,gnorm
   30    format (/,' Final Function Value :',f15.4,
     &           /,' Final RMS Gradient :  ',f15.4,
     &           /,' Final Gradient Norm : ',f15.4)
      else
         write (iout,40)  minimum,grms,gnorm
   40    format (/,' Final Function Value :',f15.4,
     &           /,' Final RMS Gradient :  ',d15.4,
     &           /,' Final Gradient Norm : ',d15.4)
      end if
c
c     write out final values of the lattice parameters
c
      write (iout,50)  xbox,ybox,zbox,alpha,beta,gamma
   50 format (/,' Unit Cell Dimensions :      a    =',f10.4,
     &        /,'                             b    =',f10.4,
     &        /,'                             c    =',f10.4,
     &        /,'                            Alpha =',f10.4,
     &        /,'                            Beta  =',f10.4,
     &        /,'                            Gamma =',f10.4)
c
c     perform any final tasks before program exit
c
      call final
      end
c
c
c     ##########################################################
c     ##                                                      ##
c     ##  function xtalopt1  --  energy/gradient for lattice  ##
c     ##                                                      ##
c     ##########################################################
c
c
      function xtalopt1 (xx,g)
      implicit none
ccccccccccccccccccccccccccccccccccccccccccc
      include 'sizes.i'
      include 'bath.i'
      include 'virial.i'
      real*8 derivs(3,maxatm)
ccccccccccccccccccccccccccccccccccccccccccc
      include 'bound.i'
      include 'boxes.i'
      real*8 xtalopt1,energy
      real*8 e,e0,old,eps
      real*8 xx(6),g(6)
c
c
c     translate optimization variables to lattice parameters
c
      xbox = xx(1)
      ybox = xx(2)
      zbox = xx(3)
      alpha = xx(4)
      beta = xx(5)
      gamma = xx(6)
c
c     compute and store the base potential energy value
c
      call lattice
      use_box = .false.
      e0 = energy ()
      xtalopt1 = e0
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      isobaric = .true.
      call gradient (e0,derivs)
      write (*,98)  virx,viry,virz
   98 format (12x,3f11.4)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     find derivatives with respect to lattice angles
c
      eps = 0.00001d0
      old = xbox
      xbox = xbox + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(1) = (e - e0) / eps
      xbox = old
      old = ybox
      ybox = ybox + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(2) = (e - e0) / eps
      ybox = old
      old = zbox
      zbox = zbox + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(3) = (e - e0) / eps
      zbox = old
c
c     find derivatives with respect to lattice angles
c
      eps = 0.0001d0
      old = alpha
      alpha = alpha + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(4) = (e - e0) / eps
      alpha = old
      old = beta
      beta = beta + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(5) = (e - e0) / eps
      beta = old
      old = gamma
      gamma = gamma + eps
      call lattice
      use_box = .false.
      e = energy ()
      g(6) = (e - e0) / eps
      gamma = old
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      write (*,99)  e0,g(1),g(2),g(3),g(4),g(5),g(6)
   99 format (1x,7f11.4)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      return
      end
