c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     #################################################################
c     ##                                                             ##
c     ##  program dynamic  --  run molecular or stochastic dynamics  ##
c     ##                                                             ##
c     #################################################################
c
c
c     "dynamic" computes a molecular dynamics trajectory in any of
c     several statistical mechanical ensembles with optional periodic
c     boundaries and optional coupling to temperature and pressure baths;
c     alternatively a stochastic dynamics trajectory can be generated
c
c
      program dynamic
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bath.i'
      include 'bond.i'
      include 'bound.i'
      include 'inform.i'
      include 'iounit.i'
      include 'keys.i'
      include 'potent.i'
      include 'solute.i'
      include 'stodyn.i'
      include 'usage.i'
      include 'warp.i'
      integer i,mode,istep,nstep,ndump
      integer period,next,modstep
      real*8 dt,dt2,dt_2,dt2_2
      real*8 dt_8,dt2_8,dt_dump
      real*8 ratio,def_start,def_stop
      character*10 integrate
      character*20 keyword
      character*80 record,string
      logical exist,query
c
c
c     set up the structure and mechanics calculation
c
      call initial
      call getxyz
      call mechanic
c
c     set some default parameters for the dynamics run
c
      kelvin = 0.0d0
      atmsph = 0.0d0
      isothermal = .false.
      isobaric = .false.
      integrate = 'BEEMAN'
      tautemp = 0.1d0
      taupres = 2.0d0
      compress = 0.000046d0
      friction = 91.0d0
c
c     check for keywords containing any altered parameters
c
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:10) .eq. 'INTEGRATE ') then
            call getword (record,integrate,next)
            call upcase (integrate)
         else if (keyword(1:16) .eq. 'TAU-TEMPERATURE ') then
            read (record(next:80),*,err=10)  tautemp
            tautemp = max(tautemp,dt)
         else if (keyword(1:13) .eq. 'TAU-PRESSURE ') then
            read (record(next:80),*,err=10)  taupres
            taupres = max(taupres,dt)
         else if (keyword(1:9) .eq. 'COMPRESS ') then
            read (record(next:80),*,err=10)  compress
         else if (keyword(1:9) .eq. 'FRICTION ') then
            read (record(next:80),*,err=10)  friction
         end if
   10    continue
      end do
c
c     set some parameters for a stochastic dynamics run
c
      if (integrate .eq. 'STOCHASTIC') then
         use_bounds = .false.
         tautemp = max(0.5d0/friction,dt)
      end if
c
c     initialize the simulation length as number of time steps
c
      query = .true.
      call nextarg (string,exist)
      if (exist) then
         read (string,*,err=20,end=20)  nstep
         query = .false.
      end if
   20 continue
      if (query) then
         write (iout,30)
   30    format (/,' Enter the Number of Dynamics Steps to be',
     &              ' Taken :  ',$)
         read (input,40)  nstep
   40    format (i10)
      end if
c
c     get the length of the dynamics time step in picoseconds
c
      dt = -1.0d0
      call nextarg (string,exist)
      if (exist)  read (string,*,err=50,end=50)  dt
   50 continue
      if (dt .le. 0.0d0) then
         write (iout,60)
   60    format (/,' Enter the Time Step Length in Femtoseconds',
     &              ' [1.0] :  ',$)
         read (input,70)  dt
   70    format (f20.0)
      end if
      if (dt .le. 0.0d0)  dt = 1.0d0
      dt = 1.0d-3 * dt
c
c     set the time between trajectory snapshot coordinate dumps
c
      dt_dump = -1.0d0
      call nextarg (string,exist)
      if (exist)  read (string,*,err=80,end=80)  dt_dump
   80 continue
      if (dt_dump .le. 0.0d0) then
         write (iout,90)
   90    format (/,' Enter Time between Dumps in Picoseconds',
     &              ' [0.1] :  ',$)
         read (input,100)  dt_dump
  100    format (f20.0)
      end if
      if (dt_dump .le. 0.0d0)  dt_dump = 0.1d0
c
c     rate of deformation change for potential surface smoothing
c
      if (use_deform) then
         def_stop = -1.0d0
         call nextarg (string,exist)
         if (exist)  read (string,*,err=110,end=110)  def_stop
  110    continue
         if (def_stop .le. 0.0d0) then
            write (iout,120)
  120       format (/,' Enter Final Desired Deformation Parameter',
     &                 ' [0.0] :  ',$)
            read (input,130)  def_stop
  130       format (f20.0)
         end if
         if (def_stop .le. 0.0d0)  def_stop = 0.0d0
         def_start = deform - def_stop
         if (def_start .le. 0.0d0)  def_start = 0.0d0
      end if
c
c     get choice of statistical ensemble for periodic system
c
      if (use_bounds) then
         mode = 0
         call nextarg (string,exist)
         if (exist)  read (string,*,err=140,end=140)  mode
  140    continue
         if (mode.lt.1 .or. mode.gt.4) then
            write (iout,150)
  150       format (/,' Possible Statistical Mechanical Ensembles :',
     &              //,'   (1) Microcanonical (NVE)',
     &              /,'   (2) Canonical (NVT)',
     &              /,'   (3) Isoenthalpic-Isobaric (NPH)',
     &              /,'   (4) Isothermal-Isobaric (NPT)',
     &              /,' Enter the Number of the Desired Choice',
     &                 ' [1] :  ',$)
            read (input,160)  mode
  160       format (i10)
         end if
         if (mode.eq.2 .or. mode.eq.4) then
            isothermal = .true.
            kelvin = -1.0d0
            call nextarg (string,exist)
            if (exist)  read (string,*,err=170,end=170)  kelvin
  170       continue
            if (kelvin .le. 0.0d0) then
               write (iout,180)
  180          format (/,' Enter the Desired Temperature in Degrees',
     &                    ' K [300] :  ',$)
               read (input,190)  kelvin
  190          format (f20.0)
            end if
            if (kelvin .le. 0.0d0)  kelvin = 300.0d0
         end if
         if (mode.eq.3 .or. mode.eq.4) then
            isobaric = .true.
            atmsph = -1.0d0
            call nextarg (string,exist)
            if (exist)  read (string,*,err=200,end=200)  kelvin
  200       continue
            if (atmsph .le. 0.0d0) then
               write (iout,210)
  210          format (/,' Enter the Desired Pressure in Atm',
     &                    ' [1.0] :  ',$)
               read (input,220)  atmsph
  220          format (f20.0)
            end if
            if (atmsph .le. 0.0d0)  atmsph = 1.0d0
         end if
      end if
c
c     get desired temperature for nonperiodic system
c
      if (.not. use_bounds) then
         isothermal = .true.
         kelvin = -1.0d0
         call nextarg (string,exist)
         if (exist)  read (string,*,err=230,end=230)  kelvin
  230    continue
         if (kelvin .le. 0.0d0) then
            write (iout,240)
  240       format (/,' Enter the Desired Temperature in Degrees',
     &                 ' K [300] :  ',$)
            read (input,250)  kelvin
  250       format (f20.0)
         end if
         if (kelvin .le. 0.0d0)  kelvin = 300.0d0
      end if
c
c     set some time values for the dynamics integration
c
      dt2 = dt * dt
      dt_2 = dt / 2.0d0
      dt_8 = dt / 8.0d0
      dt2_2 = dt2 / 2.0d0
      dt2_8 = dt2 / 8.0d0
      ndump = nint(dt_dump/dt)
c
c     make sure all atoms have a mass of at least one
c
      do i = 1, n
         if (use(i)) then
            mass(i) = max(mass(i),1.0d0)
         end if
      end do
c
c     set initial conditions and check various options
c
      call mdinit
c
c     initialize any constraints for the rattle algorithm
c
      call shakeup
c
c     set Born radii update frequency for GB/SA solvation
c
      if (use_gbsa) then
         if (reborn .eq. 1) then
            call born
            reborn = 100
         end if
      end if
c
c     print out a header line for the dynamics computation
c
      if (integrate .eq. 'VERLET') then
         write (iout,260)
  260    format (/,' Molecular Dynamics Trajectory via',
     &              ' Velocity Verlet Algorithm')
      else if (integrate .eq. 'STOCHASTIC') then
         write (iout,270)
  270    format (/,' Stochastic Dynamics Trajectory via',
     &              ' Velocity Verlet Algorithm')
      else
         write (iout,280)
  280    format (/,' Molecular Dynamics Trajectory via',
     &              ' Modified Beeman Algorithm')
      end if
c
c     take the desired total number of dynamics steps
c
      period = 100
      do istep = 1, nstep
         modstep = mod(istep,period)
         if (verbose .and. modstep.eq.1) then
            if (integrate .eq. 'STOCHASTIC') then
               write (iout,290)
  290          format (/,'    SD Step      E Total   E Potential',
     &                   '   E Kinetic       Temp',/)
            else if (isobaric) then
               write (iout,300)
  300          format (/,'    MD Step      E Total   E Potential',
     &                   '   E Kinetic       Temp       Pres',/)
            else
               write (iout,310)
  310          format (/,'    MD Step      E Total   E Potential',
     &                   '   E Kinetic       Temp',/)
            end if
         end if
         if (use_deform) then
            ratio = (1.0d0-dble(istep)/dble(nstep))**3
            deform = def_stop + ratio*def_start
         end if
         if (integrate .eq. 'VERLET') then
            call verlet (istep,dt,dt_2,dt2_2,ndump)
         else if (integrate .eq. 'STOCHASTIC') then
            call sdstep (istep,dt,ndump)
         else
            call beeman (istep,dt,dt_8,dt2_8,ndump)
         end if
         if (nuse.eq.n .and. modstep.eq.0)  call mdrest
      end do
c
c     perform any final tasks before program exit
c
      call final
      end
