c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1999  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine ewald2  --  atom-wise Ewald summation Hessian  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "ewald2" calculates second derivatives of the Ewald summation
c     interaction energy for a single atom
c
c     note this version does not incorporate the reciprocal space
c     contribution to the Hessian calculation
c
c
      subroutine ewald2 (i)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'cell.i'
      include 'charge.i'
      include 'chgpot.i'
      include 'couple.i'
      include 'cutoff.i'
      include 'group.i'
      include 'hessn.i'
      include 'math.i'
      include 'pme.i'
      include 'units.i'
      integer i,j,k,kk
      integer jcell,skip(maxatm)
      real*8 fi,fik,fgrp,cut2
      real*8 de,d2e,d2edx,d2edy,d2edz
      real*8 xi,yi,zi,xr,yr,zr,term(3,3)
      real*8 r,r2,rew,erfc,erfterm
      real*8 expterm,scaleterm
      logical proceed
c
c
c     first see if the atom of interest carries a charge
c
      do k = 1, nion
         if (iion(k) .eq. i) then
            fi = electric * pchg(k) / dielec
            cut2 = pmecut * pmecut
            scaleterm = 1.0d0 - 1.0d0/chgscale
            goto 10
         end if
      end do
      return
   10 continue
c
c     store the coordinates of the atom of interest
c
      xi = x(i)
      yi = y(i)
      zi = z(i)
c
c     set the list of atoms to be skipped
c
      do k = 1, n
         skip(k) = 0
      end do
      skip(i) = i
      do j = 1, n12(i)
         skip(i12(j,i)) = i * chg12use
      end do
      do j = 1, n13(i)
         skip(i13(j,i)) = i * chg13use
      end do
      do j = 1, n14(i)
         skip(i14(j,i)) = i * chg14use
      end do
c
c     calculate the direct space Ewald interaction Hessian elements
c
      do kk = 1, nion
         k = iion(kk)
         proceed = .true.
         if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
         if (proceed)  proceed = (k .ne. i)
c
c     compute the energy contribution for this interaction
c
         if (proceed) then
            xr = xi - x(k)
            yr = yi - y(k)
            zr = zi - z(k)
            if (use_image)  call image (xr,yr,zr,0)
            r2 = xr*xr + yr*yr + zr*zr
            if (r2 .le. cut2) then
               r = sqrt(r2)
               fik = fi * pchg(kk)
               rew = aewald * r
               erfterm = erfc (rew)
               expterm = exp(-rew**2)
c
c     compute chain rule terms for Hessian matrix elements
c
               de = -fik * (erfterm/r2
     &                 + (2.0d0*aewald/sqrtpi)*expterm/r)
               d2e = -2.0d0*de/r + (4.0d0*fik*aewald**3/sqrtpi)*expterm
c
c     adjust the direct sum terms for neglected interactions
c
               if (skip(k) .eq. i) then
                  de = de + fik/r2
                  d2e = d2e - 2.0d0*fik/(r*r2)
               else if (skip(k) .eq. -i) then
                  de = de + (fik/r2)*scaleterm
                  d2e = d2e - 2.0d0*(fik/(r*r2))*scaleterm
               end if
c
c     scale the interaction based on its group membership
c
               if (use_group) then
                  de = de * fgrp
                  d2e = d2e * fgrp
               end if
c
c     form the individual Hessian element components
c
               de = de / r
               d2e = (d2e-de) / r2
               d2edx = d2e * xr
               d2edy = d2e * yr
               d2edz = d2e * zr
               term(1,1) = d2edx*xr + de
               term(1,2) = d2edx*yr
               term(1,3) = d2edx*zr
               term(2,1) = term(1,2)
               term(2,2) = d2edy*yr + de
               term(2,3) = d2edy*zr
               term(3,1) = term(1,3)
               term(3,2) = term(2,3)
               term(3,3) = d2edz*zr + de
c
c     increment diagonal and non-diagonal Hessian elements
c
               do j = 1, 3
                  hessx(j,i) = hessx(j,i) + term(1,j)
                  hessy(j,i) = hessy(j,i) + term(2,j)
                  hessz(j,i) = hessz(j,i) + term(3,j)
                  hessx(j,k) = hessx(j,k) - term(1,j)
                  hessy(j,k) = hessy(j,k) - term(2,j)
                  hessz(j,k) = hessz(j,k) - term(3,j)
               end do
            end if
         end if
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do kk = 1, nion
         k = iion(kk)
         proceed = .true.
         if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
c
c     compute the energy contribution for this interaction
c
         if (proceed) then
            do jcell = 1, ncell
               xr = xi - x(k)
               yr = yi - y(k)
               zr = zi - z(k)
               call image (xr,yr,zr,jcell)
               r2 = xr*xr + yr*yr + zr*zr
               if (r2 .le. cut2) then
                  r = sqrt(r2)
                  fik = fi * pchg(kk)
                  rew = aewald * r
                  erfterm = erfc (rew)
                  expterm = exp(-rew**2)
c
c     compute chain rule terms for Hessian matrix elements
c
                  de = -fik * (erfterm/r2
     &                    + (2.0d0*aewald/sqrtpi)*expterm/r)
                  d2e = -2.0d0*de/r
     &                     + (4.0d0*fik*aewald**3/sqrtpi)*expterm
c
c     scale the interaction based on its group membership
c
                  if (use_group) then
                     de = de * fgrp
                     d2e = d2e * fgrp
                  end if
c
c     form the individual Hessian element components
c
                  de = de / r
                  d2e = (d2e-de) / r2
                  d2edx = d2e * xr
                  d2edy = d2e * yr
                  d2edz = d2e * zr
                  term(1,1) = d2edx*xr + de
                  term(1,2) = d2edx*yr
                  term(1,3) = d2edx*zr
                  term(2,1) = term(1,2)
                  term(2,2) = d2edy*yr + de
                  term(2,3) = d2edy*zr
                  term(3,1) = term(1,3)
                  term(3,2) = term(2,3)
                  term(3,3) = d2edz*zr + de
c
c     increment diagonal and non-diagonal Hessian elements
c
                  do j = 1, 3
                     hessx(j,i) = hessx(j,i) + term(1,j)
                     hessy(j,i) = hessy(j,i) + term(2,j)
                     hessz(j,i) = hessz(j,i) + term(3,j)
                     hessx(j,k) = hessx(j,k) - term(1,j)
                     hessy(j,k) = hessy(j,k) - term(2,j)
                     hessz(j,k) = hessz(j,k) - term(3,j)
                  end do
               end if
            end do
         end if
      end do
      return
      end
