c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1999  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine ewald3  --  Ewald summation energy & analysis  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "ewald3" calculates the Ewald summation interaction energy;
c     also partitions the energy among the atoms
c
c
      subroutine ewald3
      implicit none
      include 'sizes.i'
      include 'action.i'
      include 'analyz.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'bound.i'
      include 'boxes.i'
      include 'cell.i'
      include 'charge.i'
      include 'chgpot.i'
      include 'couple.i'
      include 'cutoff.i'
      include 'energi.i'
      include 'group.i'
      include 'inform.i'
      include 'iounit.i'
      include 'math.i'
      include 'moment.i'
      include 'pme.i'
      include 'units.i'
      include 'usage.i'
      integer i,j,k,ii,kk,skip(maxatm)
      real*8 e,f,fi,fik,fs
      real*8 r,r2,rew,fgrp,cut2
      real*8 xi,yi,zi,xr,yr,zr
      real*8 weight,xcenter,ycenter,zcenter
      real*8 totchg,xsum,ysum,zsum
      real*8 erfc,erfterm,scaleterm
      logical header,huge,proceed,iuse
c
c
c     zero out the Ewald summation energy and partitioning
c
      new = 0
      ew = 0.0d0
      do i = 1, n
         aew(i) = 0.0d0
      end do
      if (nion .eq. 0)  return
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set conversion factor, cutoff and scaling coefficients
c
      f = electric / dielec
      cut2 = pmecut * pmecut
      scaleterm = 1.0d0 - 1.0d0/chgscale
c
c     find the center of mass of the total system
c
      weight = 0.0d0
      xcenter = 0.0d0
      ycenter = 0.0d0
      zcenter = 0.0d0
      do i = 1, n
         weight = weight + mass(i)
         xcenter = xcenter + x(i)*mass(i)
         ycenter = ycenter + y(i)*mass(i)
         zcenter = zcenter + z(i)*mass(i)
      end do
      xcenter = xcenter / weight
      ycenter = ycenter / weight
      zcenter = zcenter / weight
c
c     get net charge and dipole components relative to center of mass
c
      totchg = 0.0d0
      xsum = 0.0d0
      ysum = 0.0d0
      zsum = 0.0d0
      do ii = 1, nion
         i = iion(ii)
         totchg = totchg + pchg(ii)
         xsum = xsum + (x(i)-xcenter)*pchg(ii)
         ysum = ysum + (y(i)-ycenter)*pchg(ii)
         zsum = zsum + (z(i)-zcenter)*pchg(ii)
      end do
      netchg = netchg + totchg
      xdipole = xdipole + debye*xsum
      ydipole = ydipole + debye*ysum
      zdipole = zdipole + debye*zsum
c
c     compute the Ewald self-energy term over all the atoms
c
      fs = -electric * aewald / sqrtpi
      do ii = 1, nion
         e = fs * pchg(ii)**2
         ew = ew + e
      end do
c
c     compute the reciprocal space part of the Ewald summation
c
      call epme (e)
      ew = ew + e
c
c     compute the direct space portion of the Ewald summation
c
      do ii = 1, nion-1
         i = iion(ii)
         iuse = use(i)
         skip(i) = i
         do j = 1, n12(i)
            skip(i12(j,i)) = i * chg12use
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i * chg13use
         end do
         do j = 1, n14(i)
            skip(i14(j,i)) = i * chg14use
         end do
         xi = x(i)
         yi = y(i)
         zi = z(i)
         fi = f * pchg(ii)
         do kk = ii+1, nion
            k = iion(kk)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k))
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               xr = xi - x(k)
               yr = yi - y(k)
               zr = zi - z(k)
               if (use_image)  call image (xr,yr,zr,0)
               r2 = xr*xr + yr*yr + zr*zr
               if (r2 .le. cut2) then
                  r = sqrt(r2)
                  fik = fi * pchg(kk)
                  rew = aewald * r
                  erfterm = erfc (rew)
                  e = fik * erfterm / r
c
c     adjust the direct sum energy for neglected interactions
c
                  if (skip(k) .eq. i) then
                     e = e - fik/r
                  else if (skip(k) .eq. -i) then
                     e = e - (fik/r)*scaleterm
                  end if
c
c     scale the interaction based on its group membership
c
                  if (use_group)  e = e * fgrp
c
c     increment the overall Ewald summation energy component
c
                  new = new + 1
                  ew = ew + e
                  aew(i) = aew(i) + 0.5d0*e
                  aew(k) = aew(k) + 0.5d0*e
c
c     print a warning if the energy of this charge pair is large
c
                  huge = (abs(e) .gt. 100.0d0)
                  if (debug .or. (verbose.and.huge)) then
                     if (header) then
                        header = .false.
                        write (iout,10)
   10                   format (/,' Individual Ewald Direct Sum',
     &                             ' Interactions :',
     &                          //,' Type',13x,'Atom Names',
     &                             16x,'Charges',5x,'Distance',
     &                             5x,'Energy',/)
                     end if
                     write (iout,20)  i,name(i),k,name(k),
     &                                pchg(ii),pchg(kk),r,e
   20                format (' Ewald',6x,i5,'-',a3,1x,i5,'-',a3,
     &                          8x,2f7.2,f10.4,f12.4)
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate direct space portion involving other unit cells
c
      do ii = 1, nion
         i = iion(ii)
         iuse = use(i)
         xi = x(i)
         yi = y(i)
         zi = z(i)
         fi = f * pchg(ii)
         do kk = ii, nion
            k = iion(kk)
c
c     decide whether to compute the current interaction
c
            proceed = .true.
            if (use_group)  call groups (proceed,fgrp,2,i,k,0,0,0)
            if (proceed)  proceed = (iuse .or. use(k))
c
c     compute the energy contribution for this interaction
c
            if (proceed) then
               do j = 1, ncell
                  xr = xi - x(k)
                  yr = yi - y(k)
                  zr = zi - z(k)
                  call image (xr,yr,zr,j)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. cut2) then
                     r = sqrt(r2)
                     fik = fi * pchg(kk)
                     rew = aewald * r
                     erfterm = erfc (rew)
                     e = fik * erfterm / r
c
c     scale the interaction based on its group membership
c
                     if (use_group)  e = e * fgrp
c
c     increment the overall Ewald summation energy component;
c     interaction of an atom with its own image counts half
c
                     new = new + 1
                     if (i .eq. k) then
                        ew = ew + 0.5d0*e
                        aew(i) = aew(i) + 0.5d0*e
                     else
                        ew = ew + e
                        aew(i) = aew(i) + 0.5d0*e
                        aew(k) = aew(k) + 0.5d0*e
                     end if
c
c     print a warning if the energy of this interaction is large
c
                     huge = (abs(e) .gt. 100.0d0)
                     if (debug .or. (verbose.and.huge)) then
                        if (header) then
                           header = .false.
                           write (iout,30)
   30                      format (/,' Individual Ewald Direct Sum',
     &                                ' Interactions :',
     &                             //,' Type',13x,'Atom Names',
     &                                16x,'Charges',5x,'Distance',
     &                                5x,'Energy',/)
                        end if
                        write (iout,40)  i,name(i),k,name(k),
     &                                   pchg(ii),pchg(kk),r,e
   40                   format (' Ewald',6x,i5,'-',a3,1x,i5,'-',a3,
     &                             1x,'(XTAL)',1x,2f7.2,f10.4,f12.4)
                     end if
                  end if
               end do
            end if
         end do
      end do
      return
      end
