c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1999  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ################################################################
c     ##                                                            ##
c     ##  subroutine kopdist  --  out-of-plane distance parameters  ##
c     ##                                                            ##
c     ################################################################
c
c
c     "kopdist" assigns the force constants for out-of-plane
c     distance at trigonal centers via the central atom height;
c     also processes any new or changed parameter values
c
c
      subroutine kopdist
      implicit none
      include 'sizes.i'
      include 'atmtyp.i'
      include 'atoms.i'
      include 'couple.i'
      include 'inform.i'
      include 'iounit.i'
      include 'keys.i'
      include 'kopdst.i'
      include 'opdist.i'
      include 'potent.i'
      integer i,j,k,size,next
      integer ia,ib,ic,id
      integer ita,itb,itc,itd
      real*8 fopd
      character*3 pa,pb,pc,pd
      character*12 blank,pti,pt(6)
      character*20 keyword
      character*80 record,string
      logical header
      data blank  / '            ' /
c
c
c     process keywords containing out-of-plane distance parameters
c
      header = .true.
      do i = 1, nkey
         next = 1
         record = keyline(i)
         call gettext (record,keyword,next)
         call upcase (keyword)
         if (keyword(1:7) .eq. 'OPDIST ') then
            ia = 0
            ib = 0
            ic = 0
            id = 0
            fopd = 0.0d0
            string = record(next:80)
            read (string,*,err=10,end=10)  ia,ib,ic,id,fopd
   10       continue
            size = 3
            call numeral (ia,pa,size)
            call numeral (ib,pb,size)
            call numeral (ic,pc,size)
            call numeral (id,pd,size)
            pti = pa//pb//pc//pd
            if (header) then
               header = .false.
               write (iout,20)
   20          format (/,' Additional Out-of-Plane Distance',
     &                    ' Parameters :',
     &                 //,5x,'Atom Classes',12x,'K(OPD)',/)
            end if
            write (iout,30)  ia,ib,ic,id,fopd
   30       format (1x,4i4,4x,2f14.4)
            do j = 1, maxnopd
               if (kaopd(j).eq.blank .or. kaopd(j).eq.pti) then
                  kaopd(j) = pti
                  copd(j) = fopd
                  goto 50
               end if
            end do
            write (iout,40)
   40       format (/,' KOPDIST  --  Too many Out-of-Plane Distance',
     &                 ' Parameters')
            abort = .true.
   50       continue
         end if
      end do
c
c     assign out-of-plane distance parameters for trigonal sites
c
      nopdist = 0
      do i = 1, n
         if (n12(i) .eq. 3) then
            ia = i
            ib = i12(1,i)
            ic = i12(2,i)
            id = i12(3,i)
            ita = class(ia)
            itb = class(ib)
            itc = class(ic)
            itd = class(id)
            size = 3
            call numeral (ita,pa,size)
            call numeral (itb,pb,size)
            call numeral (itc,pc,size)
            call numeral (itd,pd,size)
            do j = 1, maxnopd
               if (kaopd(j)(1:3) .eq. pa) then
                  pt(1) = pa//pb//pc//pd
                  pt(2) = pa//pb//pd//pc
                  pt(3) = pa//pc//pb//pd
                  pt(4) = pa//pc//pd//pb
                  pt(5) = pa//pd//pb//pc
                  pt(6) = pa//pd//pc//pb
                  do k = 1, 6
                     if (kaopd(j) .eq. pt(k)) then
                        nopdist = nopdist + 1
                        iopd(1,nopdist) = ia
                        if (k.eq.1 .or. k.eq.2)  iopd(2,nopdist) = ib
                        if (k.eq.3 .or. k.eq.4)  iopd(2,nopdist) = ic
                        if (k.eq.5 .or. k.eq.6)  iopd(2,nopdist) = id
                        if (k.eq.3 .or. k.eq.5)  iopd(3,nopdist) = ib
                        if (k.eq.1 .or. k.eq.6)  iopd(3,nopdist) = ic
                        if (k.eq.2 .or. k.eq.4)  iopd(3,nopdist) = id
                        if (k.eq.4 .or. k.eq.6)  iopd(4,nopdist) = ib
                        if (k.eq.2 .or. k.eq.5)  iopd(4,nopdist) = ic
                        if (k.eq.1 .or. k.eq.3)  iopd(4,nopdist) = id
                        kopd(nopdist) = copd(j)
                        goto 60
                     end if
                  end do
               end if
            end do
   60       continue
         end if
      end do
c
c     if no out-of-plane distances are used, turn off the potential
c
      if (nopdist .eq. 0)  use_opdist = .false.
      return
      end
