c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine pressure  --  constant pressure via coupling  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "pressure" uses the internal virial to find the pressure
c     in a periodic box and maintains a constant desired pressure
c     by scaling the coordinates via coupling to an external
c     constant pressure bath
c
c     literature reference :
c
c     H. J. C. Berendsen, J. P. M. Postma, W. F. van Gunsteren,
c     A. DiNola and J. R. Hauk, "Molecular Dynamics with Coupling
c     to an External Bath", Journal of Chemical Physics, 81,
c     3684-3690 (1984)
c
c
      subroutine pressure (dt,pres,e_kin)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bath.i'
      include 'boxes.i'
      include 'units.i'
      include 'usage.i'
      include 'virial.i'
      integer i
      real*8 dt,e_kin,pres
      real*8 virial,third,scale
c
c
c     compute the value of the internal virial and pressure
c
      virial = virx + viry + virz
      pres = (2.0d0*e_kin - virial) / (3.0d0*volbox)
      pres = pres * prescon
c
c     find the scale factor to maintain constant pressure
c
      third = 1.0d0 / 3.0d0
      scale = (1.0d0 + (dt*compress/taupres)*(pres-atmsph))**third
c
c     alter the size of the periodic box dimensions
c
      xbox = scale * xbox
      ybox = scale * ybox
      zbox = scale * zbox
c
c     update the lattice parameters for the periodic box
c
      call lattice
c
c     couple to external pressure bath via atom scaling
c
      do i = 1, n
         if (use(i)) then
            x(i) = scale * x(i)
            y(i) = scale * y(i)
            z(i) = scale * z(i)
         end if
      end do
      return
      end
