c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1990  by  Jay William Ponder  ##
c     ##              All Rights Reserved              ##
c     ###################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine prtprm  --  output of force field parameters  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "prtprm" writes out a formatted listing of the default
c     set of potential energy parameters for a force field
c
c
      subroutine prtprm (itxt)
      implicit none
      include 'sizes.i'
      include 'angpot.i'
      include 'bndpot.i'
      include 'fields.i'
      include 'kanang.i'
      include 'kangs.i'
      include 'katoms.i'
      include 'kbonds.i'
      include 'kchrge.i'
      include 'kdipol.i'
      include 'khbond.i'
      include 'kiprop.i'
      include 'kitors.i'
      include 'kmulti.i'
      include 'kopbnd.i'
      include 'kopdst.i'
      include 'korbs.i'
      include 'kpolr.i'
      include 'kstbnd.i'
      include 'ksttor.i'
      include 'ktorsn.i'
      include 'kurybr.i'
      include 'kvdws.i'
      include 'kvdwpr.i'
      integer i,j,k,itxt,number
      integer k1,k2,k3,k4
      integer fold(6)
      real*8 ampli(6),phase(6)
      character*1 formfeed
      logical exist
c
c
c     set the string value of the formfeed character (Ctrl-L)
c
      formfeed = char(12)
c
c     force field atom type definitions
c
      exist = .false.
      do i = 1, maxtyp
         if (symbol(i) .ne. '   ')  exist = .true.
      end do
      if (exist) then
         write (itxt,10)  forcefield
   10    format (//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,20)
   20    format (//,17x,'Force Field Atom Definitions',
     &           //,52x,'Atomic',4x,'Atomic',
     &           /,7x,'Type',3x,'Class',3x,'Symbol',3x,'Description',
     &              10x,'Number',4x,'Weight',3x,'Valence',/)
         do i = 1, maxtyp
            if (symbol(i) .ne. '   ') then
               write (itxt,30)  i,atmcls(i),symbol(i),describe(i),
     &                          atmnum(i),weight(i),ligand(i)
   30          format (5x,i5,3x,i5,5x,a3,5x,a20,i5,f12.3,i7)
            end if
         end do
      end if
c
c     van der Waals parameters for atom types
c
      exist = .false.
      do i = 1, maxclass
         if (rad(i) .ne. 0.0d0)  exist = .true.
      end do
      if (exist) then
         write (itxt,40)  formfeed,forcefield
   40    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,50)
   50    format (//,17x,'Van der Waals Parameters',
     &           ///,22x,'Class',7x,'Radius',6x,'Epsilon',
     &                 4x,'Reduction',/)
         k = 0
         do i = 1, maxclass
            if (rad(i) .ne. 0.0d0) then
               k = k + 1
               write (itxt,60)  k,i,rad(i),eps(i),reduct(i)
   60          format (13x,i5,5x,i3,2x,3f12.3)
            end if
         end do
      end if
c
c     van der Waals 1-4 parameters for atom types
c
      exist = .false.
      do i = 1, maxclass
         if (rad4(i) .ne. 0.0d0)  exist = .true.
      end do
      if (exist) then
         write (itxt,70)
   70    format (//,17x,'Van der Waals Parameters for 1-4 Interactions',
     &           ///,22x,'Class',7x,'Radius',6x,'Epsilon',/)
         k = 0
         do i = 1, maxclass
            if (rad4(i) .ne. 0.0d0) then
               k = k + 1
               write (itxt,80)  k,i,rad4(i),eps4(i)
   80          format (13x,i5,5x,i3,2x,2f12.3)
            end if
         end do
      end if
c
c     van der Waals parameters for specific atom pairs
c
      if (kvpr(1) .ne. '      ') then
         write (itxt,90)
   90    format (//,17x,'Van der Waals Parameters for Atom Pairs',
     &           ///,23x,'Classes',7x,'Radii Sum',4x,'Epsilon',/)
         do i = 1, maxnvp
            if (kvpr(i) .eq. '      ')  goto 110
            k1 = number (kvpr(i)(1:3))
            k2 = number (kvpr(i)(4:6))
            write (itxt,100)  i,k1,k2,radpr(i),epspr(i)
  100       format (13x,i5,5x,i3,'-',i3,2x,2f12.3)
         end do
  110    continue
      end if
c
c     hydrogen bonding parameters for specific atom pairs
c
      if (khb(1) .ne. '      ') then
         write (itxt,120)
  120    format (//,17x,'Hydrogen Bonding Parameters for Atom Pairs',
     &           ///,23x,'Classes',7x,'Radii Sum',4x,'Epsilon',/)
         do i = 1, maxnhb
            if (khb(i) .eq. '      ')  goto 140
            k1 = number (khb(i)(1:3))
            k2 = number (khb(i)(4:6))
            write (itxt,130)  i,k1,k2,radhb(i),epshb(i)
  130       format (13x,i5,5x,i3,'-',i3,2x,2f12.3)
         end do
  140    continue
      end if
c
c     harmonic bond stretching parameters
c
      if (kb(1) .ne. '      ') then
         write (itxt,150)  formfeed,forcefield
  150    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,160)
  160    format (//,17x,'Bond Stretching Parameters',
     &           ///,23x,'Classes',15x,'KS',7x,'Length',/)
         do i = 1, maxnb
            if (kb(i) .eq. '      ')  goto 180
            k1 = number (kb(i)(1:3))
            k2 = number (kb(i)(4:6))
            write (itxt,170)  i,k1,k2,bcon(i),blen(i)
  170       format (13x,i5,5x,i3,'-',i3,6x,f12.3,f12.4)
         end do
  180    continue
      end if
c
c     bond stretching parameters for 5-membered rings
c
      if (kb5(1) .ne. '      ') then
         write (itxt,190)
  190    format (//,17x,'5-Membered Ring Stretch Parameters',
     &           ///,23x,'Classes',15x,'KS',7x,'Length',/)
         do i = 1, maxnb5
            if (kb5(i) .eq. '      ')  goto 210
            k1 = number (kb5(i)(1:3))
            k2 = number (kb5(i)(4:6))
            write (itxt,200)  i,k1,k2,bcon5(i),blen5(i)
  200       format (13x,i5,5x,i3,'-',i3,6x,f12.3,f12.4)
         end do
  210    continue
      end if
c
c     bond stretching parameters for 4-membered rings
c
      if (kb4(1) .ne. '      ') then
         write (itxt,220)
  220    format (//,17x,'4-Membered Ring Stretch Parameters',
     &           ///,23x,'Classes',15x,'KS',7x,'Length',/)
         do i = 1, maxnb4
            if (kb4(i) .eq. '      ')  goto 240
            k1 = number (kb4(i)(1:3))
            k2 = number (kb4(i)(4:6))
            write (itxt,260)  i,k1,k2,bcon4(i),blen4(i)
  230       format (13x,i5,5x,i3,'-',i3,6x,f12.3,f12.4)
         end do
  240    continue
      end if
c
c     bond stretching parameters for 3-membered rings
c
      if (kb3(1) .ne. '      ') then
         write (itxt,250)
  250    format (//,17x,'3-Membered Ring Stretch Parameters',
     &           ///,23x,'Classes',15x,'KS',7x,'Length',/)
         do i = 1, maxnb3
            if (kb3(i) .eq. '      ')  goto 270
            k1 = number (kb3(i)(1:3))
            k2 = number (kb3(i)(4:6))
            write (itxt,260)  i,k1,k2,bcon3(i),blen3(i)
  260       format (13x,i5,5x,i3,'-',i3,6x,f12.3,f12.4)
         end do
  270    continue
      end if
c
c     cubic and quartic bond stretching parameters
c
      if (cbnd.ne.0.0d0 .or. qbnd.ne.0.0d0) then
         write (itxt,280)  cbnd,qbnd
  280    format (//,17x,'Higher Order Stretching Constants',
     &           ///,23x,'Cubic',f17.3,/,23x,'Quartic',f15.3)
      end if
c
c     harmonic bond angle bending parameters
c
      if (ka(1) .ne. '         ') then
         write (itxt,290)  formfeed,forcefield
  290    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,300)
  300    format (//,17x,'Angle Bending Parameters',
     &           ///,18x,'Classes',10x,'KB',6x,'Value 1',
     &              5x,'Value 2',5x,'Value 3',
     &           /,43x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         do i = 1, maxna
            if (ka(i) .eq. '         ')  goto 330
            k1 = number (ka(i)(1:3))
            k2 = number (ka(i)(4:6))
            k3 = number (ka(i)(7:9))
            if (ang(2,i).eq.0.0d0 .and. ang(3,i).eq.0.0d0) then
               write (itxt,310)  i,k1,k2,k3,acon(i),ang(1,i)
  310          format (5x,i5,5x,i3,'-',i3,'-',i3,2f12.3)
            else
               write (itxt,320)  i,k1,k2,k3,acon(i),(ang(j,i),j=1,3)
  320          format (5x,i5,5x,i3,'-',i3,'-',i3,4f12.3)
            end if
         end do
  330    continue
      end if
c
c     bond angle bending parameters for 5-membered rings
c
      if (ka5(1) .ne. '         ') then
         write (itxt,340)
  340    format (//,17x,'5-Membered Ring Bend Parameters',
     &           ///,18x,'Classes',10x,'KB',6x,'Value 1',
     &              5x,'Value 2',5x,'Value 3',
     &           /,43x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         do i = 1, maxna5
            if (ka5(i) .eq. '         ')  goto 370
            k1 = number (ka5(i)(1:3))
            k2 = number (ka5(i)(4:6))
            k3 = number (ka5(i)(7:9))
            if (ang5(2,i).eq.0.0d0 .and. ang5(3,i).eq.0.0d0) then
               write (itxt,350)  i,k1,k2,k3,acon5(i),ang5(1,i)
  350          format (5x,i5,5x,i3,'-',i3,'-',i3,2f12.3)
            else
               write (itxt,360)  i,k1,k2,k3,acon5(i),(ang5(j,i),j=1,3)
  360          format (5x,i5,5x,i3,'-',i3,'-',i3,4f12.3)
            end if
         end do
  370    continue
      end if
c
c     bond angle bending parameters for 4-membered rings
c
      if (ka4(1) .ne. '         ') then
         write (itxt,380)
  380    format (//,17x,'4-Membered Ring Bend Parameters',
     &           ///,18x,'Classes',10x,'KB',6x,'Value 1',
     &              5x,'Value 2',5x,'Value 3',
     &           /,43x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         do i = 1, maxna4
            if (ka4(i) .eq. '         ')  goto 410
            k1 = number (ka4(i)(1:3))
            k2 = number (ka4(i)(4:6))
            k3 = number (ka4(i)(7:9))
            if (ang4(2,i).eq.0.0d0 .and. ang4(3,i).eq.0.0d0) then
               write (itxt,390)  i,k1,k2,k3,acon4(i),ang4(1,i)
  390          format (5x,i5,5x,i3,'-',i3,'-',i3,2f12.3)
            else
               write (itxt,400)  i,k1,k2,k3,acon4(i),(ang4(j,i),j=1,3)
  400          format (5x,i5,5x,i3,'-',i3,'-',i3,4f12.3)
            end if
         end do
  410    continue
      end if
c
c     bond angle bending parameters for 3-membered rings
c
      if (ka3(1) .ne. '         ') then
         write (itxt,420)
  420    format (//,17x,'3-Membered Ring Bend Parameters',
     &           ///,18x,'Classes',10x,'KB',6x,'Value 1',
     &              5x,'Value 2',5x,'Value 3',
     &           /,43x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         do  i = 1, maxna3
            if (ka3(i) .eq. '         ')  goto 450
            k1 = number (ka3(i)(1:3))
            k2 = number (ka3(i)(4:6))
            k3 = number (ka3(i)(7:9))
            if (ang3(2,i).eq.0.0d0 .and. ang3(3,i).eq.0.0d0) then
               write (itxt,430)  i,k1,k2,k3,acon3(i),ang3(1,i)
  430          format (5x,i5,5x,i3,'-',i3,'-',i3,2f12.3)
            else
               write (itxt,440)  i,k1,k2,k3,acon3(i),(ang3(j,i),j=1,3)
  440          format (5x,i5,5x,i3,'-',i3,'-',i3,4f12.3)
            end if
         end do
  450    continue
      end if
c
c     Fourier bond angle bending parameters
c
      if (kaf(1) .ne. '         ') then
         write (itxt,460)
  460    format (//,17x,'Fourier Angle Bending Parameters',
     &           ///,18x,'Classes',10x,'KB',8x,'Shift',6x,'Period',/)
         do  i = 1, maxnaf
            if (kaf(i) .eq. '         ')  goto 480
            k1 = number (kaf(i)(1:3))
            k2 = number (kaf(i)(4:6))
            k3 = number (kaf(i)(7:9))
            write (itxt,470)  i,k1,k2,k3,aconf(i),(angf(j,i),j=1,2)
  470       format (5x,i5,5x,i3,'-',i3,'-',i3,3f12.3)
         end do
  480    continue
      end if
c
c     cubic through sextic bond angle bending parameters
c
      if (cang.ne.0.0d0 .or. qang.ne.0.0d0 .or.
     &    pang.ne.0.0d0 .or. sang.ne.0.0d0) then
         write (itxt,490)  cang,qang,pang,sang
  490    format (//,17x,'Higher Order Bending Constants',
     &           ///,21x,'Cubic',d17.3,/,21x,'Quartic',d15.3,
     &           /,21x,'Pentic',d16.3,/,21x,'Sextic',d16.3)
      end if
c
c     stretch-bend parameters
c
      exist = .false.
      do i = 1, maxclass
         do k = 1, 3
            if (stbn(k,i) .ne. 0.0d0)  exist = .true.
         end do
      end do
      if (exist) then
         write (itxt,500)  formfeed,forcefield
  500    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,510)
  510    format (//,17x,'Stretch-Bend Parameters',
     &           ///,21x,'Class',9x,'KSB 1',7x,'KSB 2',7x,'KSB 3',
     &           /,34x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         k = 0
         do i = 1, maxclass
            if (stbn(1,i).ne.0.0d0 .or. stbn(2,i).ne.0.0d0
     &               .or. stbn(3,i).ne.0.0d0) then
               k = k + 1
               write (itxt,520)  k,i,(stbn(j,i),j=1,3)
  520          format (10x,i5,7x,i3,3x,3f12.3)
            end if
         end do
      end if
c
c     Urey-Bradley parameters
c
      if (ku(1) .ne. '         ') then
         write (itxt,530)  formfeed,forcefield
  530    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,540)
  540    format (//,17x,'Urey-Bradley Parameters',
     &           ///,18x,'Classes',10x,'KB',6x,'Distance',/)
         do i = 1, maxnu
            if (ku(i) .eq. '         ')  goto 560
            k1 = number (ku(i)(1:3))
            k2 = number (ku(i)(4:6))
            k3 = number (ku(i)(7:9))
            write (itxt,550)  i,k1,k2,k3,ucon(i),dst13(i)
  550       format (5x,i5,5x,i3,'-',i3,'-',i3,f12.3,f12.4)
         end do
  560    continue
      end if
c
c     angle-angle parameters
c
      exist = .false.
      do i = 1, maxclass
         do k = 1, 3
            if (anan(k,i) .ne. 0.0d0)  exist = .true.
         end do
      end do
      if (exist) then
         write (itxt,570)  formfeed,forcefield
  570    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,580)
  580    format (//,17x,'Angle-Angle Parameters',
     &           ///,21x,'Class',9x,'KAA 1',7x,'KAA 2',7x,'KAA 3',
     &           /,34x,'(R-X-R)',5x,'(R-X-H)',5x,'(H-X-H)',/)
         k = 0
         do i = 1, maxclass
            if (anan(1,i).ne.0.0d0 .or. anan(2,i).ne.0.0d0
     &               .or. anan(3,i).ne.0.0d0) then
               k = k + 1
               write (itxt,590)  k,i,(anan(j,i),j=1,3)
  590          format (10x,i5,7x,i3,3x,3f12.3)
            end if
         end do
      end if
c
c     out-of-plane bending parameters
c
      if (kaopb(1) .ne. '      ') then
         write (itxt,600)  formfeed,forcefield
  600    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,610)
  610    format (//,17x,'Out-of-Plane Bend Parameters',
     &           ///,23x,'Classes',8x,'KOPB',/)
         do i = 1, maxnopb
            if (kaopb(i) .eq. '      ')  goto 630
            k1 = number (kaopb(i)(1:3))
            k2 = number (kaopb(i)(4:6))
            write (itxt,620)  i,k1,k2,copb(i)
  620       format (10x,i5,8x,i3,'-',i3,f12.3)
         end do
  630    continue
      end if
c
c     out-of-plane distance parameters
c
      if (kaopd(1) .ne. '            ') then
         write (itxt,640)  formfeed,forcefield
  640    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,650)
  650    format (//,17x,'Out-of-Plane Distance Parameters',
     &           ///,20x,'Classes',10x,'KOPD',/)
         do i = 1, maxnopd
            if (kaopd(i) .eq. '            ')  goto 670
            k1 = number (kaopd(i)(1:3))
            k2 = number (kaopd(i)(4:6))
            k3 = number (kaopd(i)(7:9))
            k4 = number (kaopd(i)(10:12))
            write (itxt,660)  i,k1,k2,k3,k4,copd(i)
  660       format (5x,i5,5x,i3,'-',i3,'-',i3,'-',i3,f12.3)
         end do
  670    continue
      end if
c
c     improper dihedral parameters
c
      if (kdi(1) .ne. '            ') then
         write (itxt,680)  formfeed,forcefield
  680    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,690)
  690    format (//,17x,'Improper Dihedral Parameters',
     &           ///,20x,'Classes',11x,'KID',7x,'Target',/)
         do i = 1, maxndi
            if (kdi(i) .eq. '            ')  goto 710
            k1 = number (kdi(i)(1:3))
            k2 = number (kdi(i)(4:6))
            k3 = number (kdi(i)(7:9))
            k4 = number (kdi(i)(10:12))
            write (itxt,700)  i,k1,k2,k3,k4,dcon(i),tdi(i)
  700       format (5x,i5,5x,i3,'-',i3,'-',i3,'-',i3,f12.3,f12.4)
         end do
  710    continue
      end if
c
c     improper torsional parameters
c
      if (kti(1) .ne. '            ') then
         write (itxt,720)  formfeed,forcefield
  720    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,730)
  730    format (//,17x,'Improper Torsion Parameters',
     &           ///,18x,'Classes',15x,'KTI Values',/)
         do i = 1, maxnti
            if (kti(i) .eq. '            ')  goto 750
            k1 = number (kti(i)(1:3))
            k2 = number (kti(i)(4:6))
            k3 = number (kti(i)(7:9))
            k4 = number (kti(i)(10:12))
            j = 0
            if (ti1(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 1
               ampli(j) = ti1(1,i)
               phase(j) = ti1(2,i)
            end if
            if (ti2(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 2
               ampli(j) = ti2(1,i)
               phase(j) = ti2(2,i)
            end if
            if (ti3(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 3
               ampli(j) = ti3(1,i)
               phase(j) = ti3(2,i)
            end if
            write (itxt,740)  i,k1,k2,k3,k4,(ampli(k),
     &                        phase(k),fold(k),k=1,j)
  740       format (3x,i5,5x,i3,'-',i3,'-',i3,'-',i3,2x,3(f8.3,f6.1,i2))
         end do
  750    continue
      end if
c
c     torsional angle parameters
c
      if (kt(1) .ne. '            ') then
         write (itxt,760)  formfeed,forcefield
  760    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,770)
  770    format (//,17x,'Torsional Parameters',
     &           ///,18x,'Classes',15x,'KT Values',/)
         do i = 1, maxnt
            if (kt(i) .eq. '            ')  goto 790
            k1 = number (kt(i)(1:3))
            k2 = number (kt(i)(4:6))
            k3 = number (kt(i)(7:9))
            k4 = number (kt(i)(10:12))
            j = 0
            if (t1(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 1
               ampli(j) = t1(1,i)
               phase(j) = t1(2,i)
            end if
            if (t2(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 2
               ampli(j) = t2(1,i)
               phase(j) = t2(2,i)
            end if
            if (t3(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 3
               ampli(j) = t3(1,i)
               phase(j) = t3(2,i)
            end if
            if (t4(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 4
               ampli(j) = t4(1,i)
               phase(j) = t4(2,i)
            end if
            if (t5(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 5
               ampli(j) = t5(1,i)
               phase(j) = t5(2,i)
            end if
            if (t6(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 6
               ampli(j) = t6(1,i)
               phase(j) = t6(2,i)
            end if
            write (itxt,780)  i,k1,k2,k3,k4,(ampli(k),
     &                        phase(k),fold(k),k=1,j)
  780       format (3x,i5,5x,i3,'-',i3,'-',i3,'-',i3,2x,6(f8.3,f6.1,i2))
         end do
  790    continue
      end if
c
c     torsional angle parameters for 5-membered rings
c
      if (kt5(1) .ne. '            ') then
         write (itxt,800)
  800    format (//,17x,'5-Membered Ring Torsion Parameters',
     &           ///,18x,'Classes',15x,'KT Values',/)
         do i = 1, maxnt5
            if (kt5(i) .eq. '            ')  goto 820
            k1 = number (kt5(i)(1:3))
            k2 = number (kt5(i)(4:6))
            k3 = number (kt5(i)(7:9))
            k4 = number (kt5(i)(10:12))
            j = 0
            if (t15(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 1
               ampli(j) = t15(1,i)
               phase(j) = t15(2,i)
            end if
            if (t25(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 2
               ampli(j) = t25(1,i)
               phase(j) = t25(2,i)
            end if
            if (t35(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 3
               ampli(j) = t35(1,i)
               phase(j) = t35(2,i)
            end if
            if (t45(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 4
               ampli(j) = t45(1,i)
               phase(j) = t45(2,i)
            end if
            if (t55(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 5
               ampli(j) = t55(1,i)
               phase(j) = t55(2,i)
            end if
            if (t65(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 6
               ampli(j) = t65(1,i)
               phase(j) = t65(2,i)
            end if
            write (itxt,810)  i,k1,k2,k3,k4,(ampli(k),
     &                        phase(k),fold(k),k=1,j)
  810       format (3x,i5,5x,i3,'-',i3,'-',i3,'-',i3,2x,6(f8.3,f6.1,i2))
         end do
  820    continue
      end if
c
c     torsional angle parameters for 4-membered rings
c
      if (kt4(1) .ne. '            ') then
         write (itxt,830)
  830    format (//,17x,'4-Membered Ring Torsion Parameters',
     &           ///,18x,'Classes',15x,'KT Values',/)
         do i = 1, maxnt4
            if (kt4(i) .eq. '            ')  goto 850
            k1 = number (kt4(i)(1:3))
            k2 = number (kt4(i)(4:6))
            k3 = number (kt4(i)(7:9))
            k4 = number (kt4(i)(10:12))
            j = 0
            if (t14(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 1
               ampli(j) = t14(1,i)
               phase(j) = t14(2,i)
            end if
            if (t24(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 2
               ampli(j) = t24(1,i)
               phase(j) = t24(2,i)
            end if
            if (t34(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 3
               ampli(j) = t34(1,i)
               phase(j) = t34(2,i)
            end if
            if (t44(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 4
               ampli(j) = t44(1,i)
               phase(j) = t44(2,i)
            end if
            if (t54(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 5
               ampli(j) = t54(1,i)
               phase(j) = t54(2,i)
            end if
            if (t64(1,i) .ne. 0.0d0) then
               j = j + 1
               fold(j) = 6
               ampli(j) = t64(1,i)
               phase(j) = t64(2,i)
            end if
            write (itxt,840)  i,k1,k2,k3,k4,(ampli(k),
     &                        phase(k),fold(k),k=1,j)
  840       format (3x,i5,5x,i3,'-',i3,'-',i3,'-',i3,2x,6(f8.3,f6.1,i2))
         end do
  850    continue
      end if
c
c     stretch-torsion parameters
c
      if (kbt(1) .ne. '      ') then
         write (itxt,860)  formfeed,forcefield
  860    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,870)
  870    format (//,17x,'Stretch-Torsion Parameters',
     &           ///,24x,'Classes',14x,'KST1',8x,'KST2',8x,'KST3',/)
         do i = 1, maxnbt
            if (kbt(i) .eq. '      ')  goto 890
            k1 = number (kbt(i)(1:3))
            k2 = number (kbt(i)(4:6))
            write (itxt,880)  i,k1,k2,(btcon(j,i),j=1,3)
  880       format (14x,i5,5x,i3,'-',i3,6x,3f12.3)
         end do
  890    continue
      end if
c
c     atomic partial charge parameters
c
      exist = .false.
      do i = 1, maxtyp
         if (chg(i) .ne. 0.0d0)  exist = .true.
      end do
      if (exist) then
         write (itxt,900)  formfeed,forcefield
  900    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,910)
  910    format (//,17x,'Atomic Partial Charge Parameters',
     &           ///,28x,'Type',9x,'Partial Chg',/)
         k = 0
         do i = 1, maxtyp
            if (chg(i) .ne. 0.0d0) then
               k = k + 1
               write (itxt,920)  k,i,chg(i)
  920          format (16x,i5,7x,i3,6x,f12.3)
            end if
         end do
      end if
c
c     bond dipole moment parameters
c
      if (kd(1) .ne. '      ') then
         write (itxt,930)  formfeed,forcefield
  930    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,940)
  940    format (//,17x,'Bond Dipole Moment Parameters',
     &           ///,26x,'Types',9x,'Bond Dipole',4x,'Position',/)
         do i = 1, maxnd
            if (kd(i) .eq. '      ')  goto 960
            k1 = number (kd(i)(1:3))
            k2 = number (kd(i)(4:6))
            write (itxt,950)  i,k1,k2,dpl(i),pos(i)
  950       format (14x,i5,5x,i3,'-',i3,6x,2f12.3)
         end do
  960    continue
      end if
c
c     bond dipole moment parameters for 5-membered rings
c
      if (kd5(1) .ne. '      ') then
         write (itxt,970)
  970    format (//,17x,'5-Membered Ring Bond Dipole Parameters',
     &           ///,26x,'Types',9x,'Bond Dipole',4x,'Position',/)
         do i = 1, maxnd5
            if (kd5(i) .eq. '      ')  goto 990
            k1 = number (kd5(i)(1:3))
            k2 = number (kd5(i)(4:6))
            write (itxt,980)  i,k1,k2,dpl5(i),pos5(i)
  980       format (14x,i5,5x,i3,'-',i3,6x,2f12.3)
         end do
  990    continue
      end if
c
c     bond dipole moment parameters for 4-membered rings
c
      if (kd4(1) .ne. '      ') then
         write (itxt,1000)
 1000    format (//,17x,'4-Membered Ring Bond Dipole Parameters',
     &           ///,26x,'Types',9x,'Bond Dipole',4x,'Position',/)
         do i = 1, maxnd4
            if (kd4(i) .eq. '      ')  goto 1020
            k1 = number (kd4(i)(1:3))
            k2 = number (kd4(i)(4:6))
            write (itxt,1010)  i,k1,k2,dpl4(i),pos4(i)
 1010       format (14x,i5,5x,i3,'-',i3,6x,2f12.3)
         end do
 1020    continue
      end if
c
c     bond dipole moment parameters for 3-membered rings
c
      if (kd3(1) .ne. '      ') then
         write (itxt,1030)
 1030    format (//,17x,'3-Membered Ring Bond Dipole Parameters',
     &           ///,26x,'Types',9x,'Bond Dipole',4x,'Position',/)
         do i = 1, maxnd3
            if (kd3(i) .eq. '      ')  goto 1050
            k1 = number (kd3(i)(1:3))
            k2 = number (kd3(i)(4:6))
            write (itxt,1040)  i,k1,k2,dpl3(i),pos3(i)
 1040       format (14x,i5,5x,i3,'-',i3,6x,2f12.3)
         end do
 1050    continue
      end if
c
c     atomic multipole electrostatic parameters
c
      if (kmp(1) .ne. '         ') then
         write (itxt,1060)  formfeed,forcefield
 1060    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,1070)
 1070    format (//,17x,'Atomic Multipole Parameters',
     &           ///,17x,'Type',3x,'Axis Types',6x,'Frame',
     &              8x,'Multipoles (M-D-Q)',/)
         do i = 1, maxnmp
            if (kmp(i) .eq. '         ')  goto 1090
            k1 = number (kmp(i)(1:3))
            k2 = number (kmp(i)(4:6))
            k3 = number (kmp(i)(7:9))
            write (itxt,1080)  i,k1,k2,k3,mpaxis(i),multip(1,i),
     &                        multip(2,i),multip(3,i),multip(4,i),
     &                        multip(5,i),multip(8,i),multip(9,i),
     &                        multip(11,i),multip(12,i),multip(13,i)
 1080       format (5x,i5,7x,i3,5x,i3,2x,i3,5x,a8,10x,f10.5,
     &                 /,46x,3f10.5,/,46x,f10.5,
     &                 /,46x,2f10.5,/,46x,3f10.5)
         end do
 1090    continue
      end if
c
c     atomic dipole polarizability parameters
c
      exist = .false.
      do i = 1, maxtyp
         if (polr(i) .ne. 0.0d0)  exist = .true.
      end do
      if (exist) then
         write (itxt,1100)  formfeed,forcefield
 1100    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,1110)
 1110    format (//,17x,'Dipole Polarizability Parameters',
     &           ///,28x,'Type',14x,'Alpha',/)
         k = 0
         do i = 1, maxtyp
            if (polr(i) .ne. 0.0d0) then
               k = k + 1
               write (itxt,1120)  k,i,polr(i)
 1120          format (16x,i5,7x,i3,6x,f12.3)
            end if
         end do
      end if
c
c     conjugated pisystem atom parameters
c
      exist = .false.
      do i = 1, maxclass
         if (ionize(i) .ne. 0.0d0)  exist = .true.
      end do
      if (exist) then
         write (itxt,1130)  formfeed,forcefield
 1130    format (a1,//,17x,'TINKER Force Field Parameters for ',a20)
         write (itxt,1140)
 1140    format (//,17x,'Conjugated Pisystem Atom Parameters',
     &           ///,21x,'Class',3x,'Electron',3x,
     &                 'Ionization',3x,'Repulsion',/)
         k = 0
         do i = 1, maxclass
            if (ionize(i) .ne. 0.0d0) then
               k = k + 1
               write (itxt,1150)  k,i,electron(i),ionize(i),repulse(i)
 1150          format (10x,i5,7x,i3,f10.1,2x,2f12.3)
            end if
         end do
      end if
c
c     conjugated pisystem bond parameters
c
      if (kpi(1) .ne. '      ') then
         write (itxt,1160)
 1160    format (//,17x,'Conjugated Pisystem Bond Parameters',
     &           ///,20x,'Classes',8x,'d Force',4x,'d Length',/)
         do i = 1, maxnpi
            if (kpi(i) .eq. '      ')  goto 1180
            k1 = number (kpi(i)(1:3))
            k2 = number (kpi(i)(4:6))
            write (itxt,1170)  i,k1,k2,sslope(i),tslope(i)
 1170       format (10x,i5,5x,i3,'-',i3,3x,f12.3,f12.3)
         end do
 1180    continue
      end if
      return
      end
