c
c
c     ###################################################
c     ##  COPYRIGHT (C)  1994  by  Jay William Ponder  ##
c     ###################################################
c
c     #########################################################
c     ##                                                     ##
c     ##  subroutine empole  --  multipole potential energy  ##
c     ##                                                     ##
c     #########################################################
c
c
      subroutine empole
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'bound.i'
      include 'cell.i'
      include 'couple.i'
      include 'electr.i'
      include 'energi.i'
      include 'mpole.i'
      include 'shunt.i'
      include 'usage.i'
      integer i,j,k,m,skip(maxatm)
      integer ii,iz,ix,kk,kz,kx
      real*8 e,emdqik,a(3,3)
      real*8 shift,taper,trans
      real*8 f,fi,fik,xr,yr,zr
      real*8 r,r2,r3,r4,r5,r6,r7
      real*8 rpi(13),rpk(13)
      logical iuse,kuse
      real*8 beta,gamma,afsw,bfsw,cfsw,dfsw
c
c
c     zero out the atomic multipole interaction energy
c
      emdq = 0.0d0
c
c     zero out the list of atoms to be skipped
c
      do i = 1, n
         skip(i) = 0
      end do
c
c     set the switching function coefficients
c
      f = 332.05382d0 / dielec
      call smooth ('CHARGE')
c
c     rotate the multipole components into the global frame
c
      do i = 1, npole
         call rotmat (i,a)
         call rotpole (i,a)
      end do
c
c     calculate the multipole interaction energy term
c
      do ii = 1, npole-1
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, n12(i)
            skip(i12(j,i)) = i
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i
         end do
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii+1, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               if (skip(k) .ne. i) then
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  if (use_image)  call image (xr,yr,zr,0)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Brooks' fsw force switch method
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                    gamma = (off2 - cut2)**3
c                    if (r2 .gt. cut2) then
c                       afsw = off**4 * (off2-3.0d0*cut2) / gamma
c                       bfsw = 6.0d0 * (cut*off)**2 / gamma
c                       cfsw = -(cut2+off2) / gamma
c                       dfsw = 2.0d0 / (5.0d0 * gamma)
c                       e = (e*r) * (  afsw*(1.0d0/r-1.0d0/off)
c    &                                 + bfsw*(off-r)
c    &                                 + cfsw*(off**3-r**3)
c    &                                 + dfsw*(off**5-r**5)  )
c                    else
c                       shift = 8.0d0 * (e*r) *
c    &        ((cut*off)**2*(off-cut)-(off**5-cut**5)/5.0d0) / gamma
c                      e = e + shift
c                    end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Brooks' fsw2 force switch method
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                    if (r2 .gt. cut2) then
c                       e = (1.0d0/sqrt(r)-1.0d0/sqrt(off))**2 * (e*r)
c    &                         * sqrt(off) / (sqrt(off) - sqrt(cut))
c                    else
c                       e = e - (e*r)/sqrt(cut*off)
c                    end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       Brooks' fsh force shift method
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c                    beta = 0.000001d0
c                    taper = 1.0d0 + (r/off) *
c    &                        (r**beta - (beta+1.0d0)*off**beta)
c    &                            / (beta * off**beta)
c                    e = e * taper
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if


                     emdq = emdq + e
                  end if
               end if
            end if
         end do
      end do
c
c     for periodic boundary conditions with large cutoffs
c     neighbors must be found by the replicates method
c
      if (.not. use_replica)  return
c
c     calculate interaction energy with other unit cells
c
      do ii = 1, npole
         i = ipole(ii)
         iz = zaxis(ii)
         ix = xaxis(ii)
         iuse = (use(i) .or. use(iz) .or. use(ix))
         do j = 1, n12(i)
            skip(i12(j,i)) = i
         end do
         do j = 1, n13(i)
            skip(i13(j,i)) = i
         end do
         do j = 1, mdqsiz
            rpi(j) = rpole(j,ii)
         end do
         fi = f * rpi(1)
         do kk = ii, npole
            k = ipole(kk)
            kz = zaxis(kk)
            kx = xaxis(kk)
            kuse = (use(k) .or. use(kz) .or. use(kx))
            if (iuse .or. kuse) then
               do m = 1, ncell
                  xr = x(k) - x(i)
                  yr = y(k) - y(i)
                  zr = z(k) - z(i)
                  call image (xr,yr,zr,m)
                  r2 = xr*xr + yr*yr + zr*zr
                  if (r2 .le. off2) then
                     do j = 1, mdqsiz
                        rpk(j) = rpole(j,kk)
                     end do
                     fik = fi * rpk(1)
                     r = sqrt(r2)
                     e = emdqik (r,xr,yr,zr,rpi,rpk)
                     shift = fik / (0.5d0*(off+cut))
                     e = e - shift
                     if (r2 .gt. cut2) then
                        r3 = r2 * r
                        r4 = r2 * r2
                        r5 = r2 * r3
                        r6 = r3 * r3
                        r7 = r3 * r4
                        taper = c5*r5 + c4*r4 + c3*r3
     &                             + c2*r2 + c1*r + c0
                        trans = fik * (f7*r7 + f6*r6 + f5*r5 + f4*r4
     &                                  + f3*r3 + f2*r2 + f1*r + f0)
                        e = e * taper + trans
                     end if
                     if (i .eq. k)  e = 0.5d0 * e
                     emdq = emdq + e
                  end if
               end do
            end if
         end do
      end do
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  subroutine rotmat  ##
c     ##                     ##
c     #########################
c
c
      subroutine rotmat (i,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i
      real*8 dx,dy,dz,r
      real*8 dotxz,a(3,3)
c
c
c     find rotation matrix that converts local to global coordinates
c
      dx = x(zaxis(i)) - x(ipole(i))
      dy = y(zaxis(i)) - y(ipole(i))
      dz = z(zaxis(i)) - z(ipole(i))
      r = sqrt(dx*dx + dy*dy + dz*dz)
      a(1,3) = dx / r
      a(2,3) = dy / r
      a(3,3) = dz / r
c
c
      dx = x(xaxis(i)) - x(ipole(i))
      dy = y(xaxis(i)) - y(ipole(i))
      dz = z(xaxis(i)) - z(ipole(i))
      dotxz = dx*a(1,3) + dy*a(2,3) + dz*a(3,3)
      dx = dx - dotxz*a(1,3)
      dy = dy - dotxz*a(2,3)
      dz = dz - dotxz*a(3,3)
      r = sqrt(dx*dx + dy*dy + dz*dz)
      a(1,1) = dx / r
      a(2,1) = dy / r
      a(3,1) = dz / r
c
c
      a(1,2) = a(3,1)*a(2,3) - a(2,1)*a(3,3)
      a(2,2) = a(1,1)*a(3,3) - a(3,1)*a(1,3)
      a(3,2) = a(2,1)*a(1,3) - a(1,1)*a(2,3)
      return
      end
c
c
c     ##########################
c     ##                      ##
c     ##  subroutine rotpole  ##
c     ##                      ##
c     ##########################
c
c
      subroutine rotpole (imdq,a)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'mpole.i'
      integer i,j,k,m,imdq
      real*8 m2(3,3),r2(3,3),a(3,3)
c
c
c     monopoles have the same value in any coordinate frame
c
      rpole(1,imdq) = pole(1,imdq)
c
c     rotate the dipoles to the global coordinate frame
c
      do i = 2, 4
         rpole(i,imdq) = 0.0d0
         do j = 2, 4
            rpole(i,imdq) = rpole(i,imdq) + pole(j,imdq)*a(i-1,j-1)
         end do
      end do
c
c     rotate the quadrupoles to the global coordinate frame
c
      k = 5
      do i = 1, 3
         do j = 1, 3
            m2(i,j) = pole(k,imdq)
            r2(i,j) = 0.0d0
            k = k + 1
         end do
      end do
      do i = 1, 3
         do j = 1, 3
            if (j .lt. i) then
               r2(i,j) = r2(j,i)
            else
               do k = 1, 3
                  do m = 1, 3
                     r2(i,j) = r2(i,j) + a(i,k)*a(j,m)*m2(k,m)
                  end do
               end do
            end if
         end do
      end do
      k = 5
      do i = 1, 3
         do j = 1, 3
            rpole(k,imdq) = r2(i,j)
            k = k + 1
         end do
      end do
      return
      end
c
c
c     #######################
c     ##                   ##
c     ##  function emdqik  ##
c     ##                   ##
c     #######################
c
c
      function emdqik (r,xr,yr,zr,rpi,rpk)
      implicit none
      include 'sizes.i'
      include 'atoms.i'
      include 'electr.i'
      include 'mpole.i'
      integer i,j,k
      real*8 emdqik,t2matrix
      real*8 r,r2,r3,r4,r5
      real*8 xr,yr,zr,zrr
      real*8 xrr,xrr2,xrr3,xrr4
      real*8 yrr,yrr2,yrr3,yrr4
      real*8 rpi(13),rpk(13)
      real*8 inter(13),t2(13,13)
      real*8 bx(0:5,0:5),by(11,0:5,0:1),bz(11,0:1)
c
c
c     zeroth order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 1) then
         bz(1,0) = c(1,0,0)
         by(1,0,0) = c(1,0,0) * bz(1,0)
         bx(0,0) = c(1,0,0)
      end if
c
c     first order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 4) then
         r2 = r * r
         zrr = zr / r
         bz(3,0) = c(3,0,0)
         bz(1,1) = c(1,1,1) * zrr
         yrr = yr / r
         by(3,0,0) = c(3,0,0) * bz(3,0)
         by(1,0,1) = c(1,0,0) * bz(1,1)
         by(1,1,0) = c(1,1,1) * bz(3,0) * yrr
         xrr = xr / r
         bx(1,1) = c(1,1,1) * xrr
c
c     second order coefficients to speed the T2 calculation
c
         r3 = r2 * r
         bz(5,0) = c(5,0,0)
         bz(3,1) = c(3,1,1) * zrr
         yrr2 = yrr * yrr
         by(5,0,0) = c(5,0,0) * bz(5,0)
         by(3,0,1) = c(3,0,0) * bz(3,1)
         by(3,1,0) = c(3,1,1) * bz(5,0) * yrr
         by(1,1,1) = c(1,1,1) * bz(3,1) * yrr
         by(1,2,0) = c(1,2,0)*bz(3,0) + c(1,2,2)*bz(5,0)*yrr2
         xrr2 = xrr * xrr
         bx(2,0) = c(1,2,0)
         bx(2,2) = c(1,2,2) * xrr2
      end if
c
c     third order coefficients to speed the T2 calculation
c
      if (mdqsiz .ge. 13) then
         r4 = r2 * r2
         bz(7,0) = c(7,0,0)
         bz(5,1) = c(5,1,1) * zrr
         yrr3 = yrr2 * yrr
         by(7,0,0) = c(7,0,0)*bz(7,0)
         by(5,0,1) = c(5,0,0)*bz(5,1)
         by(5,1,0) = c(5,1,1)*bz(7,0)*yrr
         by(3,1,1) = c(3,1,1)*bz(5,1)*yrr
         by(3,2,0) = c(3,2,0)*bz(5,0) + c(3,2,2)*bz(7,0)*yrr2
         by(1,2,1) = c(1,2,0)*bz(3,1) + c(1,2,2)*bz(5,1)*yrr2
         by(1,3,0) = c(1,3,1)*bz(5,0)*yrr + c(1,3,3)*bz(7,0)*yrr3
         xrr3 = xrr2 * xrr
         bx(3,1) = c(1,3,1) * xrr
         bx(3,3) = c(1,3,3) * xrr3
c
c     fourth order coefficients to speed the T2 calculation
c
         r5 = r2 * r3
         bz(9,0) = c(9,0,0)
         bz(7,1) = c(7,1,1) * zrr
         yrr4 = yrr2 * yrr2
         by(9,0,0) = c(9,0,0)*bz(9,0)
         by(7,0,1) = c(7,0,0)*bz(7,1)
         by(7,1,0) = c(7,1,1)*bz(9,0)*yrr
         by(5,1,1) = c(5,1,1)*bz(7,1)*yrr
         by(5,2,0) = c(5,2,0)*bz(7,0) + c(5,2,2)*bz(9,0)*yrr2
         by(3,2,1) = c(3,2,0)*bz(5,1) + c(3,2,2)*bz(7,1)*yrr2
         by(3,3,0) = c(3,3,1)*bz(7,0)*yrr + c(3,3,3)*bz(9,0)*yrr3
         by(1,3,1) = c(1,3,1)*bz(5,1)*yrr + c(1,3,3)*bz(7,1)*yrr3
         by(1,4,0) = c(1,4,0)*bz(5,0) + c(1,4,2)*bz(7,0)*yrr2
     &                  + c(1,4,4)*bz(9,0)*yrr4
         xrr4 = xrr2 * xrr2
         bx(4,0) = c(1,4,0)
         bx(4,2) = c(1,4,2) * xrr2
         bx(4,4) = c(1,4,4) * xrr4
      end if
c
c     zeroth order T2 matrix elements
c
      if (mdqsiz .ge. 1) then
         t2(1,1) = t2matrix (0,0,0,r,bx,by)
      end if
c
c     first order T2 matrix elements
c
      if (mdqsiz .ge. 4) then
         t2(2,1) = t2matrix (0,0,1,r2,bx,by)
         t2(3,1) = t2matrix (0,1,0,r2,bx,by)
         t2(4,1) = t2matrix (1,0,0,r2,bx,by)
         t2(1,2) = -t2(2,1)
         t2(1,3) = -t2(3,1)
         t2(1,4) = -t2(4,1)
c
c     second order T2 matrix elements
c
         t2(2,2) = -t2matrix (0,0,2,r3,bx,by)
         t2(3,2) = -t2matrix (0,1,1,r3,bx,by)
         t2(4,2) = -t2matrix (1,0,1,r3,bx,by)
         t2(3,3) = -t2matrix (0,2,0,r3,bx,by)
         t2(4,3) = -t2matrix (1,1,0,r3,bx,by)
         t2(4,4) = -t2(2,2) - t2(3,3)
         t2(2,3) = t2(3,2)
         t2(2,4) = t2(4,2)
         t2(3,4) = t2(4,3)
         k = 5
         do i = 2, 4
            do j = 2, 4
               t2(1,k) = -t2(i,j)
               t2(k,1) = t2(1,k)
               k = k + 1
            end do
         end do
      end if
c
c     third order T2 matrix elements
c
      if (mdqsiz .ge. 13) then
         t2(2,5) = t2matrix (0,0,3,r4,bx,by)
         t2(3,5) = t2matrix (0,1,2,r4,bx,by)
         t2(4,5) = t2matrix (1,0,2,r4,bx,by)
         t2(3,6) = t2matrix (0,2,1,r4,bx,by)
         t2(4,6) = t2matrix (1,1,1,r4,bx,by)
         t2(4,7) = -t2(2,5) - t2(3,6)
         t2(2,6) = t2(3,5)
         t2(2,7) = t2(4,5)
         t2(3,7) = t2(4,6)
         t2(2,8) = t2(2,6)
         t2(2,9) = t2(3,6)
         t2(2,10) = t2(4,6)
         t2(3,9) = t2matrix (0,3,0,r4,bx,by)
         t2(3,10) = t2matrix (1,2,0,r4,bx,by)
         t2(4,10) = -t2(2,8) - t2(3,9)
         t2(3,8) = t2(2,9)
         t2(4,8) = t2(2,10)
         t2(4,9) = t2(3,10)
         t2(2,11) = t2(2,7)
         t2(2,12) = t2(3,7)
         t2(2,13) = t2(4,7)
         t2(3,12) = t2(3,10)
         t2(3,13) = t2(4,10)
         t2(4,13) = -t2(2,11) - t2(3,12)
         t2(3,11) = t2(2,12)
         t2(4,11) = t2(2,13)
         t2(4,12) = t2(3,13)
         do i = 5, 13
            do j = 2, 4
               t2(i,j) = -t2(j,i)
            end do
         end do
c
c     fourth order T2 matrix elements
c
         t2(5,5) = t2matrix (0,0,4,r5,bx,by)
         t2(6,5) = t2matrix (0,1,3,r5,bx,by)
         t2(7,5) = t2matrix (1,0,3,r5,bx,by)
         t2(6,6) = t2matrix (0,2,2,r5,bx,by)
         t2(7,6) = t2matrix (1,1,2,r5,bx,by)
         t2(7,7) = -t2(5,5) - t2(6,6)
         t2(8,5) = t2(6,5)
         t2(9,5) = t2(6,6)
         t2(10,5) = t2(7,6)
         t2(9,6) = t2matrix (0,3,1,r5,bx,by)
         t2(10,6) = t2matrix (1,2,1,r5,bx,by)
         t2(10,7) = -t2(8,5) - t2(9,6)
         t2(8,6) = t2(9,5)
         t2(8,7) = t2(10,5)
         t2(9,7) = t2(10,6)
         t2(11,5) = t2(7,5)
         t2(12,5) = t2(7,6)
         t2(13,5) = t2(7,7)
         t2(12,6) = t2(10,6)
         t2(13,6) = t2(10,7)
         t2(13,7) = -t2(11,5) - t2(12,6)
         t2(11,6) = t2(12,5)
         t2(11,7) = t2(13,5)
         t2(12,7) = t2(13,6)
         t2(8,8) = t2(8,6)
         t2(9,8) = t2(9,6)
         t2(10,8) = t2(10,6)
         t2(9,9) = t2matrix (0,4,0,r5,bx,by)
         t2(10,9) = t2matrix (1,3,0,r5,bx,by)
         t2(10,10) = -t2(8,8) - t2(9,9)
         t2(11,8) = t2(11,6)
         t2(12,8) = t2(12,6)
         t2(13,8) = t2(13,6)
         t2(12,9) = t2(10,9)
         t2(13,9) = t2(10,10)
         t2(13,10) = -t2(11,8) - t2(12,9)
         t2(11,9) = t2(12,8)
         t2(11,10) = t2(13,8)
         t2(12,10) = t2(13,9)
         t2(11,11) = t2(11,7)
         t2(12,11) = t2(12,7)
         t2(13,11) = t2(13,7)
         t2(12,12) = t2(12,10)
         t2(13,12) = t2(13,10)
         t2(13,13) = -t2(11,11) - t2(12,12)
         do i = 5, 12
            do j = i+1, 13
               t2(i,j) = t2(j,i)
            end do
         end do
      end if
c
c     compute interaction between the two multipole sites
c
      do i = 1, mdqsiz
         inter(i) = 0.0d0
         do j = 1, mdqsiz
            inter(i) = inter(i) + rpk(j)*t2(j,i)
         end do
      end do
      emdqik = 0.0d0
      do i = 1, mdqsiz
         emdqik = emdqik + inter(i)*rpi(i)
      end do
      emdqik = (332.05382d0 / dielec) * emdqik
      return
      end
c
c
c     #########################
c     ##                     ##
c     ##  function t2matrix  ##
c     ##                     ##
c     #########################
c
c
      function t2matrix (di,dj,dk,rr,bx,by)
      implicit none
      include 'sizes.i'
      include 'mpole.i'
      integer k,di,dj,dk
      real*8 t2matrix,rr
      real*8 bx(0:5,0:5),by(11,0:5,0:1)
c
c
c     get a single element of the Applequist-Dijkstra T2 matrix
c
      t2matrix = 0.0d0
      do k = dk, 0, -2
         t2matrix = t2matrix + bx(dk,k)*by(k+dk+1,dj,di)
      end do
      t2matrix = t2matrix / rr
      return
      end
