c
c
c     ############################################################
c     ##  COPYRIGHT (C) 1996 by Yong Kong & Jay William Ponder  ##
c     ##                  All Rights Reserved                   ##
c     ############################################################
c
c     ###############################################################
c     ##                                                           ##
c     ##  subroutine esphhar  --  linear spherical harmonic method  ##
c     ##                                                           ##
c     ###############################################################
c
c
c     "esphhar" calculates the macroscopic reaction field energy
c
c
      subroutine esphhar

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     spherical harmonics: method1
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      write(*,*) 
      write(*,*) "Spherical Harmonics Method1"

      do i=1,n
         xi = x(i)
         yi = y(i)
         atheta(i) = z(i)/r(i)
         xy = dsqrt(xi*xi + yi*yi)
         cp(i) = yi/xy
         sp(i) = xi/xy
c         aphi(i) = dacos(cp(i))
      end do

      do i=1,n
         rn(i) = 1.0d0
         mcp(i) = 1.0d0
         msp(i) = 0.0d0
      end do

      energy = 0.0d0
      call fsp(UPN,upper_n,fsp_nm)


      do ii=0, upper_n
         do j=0,ii
            yn(j) = 0.0d0
            mn(j) = 0.0d0
         end do

         do i=1,n
            thetai = atheta(i)
c            phii = aphi(i)
            cpi = cp(i)
            spi = sp(i)

            mcpi = 1.0d0
            mspi = 0.0d0
            do j=0,ii
               ttt = legendre(ii,j,thetai)*rn(i)*q(i)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       This is good example why we should avoid
c       sin(x) and cos(x) (acos(x) and asin(x)
c       are even worse)


c               yn(j) = yn(j) + ttt*dcos(j*phii)
c               mn(j) = mn(j) + ttt*dsin(j*phii)
c       If these two lines are used, the results are far-off.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

               yn(j) = yn(j) + ttt*mcpi
               mn(j) = mn(j) + ttt*mspi
               mcpi1 = mcpi
               mspi1 = mspi
               mcpi = mcpi1*cpi - mspi1*spi
               mspi = mspi1*cpi + mcpi1*spi
            end do

            rn(i) = rn(i)*r(i)
         end do

         en = 0.0d0
         do j=0,ii
            en = en + (yn(j)*yn(j) + mn(j)*mn(j))*fsp_nm(ii,j)
c            write(*,*) yn(j),mn(j),fsp_nm(ii,j)
         end do
         en = en * h(ii)
         energy = energy + en
         write(*,11) ii, en, energy
      end do

      end do
      write(*,1000) energy

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     spherical harmonics: method2
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      write(*,*) 
      write(*,*) "Spherical Harmonics Method2"

      do i=1,n
         xi = x(i)
         yi = y(i)
         atheta(i) = z(i)/r(i)
         xy = dsqrt(xi*xi + yi*yi)
         cp(i) = yi/xy
         sp(i) = xi/xy
      end do

      do i=1,n
         rn(i) = 1.0d0
         mcp(i) = 1.0d0
         msp(i) = 0.0d0
      end do

      energy = 0.0d0
      call fsp(UPN,upper_n,fsp_nm)
      do i=0,upper_n
         ttt = h(i)
         do j=0,upper_n
            fsp_nm(i,j) = ttt*fsp_nm(i,j)
         end do
      end do


      do ii=0, upper_n
         do j=ii,upper_n
            yn(j) = 0.0d0
            mn(j) = 0.0d0
         end do

         do i=1,n
            thetai = atheta(i)
            cpi = cp(i)
            spi = sp(i)
            mcpi = mcp(i)
            mspi = msp(i)

            rni = r(i)**ii

            do j=ii,upper_n
               ttt = legendre(j,ii,thetai)*rni*q(i)
               yn(j) = yn(j) + ttt*mcpi
               mn(j) = mn(j) + ttt*mspi
               rni = rni*r(i)
            end do

            mcp(i) = mcpi1*cpi - mspi1*spi
            msp(i) = mspi1*cpi + mcpi1*spi

         end do

         en = 0.0d0
         do j=ii,upper_n
            en = en + (yn(j)*yn(j) + mn(j)*mn(j))*fsp_nm(j,ii)
         end do
         energy = energy + en
         write(*,11) ii, en, energy
      end do

      end do
      write(*,*) 
      write(*,*) "Spherical Harmonics Method2"
      write(*,1000) energy

      end

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     factors for spherical harmonics
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine fsp(upn,upper_n,fsp_nm)

      implicit none

      integer upn,upper_n
      real*8 fsp_nm(0:upn,0:upper_n),frac(0:2*upper_n)
      integer i,j
      
      frac(0) = 1.0d0
      do i=1,2*upper_n
         frac(i) = i*frac(i-1)
      end do

      do i=0,upper_n
         do j=0,i
            fsp_nm(i,j) = 2.0d0*frac(i-j)/frac(i+j)
            if(j .eq. 0) fsp_nm(i,j) = 1.0d0
         end do
      end do

c      do i=0,upper_n
c         write(*,*) (fsp_nm(i,j),j=0,i)
c      end do

      return
      end
